#
# Copyright (c) 1991 University College London
# All rights reserved.
#
# Redistribution and use in source and binary forms are permitted
# provided that the above copyright notice and this paragraph are
# duplicated in all such forms and that any documentation,
# advertising materials, and other materials related to such
# distribution and use acknowledge that the software was developed
# by the Department of Computer Science, University College London.
# The name of the University may not be used to
# endorse or promote products derived from this software without
# specific prior written permission.
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#

# Configuration information for all Makefiles
#
# By George Pavlou, June 1991

# The first part of this file serves to configure the OSIMIS tree
# to cope with optional components which may have not been distributed.
# These are the following:
#
#	BROWSER		- the MIB browser
#	RMIB		- the Remote MIB object-oriented manager access API
#	SIMTIME		- the simulated time support service
#	LT		- the X.500-based location transparency feature
#	MONMET		- the monitor metric objects
#	MONSUP		- the generic summarization monitor support objects
#	OIM		- the native implementation of the OSI Internet MIB
#	IQA		- the Internet Q-Adaptor (CMIS->SNMPv1 proxy)
#	XMP		- the experimental XMP (no XOM yet) API
#       SECURITY	- authenticated associations and integrity checks
#
# Note that you need NOT modify anything in this part unless
# you do not want to build any of the above supplied components.
# In this case simply comment out all the relevant lines. You should NOT
# comment-in though any of the non-supplied components as building will fail.


# the MIB browser - this needs InterViews 2.6 and g++ 2.2.2 or CC 2.1
# comment it out for g++ 2.5.x/2.6, you may use the SPARC/SunOS 4.1.3 binary
#
#BROWSER		= browser

# RMIB and SIMTIME
#
# if RMIB is empty, SIMTIME should be also empty as it relies on it
#
RMIB		= rmib
SIMTIME		= simtime

# LT
#
#CCONFIG		+= -DUSE_X500
#LT		= lt
#LT_LIB		= $(TOP)/lt/liblt.a
#X500_LIB	= $(ISODE)/libdsap-osisec.a
#X500_LIB	= $(ISODE)/libdsap.a

# MONMET
#
CCONFIG		+= -DMONMET
MONMET_MIB	= monmet_mib
MONMET_MIB_LIB	= $(TOP)/agent/monmet_mib/libmonmetmib.a
MONMET_SNTX_LIB	= $(TOP)/agent/monmet_mib/libmonmetsntx.a

# MONSUP
#
CCONFIG         += -DMONSUP
MONSUP_MIB	= monsup_mib
MONSUP_MIB_LIB  = $(TOP)/agent/monsup_mib/libmonsupmib.a

# OIM
# this has not yet been ported to this OSIMIS, it is not even in the tar image
#
#CCONFIG		+= -DOIM
#OIM		= oim
#OIM_MIB		= oim_mib
#OIM_SMA		= oim_sma
#OIM_SNTX_LIB	= $(TOP)/agent/oim_mib/liboimsntx.a

# IQA
# this is only now being ported to this OSIMIS, it is not (yet) in the tar image
#
CCONFIG		+= -DIQA
IQA		= iqa
IQA_SNTX_LIB	= $(TOP)/proxy/iqa/proxy/libiqasyntaxes.a

# XMP
#
#XMP		= xmp

# SECURITY
# you need OSISEC as well as OSIMIS/ISODE for this feature
#
#CCONFIG		+= -DSECURITY
#SECURITY	= security
#SECURITY_LIB	= $(TOP)/util/security/libsecurity.a
#OSISEC_LIB =	$(OSISEC)/lib/libosisec.a \
		$(OSISEC)/lib/libcrypt.a \
		$(OSISEC)/lib/libasn.a \
		$(OSISEC)/lib/libdsap-osisec.a
# If you need SECURITY, also check the OSISEC vars down below




# The following part configures the local environment.
# Please read carefully the ./README file before you proceed.
#

# LSOCKET should be as in the ISODE CONFIG.make
# e.g. /usr/sunlink/osi/lib/libosi.a for SunLink OSI, -lbsd for AIX etc.
# You may also put any other needed system libraries in it
LSOCKET	=
#LSOCKET	= -lsocket -lnsl -lelf /usr/ucblib/libucb.a	# for Solaris


# prefer gcc with g++
# GNU C version 2.5.8
#CC	=	/cs/research/icm/proj/kinou/gcc-bin/bin/gcc -traditional
# GNU C version 2.6.3
# for SunOS 4.1.3
CC	=	/cs/research/mice/speedy/common/sun4/bin/gcc -traditional
# for Solaris 2.4
#CC	=	/cs/research/mice/speedy/common/sun5/bin/gcc -traditional
# use cc with ATT CC
#CC	=	cc


# GNU C++ version 2.5.8
#CCPLUS	=	/cs/research/icm/proj/kinou/gcc-bin/bin/g++
# GNU C++ version 2.6.3
# for SunOS 4.1.3
CCPLUS  =	/cs/research/mice/speedy/common/sun4/bin/g++
# for Solaris 2.4
#CCPLUS	=	/cs/research/mice/speedy/common/sun5/bin/g++
# Sun ATT C++ version 2.1
#CCPLUS	=	CC

#PURIFY	=	/cs/research/mhs/pink/purify-2.1-sunos4/purify
# -collector only required for g++
#PURIFYPLUS =	$(PURIFY) -collector=/usr/ucl/lib/gcc-lib/sparc-sun-sunos4.1/2.2.2/ld

PEPSY	=	/cs/research/icm/other/kinou/isode-8.0/pepsy
#PEPSY	=	/cs/research/icm/proj2/athena/gpavlou/isode-solaris/isode-8.0/pepsy/xpepsy
#PEPSY	=	/cs/research/sec/pink/osisec2/ic-r1.0/isode/src/lib/asn1/pepsy/xpepsy
#PEPSY	=	/cs/research/sec/pink/osisec2/IC-install/usr/bin/pepsy

ISODE	=	/cs/research/icm/other/kinou/isode-8.0
#ISODE	=	/cs/research/icm/proj2/athena/gpavlou/isode-solaris/isode-8.0
#ISODE	=	/cs/research/sec/pink/osisec2/ic-r1.0/isode/src/lib
#ISODE	=	/cs/research/sec/pink/osisec2/IC-install/usr/lib

IVIEWS	=	/cs/research/icm/other/kinou/iviews
#IVIEWS	=	/cs/research/icm/osimis/kinou/InterViews/iv/src/libInterViews/sun4/
X11	=				# for plain X11
# X11	=	-L/usr/openwin/lib	# for Sun OpenWindows X11
TCLTK	=	/cs/research/icm/other/kinou/tcltk/lib
ITCL	=	/cs/research/icm/other/kinou/itcl/lib

OSISEC	=	/cs/research/midas2/poteen/common/osisec-1.0
OSISEC-INC =    -I$(OSISEC)/h -I$(TOP)/include/isode
OSISEC-OPT =    -DUCLISODE $(OSISEC-INC)

# none of the next five should be /usr/include, see README file
X11INC	=	/usr/openwin/include	# for Sun OpenWindows X11
IVIEWSINC=	/usr/ucl/include
# the next is only needed for building the optional kernel/TkCoord.o
# (see kernel/Makefile)
TCLINC	=	/cs/research/icm/other/kinou/tcltk/tcl7.3
TKINC	=	/cs/research/icm/other/kinou/tcltk/tk3.6
ITCLINC	=	/cs/research/icm/other/kinou/itcl/include

#LPP	=	-lpp
CMPL	=
#CMPL	=	.$(CCPLUS)
ARCH	=
#ARCH	=	.sun3
#ARCH	=	.sun4

LDFLAGS	=	-s


# do not change the following for building OSIMIS
#
LIB	=	$(TOP)/lib$(CMPL)$(ARCH)
BIN	=	$(TOP)/bin$(ARCH)
OSIMIS-INC =	$(TOP)/include
H	=	$(TOP)/h
ETC	=	$(TOP)/etc
MAN	=	$(TOP)/man

# configure the following in order to install outside the OSIMIS tree
# (they are currently configured as at UCL) - you will obviously need
# to uncomment them to overwrite the options above
#
#LIB	=	/usr/local/lib/osimis
#BIN	=	/usr/local/bin
#OSIMIS-INC =	/usr/local/include/osimis
#H	=	/usr/local/include/osimis
#ETC	=	/usr/local/etc/osimis
#MAN	=	/usr/local/man


CCONFIG +=	-DDEBUG -DLOGGING -DPERSISTENCY # -DICMPON -DSOLARIS -DSUNCC4 -DTIMING # -DICR1 # -DLPP

INC	=	-I$(ITCLINC) -I$(TCLINC) -I$(X11INC) -I$(TKINC) -I$(IVIEWSINC)
OPTIONS =	-I. -I$(OSIMIS-INC) -I$(OSIMIS-INC)/isode/pepsy \
		$(INC) -I$(H) $(CCONFIG)
CCFLAGS	=	$(OPTIONS) -g # -O
PFLAGS	=	-A -h -m	 # pepsy flags, do not change
LFLAGS	=	-bhuz $(OPTIONS) # lint flags, do not change


YACC		= $(TOP)/compilers/mocompiler/byacc/byacc 
YACCFLAGS	= -v
LEX		= $(TOP)/compilers/mocompiler/flex/flex 
LEXFLAGS	= 


AR              = /bin/ar
ARFLAGS         = rv
RANLIB          = /bin/ranlib


ASN-CMPL	= $(TOP)/compilers/asncompiler/asn-cmpl
GDMO		= $(TOP)/compilers/mocompiler/gdmo/gdmo
GDMO-CMPL	= $(TOP)/compilers/mocompiler/gdmo/gdmo-cmpl
RUN-GDMO	= $(GDMO-CMPL)	# for backwards compatibility
GDMOASN-CMPL	= $(TOP)/compilers/mocompiler/gdmo/gdmoasn-cmpl


.SUFFIXES:	.py .c .cc .o .asn -types.h _tables.c

.asn_tables.c .asn-types.h:;	$(PEPSY) $(PFLAGS) $<

.c.o:;		$(CC) $(CCFLAGS) -c $*.c

.cc.o:;		$(CCPLUS) $(CCFLAGS) -c $*.cc
