//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// EventLog.h - Logging for Event Reports

// Saleem N. Bhatti
// April 1992

#if !defined(_EventLog_h_)
#define _EventLog_h_

#include "Top.h"

class ObjectClass;
class ObjectInstance;
class EventType;
class Time;
class String;
class Integer;


typedef enum LogRecordInfo_e {
    LogRecordInfo_failed,          /* Failure */
    LogRecordInfo_ok,              /* OK */
    LogRecordInfo_wrongRecordType, /* setInfo() needs redefining */
    LogRecordInfo_noRecordType     /* no record type for this notification */
} LogRecordInfo;


class LogRecord : public Top {

    static
    MOClassInfo _info;

protected :

    static const int
    _level;          // 1
    enum {
    I_logRecordId,   // 0 Integer
    I_loggingTime,   // 1 Time
    I_logRecordNAttrs};

    Attr *_attrs[I_logRecordNAttrs];

    CMISErrors cmisDelete();
    LogRecord(RDN, MO *);

public:

    static
    int initialiseClass();
    static
    MOClassInfo *getClassInfo();

    virtual
    LogRecordInfo setInfo(const void *notification);
    virtual
    void setInfo(OID object, DN name, OID event);

    Integer * logRecordId();
};

/*
** LogRecord methods
*/

inline MOClassInfo *
LogRecord::getClassInfo()
{ return(&_info); }

inline LogRecordInfo 
LogRecord::setInfo(const void *) { return LogRecordInfo_failed; }

inline void
LogRecord::setInfo(OID, DN, OID) {}

inline Integer *
LogRecord::logRecordId()
{ return (Integer *) _attrs[I_logRecordId]; }


class EventLogRecord : public LogRecord {

    static
    MOClassInfo _info;

protected:

    static const int
    _level;                  // 2
    enum {
    I_managedObjectClass,    // 0 ObjectClass
    I_managedObjectInstance, // 1 ObjectInstance
    I_eventType,             // 2 EventType
    I_eventTime,             // 3 Time
    I_additionalText,        // 4 GraphicString
    I_eventLogRecordNAttrs};

    Attr *_attrs[I_eventLogRecordNAttrs];

public:

    static
    int initialiseClass();
    static
    MOClassInfo *getClassInfo();
    static
    MO *create(RDN, MO *);

    void setInfo(OID object, DN name, OID event);

    LogRecordInfo setInfo(const void *notification);
    void setInfo(const PE info);

    ObjectClass *managedObjectClass();
    ObjectInstance *managedObjectInstance();
    EventType *eventType();

    EventLogRecord(RDN, MO *, Bool);
    ~EventLogRecord();
};

/*
** EventLogRecord methods
*/

inline ObjectClass *
EventLogRecord::managedObjectClass()
{ return (ObjectClass *) _attrs[I_managedObjectClass]; }

inline ObjectInstance *
EventLogRecord::managedObjectInstance()
{ return (ObjectInstance *) _attrs[I_managedObjectInstance]; }

inline EventType *
EventLogRecord::eventType()
{ return (EventType *) _attrs[I_eventType]; }

inline MOClassInfo *
EventLogRecord::getClassInfo()
{ return(&_info); }

inline MO *
EventLogRecord::create(RDN rdn, MO *superior)
{ return(new EventLogRecord(rdn, superior, True)); }

#endif /* _EventLog_h_ */
