//
// Copyright (c) 1993 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// Cache of log records

// Saleem N. Bhatti

#if !defined(_RecordInfo_h_)
#define _RecordInfo_h_

#include "msap.h"
#include "EventLog.h"
#include "Create.h"

class RecordInfo {

    friend class NotificationMapper;

    static
    int                 _count; /* for use in constructing RDN */

    const MOClass             *_recordInfo;
    const NotificationMapping *_mappingInfo;
    LogRecord                 *_record;
    sntx_table                *_syntax;

    LogRecord *getRecord();

    RecordInfo(const MOClass *recordInfo,
               const NotificationMapping *mappingInfo);
    ~RecordInfo();
};

class NotificationMapper {

    static
    RecordInfo **_recordInfoTable;

    RecordInfo *getRecordInfo(const char *notificationName);

public:

    LogRecord *mapNotification(const char *name);
    PE        encodeNotification(const char *name, const void *notification);
    LogRecord *getRecord(const char *notificationName);
    void      freeNotification(const char *name, const void *notification);

    NotificationMapper();
    ~NotificationMapper();
};

extern NotificationMapper *notificationMapper;

#endif /* _RecordInfo_h_ */
