/*
 * Copyright (c) 1990 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* IsodeSntx.c - ISODE MO attribute and event report syntaxes */

/* By George Pavlou, May 1991 */


#include <stdio.h>
#include "msap.h"
#include "IsodeSntx.h"


/* TProtocolClass */

PE  TProtocolClass_encode (tclass)
struct type_ISODE_TProtocolClass* tclass;
{
    PE pe;
    if (build_ISODE_TProtocolClass(&pe, 1, 0, NULLCP, tclass) == NOTOK)
	return NULLPE;
    return pe;
}


struct type_ISODE_TProtocolClass*  TProtocolClass_decode (pe)
PE pe;
{
    struct type_ISODE_TProtocolClass* tclass;

    if (parse_ISODE_TProtocolClass(pe, 1, NULLIP, NULLVP, &tclass) == NOTOK)
	return NULL;

    return tclass;
}


void TProtocolClass_free (tclass)
struct type_ISODE_TProtocolClass* tclass;
{
    free_ISODE_TProtocolClass(tclass);
}


/* ARGSUSED */

int  TProtocolClass_print (ps, tclass, format)
PS ps;
struct type_ISODE_TProtocolClass* tclass;
int format;
{
    switch (tclass->parm) {
	case int_ISODE_TProtocolClass_undefined:
	    ps_printf(ps, "undef");
	    break;
	case int_ISODE_TProtocolClass_tp0:
	    ps_printf(ps, "tp0");
	    break;
	case int_ISODE_TProtocolClass_tp1:
	    ps_printf(ps, "tp1");
	    break;
	case int_ISODE_TProtocolClass_tp2:
	    ps_printf(ps, "tp2");
	    break;
	case int_ISODE_TProtocolClass_tp3:
	    ps_printf(ps, "tp3");
	    break;
	case int_ISODE_TProtocolClass_tp4:
	    ps_printf(ps, "tp4");
	    break;
	default:	/* should not reach here - decoder will take care */
	    ps_printf(ps, "undef");
	    break;
    }
    return OK;
}


int TProtocolClass_cmp (tclass1, tclass2)
struct type_ISODE_TProtocolClass *tclass1, *tclass2;
{
    if (tclass1->parm == tclass2->parm)
	return 0;
    return 2;
}


void  TProtocolClass_syntax ()
{
    (void) add_attribute_syntax("TProtocolClass",
		(IFP) TProtocolClass_encode,
		(IFP) TProtocolClass_decode,
		NULLIFP,
		TProtocolClass_print,
		NULLIFP,
		TProtocolClass_cmp,
		TProtocolClass_free,
		NULLIFP, NULLIFP, FALSE);
}


/* isode syntaxes */

void  isode_attr_syntaxes ()
{
    TProtocolClass_syntax();
}


void  isode_syntaxes ()
{
    isode_attr_syntaxes();
}
