//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// Sma.h - the classes the agent has knowledge of and create information

// By George Pavlou, May 1991


#include "Create.h"
#include "SmiSntx.h"

#include "System.h"
#include "eventForwardingDiscriminator.h"
#include "EventLog.h"
#ifdef	LOGGING
#include "Log.h"
#endif


#ifdef	MONMET
#include "Monitor-Sntx.h"
#include "monitorMetric.h"
#include "movingAverageMeanMonitor.h"
#endif


#include "Isode-Sntx.h"
#include "subsystem.h"
#include "transportEntity.h"
#include "transportConnection.h"

#include "monitoringSupportMode2.h"
#include "monitoringSupportMode1.h"
#include "probabilityEstimator.h"


#include "uxObj1.h"
#include "uxObj2.h"


char*	service = "GSMO";			// application name

#define	MIB_INIT_FILE	"mib.init.gsmo"		/* MIB initialisation info */



// a table of all the MO classes this application supports -
// used to create the meta-class tree

MOClass moclasses[] =
{
//  SMF-related classes first and the MIT-top system

{ "top",
   Top::getClassInfo,
   Top::initialiseClass },

{ "system",
   System::getClassInfo,
   System::initialiseClass },

{ "discriminator",
   discriminator::getClassInfo,
   discriminator::initialiseClass },

{ "eventForwardingDiscriminator",
   eventForwardingDiscriminator::getClassInfo,
   eventForwardingDiscriminator::initialiseClass },

#ifdef	LOGGING
{ "log",
   Log::getClassInfo,
   Log::initialiseClass },
#endif

{ "logRecord",
   LogRecord::getClassInfo,
   LogRecord::initialiseClass },

{ "eventLogRecord",
   EventLogRecord::getClassInfo,
   EventLogRecord::initialiseClass },


#ifdef	MONMET
{ "scanner",
   scanner::getClassInfo,
   scanner::initialiseClass },

{ "monitorMetric",
   monitorMetric::getClassInfo,
   monitorMetric::initialiseClass },

{ "movingAverageMeanMonitor",
   movingAverageMeanMonitor::getClassInfo,
   movingAverageMeanMonitor::initialiseClass },
#endif


//  classes specific to this application i.e. SMA

{ "uxObj1",
   uxObj1::getClassInfo,
   uxObj1::initialiseClass },

{ "uxObj2",
   uxObj2::getClassInfo,
   uxObj2::initialiseClass },


{ "entity",
   entity::getClassInfo,
   entity::initialiseClass },

{ "connection",
   connection::getClassInfo,
   connection::initialiseClass },

{ "transportEntity",
   transportEntity::getClassInfo,
   transportEntity::initialiseClass },

{ "transportConnection",
   transportConnection::getClassInfo,
   transportConnection::initialiseClass },

{ "MSscanner",
   MSscanner::getClassInfo,
   MSscanner::initialiseClass },

{ "monitoringSupport",
   monitoringSupport::getClassInfo,
   monitoringSupport::initialiseClass },

{ "monitoringSupportMode1",
   monitoringSupportMode1::getClassInfo,
   monitoringSupportMode1::initialiseClass },

{ "monitoringSupportMode2",
   monitoringSupportMode2::getClassInfo,
   monitoringSupportMode2::initialiseClass },

{ "probabilityEstimator",
   probabilityEstimator::getClassInfo,
   probabilityEstimator::initialiseClass },

{ "subsystem",
   subsystem::getClassInfo,
   subsystem::initialiseClass },

};

int NCLASSES = sizeof moclasses / sizeof moclasses[0];



// a table mapping notifications to specific log records -
// used to process notifications 

NotificationMapping notificationMappingTable[] =
{
{"objectCreation", 		"eventLogRecord"},
{"objectDeletion",	 	"eventLogRecord"},
{"attributeValueChange",	"eventLogRecord"},
{"stateChange",			"eventLogRecord"},
{"qualityofServiceAlarm",       "eventLogRecord"},
};

int NNOTIFICATIONS = sizeof notificationMappingTable /
			    sizeof notificationMappingTable[0];



// a table of syntaxes this application uses

VoidFunc	syntaxes[] = {
		    smi_syntaxes,
#ifdef	MONMET
		    monitor_attr_syntaxes,
#endif
		    isode_attr_syntaxes,
		    NULL
		};

