//
// NAME
//   buffered.cc
//
// DESCRIPTION
//   member functions of the class for the statistics
//   of the Monitoring Support objects
//
// IMPLEMENTORS
//   George Mykoniatis
//   NTUA
//   mykoniat@ektor.ntua.gr
//

#include "buffered.h"
#include <iostream.h>

// ------------------------------------------------------------------
double buffered::get_last_value()
{
  if (index)
    return( array[index-1] );
  else
    return( array[no_of_samples-1] );
} //buffered::get_last_value

// ------------------------------------------------------------------
int buffered::initialize(int win, int prd, double thres, double iv)
{
  // Store the parameters
  window = win;
  period = prd;
  threshold = thres;

  index = 0;		        // point to first entry;
  buffer_flag = 0;		// Set to false

  // Check if invalid window or period
  if (prd == 0)
    {
    fprintf( stderr, "sumstat :: Invalid period %d\n", period);
    return NOTOK;
    }
  if (win < prd)
    {
    fprintf( stderr, "sumstat :: window < period %d<%d\n", win, period);
    return NOTOK;
    }
  // --------------------------
  // Calculate size of array
  no_of_samples = win/prd;
  if (win%prd)      		// In case of not exact multiple...
    no_of_samples++;
  // Allocate buffer
  array = new double[no_of_samples];
  if (array==NULL)
  {
    fprintf( stderr, "sumstat :: Failed to allocate memory\n", win, period);
    return NOTOK;
  }
  // Initialize the buffer
  for (int i = 0; i<no_of_samples ; i++)
    array[i] = iv;

  // Initialise the rest of the variables
  result = find_probability();	// Initialise result

  return OK;
}  //buffered::initialize

// ------------------------------------------------------------------
buffered::~buffered()
{
#ifdef DEBUG
  fprintf(stderr, "~buffered: Executing..\n");
#endif

  if (array)	// release array
    delete array;
  return;
} //buffered::~buffered

// ------------------------------------------------------------------
int buffered::append_value(double val)
{
  array[index] = val;
  index++;
  if (index == no_of_samples) 	// If buffer full ...
  {
    index = 0;  		// reset pointer
	if (buffer_flag == 0)
	{
	  double dtmp=0;
	  for(int i=0; i < no_of_samples; i++)
		dtmp += array[i];
	  buffer_flag = 1;    // Set buffer full flag
    	}
  }
  result = find_probability();	// Update result
  return(index);
} //buffered::append_value

// ------------------------------------------------------------------
double buffered::find_probability()
{
  int counter = 0;
  for( register int i=0; i<no_of_samples ; i++)
	{
	if (array[i] >= threshold)	
	    counter++;
	}
  // Casting is required
  return counter/((double)no_of_samples);
}

// ------------------------------------------------------------------
void buffered::print_buffered()
{
  fprintf(stdout, "\n-------buffered--------\n");
  fprintf(stdout, "Window : %d, Period : %d, Threshold : %lf, Samples : %d\n",
		   window, period, threshold, no_of_samples);
  for (int i = 0; i<no_of_samples; i++)
    fprintf(stdout, "\tSample no %d : %lf\n", i, array[i]);
  fprintf(stdout, "\t\t*** Result : %lf\tFlag : %d\n", result, buffer_flag);
  fprintf(stdout, "-----------------------\n\n");
  return;
} //buffered::print_buffered

