//
// NAME
//   monitoringSupportMode1.inc.cc
//
// DESCRIPTION
//   for monitoringSupportMode1 MOC, via OSIMIS GDMO compiler
//
// IMPLEMENTORS
//   George Mykoniatis
//   NTUA
//   mykoniat@ektor.ntua.gr
//

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "Integer.h"

#include "AdministrativeState.h"/* for scanner::administrativeState */
#include "AttributeIdList.h"
#include "ObjectInstanceList.h"

#include "sumstat.h"    
#include "observed_attr.h"      

static OID OID_monitoredAttributesList = NULL;
static OID OID_monitoredObjectsList    = NULL;
static OID OID_ResourceRequestRate    = NULL;
        //
        // These are initialized (just once) in monitorMetric::createRR below.
        //
        // We can *NOT* use: name2oid("~~~") here, since it depends
        // on an OID hashtable being initialized first to work properly.
        // Hashtable is not initialized before above OID handles are.
        //

// --------------------------------------------------------------
int monitoringSupportMode1::DecomposeAttrList()
{
  int noOfAttrs = 0;
  int noOfObjs = 0;

  _oidlist = (MIDListVal*) (monitoredAttributesList() ->getval());

  _dnlist = (MNListVal*) (monitoredObjectsList() ->getval());

  MIDListVal* 	cp_oidlist 	= _oidlist;
  MNListVal*  	cp_dnlist 	= _dnlist;

  // Find the number of attributes
  while (cp_oidlist)
  {
	noOfAttrs++;
        cp_oidlist = cp_oidlist->midl_next;
  }

  // Find the number of object instances
  while (cp_dnlist)
  {
	noOfObjs++;
        cp_dnlist = cp_dnlist->mnl_next;
  }

  if (noOfAttrs != noOfObjs)
  {
	cout << "monitoringSupportMode1 : different number of OIDs and DNs" << endl;
	return NOTOK;
  }

  no_of_attributes = noOfAttrs;

} // monitoringSupportMode1::DecomposeAttrList


// --------------------------------------------------------------
int monitoringSupportMode1::InitObservedAttrs()
{
   // Allocate space for the oa_array
   oa_array = new observed_attr *[no_of_attributes];
   if (oa_array == NULL)
   {
	cout << "monitoringSupportMode1 : Failed to allocate memory for oa_array\n";
	return NOTOK;
   }

   for(int i=0;i<no_of_attributes;i++)
	oa_array[i] = NULL;			// Initialise to NULL

   MO 		*pMO, *top;
   DN		dnMO;
   OID		oid;

   MIDListVal*  cp_oidlist      = _oidlist;
   MNListVal*   cp_dnlist       = _dnlist;

   for (i=0;i<no_of_attributes;i++)
   {
 	oa_array[i]=new observed_attr();
	
   	if (oa_array[i] == NULL)
   	{
	 	cout << "monitoringSupportMode1 : Failed to allocate memory\n";
	 	return NOTOK;
   	}

	//Find the OID from oidlist
	oid = (cp_oidlist->midl_mid).mid_global;

   	dnMO = (cp_dnlist->mnl_mn).mn_dn;

	//Find the MO* from the DN from dnlist
	_typeMO = (cp_dnlist->mnl_mn).mn_type;
   	top = _typeMO == MN_DN ?
		MO::getRoot() : MO::getRoot() -> getSubordinate();

	pMO = top -> getMO(dnMO, True, DISCARDAVA);
	if (!pMO)
	{
		cout << "Observed MO not found" << endl;
		return NOTOK;
	}
	  
   	if ((oa_array[i])->initialise(pMO, oid) == NOTOK)
		return NOTOK;   // Initialise

	cp_dnlist = cp_dnlist->mnl_next;
	cp_oidlist = cp_oidlist->midl_next;
   }

    for(i=0;i<no_of_attributes;i++)
#ifdef DEBUG
     if (oa_array[i]->get_flag() != NOTOK) 
        oa_array[i]->print_oa();
#endif
   // Now, all the class attributes are initialized...
   // so, return ok
   return OK;
} //monitoringSupportMode1::InitObservedAttrs


// --------------------------------------------------------------
int monitoringSupportMode1::CheckFirstTime() 
{
  if (DecomposeAttrList() == NOTOK)
     return NOTOK;
  
  if (InitObservedAttrs() == NOTOK)
     return NOTOK;
  
  return OK;

} // monitoringSupportMode1::CheckFirstTime


// --------------------------------------------------------------
int monitoringSupportMode1::CheckObservedMOs()
{

   MO 		*pMO, *top;
   DN		dnMO;
   OID		oid;

   MIDListVal*  cp_oidlist      = _oidlist;
   MNListVal*   cp_dnlist       = _dnlist;

   for (int i=0;i<no_of_attributes;i++)
   {

	//Find the OID from oidlist
	oid = (cp_oidlist->midl_mid).mid_global;

   	dnMO = (cp_dnlist->mnl_mn).mn_dn;

	//Find the MO* from the DN from dnlist
	_typeMO = (cp_dnlist->mnl_mn).mn_type;
   	top = _typeMO == MN_DN ?
		MO::getRoot() : MO::getRoot() -> getSubordinate();

	pMO = top -> getMO(dnMO, True, DISCARDAVA);
	if (!pMO)
	{
		cout << "Observed MO not found" << endl;
		return NOTOK;
	}
	  
	cp_dnlist = cp_dnlist->mnl_next;
	cp_oidlist = cp_oidlist->midl_next;
   }

   // Now, all the class attributes are checked...
   // so, return ok

   return OK;

} //monitoringSupportMode1::CheckObservedMOs

// --------------------------------------------------------------
int monitoringSupportMode1::CheckConsistency() 
{
   if (CheckObservedMOs() == NOTOK)
	return NOTOK;

   return OK;

} // monitoringSupportMode1::CheckConsistency


// --------------------------------------------------------------
int monitoringSupportMode1::StartUp()
{
 int window, period, algorithm ;

 // create the observed_attr instances
 if ( CheckFirstTime() == OK )		// First, decompose list
 {
   // Allocate memory for the operation matrix
   oaval = new double[no_of_attributes];   
   if (oaval == NULL)
   {
	 fprintf( stderr, "StartUp: Failed to allocate memory \n");
	 return NOTOK;
   }
   // Allocate memory for the result sumstat class
   // but get the window, period and algorithm first.

   // Create all the observed_attr instances required
   // but before that, get the values for window

   window = * (int *) (collectionPeriod()->get());

   int iPollInterval           = 0;
   type_Monitor_TimePeriod* tp =
     (type_Monitor_TimePeriod*)
     ((TimePeriod*)(MSscanner::_attrs[I_granularityPeriod]))->getval();

   if ((iPollInterval = TimePeriod2seconds(tp)) == 0) {
     iPollInterval = 60; /* default value */
   }

   if ( (window == 0) || (window < iPollInterval) )		// Check window also 
   {
	fprintf( stderr, "monitoringSupportMode1 : Invalid movingTimePeriod\n");
	return NOTOK;
   }

   algorithm= * (int *) (algorithmType()->get());

   stat_p = new sumstats;
   if (stat_p == NULL)
   {
         fprintf( stderr, "StartUp: Failed to allocate memory \n");
         return NOTOK;
   }
   // Set initial value to 0 (?)
   if (stat_p->initialize(window, iPollInterval, algorithm, 0) == NOTOK)
   {  
        fprintf( stderr, "StartUp: Failed to initialize \n");
        return NOTOK;
   }

   // Since everything went ok
   return OK;
 }
 // Failed to initialise the attributes (DecomposeAttrList failed)...
 return NOTOK;
} //monitoringSupportMode1::StartUp

// ----------------------------------------------------------------
int monitoringSupportMode1::set (CMISModifyOp mode, 
					int attrId, 
					int classLevel,
                      			void* value,
					AVA*& err,
					Bool checkOnly, 
					int)
{
    
    if (classLevel != _level) 
        return monitoringSupport::set(mode, attrId, classLevel, value, err, checkOnly);

#ifdef DEBUG
  fprintf(stdout, "monitoringSupportMode1::set-Executing.....\n");
#endif

    AdminStateVal admState = *(AdminStateVal*)
        MSscanner::administrativeState()->getval();

    switch (attrId)
    {
      case I_monitoredAttributesList:
      case I_monitoredObjectsList:
        if (admState != as_locked)
        {
           if (checkOnly)
	     return NOTOK;
	   err = new AVA(m_invalidOperation);
   	   return NOTOK;
        }
        break;
      default:
        break;

    } // Switch attrid

    return OK;
} //monitoringSupportMode1::set


//
// used to initialize _thldAlarmInfo pointer during createRR()
// freed by deleteRR()
//
type_SMI_AlarmInfo*
monitoringSupportMode1::_thldAlarmInfo_alloc ()
{
    struct type_SMI_AlarmInfo* elem;
    if ((elem = ThresholdAlarmInfo_ProbsAttrs_alloc (NULL, 1, 2))== NULL) {
       return NULL;
    }

    /* specificProblems */
    ((MIDListVal*) elem->specificProblems)->midl_mid.mid_global =
        ::OID_ResourceRequestRate;
    /* monitoredAttrs */

    /* insert monitoredObjectsList OID/value */
    ((CMISParamListVal*) elem->monitoredAttrs)->mpl_id.mid_global =
       ::OID_monitoredObjectsList;
    ((CMISParamListVal*) elem->monitoredAttrs)->mpl_val =
        monitoredObjectsList()->encode();

    /* insert monitoredAttributesList OID/value */
    ((CMISParamListVal*) elem->monitoredAttrs)->mpl_next->mpl_id.mid_global =
       ::OID_monitoredAttributesList;
    ((CMISParamListVal*) elem->monitoredAttrs)->mpl_next->mpl_val =
        monitoredAttributesList()->encode();

    return elem;
}

int monitoringSupportMode1::createRR (AVA*& err, void* pv, int)
{

  // correct OO style
  if (monitoringSupport::createRR(err, pv) == NOTOK)
    return NOTOK;

  static Bool bClassInitialised = False;

  if (!bClassInitialised) {

    ::OID_monitoredAttributesList = name2oid("monitoredAttributesList");
    ::OID_monitoredObjectsList = name2oid("monitoredObjectsList");
    ::OID_ResourceRequestRate = name2oid("resourceRequestRate");

    bClassInitialised = True;
  }
#ifdef DEBUG
// Print a message that initialisation has finished
   fprintf( stdout, "Finished createRR\n");
#endif
   return OK;
} //monitoringSupportMode1::createRR

