//
// NAME
//   observed_attr.cc
//
// DESCRIPTION
//   methods of the observed_attr class
//
// IMPLEMENTORS
//   George Mykoniatis
//   NTUA
//   mykoniat@ektor.ntua.gr
//

#include "observed_attr.h"

// --------------------------------------------------------------------------
observed_attr::observed_attr()
{
  attr_flag = NOTOK;
  _pvObservedMOattribute = NULL;        // Initialise to NULL

  pObservedMO = NULL;			// Initialise to NULL
} //observed_attr::observed_attr


// --------------------------------------------------------------------------
observed_attr::~observed_attr()
{
#ifdef DEBUG
  fprintf(stderr, " ~observed_attr: Executing\n");
#endif
} //observed_attr::~observed_attr


// --------------------------------------------------------------------------
int observed_attr::initialise(MO *pMO, OID oidObsAt)
{
  double dtmp = 0;

  // Some basic initializations
  _oidObservedAttr = oidObsAt;
  pObservedMO = pMO;

  if ((attr_flag = check_attr()) == NOTOK)
  {
    fprintf(stdout, "observed_attr: Failed to find Attribute\n");
    return NOTOK;
  }
  else
    dtmp = getval_attr();    // Get current value. 

  return OK;
} //observed_attr::initialise


// --------------------------------------------------------------------------
// Finds the existance of the observed attribute and fills
// the pointers
int observed_attr::check_attr()
{
   char* inst;

   if (pObservedMO == NULLMO) 
   {
     advise (
       NULL,
       "Error: NULLMO  in observed_attr::check_attr()\n"
	    );
     delete inst;
     _pvObservedMOattribute = NULL;
     return NOTOK;
   } 
   else 
   {
     _strObservedAttr =
        oid2name(_oidObservedAttr, OIDPART);       // only the last name
     _pvObservedMOattribute =
        pObservedMO->getAttr(_oidObservedAttr);   // NOT with string arg
     if (_pvObservedMOattribute == NULLVD) 
     {
        // convert OID to Attribute pointer inside MO
        advise(
           NULL,
           "Error: no attribute %s in MO \n",
           _strObservedAttr
         );
	 delete inst;
         return NOTOK;
      }
   }

   //
   // Determine type 
   //
   char* sntx = _pvObservedMOattribute->getSyntax();
   if (strcasecmp(sntx, "ObservedValue") == 0) 
   {
     _gaugeType = ((Gauge*)(_pvObservedMOattribute))->getType();
     _observedAttrType = MS_OA_GAUGE;
   }
   else
     if (strcasecmp(sntx, "Count") == 0) 
     {
       _gaugeType = OV_INTEGER;
       _observedAttrType = MS_OA_COUNTER;
     } 
     else
       if (strcasecmp(sntx, "SnmpInteger") == 0 ||
           strcasecmp(sntx, "SnmpCounter") == 0 ||
	   strcasecmp(sntx, "SnmpGauge"  ) == 0   )
       {
         _gaugeType        = OV_INTEGER ;
	 _observedAttrType = MS_OA_SNMP ;
       }
       else
         {
           advise(NULL, "attribute %s not a gauge, counter or SNMP type\n",
                 _strObservedAttr);
           return NOTOK;
         }
 
    return OK;
} //observed_attr::check_attr


// --------------------------------------------------------------------------
// Updates the value of the attribute
double  observed_attr::getval_attr()
{
  double iGaugeValue = 0;

  switch (this->GetOAType())  // Depending on type ...
  {
    case MS_OA_GAUGE:
      switch(this->GetGType()) {
        case OV_INTEGER:
          iGaugeValue = 
            ((ObservedValue*) forceGet (pObservedMO, _oidObservedAttr))
            -> ov_integer;
          break;

        case OV_REAL:
          iGaugeValue =
            ((ObservedValue*)forceGet (pObservedMO, _oidObservedAttr))
            -> ov_real;
          break;
      }
      break;
     case MS_OA_COUNTER:
      iGaugeValue =  *(int*)(forceGet (pObservedMO, _oidObservedAttr));
      break;
     case MS_OA_SNMP:
      iGaugeValue =  *(int*)(forceGet (pObservedMO, _oidObservedAttr));
      break ;
  }   

  // Return current sample....
  return(iGaugeValue);
} //observed_attr::getval_attr


// --------------------------------------------------------------------------
void observed_attr::print_oa()
{
  fprintf(stdout, "\n-------- observed_attr ------------\n");
  fprintf(stdout, "Class Name->%s\n", pObservedMO->getClassName());
  return;
}


// --------------------------------------------------------------------------
//
// force update of watched MO attribute
//
void* observed_attr::forceGet (MO* pmo, OID oid) 
{

  if (pmo==NULL) return NULL;

  void*         attrValues[1];
  Attr* attr = pmo->getUpdatedAttr(oid, DISCARDAVA);
  if (!attr)
    return NULLVD;

  return attr->get();

} //observed_attr::forceGet

