//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// AvailabilityStatus.cc

// By Saleem Bhatti


#include <stdlib.h>
#include "msap.h"
#include "AvailabilityStatus.h"


AvailabilityStatus::AvailabilityStatus()
{
    AvailabilityStatusValue *v = (AvailabilityStatusValue *) malloc(sizeof(AvailabilityStatusValue));
    v->nValues = 0;
    v->values = NULL;
    setval(v);
}

AvailabilityStatus::AvailabilityStatus(AvailabilityStatusValue *v)
{ set(v); }

int
AvailabilityStatus::add(void *v)
{
    register
    int i, *ip, values, nValues;
    register
    AvailabilityStatusValue *set1 = (AvailabilityStatusValue *) v,
                            *set2 = (AvailabilityStatusValue *) getval();

    if (set1->nValues == 0) return(NOTOK); /* "empty set" */

    for(i = 0, ip = set2->values, values = 0; i < set2->nValues; ++i, ++ip)
         values |= (1 << *ip); /* "old set" */

    nValues = set2->nValues;
    for(i = 0, ip = set1->values; i < set1->nValues; ++i, ++ip) {
         if (values & (1 << *ip)) continue; /* Don't count duplicates */

         ++nValues;
         values |= (1 << *ip); /* "set union" */
    }

    set2->nValues = nValues;
    if (set2->values != NULL) free((char *) set2->values);
    set2->values = nValues > 0 ? (int *) malloc(nValues * sizeof(int)) : NULL;
    for(i = 0, nValues = 0, ip = set2->values; nValues < set2->nValues; ++i) {
         if (values & (1 << i)) {
              *ip = i;
              ++ip;
              ++nValues;
         }
    }
    replval((void *) set2);
    return(OK);
}

int
AvailabilityStatus::add(int member)
{
    AvailabilityStatusValue v;
    int value;

    v.nValues = 1;
    v.values = &value;
    value = member;
    return(add((void *) &v));
}

int
AvailabilityStatus::remove(void *v)
{
    register
    int i, *ip, values, nValues;
    register
    AvailabilityStatusValue *set1 = (AvailabilityStatusValue *) v,
                            *set2 = (AvailabilityStatusValue *) getval();

    if (set1->nValues == 0) return(NOTOK); /* "empty set" */

    for(i = 0, ip = set2->values, values = 0; i < set2->nValues; ++i, ++ip)
         values |= (1 << *ip); /* "old set" */

    nValues = set2->nValues;
    for(i = 0, ip = set1->values; i < set1->nValues; ++i, ++ip) {
        if (values & (1 << *ip)) { /* Don't count non-members */
            --nValues;
	    values ^= (1 << *ip); /* "set difference" */
	}
    }

    set2->nValues = nValues;
    if (set2->values != NULL) free((char *) set2->values);
    set2->values = nValues > 0 ? (int *) malloc(nValues * sizeof(int)) : NULL;
    for(i = 0, nValues = 0, ip = set2->values; nValues < set2->nValues; ++i) {
        if (values & (1 << i)) {
            *ip = i;
            ++ip;
            ++nValues;
        }
    }
    replval((void *) set2);
    return(OK);
}

int
AvailabilityStatus::remove(int member)
{
    AvailabilityStatusValue v;
    int value;

    v.nValues = 1;
    v.values = &value;
    value = member;
    return(remove((void *) &v));
}

Bool
AvailabilityStatus::isMember(int member)
{
    register
    AvailabilityStatusValue *v = (AvailabilityStatusValue *) getval();
    register
    int i, *ip;
    Bool found;

    for(found = False, i = 0, ip = v->values; i < v->nValues; ++i, ++ip) {
        if (member == *ip) {
            found = True;
            break;
        }
    }

    return(found);
}

Bool
AvailabilityStatus::filter(int type, void* val)
{
    AvailabilityStatusValue *as0 = (AvailabilityStatusValue *) val;
    AvailabilityStatusValue *as1 = (AvailabilityStatusValue *) get();

    register
    int av0, av1, i, *ip;

    av0 = 0;
    for(i = 0, ip = as0->values; i < as0->nValues; ++i, ++ip)
	av0 |= (1 << *ip);
    av1 = 0;
    for(i = 0, ip = as1->values; i < as1->nValues; ++i, ++ip)
	av1 |= (1 << *ip);

    Bool result;

    switch (type) {

    case FI_Equality :
	result = av0 == av1 ? True : False;
	break;

    case FI_GreaterOrEqual :
	if (as0->nValues > 1) result = False;
	else result = ffs(av0) >= ffs(av1) ? True : False;
	break;

    case FI_LessOrEqual :
	if (as0->nValues > 1) result = False;
	else result = ffs(av0) <= ffs(av1) ? True : False;
	break;

    case FI_SubsetOf :
	result = (av0 & av1) == av0 ? True : False;
	break;

    case FI_SupersetOf :
	result = (av0 & av1) == av1 ? True : False;
        break;

    case FI_NonNullSetIntersect :
	result = av0 & av1 ? True : False;
        break;

    default :
	result = False;
        break;
    }
    return(result);
}
