//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// GaugeThld.h

// By George Pavlou, February 1991


#ifndef _GaugeThld_h_
#define _GaugeThld_h_


#ifndef _GENERIC_ATTR_
#include "GenericAttr.h"
#endif

#ifndef _SMI_SNTX_
#include "SmiSntx.h"
#endif

#ifndef _Gauge_h_
#include "Gauge.h"
#endif


class GaugeThreshold : public GThreshold
{
    int    _type;

protected:
    inline PE	 _encode ()		{ return GaugeThreshold_encode(
					  (GaugeThresholdVal*) getval()); }
    inline void* _decode (PE pe)	{ return GaugeThreshold_decode(pe); }
    inline void	 _free ()		{ GaugeThreshold_free(
					  (GaugeThresholdVal*) getval()); }
    inline void* _parse (char* str)	{ return GaugeThreshold_parse(str); }
    inline char* _print ()		{ return attrv2str(getval(),
					  (PRINT_FNX) GaugeThreshold_print); }
    inline int	 _compare (void* v1, void* v2)
					{ return GaugeThresholdVal_cmp(
					  (GaugeThresholdVal*) v1,
					  (GaugeThresholdVal*) v2); }
    inline void** _getnext (void* v)	{ return (void**)GaugeThreshold_getnext
					  ((GaugeThresholdVal*) v); }

    inline GaugeThreshold (int type) : GThreshold ()
					{ _type = type; }

public:
    int		set (void*);
    Switch	check (double, double);
    int		add (double, Switch, double, Switch);
    inline int	add (void* v)		{ return Attr::add(v); }
    void	associateGauge (Gauge*);
    int		setType (int);
    inline char* getSyntax ()		{ return "GaugeThreshold"; }

    inline GaugeThreshold ()		{ _type = OV_INTEGER; setval(NULL); }
};


class GaugeThresholdInt : public GaugeThreshold
{
public:
    inline	GaugeThresholdInt ()	: GaugeThreshold(OV_INTEGER) {}
    inline	GaugeThresholdInt (int low, Switch slow, int high, Switch shigh)
					: GaugeThreshold(OV_INTEGER)
					{ add(low, slow, high, shigh); }
};


class GaugeThresholdReal : public GaugeThreshold
{
public:
    inline	GaugeThresholdReal ()	: GaugeThreshold(OV_REAL) {}
    inline	GaugeThresholdReal (double low, Switch slow,
				    double high, Switch shigh)
					: GaugeThreshold(OV_REAL)
					{ add(low, slow, high, shigh); }
};


#endif
