//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// OctetString.cc

// By George Pavlou, February 1991


#include <stdlib.h>
#include <string.h>
#include "msap.h"
#include "Error.h"
#include "OctetString.h"


Bool  OctetString::filter (int type, void* value)
{
    qbuf* qbval = (qbuf*) get();
    int   thislen = ((qbuf*) value) -> qb_len,   len = qbval -> qb_len;
    char *thisval = ((qbuf*) value) -> qb_data, *val = qbval -> qb_data;

    int cmp = memcmp(thisval, val, len < thislen ? len : thislen);

    switch (type) {
    case FI_Equality:
	if (thislen == len && cmp == 0)
	    return True;
	return False;

    case FI_GreaterOrEqual:
	if (cmp > 0 || (cmp == 0 && thislen >= len))
	    return True;
	return False;

    case FI_LessOrEqual:
	if (cmp < 0 || (cmp == 0 && thislen <= len))
	    return True;
	return False;

    default:
	return False;
    }
}


Bool  OctetString::filterSubstring (int type, void* value, Bool first)
{
    static char* val;
    static int   ln;

    if (first) {
	qbuf* qbval = (qbuf*) get();
	val = qbval -> qb_data;
	ln  = qbval -> qb_len;
    }

    char *cp, *thisval = ((qbuf*) value) -> qb_data;
    int   l,   thisln  = ((qbuf*) value) -> qb_len;

    if (thisln > ln)
        return False;

    switch (type) {
    case FI_Substring_Initial:
	if (memcmp(val, thisval, thisln) == 0) {
	    val += thisln;
	    ln  -= thisln;
	    return True;
	}
	return False;

    case FI_Substring_Any:
	cp = val;
	l  = ln;
	while ((cp = (char*) memchr(cp, *thisval, l)) != NULLCP) {
	    if ((l = ln-abs(cp-val)) < thisln)
		return False;
	    if (memcmp(cp, thisval, thisln) == 0) {
		val = cp + thisln;
		ln  = l - thisln;
		return True;
	    }
	    ++cp; --l;		// to avoid an infinite loop...
	}
	return False;

    case FI_Substring_Final:
	if (memcmp(val+ln-thisln, thisval, thisln) == 0)
	    // does not need to adjust val and ln anymore...
	    return True;
	return False;

    default:
	return False;
    }
}

