/*
 * Copyright (c) 1990 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* SmiSntx.c - SMI syntaxes */

/* By George Pavlou / James Cowan / Saleem Bhatti, May 1991 */


#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <isode/quipu/ds_search.h>
#include "msap.h"
#include "SmiSntx.h"


#define BUFSZ	512

extern int  lexequ ();
extern int  sfree ();

extern int  empty_asn1_type;	/* from the msap library (attrv.c) */
extern PE prnt_pe ;
char   sntxbuf [BUFSZ];		/* to be used by all syntax "methods" */
void*  getelem;

char* 
tidyString (str, isset)
char* str;
Bool isset;
{
    char *buf = NULLCP ;
    register char *s, *t, *bs;

    if( (str == NULLCP) || (buf = (char*)malloc( BUFSZ )) == NULLCP )
    	return NULLCP ;

    strncpy(buf, str, BUFSZ);

    if (!isset) {
	s = buf;
	t = buf + strlen(buf) - 1;
    }
    else {	/* isset */
	if ((s = strstr(buf, "{")) == NULLCP ||
			(t = strstr(buf, "}")) == NULLCP) {
	    (void)free( buf ) ;
	    return NULLCP;
	}
			
	++s; *t = '\0'; --t;
    }

    /* remove preceding and trailing spaces */
    for (; *s == ' '; s++);
    for (; *t == ' '; t--);
    if (isset && *t == '}')
	*t = '\0';
    else
	*(t+1) = '\0';

    if( s != buf )
        strcpy( buf, s ) ; /* Ensure that the pointer used to free the malloc'ed
			      buffer points to the beggining of the block */
    return buf ;
}


char*  hexprint (str, len)
char* str;
int len;
{
    char *cp = (char*) malloc(3*len + 1), *tp = cp;
    if (!cp)
	return NULLCP;

    while (len--) {
	sprintf(tp, "\\%02x", (unsigned char)*str);
	tp += 3; str++;
    }
    return cp;
}


int  enumint_print (ps, val, strarray, size)
PS ps;
int* val;
char** strarray;
int size;
{
    if (*val < 0 || *val >= size)
	ps_printf(ps, "unknown");
    ps_printf(ps, "%s", strarray[*val]);
    return OK;
}


int*  enumint_parse (val, strarray, size)
char* val;
char** strarray;
int size;
{
    register int i;
    int* v;
    for (i = 0; i < size; i++)
	if (strcasecmp(val, strarray[i]) == 0) {
	    if ((v = (int*) malloc(sizeof(int))) == NULLIP)
		return NULLIP;
	    *v = i;
	    return v;
	}
    return NULLIP;
}


int  enumint_str_check (val, strarray, size)
char* val;
char** strarray;
int size;
{
    register int i;
    int* v;
    for (i = 0; i < size; i++)
	if (strcasecmp(val, strarray[i]) == 0)
	    return OK;
    return NOTOK;
}


int enumint_list_print (ps, val, getelem, getnext, strarray, size)
PS ps;
void* val;
IFP getelem;
IFP getnext;
char** strarray;
int size;
{
    void** v = &val;

    ps_printf(ps, "{");
    if (*v) {
        ps_printf(ps, " ");
        while (*v) {
	    enumint_print(ps, *(int**) getelem(*v), strarray, size);
	    if (*(v = (void**) getnext(*v)))
	        ps_printf(ps, " ");
        }
    }
    ps_printf(ps, " }");
    return OK;
}


IntegerListVal*  enumint_list_parse (val, strarray, size)
char* val;
char** strarray;
int size;
{
    StringListVal* slist = StringList_parse(val);
    register StringListVal* selem;
    IntegerListVal* ilist;
    register IntegerListVal **ielem = &ilist;
    int* ip;

    if (! (selem = slist))
	return NULL;

    while (selem) {
	if (enumint_str_check(selem->string, strarray, size) != OK) {
	    StringList_free(slist);
	    return NULL;
	}
	selem = selem->next;
    }

    selem = slist;
    while (selem) {
	*ielem = (IntegerListVal*) malloc(sizeof(IntegerListVal));
	ip = enumint_parse(selem->string, strarray, size);
	(*ielem)->integer = *ip; free((char*) ip);
	ielem = &(*ielem)->next;
	selem = selem->next;
    }
    *ielem = NULL;
    
    StringList_free(slist);
    return ilist;
}


int enumint_list_check (val, getelem, getnext, size)
void* val;
IFP getelem;
IFP getnext;
int size;
{
    void** v = &val;

    while (*v) {
	if (enumint_check(*(int**) getelem(*v), size) != OK)
	    return NOTOK;
	v = (void**) getnext(*v);
    }
    return OK;
}


int list_print (ps, val, getelem, getnext, elem_print)
PS ps;
void* val;
IFP getelem;
IFP getnext;
IFP elem_print;
{
    void** v = &val;

    ps_printf(ps, "{");
    if (*v) {
        ps_printf(ps, " ");
        while (*v) {
	    elem_print(ps, *(void**) getelem(*v), 0);
	    if (*(v = (void**) getnext(*v)))
	        ps_printf(ps, " ");
        }
    }
    ps_printf(ps, " }");
    return OK;
}


/* the following will work at the moment only for 0 nofspaces */

void* list_parse (str, getelem, getnext, elem_parse, nofspaces)
char* str;
IFP getelem;
IFP getnext;
IFP elem_parse;
int nofspaces;
{
    void *val = NULL, **next = &val;
    int len;
    char *tcp;

    if (nofspaces > 0) {
	fprintf(stderr, "list_parse: only 0 nofspaces supported at present!");
	return NULL;
    }

    empty_asn1_type = 0;
    if ( (tcp = str = tidyString(str, True)) == NULLCP )
        return NULL;

    if (strlen(str) == 0) {
	empty_asn1_type = 1;
	free(tcp) ;
	return NULL;
    }

    while (str != '\0') {
	if (sscanf(str, "%s", sntxbuf) <= 0) {
	    free(tcp) ;
	    return val;
	}

	len = strlen(sntxbuf);
	str = strstr(str, sntxbuf);	/* to skip preceding blanks */
	str += len;

	if ((*next = (void*) malloc(2*sizeof(void*))) == NULL) {
	    free(tcp) ;
	    return NULL;
	}
	*(void**) getnext(*next) = NULL;
	if (! (*(void**) getelem(*next) = (void*) elem_parse(sntxbuf))) {
	    /* list_free(val); */	/* OK, not perfect but this error
					   is likely to happen either for GUIs
					   or in the development stage only */
	    free(tcp);
	    return NULL;
	}

	next = (void**) getnext(*next);
    }

    free(tcp);
    
    return val;
}




/* Integer */

int  atoi();

PE  int_enc (i)
int *i;
{
    return int2prim(*i);
}


int  *int_dec (pe)
PE pe;
{
    int *i;

    if ((i = (int *)malloc(sizeof(int))) == NULL)
	return(NULL);
    if ((*i = prim2num(pe)) == NOTOK && pe -> pe_errno != PE_ERR_NONE)
	return NULL;

    return i;
}


int  *int_parse (str)
char *str;
{
    int *i;

    if ((i = (int *)malloc(sizeof(int))) == NULL)
        return(NULL);

    *i = atoi(str);
    return(i);
}


/* ARGSUSED */
int_print (ps, i, format)
PS ps;
int *i, format;
{
    ps_printf(ps, "%d", *i);
}


int int_free(i)
int  *i;
{
    free((char *) i);
    return(0);
}


int  *int_cpy (i)
int *i;
{
    int *new_i;

    if ((new_i = (int *)malloc(sizeof(int))) == NULL)
        return(NULL);

    *new_i = *i;
    return new_i;
}


int  int_cmp (i, j)
register int *i, *j; 
{
    return (*i == *j ? 0 : (*i > *j ? 1 : -1) );
}


void  Integer_syntax ()
{
    (void) add_attribute_syntax("Integer",
		(IFP) int_enc,		(IFP) int_dec,
		(IFP) int_parse,	int_print,
		(IFP) int_cpy,		int_cmp,
		int_free,		NULLIFP,
		NULLIFP,		FALSE);
}


/* Boolean */

PE  bool_enc (x)
Bool *x;
{
    return bool2prim(*x == True ? 1 : 0);
}


Bool* bool_dec (pe)
PE pe;
{
    Bool *i;

    if (! test_prim_pe(pe,PE_CLASS_UNIV,PE_PRIM_BOOL))
	return NULL;

    if ((i = (Bool *)malloc(sizeof(Bool))) == NULL)
        return(NULL);

    if (prim2flag (pe) == 1)
	*i = True;
    else
	*i = False;

    return i;
}


Bool* bool_parse (x)
char* x;
{
    Bool *b; Bool i;

    if (strcasecmp(x, "true") == 0)
	i = True;
    else
    if (strcasecmp(x, "false") == 0)
	i = False;
    else
	return NULL;

    
    if ((b = (Bool*) malloc(sizeof(Bool))) == NULL)
	return NULL;

    *b = i;
    return b;
}


/* ARGSUSED */
bool_print (ps, i, format)
PS ps;
Bool *i, format;
{
    switch (*i) {
    case True:
	ps_printf(ps, "True");
	break;
    case False:
	ps_printf(ps, "False");
	break;
    default:
	ps_printf(ps, "unknown");
	break;
    }
}


Boolean_syntax ()
{
	(void) add_attribute_syntax ("Boolean",
		(IFP) bool_enc,	(IFP) bool_dec,
		(IFP) bool_parse, bool_print,
		(IFP) strdup,	lexequ,
		sfree,		NULLIFP,
		NULLIFP,	FALSE);
}


/* IntegerList */

PE  IntegerList_encode (intlist)
IntegerListVal* intlist;
{
    PE pe;
    if (build_SMI_IntegerList(&pe, 1, 0, NULLCP, intlist) == NOTOK)
        return NULLPE;
    return pe;
}


IntegerListVal*  IntegerList_decode (pe)
PE  pe;
{
    IntegerListVal* intlist;
    if (parse_SMI_IntegerList(pe, 1, NULLIP, NULLVP, &intlist) == NOTOK)
	return NULL;
    return intlist;
}


int  IntegerList_free (intlist)
IntegerListVal* intlist;
{
    free_SMI_IntegerList(intlist);
    return OK;
}


IntegerListVal*  IntegerList_parse (str)
char* str;
{
    IntegerListVal *val = NULL, **next = &val;
    int len;
    char * tcp ;

    empty_asn1_type = 0;
    if ( (tcp = str = tidyString(str, True)) == NULLCP )
        return NULL;
    if (strlen(str) == 0) {
	empty_asn1_type = 1;
	(void)free( tcp ) ;
	return NULL;
    }

    while (str != '\0') {
	if (sscanf(str, "%s", sntxbuf) <= 0) {
	    (void)free( tcp ) ;
	    return val;
	}

	len = strlen(sntxbuf);
	str = strstr(str, sntxbuf);	/* to skip preceding blanks */
	str += len;

	if ((*next = (IntegerListVal*) malloc(sizeof(IntegerListVal))) == NULL) {
	    (void)free( tcp ) ;
	    return NULL;
	}
	(*next) -> integer = atoi(sntxbuf);
	(*next) -> next = NULL;

	next = &(*next)->next;
    }

    (void)free( tcp ) ;

    return val;
}


/* ARGSUSED */
int  IntegerList_print (ps, intlist, format)
PS ps;
IntegerListVal* intlist;
int format;
{
    return list_print(ps, intlist,
	IntegerList_getelem, IntegerList_getnext, int_print);
}


int  IntegerListVal_cmp (v1, v2)
IntegerListVal *v1, *v2;
{
    return int_cmp(&v1->integer, &v2->integer);
}


int**  IntegerList_getelem (intlist)
IntegerListVal* intlist;
{
    getelem = &intlist->integer;
    return (int**) &getelem;
}


IntegerListVal**  IntegerList_getnext (intlist)
IntegerListVal* intlist;
{
    return &intlist->next;
}


void  IntegerList_syntax ()
{
    (void) add_attribute_syntax("IntegerList",
		(IFP) IntegerList_encode,
		(IFP) IntegerList_decode,
		(IFP) IntegerList_parse,
		IntegerList_print,
		NULLIFP,
		IntegerListVal_cmp,
		IntegerList_free,
		(IFP) IntegerList_getelem,
		(IFP) IntegerList_getnext,
		FALSE);
}


/* RealList */

PE  RealList_encode (realist)
RealListVal* realist;
{
    PE pe;
    if (build_SMI_RealList(&pe, 1, 0, NULLCP, realist) == NOTOK)
        return NULLPE;
    return pe;
}


RealListVal*  RealList_decode (pe)
PE  pe;
{
    RealListVal* realist;
    if (parse_SMI_RealList(pe, 1, NULLIP, NULLVP, &realist) == NOTOK)
	return NULL;
    return realist;
}


int  RealList_free (realist)
RealListVal* realist;
{
    free_SMI_RealList(realist);
    return OK;
}


RealListVal*  RealList_parse (str)
char* str;
{
    RealListVal *val = NULL, **next = &val;
    int len;
    char *tcp ;

    empty_asn1_type = 0;
    if ( (tcp = str = tidyString(str, True)) == NULLCP )
        return NULL;
    if (strlen(str) == 0) {
	empty_asn1_type = 1;
	(void)free( tcp ) ;
	return NULL;
    }

    while (str != '\0') {
	if (sscanf(str, "%s", sntxbuf) <= 0) {
	    (void)free( tcp ) ;
	    return val;
	}

	len = strlen(sntxbuf);
	str = strstr(str, sntxbuf);	/* to skip preceding blanks */
	str += len;

	if ((*next = (RealListVal*) malloc(sizeof(RealListVal))) == NULL) {
	    (void)free( tcp ) ;
	    return NULL;
	}
	(*next) -> real = atof(sntxbuf);
	(*next) -> next = NULL;

	next = &(*next)->next;
    }

    (void)free( tcp ) ;
    
    return val;
}


/* ARGSUSED */
int  RealList_print (ps, realist, format)
PS ps;
register RealListVal* realist;
int format;
{
    return list_print(ps, realist,
	RealList_getelem, RealList_getnext, real_print);
}


int  RealListVal_cmp (v1, v2)
RealListVal *v1, *v2;
{
    return real_cmp(&v1->real, &v2->real);
}


double**  RealList_getelem (realist)
RealListVal* realist;
{
    getelem = &realist->real;
    return (double**) &getelem;
}


RealListVal**  RealList_getnext (realist)
RealListVal* realist;
{
    return &realist->next;
}


void  RealList_syntax ()
{
    (void) add_attribute_syntax("RealList",
		(IFP) RealList_encode,
		(IFP) RealList_decode,
		(IFP) RealList_parse,
		RealList_print,
		NULLIFP,
		RealListVal_cmp,
		RealList_free,
		(IFP) RealList_getelem,
		(IFP) RealList_getnext,
		FALSE);
}


/* StringList */

int  StringList_free (strlist)
register StringListVal* strlist;
{
    free_SMI_IA5StringList(strlist);
    return OK;
}


StringListVal*  StringList_parse (str)
char* str;
{
    StringListVal *val = NULL, **next = &val;
    OID oid;
    int len;
    char *tcp;

    empty_asn1_type = 0;
    if ( (tcp = str = tidyString(str, True)) == NULLCP )
        return NULL;

    if (strlen(str) == 0) {
	empty_asn1_type = 1;
	(void)free( tcp ) ;
	return NULL;
    }

    while (str != '\0') {
	if (sscanf(str, "%s", sntxbuf) <= 0) {
	    (void)free( tcp ) ;
	    return val;
	}

	len = strlen(sntxbuf);
	str = strstr(str, sntxbuf);	/* to skip preceding blanks */
	str += len;

	if ((*next = (StringListVal*) malloc(sizeof(StringListVal))) == NULL) {
	    (void)free( tcp ) ;
	    return NULL;
	}
	(*next) -> string = strdup(sntxbuf);
	(*next) -> next = NULL;

	next = &(*next)->next;
    }

    (void)free( tcp ) ;
    
    return val;
}


/* ARGSUSED */
int  StringList_print (ps, strlist, format)
PS ps;
register StringListVal* strlist;
int format;
{
    return list_print(ps, strlist,
	StringList_getelem, StringList_getnext, strprint);
}


int  StringListVal_cmp (v1, v2)
StringListVal *v1, *v2;
{
    return strcmp(v1->string, v2->string);
}


char**  StringList_getelem (strlist)
StringListVal* strlist;
{
    return &strlist->string;
}


StringListVal**  StringList_getnext (strlist)
StringListVal* strlist;
{
    return &strlist->next;
}


/* GraphicStringList */

PE  GraphicStringList_encode (strlist)
StringListVal* strlist;
{
    PE pe;
    if (build_SMI_GraphicStringList(&pe, 1, 0, NULLCP, strlist) == NOTOK)
        return NULLPE;
    return pe;
}


StringListVal*  GraphicStringList_decode (pe)
PE  pe;
{
    StringListVal* strlist;
    if (parse_SMI_GraphicStringList(pe, 1, NULLIP, NULLVP, &strlist) == NOTOK)
	return NULL;
    return strlist;
}


void  GraphicStringList_syntax ()
{
    (void) add_attribute_syntax("GraphicStringList",
		(IFP) GraphicStringList_encode,
		(IFP) GraphicStringList_decode,
		(IFP) StringList_parse,
		StringList_print,
		NULLIFP,
		StringListVal_cmp,
		StringList_free,
		(IFP) StringList_getelem,
		(IFP) StringList_getnext,
		FALSE);
}


/* IA5StringList */

PE  IA5StringList_encode (strlist)
StringListVal* strlist;
{
    PE pe;
    if (build_SMI_IA5StringList(&pe, 1, 0, NULLCP, strlist) == NOTOK)
        return NULLPE;
    return pe;
}


StringListVal*  IA5StringList_decode (pe)
PE  pe;
{
    StringListVal* strlist;
    if (parse_SMI_IA5StringList(pe, 1, NULLIP, NULLVP, &strlist) == NOTOK)
	return NULL;
    return strlist;
}


void  IA5StringList_syntax ()
{
    (void) add_attribute_syntax("IA5StringList",
		(IFP) IA5StringList_encode,
		(IFP) IA5StringList_decode,
		(IFP) StringList_parse,
		StringList_print,
		NULLIFP,
		StringListVal_cmp,
		StringList_free,
		(IFP) StringList_getelem,
		(IFP) StringList_getnext,
		FALSE);
}


/* PrintableStringList */

PE  PrintableStringList_encode (strlist)
StringListVal* strlist;
{
    PE pe;
    if (build_SMI_PrintableStringList(&pe, 1, 0, NULLCP, strlist) == NOTOK)
        return NULLPE;
    return pe;
}


StringListVal*  PrintableStringList_decode (pe)
PE  pe;
{
    StringListVal* strlist;
    if (parse_SMI_PrintableStringList(pe, 1, NULLIP, NULLVP, &strlist) == NOTOK)
	return NULL;
    return strlist;
}


void  PrintableStringList_syntax ()
{
    (void) add_attribute_syntax("PrintableStringList",
		(IFP) PrintableStringList_encode,
		(IFP) PrintableStringList_decode,
		(IFP) StringList_parse,
		StringList_print,
		NULLIFP,
		StringListVal_cmp,
		StringList_free,
		(IFP) StringList_getelem,
		(IFP) StringList_getnext,
		FALSE);
}


/* Real */

double  atof();

PE  real_enc (real)
double *real;
{
    return double2prim(*real);
}


double  *real_dec (pe)
PE pe;
{
    double *real;

    if ((real = (double *)malloc(sizeof(double))) == NULL)
	return(NULL);
    *real = prim2real(pe);

    return real;
}


double  *real_parse (str)
char *str;
{
    double *real;

    if ((real = (double *)malloc(sizeof(double))) == NULL)
        return(NULL);

    *real = atof(str);
    return(real);
}


/* ARGSUSED */
real_print (ps, real, format)
PS ps;
double *real;
int format;
{
    ps_printf(ps, "%f", *real);
}


int real_free(real)
double  *real;
{
    free((char *) real);
    return(0);
}


double  *real_cpy (real)
double  *real;
{
    double *new_real;

    if ((new_real = (double *)malloc(sizeof(double))) == NULL)
        return(NULL);

    *new_real = *real;
    return new_real;
}


int  real_cmp (i, j)
register double *i, *j; 
{
    return (*i == *j ? 0 : (*i > *j ? 1 : -1) );
}


void  Real_syntax ()
{
    (void) add_attribute_syntax("Real",
		(IFP) real_enc,		(IFP) real_dec,
		(IFP) real_parse,	real_print,
		(IFP) real_cpy,		real_cmp,
		real_free,		NULLIFP,
		NULLIFP,		FALSE);
}


/* SimpleNameType */

PE  SimpleNameType_encode (name)
SimpleNameTypeVal* name;
{
    PE pe;
    if (build_SMI_SimpleNameType(&pe, 1, 0, NULLCP, name) == NOTOK)
	return NULLPE;
    return pe;
}


SimpleNameTypeVal*  SimpleNameType_decode (pe)
PE pe;
{
    SimpleNameTypeVal* name;

    if (parse_SMI_SimpleNameType(pe, 1, NULLIP, NULLVP, &name) == NOTOK)
	return NULL;

    return name;
}


SimpleNameTypeVal*  SimpleNameType_parse (str)
char* str;
{
    SimpleNameTypeVal* name;
    int type = SN_NUMBER; register char* cp, *tcp;

    if ( (tcp = str = tidyString(str, False)) == NULLCP )
        return NULL;

    for (cp = str; *cp != '\0'; cp++)
	if (!isdigit(*cp)) {
	    type = SN_STRING;
	    break;
	}

    if ((name = (SimpleNameTypeVal*) malloc(sizeof(SimpleNameTypeVal))) == NULL) {
        (void)free( tcp ) ;
	return NULL ;
    }

    switch (name -> sn_type = type) {
    case SN_NUMBER:
	name -> sn_number = atoi(str);
	break;
    case SN_STRING:
	name -> sn_string = strdup(str);
	break;
    }

    (void)free( tcp ) ;
    
    return name;
}


/* ARGSUSED */
int  SimpleNameType_print (ps, name, format)
PS ps;
register SimpleNameTypeVal* name;
int format;
{
    if (name -> sn_type == SN_NUMBER)
	ps_printf(ps, "%d", name -> sn_number);
    else
	ps_printf(ps, "%s", name -> sn_string);

    return OK;
}


SimpleNameTypeVal*  SimpleNameType_cpy (name)
SimpleNameTypeVal* name;
{
    SimpleNameTypeVal* cpy;

    if ((cpy = (SimpleNameTypeVal*) malloc(sizeof(SimpleNameTypeVal))) == NULL)
	return NULL;

    if ((cpy->sn_type = name->sn_type) == SN_NUMBER)
	cpy->sn_number = name->sn_number;
    else
	cpy->sn_string = strdup(name->sn_string);

    return cpy;
}


int  SimpleNameType_free (name)
SimpleNameTypeVal*  name;
{
    free_SMI_SimpleNameType(name);
    return OK;
}


int  SimpleNameType_cmp (sn1, sn2)
SimpleNameTypeVal *sn1, *sn2;
{
    if (sn1->sn_type != sn2->sn_type)
	return 2;
    switch (sn1->sn_type) {
    case SN_NUMBER:
	return int_cmp(&sn1->sn_number, &sn2->sn_number);

    case SN_STRING:
	return pstrcmp(sn1->sn_string, sn2->sn_string);
    }
    return 2;	/* for lint */
}


void  SimpleNameType_syntax ()
{
    (void) add_attribute_syntax("SimpleNameType",
		(IFP) SimpleNameType_encode,
		(IFP) SimpleNameType_decode,
		(IFP) SimpleNameType_parse,
		SimpleNameType_print,
		(IFP) SimpleNameType_cpy,
		SimpleNameType_cmp,
		SimpleNameType_free,
		NULLIFP, NULLIFP, FALSE);
}


/* Time */

PE  time_enc (x)
char *x;
{
    return str2prim (x, strlen(x), PE_CLASS_UNIV, PE_DEFN_UTCT);
}


char*  time_dec (pe)
PE pe;
{
    int x;
    return prim2str (pe, &x);
}


/* ARGSUSED */
int  time_print (ps, strtime, format)
PS ps;
char* strtime;
int format;
{
    /*
    char* asct = utcts2asctime(strtime);
    asct[strlen(asct) - 1] = '\0';
    */
    /* gets rid of '\n' at the end */

    ps_printf (ps, "%s", strtime);
    return OK;
}


int  time_cmp (a, b)
char   *a, *b;
{
    long    a_time,
            mdiff;
    UTC     ut;

    if ((ut = str2utct (a, strlen (a))) == NULL)
	return pstrcmp (a, b);
    a_time = gtime (ut2tm (ut));

    if ((ut = str2utct (b, strlen (b))) == NULL)
	return pstrcmp (a, b);

    return ((mdiff = a_time - gtime (ut2tm (ut))) == 0L ? 0
							: mdiff > 0L ? 1 : -1);
}


static time_syntax ()
{
    (void) add_attribute_syntax ("UTCTime",
			(IFP) time_enc,	(IFP) time_dec,
			(IFP) strdup,	time_print,
			(IFP) strdup,	time_cmp,
			sfree,		NULLIFP,
			NULLIFP,	FALSE);
}

/* MID - Management IDentifier */
/* not real syntax but used for ObjectClass, EventType etc. */

MID  MID_parse (str)
char* str;
{
    /* this routine will will only work for globalForm identifiers */
    OID oid;
    MID mid;
    if ((oid = name2oid(str)) == NULLOID ||
		(mid = (MID) malloc(sizeof(MIDentifier))) == NULLMID)
	return NULLMID;
    mid->mid_type = MID_GLOBAL;
    mid->mid_global = oid;
    return mid;
}


/* ARGSUSED */
int  MID_print (ps, mid, format)
PS ps;
MID mid;
int format;
{
    if (mid->mid_type == MID_GLOBAL)
	ps_printf(ps, "%s", oid2name(mid->mid_global, OIDPART));
    else	/* local name */
	ps_printf(ps, "%d", mid->mid_local);
    return OK;
}


int  MID_free (mid)
MID  mid;
{
    mid_free(mid);
    free((char*) mid);
    return OK;
}


int  MID_cmp (mid1, mid2)
MID mid1, mid2;
{
    if (mid1->mid_type != mid2->mid_type)
	return 2;
    if (mid1->mid_type == MID_GLOBAL)
	return oid_cmp(mid1->mid_global, mid2->mid_global);
    else	/* dn/local dn */
	return int_cmp(&mid1->mid_local, &mid2->mid_local);
}


/* MIDList - Management IDentifier List */
/* not real syntax but used for ObjectClassList, AttributeIdList etc. */

MIDListVal*  MIDList_parse (str)
char* str;
{
    MIDListVal *val = NULL, **next = &val;
    OID oid;
    int len;
    char *tcp;

    empty_asn1_type = 0;
    if ( (tcp = str = tidyString(str, True)) == NULLCP )
        return NULL;
    if (strlen(str) == 0) {
	empty_asn1_type = 1;
	(void)free( tcp ) ;
	return NULL;
    }

    while (str != '\0') {
	if (sscanf(str, "%s", sntxbuf) <= 0) {
	    (void)free( tcp ) ;
	    return val;
	}

	len = strlen(sntxbuf);
	str = strstr(str, sntxbuf);	/* to skip preceding blanks */
	str += len;

	if ((oid = name2oid(sntxbuf)) == NULLOID) {
	    if (val)
		(void) MIDList_free(val);
	    (void)free( tcp ) ;
	    return NULL;
	}

	if ((*next = (MIDListVal*) malloc(sizeof(MIDListVal))) == NULL) {
	    (void)free( tcp ) ;
	    return NULL;
	}
	(*next) -> midl_mid.mid_type = MID_GLOBAL;
	(*next) -> midl_mid.mid_global = oid;
	(*next) -> midl_next = NULL;

	next = &(*next)->midl_next;
    }

    (void)free( tcp ) ;
    
    return val;
}


/* ARGSUSED */
int  MIDList_print (ps, midlist, format)
PS ps;
register MIDListVal* midlist;
int format;
{
    return list_print(ps, midlist,
	MIDList_getelem, MIDList_getnext, MID_print);
}


int  MIDList_free (midl)
register MIDListVal* midl;
{
    free_SMI_MIDentifierList(midl);
    return OK;
}


int  MIDListVal_cmp (v1, v2)
MIDListVal *v1, *v2;
{
    return MID_cmp(&v1->midl_mid, &v2->midl_mid);
}


MID*  MIDList_getelem (midlist)
MIDListVal* midlist;
{
    getelem = &midlist->midl_mid;
    return (MID*) &getelem;
}


MIDListVal**  MIDList_getnext (midlist)
MIDListVal* midlist;
{
    return &midlist->midl_next;
}


/* AttributeId */

PE  AttributeId_encode (attrId)
MID attrId;
{
    PE pe;
    if (build_CMIP_AttributeId(&pe, 1, 0, NULLCP, attrId) == NOTOK)
	return NULLPE;
    return pe;
}


MID  AttributeId_decode (pe)
PE pe;
{
    MID attrId;
    if (parse_CMIP_AttributeId(pe, 1, NULLIP, NULLVP, &attrId) == NOTOK)
        return NULLMID;
    return attrId;
}


void  AttributeId_syntax ()
{
    (void) add_attribute_syntax("AttributeId",
		(IFP) AttributeId_encode,
		(IFP) AttributeId_decode,
		(IFP) MID_parse,
		MID_print,
		NULLIFP,
		MID_cmp,
		MID_free,
		NULLIFP, NULLIFP, FALSE);

}


/* AttributeIdList */

PE  AttributeIdList_encode (attrIdList)
MIDListVal* attrIdList;
{
    PE pe;
    if (build_CMIP_AttributeIdList(&pe, 1, 0, NULLCP, attrIdList) == NOTOK)
	return NULLPE;
    return pe;
}


MIDListVal*  AttributeIdList_decode(pe)
PE pe;
{
    MIDListVal* attrIdList;
    if (parse_CMIP_AttributeIdList(pe, 1, NULLIP, NULLVP, &attrIdList)
		== NOTOK)
        return NULL;
    return attrIdList;
}


void  AttributeIdList_syntax ()
{
    (void) add_attribute_syntax("AttributeIdList",
		(IFP) AttributeIdList_encode,
		(IFP) AttributeIdList_decode,
		(IFP) MIDList_parse,
		MIDList_print,
		NULLIFP,
		MIDListVal_cmp,
		MIDList_free,
		(IFP) MIDList_getelem,
		(IFP) MIDList_getnext,
		FALSE);
}


/* AttributeList */

PE  AttributeList_encode (attrList)
CMISParamListVal* attrList;
{
    PE pe;
    if (build_CMIP_AttributeList(&pe, 1, 0, NULLCP, attrList) == NOTOK)
	return NULLPE;
    return pe;
}


CMISParamListVal*  AttributeList_decode(pe)
PE pe;
{
    CMISParamListVal* attrList;
    if (parse_CMIP_AttributeList(pe, 1, NULLIP, NULLVP, &attrList) == NOTOK)
        return NULL;
    return attrList;
}


int  AttributeList_free (attrList)
CMISParamListVal* attrList;
{
    free_CMIP_AttributeList(attrList);
    return OK;
}


/* ARGSUSED */
int  Attribute_print (ps, elem, quoted)
PS ps;
CMISParam *elem;
int quoted;
{
    char *strType, *strVal;

    if (elem->mp_id.mid_type != MID_GLOBAL) {
        ps_printf(ps, "%d", elem->mp_id.mid_local);
	return OK;
    }

    ava2str(elem->mp_id.mid_global, elem->mp_val, &strType, &strVal);
    ps_printf(ps, "%s: %s", strType, strVal);
    free(strType); free(strVal);

    return OK;
}


/* ARGSUSED */
AttributeList_print (ps, attrList, quoted)
PS ps;
CMISParamListVal *attrList;
int quoted;
{
    return list_print(ps, attrList,
	AttributeList_getelem, AttributeList_getnext, Attribute_print);
}


CMISParam**  AttributeList_getelem (attrList)
CMISParamListVal* attrList;
{
    getelem = &attrList->mpl_ava;
    return (CMISParam**) &getelem;
}


CMISParamListVal**  AttributeList_getnext (attrList)
CMISParamListVal* attrList;
{
    return &attrList->mpl_next;
}


void  AttributeList_syntax ()
{
    (void) add_attribute_syntax("AttributeList",
		(IFP) AttributeList_encode,
		(IFP) AttributeList_decode,
		NULLIFP,
		AttributeList_print,
		NULLIFP,
		NULLIFP,
		AttributeList_free,
		(IFP) AttributeList_getelem,
		(IFP) AttributeList_getnext,
		FALSE);
}


/* ObjectClass */

PE  ObjectClass_encode (objClass)
MID objClass;
{
    PE pe;
    if (build_CMIP_ObjectClass(&pe, 1, 0, NULLCP, objClass) == NOTOK)
	return NULLPE;
    return pe;
}


MID  ObjectClass_decode (pe)
PE pe;
{
    MID objClass;
    if (parse_CMIP_ObjectClass(pe, 1, NULLIP, NULLVP, &objClass) == NOTOK)
        return NULLMID;
    return objClass;
}


void  ObjectClass_syntax ()
{
    (void) add_attribute_syntax("ObjectClass",
		(IFP) ObjectClass_encode,
		(IFP) ObjectClass_decode,
		(IFP) MID_parse,
		MID_print,
		NULLIFP,
		MID_cmp,
		MID_free,
		NULLIFP, NULLIFP, FALSE);

}


/* ObjectClassList */

PE  ObjectClassList_encode (objClassList)
MIDListVal* objClassList;
{
    PE pe;
    if (build_SMI_ObjectClassList(&pe, 1, 0, NULLCP, objClassList) == NOTOK)
	return NULLPE;
    return pe;
}


MIDListVal*  ObjectClassList_decode(pe)
PE pe;
{
    MIDListVal* objClassList;
    if (parse_SMI_ObjectClassList(pe, 1, NULLIP, NULLVP, &objClassList)
		== NOTOK)
        return NULL;
    return objClassList;
}


void  ObjectClassList_syntax ()
{
    (void) add_attribute_syntax("ObjectClassList",
		(IFP) ObjectClassList_encode,
		(IFP) ObjectClassList_decode,
		(IFP) MIDList_parse,
		MIDList_print,
		NULLIFP,
		MIDListVal_cmp,
		MIDList_free,
		(IFP) MIDList_getelem,
		(IFP) MIDList_getnext,
		FALSE);
}


/* EventType */

PE  EventType_encode (eventType)
MID eventType;
{
    PE pe;
    if (build_CMIP_EventTypeId(&pe, 1, 0, NULLCP, eventType) == NOTOK)
	return NULLPE;
    return pe;
}


MID  EventType_decode (pe)
PE pe;
{
    MID eventType;
    if (parse_CMIP_EventTypeId(pe, 1, NULLIP, NULLVP, &eventType) == NOTOK)
        return NULLMID;
    return eventType;
}


void  EventType_syntax ()
{
    (void) add_attribute_syntax("EventType",
		(IFP) EventType_encode,
		(IFP) EventType_decode,
		(IFP) MID_parse,
		MID_print,
		NULLIFP,
		MID_cmp,
		MID_free,
		NULLIFP, NULLIFP, FALSE);

}


/* ObjectInstance */

PE  ObjectInstance_encode (objInst)
MN objInst;
{
    PE pe;
    if (build_CMIP_ObjectInstance(&pe, 1, 0, NULLCP, objInst) == NOTOK)
	return NULLPE;
    return pe;
}


MN  ObjectInstance_decode(pe)
PE pe;
{
    MN objInst;
    if (parse_CMIP_ObjectInstance(pe, 1, NULLIP, NULLVP, &objInst) == NOTOK)
        return NULLMN;
    return objInst;
}


MN  ObjectInstance_parse (str)
char* str;
{
    DN dn;
    MN objInst;

    strncpy(sntxbuf, str, BUFSZ);

    if ((dn = str2dn(sntxbuf)) == NULLDN)
	return NULLMN;
    if ((objInst = (MN) malloc(sizeof(MName))) == NULLMN) {
	dn_free(dn);
	return NULLMN;
    }
    if (/* strncasecmp(str, "systemId=", 9) == 0 || */
		strncasecmp(str, "countryName=", 13) == 0 ||
		strncasecmp(str, "c=", 2) == 0)
	objInst->mn_type = MN_DN;
    else
	objInst->mn_type = MN_LOCALDN;
    objInst->mn_dn = dn;

    return objInst;
}


/* ARGSUSED */
int  ObjectInstance_print (ps, objInst, format)
PS ps;
MN objInst;
int format;
{
    char* cp = NULLCP;
    ps_printf(ps, "%s", objInst->mn_type == MN_NONSPEC ?
			objInst->mn_nonspec : (cp = dn2str(objInst->mn_dn)));
    if (cp) free(cp);
    return OK;
}


int  ObjectInstance_free (objInst)
MN objInst;
{
    mn_free(objInst);
    free((char*) objInst);
    return OK;
}


int  ObjectInstance_cmp (oi1, oi2)
MN oi1, oi2;
{
    if (oi1->mn_type != oi2->mn_type)
	return 2;
    if (oi1->mn_type == MN_NONSPEC)
	return strcmp(oi1->mn_nonspec, oi2->mn_nonspec) == 0 ? 0 : 2;
    else	/* dn/local dn */
	return dn_mgmt_order_cmp(oi1->mn_dn, oi2->mn_dn);
}


void  ObjectInstance_syntax ()
{
    (void) add_attribute_syntax("ObjectInstance",
		(IFP) ObjectInstance_encode,
		(IFP) ObjectInstance_decode,
		(IFP) ObjectInstance_parse,
		ObjectInstance_print,
		NULLIFP,
		ObjectInstance_cmp,
		ObjectInstance_free,
		NULLIFP, NULLIFP, FALSE);

}


/* Object Instance List  */

PE  ObjectInstanceList_encode (objInstList)
MNListVal* objInstList;
{
    PE pe;
    if (build_SMI_ObjectInstanceList(&pe, 1, 0, NULLCP, objInstList) == NOTOK)
	return NULLPE;
    return pe;
}


MNListVal*  ObjectInstanceList_decode(pe)
PE pe;
{
    MNListVal* objInstList;
    if (parse_SMI_ObjectInstanceList(pe, 1, NULLIP, NULLVP, &objInstList)
		== NOTOK)
        return NULL;
    return objInstList;
}


MNListVal*  ObjectInstanceList_parse (str)
char* str;
{
    MNListVal *val = NULL, **next = &val;
    MN mn;
    int len;
    char *tcp;

    empty_asn1_type = 0;
    if ( (tcp = str = tidyString(str, True)) == NULLCP )
        return NULL;
    if (strlen(str) == 0) {
	empty_asn1_type = 1;
	(void)free( tcp ) ;
	return NULL;
    }

    while (str != '\0') {
	if (sscanf(str, "%s", sntxbuf) <= 0) {
	    (void)free( tcp ) ;
	    return val;
	}

	len = strlen(sntxbuf);
	str = strstr(str, sntxbuf);	/* to skip preceding blanks */
	str += len;

	if ((mn = ObjectInstance_parse(sntxbuf)) == NULLMN) {
	    if (val)
		(void) ObjectInstanceList_free(val);
	    (void)free( tcp ) ;
	    return NULL;
	}

	if ((*next = (MNListVal*) malloc(sizeof(MNListVal))) == NULL) {
	    (void)free( tcp ) ;
	    return NULL;
	}
	(*next) -> mnl_mn = *mn;	/* struct cpy */
	(*next) -> mnl_next = NULL;
	free((char*) mn);		/* not the contents */

	next = &(*next)->mnl_next;
    }

    (void)free( tcp ) ;
    
    return val;
}


/* ARGSUSED */
int  ObjectInstanceList_print (ps, objInstList, format)
PS ps;
MNListVal* objInstList;
int format;
{
    return list_print(ps, objInstList,
	ObjectInstanceList_getelem,
	ObjectInstanceList_getnext,
	ObjectInstance_print);
}


ObjectInstanceList_free (objInstList)
register MNListVal* objInstList;
{
    /* free_SMI_ObjectInstanceList(objInstList); */

    MNListVal* prev;

    while (objInstList) {
	mn_free(&objInstList->mnl_mn);
	prev = objInstList;
	objInstList = objInstList->mnl_next;
	free((char*) prev);
    }
    return OK;
}


int  ObjectInstanceListVal_cmp (v1, v2)
MNListVal *v1, *v2;
{
    return ObjectInstance_cmp(&v1->mnl_mn, &v2->mnl_mn);
}


MN*  ObjectInstanceList_getelem (objInstList)
MNListVal* objInstList;
{
    getelem = &objInstList->mnl_mn;
    return (MN*) &getelem;
}


MNListVal**  ObjectInstanceList_getnext (objInstList)
MNListVal* objInstList;
{
    return &objInstList->mnl_next;
}


void  ObjectInstanceList_syntax ()
{
    (void) add_attribute_syntax("ObjectInstanceList",
		(IFP) ObjectInstanceList_encode,
		(IFP) ObjectInstanceList_decode,
		(IFP) ObjectInstanceList_parse,
		ObjectInstanceList_print,
		NULLIFP,
		ObjectInstanceListVal_cmp,
		ObjectInstanceList_free,
		(IFP) ObjectInstanceList_getelem,
		(IFP) ObjectInstanceList_getnext,
		FALSE);
}


/* CMIS Scope */

PE  CMISScope_encode (scope)
CMISScope* scope;
{
    PE pe;
    if (build_CMIP_Scope(&pe, 1, 0, NULLCP, scope) == NOTOK)
	return NULLPE;
    return pe;
}


CMISScope*  CMISScope_decode(pe)
PE pe;
{
    CMISScope* scope;
    if (parse_CMIP_Scope(pe, 1, NULLIP, NULLVP, &scope) == NOTOK)
        return NULLMSCOPE;
    return scope;
}


/* ARGSUSED */
CMISScope_print (ps, scope, format)
PS ps;
CMISScope* scope;
int format;
{
    ps_printf(ps, "%s", sprintscope(scope));
}


void  CMISScope_syntax ()
{
    (void) add_attribute_syntax("CMISScope",
		(IFP) CMISScope_encode,
		(IFP) CMISScope_decode,
		(IFP) str2scope,
		CMISScope_print,
		NULLIFP, NULLIFP,
		(IFP) sfree,
		NULLIFP, NULLIFP, FALSE);

}


/* CMIS Filter */

PE  CMISFilter_encode (filter)
CMISFilter* filter;
{
    PE pe;
    if (build_CMIP_CMISFilter(&pe, 1, 0, NULLCP, filter) == NOTOK)
	return NULLPE;
    return pe;
}


CMISFilter*  CMISFilter_decode(pe)
PE pe;
{
    CMISFilter* filter;
    if (parse_CMIP_CMISFilter(pe, 1, NULLIP, NULLVP, &filter) == NOTOK)
        return NULLMFILTER;
    return filter;
}


/* ARGSUSED */
CMISFilter_print (ps, filter, format)
PS ps;
CMISFilter* filter;
int format;
{
    ps_printf(ps, "%s", sprintmfilter(filter));
}


void  CMISFilter_syntax ()
{
    (void) add_attribute_syntax("CMISFilter",
		(IFP) CMISFilter_encode,
		(IFP) CMISFilter_decode,
		(IFP) str2mfilter,
		CMISFilter_print,
		NULLIFP, NULLIFP,
		(IFP) mfilter_free,
		NULLIFP, NULLIFP, FALSE);

}


/* OIDList */

PE  OIDList_encode (oidlist)
ObjIdListVal* oidlist;
{
    PE pe;
    if (build_SMI_OIDList(&pe, 1, 0, NULLCP, oidlist) == NOTOK)
        return NULLPE;
    return pe;
}


ObjIdListVal*  OIDList_decode (pe)
PE  pe;
{
    ObjIdListVal* oidlist;
    if (parse_SMI_OIDList(pe, 1, NULLIP, NULLVP, &oidlist) == NOTOK)
	return NULL;
    return oidlist;
}


ObjIdListVal*  OIDList_parse (str)
char* str;
{
    ObjIdListVal *val = NULL, **next = &val;
    OID oid;
    int len;
    char *tcp;

    empty_asn1_type = 0;
    if ( (tcp = str = tidyString(str, True)) == NULLCP )
        return NULL;
    if (strlen(str) == 0) {
	empty_asn1_type = 1;
	(void)free( tcp ) ;
	return NULL;
    }

    while (str != '\0') {
	if (sscanf(str, "%s", sntxbuf) <= 0) {
	    (void)free( tcp ) ;
	    return val;
	}

	len = strlen(sntxbuf);
	str = strstr(str, sntxbuf);	/* to skip preceding blanks */
	str += len;

	if ((oid = name2oid(sntxbuf)) == NULLOID) {
	    if (val)
		(void) OIDList_free(val);
	    (void)free( tcp ) ;
	    return NULL;
	}

	if ((*next = (ObjIdListVal*) malloc(sizeof(ObjIdListVal))) == NULL) {
	    (void)free( tcp ) ;
	    return NULL;
	}
	(*next) -> oidl_oid = oid;
	(*next) -> oidl_next = NULL;

	next = &(*next)->oidl_next;
    }

    (void)free( tcp ) ;
    
    return val;
}


/* ARGSUSED */
int  OIDList_print (ps, oidlist, format)
PS ps;
register ObjIdListVal* oidlist;
int format;
{
    return list_print(ps, oidlist,
	OIDList_getelem, OIDList_getnext, oidprint);
}


int  OIDList_free (oidlist)
ObjIdListVal* oidlist;
{
    free_SMI_OIDList(oidlist);
    return OK;
}


int  OIDListVal_cmp (v1, v2)
ObjIdListVal *v1, *v2;
{
    return oid_cmp(v1->oidl_oid, v2->oidl_oid);
}


OID*  OIDList_getelem (oidlist)
ObjIdListVal* oidlist;
{
    return &oidlist->oidl_oid;
}


ObjIdListVal**  OIDList_getnext (oidlist)
ObjIdListVal* oidlist;
{
    return &oidlist->oidl_next;
}


void  OIDList_syntax ()
{
    (void) add_attribute_syntax("OIDList",
		(IFP) OIDList_encode,
		(IFP) OIDList_decode,
		(IFP) OIDList_parse,
		OIDList_print,
		NULLIFP,
		OIDListVal_cmp,
		OIDList_free,
		(IFP) OIDList_getelem,
		(IFP) OIDList_getnext,
		FALSE);

}


/* Count */

void  Count_syntax ()
{
    (void) add_attribute_syntax("Count",
		(IFP) int_enc,		(IFP) int_dec,
		(IFP) int_parse,	int_print,
		(IFP) int_cpy,		int_cmp,
		int_free,		NULLIFP,
		NULLIFP,		FALSE);
}


/* ObservedValue */

PE  ObservedValue_encode (ovalue)
ObservedValue* ovalue;
{
    PE pe;
    if (build_SMI_ObservedValue(&pe, 1, 0, NULLCP, ovalue)
		== NOTOK)
	return NULLPE;
    return pe;
}


ObservedValue*  ObservedValue_decode (pe)
PE pe;
{
    ObservedValue* ovalue;

    if (parse_SMI_ObservedValue(pe, 1, NULLIP, NULLVP, &ovalue) == NOTOK)
	return NULL;

    return ovalue;
}


ObservedValue*  ObservedValue_parse (str)
register char* str;
{
    ObservedValue* ovalue;
    int type, ival;
    double rval;

    if (sscanf(str, "%d", &ival) == 1)
	type = OV_INTEGER;
    else
    if (sscanf(str, "%lf", &rval) == 1)
	type = OV_REAL;
    else	/* wrong string value */
	return NULL;

    if ((ovalue = (ObservedValue*) malloc(sizeof(ObservedValue))) == NULL)
	return NULL ;

    switch (ovalue -> ov_type = type) {
    case OV_INTEGER:
	ovalue -> ov_integer = ival;
	break;
    case OV_REAL:
	ovalue -> ov_real = rval;
	break;
    }
    return ovalue;
}


/* ARGSUSED */
int  ObservedValue_print (ps, ovalue, format)
PS ps;
register ObservedValue* ovalue;
int format;
{
    if (ovalue -> ov_type == OV_INTEGER)
	ps_printf(ps, "%d", ovalue -> ov_integer);
    else
	ps_printf(ps, "%.3f", ovalue -> ov_real);

    return OK;
}


int  ObservedValue_free (ovalue)
register ObservedValue* ovalue;
{
    free((char*) ovalue);
    return OK;
}


ObservedValue*  ObservedValue_cpy (ovalue)
register ObservedValue*  ovalue;
{
    register ObservedValue* new_ovalue;

    if ((new_ovalue = (ObservedValue*) malloc(sizeof(ObservedValue))) == NULL)
	return NULL;

    *new_ovalue = *ovalue;
    return new_ovalue;
}


int  ObservedValue_cmp (ov1, ov2)
ObservedValue *ov1, *ov2;
{
    if (ov1->ov_type != ov2->ov_type)
	return 2;
    switch (ov1->ov_type) {
    case OV_INTEGER:
	return ov1->ov_integer == ov2->ov_integer ? 0 :
		(ov1->ov_integer < ov2->ov_integer ? -1 : 1);
    case OV_REAL:
	return ov1->ov_real == ov2->ov_real ? 0 :
		(ov1->ov_real < ov2->ov_real ? -1 : 1);
    }
    return 2;	/* for lint */
}


void  ObservedValue_syntax ()
{
    (void) add_attribute_syntax("ObservedValue",
		(IFP) ObservedValue_encode,
		(IFP) ObservedValue_decode,
		(IFP) ObservedValue_parse,
		ObservedValue_print,
		(IFP) ObservedValue_cpy,
		ObservedValue_cmp,
		ObservedValue_free,
		NULLIFP, NULLIFP, FALSE);
}


/* CounterThreshold */

int  CounterThresholdVal_cmp (v1, v2)
CounterThresholdVal *v1, *v2;
{
    if (v1 -> ct_compLevel == v2 -> ct_compLevel &&
		v1 -> ct_offsetVal == v2 -> ct_offsetVal &&
		v1 -> ct_notifyOnOff == v2 -> ct_notifyOnOff)
	return 0;
    return 2;
}


CounterThresholdVal**  CounterThreshold_getelem (cthld)
CounterThresholdVal* cthld;
{
    getelem = cthld;
    return (CounterThresholdVal**) &getelem;
}


CounterThresholdVal**  CounterThreshold_getnext (cthld)
CounterThresholdVal* cthld;
{
    return &cthld->ct_next;
}


/* ARGSUSED */
int  CounterThresholdVal_print (ps, cthld, format)
PS ps;
register CounterThresholdVal* cthld;
int format;
{
    if (!cthld)
	return NOTOK;

    ps_printf(ps, "Level:%d Offset:%d Switch:%s",
			cthld -> ct_compLevel,
			cthld -> ct_offsetVal,
			cthld -> ct_notifyOnOff == On ? "On" : "Off");
    return OK;
}


CounterThresholdVal*  CounterThresholdVal_cpy (cthld)
register CounterThresholdVal*  cthld;
{
    register CounterThresholdVal* new_cthld;

    if (!cthld)
	return NULL;

    if ((new_cthld = (CounterThresholdVal*)
		malloc(sizeof(CounterThresholdVal))) == NULL)
	return NULL;

    *new_cthld = *cthld;	/* struct copy */
    return new_cthld;
}


PE  CounterThreshold_encode (cthld)
CounterThresholdVal* cthld;
{
    PE pe;
    if (build_SMI_CounterThreshold(&pe, 1, 0, NULLCP, cthld)
		== NOTOK)
	return NULLPE;
    return pe;
}


CounterThresholdVal*  CounterThreshold_decode (pe)
PE pe;
{
    CounterThresholdVal* cthld;
    if (parse_SMI_CounterThreshold(pe, 1, NULLIP, NULLVP, &cthld)
		== NOTOK)
	return NULL;
    return cthld;
}


CounterThresholdVal*  CounterThreshold_parse (parse_string)
char* parse_string;
{
    CounterThresholdVal *first_cthld = NULL, *next_cthld;
    int compLevel, offsetValue;
    char onOffSwitch[4];
    register char *s;
    char *tcp;

    empty_asn1_type = 0;
    if ( (tcp = parse_string = tidyString(parse_string, True)) == NULLCP )
        return NULL;
    if (strlen(parse_string) == 0) {
	empty_asn1_type = 1;
	(void)free( tcp ) ;
	return NULL;
    }
    s = parse_string;

    do
    {
    CounterThresholdVal* cthld;

	    if (sscanf(s, "Level: %d Offset: %d Switch: %3s",
				&compLevel, &offsetValue, onOffSwitch) != 3)
	    {
		/* Unexpected EOF */

		if (first_cthld)
			(void) CounterThreshold_free (first_cthld);
		(void)free( tcp ) ;
		return NULL;
	    }

	    if ((cthld = (CounterThresholdVal*)
			malloc(sizeof(CounterThresholdVal))) == NULL) {
		(void)free( tcp ) ;
		return NULL;
	    }
	    cthld -> ct_compLevel = compLevel;
	    cthld -> ct_offsetVal = offsetValue;
	    cthld -> ct_notifyOnOff =
			strncasecmp(onOffSwitch, "On", 2) == 0 ? On : Off;
	    cthld -> ct_next = NULL;
	    if (!first_cthld)
	    	first_cthld = cthld;
	    else
	    {
		if (!first_cthld->ct_next)
			first_cthld->ct_next = next_cthld = cthld;
		else
			next_cthld->ct_next = cthld;
		next_cthld = cthld;
	    }

            /* skip to next counterthreshold in string */

	    s = strstr(++s, "Level");
    }
    while (s);

    (void)free( tcp ) ;

    return first_cthld;
}


int  CounterThreshold_print (ps, cthld, format)
PS ps;
register CounterThresholdVal* cthld;
int format;
{
    ps_print(ps, "{");
    while(cthld) {
	(void) CounterThresholdVal_print(ps, cthld, format);
	if (cthld = cthld -> ct_next)
	    ps_print(ps," % ");
    }
    ps_print(ps, "}");
    return 0;
}


int  CounterThreshold_free (cthld)
register CounterThresholdVal* cthld;
{
    CounterThresholdVal* prev;

    while(cthld) {
	prev = cthld;
	cthld = cthld -> ct_next;
	free((char*) prev);
    }
    return 0;
}


void  CounterThreshold_syntax ()
{
    (void) add_attribute_syntax("CounterThreshold",
		(IFP) CounterThreshold_encode,
		(IFP) CounterThreshold_decode,
		(IFP) CounterThreshold_parse,
		CounterThreshold_print,
		NULLIFP,
		CounterThresholdVal_cmp,
		CounterThreshold_free,
		(IFP) CounterThreshold_getelem,
		(IFP) CounterThreshold_getnext,
		FALSE);
}


/* GaugeThreshold */

GaugeThresholdVal*  GaugeThresholdVal_cpy (gthld)
register GaugeThresholdVal*  gthld;
{
    register GaugeThresholdVal* new_gthld;

    if (!gthld)
	return NULL;

    if ((new_gthld = (GaugeThresholdVal*)
		malloc(sizeof(GaugeThresholdVal))) == NULL)
	return NULL;

    *new_gthld = *gthld;	/* struct copy */
    return new_gthld;
}


/* ARGSUSED */
int  GaugeThresholdVal_print (ps, gthld, format)
PS ps;
register GaugeThresholdVal* gthld;
int format;
{
    if (!gthld)
	return NOTOK;

    if (gthld -> gt_notifyLow.nt_threshold.ov_type == OV_INTEGER)
	ps_printf(ps, "Low:%d Switch:%s ",
		gthld ->gt_notifyLow.nt_threshold.ov_integer,
		gthld ->gt_notifyLow.nt_notifyOnOff == On ? "On" : "Off");
    else
	ps_printf(ps, "Low:%.2f Switch:%s ",
		gthld ->gt_notifyLow.nt_threshold.ov_real,
		gthld ->gt_notifyLow.nt_notifyOnOff == On ? "On" : "Off");
	
    if (gthld -> gt_notifyHigh.nt_threshold.ov_type == OV_INTEGER)
	ps_printf(ps, "High:%d Switch:%s",
		gthld ->gt_notifyHigh.nt_threshold.ov_integer,
		gthld ->gt_notifyHigh.nt_notifyOnOff == On ? "On" : "Off");
    else
	ps_printf(ps, "High:%.2f Switch:%s",
		gthld ->gt_notifyHigh.nt_threshold.ov_real,
		gthld ->gt_notifyHigh.nt_notifyOnOff == On ? "On" : "Off");
	
    return OK;
}


int  GaugeThresholdVal_cmp (v1, v2)
GaugeThresholdVal *v1, *v2;
{
    if (v1->gt_notifyLow.nt_threshold.ov_type ==
        v2->gt_notifyLow.nt_threshold.ov_type &&
	((v1->gt_notifyLow.nt_threshold.ov_type == OV_INTEGER &&
          v1->gt_notifyLow.nt_threshold.ov_integer ==
          v2->gt_notifyLow.nt_threshold.ov_integer) ||
	 (v1->gt_notifyLow.nt_threshold.ov_type == OV_REAL &&
          v1->gt_notifyLow.nt_threshold.ov_real ==
          v2->gt_notifyLow.nt_threshold.ov_real)) &&
        v1->gt_notifyLow.nt_notifyOnOff ==
        v2->gt_notifyLow.nt_notifyOnOff)
	return 0;

    return 2;
}


PE  GaugeThreshold_encode (gthld)
GaugeThresholdVal* gthld;
{
    PE pe;
    if (build_SMI_GaugeThreshold(&pe, 1, 0, NULLCP, gthld)
		== NOTOK)
	return NULLPE;
    return pe;
}


GaugeThresholdVal*  GaugeThreshold_decode (pe)
PE pe;
{
    GaugeThresholdVal* gthld;
    if (parse_SMI_GaugeThreshold(pe, 1, NULLIP, NULLVP, &gthld)
		== NOTOK)
	return NULL;
    return gthld;
}


GaugeThresholdVal*  GaugeThreshold_parse (parse_string)
char* parse_string;
{
    GaugeThresholdVal *first_gthld = NULL, *next_gthld;
    int highval, lowval;
    double rhighval, rlowval;
    char hswitch[4], lswitch[4];
    register char *s;
    char *tcp;
    int	type;

    empty_asn1_type = 0;
    if ( (tcp = parse_string = tidyString(parse_string, True)) == NULLCP )
        return NULL;
    if (strlen(parse_string) == 0) {
	empty_asn1_type = 1;
	(void)free( tcp ) ;
	return NULL;
    }
    s = parse_string;

    do
    {
    GaugeThresholdVal* gthld;

	    if (sscanf(s, "Low: %d Switch: %3s High: %d Switch: %3s",
				&lowval, lswitch, &highval, hswitch) == 4)
		type = OV_INTEGER;
            else
	    	if (sscanf(s, "Low: %lf Switch: %3s High: %lf Switch: %3s",
				&rlowval, lswitch, &rhighval, hswitch) == 4)
			type = OV_REAL;
	    else
	    {
		/* Unexpected EOF */

		if (first_gthld)
			(void) GaugeThreshold_free (first_gthld);
		(void)free( tcp ) ;
		return NULL;
	    }

	    if ((gthld = (GaugeThresholdVal*)
			malloc(sizeof(GaugeThresholdVal))) == NULL) {
		(void)free( tcp ) ;
		return NULL;
	    }
	    
	    switch (type)
	    {
	    case OV_INTEGER:
		gthld -> gt_notifyLow.nt_threshold.ov_type = OV_INTEGER;
		gthld -> gt_notifyLow.nt_threshold.ov_integer = lowval;
		gthld -> gt_notifyHigh.nt_threshold.ov_type = OV_INTEGER;
		gthld -> gt_notifyHigh.nt_threshold.ov_integer = highval;
		break;

	    case OV_REAL:
		gthld -> gt_notifyLow.nt_threshold.ov_type = OV_REAL;
		gthld -> gt_notifyLow.nt_threshold.ov_real = rlowval;
		gthld -> gt_notifyHigh.nt_threshold.ov_type = OV_REAL;
		gthld -> gt_notifyHigh.nt_threshold.ov_real = rhighval;
		break;
	    }

	    gthld -> gt_notifyLow.nt_notifyOnOff =
				strncasecmp(lswitch, "On", 2) == 0 ? On : Off;
	    gthld -> gt_notifyHigh.nt_notifyOnOff =
				strncasecmp(hswitch, "On", 2) == 0 ? On : Off;
	    gthld -> gt_next = NULL;
	    if (!first_gthld)
	    	first_gthld = gthld;
	    else
	    {
		if (!first_gthld->gt_next)
			first_gthld->gt_next = next_gthld = gthld;
		else
			next_gthld->gt_next = gthld;
		next_gthld = gthld;
	    }

            /* skip to next gaugethreshold in string */

	    s = strstr(++s, "Low");
    }
    while (s);

    (void)free( tcp ) ;

    return first_gthld;
}


int  GaugeThreshold_print (ps, gthld, format)
PS ps;
register GaugeThresholdVal* gthld;
int format;
{
    ps_print(ps, "{");
    while(gthld) {
	(void) GaugeThresholdVal_print(ps, gthld, format);
	if (gthld = gthld -> gt_next)
	    ps_print(ps," % ");
    }
    ps_print(ps, "}");
    return 0;
}


int  GaugeThreshold_free (gthld)
register GaugeThresholdVal* gthld;
{
    GaugeThresholdVal* prev;

    while(gthld) {
	prev = gthld;
	gthld = gthld -> gt_next;
	free((char*) prev);
    }
    return 0;
}


GaugeThresholdVal**  GaugeThreshold_getelem (gthld)
GaugeThresholdVal* gthld;
{
    getelem = gthld;
    return (GaugeThresholdVal**) &getelem;
}


GaugeThresholdVal**  GaugeThreshold_getnext (gthld)
GaugeThresholdVal* gthld;
{
    return &gthld->gt_next;
}


void  GaugeThreshold_syntax ()
{
    (void) add_attribute_syntax("GaugeThreshold",
		(IFP) GaugeThreshold_encode,
		(IFP) GaugeThreshold_decode,
		(IFP) GaugeThreshold_parse,
		GaugeThreshold_print,
		NULLIFP,
		GaugeThresholdVal_cmp,
		GaugeThreshold_free,
		(IFP) GaugeThreshold_getelem,
		(IFP) GaugeThreshold_getnext,
		FALSE);
}


/* Tide Mark */

PE  TideMark_encode (tmark)
TideMarkVal* tmark;
{
    PE pe;
    if (build_SMI_TideMarkInfo(&pe, 1, 0, NULLCP, tmark)
		== NOTOK)
	return NULLPE;
    return pe;
}


TideMarkVal*  TideMark_decode (pe)
PE pe;
{
    TideMarkVal* tmark;
    if (parse_SMI_TideMarkInfo(pe, 1, NULLIP, NULLVP, &tmark)
		== NOTOK)
	return NULL;
    return tmark;
}


/* ARGSUSED */
int  TideMark_print (ps, tmark, format)
PS ps;
register TideMarkVal* tmark;
int format;
{
    if (!tmark)
        return NOTOK;

    ps_printf(ps, "%s: ", tmark->tm_type == TM_MAX ? "maximum" : "minimum");

    if (tmark->tm_curValue.ov_type == OV_INTEGER)
	ps_printf(ps, "cur %d", tmark->tm_curValue.ov_integer);
    else			/* OV_REAL */
	ps_printf(ps, "cur %.2f", tmark->tm_curValue.ov_real);

    if (tmark->tm_prevValue.ov_type == OV_INTEGER)
	ps_printf(ps, " prev %d", tmark->tm_prevValue.ov_integer);
    else			/* OV_REAL */
	ps_printf(ps, " prev %.2f", tmark->tm_prevValue.ov_real);

    ps_printf(ps, " reset %s", tmark->tm_resetTime);
    return OK;
}


int  TideMark_free (tmark)
TideMarkVal* tmark;
{
    if (tmark->tm_resetTime)
	free(tmark->tm_resetTime);
    free((char*) tmark);
    return OK;
}


int  TideMark_cmp (tm1, tm2)
TideMarkVal *tm1, *tm2;
{
    char *t1 = NULLCP, *t2 = NULLCP;
    int status = 0;

    if (tm1->tm_type != tm2->tm_type ||
	    ObservedValue_cmp(&tm1->tm_curValue, &tm2->tm_curValue) != 0 ||
	    ObservedValue_cmp(&tm1->tm_prevValue, &tm2->tm_prevValue) != 0 ||
	    time_cmp(t1 = strdup(utct2str(&tm1->tm_resetTime)),
			t2 = strdup(utct2str(&tm1->tm_resetTime))) != 0)
	status = 2;

    if (t1) {
	free(t1);
	free(t2);
    }
    return status;
}


void  TideMark_syntax ()
{
    (void) add_attribute_syntax("TideMark",
		(IFP) TideMark_encode,
		(IFP) TideMark_decode,
		NULLIFP,
		TideMark_print,
		NULLIFP,
		TideMark_cmp,
		TideMark_free,
		NULLIFP, NULLIFP, FALSE);
}


/* AdministrativeState */

PE  AdministrativeState_encode (state)
AdminStateVal* state;
{
    PE pe;
    if (build_SMI_AdministrativeState(&pe, 1, 0, NULLCP, state) == NOTOK)
	return NULLPE;
    return pe;
}


AdminStateVal*  AdministrativeState_decode (pe)
PE pe;
{
    AdminStateVal* state;

    if (parse_SMI_AdministrativeState(pe, 1, NULLIP, NULLVP,
		(struct type_SMI_AdministrativeState**) &state) == NOTOK)
	return NULL;

    return state;
}


char* _AdministrativeState [] =
	{
	"locked",
	"unlocked",
	"shuttingDown"
	};
int _AdministrativeState_size =
	sizeof _AdministrativeState / sizeof _AdministrativeState[0];


AdminStateVal* AdministrativeState_parse (str)
char* str;
{
    return (AdminStateVal*)
	enumint_parse(str, _AdministrativeState, _AdministrativeState_size);
}


/* ARGSUSED */
int AdministrativeState_print (ps, val, format)
PS ps;
AdminStateVal* val;
int format;
{
    enumint_print(ps, val, _AdministrativeState, _AdministrativeState_size);
    return OK;
}


int  AdministrativeState_free(state)
AdminStateVal* state;
{
    free((char*) state);
    return OK;
}


int  AdministrativeState_cmp (as1, as2)
AdminStateVal *as1, *as2;
{
    return *as1 == *as2 ? 0 : 2;
}


void  AdministrativeState_syntax ()
{
    (void) add_attribute_syntax("AdministrativeState",
		(IFP) AdministrativeState_encode,
		(IFP) AdministrativeState_decode,
		(IFP) AdministrativeState_parse,
		AdministrativeState_print,
		NULLIFP,
		AdministrativeState_cmp,
		AdministrativeState_free,
		NULLIFP, NULLIFP, FALSE);
}


/* OperationalState */

PE  OperationalState_encode (state)
OperStateVal* state;
{
    PE pe;

    if (build_SMI_OperationalState(&pe, 1, 0, NULLCP, state) == NOTOK)
	return NULLPE;
    return pe;
}


OperStateVal*  OperationalState_decode (pe)
PE pe;
{
    OperStateVal* state;

    if (parse_SMI_OperationalState(pe, 1, NULLIP, NULLVP,
		(struct type_SMI_OperationalState**) &state) == NOTOK)
	return NULL;

    return state;
}


char* _OperationalState [] =
	{
	"disabled",
	"enabled"
	};
int _OperationalState_size =
	sizeof _OperationalState / sizeof _OperationalState[0];


OperStateVal* OperationalState_parse (str)
char* str;
{
    return (OperStateVal*)
	enumint_parse(str, _OperationalState, _OperationalState_size);
}


/* ARGSUSED */
int OperationalState_print (ps, val, format)
PS ps;
OperStateVal* val;
int format;
{
    enumint_print(ps, val, _OperationalState, _OperationalState_size);
    return OK;
}


int  OperationalState_free(state)
OperStateVal* state;
{
    free((char*) state);
    return OK;
}


int  OperationalState_cmp (os1, os2)
OperStateVal *os1, *os2;
{
    return *os1 == *os2 ? 0 : 2;
}


void  OperationalState_syntax ()
{
    (void) add_attribute_syntax("OperationalState",
		(IFP) OperationalState_encode,
		(IFP) OperationalState_decode,
		(IFP) OperationalState_parse,
		OperationalState_print,
		NULLIFP,
		OperationalState_cmp,
		OperationalState_free,
		NULLIFP, NULLIFP, FALSE);
}


/* Usage State */

PE  UsageState_encode (state)
struct type_SMI_UsageState* state;
{
    PE pe;

    if (build_SMI_UsageState(&pe, 1, 0, NULLCP, state) == NOTOK)
	return NULLPE;
    return pe;
}


struct type_SMI_UsageState*  UsageState_decode (pe)
PE pe;
{
    struct type_SMI_UsageState* state;

    if (parse_SMI_UsageState(pe, 1, NULLIP, NULLVP, &state) == NOTOK)
	return NULL;

    return state;
}


char* _UsageState [] =
	{
	"idle",
	"active",
	"busy"
	};
int _UsageState_size =
	sizeof _UsageState / sizeof _UsageState[0];


struct type_SMI_UsageState* UsageState_parse (str)
char* str;
{
    return (struct type_SMI_UsageState*)
	enumint_parse(str, _UsageState, _UsageState_size);
}


/* ARGSUSED */
int UsageState_print (ps, val, format)
PS ps;
struct type_SMI_UsageState* val;
int format;
{
    enumint_print(ps, val, _UsageState, _UsageState_size);
    return OK;
}


int  UsageState_free(state)
struct type_SMI_UsageState* state;
{
    free_SMI_UsageState(state);
    return OK;
}


int  UsageState_cmp (s1, s2)
struct type_SMI_UsageState *s1, *s2;
{
    return s1->parm == s2->parm ? 0 : 2;
}


void  UsageState_syntax ()
{
    (void) add_attribute_syntax("UsageState",
		(IFP) UsageState_encode,
		(IFP) UsageState_decode,
		(IFP) UsageState_parse,
		UsageState_print,
		NULLIFP,
		UsageState_cmp,
		UsageState_free,
		NULLIFP, NULLIFP, FALSE);
}


/* DNList */

PE  DNList_encode (dnList)
struct type_SMI_DistinguishedNameList* dnList;
{
    PE pe;
    if (build_SMI_DistinguishedNameList(&pe, 1, 0, NULLCP, dnList) == NOTOK)
	return NULLPE;
    return pe;
}


struct type_SMI_DistinguishedNameList*  DNList_decode (pe)
PE  pe;
{
    struct type_SMI_DistinguishedNameList* dnList;
    if (parse_SMI_DistinguishedNameList(pe, 1, NULLIP, NULLVP, &dnList) == NOTOK)
	return NULL;
    return dnList;
}


DN* DNList_getelem (list)
struct type_SMI_DistinguishedNameList* list;
{
    return &list->DistinguishedName;
}

struct type_SMI_DistinguishedNameList** DNList_getnext (list)
struct type_SMI_DistinguishedNameList* list;
{
    return &list->next;
}


/* ARGSUSED */
int  dnprint (ps, dn, format)
PS ps;
DN dn;
int format;
{
    char* cp;
    ps_printf(ps, "%s", cp = dn2str(dn));
    free(cp);
    return OK;
}


int  DNList_print (ps, dnList, format)
PS ps;
struct type_SMI_DistinguishedNameList* dnList;
int format;
{
    return list_print(ps, dnList, DNList_getelem, DNList_getnext, dnprint);
}


DN  str2aetdn (str)
char* str;
{
    extern DN str2aet ();	/* in msap/minitialise.c */

    if (strchr(str, '@') || strchr(str, '='))
	return str2dn(str);
    return str2aet(str);
}


struct type_SMI_DistinguishedNameList* DNList_parse (str)
char* str;
{
    return (struct type_SMI_DistinguishedNameList*)
	list_parse(str, DNList_getelem, DNList_getnext, str2aetdn, 0);
}


int DNList_free (dnList)
register struct type_SMI_DistinguishedNameList* dnList;
{
    /* free_SMI_DistinguishedNameList(dnList); */

    struct type_SMI_DistinguishedNameList* prev;

    while (dnList) {
	prev = dnList;
	dnList = dnList->next;
	dn_free(prev->DistinguishedName);
	free((char*) prev);
    }
    return OK;
}


void  DNList_syntax ()
{
    (void) add_attribute_syntax("DNList",
		(IFP) DNList_encode,
		(IFP) DNList_decode,
		(IFP) DNList_parse,
		DNList_print,
		NULLIFP,
		NULLIFP,
		DNList_free,
		(IFP) DNList_getelem,
		(IFP) DNList_getnext,
		FALSE);
}


/* DestAddress */

PE  DestAddress_encode (destAddr)
struct type_SMI_Destination* destAddr;
{
    PE pe;
    if (build_SMI_Destination(&pe, 1, 0, NULLCP, destAddr) == NOTOK)
	return NULLPE;
    return pe;
}


struct type_SMI_Destination*  DestAddress_decode (pe)
PE  pe;
{
    struct type_SMI_Destination* destAddr;
    if (parse_SMI_Destination(pe, 1, NULLIP, NULLVP, &destAddr) == NOTOK)
	return NULL;
    return destAddr;
}


struct type_SMI_Destination* DestAddress_parse (str)
char* str;
{
    struct type_SMI_Destination* destAddr;
    DN singleAET;
    struct type_SMI_DistinguishedNameList *multipleAET;

    /* skip preceding blanks */
    int i = strspn(str, " ");
    if (str[i] != '{') {	/* single AET */
	if (! (singleAET = str2aetdn(str)))
	    return NULL;
    }
    else {			/* multiple AET */
	if (! (multipleAET = DNList_parse(str)))
	    return NULL;
    }

    if ((destAddr = (struct type_SMI_Destination*)
		malloc(sizeof(struct type_SMI_Destination))) == NULL)
	return NULL;

    if (str[i] != '{') {
        destAddr->offset = type_SMI_Destination_singleAET;
        destAddr->un.singleAET = singleAET;
    }
    else {
        destAddr->offset = type_SMI_Destination_multipleAET;
        destAddr->un.multipleAET = multipleAET;
    }

    return destAddr;
}


int  DestAddress_print (ps, destAddr, format)
PS ps;
struct type_SMI_Destination* destAddr;
int format;
{
    switch (destAddr->offset) {
    case type_SMI_Destination_singleAET:
	dn_print(ps, destAddr->un.singleAET, format);
	break;

    case type_SMI_Destination_multipleAET:
	list_print(ps, destAddr->un.multipleAET,
			  DNList_getelem, DNList_getnext, dnprint);
	break;
    default:
	return NOTOK;
    }
    return OK;
}


int  DestAddress_free (destAddr)
struct type_SMI_Destination* destAddr;
{
    /* free_SMI_Destination(destAddr); */

    switch (destAddr->offset) {
    case type_SMI_Destination_singleAET:
	dn_free(destAddr->un.singleAET);
	break;

    case type_SMI_Destination_multipleAET:
	DNList_free(destAddr->un.multipleAET);
	break;

    default:
	return NOTOK;
    }
    free((char*) destAddr);
    return OK;
}


void  DestAddress_syntax ()
{
    (void) add_attribute_syntax("DestinationAddress",
		(IFP) DestAddress_encode,
		(IFP) DestAddress_decode,
		(IFP) DestAddress_parse,
		DestAddress_print,
		NULLIFP,
		NULLIFP,
		DestAddress_free,
		NULLIFP, NULLIFP, FALSE);
}


/* ObjectInfo */

PE ObjectInfo_encode (report)
struct type_SMI_ObjectInfo *report;
{
        PE pe;

        if (build_SMI_ObjectInfo(&pe, 1, 0, NULLCP, report) == NOTOK)
           return(NULLPE);
        else
           return(pe);
}


struct type_SMI_ObjectInfo* ObjectInfo_decode (pe)
PE pe;
{
        struct type_SMI_ObjectInfo *report;

        if (parse_SMI_ObjectInfo(pe, 1, NULLIP, NULLVP, &report) == NOTOK)
           return(NULL);
        else
           return(report);
}


/* ARGSUSED */
int SourceIndicator_print (ps, elem, quoted)
PS ps;
struct type_SMI_SourceIndicator *elem;
int quoted;
{
    ps_printf(ps, "SourceIndicator: ");

    switch (elem->parm) {
    case int_SMI_SourceIndicator_resourceOperation:
	ps_printf(ps, "resourceOperation");
	break;

    case int_SMI_SourceIndicator_managementOperation:
	ps_printf(ps, "managementOperation");
	break;

    case int_SMI_SourceIndicator_unknown:
    default:
	ps_printf(ps, "unknown");
	break;
    }
}


int ObjectInfo_print(ps, elem, quoted)
PS ps;
struct type_SMI_ObjectInfo *elem;
int quoted;
{
    CMISParamListVal* attrList;
    char *attrid, *attrval;

    ps_printf(ps, "ObjectInfo: ");

    if (elem->sourceIndicator)
        SourceIndicator_print(ps, elem->sourceIndicator, quoted);

    if (attrList = (CMISParamListVal*) elem->attributeList) {
        ps_print(ps, "\n{\n");
        for (; attrList; attrList = attrList->mpl_next) {
	    if (attrList->mpl_id.mid_type != MID_GLOBAL) {
	        ps_printf(ps, "attrId with local form!\n");
	        continue;
	    }
	    ava2str(attrList->mpl_id.mid_global, attrList->mpl_val,
		    &attrid, &attrval);
	    ps_printf(ps, "%s:  %s\n", attrid, attrval);
	    free(attrid); free(attrval);
        }
        ps_print(ps, "}\n");
    }

    if (elem->additionalText)
        ps_printf(ps, "additionalText: %s", (char*) elem->additionalText);

    return OK;
}


void ObjectInfo_free(theElem)
struct type_SMI_ObjectInfo* theElem;
{
    free_SMI_ObjectInfo(theElem);
}


/* free the ObjectInfo structure but DO NOT free the attribute OIDs and PEs */
   
void ObjectInfo_free_aux (elem)
struct type_SMI_ObjectInfo* elem;
{
    CMISParamListVal *attrList = (CMISParamListVal*) elem->attributeList;
    
    while (attrList) {
	attrList->mpl_id.mid_global = NULLOID;
	attrList->mpl_val = NULLPE;
        attrList = attrList->mpl_next;
    }
    free_SMI_ObjectInfo(elem);
}


void ObjectInfo_syntax()
{
        (void)add_attribute_syntax("ObjectInfo",
                (IFP) ObjectInfo_encode,
                (IFP) ObjectInfo_decode,
                NULLIFP,
                (IFP) ObjectInfo_print,
                NULLIFP,
                NULLIFP,
                (IFP) ObjectInfo_free,
                NULLIFP,
                NULLIFP, 0);
}


/* AttrValChangeInfo */

struct type_SMI_AttrValChangeInfo* AttrValChangeInfo_alloc ()
{
    struct type_SMI_AttrValChangeInfo* elem;

    if (((elem = (struct type_SMI_AttrValChangeInfo*)
		malloc(sizeof(struct type_SMI_AttrValChangeInfo))) == NULL) ||
	((elem->sourceIndicator = (struct type_SMI_SourceIndicator*)
		malloc(sizeof(struct type_SMI_SourceIndicator))) == NULL) ||
	((elem->attrValChangeDefinition =
		(struct type_SMI_AttrValChangeDefinition*)
		malloc(sizeof(struct type_SMI_AttrValChangeDefinition)))
		== NULL) ||
	((elem->attrValChangeDefinition->attrValChangeDefinition_member =
		(attrValChangeDefinition_element*)
		malloc(sizeof(attrValChangeDefinition_element))) == NULL) ||
	((elem->attrValChangeDefinition->attrValChangeDefinition_member->
		attributeId = (struct MIDentifier*)
		malloc(sizeof(struct MIDentifier))) == NULL))
	return NULL;

    elem->sourceIndicator->parm = int_SMI_SourceIndicator_unknown;
    elem->attrValChangeDefinition->next = NULL;		/* ONE attribute */
    elem->attrValChangeDefinition->attrValChangeDefinition_member->
		attributeId->mid_type = MID_GLOBAL;
    elem->attrValChangeDefinition->attrValChangeDefinition_member->
		attributeId->mid_global = NULLOID;
    elem->attrValChangeDefinition->attrValChangeDefinition_member->
		oldAttrValue = NULLPE;
    elem->attrValChangeDefinition->attrValChangeDefinition_member->
		newAttrValue = NULLPE;
    elem->additionalText = NULL;

    return elem;
}


PE AttrValChangeInfo_encode (report)
struct type_SMI_AttrValChangeInfo *report;
{
        PE pe;

        if (build_SMI_AttrValChangeInfo(&pe, 1, 0, NULLCP, report) == NOTOK)
           return(NULLPE);
        else
           return(pe);
}


struct type_SMI_AttrValChangeInfo* AttrValChangeInfo_decode (pe)
PE pe;
{
        struct type_SMI_AttrValChangeInfo *report;

        if (parse_SMI_AttrValChangeInfo(pe, 1, NULLIP, NULLVP, &report)
		== NOTOK)
           return(NULL);
        else
           return(report);
}


void AttrValChangeInfo_free(theElem)
struct type_SMI_AttrValChangeInfo* theElem;
{
   if(theElem == NULL)
     return; /* Nothing to free */
   else
     (void)free_SMI_AttrValChangeInfo(theElem);
}


/* free the AttrValChangeInfo structure but do NOT free the
   attribute OID and new value PE */

void  AttrValChangeInfo_free_aux (elem)
struct type_SMI_AttrValChangeInfo* elem;
{
    struct type_SMI_AttrValChangeDefinition* cur;

    if (!elem)
	return;

    for (cur = elem -> attrValChangeDefinition; cur; cur = cur->next) {
	if (cur->attrValChangeDefinition_member->attributeId->
							mid_type == MID_GLOBAL)
	    cur->attrValChangeDefinition_member->attributeId->
				mid_global = oid_cpy(str2oid("0.0"));
	cur->attrValChangeDefinition_member->newAttrValue = int2prim(0);
    }
    AttrValChangeInfo_free(elem);
}


/* ARGSUSED */
int AttrValChangeDefinition_print (ps, elem, quoted)
PS ps;
struct type_SMI_AttrValChangeDefinition* elem;
int quoted;
{
    char *attrid, *attrval;
    ps_print(ps, "{");
    while (elem && elem->attrValChangeDefinition_member->attributeId->
						mid_type == MID_GLOBAL) {
	ava2str(elem->attrValChangeDefinition_member->
					attributeId->mid_global,
		elem->attrValChangeDefinition_member->newAttrValue,
		&attrid, &attrval);
	ps_printf(ps, "attrId: %s newValue: %s", attrid, attrval);
	free(attrid); free(attrval);
	if (elem->attrValChangeDefinition_member->oldAttrValue) {
	    ava2str(elem->attrValChangeDefinition_member->attributeId->
								mid_global,
			elem->attrValChangeDefinition_member->oldAttrValue,
			&attrid, &attrval);
	    ps_printf(ps, " oldValue: %s", attrval);
	    free(attrid); free(attrval);
	}
	if (elem = elem->next)
	    ps_printf(ps, ", ");
    }

    ps_print(ps, "}");
    return OK;
}


int AttrValChangeInfo_print(ps, elem, quoted)
PS ps;
struct type_SMI_AttrValChangeInfo *elem;
int quoted;
{
    ps_printf(ps, "AttrValChangeInfo: ");
    if (elem->sourceIndicator) {
        SourceIndicator_print(ps, elem->sourceIndicator, quoted);
        ps_printf(ps, " ");
    }
    AttrValChangeDefinition_print(ps, elem->attrValChangeDefinition, quoted);
    if (elem->additionalText)
        ps_printf(ps, " additionalText: %s", (char*) elem->additionalText);
    return OK;
}


void AttrValChangeInfo_syntax()
{
        (void)add_attribute_syntax("AttrValChangeInfo",
                (IFP) AttrValChangeInfo_encode,
                (IFP) AttrValChangeInfo_decode,
                NULLIFP,
                (IFP) AttrValChangeInfo_print,
                NULLIFP,
                NULLIFP,
                (IFP) AttrValChangeInfo_free,
                NULLIFP,
                NULLIFP, 0);
}


/* AlarmInfo */

struct type_SMI_AlarmInfo* AlarmInfo_alloc ()
{
    struct type_SMI_AlarmInfo* elem;

    if (((elem = (struct type_SMI_AlarmInfo*)
			calloc(1, sizeof(struct type_SMI_AlarmInfo))) == NULL))
	return NULL;
    return elem;
}


struct type_SMI_AlarmInfo* ThresholdAlarmInfo_alloc ()
{
    struct type_SMI_AlarmInfo* elem;

    if (((elem = AlarmInfo_alloc()) == NULL) ||
	((elem->probableCause = (struct type_SMI_MIDentifier*)
		calloc(1, sizeof(struct type_SMI_MIDentifier))) == NULL) ||
	((elem->thresholdInfo = (struct type_SMI_ThresholdInfo*)
		calloc(1, sizeof(struct type_SMI_ThresholdInfo))) == NULL) ||
	((elem->thresholdInfo->triggeredThreshold =
		(struct MIDentifier*)
		calloc(1, sizeof(struct MIDentifier))) == NULL) ||
	((elem->thresholdInfo->observedValue =
		(struct type_SMI_ObservedValue*)
		calloc(1, sizeof(struct type_SMI_ObservedValue))) == NULL) ||
	((elem->perceivedSeverity = (struct type_SMI_PerceivedSeverity*)
		calloc(1, sizeof(struct type_SMI_PerceivedSeverity))) == NULL))
	return NULL;

    elem->probableCause->offset = type_SMI_MIDentifier_globalForm;
    elem->probableCause->un.globalForm = name2oid("thresholdCrossed");
    elem->perceivedSeverity->parm = int_SMI_PerceivedSeverity_indeterminate;
    elem->thresholdInfo->triggeredThreshold->mid_type = MID_GLOBAL;

    elem->specificProblems = NULL;
    elem->monitoredAttrs = NULL;
    elem->additionalText = NULL;
    elem->additionalInformation = NULL ;

    return elem;
}


struct type_SMI_AlarmInfo* ThresholdAlarmInfo_ProbsAttrs_alloc
						(elem, nprobs, nattrs)
struct type_SMI_AlarmInfo* elem;
int nprobs, nattrs;
{
    MIDListVal** cur1;
    CMISParamListVal** cur2;
    register int i;

    if (!elem && ((elem = ThresholdAlarmInfo_alloc()) == NULL))
	return NULL;

    for (cur1 = (MIDListVal**) &elem->specificProblems, i = 0; i < nprobs;
					cur1 = &(*cur1)->midl_next, i++) {
	if (((*cur1) = (MIDListVal*) malloc(sizeof(MIDListVal))) == NULL)
	    return NULL;
	(*cur1)->midl_mid.mid_type = MID_GLOBAL;
	(*cur1)->midl_mid.mid_global = NULLOID;
	(*cur1)->midl_next = NULL;
    }

    for (cur2 = (CMISParamListVal**) &elem->monitoredAttrs, i = 0; i < nattrs;
					cur2 = &(*cur2)->mpl_next, i++) {
	if (((*cur2) = (CMISParamListVal*)
		malloc(sizeof(CMISParamListVal))) == NULL)
	    return NULL;
	(*cur2)->mpl_id.mid_type = MID_GLOBAL;
	(*cur2)->mpl_id.mid_global = NULLOID;
	(*cur2)->mpl_val = NULLPE;
	(*cur2)->mpl_next = NULL;
    }

    return elem;
}


PE AlarmInfo_encode (report)
struct type_SMI_AlarmInfo *report;
{
    PE pe;

    if (build_SMI_AlarmInfo(&pe, 1, 0, NULLCP, report) == NOTOK)
        return NULLPE;
    return(pe);
}


struct type_SMI_AlarmInfo* AlarmInfo_decode (pe)
PE pe;
{
    struct type_SMI_AlarmInfo *report;

    if (parse_SMI_AlarmInfo(pe, 1, NULLIP, NULLVP, &report) == NOTOK)
        return NULL;
    return(report);
}


void  AlarmInfo_free (elem)
struct type_SMI_AlarmInfo* elem;
{
    if (!elem) return;
    free_SMI_AlarmInfo(elem);
}


/* free the AlarmInfo structure but do NOT free the threshold OID,
   backUpObject DN, monitoredAttrs OID, specificProblems OID -
   ThresholdAlarmInfo_alloc or ThresholdAlarmInfo_ProbsAttrs_alloc
   should have been used to allocate space/initialise */

void  AlarmInfo_free_aux (elem)
struct type_SMI_AlarmInfo* elem;
{
    CMISParamListVal* attr;
    MIDListVal *prob;

    if (!elem) return;

    if (elem->thresholdInfo)
        elem->thresholdInfo->triggeredThreshold->mid_global = NULLOID;
    if (attr = (CMISParamListVal*) elem->monitoredAttrs)
	while (attr) {
	    attr->mpl_id.mid_global = NULLOID;
	    pe_free(attr->mpl_val);
	    attr->mpl_val = NULLPE;
	    attr = attr->mpl_next;
	}
    if (prob = (MIDListVal*) elem->specificProblems)
	while (prob) {
	    prob->midl_mid.mid_global = NULLOID;
	    prob = prob->midl_next;
	}
    AlarmInfo_free(elem);
}


/* ARGSUSED */
int PerceivedSeverity_print(ps, elem, quoted)
PS ps;
struct type_SMI_PerceivedSeverity *elem;
int quoted;
{
    switch (elem->parm) {
    case int_SMI_PerceivedSeverity_critical:
	ps_printf(ps, "critical");
	break;

    case int_SMI_PerceivedSeverity_major:
	ps_printf(ps, "major");
	break;

    case int_SMI_PerceivedSeverity_minor:
	ps_printf(ps, "minor");
	break;

    case int_SMI_PerceivedSeverity_warning:
	ps_printf(ps, "warning");
	break;

    case int_SMI_PerceivedSeverity_indeterminate:
    default:
	ps_printf(ps, "indeterminate");
	break;
    }

    return OK;
}


/* ARGSUSED */
int  ThresholdInfo_print (ps, thldInfo, format)
PS ps;
struct type_SMI_ThresholdInfo* thldInfo;
int format;
{
    /* type_CMIP_AttributeId is exactly the same as MIDentifier */
    MID_print(ps, (MID) thldInfo->triggeredThreshold, format);

    ps_printf(ps, " ");

    /* type_SMI_ObservedValue is exactly the same as ObservedValue */
    ObservedValue_print(ps, (ObservedValue*) thldInfo->observedValue, format);

    return OK;
}


/* ARGSUSED */
int AlarmInfo_print(ps, elem, quoted)
PS ps;
struct type_SMI_AlarmInfo *elem;
int quoted;
{
PE  buffPrnt_pe ;

    ps_printf(ps, "AlarmInfo - probableCause: ");
    MID_print(ps, (MID) elem->probableCause, quoted);

    if (elem->specificProblems) {
      ps_printf(ps, " specificProblems: ");
      MIDList_print(ps, (MIDListVal*) elem->specificProblems, quoted);
    }

    ps_printf(ps, " perceivedSeverity: ");
    PerceivedSeverity_print(ps, elem->perceivedSeverity, quoted);

    if (elem->thresholdInfo) {
        ps_printf(ps, " thresholdInfo: ");
	ThresholdInfo_print(ps, elem->thresholdInfo, quoted);
    }

    if (elem->monitoredAttrs) {
        ps_printf(ps, " monitoredAttributes: ");
        AttributeList_print(ps, (CMISParamListVal*) elem->monitoredAttrs,
								quoted);
    }

    if (elem->additionalText)
        ps_printf(ps, " additionalText: %s: ", (char*) elem->additionalText);

    if (elem->additionalInformation) {
    	/*
 	 * May be called from ava2str() which sets up 'prnt_pe' for 'AlarmInfo'...
	 */
    	buffPrnt_pe = prnt_pe, prnt_pe = NULLPE ; 
    	AdditionalInformation_print( ps, elem->additionalInformation, 
								quoted);
	prnt_pe = buffPrnt_pe ;
    }
    return OK;
}


void AlarmInfo_syntax()
{
        (void)add_attribute_syntax("AlarmInfo",
                (IFP) AlarmInfo_encode,
                (IFP) AlarmInfo_decode,
                NULLIFP,
                (IFP) AlarmInfo_print,
                NULLIFP,
                NULLIFP,
                (IFP) AlarmInfo_free,
                NULLIFP,
                NULLIFP, 0);
}


/* GMS name syntaxes */

void  gms_name_syntaxes ()
{
    string_syntaxes();
    Integer_syntax();
    SimpleNameType_syntax();
}


/* SMI syntaxes */

void  smi_syntaxes ()
{
    /* dsap library (X.500) syntaxes used as they are */
    oid_syntax();
    dn_syntax();
    psap_syntax();
    smk_syntaxes();       /* defined in libsntx.a for SMK in X500 */
#if !defined(SECURITY) && ( !defined(USE_X500) || defined( USE_LDAP ) )
    /* Already defined in OSISEC */
    time_syntax();        /* this is modified here - see above */
#else
    objectclass_syntax(); /* To override the libdsap. defn */
    quipu_syntaxes();     /* directory access */
    certificate_syntax(); /* storing security info in the Directory */
#endif

    /* GMS naming attribute syntaxes */
    gms_name_syntaxes();

    /* DMI syntaxes */
    Boolean_syntax();
    Real_syntax();
    IntegerList_syntax();
    RealList_syntax();
    GraphicStringList_syntax();
    IA5StringList_syntax();
    PrintableStringList_syntax();
    AttributeId_syntax();
    AttributeIdList_syntax();
    ObjectClass_syntax();
    ObjectClassList_syntax();
    ObjectInstance_syntax();
    ObjectInstanceList_syntax();
    DNList_syntax();
    EventType_syntax();
    CMISScope_syntax();
    CMISFilter_syntax();
    OIDList_syntax();
    Count_syntax();
    CounterThreshold_syntax();
    ObservedValue_syntax();
    GaugeThreshold_syntax();
    TideMark_syntax();
    AdministrativeState_syntax();
    OperationalState_syntax();
    UsageState_syntax();
    DestAddress_syntax();

    ObjectInfo_syntax();
    AttrValChangeInfo_syntax();
    AlarmInfo_syntax();

    smi2_syntaxes() ;
}
