//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// TideMark.cc

// By George Pavlou, February 1991


#include <stdlib.h>
#include "ucbext.h"
#include "msap.h"
#include "Error.h"
#include "TideMark.h"


TideMark::TideMark (int type)
{
    TideMarkVal* tm = new TideMarkVal;
    bzero((char*) tm, sizeof(TideMarkVal));
    tm->tm_curValue.ov_type = tm->tm_prevValue.ov_type = OV_INTEGER; // default
    tm->tm_type = type;
    resetTime(&tm->tm_resetTime);	// now
    gauge = NULL;
    setval(tm);
}


TideMark::TideMark ()
{
    TideMarkVal* tm = new TideMarkVal;
    bzero((char*) tm, sizeof(TideMarkVal));
    tm->tm_curValue.ov_type = tm->tm_prevValue.ov_type = OV_INTEGER; // default
    tm->tm_type = TM_MAX;		// default
    resetTime(&tm->tm_resetTime);	// now
    gauge = NULL;
    setval(tm);
}


int  TideMark::setType (int type)
{
    TideMarkVal* tm = (TideMarkVal*) getval();

    switch (type) {
    case TM_MAX:
    case TM_MIN:
	tm -> tm_type = type;
	return OK;

    default:
	advise(NULLCP, "TideMark: setType: invalid type!");
	return NOTOK;
    }
}


Switch  TideMark::check (double newVal)
{
    TideMarkVal* tm = (TideMarkVal*) getval();
    Switch trigger = Off;

    switch (tm->tm_curValue.ov_type) {
    case OV_INTEGER:
	switch (tm->tm_type) {
	case TM_MAX:
	    if ((int) newVal > tm->tm_curValue.ov_integer)
		trigger = On;
	    break;

	case TM_MIN:
	    if ((int) newVal < tm->tm_curValue.ov_integer)
		trigger = On;
	    break;
	}
	if (trigger == On)
	    tm->tm_curValue.ov_integer = (int) newVal;
	break;

    case OV_REAL:
	switch (tm->tm_type) {
	case TM_MAX:
	    if (newVal > tm->tm_curValue.ov_real)
		trigger = On;
	    break;

	case TM_MIN:
	    if (newVal < tm->tm_curValue.ov_real)
		trigger = On;
	    break;
	}
	if (trigger == On)
	    tm->tm_curValue.ov_real = newVal;
	break;
    }

    if (trigger == On)
	replval(tm);
    return trigger;
}


int  TideMark::setDefault (void* dflt)
{
    if (gauge == NULL)
	return OK;

    TideMarkVal* tm = (TideMarkVal*) getval();
    if (tm->tm_curValue.ov_type == OV_INTEGER) {
	tm->tm_prevValue.ov_integer = tm->tm_curValue.ov_integer;
	tm->tm_curValue.ov_integer  = ((GaugeInt*) gauge) -> getint();
    }
    else {			// OV_REAL
	tm->tm_prevValue.ov_real = tm->tm_curValue.ov_real;
	tm->tm_curValue.ov_real  = ((GaugeReal*) gauge) -> getreal();
    }
    if (tm->tm_resetTime)
	free(tm->tm_resetTime);
    resetTime(&tm->tm_resetTime);
    
    replval(tm);
    return OK;
}


void  TideMark::associateGauge (Gauge* g)
{
    TideMarkVal* tm = (TideMarkVal*) getval();
    gauge = g;

    tm->tm_curValue.ov_type = tm->tm_prevValue.ov_type =
						gauge->getType();
    if (tm->tm_curValue.ov_type == OV_INTEGER)
	tm->tm_curValue.ov_integer = tm->tm_prevValue.ov_integer =
						((GaugeInt*) gauge)->getint();
    else			// OV_REAL
	tm->tm_curValue.ov_real = tm->tm_prevValue.ov_real =
						((GaugeReal*) gauge)->getreal();
    resetTime(&tm->tm_resetTime);

    replval(tm);
}

