//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// Time.h

// By George Pavlou, February 1991


#ifndef _Time_h_
#define _Time_h_

#include <time.h>

#ifndef _GENERIC_ATTR_
#include "GenericAttr.h"
#endif

#ifndef _SMI_SNTX_
#include "SmiSntx.h"
#endif


class Time : public Attr
{
    long	sectime;
    Bool	converted;

    void	convert ()		{ UTCtime utct;
					  tm2ut(localtime(&sectime), &utct);
					  char* strt = utct2str(&utct);
					  converted = True;
					  setval(strdup(strt)); }
public:
    inline PE	 _encode ()		{ if (!converted) convert();
					  return time_enc((char*) getval()); }
    inline void* _decode (PE pe)	{ return time_dec(pe); }
    inline void	 _free ()		{ free((char*) getval()); }
    inline void* _parse (char* str)	{ return strdup(str); }
    inline char* _print ()		{ if (!converted) convert();
					  return strdup((char*) getval()); }
    inline int	 _compare (void* v1, void* v2)
					{ return time_cmp(
						(char*) v1, (char*) v2); }
public:
    inline void* get ()			{ if (!converted) convert();
					  return getval(); }
    inline long getsec ()		{ return sectime; }
    inline UTC  getutc ()		{ if (!converted) convert();
					  char* strt = (char*) getval();
					  return str2utct(strt, strlen(strt)); }
    inline void set (long newtime)	{ converted = False; freePE();
					  sectime = newtime; }
    inline void set (UTC newtime)	{ converted = False; freePE();
#if	defined(SOLARIS) || defined(__HPUX__) || defined(AIXV3) || defined(__BSDI__)
					  sectime = mktime(ut2tm(newtime));
#else
					  sectime = timelocal(ut2tm(newtime));
#endif
					}
    inline int	set (void* newtime)	{ setval(strdup((char*) newtime));
					  converted = True; return OK; }
    inline void	set ()			{ sectime = time((long*) 0); // now
					  converted = False; freePE(); }

    inline Time ()			{ set(); }
    inline Time (long mytime)		{ sectime = mytime; converted = False; }
};


#endif
