
RDN uxObj1::makeRdn (MO*)
{
    // build the RDN

    char buf[20];
    sprintf(buf, "%s", "uxObjId=test");
    return str2rdn(buf);
}


int uxObj1::createRR (AVA*& err, void*, int)
{
    static Bool classInitialised;

    // this is a single-instanced class, so check if this is the 2nd instance

    if (MO::getClassInfo() -> getFirst() -> getClassNext()) {
	err = new AVA(m_processingFailure, "singleInstanceClass");
	return NOTOK;
    }


    // call the parent class createRR method -
    // in this case it will do nothing but for correct OO style...

    if (Top::createRR(err) == NOTOK)
	return NOTOK;


    // the wiseSaying attribute
    // set the initial value (the GDMO compiler leaves it empty)
    // only if it is not already set through CMIS

    char* val = (char*) wiseSaying() -> get();
    if (!strlen(val))
	wiseSaying() -> set("Hello world");


    // set the default value only once for the class
    // if we do not set it, it defaults to an empty string
    
    if (!classInitialised)
	_classInfo->setDefault(I_wiseSaying, new String("Hello world"));


    // the nUsers attribute
    // need to set the type first as the GDMO compiler cannot "know":
    // OV_INTEGER or OV_REAL

    nUsers() -> setType(OV_INTEGER);

    // update it

    updateNUsers();


    if (!classInitialised)
	classInitialised = True;

    return OK;
}


int  uxObj1::get (int attrId, int classLevel, AVA*& err, Bool checkOnly, int)
{
    // a fetch-upon-request policy is exercised to return the current value
    // of the nUsers gauge, so this polymorphic method is redefined

    if (classLevel != _level && classLevel != MO_ALLEVELS)
	return Top::get(attrId, classLevel, err, checkOnly);
	// will do nothing but for correct OO style...

    if (checkOnly)
	return OK;

    switch (attrId) {

    case MO_ALLATTRS:
    case I_nUsers:
	updateNUsers();	// update the nUsers attribute
	break;

    default:
	break;
    }

    return OK;
}


int  uxObj1::action (int actionId, int classLevel,
			void* actionArg, void*& actionReply, Bool& freeFlag,
			AVA*& err, Bool checkOnly, int)
{
    if (classLevel != _level)
	return Top::action(actionId, classLevel,
			actionArg, actionReply, freeFlag, err, checkOnly);
	// will do nothing but for correct OO style...

    if (checkOnly)
	return OK;	// we can always do it

    char* strList;

    switch (actionId) {

    case I_getUserNames:
	// there is no actionArg for this action (i.e. it is NULLVD)

	strList = updateNUsers();	// get their names
	actionReply = _classInfo->getActionReplyTemplate(I_getUserNames)
							->parse(strList);
	// we do not set the freeFlag which assumes True
	// i.e. the GMS will free the actionReply
	// (all parsing methods allocate space)

	break;

    default:
	break;
    }

    return OK;
}


//
//  uxObj1::updateNUsers - update the nUsers attribute
//  This is just called by uxObj1::wakeUp as a result of a prod from
//  the coordinator. 
//

char* uxObj1::updateNUsers ()
{
    //  Update the nUsers attribute (a gauge)
    FILE *commandp;
    static char buf[256];
    int nusers = 0;

    //  Run the "users" command, put the result in a buffer and scan it
    //  to find the number of users (i.e. words)

    if ((commandp = popen("users", "r")) == NULL) {
        fprintf(stderr, "Cannot open pipe to find nUsers\n");
	return NULLCP;
    }
    else {
	int nitems = fread(buf+1, 1, 256, commandp);
	pclose(commandp);
    	buf[nitems] = '\0';

	register char* cp = buf+1;
	while (cp = strchr(cp, ' ')) {
	    ++cp; ++nusers;
	}
	if (buf[1] != '\0')
	    ++nusers;	// always one more word than the separating spaces
    }

    fprintf(stdout, "uxObj1::update nUsers %d\n", nusers);

    // Adjust nUsers.

    nUsers() -> setreal(nusers);


    buf[0] = '{'; *strchr(buf, '\0') = '}';	// StringList action syntax
    return buf;
}

