#include "SmiSntx.h"


int uxObj2::createRR (AVA*& err, void*, int)
{
    static Bool classInitialised;

    // initialise instance variables

    _thldAlarmInfo = NULL;


    // call the parent class createRR method

    if (uxObj1::createRR(err) == NOTOK)
	return NOTOK;


    // the nUsersThld attribute
    // set the type in a similar fashion to the associated gauge in uxObj1

    nUsersThld() -> setType(OV_INTEGER);


    // set the initial value (the GDMO compiler leaves it empty)
    // only if it is not already set through CMIS

    if (!nUsersThld() -> get())
	nUsersThld() -> add(NUsersThldLow, On, NUsersThldHigh, On);


    // set the default value only once for the class
    // if we do not set it, it defaults to an empty gauge threshold

    if (!classInitialised)
	_classInfo->setDefault(I_nUsersThld,
		new GaugeThresholdInt(NUsersThldLow, On, NUsersThldHigh, On));


    // associate the threshold to the gauge

    uxObj1::nUsers() -> associateThreshold(nUsersThld());


    // the nUsersTideMark attribute
    // need to set the type first as the GDMO compiler cannot "know":
    // TM_MAX or TM_MIN

    nUsersTideMark() -> setType(TM_MAX);


    // associate the tide-mark to the gauge

    uxObj1::nUsers() -> associateTideMark(nUsersTideMark());

    if (!classInitialised)
	classInitialised = True;


    scheduleWakeUps(UxInterval, NULLCP, True);	// for notifications

    addAllomorphicClass(oid_cpy(uxObj1::getClassInfo()->getClass()));

    return OK;
}


int uxObj2::wakeUp (char*)
{
    uxObj1::updateNUsers();

    if (uxObj1::nUsers() -> triggeredThreshold() == On)
	triggerEvent(I_qualityofServiceAlarm, _level);

    if (uxObj1::nUsers() -> triggeredTideMark() == On)
	triggerAttributeValueChange(I_nUsersTideMark, _level);

    return OK;
}


int uxObj2::deleteRR (AVA*& err, void* info, Bool checkOnly, int)
{
    if (!checkOnly) {	// if atomicity is checked, we do nothing
			// as we are sure the simple tasks below will succeed
        // free event report structures first
        AlarmInfo_free_aux(_thldAlarmInfo);

        cancelWakeUps(NULLCP);
    }

    return uxObj1::deleteRR(err, info, checkOnly);
}


// the only reason the following method is needed is because when
// a tide-mark is set to default, the current value of the associated
// gauge is needed so we should update it

int  uxObj2::set (CMISModifyOp mode, int attrId, int classLevel,
			 void* value, AVA*& err, Bool checkOnly, int)
{
    if (classLevel != _level)
	return uxObj1::set(mode, attrId, classLevel, value, err, checkOnly);

    if (checkOnly)
	return OK;	// we can always (re)set the nUsersTideMark

    // if the attribute is the nUsersTideMark, update the associated
    // nUsers gauge as its value is needed
    switch (attrId) {

    case I_nUsersTideMark:
	if (mode == m_setToDefault)
	    uxObj1::updateNUsers();	// update the uxObj1 nUsers attribute
	break;

    default:
	break;
    }

    return OK;
}


//
// uxObj2::buildReport () - construct the reports that will be sent
// with event reports. Called by Top::processNotification when an
// Event Forwarding Discriminator's or Log's filter evaluates to true.
// 

int uxObj2::buildReport (int eventId, int classLevel,
						void*& info, Bool& freeFlag)
{
    if (classLevel != _level)
	return uxObj1::buildReport(eventId, classLevel, info, freeFlag);

    freeFlag = False;	// not to free the event info for efficiency -
			// it is free'd in the destructor

    switch (eventId) {

    case I_qualityofServiceAlarm:
	if (!_thldAlarmInfo &&
		(_thldAlarmInfo = ThresholdAlarmInfo_alloc()) == NULL)
	    return NOTOK;

	_thldAlarmInfo->thresholdInfo->triggeredThreshold->mid_global =
			_classInfo->getAttrOid(I_nUsersThld);
			// NOT to be free'd

	_thldAlarmInfo->thresholdInfo->observedValue->offset = OV_INTEGER;
	_thldAlarmInfo->thresholdInfo->observedValue->un.integer =
			((GaugeInt*) uxObj1::nUsers()) -> getint();

	info = _thldAlarmInfo;
	break;

    default:
	return NOTOK;
    }

    return OK;
}

