/*
 * Copyright (c) 1992 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London,
 * London, England. Neither the name of the University, nor its
 * registered trademark, UCL, may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/
/****************************************************************/
/*
	GDMO.cc

	methods of class GDMO

	Author: J.Cowan		(j.cowan@.cs.ucl.ac.uk)
*/
/****************************************************************/

#ifdef	__MSDOS__
#include <io.h>
#else
#include <unistd.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include "GDMOGen.h"

#include <stdio.h>
#include <stdlib.h>
#include "DebugMem.h"
#include "GDMOGen.h"
#include "GDMOOid.h"
#include "GDMOAttr.h"
#include "GDMOAct.h"
#include "GDMONot.h"
#include "GDMOBeh.h"
#include "GDMOPkg.h"
#include "GDMOClas.h"
#include "GDMO.h"
#include "SymExec.h"
#include "IIMC.h"

/****************************************************************/
/* Lex globals */

FILE	*yyin, *yyout;

/*******************************************************/
/* Initialise */
/*******************************************************/

void	GDMO::Initialise()
{
	_error = False;
	_curMoClass = (GDMOClass *) 0;
	_curPackage = (GDMOPackage *) 0;
	_curAttribute = (GDMOAttribute *) 0;
	_curNameBinding = (GDMONameBinding *) 0;
	_curAction = (GDMOAction *) 0;
	_curNotification = (GDMONotification *) 0;
	_curAttrGroup = (GDMOAttributeGroup *) 0;

} /* Initialise */

/*******************************************************/
/* Parse GDMO script */
/*******************************************************/

Bool	GDMO::Parse(char *fileName)
{
Bool	result = True;
extern	int	line, GdmoParseErrors();

	if ((yyin=fopen(fileName, "r")) != (FILE *) 0)
	{
		printf("\nParse File: %s\n", fileName);

		line = 1;

		/* Call parser */

		ParseGDMO();
		fclose(yyin);

		if (GdmoParseErrors() > 0)
		{
			printf("\nErrors: %d\n", GdmoParseErrors());
			result = False;
		}
		else
			if (_error == False)
				result = UpdateRefs();
			else
				result = False;
	}
	else
	{
	GString	tmp = GString("Cannot find file ") + GString(fileName);

		PrintError((char *)tmp);
		result = False;
	}

	if (result)
		printf("\nNo Errors found!\n");

	return result;

} /* Parse */

/****************************************************************/
/* Parse IIMC statements */
/****************************************************************/

Bool	GDMO::ParseIIMCBehaviour()
{
Bool	noError = True;

	// Search MO behaviour clauses

	ListIterator	moIterator(&_moClassList);
	GDMOClass	*moClass;

	while ((moClass=(GDMOClass *)moIterator.getNext()) !=
		(GDMOClass *) 0)

		if (!ParseIIMC(moClass, MOCBEGINSTATE))
			noError = False;

	// Search Attribute behaviour clauses

	ListIterator	attrIterator(&_attrList);
	GDMOAttribute	*attr;

	while ((attr=(GDMOAttribute *)attrIterator.getNext()) !=
		(GDMOAttribute *) 0)

		if (!ParseIIMC(attr, ATTRBEGINSTATE))
			noError = False;

	// Search name binding behaviour clauses

	ListIterator	bindingIterator(&_nameBindingList);
	GDMONameBinding	*nameBinding;

	while ((nameBinding=(GDMONameBinding *)bindingIterator.getNext()) !=
		(GDMONameBinding *) 0)

		if (!ParseIIMC(nameBinding, NBBEGINSTATE))
			noError = False;

	return noError;

} /* ParseIIMCBehaviour */

/****************************************************************/

Bool	GDMO::ParseIIMC(MetaClass *metaClass, IIMCBeginState state)
{
Bool		noError = True;
ListSymbol	*list = 
	metaClass->FindSymbol("BehaviourReferenceList")->GetListSymbol();
ListIterator	iterator(list);
StringSymbol	*ssym;
IIMC		iimc(metaClass);

	while ((ssym=(StringSymbol *)iterator.getNext()) !=
		(StringSymbol *) 0)
	{
	char	*behavRef = ssym->GetValue();
	GDMOBehaviour	*behaviour = _behaviourList.FindBehaviour(behavRef);

		if (behaviour == (GDMOBehaviour *)0)
		{
			printf("\nMissing behaviour template %s", behavRef);
			continue;
		}

		char	*definition = behaviour->GetDefinition();

		if (iimc.IsParseable(definition))
		{
			printf("\nParsing IIMC behaviour definition %s",
				behavRef);

			if (!iimc.Compile(definition, 1, state))
				noError = False;
		}
	}

	return (noError);

} /* ParseIIMC */

/****************************************************************/
/* Check Valid use of pointers */
/****************************************************************/

void	GDMO::checkCurrentMoClass(int pos)
{
	if (_curMoClass == (GDMOClass *) 0)
	{
		printf("\nNull GDMO Class pos %d", pos);
		exit(1);
	}

} /* checkCurrentMoClass */

/****************************************************************/

void	GDMO::checkCurrentPackage(int pos)
{
	if (_curPackage == (GDMOPackage *) 0)
	{
		printf("\nNull GDMO Package pos %d", pos);
		exit(1);
	}

} /* checkCurrentPackage */

/****************************************************************/

void	GDMO::checkCurrentAttribute(int pos)
{
	if (_curAttribute == (GDMOAttribute *) 0)
	{
		printf("\nNull GDMO Attribute pos %d", pos);
		exit(1);
	}

} /* checkCurrentAttribute */

/****************************************************************/

void	GDMO::checkCurrentNameBinding(int pos)
{
	if (_curNameBinding == (GDMONameBinding *) 0)
	{
		printf("\nNull GDMO Name Binding pos %d", pos);
		exit(1);
	}

} /* checkCurrentNameBinding */

/****************************************************************/

void	GDMO::checkCurrentAction(int pos)
{
	if (_curAction == (GDMOAction *) 0)
	{
		printf("\nNull GDMO Action pos %d", pos);
		exit(1);
	}
} /* checkCurrentAction */

/****************************************************************/

void	GDMO::checkCurrentNotification(int pos)
{
	if (_curNotification == (GDMONotification *) 0)
	{
		printf("\nNull GDMO Notification pos %d", pos);
		exit(1);
	}

} /* checkCurrentNotification */

/****************************************************************/

void	GDMO::checkCurrentAttributeGroup(int pos)
{
	if (_curAttrGroup == (GDMOAttributeGroup *) 0)
	{
		printf("\nNull GDMO Attribute Group pos %d", pos);
		exit(1);
	}

} /* checkCurrentAttributeGroup */

/****************************************************************/

void	GDMO::checkCurrentBehaviour(int pos)
{
	if (_curBehaviour == (GDMOBehaviour *) 0)
	{
		printf("\nNull GDMO Behaviour pos %d", pos);
		exit(1);
	}

} /* checkCurrentBehaviour */

/****************************************************************/
/* Lots	of parse member functions called from Yacc script */
/****************************************************************/

void	GDMO::NewMoClass(char *moClassName)
{
GString	headerFname(moClassName);

	headerFname += ".h";
	_curMoClass = (NEW GDMOClass(moClassName, moClassName,
				     (char *) headerFname, False));

	_moClassList.append(_curMoClass);

} /* NewMoClass */

/****************************************************************/

void	GDMO::AddMoParent(char *parentName)
{
	checkCurrentMoClass(1);
	_curMoClass->AddParentRef(parentName);

} /* AddMoParent */

/****************************************************************/

void	GDMO::AddMoPackage(char *packageName)
{
	checkCurrentMoClass(2);
	_curMoClass->AddPackageRef(packageName);

} /* AddMoPackage */

/****************************************************************/

void	GDMO::NewPackage(char *packageName)
{
	_curPackage = (NEW GDMOPackage(packageName));

	_packageList.append(_curPackage);

} /* NewPackage */

/****************************************************************/

void	GDMO::AddPackageAttribute(char *attribute)
{
	checkCurrentPackage(1);
	_curPkgAttribute = _curPackage->AddAttribute(attribute);

} /* AddPackageAttribute */

/****************************************************************/

void	GDMO::UpdatePackageAttribute(char *attribute, char *value)
{
	checkCurrentPackage(5);
	_curPkgAttribute->UpdateSymbol(attribute, value);

} /* UpdatePackageAttribute */

/****************************************************************/

void	GDMO::UpdatePackageAttribute(char *attribute, int value,
				     UpdateMode mode)
{
	checkCurrentPackage(6);
	_curPkgAttribute->UpdateSymbol(attribute, value, mode);

} /* UpdatePackageAttribute */

/****************************************************************/

void	GDMO::AddPackageAttributeGroup(char *name)
{
	checkCurrentPackage(7);
	_curAttrGroup = (NEW GDMOAttributeGroup(name));
	_curPackage->UpdateSymbolList("AttributeGroupList", _curAttrGroup);

} /* AddPackageAttributeGroup */

/****************************************************************/

void	GDMO::UpdatePackageAttrGroup(char *attribute, Symbol *value)
{
	checkCurrentAttributeGroup(8);
	_curAttrGroup->UpdateSymbolList(attribute, value);

} /* UpdatePackageAttribute */

/****************************************************************/

void	GDMO::AddPackageAction(char *action)
{
	checkCurrentPackage(2);
	_curPackage->AddAction(action);

} /* AddPackageAction */

/****************************************************************/

void	GDMO::AddPackageNotification(char *notification)
{
	checkCurrentPackage(3);
	_curPackage->AddNotification(notification);

} /* AddPackageNotification */

/****************************************************************/

void	GDMO::NewNameBinding(char *bindingName)
{
	_curNameBinding = (NEW GDMONameBinding(bindingName));

	_nameBindingList.append(_curNameBinding);

} /* NewNameBinding */

/****************************************************************/

void	GDMO::AddNameBinding(char *subordinate, char *superior,
			     char *attribute)
{
	checkCurrentNameBinding(1);
	_curNameBinding->AddNameBinding(subordinate, superior, attribute);

} /* AddNameBinding */

/****************************************************************/

void	GDMO::AddCreateModifier(int modifier)
{
	checkCurrentNameBinding(2);
	_curNameBinding->AddCreateModifier(modifier);

} /* AddCreateModifier */

/****************************************************************/

void	GDMO::AddDeleteModifier(int modifier)
{
	checkCurrentNameBinding(3);
	_curNameBinding->AddDeleteModifier(modifier);

} /* AddDeleteModifier */

/****************************************************************/

void	GDMO::NewAttribute(char *attributeName)
{
	_curAttribute = (NEW GDMOAttribute(attributeName));

	_attrList.append(_curAttribute);

} /* NewAttribute */

/****************************************************************/

void	GDMO::UpdateAttribute(char *attribute, char *value)
{
	checkCurrentAttribute(5);
	_curAttribute->UpdateSymbol(attribute, value);

} /* UpdateAttribute */

/****************************************************************/

void	GDMO::UpdateAttribute(char *attribute, int value, UpdateMode mode)
{
	checkCurrentAttribute(6);
	_curAttribute->UpdateSymbol(attribute, value, mode);

} /* UpdateAttribute */

/****************************************************************/

void	GDMO::AddAttributeDerivation(char *derivedName)
{
	checkCurrentAttribute(1);
	_curAttribute->AddDerivation(derivedName, _attrList);

} /* AddAttributeDerivation */

/****************************************************************/

void	GDMO::AddAttributeSyntax(char *syntaxName)
{
	checkCurrentAttribute(2);
	_curAttribute->AddSyntax(syntaxName);

} /* AddAttributeSyntax */

/****************************************************************/

void	GDMO::NewAction(char *actionName)
{
	_curAction = (NEW GDMOAction(actionName));

	_actionList.append(_curAction);

} /* NewAction */

/****************************************************************/

void	GDMO::UpdateAction(char *attribute, char *value)
{
	checkCurrentAction(5);
	_curAction->UpdateSymbol(attribute, value);

} /* UpdateAction */

/****************************************************************/

void	GDMO::UpdateAction(char *attribute, int value, UpdateMode mode)
{
	checkCurrentAction(6);
	_curAction->UpdateSymbol(attribute, value, mode);

} /* UpdateAction */

/****************************************************************/

void	GDMO::AddActionInfoSyntax(char *syntaxName)
{
	checkCurrentAction(1);
	_curAction->AddInfoSyntax(syntaxName);

} /* AddActionInfoSyntax */

/****************************************************************/

void	GDMO::AddActionReplySyntax(char *syntaxName)
{
	checkCurrentAction(2);
	_curAction->AddReplySyntax(syntaxName);

} /* AddActionReplySyntax */

/****************************************************************/

void	GDMO::NewNotification(char *notificationName)
{
	_curNotification = (NEW GDMONotification(notificationName));

	_notificationList.append(_curNotification);

} /* NewNotification */

/****************************************************************/

void	GDMO::UpdateNotification(char *attribute, char *value)
{
	checkCurrentNotification(5);
	_curNotification->UpdateSymbol(attribute, value);

} /* UpdateNotification */

/****************************************************************/

void	GDMO::UpdateNotification(char *attribute, int value, UpdateMode mode)
{
	checkCurrentNotification(6);
	_curNotification->UpdateSymbol(attribute, value, mode);

} /* UpdateNotification */

/****************************************************************/

void	GDMO::AddNotificationInfoSyntax(char *syntaxName)
{
	checkCurrentNotification(1);
	_curNotification->AddInfoSyntax(syntaxName);

} /* AddNotificationInfoSyntax */

/****************************************************************/

void	GDMO::AddNotificationReplySyntax(char *syntaxName)
{
	checkCurrentNotification(2);
	_curNotification->AddReplySyntax(syntaxName);

} /* AddNotificationReplySyntax */

/****************************************************************/

void	GDMO::NewAttributeGroup(char *name)
{
	_curAttrGroup = (NEW GDMOAttributeGroup(name));

	_attrGroupList.append(_curAttrGroup);

} /* NewAttributeGroup */

/****************************************************************/

void	GDMO::UpdateAttributeGroup(char *attribute, Symbol *value)
{
	checkCurrentAttributeGroup(1);
	_curAttrGroup->UpdateSymbolList(attribute, value);

} /* UpdateAttributeGroup */

/****************************************************************/

void	GDMO::UpdateAttributeGroup(char *attribute, int value, UpdateMode mode)
{
	checkCurrentAttributeGroup(2);
	_curAttrGroup->UpdateSymbol(attribute, value, mode);

} /* UpdateAttributeGroup */

/****************************************************************/

void	GDMO::NewBehaviour(char *behaviourName)
{
	_curBehaviour = (NEW GDMOBehaviour(behaviourName));

	_behaviourList.append(_curBehaviour);

} /* NewAttribute */

/****************************************************************/

void	GDMO::UpdateBehaviour(char *attribute, char *value)
{

	checkCurrentBehaviour(1);
	_curBehaviour->UpdateSymbol(attribute, value);

} /* UpdateBehaviour */

/*******************************************************/
// Delete previous registration

void	GDMO::ClearRegistration()
{
	_curOid.clear();

} /* ClearRegistration */

/*******************************************************/

void	GDMO::AddRegistration(char *oidElement)
{
StringSymbol	*newOidElement = (NEW StringSymbol(oidElement, ""));

	_curOid.append(newOidElement);

} /* AddRegistration */

/*******************************************************/

void	GDMO::AddMoRegistration()
{
	checkCurrentMoClass(3);
	_curOid.Copy(_curMoClass->GetOid());

} /* AddMoRegistration */

/*******************************************************/

void	GDMO::AddPackageRegistration()
{
	checkCurrentPackage(4);
	_curOid.Copy(_curPackage->GetOid());

} /* AddPackageRegistration */

/*******************************************************/

void	GDMO::AddNameBindingRegistration()
{
	checkCurrentNameBinding(4);
	_curOid.Copy(_curNameBinding->GetOid());

} /* AddNameBindingRegistration */

/*******************************************************/

void	GDMO::AddAttributeRegistration()
{
	checkCurrentAttribute(3);
	_curOid.Copy(_curAttribute->GetOid());

} /* AddAttributeRegistration */

/*******************************************************/

void	GDMO::AddAttrGroupRegistration()
{
	checkCurrentAttributeGroup(3);
	_curOid.Copy(_curAttrGroup->GetOid());

} /* AddAttrGroupRegistration */

/*******************************************************/

void	GDMO::AddActionRegistration()
{
	checkCurrentAction(3);
	_curOid.Copy(_curAction->GetOid());

} /* AddActionRegistration */

/*******************************************************/

void	GDMO::AddNotificationRegistration()
{
	checkCurrentNotification(3);
	_curOid.Copy(_curNotification->GetOid());

} /* AddNotificationRegistration */

/*******************************************************/
/* Update References */
/*******************************************************/

Bool	GDMO::UpdateRefs()
{
GDMOAttributeGroup     *attrGroup;
ListIterator		attrGroupIterator(&_attrGroupList);
GDMOClass		*moClass;
ListIterator		moIterator(&_moClassList);
GDMOPackage		*package;
ListIterator		pkgIterator(&_packageList);
ListIterator		bindingIterator(&_nameBindingList);
GDMONameBinding		*nameBinding;

	while ((attrGroup=(GDMOAttributeGroup *)attrGroupIterator.getNext())
		!= (GDMOAttributeGroup *) 0)

		if (attrGroup->UpdateRefs(_attrList) == False)
			return False;

	while ((package=(GDMOPackage *)pkgIterator.getNext()) !=
		(GDMOPackage *) 0)

		if (package->UpdateRefs(_attrList, _actionList,
					_notificationList, _attrGroupList)
				== False)
			return False;

	while ((moClass=(GDMOClass *)moIterator.getNext()) !=
		(GDMOClass *) 0)

		if (moClass->UpdateRefs(&_moClassList, &_packageList)
					== False)
			return False;

	while ((nameBinding=(GDMONameBinding *)bindingIterator.getNext()) !=
		(GDMONameBinding *) 0)

		if (nameBinding->UpdateRefs(&_moClassList) == False)
			return False;

	return True;

} /* UpdateRefs */

/*******************************************************/
/* Code Generate */
/*******************************************************/

void	GDMO::CodeGenerate(char *scriptName)
{
ListIterator	moIterator(&_moClassList);
InterPreter	*interPreter;

	// See if STARTCODE in local directory

	if (access(scriptName, 0) == 0)
	{
		printf("\nExecuting %s", scriptName);
		interPreter = (NEW InterPreter(scriptName, CODELOG,
					       GetSymbolTable()));
	}
	else
	{
	GString	tmp = GString(_gdmoDirectory.GetValue()) + GString(scriptName);

		if (access((char *)tmp, 0) == 0)
		{
			printf("\nExecuting %s", (char *) tmp);
			interPreter = (NEW InterPreter((char *) tmp, CODELOG,
						       GetSymbolTable()));
		}
		else
		{
			printf("\nCannot find file %s", (char *)tmp);
			return;
		}
	}

	interPreter->Execute();

	DELETE interPreter;

} /* CodeGenerate */

/*******************************************************/
/* Read	Mo Database */
/*******************************************************/

void	GDMO::ReadMoDatabase(char *moDatabase)
{
FILE	*fp;
static	char	buf[150];
Bool	foundTop = False;

	if ((fp=fopen(moDatabase, "r")) == (FILE *) 0)
	{
		printf("\nCannot find MO database %s",
			moDatabase);
		exit(1);
	}

	// Read the database

	printf("\nReading MO database %s\n", moDatabase);

	while (fgets(buf, sizeof(buf), fp) != (char *) 0)
	{
	char	moName[50], className[50], headerName[50];
	int	scanRes;

		if (buf[0] == '#')
			continue;

		scanRes = sscanf(buf,"%s%s%s", &moName, &className,
				&headerName);

		if (scanRes == EOF)
			continue;

		if (scanRes != 3)
		{
			printf("\nError in MO database %s line: %s",
				moDatabase, buf);
			exit(1);
		}

		if (!foundTop && !strcmp(moName, TopMo))
			foundTop = True;

		_moClassList.append((NEW GDMOClass(moName, className,
						   headerName, True)));
	}

	fclose(fp);

	if (!foundTop)
	{
		printf("\nCannot find Top MO %s in MO database %s",
			TopMo, moDatabase);
		exit(1);
	}

} /* ReadMoDatabase */

/*******************************************************/
/* Print MOS, hierarchies etc */
/*******************************************************/

void	GDMO::PrintMoList()
{
GDMOClass	*moClass;
ListIterator	moIterator(&_moClassList);

	while ((moClass=(GDMOClass *)moIterator.getNext()) !=
		(GDMOClass *) 0)
	{
	GDMOAttrList	*attrList;
	GDMOAttribute	*attr;
	ListIterator	regIterator(moClass->GetOid());
	StringSymbol	*oid;

		printf("\n\nGDMO Class: %s Oid: ", moClass->GetName());

		while ((oid=(StringSymbol *)regIterator.getNext()) !=
			(StringSymbol *) 0)
			printf("%s ", oid->GetValue());

		attrList = moClass->GetAttributes();

		ListIterator	attrIterator(attrList);

		while ((attr=(GDMOAttribute *)attrIterator.getNext()) !=
				(GDMOAttribute *) 0)
			printf("\n\tAttribute: %s\n\t\tSyntax: %s",
				attr->GetName(), attr->GetSyntax());
	}

} /* PrintMoList */

/*******************************************************/

void	GDMO::PrintPackageList()
{
GDMOPackage	*package;
ListIterator	packageIterator(&_packageList);

	while ((package=(GDMOPackage *)packageIterator.getNext()) !=
		(GDMOPackage *) 0)
	{
	GDMOAttrList	*attrList;
	GDMOAttribute	*attr;

		printf("\n\nPackage: %s", package->GetName());

		attrList = package->GetAttributes();

		ListIterator	attrIterator(attrList);

		while ((attr=(GDMOAttribute *)attrIterator.getNext()) !=
				(GDMOAttribute *) 0)
			printf("\n\tAttribute: %s\n\t\tSyntax: %s",
				attr->GetName(), attr->GetSyntax());
	}

} /* PrintMoList */

/*******************************************************/

void	GDMO::PrintAttrList()
{
GDMOAttribute	*attr;
ListIterator	attrIterator(&_attrList);

	while ((attr=(GDMOAttribute *)attrIterator.getNext()) !=
		(GDMOAttribute *) 0)

		printf("\n\nAttribute: %s\n\tSyntax: %s",
			attr->GetName(), attr->GetSyntax());

} /* PrintAttrList */

/*******************************************************/

void	GDMO::PrintInheritanceHierarchy()
{
	printf("\n\nInheritance hierarchy\n");
	PrintChildren(TopMo, 0);

} /* PrintInheritanceHierarchy */

/*******************************************************/

void	GDMO::PrintContainmentHierarchy()
{
	printf("\n\nContainment hierarchy\n");

	// Search the name binding list for a binding that
	// refers to a superior MO that is not also a
	// subordinate MO - this is the top MO!

	ListIterator	supIterator(&_nameBindingList);
	GDMONameBinding	*supBinding;
	GDMOClass	*topMo;

	while ((supBinding=(GDMONameBinding *)supIterator.getNext()) !=
		(GDMONameBinding *) 0)
	{
	ListIterator	subIterator(&_nameBindingList);
	GDMONameBinding	*subBinding;

		topMo =	supBinding->GetSuperiorMo();
		while ((subBinding=(GDMONameBinding *) subIterator.getNext())
			!= (GDMONameBinding *) 0)

			if (subBinding->GetSubordinateMo() == topMo)
				break;

		if (!subBinding)
			break;
	}

	if (!supBinding)
		PrintError("Cannot find Top MO in containment hierarchy");
	else
		PrintSubordinates(topMo, 0);

} /* PrintContainmentHierarchy */

/*******************************************************/

void	GDMO::PrintSubordinates(GDMOClass *mo, int indent)
{
ListIterator		subIterator(&_nameBindingList);
GDMONameBinding		*subBinding;

	printf("\n");
	for (int count=0; count<indent; count++)
		printf(" ");

	printf("%s", (char *) mo->GetName());

	while ((subBinding=(GDMONameBinding *) subIterator.getNext()) !=
		(GDMONameBinding *) 0)
	{
		if (subBinding->GetSuperiorMo() != mo)
			continue;

		GDMOClass *subMo = subBinding->GetSubordinateMo();

		PrintSubordinates(subMo, indent+5);
	}

} /* PrintSubordinates */

/*******************************************************/

void	GDMO::PrintParents()
{
GDMOClass	*moClass;
ListIterator	moIterator(&_moClassList);

	while ((moClass=(GDMOClass *)moIterator.getNext()) !=
		(GDMOClass *) 0)
	{
	GDMOList	*parentList;
	GDMOClass	*parent;

		printf("\n\nMO Class: %s\n\tParents:", moClass->GetName());

		parentList = moClass->GetParents();

		ListIterator	parentIterator(parentList);

		while ((parent=(GDMOClass *)parentIterator.getNext()) !=
				(GDMOClass *) 0)
			printf("%s", parent->GetName());
	}

} /* PrintParents */

/*******************************************************/

void	GDMO::PrintChildren(char *moName, int indent)
{
GDMOClass	*moChild, *moClass;
GDMOList	*childList;

	if (!(moClass=_moClassList.FindMo(moName)))
	{
		printf("\nCannot find %s", moName);
		return;
	}

	printf("\n");

	for (int count=0; count<indent; count++)
		printf(" ");

	printf("%s", moName);

	childList = moClass->GetChildren();

	ListIterator	moIterator(childList);

	while ((moChild=(GDMOClass *)moIterator.getNext()) != (GDMOClass *) 0)

		PrintChildren(moChild->GetName(), indent+5);

} /* PrintChildren */

/*******************************************************/
/* End of file */
/*******************************************************/


