/*
 * Copyright (c) 1992 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London,
 * London, England. Neither the name of the University, nor its
 * registered trademark, UCL, may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/
/****************************************************************/
/*
	GDMOAct.h

	Class that represents a	GDMO Action

	Author:	J.Cowan		(j.cowan@.cs.ucl.ac.uk)

	Updated: K.M.T.McCarthy - header file attributes added

*/
/****************************************************************/

#ifndef	_SYMBOL
#include "Symbol.h"
#endif

#ifndef	_GENERIC_LIST_
#include "GenList.h"
#endif

#ifndef	_LIST_ITERATOR_
#include "ListIterator.h"
#endif

#ifndef	_GENERIC_STRING_
#include "GenStr.h"
#endif

/****************************************************************/

typedef	enum	{ModeConfirmed=1, ModeNonConfirmed=2} ModeTypes;

class GDMOAction : public MetaClass
{
	StringSymbol	_name,
			_infoSyntax, _replySyntax,
			_infoSyntaxClassName, _replySyntaxClassName,
			_infoSyntaxHeaderFileName, _replySyntaxHeaderFileName;
	IntegerSymbol	_mode;
	GDMOOid		_oid;
public:
	GDMOAction(char *name) :
			MetaClass("GDMOAction"),
			_name(name, "ActionName"),
			_mode(False, "Mode"),
			_infoSyntax("", "InfoSyntax"),
			_replySyntax("", "ReplySyntax"),
			_infoSyntaxClassName("", "InfoSyntaxClassName"),
			_replySyntaxClassName("", "ReplySyntaxClassName"),
			_infoSyntaxHeaderFileName("", "InfoSyntaxHeaderFileName"),
			_replySyntaxHeaderFileName("", "ReplySyntaxHeaderFileName"),
			_oid("Oid")
	{
		AddSymbol(&_name);
		AddSymbol(&_mode);
		AddSymbol(&_infoSyntax);
		AddSymbol(&_replySyntax);
		AddSymbol(&_infoSyntaxClassName);
		AddSymbol(&_replySyntaxClassName);
		AddSymbol(&_infoSyntaxHeaderFileName);
		AddSymbol(&_replySyntaxHeaderFileName);
		AddSymbol(&_oid);
	}

	void	AddInfoSyntax(char *syntax)
	{
	ConvertEntry* entry ;
	
		_infoSyntax.StrReplace(syntax);

		if ((entry=GDMOAttribute::LookupSyntaxDatabase(syntax)) != (ConvertEntry *)0)
		{
			_infoSyntaxClassName.StrReplace(entry->GetClassName());
			_infoSyntaxHeaderFileName.StrReplace(entry->GetHeaderFName());
		}
		else
		{
			GString	hFileName(syntax);
			
			hFileName += ".h";

			printf("\nAttribute %s has unknown syntax %s",
				_name.print(), syntax);
			_infoSyntaxHeaderFileName.StrReplace((char *)hFileName);
		}
	}

	void	AddReplySyntax(char *syntax)
	{
	ConvertEntry* entry ;
	
		_replySyntax.StrReplace(syntax);

		if ((entry=GDMOAttribute::LookupSyntaxDatabase(syntax)) != (ConvertEntry *)0)
		{
			_replySyntaxClassName.StrReplace(entry->GetClassName());
			_replySyntaxHeaderFileName.StrReplace(entry->GetHeaderFName());
		}
		else
		{
			GString	hFileName(syntax);
			
			hFileName += ".h";

			printf("\nAttribute %s has unknown syntax %s",
				_name.print(), syntax);
			_replySyntaxHeaderFileName.StrReplace((char *)hFileName);
		}
	}

	char	*GetName() { return (char *) _name;}

	GDMOOid	*GetOid() { return &_oid;}
};

/****************************************************************/

class	GDMOActionList : public ListSymbol
{
protected:
	void	deleteElem (void*);
public:
	GDMOActionList(Bool del, char *name)
	: ListSymbol(name, "GDMOAction", ObjectTypeDef, del) {}
	GDMOActionList(char *name)
	: ListSymbol(name, "GDMOAction", ObjectTypeDef) {}
	~GDMOActionList() { clear();}

	GDMOAction *FindAction(char *actionName)
	{
	GDMOAction	*action;
	ListIterator	actionIterator(this);

		while ((action=(GDMOAction *)actionIterator.getNext()) !=
				(GDMOAction *) 0)

			if (!strcmp(action->GetName(), actionName))
				break;

		return action;
	}
};

/****************************************************************/

class GDMOActionRef
{
	GString		_name;
	GDMOAction	*_action;
public:
	GDMOActionRef(char *name)
	{
		_name = name;
		_action = (GDMOAction *) 0;
	}

	void	UpdateRef(GDMOAction *action)
	{
		_action = action;
	}
	char	*GetName()
	{
		return (char *)_name;
	}
	GDMOAction *GetAction()
	{
		if (!_action)
			printf("\nAction %s not updated",(char *) _name);

		return _action;
	}
};

/****************************************************************/

class	GDMOActionRefList : public List
{
protected:
	void	deleteElem (void*);
public:
	~GDMOActionRefList() { clear();}
};

/****************************************************************/
// inline methods for GDMOActionList

inline void  GDMOActionList::deleteElem (void* val)
{  if (del) delete (GDMOAction *) val; }

/****************************************************************/
// inline methods for GDMOActionRefList

inline void  GDMOActionRefList::deleteElem (void* val)
{  if (del) delete (GDMOActionRef *) val; }

/****************************************************************/





