/*
 * Copyright (c) 1992 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London,
 * London, England. Neither the name of the University, nor its
 * registered trademark, UCL, may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/
/****************************************************************/
/*
	GDMOAttr.cc

	Methods of a GDMO Attribute Class

	Author: J.Cowan		(j.cowan@.cs.ucl.ac.uk)
*/
/****************************************************************/

#include <stdio.h>
#if defined(__GNUG__)
#include <std.h>
#endif
#include <stdlib.h>
#ifndef	__MSDOS__
#ifdef	__GNUG__
#include <string.h>
#else
#include <strings.h>
#endif
#endif
#include "DebugMem.h"
#include "GDMOGen.h"
#include "GDMOOid.h"
#include "GDMOAttr.h"

/****************************************************************/
/* Attribute Methods */
/****************************************************************/
/* Add syntax */

void	GDMOAttribute::AddSyntax(char *syntax)
{
ConvertEntry	    *entry;

	_syntax.StrReplace(syntax);

	if ((entry=_syntaxTable.Lookup(syntax)) != (ConvertEntry *)0)
	{
		_codeName.StrReplace(entry->GetClassName());
		_headerFileName.StrReplace(entry->GetHeaderFName());
	}
	else
	{
	GString	hFileName(syntax);

		hFileName += ".h";

		printf("\nAttribute %s has unknown syntax %s",
			_name.print(), syntax);
		_codeName.StrReplace(syntax);
		_headerFileName.StrReplace((char *)hFileName);
	}

} /* AddSyntax */

/****************************************************************/
/* Add derivation */

void	GDMOAttribute::AddDerivation(char *derived, GDMOAttrList &attrList)
{
ConvertEntry	    *entry;

	_derived.StrReplace(derived);

	// Check if derived type in attribute list

	GDMOAttribute	*attr;

	if ((attr=attrList.FindAttr(derived)) != (GDMOAttribute *) 0)
	{
		AddSyntax(attr->GetSyntax());
		return;
	}

	// must	be external derived type

	if ((entry=_attrTable.Lookup(derived)) != (ConvertEntry *)0)
	{
		_syntax.StrReplace(entry->GetConvertName());
		_codeName.StrReplace(entry->GetClassName());
		_headerFileName.StrReplace(entry->GetHeaderFName());
	}
	else
	{
	GString	hFileName(derived);

		hFileName += ".h";

		printf("\nAttribute %s references unknown derived type %s",
			_name.print(), derived);
		_codeName.StrReplace(derived);
		_headerFileName.StrReplace((char *)hFileName);
	}

} /* AddDerivation */

/****************************************************************/

ConvertEntry	*GDMOAttribute::LookupSyntaxDatabase(char *syntaxName)
{
	return _syntaxTable.Lookup(syntaxName);

} /* LookupAttributeDatabase */

/****************************************************************/

ConvertEntry	*GDMOAttribute::LookupAttributeDatabase(char *attrName)
{
	return _attrTable.Lookup(attrName);

} /* LookupAttributeDatabase */

/****************************************************************/

void GDMOAttribute::AddAttributeDatabase(char *attrDbase)
{
	_attrTable.Initialise(attrDbase);

} /* AddAttributeDatabase */

/****************************************************************/

void GDMOAttribute::AddSyntaxDatabase(char *syntaxDbase)
{
	_syntaxTable.Initialise(syntaxDbase);

} /* AddSyntaxDatabase */

/****************************************************************/
/* Update package attribute reference with attribute template */

void GDMOAttribute::UpdatePkgAttrRef(GDMOAttribute *attr)
{
static	char	*attrNames[] =
	{"Syntax", "Derived", "ClassName", "headerFileName"};
int	lgth = sizeof(attrNames) / sizeof(char *);

	for (int count=0; count<lgth; count++)
	{
	char	*name =	attrNames[count];

		Copy(FindSymbol(name), attr->FindSymbol(name));
	}

	// Now copy the oid

	// GetOid()->Copy(attr->GetOid());

	attr->GetOid()->Copy(GetOid());

} /* UpdatePkgAttrRef */

/****************************************************************/
/* Update package attribute reference with external attribute */

void GDMOAttribute::UpdatePkgAttrRef(ConvertEntry *convEntry)
{
	UpdateSymbol("ClassName", convEntry->GetClassName());
	UpdateSymbol("headerFileName", convEntry->GetHeaderFName());

} /* UpdatePkgAttrRef */

/****************************************************************/
/* Attribute list methods */
/****************************************************************/

GDMOAttribute *GDMOAttrList::FindAttr(char *attributeName)
{
GDMOAttribute	*attr;
ListIterator	attrIterator(this);
/*
	printf("\nSearching for %s", attributeName);
*/
	while ((attr=(GDMOAttribute *)attrIterator.getNext()) !=
		(GDMOAttribute *) 0)
	{
/*
	printf("\nComparing %s", attr->GetName());
*/
		if (!strcmp(attr->GetName(), attributeName))
			break;
	}
	return attr;

} /* FindAttr */

/****************************************************************/
/* Attribute Group methods */
/****************************************************************/
/* Update Group Attribute Refs */

Bool	GDMOAttributeGroup::UpdateRefs(GDMOAttrList &attrList)
{
ListIterator	elementIterator(&_elements);
ListIterator	extensionIterator(&_extensions);
/*
	printf("\nChecking AGROUP %s Elements %d Fixed %d Extensions %d",
		_name.print(), _elements.getCount(),
		_fixed.GetValue(), _extensions.getCount());
*/
	if (!FindAttr(attrList, elementIterator))
		return False;

	if (_fixed.GetValue() && _extensions.getCount())
	{
		printf("\nAttribute Group %s is FIXED but has been extended",
			_name.print());
		return False;
	}

	if (!FindAttr(attrList, extensionIterator))
		return False;

	return True;

} /* UpdateRefs	*/

/****************************************************************/

Bool	GDMOAttributeGroup::FindAttr(GDMOAttrList &attrList,
				     ListIterator &listIterator)
{
StringSymbol	*sym;

	while ((sym=(StringSymbol *)listIterator.getNext()) !=
		(StringSymbol *) 0)
	{
	char	*name = sym->GetValue();
/*
	printf("\nSearch group %s attribute %s", 
		_name.GetValue(), name);
*/
		if (attrList.FindAttr(name) == (GDMOAttribute *) 0)
		{
		GString	msg =	GString("Warning: Attribute ") + GString(name) +
				GString(" not in attribute Group ") +
				GString(_name.GetValue());

			printf("\n%s", msg.print());
			// return False; make a warning
			return True;
		}
	}

	return True;

} /* FindAttr */

/****************************************************************/
/* Attribute group list methods */
/****************************************************************/

GDMOAttributeGroup *GDMOAttrGroupList::FindAttrGroup(char *name)
{
GDMOAttributeGroup	*group;
ListIterator		groupIterator(this);

	while ((group=(GDMOAttributeGroup *)groupIterator.getNext()) !=
		(GDMOAttributeGroup *) 0)

		if (!strcmp(group->GetName(), name))
			break;

	return group;

} /* FindAttrGroup */

/****************************************************************/
/* ConvertTable Methods */
/****************************************************************/

ConvertTable	    GDMOAttribute::_attrTable;
ConvertTable	    GDMOAttribute::_syntaxTable;

/****************************************************************/
/* Initialise attribute conversion table */

void	ConvertTable::Initialise(char *fileName)
{
FILE	*fp;
static	char	buf[150];

	if ((fp=fopen(fileName, "r")) == (FILE *) 0)
	{
		printf("\nCannot find file %s",fileName);
		exit(1);
	}

	// Read the database

	printf("\nReading file %s\n", fileName);

	while (fgets(buf, sizeof(buf), fp) != (char *) 0)
	{
	char	convert[100], className[100], headerName[100];
	ConvertEntry	    *entry;
	int	scanRes;

		if (buf[0] == '#')
			continue;

		scanRes = sscanf(buf,"%s%s%s", &convert, &className,
				&headerName);

		if (scanRes == EOF)	// blank line
			continue;

		if (scanRes != 3)
		{
			printf("\nError in attribute database %s line: %s",
				fileName, buf);
			exit(1);
		}
		entry = (NEW ConvertEntry(convert, className, headerName));
		_convertList.append(entry);
	}

	fclose(fp);

} /* Initialise */

/****************************************************************/
/* Used to be static

ConvertList ConvertTable::_convertList;
*/
/****************************************************************/

ConvertEntry *ConvertTable::Lookup(char *convertName)
{
ConvertEntry *entry = (ConvertEntry *) 0;
ListIterator	iterator(&_convertList);

	while ((entry=(ConvertEntry *)iterator.getNext()) !=
		(ConvertEntry *) 0)
#ifdef	__MSDOS__
		if (!stricmp(entry->GetConvertName(), convertName))
#else
		if (!strcasecmp(entry->GetConvertName(), convertName))
#endif
			break;

	return entry;

} /* Lookup */

/****************************************************************/
/* End of file */
/****************************************************************/

