/*
 * Copyright (c) 1992 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London,
 * London, England. Neither the name of the University, nor its
 * registered trademark, UCL, may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/
/****************************************************************/
/*
	GDMOBehav.h

	Class that represents a	GDMO Behaviour

	Author:	J.Cowan		(j.cowan@.cs.ucl.ac.uk)

*/
/****************************************************************/

#ifndef	_SYMBOL
#include "Symbol.h"
#endif

#ifndef	_GENERIC_LIST_
#include "GenList.h"
#endif

#ifndef	_LIST_ITERATOR_
#include "ListIterator.h"
#endif

#ifndef	_GENERIC_STRING_
#include "GenStr.h"
#endif

/****************************************************************/

class GDMOBehaviour : public MetaClass
{
	StringSymbol	_name, _definition;
public:
	GDMOBehaviour(char *name) :
			MetaClass("GDMOBehaviour"),
			_name(name, "BehaviourName"),
			_definition("",	"Definition")
	{
		AddSymbol(&_name);
		AddSymbol(&_definition);
	}

	char    *GetName() { return (char *) _name;}
	char    *GetDefinition() { return (char *) _definition;}
};

/****************************************************************/

class	GDMOBehaviourList : public ListSymbol
{
protected:
	void	deleteElem (void*);
public:
	GDMOBehaviourList(Bool del, char *name)
	: ListSymbol(name, "GDMOBehaviour", ObjectTypeDef, del) {}
	GDMOBehaviourList(char *name)
	: ListSymbol(name, "GDMOBehaviour", ObjectTypeDef) {}
	~GDMOBehaviourList()	{ clear();}

	GDMOBehaviour *FindBehaviour(char *behaviourName)
	{
	GDMOBehaviour	*behaviour;
	ListIterator	iterator(this);

		while ((behaviour=(GDMOBehaviour *) iterator.getNext())
				!= (GDMOBehaviour *) 0)

			if (!strcmp(behaviour->GetName(), behaviourName))
				break;

		return behaviour;
	}
};

/****************************************************************/
// inline methods for GDMOBehaviourList

inline void  GDMOBehaviourList::deleteElem (void* val)
{  if (del) delete (GDMOBehaviour *) val; }

/****************************************************************/





