/* Lex declarations */

/* %C       target language is C */

%{
#undef yywrap
static  int yywrap();
static  int yyinput();
static	int readASN();
extern	int line;
#ifdef  FLEX_SCANNER
/*
extern  char *yytext;
*/
static  void yyunput(char, char *);
#define unput(c) yyunput( c, yytext )
#endif
#undef YY_DECL
#define YY_DECL int GDMO::Lex()
%}

%e  2000 /* nodes for the syntax tree */
%n  2000 /* transitions */
%p 20000 /* positions */
%a 15000 /* packed transitions */
%o 15000 /* table size */
%k   100 /* packed char classes */

/* constant definitions for lexemes */

LFCR    [\r\n]+
SPBSTAB [ \b\t]+
DIGIT   [0-9]+
LETTER  [A-Za-z]
COMMA   [,]
POINT   [.]
COLON   [:]
SEMI    [;]
LBRACE  [{]
RBRACE  [}]
LPARENTHESIS  [(]
RPARENTHESIS  [)]
SLASH   [/]
HYPHEN  [-]
QUOTE   ["]
ASNSEQ  [A-Za-z0-9/\-]*
SEQ     [A-Za-z0-9/\-]*
LO      [a-z]
UP      [A-Z]
TEXT    [\n\t\ -\!\#-\~]*

%{
/***********************************************************************
 * Scanner module for simple GDMO parser
 *
 * written by   Marcus Wittig
 *              X.400:wittig@fokus.berlin.gmd.dbp.de
 *              Gesellschaft fuer Mathematik und Datenverarbeitung (GMD)
 *              Forschungszentrum fuer offene Kommunikationssysteme (FOKUS)
 *              Hardenbergplatz 2
 *              W-1000 Berlin 12
 *
 *              Copyright 1992, GMD FOKUS
 *
 * Permission to copy this material, to redistribute it, and to
 * use it for any non-commercial purpose is granted, subject to the following
 * restrictions and understandings.
 *
 * 1. Any copy made of this material must include this copyright notice in
 *    full.
 *
 * 2. Users of this material agree to make their best efforts to inform GMD
 *    FOKUS of noteworthy uses of this material.
 *
 * 3. All materials developed as a consequence of the use of this material
 *    shall duly acknowledge such use, in accordance with the usual standards
 *    of acknowledging credit in academic research.
 *
 * 4. GMD FOKUS has made no warrantee or representation that this
 *    material (including the operation of software contained therein) will be
 *    error-free, and GMD FOKUS is under no obligation to provide any
 *    services, by way of maintenance, update, or otherwise.
 *
 * 5. In conjunction with products arising from the use of this material,
 *    there shall be no use of the name of GMD FOKUS nor of any
 *    adaptation thereof in any advertising, promotional, or sales literature
 *    without prior written consent from GMD FOKUS in each case.
 **********************************************************************/

/*
#include "gdmolex.h"
*/
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#define BOOLEAN int
#define TRUE    1
#define FALSE   0

static  void lc()
{
        register char *s;
        for (s = yytext; *s ; s++)
                if (*s == '\n')
                        line++;
}


/**********************************************************************/

static int checkRContext()
{
        int result = WITH; /* default token */
        BOOLEAN lookRight = TRUE; /* race condition for look ahead */
        char sym;
        char cs[6]; /* scan string */
        int x;

        do
                switch (sym = yyinput())
                {
                case '\n':
                case '\t':
                case ' ' : /* skip symbols */
                        break;
                case 'I': /* check for lexeme 'INFORMATION' */
                        cs[0] = sym;
                        for (x=1; x<5; x++)
                                cs[x] = yyinput();
                        cs[5] = '\0';
                        result = (!strcmp(cs, "INFOR"))?INFO_WITH:WITH;
                        for (x=4; x>=0; x--) unput(cs[x]); /* push back symbols */
                        lookRight = FALSE;
                        break;
                case 'R': /* check for lexeme 'REPLY' */
                        cs[0] = sym;
                        for (x=1; x<5; x++)
                                cs[x] = yyinput();
                        cs[5] = '\0';
                        result = (!strcmp(cs, "REPLY"))?REPLY_WITH:WITH;
                        for (x=4; x>=0; x--) unput(cs[x]); /* push back symbols */
                        lookRight = FALSE;
                        break;
                default:
                        /* push back and abort */
                        unput(sym);
                        lookRight = FALSE;
                        break;
                } /* switch */
        while (lookRight);

        return result;
} /* checkRContext() */

/**********************************************************************/

%}

/* Anfangsbedingungen */

%START ASN ASN1

%% /* REGELN */

"--"                            {       /* Kommentarzeilen */
                                        BOOLEAN commentEnd = FALSE;
                                        char c;

                                        while (commentEnd == FALSE)
                                        {
                                                switch(yyinput())
                                                {
                                                case '\n':
                                                        line++;
                                                case '\0':
                                                        commentEnd = TRUE;
                                                        break;
                                                case '-':
                                                        switch((c = yyinput()))
                                                        {
                                                        case '-':
                                                            commentEnd = TRUE;
                                                            break;
                                                        default :
                                                            unput(c);
                                                            break;
                                                        }
                                                        break;
                                                default :
                                                        break;
                                                }
                                        }
                                }

"LIBCOMMENT"                    { return (LIBCOMMENT);}
"LIBAUTHOR"                     { return (LIBAUTHOR);}

"REGISTERED"                    { return (REGISTERED);}
"AS"                            { return (AS);}
"PARAMETERS"                    { return (PARAMETERS);}
"BEHAVIOUR"                     { return (BEHAVIOUR);}

"MANAGED"                       { return (MANAGED);}
"OBJECT"                        { return (OBJECT);}
"CLASS"                         { return (CLASS);}
"DERIVED"                       { return (DERIVED);}
"FROM"                          { return (FROM);}
"CHARACTERIZED"                 { return (CHARACTERIZED);}
"BY"                            { return (BY);}
"CONDITIONAL"                   { return (CONDITIONAL);}
"PACKAGES"                      { return (PACKAGES);}
"PRESENT"                       { return (PRESENT);}
"IF"                            { return (IF);}
"WITH"                          { return (checkRContext());}

"PACKAGE"                       { return (PACKAGE);}
"ATTRIBUTES"                    { return (ATTRIBUTES);}
"ATTRIBUTE"                     { return (ATTRIBUTE);}
"GROUPS"                        { return (GROUPS);}
"ACTIONS"                       { return (ACTIONS);}
"NOTIFICATIONS"                 { return (NOTIFS);}
"REPLACE-WITH-DEFAULT"          { return (REPL_DEFAULT);}
"VALUES"                        { BEGIN ASN; return (VALUES);}
"VALUE"                         { BEGIN ASN; return (VALUE);}
"DEFAULT"                       { return (DEFAULT);}
"INITIAL"                       { return (INITIAL_T);}
"PERMITTED"                     { return (PERMITTED);}
"REQUIRED"                      { return (REQUIRED);}
<ASN>"DERIVATION"               { BEGIN 0; return (DERIVATION);}
"RULE"                          { return (RULE);}
"GET"                           { return (GET_T);}
"REPLACE"                       { return (REPLACE_T);}
"GET-REPLACE"                   { return (GET_REPLACE_T);}
"ADD"                           { return (ADD_T);}
"REMOVE"                        { return (REMOVE_T);}
"ADD-REMOVE"                    { return (ADD_REMOVE_T);}
"SET"                           { return (SET);}

"PARAMETER"                     { return (PARAMETER);}
"CONTEXT"                       { return (CONTEXT);}
"ACTION-INFO"                   { return (ACTION_INFO);}
"ACTION-REPLY"                  { return (ACTION_REPLY);}
"EVENT-INFO"                    { return (EVENT_INFO);}
"EVENT-REPLY"                   { return (EVENT_REPLY);}
"SPECIFIC-ERROR"                { return (SPECIFIC_ERR);}
"COMMON-ERROR"                  { return (COMMON_ERR);}
"SYNTAX"                        { BEGIN ASN; return (SYNTAX);}

"NAME"                          { return (NAME);}
"BINDING"                       { return (BINDING);}
"SUBORDINATE"                   { return (SUBORDINATE);}
"SUBCLASSES"                    { return (SUBCLASSES);}
"NAMED"                         { return (NAMED);}
"SUPERIOR"                      { return (SUPERIOR);}
"CREATE"                        { return (CREATE_T);}
"DELETE"                        { return (DELETE_T);}
"WITH-REFERENCE-OBJECT"         { return (WITH_REFERENCE_O);}
"WITH-AUTOMATIC-INSTANCE-NAMING" { return (WITH_AUTO_INST_N);}
"ONLY-IF-NO-CONTAINED-OBJECTS"  { return (ONLY_IF_NO_CONT_O);}
"DELETES-CONTAINED-OBJECTS"     { return (DELETE_CONT_O);}

"ATTRIBUTE"                     { return (ATTRIBUTE);}
"MATCHES"                       { return (MATCHES);}
"FOR"                           { return (FOR);}
"EQUALITY"                      { return (EQUALITY);}
"ORDERING"                      { return (ORDERING);}
"SUBSTRINGS"                    { return (SUBSTRINGS);}
"SET-COMPARISON"                { return (SET_COMP);}
"SET-INTERSECTION"              { return (SET_INTERSEC);}

"GROUP"                         { return (GROUP);}
"ELEMENTS"                      { return (ELEMENTS);}
"FIXED"                         { return (FIXED_T);}
"DESCRIPTION"                   { return (DESCRIPTION);}

"DEFINED"                       { return (DEFINED);}

"ACTION"                        { return (ACTION);}
"MODE"                          { return (MODE_T);}
"CONFIRMED"                     { return (CONFIRMED_T);}
"NON-CONFIRMED"                 { return (NONCONFIRMED_T);}
"INFORMATION"                   { return (INFORMATION);}
"REPLY"                         { return (REPLY);}

"NOTIFICATION"                  { return (NOTIFICATION);}
"AND"                           { return (AND);}
"IDS"                           { return (IDS);}

{DIGIT}                         { yylval.string_val = HeapAlloc(yytext);
                                  return (ID);}
<ASN>{UP}{ASNSEQ}               { yylval.string_val = HeapAlloc(yytext);
				  return readASN();}
<ASN1>{UP}{ASNSEQ}              { BEGIN 0;
                                  yylval.string_val = HeapAlloc(yytext);
                                  return (ASN1_TYPEREF);}
<ASN1>{LO}{ASNSEQ}              { BEGIN 0;
                                  yylval.string_val = HeapAlloc(yytext);
                                  return (ASN1_VALREF);}
{LETTER}{SEQ}                   { yylval.string_val = HeapAlloc(yytext);
                                  return (LABEL);}
{QUOTE}                         { return (QUOTE);}
{POINT}                         { return (POINT);}
{COLON}                         { return (COLON);}
{SEMI}                          { return (SEMI);}
{LBRACE}                        { return (LBRACE);}
{RBRACE}                        { return (RBRACE);}
{COMMA}                         { return (COMMA);}
{LFCR}                          { lc();}
{SPBSTAB}                       { /* wird ignoriert */}

%% /* Funktionen */

/**********************************************************************/

#define SCAN_CONTEXT    int

#define SCAN_TEXT       1
#define SCAN_DID        2
#define SCAN_SKIP       3

/****************************************************************/

char	*GDMO::readToQuote(SCAN_CONTEXT what)
{
        BOOLEAN end = FALSE;
        BOOLEAN cont = FALSE;
        char c, c1;
        int x = 0;

        while ((x<MAXLEXBUFFER) && (end == FALSE))
        {
                switch((c = yyinput()))
                {
                case '\n':
                        line++;
                        break;
                case '\0':
                        end = TRUE;
                        switch(what)
                        {
                        case SCAN_TEXT:
                        case SCAN_DID:
                                fprintf(stderr,"\nUnexpected end of file\n");
                                break;
                        default:
                                break;
                        }
                        break;
                case '\\':
                        if (cont != TRUE)
                        {
                                switch((c1 = yyinput()))
                                {
                                case '"':
                                    cont = TRUE;
                                    break;
                                default :
                                    end = TRUE;
                                    break;
                                }
                                unput(c1);
                        }
                        break;
                case '"':
                        if (cont != TRUE)
                        {
                                switch((c1 = yyinput()))
                                {
                                case '"':
                                    cont = TRUE;
                                    break;
                                default :
                                    end = TRUE;
                                    break;
                                }
                                unput(c1);
                        }
                        else
                                cont = FALSE;
                        break;
                default :
                        break;
                } /* switch */

                if (end == TRUE)
                {
                        _lexBuffer[x] = '\0';
                        unput(c);

                }
                else
                        _lexBuffer[x] = c;
                x++;

        } /* while */

        if (end == FALSE)
        {
                _lexBuffer[x] = '\0';
                switch(what)
                {
                case SCAN_TEXT:
                        fprintf(stderr, "\nText-field thas more than 1024 chars!\n");
                        break;
                case SCAN_DID:
                        fprintf(stderr, "\nDocument id has more than 100 chars!\n");
                        break;
                default:
                        break;
                }
                (void)readToQuote(SCAN_SKIP);
        }

        return HeapAlloc(_lexBuffer);

} /* readToQuote() */

/****************************************************************/

char	*GDMO::getText()
{
        return readToQuote(SCAN_TEXT);

} /* getText() */

/****************************************************************/

char *GDMO::getDocId()
{
        return readToQuote(SCAN_DID);

} /* getDocId() */

/****************************************************************/

char	*GDMO::getDelimitedText()
{
int     first;
int	charCount = 0;

        /* Read first character */
        do
        {
                first = yyinput();
                if (!isspace(first))
                        break;
		else
			if (first == '\n')
				line++;
        }
        while (1);

        switch (first)
        {
        default:
                fprintf(stderr, "\nInvalid Text delimiter");
                return "";

        case '!': case '"': case '#': case '$': case '%':
        case '^': case '&': case '*': case '\'': case '`':
        case '~': case '?': case '@': case '\\':
                break;
        }

        /* Now read to delimiter */

        do
        {
        int     ch;

                ch = yyinput();
                if (ch == first)
                {
                int     next = yyinput();

                        if (next != first)
                        {
                                unput(next);
				_lexBuffer[charCount] = '\0';
                                return(HeapAlloc(_lexBuffer));
                        }
			// 2 of the same ie quoted

			_lexBuffer[charCount++] = ch;
                }
                switch (ch)
                {
                case EOF:
                        fprintf(stderr, "\nText missing delimiter");
                        exit(1);
                        return "";

                case '\n':
                        line++;
                        break;
                }
		if (charCount<MAXLEXBUFFER)
			_lexBuffer[charCount++] = ch;
		else
		{
                        fprintf(stderr, "\nText missing delimiter");
                        exit(1);
		}
        }
        while (1);

} /* getDelimitedText */

/****************************************************************/

char    *GDMO::HeapAlloc(char *text)
{
StringSymbol    *ssym;

        ssym = (NEW StringSymbol(text, ""));
        _lexSymTable.Add(ssym, False);
        return(ssym->GetValue());

} /* HeapAlloc */

/****************************************************************/

static  void switchToASN1()
{
        BEGIN ASN;
} /* switchToASN1 */

static  int yywrap()
{
        (void)printf("\nNumber of lines : %d\n", line);
        return (1);
}
/* yywrap() */

/**********************************************************************/

static int readASN()
{
static	char	*basicTypes[] =
	{
	"Integer",
	"Real",
	"Null",
	"Boolean",
	"Enumerated",
	"BitString",
	"OctetString",
	"ObjectIdentifier",
	"NumericString",
	"PrintableString",
	"TeletexString",
	"VideotexString",
	"VisibleString",
	"Ia5String",
	"GraphicString",
	"GeneralString",
	"GeneralizedTime"
	};
int	count;
int	lgth = sizeof(basicTypes)/sizeof(char *);

	/* Saleem and I added this function so that you could access 
	Basic types in a GDMO script but now taken out as not in 
	the standard - a good enhancement though! now commented out */

	if (1)
	{
		BEGIN ASN1;
         	return (ASN1_MODREF);
	}
	
	for (count=0; count<lgth; count++)
	{
	char	*text = basicTypes[count];
#ifdef  __MSDOS__
                if (!stricmp(text, yytext))
#else
                if (!strcasecmp(text, yytext))
#endif
			break;
	}

	if (count < lgth)
	{
		BEGIN 0;
         	return (ASN1_BASICTYPEREF);
	}
	else
	{
		BEGIN ASN1;
         	return (ASN1_MODREF);
	}

} /* readAsn */

/**********************************************************************/
