/*
 * Copyright (c) 1992 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London,
 * London, England. Neither the name of the University, nor its
 * registered trademark, UCL, may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/
/****************************************************************/
/*
	GDMOPackage.h

	Class that represents a	GDMO Package

	Author:	J.Cowan		(j.cowan@.cs.ucl.ac.uk)
*/
/****************************************************************/

#ifndef	_SYMBOL
#include "Symbol.h"
#endif

#ifndef	_GENERIC_LIST_
#include "GenList.h"
#endif

#ifndef	_GENERIC_STRING_
#include "GenStr.h"
#endif

/****************************************************************/

class GDMOPackage : public MetaClass
{
	StringSymbol		_name;
	GDMOAttrList		_attrRefList;
	GDMOActionRefList	_actionRefList;
	GDMOActionList		_actionList;
	GDMONotificationRefList	_notificationRefList;
	GDMONotificationList	_notificationList;
	GDMOAttrGroupList	_attributeGroupList;
	GDMOOid			_oid;

	Bool	UpdateAttrRefs(GDMOAttrList &);
	Bool	UpdateAttrGroupRefs(GDMOAttrList &, GDMOAttrGroupList &);
	Bool	UpdateActionRefs(GDMOActionList &);
	Bool	UpdateNotificationRefs(GDMONotificationList &);
	void	CopyAttributes(GDMOAttrList *);
	void	CopyAttributeGroups(GDMOAttrGroupList *);
	void	CopyActions(GDMOActionList *);
	void	CopyNotifications(GDMONotificationList *);
	void	PrintError(char *msg)
	{
		printf("\nError: %s", msg);
	}
public:
	GDMOPackage(char *name) :
		MetaClass("GDMOPackage"),
		_name(name, "PackageName"),
		_attrRefList("AttributeList"),
		_actionList(False, "ActionList"),
		_notificationList(False, "NotificationList"),
		_attributeGroupList("AttributeGroupList"),
		_oid("Oid")
	{
		AddSymbol(&_name);
		AddSymbol(&_attrRefList);
		AddSymbol(&_actionList);
		AddSymbol(&_notificationList);
		AddSymbol(&_attributeGroupList);
		AddSymbol(&_oid);
	}

	char	*GetName()
	{
		return _name.print();
	}

	GDMOAttribute	*AddAttribute(char *);

	void	AddAction(char *);

	void	AddNotification(char *);

	GDMOAttrList *GetAttributes() { return &_attrRefList; }

	void	CopyPackage(GDMOAttrList *, GDMOActionList *,
			    GDMONotificationList *,
			    GDMOAttrGroupList *);

	Bool	UpdateRefs(GDMOAttrList &, GDMOActionList &,
			   GDMONotificationList &, GDMOAttrGroupList &);

	GDMOOid	*GetOid() { return &_oid;}
};

/****************************************************************/

class GDMOPackageRef
{
	GString		_name;
	GDMOPackage	*_package;
public:
	GDMOPackageRef(char *name)
	{
		_name = name;
		_package = (GDMOPackage *) 0;
	}
	void	UpdateRef(GDMOPackage *package)
	{
		_package = package;
	}
	char	*GetName()
	{
		return (char *)_name;
	}
	GDMOPackage	*GetPackage()
	{
		if (!_package)
			printf("\nPackage %s not updated",
				(char *) _name);
		return _package;
	}
};

/****************************************************************/

class	GDMOPackageList : public ListSymbol
{
protected:
	void	deleteElem (void*);
public:
	GDMOPackageList(Bool del, char *name)
	: ListSymbol(name, "GDMOPackage", ObjectTypeDef, del) {}
	GDMOPackageList(char *name)
	: ListSymbol(name, "GDMOPackage", ObjectTypeDef) {}
	~GDMOPackageList() { clear();}

	GDMOPackage	*FindPackage(char *);
};

inline void  GDMOPackageList::deleteElem (void* val)
{  if (del) delete (GDMOPackage *) val; }

/****************************************************************/

class	GDMOPackageRefList : public List
{
protected:
	void	deleteElem (void*);
public:
	~GDMOPackageRefList() { clear();}
};

inline void  GDMOPackageRefList::deleteElem (void* val)
{  if (del) delete (GDMOPackageRef *) val; }

/****************************************************************/




