/***********************************************************************
 * gdmo - a simple parser for GDMO templates
 *
 * written by   Marcus Wittig
 *              X.400:wittig@fokus.berlin.gmd.dbp.de
 *              Gesellschaft fuer Mathematik und Datenverarbeitung (GMD)
 *              Forschungszentrum fuer offene Kommunikationssysteme (FOKUS)
 *              Hardenbergplatz 2
 *              W-1000 Berlin 12
 *
 *              Copyright 1992, GMD FOKUS
 *
 * Permission to copy this material, to redistribute it, and to
 * use it for any non-commercial purpose is granted, subject to the following
 * restrictions and understandings.
 *
 * 1. Any copy made of this material must include this copyright notice in
 *    full.
 *
 * 2. Users of this material agree to make their best efforts to inform GMD
 *    FOKUS of noteworthy uses of this material.
 *
 * 3. All materials developed as a consequence of the use of this material
 *    shall duly acknowledge such use, in accordance with the usual standards
 *    of acknowledging credit in academic research.
 *
 * 4. GMD FOKUS has made no warrantee or representation that this
 *    material (including the operation of software contained therein) will be
 *    error-free, and GMD FOKUS is under no obligation to provide any
 *    services, by way of maintenance, update, or otherwise.
 *
 * 5. In conjunction with products arising from the use of this material,
 *    there shall be no use of the name of GMD FOKUS nor of any
 *    adaptation thereof in any advertising, promotional, or sales literature
 *    without prior written consent from GMD FOKUS in each case.
 **********************************************************************/



/* DEFINITIONS */
%{

#include <stdio.h>
#include <string.h>
#include "DebugMem.h"
#include "GDMOOid.h"
#include "GDMOAttr.h"
#include "GDMOAct.h"
#include "GDMOBeh.h"
#include "GDMONot.h"
#include "GDMOPkg.h"
#include "GDMOClas.h"
#include "GDMO.h"

extern "C" {
extern FILE *yyin;
static char *yytext;
int line = 1;
}
#define yylex   Lex
/*
#define YYDEBUG   1
*/
extern int GdmoParseErrors();

%}

%union
        {
        int     int_val;
        char    *string_val;
        }

/* Startsymbol of the grammar */

%start templates


/* tokens */

%token <string_val> LABEL       /* <LABEL> */
%token <string_val> ID          /* <ID> */
%token QUOTE                    /* """ */
%token POINT                    /* "." */
%token COMMA                    /* "," */
%token COLON                    /* ":" */
%token SEMI                     /* ";" */
%token LBRACE                   /* "{" */
%token RBRACE                   /* "}" */
%token LPARENTHESIS             /* "(" */
%token RPARENTHESIS             /* ")" */
%token ASN1_BASICTYPEREF        /* ASN1 BASIC TYPE */
%token ASN1_MODREF              /* ASN1 MODULE REFERENCE */
%token ASN1_TYPEREF             /* ASN1 TYPE REFERENCE */
%token ASN1_VALREF              /* ASN1 VALUE REFERENCE */
%token REGISTERED               /* "REGISTERED" .AS. */
%token AS                       /* .REGISTERED. "AS" */
%token PARAMETERS               /* "PARAMETERS" */
%token BEHAVIOUR                /* "BEHAVIOUR" */
%token DERIVED                  /* "DERIVED" .FROM. */
%token FROM                     /* .DERIVED. "FROM" */
%token WITH                     /* "WITH" */
%token INFO_WITH                /* "WITH" .INFORMATION.  */
%token REPLY_WITH               /* "WITH" .REPLY. */
%token ATTRIBUTE                /* "ATTRIBUTE" */
%token BY                       /* "BY" */
%token AND                      /* "AND" */

/* Specific tokens for Library Info */

%token LIBCOMMENT               /* "LIBCOMMENT" */
%token LIBAUTHOR                /* "LIBAUTHOR" */

/* Specific tokens for Man. Object Class-Templates */

%token MANAGED                  /* "MANAGED" .OBJECT. .CLASS. */
%token OBJECT                   /* .MANAGED. "OBJECT" .CLASS. */
%token CLASS                    /* .MANAGED. .OBJECT. "CLASS" */
%token CHARACTERIZED            /* "CHARACTERIZED" .BY. */
%token CONDITIONAL              /* "CONDITIONAL" .PACKAGES. */
%token PACKAGES                 /* .CONDITIONAL. "PACKAGES" */
%token PRESENT                  /* "PRESENT" .IF. */
%token IF                       /* .PRESENT. "IF" */

/* Specific tokens for Package Templates */

%token PACKAGE                  /* "PACKAGE" */
%token ATTRIBUTES               /* "ATTRIBUTES" */
%token GROUPS                   /* .ATTRIBUTE. "GROUPS" */
%token ACTIONS                  /* "ACTIONS" */
%token NOTIFS                   /* "NOTIFICATIONS" */
%token REPL_DEFAULT             /* "REPLACE-WITH-DEFAULT" */
%token VALUE                    /* {.DEFAULT.|.INITIAL.} "VALUE" */
%token VALUES                   /* {.PERMITTED.|.REQUIRED.} "VALUES" */
%token DEFAULT                  /* "DEFAULT" .VALUE. */
%token INITIAL_T                /* "INITIAL" .VALUE. */
%token PERMITTED                /* "PERMITTED" .VALUES. */
%token REQUIRED                 /* "REQUIRED" .VALUES. */
%token DERIVATION               /* "DERIVATION" .RULE. */
%token RULE                     /* .DERIVATION. "RULE" */
%token GET_T                    /* "GET" */
%token REPLACE_T                /* "REPLACE" */
%token GET_REPLACE_T            /* "GET-REPLACE" */
%token ADD_T                    /* "ADD" */
%token REMOVE_T                 /* "REMOVE" */
%token ADD_REMOVE_T             /* "ADD-REMOVE" */

/* Specific tokens for Parameter Templates */

%token PARAMETER                /* "PARAMETER" */
%token CONTEXT                  /* "CONTEXT" */
%token ACTION_INFO              /* "ACTION-INFO" */
%token ACTION_REPLY             /* "ACTION-REPLY" */
%token EVENT_INFO               /* "EVENT-INFO" */
%token EVENT_REPLY              /* "EVENT-REPLY" */
%token SPECIFIC_ERR             /* "SPECIFIC-ERROR" */
%token COMMON_ERR               /* "COMMON-ERROR" */
%token SYNTAX                   /* .WITH. "SYNTAX" */

/* Specific tokens for Name Binding Templates */

                                /* "WITH ATTRIBUTE" */
%token NAME                     /* "NAME" .BINDING. */
%token BINDING                  /* .NAME. "BINDING" */
%token SUBORDINATE              /* "SUBORDINATE" .OBJECT. .CLASS. */
%token SUBCLASSES               /* .AND. "SUBCLASSES" */
%token NAMED                    /* "NAMED" .BY. */
%token SUPERIOR                 /* "SUPERIOR" .OBJECT. .CLASS. */
%token CREATE_T                 /* "CREATE" */
%token DELETE_T                 /* "DELETE" */
%token WITH_REFERENCE_O         /* "WITH-REFERENCE-OBJECT" */
%token WITH_AUTO_INST_N         /* "WITH-AUTO-INSTANCE-NAMING" */
%token ONLY_IF_NO_CONT_O        /* "ONLY-IF-NO-CONTAINED-OBJECTS" */
%token DELETE_CONT_O            /* "DELETES-CONTAINED-OBJECTS" */

/* Specific tokens for Attribute Templates */

                                /* "WITH ATTRIBUTE SYNTAX" */
%token MATCHES                  /* "MATCHES" .FOR. */
%token FOR                      /* .MATCHES. "FOR" */
%token EQUALITY                 /* "EQUALITY" */
%token ORDERING                 /* "ORDERING" */
%token SUBSTRINGS               /* "SUBSTRINGS" */
%token SET_COMP                 /* "SET-COMPARISON" */
%token SET_INTERSEC             /* "SET-INTERSECTION" */

/* Specific tokens for Attribute Group Templates */

%token GROUP                    /* {.ATTRIBUTE."GROUP"}|{"GROUP" .ELEMENTS.} */
%token ELEMENTS                 /* .GROUP. "ELEMENTS" */
%token FIXED_T                  /* "FIXED" */
%token DESCRIPTION              /* "DESCRIPTION" */

/* Specific tokens for Behaviour Templates */

%token DEFINED                  /* "DEFINED" .AS. */

/* Specific tokens for Action Templates */

%token ACTION                   /* "ACTION" */
%token MODE_T                   /* "MODE" .CONFIRMED. */
%token CONFIRMED_T              /* .MODE. "CONFIRMED" */
%token NONCONFIRMED_T           /* .MODE. "NONCONFIRMED" */
%token INFORMATION              /* .WITH. "INFORMATION" .SYNTAX. */
%token REPLY                    /* .WITH. "REPLY" .SYNTAX. */

/* Specific tokens for Notification Templates */

%token NOTIFICATION             /* "NOTIFICATION" */
%token IDS                      /* .AND. .ATTRIBUTE. "IDS" */

%type  <string_val> template_name
%type  <string_val> string number
%type  <string_val> ref
%type  <string_val> attrib_ref attribute_templ action_templ notif_templ
%type  <string_val> att_group_templ pack_ref group_ref
%type  <string_val> class_ref sub_class_def sup_class_def attribute_def
%type  <string_val> moc_templ package_templ parameter_templ n_binding_templ
%type  <string_val> s_label module_ref type_ref value_ref value_spec
%type  <string_val> descript_txt behaviour_templ behav_ref
%type  <string_val> ASN1_BASICTYPEREF ASN1_MODREF ASN1_TYPEREF ASN1_VALREF
%type  <int_val>    mode_def mode_conf

%% /* grammar rules */

templates       :       template
                        |
                        templates template
                        ;
template        :       moc_templ
                        |
                        package_templ
                        |
                        parameter_templ
                        |
                        n_binding_templ
                        |
                        attribute_templ
                        |
                        att_group_templ
                        |
                        behaviour_templ
                        |
                        action_templ
                        |
                        notif_templ
                        |
                        erroneous_templ
                        ;
moc_templ       :       template_name MANAGED OBJECT CLASS
                        {
                                NewMoClass($1);
                        }
                        moc_body SEMI
                        ;
package_templ   :       template_name
                        {
                                NewPackage($1);
                        }
                        PACKAGE pack_body SEMI
                        ;
parameter_templ :       template_name PARAMETER param_body SEMI
                        ;
n_binding_templ :       template_name
                        {
                                NewNameBinding($1);
                        }
                        NAME BINDING n_bind_body SEMI
                        ;
attribute_templ :       template_name ATTRIBUTE
                        {
                                NewAttribute($1);
                                $$ = $1;
                        }
                        attribute_body SEMI
                        ;
att_group_templ :       template_name ATTRIBUTE GROUP
                        {
                                NewAttributeGroup($1);
                                $$ = $1;
                        }
                        att_group_body SEMI
                        ;
behaviour_templ :       template_name BEHAVIOUR 
                        {
                                NewBehaviour($1);
                                $$ = $1;
                        }
			behaviour_body SEMI
                        ;
action_templ    :       template_name ACTION
                        {
                                NewAction($1);
                                $$ = $1;
                        }
                        action_body SEMI
                        ;
notif_templ     :       template_name NOTIFICATION
                        {
                                NewNotification($1);
                                $$ = $1;
                        }
                        notif_body SEMI
                        ;
erroneous_templ :       error
                        ;
template_name   :       opt_document_ref string
                        {
                                $$ = $2;
                        }
                        ;
opt_document_ref:       QUOTE document_id QUOTE COLON
			|
			oid COLON
                        |
                        empty
                        ;
document_id     :       empty
                        {
                                (void)getDocId();
                        }
                        ;
opt_libcomment  :       LIBCOMMENT descript_txt SEMI
                        |
                        empty
                        ;
opt_libauthor   :       LIBAUTHOR descript_txt SEMI
                        |
                        empty
                        ;
class_ref       :       ref
                        {
                                $$ = $1;
                        }
                        |
                        moc_templ
                        ;
pack_ref        :       ref
                        |
                        package_templ
                        ;
param_ref       :       ref
                        |
                        parameter_templ
                        ;
attrib_ref      :       ref
                        {
                                $$ = $1;
                        }
                        |
                        attribute_templ
                        {
                                $$ = $1;
                        }
                        ;
group_ref       :       ref
                        {
                                $$ = $1;
                        }
                        |
                        att_group_templ
                        {
                                $$ = $1;
                        }
                        ;
behav_ref       :       ref
			{
				$$ = $1;
			}
                        |
                        behaviour_templ
			{
				$$ = $1;
			}
                        ;
action_ref      :       ref
                        {
                                AddPackageAction($1);
                        }
                        |
                        action_templ
                        {
                                AddPackageAction($1);
                        }
                        ;
notif_ref       :       ref
                        {
                                AddPackageNotification($1);
                        }
                        |
                        notif_templ
                        {
                                AddPackageNotification($1);
                        }
                        ;
moc_body        :       opt_libcomment
                        opt_libauthor
                        opt_deriv_from
                        opt_char_by
                        opt_cond_pack
                        reg_as
                        {
                                AddMoRegistration();
                        }
                        ;
reg_as          :       REGISTERED AS
                        {
                                ClearRegistration();
                        }
                        oid
                        ;
opt_deriv_from  :       DERIVED FROM class_ref_l SEMI
                        |
                        empty
                        ;
class_ref_l     :       class_ref
                        {
                                AddMoParent($1);
                        }
                        |
                        class_ref_l COMMA class_ref
                        {
                                AddMoParent($3);
                        }
                        ;

opt_char_by     :       CHARACTERIZED BY pack_ref_l SEMI
                        |
                        empty
                        ;
pack_ref_l      :       pack_ref
                        {
                                AddMoPackage($1);
                        }
                        |
                        pack_ref_l COMMA pack_ref
                        {
                                AddMoPackage($3);
                        }
                        ;
opt_cond_pack   :       CONDITIONAL PACKAGES cpack_ref_l SEMI
                        |
                        empty
                        ;
cpack_ref_l     :       pack_ref PRESENT IF descript_txt
                        |
                        cpack_ref_l COMMA pack_ref PRESENT IF descript_txt
                        ;
opt_param       :       PARAMETERS para_ref_l SEMI
                        |
                        empty
                        ;
empty           :       /* leer */
                        ;
para_ref_l      :       param_ref
                        |
                        para_ref_l COMMA param_ref
                        ;
pack_body       :       opt_libcomment
                        opt_libauthor
                        opt_behaviour
                        opt_attributes
                        opt_groupatts
                        opt_actions
                        opt_notifs
                        opt_reg_as
                        {
				_behavRefList.Copy(_curPackage->FindSymbol(
						   "BehaviourReferenceList"));
                                AddPackageRegistration();
                        }
                        ;
opt_behaviour   :       BEHAVIOUR 
			{
				_behavRefList.clear();	// clear list
			}
			behav_ref_l SEMI
                        |
                        empty
			{
				_behavRefList.clear();	// clear list
			}
                        ;
behav_ref_l     :       behav_ref
			{
				_behavRefList.append(NEW StringSymbol($1, ""));
			}
                        |
                        behav_ref_l COMMA behav_ref
			{
				_behavRefList.append(NEW StringSymbol($3, ""));
			}
                        ;
opt_attributes  :       ATTRIBUTES attrib_ref_l SEMI
                        |
                        empty
                        ;
attrib_ref_l    :       attrib_ref
                        {
                                AddPackageAttribute($1);
                        }
                        property_l opt_rel_params
                        |
                        attrib_ref_l COMMA attrib_ref
                        {
                                AddPackageAttribute($3);
                        }
                        property_l opt_rel_params
                        ;
property_l      :       opt_replace
                        opt_def_val
                        opt_init_val
                        opt_perm_val
                        opt_req_val
                        opt_get_repl
                        opt_add_rem
                        ;
opt_replace     :       REPL_DEFAULT
                        {
                                UpdatePackageAttribute("ReplaceWithDefault",
                                                        True, Assign);
                        }
                        |
                        empty
                        ;
opt_def_val     :       DEFAULT VALUE value_spec
                        {
                                UpdatePackageAttribute("DefaultValue", $3);
                        }
                        |
                        empty
                        ;
opt_init_val    :       INITIAL_T VALUE value_spec
                        {
                                UpdatePackageAttribute("InitialValue", $3);
                        }
                        |
                        empty
                        ;
opt_perm_val    :       PERMITTED VALUES s_label
                        {
                                UpdatePackageAttribute("PermittedValue", $3);
                        }
                        |
                        empty
                        ;
opt_req_val     :       REQUIRED VALUES s_label
                        {
                                UpdatePackageAttribute("RequiredValue", $3);
                        }
                        |
                        empty
                        ;
opt_get_repl    :       GET_T
                        {
                                UpdatePackageAttribute("GetReplace", Get,
                                                        Assign);
                        }
                        |
                        REPLACE_T
                        {
                                UpdatePackageAttribute("GetReplace", Replace,
                                                        Assign);
                        }
                        |
                        GET_REPLACE_T
                        {
                                UpdatePackageAttribute("GetReplace",
                                                        GetReplace,
                                                        Assign);
                        }
                        |
                        empty
                        ;
opt_add_rem     :       ADD_T
                        {
                                UpdatePackageAttribute("AddRemove", Add,
                                                        Assign);
                        }
                        |
                        REMOVE_T
                        {
                                UpdatePackageAttribute("AddRemove", Remove,
                                                        Assign);
                        }
                        |
                        ADD_REMOVE_T
                        {
                                UpdatePackageAttribute("AddRemove", AddRemove,
                                                        Assign);
                        }
                        |
                        empty
                        ;
value_spec      :       s_label
                        {
                                $$ = $1;
                        }
                        |
                        DERIVATION RULE behav_ref
                        {
                                $$ = "";        // stop type clash
                        }
                        ;
opt_rel_params  :       rel_param_l
                        |
                        empty
                        ;
rel_param_l     :       param_ref
                        |
                        rel_param_l param_ref
                        ;
opt_groupatts   :       ATTRIBUTE GROUPS group_ref_l SEMI
                        |
                        empty
                        ;
group_ref_l     :       group_ref
                        {
                                AddPackageAttributeGroup($1);
                        }
                        opt_rel_atts
                        |
                        group_ref_l COMMA group_ref
                        {
                                AddPackageAttributeGroup($3);
                        }
                        opt_rel_atts
                        ;
opt_rel_atts    :       rel_att_ref_l
                        |
                        empty
                        ;
rel_att_ref_l   :       attrib_ref
                        {
                        StringSymbol *sym = (NEW StringSymbol($1, ""));

                                UpdatePackageAttrGroup("Extensions", sym);
                        }
                        |
                        rel_att_ref_l attrib_ref
                        {
                        StringSymbol *sym = (NEW StringSymbol($2, ""));

                                UpdatePackageAttrGroup("Extensions", sym);
                        }
                        ;
opt_actions     :       ACTIONS action_ref_l SEMI
                        |
                        empty
                        ;
action_ref_l    :       action_ref opt_rel_params
                        |
                        action_ref_l COMMA action_ref opt_rel_params
                        ;
opt_notifs      :       NOTIFS notif_ref_l SEMI
                        |
                        empty
                        ;
notif_ref_l     :       notif_ref opt_rel_params
                        |
                        notif_ref_l COMMA notif_ref opt_rel_params
                        ;
param_body      :       opt_libcomment
                        opt_libauthor
                        context_def
                        syntax_or_attr
                        opt_behaviour
                        opt_reg_as
                        ;
context_def     :       CONTEXT ACTION_INFO SEMI
                        |
                        CONTEXT ACTION_REPLY SEMI
                        |
                        CONTEXT EVENT_INFO SEMI
                        |
                        CONTEXT EVENT_REPLY SEMI
                        |
                        CONTEXT SPECIFIC_ERR SEMI
                        |
                        CONTEXT string SEMI
                        ;
syntax_or_attr  :       WITH SYNTAX s_label SEMI
                        |
                        ATTRIBUTE attrib_ref SEMI
                        ;
n_bind_body     :       opt_libcomment
                        opt_libauthor
                        sub_class_def
                        sup_class_def
                        attribute_def
                        opt_behaviour
                        opt_create
                        opt_delete
                        reg_as
                        {
                                AddNameBinding($3, $4, $5);
                                AddNameBindingRegistration();
				_behavRefList.Copy(_curNameBinding->FindSymbol(
						   "BehaviourReferenceList"));
                        }
                        ;
sub_class_def   :       SUBORDINATE
                        OBJECT CLASS class_ref opt_and_subclass SEMI
                        {
                                $$ = $4;
                        }
                        ;
sup_class_def   :       NAMED BY
                        SUPERIOR OBJECT CLASS class_ref opt_and_subclass SEMI
                        {
                                $$ = $6;
                        }
                        ;
opt_and_subclass:       AND SUBCLASSES
                        |
                        empty
                        ;
attribute_def   :       WITH ATTRIBUTE attrib_ref SEMI
                        {
                                $$ = $3;
                        }
                        ;
opt_create      :       CREATE_T create_modifier opt_rel_params SEMI
                        |
                        empty
                        ;
create_modifier :       cmod COMMA cmod
                        |
                        cmod
                        |
                        empty
                        {
                                AddCreateModifier(Create);
                        }
                        ;
cmod            :       WITH_REFERENCE_O
                        {
                                AddCreateModifier(CreateWithReference);
                        }
                        |
                        WITH_AUTO_INST_N
                        {
                                AddCreateModifier(CreateWithAutoInstance);
                        }
                        ;
opt_delete      :       DELETE_T delete_modifier opt_rel_params SEMI
                        |
                        empty
                        ;
delete_modifier :       dmod
                        |
                        empty
                        {
                        {
                                AddDeleteModifier(Delete);
                        }
                        }
                        ;
dmod            :       ONLY_IF_NO_CONT_O
                        {
                                AddDeleteModifier(DeleteIfNoContained);
                        }
                        |
                        DELETE_CONT_O
                        {
                                AddDeleteModifier(DeleteIfContained);
                        }
                        ;
attribute_body  :       opt_libcomment
                        opt_libauthor
                        deriv_or_syntax
                        opt_matches_for
                        opt_behaviour
                        opt_param
                        opt_reg_as
                        {
                                AddAttributeRegistration();
				_behavRefList.Copy(_curAttribute->FindSymbol(
						   "BehaviourReferenceList"));
                        }
                        ;
deriv_or_syntax :       DERIVED FROM attrib_ref SEMI
                        {
                                AddAttributeDerivation($3);
                        }
                        |
                        WITH ATTRIBUTE SYNTAX s_label SEMI
                        {
                                AddAttributeSyntax($4);
                        }
                        ;
opt_matches_for :       MATCHES FOR match_l SEMI
                        |
                        empty
                        ;
match_l         :       match
                        |
                        match_l COMMA match
                        ;
match           :       EQUALITY
                        {
                                UpdateAttribute("MatchesFor", Equality, Or);
                        }
                        |
                        ORDERING
                        {
                                UpdateAttribute("MatchesFor", Ordering, Or);
                        }
                        |
                        SUBSTRINGS
                        {
                                UpdateAttribute("MatchesFor", SubStrings, Or);
                        }
                        |
                        SET_COMP
                        {
                                UpdateAttribute("MatchesFor", SetComparison,
                                                Or);
                        }
                        |
                        SET_INTERSEC
                        {
                                UpdateAttribute("MatchesFor", SetInterSection,
                                                Or);
                        }
                        ;
att_group_body  :       opt_libcomment
                        opt_libauthor
                        opt_group_elems
                        opt_fixed
                        opt_description
                        reg_as
                        {
                                AddAttrGroupRegistration();
                        }
                        ;
opt_group_elems :       GROUP ELEMENTS att_ref_l SEMI
                        |
                        empty
                        ;
opt_fixed       :       FIXED_T SEMI
                        {
                                UpdateAttributeGroup("Fixed", True, Assign);
                        }
                        |
                        empty
                        ;
att_ref_l       :       attrib_ref
                        {
                        StringSymbol *sym = (NEW StringSymbol($1, ""));

                                UpdateAttributeGroup("Elements", sym);
                        }
                        |
                        att_ref_l COMMA attrib_ref
                        {
                        StringSymbol *sym = (NEW StringSymbol($3, ""));

                                UpdateAttributeGroup("Elements", sym);
                        }
                        ;
opt_description :       DESCRIPTION descript_txt SEMI
                        |
                        empty
                        ;
behaviour_body  :       opt_libcomment
                        opt_libauthor
                        defined_as_def
                        ;
defined_as_def  :       DEFINED AS descript_txt
			{
				UpdateBehaviour("Definition", $3);
			}
                        ;
action_body     :       opt_libcomment
                        opt_libauthor
                        opt_behaviour
                        mode_def
                        {
                                UpdateAction("Mode", $4, Assign);
                        }
                        opt_param
                        opt_info_syntax
                        opt_reply_syntax
                        reg_as
                        {
                                AddActionRegistration();
                        }
                        ;
mode_def        :       MODE_T mode_conf SEMI
                        {
                                $$ = $2;
                        }
                        |
                        empty
                        {
                                $$ = False;
                        }
                        ;
mode_conf       :       CONFIRMED_T
                        {
                                $$ = ModeConfirmed;
                        }
                        |
                        NONCONFIRMED_T
                        {
                                $$ = ModeNonConfirmed;
                        }
                        ;
opt_info_syntax :       INFO_WITH INFORMATION SYNTAX s_label SEMI
                        {
                                AddActionInfoSyntax($4);
                        }
                        |
                        empty
                        ;
opt_reply_syntax:       REPLY_WITH REPLY SYNTAX s_label SEMI
                        {
                                AddActionReplySyntax($4);
                        }
                        |
                        empty
                        ;
notif_body      :       opt_libcomment
                        opt_libauthor
                        opt_behaviour
                        mode_def
                        {
                                UpdateNotification("Mode", $4, Assign);
                        }
                        opt_param
                        opt_syn_a_atts
                        opt_notif_reply_syntax
                        reg_as
                        {
                                AddNotificationRegistration();
                        }
                        ;
opt_syn_a_atts  :       INFO_WITH INFORMATION SYNTAX s_label opt_att_ids SEMI
                        {
                                AddNotificationInfoSyntax($4);
                        }
                        |
                        empty
                        ;
opt_notif_reply_syntax: REPLY_WITH REPLY SYNTAX s_label SEMI
                        {
                                AddNotificationReplySyntax($4);
                        }
                        |
                        empty
                        ;
opt_att_ids     :       AND ATTRIBUTE IDS att_ids_l
                        |
                        empty
                        ;
att_ids_l       :       string attrib_ref
                        |
                        att_ids_l COMMA string attrib_ref
                        ;
opt_reg_as      :       reg_as
                        |
                        empty
                        {
                                ClearRegistration();
                        }
                        ;
ref             :       opt_document_ref string
                        {
                                $$ = $2;
                        }
                        |
                        error
                        {
                                $$ = "";        /* Prevent type clash */
                        }
                        ;
/*
descript_txt      :     QUOTE text QUOTE
*/
descript_txt      :     empty
                        {
                                $$ = getDelimitedText();
                        }
                        ;
/*
text            :       empty
                        {
                                (void)getText();
                        }
                        ;
*/
s_label         :       module_ref point value_ref
                        {
                                $$ = $3;
                        }
                        |
                        module_ref point type_ref
                        {
                                $$ = $3;
                        }
/*	Saleem and I added this so that you could access Basic types
	in a GDMO script but now taken out as not in the standard 
	- a good enhancement though! now commented out
			|
                        ASN1_BASICTYPEREF
                        {

                                $$ = $1;
                        }
*/
                        |
                        error
                        {
                                $$ = "";        /* Prevent type clash */
                        }
                        ;
module_ref      :       ASN1_MODREF
			{
				$$ = $1;
			}
                        ;
type_ref        :       ASN1_TYPEREF
                        {
                                $$ = $1;
                        }
                        ;
value_ref       :       ASN1_VALREF
                        {
                                $$ = $1;
                        }
                        ;
string          :       LABEL
                        {
                                $$ = $1;
                        }
                        ;
oid             :       LBRACE id_complist RBRACE
/*
                        |
                        id_complist
*/
                        ;
id_complist     :       id_complist opt_point id_component
                        |
                        id_component
                        ;
id_component    :       nameform
                        |
                        numberform
                        |
                        namenumberform
                        ;
nameform        :       string
                        {
                                AddRegistration($1);
                        }
                        ;
numberform      :       number
                        {
                                AddRegistration($1);
                        }
                        ;
number          :       ID
                        {
                                $$ = $1;
                        }
                        ;
namenumberform  :       string LBRACE number RBRACE
                        {
                                AddRegistration($1);
                                AddRegistration($3);
                        }
                        ;
opt_point       :       point
                        |
                        empty
                        ;
point           :       POINT
                        ;

%% /* functions */

static void yyerror(char *s)
{
        printf ("\nError: %s in line %d at token <%s>\n",
                s, line, yytext);
}

int GdmoParseErrors()
{
        return yynerrs;
}

