/*
 * Copyright (c) 1992 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London,
 * London, England. Neither the name of the University, nor its
 * registered trademark, UCL, may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/
/****************************************************************/
/*
	IIMCPar.cc

	IIMC Parser module (includes Lex and Yacc)

	Note:	this software uses multiple lex and yacc parses.
		Because yacc and lex communicate via a variable
		yylval of type YYSTYPE, the yacc and lex C code
		must be compiled in the same module so that they
		can both access yylval, without it being a global
		variable (ie it is a static variable).

		This is a bit horrible! Oh for a public domain
		version of yacc++ and lex++!

	Author: J.Cowan		(j.cowan@.cs.ucl.ac.uk)
*/
/****************************************************************/
/* Supply main test proram */
/*
#define	MAIN	1
*/
/****************************************************************/

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>

#include <string.h>
#include <sys/stat.h>

#include "IIMC.h"

/****************************************************************/

static	int	LineNumber;	/* Line number */

/****************************************************************/

#include "IIMCYacc.cc"

#include "IIMCLex.cc"

/****************************************************************/
/* IIMC methods */
/****************************************************************/

Bool	IIMC::_restart = False;

/****************************************************************/

int	IIMC::ParseErrors()
{
        return yynerrs;

} /* ParseErrors */

/****************************************************************/

Bool	IIMC::IsParseable(char *text)
{
char	*ptr = text;

	for (; *ptr; ptr++)
		if (!isspace(*ptr))
			break;
	if (*ptr)
	{
#ifdef  __MSDOS__
		if (!strnicmp(ptr, "BEGINPARSE", strlen("BEGINPARSE")))
#else
		if (!strncasecmp(ptr, "BEGINPARSE", strlen("BEGINPARSE")))
#endif
			return True;
	}

	return False;

} /* IsParseable */

/****************************************************************/
/* Main function */
/****************************************************************/

#ifdef	MAIN

FILE	*yyin, *yyout;
int	IIMCLex();

main(int argc, char *argv[])
{
struct	stat	statbuf;
char	*buffer;
FILE	*fp;
int	size;
int	nbytes;

	if (argc != 2)
	{
		printf("\nUsage is %s filename", argv[0]);
		exit(1);
	}

	if ((stat(argv[1], &statbuf)) || 
			((fp=fopen(argv[1], "r")) == (FILE *) 0))
	{
		printf("\nCannot open %s", argv[1]);
		exit(1);
	}

	size = ((int)statbuf.st_size) + 1;
	if ((buffer=(char *)malloc(size)) == (char *) 0)
	{
		printf("\nCannot allocate %d of memory", size);
		exit(1);
	}

	for (nbytes=0; nbytes<size-1; nbytes++)
	{
	int	c = fgetc(fp);

		if (c == EOF)
		{
			buffer[nbytes] = '\0';
			break;
		}
		buffer[nbytes] = c;
	}
	fclose(fp);

	printf("\nRead nbytes %d size %d", nbytes, size);
	if (nbytes)
	{
		printf("\nFile contents: %s", buffer);

		InitIIMCLex(buffer, 1, NBBEGINSTATE);

		IIMCParse();
	}
	free(buffer);
	return 0;

} /* main */

#endif

/****************************************************************/
/* End of module */
/****************************************************************/

