/*
 * Copyright (c) 1992 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London,
 * London, England. Neither the name of the University, nor its
 * registered trademark, UCL, may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/
/****************************************************************/
/*
        IIMCYacc.y

        YACC Parser for IIMC Parse statements

        Author: J.Cowan         (j.cowan@.cs.ucl.ac.uk)
*/
/****************************************************************/

/* DEFINITIONS */
%{
static  char    *yytext;
#define yylex   Lex
#define	YYDEBUG	1
%}

/* Yacc Stack Typedef */

%union
        {
        char            *text;
        }

/* Start symbol of the grammar */

%start begin

%token	BEGINMOCPARSE BEGINATTRIBUTEPARSE BEGINNAMEBINDINGPARSE
%token	REFERENCE DESCRIPTION INDEX AUGMENTS UNITS DEFVAL DISPLAYHINT
%token  TEXT DELETEATT DELETEVALUE ENDPARSE STRING
%token  SEMI NULLTOKEN COMMA IMPLIED DOT 
%token	LBRACE RBRACE 
%token  <text> NUMBER STRING VALUE

%% /* grammar rules */

/****************************************************************/

begin:		BEGINMOCPARSE mocparse
		|
		BEGINATTRIBUTEPARSE attrparse
		|
		BEGINNAMEBINDINGPARSE nbparse
		|
		;

/****************************************************************/
/* Managed Object Parse statements */

mocparse:	reference description index augments ENDPARSE
		;

/****************************************************************/
/* Attribute Parse statements */

attrparse:	reference description units attrdefvalue displayhint ENDPARSE

/****************************************************************/
/* Name Binding parse statements */

nbparse:	reference description index augments nbdelete ENDPARSE

/****************************************************************/
/* reference, description non terminals etc */

reference:	empty
		|
		REFERENCE TEXT SEMI
		;

description:	empty
		|
		DESCRIPTION TEXT SEMI
		;

index:		empty
		|
		INDEX indexAttrlist SEMI
		;

indexAttrlist:	NULLTOKEN
		|
		indexAttrLabel
		|
		indexAttrlist COMMA indexAttrLabel
		;

indexAttrLabel:	implied STRING DOT STRING
		{
		StringSymbol *sym;

			sym = (NEW StringSymbol($4, ""));
			_metaClass->UpdateSymbolList("IndexLabels", sym);
		}
		;

implied:	empty
		|
		IMPLIED
		;
	
augments:	empty
		|
		AUGMENTS entrylabel SEMI
		;

entrylabel:	STRING
		{
		StringSymbol *sym;

			sym = (NEW StringSymbol($1, ""));
			_metaClass->UpdateSymbolList("EntryLabel", sym);
		}
		|
		LBRACE STRING 
		{
		StringSymbol *sym;

			sym = (NEW StringSymbol($2, ""));
			_metaClass->UpdateSymbolList("EntryLabel", sym);
		}
		numericLabel RBRACE
		;

numericLabel:	empty
		|
		NUMBER
		{
		StringSymbol *sym;

			sym = (NEW StringSymbol($1, ""));
			_metaClass->UpdateSymbolList("EntryLabel", sym);
		}
		;

units:		empty
		|
		UNITS TEXT SEMI
		;

attrdefvalue:	empty
		|
		DEFVAL VALUE SEMI
		{
			_metaClass->UpdateSymbol("DefVal", $2);
		}
		;

displayhint:	empty
		|
		DISPLAYHINT TEXT SEMI
		;

nbdelete:	empty
		|
		DELETEATT STRING SEMI DELETEVALUE VALUE SEMI
		{
			_metaClass->UpdateSymbol("DeleteAttribute", $2);
			_metaClass->UpdateSymbol("DeleteValue", $5);
		}
		;

/****************************************************************/
/* Empty */

empty:		
		;

/****************************************************************/
%% /* functions */

static void yyerror(char *s)
{
        printf ("\nError: %s in line %d at token <%s>\n",
                s, LineNumber, yytext);
}

/****************************************************************/
/* End of Module */
/****************************************************************/


