//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//


#include <stdlib.h>
#include "msap.h"
#include "Error.h"
#include "ODP-Sntx.h"
#include "RMIBAgent.h"

#include "Real.h"
#include "RealList.h"


VoidFunc syntaxes[] = { smi_syntaxes, odp_syntaxes, NULL };

// the remote MIB agent
RMIBAgent agent;


main (int argc, char** argv)
{
    register char* cp;
    Bool host = False;

    if (argc < 2)
	goto usage;

    // load syntaxes

    if (initialiseSyntaxes(NULLCP, syntaxes) == NOTOK)
	error(NULLCP, "mibdump: cannot initialise syntaxes");

    // check if the optional [host] argument was given,
    // if not, location transparency will be used (if compiled in) -
    // we assume a host name will have at least one alphabetic character

    for (cp = argv[1]; *cp != '\0'; cp++)
        if (!isdigit(*cp)) {
	    host = True;
	    break;
	}

    // initialise the remote MIB agent

    if (agent.connect("ODPTEST", host ? argv[1] : NULLCP) != OK)
	error(NULLCP, "cannot establish association to ODPTEST server");

    CMISObjectList* objectList;
    AVA* actionArg;

    // prepare the argument according to the action type

    if ((!host && argc == 2) || (host && argc == 3)) 
			// 1 number - sqrt service asked
	actionArg = new AVA("calcSqrt", new Real(atof(argv[host ? 2 : 1])));
    else {		// > 1 numbers - meanStdDev service asked
	RealList* realList = new RealList;
	for (register int i = host ? 2 : 1; i < argc; i++)
	    realList->addval(atof(argv[i]));
	actionArg = new AVA("calcMeanStdDev", realList);
    }

    // perform the action (remote method call) and print the result/error -
    // as we do not know the name of the MO, we use scoping/filtering;
    // the only assumption is the that it is bound to the top MO (system)

    if (agent.Action(NULLCP, NULLCP, Sc_FirstLevel, 0,
		"(objectClass=simpleStats)", NULLMACCESS, NULLMSYNC,
		actionArg, objectList)
	    == OK) {
	objectList->print();
	delete objectList;
    }
    else
	advise(NULLCP, "Action error:  %s", objectList->getErrorStr());

    agent.disconnect();
    exit(OK);

usage:
    error(NULLCP, "usage: odptestclnt [host] <number> or \n\
       odptestclnt [host] <number> <number> ...");
}

