#include "math.h"


RDN simpleStats::makeRdn (MO*)
{
    // build the RDN

    char buf[20];
    sprintf(buf, "%s", "simpleStats=test");
    return str2rdn(buf);
}


int simpleStats::createRR (AVA*& err, void*, int)
{
    // this is a single-instanced class, so check if this is the 2nd instance

    if (MO::getClassInfo() -> getFirst() -> getClassNext()) {
	err = new AVA(m_processingFailure, "singleInstanceClass");
	return NOTOK;
    }
    return OK;
}


double calcSqrt (double);
void calcMeanStdDev (RealListVal*, double&, double&);


int  simpleStats::action (int actionId, int classLevel,
			void* actionArg, void*& actionReply, Bool& freeFlag,
			AVA*& err, Bool checkOnly, int)
{
    if (checkOnly)
	return OK;	// atomicity chack: we can always do it

    static double sqrt;
    static type_ODP_MeanStdDev meanStdDev;
    double mean, stdDev;

    switch (actionId) {

    case I_calcSqrt:
	if ((sqrt = *(double*) actionArg) < 0) {
	    err = new AVA(m_invalidArgumentValue);
	    return NOTOK;
	}
	
	sqrt = calcSqrt(sqrt);
	actionReply = &sqrt;
	freeFlag = False;	// we return a pointer in static storage
	break;

    case I_calcMeanStdDev:
	calcMeanStdDev((RealListVal*) actionArg, mean, stdDev);
	meanStdDev.mean = mean;
	meanStdDev.stdDev = stdDev;
	actionReply = &meanStdDev;
	freeFlag = False;	// we return a pointer in static storage
	break;

    default:
	// never reached
	break;
    }

    return OK;
}


double calcSqrt (double arg)
{
    if (arg < 0)
	return 0;
    return sqrt(arg);
}


void  calcMeanStdDev (register RealListVal* arg, double& mean, double& stdDev)
{
    register int sz;
    mean = stdDev = 0;

    for (sz = 0; arg; arg = arg->next, sz++) {
	mean += arg->real;
	stdDev += arg->real * arg->real;
    }

    mean = mean / sz;
    stdDev = sqrt( stdDev/sz- mean*mean );
}
