//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// IsodeAgent.h - generic ISODE association agent

// By George Pavlou, June 1993


/*
 * A generic ISODE agent abstract class that services association requests.
 * The ISODE coordinator tells it when an association request is pending
 * by using the single callback method provided. An instance of a derived
 * class should have previously registered with the ISODE coordinator
 * by using the registerAgent method of the latter.
 */


#ifndef _ISODE_AGENT_
#define _ISODE_AGENT_


#ifndef	_GENERIC_KS_
#include "GenericKS.h"
#endif


class ISODEAgent : public KS	// the generic ISODE association agent
{
protected:
    ISODEAgent ();		// abstract class

public:
    // callback for notifying association requests
    // (the ISODECoordinator invokes it)

    virtual int associationRequest (int, char**);
};


// inline methods

inline ISODEAgent::ISODEAgent ()			{ }
inline int  ISODEAgent::associationRequest (int, char**){ return NOTOK; }


#endif
