//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// TimeManager.h - time manager declaration

// By George Pavlou, February 1994


#ifndef _TIME_MANAGER_
#define _TIME_MANAGER_


#ifndef _GEN_ARRAY_
#include "GenArray.h"
#endif

#ifndef _COORDINATOR_
#include "Coordinator.h"
#endif

#ifndef _RMIB_AGENT_
#include "RMIBAgent.h"		/* includes RMIBManager.h */
#endif


class TimeManager : public RMIBManager
{
    KsWakeUpList _wakeUps;

    int		_timeMode;
    double	_simTimeIntvl;
    double	_simulatedTime;
    OID		_simulatedTimeOID;
    RMIBAgent*	_rmibAgent;

    static TimeManager* _instance;

    int		serviceWakeUp ();

public:
    // interface to knowledge sources

    int		scheduleWakeUps (KS*, char*, long, Bool, Bool);
    int		cancelWakeUps (KS*, char*);


    // methods to get the mode and simulated time

#define TM_REAL		0
#define TM_SIMULATED	1

    int		getTimeMode ();
    double	getSimulatedTime ();
    double	getSimTimeInterval ();


    // method to update the time - used only at the lowest end
    // of a management hierarchy (e.g. NEs, QAs in a TMN)

    void	updateTime (double);


    // a polymorphic method from RMIBManager to receive the new time event

    int		eventNotification (char*, char*, char*, char*, Attr*,
								RMIBAgent*);

    // initialisation

    int		initialise (RMIBAgent*);


    // instance access

    static TimeManager* getInstance ();


    // the default mode is TM_REAL, only simulated agents at the lowest level
    // of the hierarchy (e.g. QAs) should be instantiated with TM_SIMULATED

    TimeManager (int = TM_REAL);
    ~TimeManager ();
};


// inline methods

inline int  TimeManager::getTimeMode ()
{ return _timeMode; }

inline double  TimeManager::getSimulatedTime ()
{ return _simulatedTime; }

inline double  TimeManager::getSimTimeInterval ()
{ return _simTimeIntvl; }

inline TimeManager*  TimeManager::getInstance ()
{ return _instance; }

inline  TimeManager::~TimeManager ()
{ if (_simulatedTimeOID) oid_free(_simulatedTimeOID); }


#endif

