//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// XCoord.h - a special coordinator for applications that
//            need XWindows-based graphical user interfaces

// By George Pavlou, Xavier Pandolfi (Alcatel ISR) - June 1993


/*
 * This is a special coordinator for OSIMIS applications that wish to use
 * X-Windows based graphical user interfaces (usually applications in
 * manager roles only). It essentially integrates the OSIMIS coordinating
 * mechanism with that of X-Windows.
 *
 * Note that it is not possible for such applications to receive and service
 * association requests e.g. in cases where an agent tries to establish
 * one in order to send an event report (see the IsodeCoordinator class).
 * They may request and receive event reports though through an existing
 * association that they have established to an agent.
 */


#ifndef _X_COORD_
#define _X_COORD_


#ifndef _XtIntrinsic_h
#undef	Boolean		/* X uses also Boolean, Bool, True, False */
#undef	Bool
#undef	True
#undef	False
#define	XTFUNCPROTO	/* X manifest for ANSI C,  C++ */
#include <X11/Intrinsic.h>
#undef	Boolean
#undef	Bool
#undef	True
#undef	False
#endif

#ifndef _COORDINATOR_
#include "Coordinator.h"
#endif


class XCoordinator : public Coordinator
{
    XtInputId	  _commEndpointId [MAX_ENDPOINT];
    XtIntervalId  _lastIntervalId;
    XtAppContext* _appContext;

protected:
    int		commEndpointMaskChange (int, int);
    int		nextWakeUpIntvlChange (long);

public:
    static void	XReadCommEndpoint (XtPointer, int*, XtInputId*);
    static void	XWakeUp (XtPointer, XtIntervalId*);

    XCoordinator (XtAppContext*);
};


#endif
