//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// dbm-lock.h - "nice" interface to gdbm

// Saleem N. Bhatti
// October 1992

/*****************************************************************************
To help with gdbm.

   dbmLock(), dbmUnlock(), dbmStore() and dbmDelete() return 0 on success,
   -1 on failure.
   dbmOpen() and dbmFetch() return NULL on failure.
   dbmGetKey() returns NULL if no keys exist or after it has returned the
   final key.

   NOTE :
   dbmFetch() and dbmGetKey() return a pointer to a static area that will be
   overwritten on the next call to those functions.

dbmLock() and dbmUnLock() use flock(2). Care is required, as flock(2) does
not interact with locks from fcntl(2) or lockf(3). This means that the file
may be open to remote access, e.g. if hosts share filespace. flock(2) was
used as it incurred the least processing overhead, because it does not need to
use lockd(8) (and also required less time to implement :-). If it is found
that flock(2) provides insufficient protection for files, then these function
definitions may be changed to use fcntl(2) or lockf(3).
*****************************************************************************/

#if !defined(_dbm_lock_h_)
#define _dbm_lock_h_

extern "C" {

#if !defined(_dbm_lock_cc_)

typedef struct {
    char *dptr;
    int  dsize;
} datum;

typedef struct { int dummy[10]; } gdbm_file_info;

#endif

typedef gdbm_file_info DbmFile; /* This is a cookie! */
}

#define DBM_GetFirstKey   0
#define DBM_GetNextKey    1

#define DBM_NoLock        0
#define DBM_SharedLock    1 /* Non-blocking, no re-tries */
#define DBM_ExclusiveLock 2 /* Non-blocking, no re-tries */

#define DBM_ReadOnly      0
#define DBM_ReadWrite     1 /* Will create if required */

#define DBM_Replace       0
#define DBM_Insert        1

#define DBM_MaxRecordSize 81920 /* Don't really need this any more ... */

int     dbmLock(DbmFile *db, int lock = DBM_SharedLock);
int     dbmUnlock(DbmFile *db);
DbmFile *dbmOpen(char *name, int flags = DBM_ReadOnly, int mode = 0644);
void    dbmClose(DbmFile *db);
datum   *dbmFetch(DbmFile *db, datum *key, int lock = DBM_SharedLock);
int     dbmStore(DbmFile *db, datum *key, datum *entry,
                 int what = DBM_Replace, int lock = DBM_ExclusiveLock);
int     dbmDelete(DbmFile *db, datum *key, int lock = DBM_ExclusiveLock);
datum   *dbmGetKey(DbmFile *db, int what = DBM_GetNextKey,
                   int lock = DBM_SharedLock);

#endif /* !_dbm_lock_h_ */
