/*
 * Copyright (c) 1993 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* header file for Common Management Information Service providers  */

/*
 * By George Pavlou, June 1991
 */

#ifndef _MSAP_U_
#define _MSAP_U_


#ifndef FunctionPrototypes
#if defined(__STDC__) || defined(__cplusplus) || defined(__GNUG__)
#define FunctionPrototypes
#endif
#endif

#if defined(__cplusplus) || defined(__GNUG__)
extern "C" {
#endif


#ifndef _RoSAP_
#include <isode/rosap.h>
#endif

#ifndef _TSAP_
#include <isode/tsap.h>
#endif

#ifndef _MPARM_
#include "mparm.h"
#endif


#ifdef FunctionPrototypes

/* M-EVENT-REPORT.REQUEST (conf./non-conf.) */
int	M_EventRepAux_u (int, int, int, MID, MN, MID, char*, PE,
			MSAPIndication*);

/* M-EVENT-REPORT.RESPONSE */
int	M_EventRepRes_u (int, int, MID, MN, char*, CMISParam*,
			CMISErrors, CMISErrorInfo*, MSAPIndication*);

/* M-GET.REQUEST */
int	M_Get_u (int, int, MID, MN, CMISScope*, CMISFilter*,
			External*, CMISSync, int, MID, MSAPIndication*);

int	M_Get_u_sync (int, int, MID, MN, CMISScope*, CMISFilter*,
			External*, CMISSync, int, MID, MSAPIndication*);

/* M-GET.RESPONSE */
int	M_GetRes_u (int, int, int, MID, MN, char*, int, CMISGetAttr*,
			CMISErrors, CMISErrorInfo*, MSAPIndication*);

/* M-SET.REQUEST (confirmed/non-conf.) */
int	M_SetAux_u (int, int, int, MID, MN, CMISScope*, CMISFilter*, External*,
			CMISSync, int, CMISSetAttr*, MSAPIndication*);

/* M-SET.RESPONSE */
int	M_SetRes_u (int, int, int, MID, MN, char*, int, CMISSetAttr*,
			CMISErrors, CMISErrorInfo*, MSAPIndication*);

/* M-ACTION.REQUEST (confirmed/non-conf.) */
int	M_ActionAux_u (int, int, int, MID, MN, CMISScope*, CMISFilter*,
			External*, CMISSync, MID, PE, MSAPIndication*);

/* M-ACTION.RESPONSE */
int	M_ActionRes_u (int, int, int, MID, MN, char*, CMISParam*,
			CMISErrors, CMISErrorInfo*, MSAPIndication*);

/* M-CREATE.REQUEST */
int	M_Create_u (int, int, MID, MN, int, MN, External*,
			int, CMISParam*, MSAPIndication*);
int	M_Create_u_sync (int, int, MID, MN, int, MN, External*,
			int, CMISParam*, MSAPIndication*);

/* M-CREATE.RESPONSE */
int	M_CreateRes_u (int, int, MID, MN, char*, int, CMISParam*,
			CMISErrors, CMISErrorInfo*, MSAPIndication*);

/* M-DELETE.REQUEST */
int	M_Delete_u (int, int, MID, MN, CMISScope*, CMISFilter*,
			External*, CMISSync, MSAPIndication*);

/* M-DELETE.RESPONSE */
int	M_DeleteRes_u (int, int, int, MID, MN, char*,
			CMISErrors, CMISErrorInfo*, MSAPIndication*);

/* M-CANCEL-GET.REQUEST */
int	M_CancelGet_u (int, int, int, MSAPIndication*);

/* M-CANCEL-GET.RESPONSE */
int	M_CancelGetRes_u (int, int,
			CMISErrors, CMISErrorInfo*, MSAPIndication*);

/* M-WAIT.REQUEST (pseudo) */
int	M_WaitReq_u (int, int, MSAPIndication*);


/* CMIS A-ASSOCIATE.REQUEST */
int	M_InitialiseReq_u (AEI, struct PSAPaddr*, AEI, struct PSAPaddr*,
			char*, int, int, External*, External*,
			MSAPConnect*, MSAPIndication*);

/* CMIS A-ASSOCIATE.INDICATION */
int	M_Init_u (int, char**, MSAPStart*, MSAPIndication*);

/* CMIS A-ASSOCIATE.RESPONSE */
int	M_InitialiseResp_u (MSAPStart*, int, AEI, struct PSAPaddr*, char*,
			int, int, External*, External*, MSAPIndication*);

/* CMIS A-RELEASE.REQUEST */
int	M_TerminateReq_u (int, int, PE, MSAPIndication*);

/* CMIS A-RELEASE.RESPONSE */
int	M_TerminateResp_u (int, int, int, PE, MSAPIndication*);

/* CMIS A-ABORT.REQUEST */
int	M_AbortReq_u (int, External*, MSAPIndication*);


/* higher level management association control */

int	m_initialise_u (char*, char*, MInitOption **);
int	m_terminate_u (int);


/* Freeing Routines */

void  mc_free (MSAPConnect*);
void  ms_free (MSAPStart*);
void  ma_free (MSAPAbort*);
void  mid_free (MID);
void  mn_free (MN);
void  mparm_free (CMISParam*);
void  ava_free (Ava*);
void  mfilterdata_free (CMISFilter*);
void  mfilter_free (CMISFilter*);	/* frees both data and structures */
void  mi_free (MSAPIndication*);


/* Printing Routines */

char* MOperString (int);
char* MErrString (CMISErrors);

char* sprintscope (CMISScope*);
char* sprintsync (CMISSync);

char* sprintmfilter (const CMISFilter*);
char* sprintmfilteritem (const CMISFilterItem*);
char* sprintmfiltersubstrings (const CMISFilterSubstrings*);

typedef void (*VoidFunc)();
int	initialiseSyntaxes (const char*, VoidFunc*);
int	ava2str (OID, PE, char**, char**);
char*	attrv2str (void*, PRINT_FNX);


/* Parsing Routines */

CMISScope*  str2scope (char*);
CMISSync    str2sync (char*);

CMISFilter*		str2mfilter (char*);
CMISFilterItem*		str2mfilteritem (char*);
CMISFilterSubstrings*	str2mfiltersubstrings (char*);

Ava*	str2ava (char*, char*);


/* CMISFilter encoding/parsing */

#if 0
int  build_CMIP_CMISFilter (PE*, int, int, char*, CMISFilter*);
int  parse_CMIP_CMISFilter (PE, int, int*, char**, CMISFilter**);
#endif

/* Builds a structure of type External from its arguments */

External* external_build (int, char*, int, PE, struct qbuf*);


/* various... */

int	oid_equal (OID, OID);	/* compares two OIDs for equality */
int	pe2mem (PE, char**, int*);
int	mem2pe (PE*, char*, int);
int	pe2stream (PE, FILE*);
int	stream2pe (PE*, FILE*);


#else	/* Old-style C function prototypes */


int	M_EventRepAux ();	/* M-EVENT-REPORT.REQUEST (conf./non-conf.) */
int	M_EventRepRes ();	/* M-EVENT-REPORT.RESPONSE */
int	M_Get ();		/* M-GET.REQUEST */
int	M_GetRes ();		/* M-GET.RESPONSE */
int	M_SetAux ();		/* M-SET.REQUEST (confirmed/non-conf.) */
int	M_SetRes ();		/* M-SET.RESPONSE */
int	M_ActionAux ();		/* M-ACTION.REQUEST (confirmed/non-conf.) */
int	M_ActionRes ();		/* M-ACTION.RESPONSE */
int	M_Create ();		/* M-CREATE.REQUEST */
int	M_CreateRes ();		/* M-CREATE.RESPONSE */
int	M_Delete ();		/* M-DELETE.REQUEST */
int	M_DeleteRes ();		/* M-DELETE.RESPONSE */
int	M_CancelGet ();		/* M-CANCEL-GET.REQUEST */
int	M_CancelGetRes ();	/* M-CANCEL-GET.RESPONSE */
int	M_WaitReq ();		/* M-WAIT.REQUEST (pseudo) */

int	M_InitialiseReq ();	/* CMIS A-ASSOCIATE.REQUEST */
int	M_Init ();		/* CMIS A-ASSOCIATE.INDICATION */
int	M_InitialiseResp ();	/* CMIS A-ASSOCIATE.RESPONSE */
int	M_TerminateReq ();	/* CMIS A-RELEASE.REQUEST */
int	M_TerminateResp ();	/* CMIS A-RELEASE.RESPONSE */
int	M_AbortReq ();		/* CMIS A-ABORT.REQUEST */


/* higher level management association control */

int	m_initialise ();
int	m_terminate ();


/* Freeing Routines */

void  mc_free ();
void  ms_free ();
void  ma_free ();
void  mid_free ();
void  mn_free ();
void  mparm_free ();
void  ava_free ();
void  mfilterdata_free ();
void  mfilter_free ();
void  mi_free ();


/* Printing Routines */

char* MOperString ();
char* MErrString ();

char* sprintscope ();
char* sprintsync ();

char* sprintmfilter ();
char* sprintmfilteritem ();
char* sprintmfiltersubstrings ();

typedef void (*VoidFunc)();
int	initialiseSyntaxes ();
int	ava2str ();
char*	attrv2str ();


/* Parsing Routines */

CMISScope*  str2scope ();
CMISSync    str2sync ();

CMISFilter*             str2mfilter ();
CMISFilterItem*         str2mfilteritem ();
CMISFilterSubstrings*   str2mfiltersubstrings ();

Ava*    str2ava ();


/* Builds a structure of type External from its arguments */

External* external_build ();    /* builds a structure of type External */


/* various... */

int	oid_equal ();		/* compares two OIDs for equality */
int	pe2mem ();
int	mem2pe ();
int	pe2stream ();
int	stream2pe ();
int	m_initialise ();
int	m_terminate ();


#endif	/* FunctionPrototypes */


#if defined(__cplusplus) || defined(__GNUG__)
}
#endif

#endif
