//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// AVA.cc - Attribute Value Assertion realisation

// By George Pavlou, July 1993


#include <stdlib.h>
#include "msap.h"
#include "AnyType.h"
#include "AVA.h"

AVA*  nullAvaRefAux;
AVA*& nullAvaRef = nullAvaRefAux;

Bool AVA::_createError;


AVA::AVA (char* name, void* value, CMISModifyOp modify, CMISErrors error)
{
    nullify ();
    _modify = modify;
    _error = error;

    if ((_oid = name2oid(_name = name)) == NULLOID) {
	_createError = True;
        fprintf(stderr, "AVA: attribute `%s' not registered in tables!\n",
			name);
	return;
    }

    if (!value)
	return;

    oid_table_attr*  attrEntry = oid2attr(_oid);
    if (!attrEntry->oa_syntax) {
	_createError = True;
        fprintf(stderr, "AVA: no loaded syntax for attribute `%s'!\n", name);
	return;
    }

    _value = new AnyType(attrEntry->oa_syntax, value);
    if (AnyType::createError()) {	// initialisation failed
	_createError = True;
        fprintf(stderr, "AVA: wrong value for attribute `%s'!\n",
			name);
	return;
    }
}


AVA::AVA (char* name, char* value, CMISModifyOp modify, CMISErrors error)
{
    nullify ();
    _modify = modify;
    _error = error;

    if ((_oid = name2oid(_name = name)) == NULLOID) {
	_createError = True;
        fprintf(stderr, "AVA: attribute `%s' not registered in tables!\n",
                        name);
        return;
    }

    if (!value || strlen(value) == 0)
        return;

    oid_table_attr*  attrEntry = oid2attr(_oid);
    if (!attrEntry->oa_syntax) {
	_createError = True;
        fprintf(stderr, "AVA: no loaded syntax for attribute `%s'!\n", name);
        return;
    }

    _value = new AnyType(attrEntry->oa_syntax, value);
    if (AnyType::createError()) {	// initialisation failed
	_createError = True;
        fprintf(stderr, "AVA: wrong value `%s' for attribute `%s'!\n",
			value ? value : "", name);
	return;
    }
}


AVA::AVA (char* name, Attr* value, CMISModifyOp modify, CMISErrors error)
{
    nullify ();
    _modify = modify;
    _error = error;

    if ((_oid = name2oid(_name = name)) == NULLOID) {
	_createError = True;
        fprintf(stderr, "AVA: attribute `%s' not registered in tables!\n",
			name);
	return;
    }
    _value = value;
}


AVA::AVA (CMISErrors error, char* name, Attr* value)
{
    nullify ();
    _error = error;

    if ((_oid = name2oid(_name = name)) == NULLOID) {
	_createError = True;
        fprintf(stderr, "AVA: attribute `%s' not registered in tables!\n",
			name);
	return;
    }
    _value = value;
}


AVA::AVA (OID oid, PE value, CMISErrors error, CMISModifyOp modify)
{
    nullify ();
    _error  = error;
    _modify = modify;

    if ((_name = oid2name(_oid = oid, OIDPART)) == NULLCP) {
        fprintf(stderr, "AVA: attribute `%s' not registered in tables!\n",
			sprintoid(oid));
	_createError = True;
	return;
    }

    if (!value)
	return;

    _value = new AnyType(oid, value);
    if (AnyType::createError()) {
	_createError = True;
	return;		// initialisation failed
    }
}


void  AVA::clear ()
{
    if (_oid) {
	oid_free(_oid);
	_oid = NULLOID;
    }
    if (_value) {
	_value->clear();
	delete _value;
	_value = NULLATTR;
    }
    _name = NULLCP;
    _error = m_noError;
    _modify = m_noModifyOp;
}


char*  AVA::print ()
{
    char  buf [BUFSIZE];	// should be enough
    register char* cp = buf;
    *cp = '\0';

    if (_error)
	sprintf(cp, "error: %s", getErrorStr());
    if (_oid) {
	cp += strlen(cp);
	sprintf(cp, " %s:", getName());
        if (_value) {
	    cp += strlen(cp);
	    char* v;
	    sprintf(cp, " %s", v = _value -> print());
	    free(v);
        }
    }
    if (_modify != m_noModifyOp) {
	cp += strlen(cp);
	sprintf(cp, " %s", sprintmodifyop(_modify));
    }
    return strdup(buf);
}
