//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// AnyType.cc

// By George Pavlou, December 1991


#include <stdlib.h>
#include "msap.h"
#include "AnyType.h"

Bool AnyType::_createError;


int  AnyType::setup (short sntxIndex, PE pe)
{
    if ((_syntax = get_syntax_table(sntxIndex)) == NULL) {
	_syntax = NULL;
	fprintf(stderr, "AnyType::setup: no table entry syntax %d!\n",
			sntxIndex);
	return NOTOK;
    }

    void* val;
    if ((val = _decode(pe)) == NULLVD) {
	_syntax = NULL;
	fprintf(stderr, "AnyType::setup: cannot decode value for syntax %d!\n",
			sntxIndex);
	return NOTOK;

    }

    setval(val);
    setPE(pe);
    return OK;
}


int  AnyType::setup (short sntxIndex, void* attrVal)
{
    if ((_syntax = get_syntax_table(sntxIndex)) == NULL) {
	_syntax = NULL;
	fprintf(stderr, "AnyType::setup: no table entry syntax %d!\n",
			sntxIndex);
	return NOTOK;
    }

    setval(attrVal);

    // check if the value matches the syntax
    PE pe;
    if (attrVal && (pe = encode()) == NULLPE && pe -> pe_errno != PE_ERR_NONE) {
	replval(NULLVD);	// not to free it
	return NOTOK;
    }

    return OK;
}


int  AnyType::setup (short sntxIndex, char* attrVal)
{
    if ((_syntax = get_syntax_table(sntxIndex)) == NULL) {
	_syntax = NULL;
	fprintf(stderr, "AnyType::setup: no table entry syntax %d!\n",
			sntxIndex);
	return NOTOK;
    }

    // the following will also check if the string value matches the syntax
    void* val;
    if (!_syntax->s_parse || (val = _syntax -> s_parse(attrVal)) == NULLVD){
        fprintf(stderr, "AnyType: no parser or wrong value `%s' for syntax \
`%d'!\n",
                        attrVal, sntxIndex);
        return NOTOK;
    }

    setval(val);
    return OK;
}


AnyType::AnyType (char* sntx, void* attrVal)
{
    short index;
    OID   oid;

    _createError = False;

    // check first if the sntx argument is a name
    // e.g. "logId", "pdusSentThld" etc.

    if ((oid = name2oid(sntx)) == NULLOID) {
        // ...or a syntax e.g. "Integer", "GaugeThreshold" etc.

        if (!(index = str2syntax(sntx))) {
            fprintf(stderr, "AnyType: attr or sntx `%s' not registered in \
tables!\n",
				sntx);
	    _createError = True;
            return;
        }
    }
    else {
        oid_table_attr*  attrEntry = oid2attr(oid);
	oid_free(oid);
	index = attrEntry -> oa_syntax;
    }

    if (!index || setup(index, attrVal) == NOTOK) {
	fprintf(stderr, "AnyType: cannot initialise attribute `%s'!\n", sntx);
	_createError = True;
	return;
    }
}


AnyType::AnyType (char* sntx, char* attrVal)
{
    short index;
    OID   oid;

    _createError = False;

    // check first if the sntx argument is a name
    // e.g. "logId", "pdusSentThld" etc.

    if ((oid = name2oid(sntx)) == NULLOID) {
        // ...or a syntax e.g. "Integer", "GaugeThreshold" etc.

        if (!(index = str2syntax(sntx))) {
            fprintf(stderr, "AnyType: attr or sntx `%s' not registered in \
tables!\n",
				sntx);
	    _createError = True;
            return;
        }
    }
    else {
        oid_table_attr*  attrEntry = oid2attr(oid);
	oid_free(oid);
	index = attrEntry -> oa_syntax;
    }

    if (!index || setup(index, attrVal) == NOTOK) {
	fprintf(stderr, "AnyType: cannot initialise attribute `%s' with value \
`%s'!\n",
			sntx, attrVal);
	_createError = True;
	return;
    }
}


AnyType::AnyType (OID attrId, PE attrVal)
{
    oid_table_attr*  attrEntry;

    _createError = False;

    if ((attrEntry = oid2attr(attrId)) == NULLTABLE_ATTR) {
	_createError = True;
	fprintf(stderr, "AnyType: attribute with oid %s not registered in \
tables!\n", sprintoid(attrId));
	return;
    }

    if (setup(attrEntry->oa_syntax, attrVal) == NOTOK) {
	_createError = True;
	fprintf(stderr, "Attr: cannot initialise attribute with oid %s!\n",
			sprintoid(attrId));
	return;
    }
}

