//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// GenericKS.h - generic knowledge source

// By George Pavlou, October 1991


/*
 * The generic (abstract) Knowledge Source class is an abstraction of
 *  a general management application object that can access the coordinator's
 * facilities i.e. register external communication endpoints to receive
 * information and request real time wake-ups.
 */


#ifndef _GENERIC_KS_
#define _GENERIC_KS_


#ifndef _MANIFEST_
#ifndef ICR1
#include <isode/manifest.h>
#else
#include <isode/compat/manifest.h>
#endif
#endif

#ifndef	_BOOLEAN_
#include "Boolean.h"
#endif


class KS		// the generic knowledge source 
{
protected:
    // real time wake-up capabilities

    int		scheduleWakeUp (long, char*, Bool = False);
    int		scheduleWakeUps (long, char*, Bool = False);
    int		cancelWakeUps (char*);

    // listening on external communication endpoints

    int		startListen(int);
    int		stopListen(int);

    // notify process shutdown

    int		notifyShutdown ();

    // protected constructor (abstract class)

    KS ();
    virtual ~KS ();

public:
    // callbacks for wake-ups, external events, process shutdown

    virtual int wakeUp (char*);
    virtual int readCommEndpoint (int);
    virtual int shutdown (int);
};


// inline methods

inline KS::KS ()				{ }
inline KS::~KS ()				{ }
inline int  KS::wakeUp (char*)			{ return NOTOK; }
inline int  KS::readCommEndpoint (int)		{ return NOTOK; }
inline int  KS::shutdown (int)			{ return NOTOK; }

#define NULLKS	((KS*) 0)


#endif
