//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// IsodeCoord.cc - ISODE coordinator realisation

// By George Pavlou, January 1993


#ifdef  __GNUG__
#include <std.h>
#endif
#include <stdlib.h>
#include <signal.h>
#include <errno.h>
#ifndef ICR1
#include <isode/acsap.h>
#include <isode/tsap.h>
#else
#include <isode/ul/acsap.h>
#include <isode/ll/tsap.h>
#endif
#include "IsodeCoord.h"


static const int sigMask = 0x7fffffff;		// all signals
static fd_set ifds;
static TSAPdisconnect tds;

ISODEAgent* ISODECoordinator::_agent;


int  ISODECoordinator::registerAgent (ISODEAgent* agent)
{
    if (_agent) {
	fprintf(stderr, "ISODECoordinator::registerAgent: agent already \
registered!\n");
	return NOTOK;
    }

    _agent = agent;
    return OK;
}


int  ISODECoordinator::readCommEndpoints ()
{
    ifds = _mask;
    switch (iserver_wait(ISODECoordinator::associationInit,
			 ISODECoordinator::associationWork,
			 ISODECoordinator::associationLose,
			 _nfds, &ifds, NULLFD, NULLFD, NOTOK, &tds)) {
    case DONE:
    case NOTOK:
	return NOTOK;

    case OK:
	if (errno == EINTR) {
	    errno = 0;
	    return OK;
	}
	break;
    }

    for (register int fd = 0; fd < _nfds; fd++)
	if (FD_ISSET(fd, &ifds) && _ks[fd] != NULL) {
	    int oldmask = sigblock(sigMask);	// block signals
	    _ks[fd]->readCommEndpoint(fd);	// managed entity data
	    sigsetmask(oldmask);		// restore them
	}

    return OK;
}


int  ISODECoordinator::associationInit (int vecp, char** vec)
{
    int oldmask = sigblock(sigMask);	// block signals
    int sd = _agent->associationRequest(vecp, vec);
    sigsetmask(oldmask);		// restore them

    if (sd != NOTOK)
	((ISODECoordinator*) getInstance()) ->
				commEndpointMaskChange(sd, MASK_ADD);
    return sd;
}


int  ISODECoordinator::associationWork (int sd)
{
    int oldmask = sigblock(sigMask);		// block signals
    int status = _agent->readCommEndpoint(sd);
    sigsetmask(oldmask);			// restore them

    if (status == NOTOK)
	((ISODECoordinator*) getInstance()) ->
				commEndpointMaskChange(sd, MASK_REMOVE);
    return status;
}


int  ISODECoordinator::associationLose (TSAPdisconnect* td)
{
    if (errno == EINTR)
	return OK;

    if (td -> td_cc > 0)
	fprintf(stderr, "ISODECoordinator: TNetAccept: [%s] %*.*s\n",
		TErrString(td -> td_reason),
		td -> td_cc, td -> td_cc, td -> td_data);
    else
	fprintf(stderr, "ISODECoordinator: TNetAccept: [%s]\n",
		TErrString(td -> td_reason));

    return OK;
}
