//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// IsodeCoord.h - a special coordinator for ISODE OSI applications
//                that receive and service association requests

// By George Pavlou, January 1993


/*
 * This is a special coordinator object which is needed by all OSIMIS
 * applications that wish to receive association requests (management
 * agents or hybrid applications). The reason the standard coordinator cannot
 * be used is because ISODE uses a special mechanism to initialise
 * a process listening for associations (iserver_init) and to listen
 * for incoming association requests or traffic on existing associations
 * (iserver_wait) - see ISODE User Manual Volume 1 Chapter 2.
 * These hide the TSAP descriptor so that explicit use of the (x)select
 * facility is not possible.
 *
 * As a consequence of the ISODECoordinator mechanism, it is not possible
 * to have X-window GUIs as parts of such processes but this should not
 * be needed for agents or hybrid units so it's ok.
 */


#ifndef _ISODE_COORD_
#define _ISODE_COORD_


#ifndef _TSAP_
#ifndef	ICR1
#include <isode/tsap.h>
#else
#include <isode/ll/tsap.h>
#endif
#endif

#ifndef _COORDINATOR_
#include "Coordinator.h"
#endif

#ifndef _ISODE_AGENT_
#include "IsodeAgent.h"
#endif


class ISODECoordinator : public Coordinator
{
    static ISODEAgent*	_agent;

    static int	associationInit (int, char**);
    static int	associationWork (int);
    static int	associationLose (TSAPdisconnect*);

protected:
    int		readCommEndpoints ();

public:
    static int	registerAgent (ISODEAgent*);
};


#endif
