//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// XCoord.cc - X coordinator realisation

// By George Pavlou, Xavier Pandolfi (Alcatel ISR) - June 1993


#ifdef  __GNUG__
#include <std.h>
#endif
#include <stdlib.h>
#include <signal.h>
#include "XCoord.h"


XCoordinator::XCoordinator (XtAppContext* appContext)
{
    _lastIntervalId = (XtIntervalId) NULL;
    setAlarmMode(False);	// default: X controls timers
    _appContext = appContext;
}


int  XCoordinator::commEndpointMaskChange (int fd, int mode)
{
    switch (mode) {
    case MASK_ADD:
	_commEndpointId[fd] = XtAppAddInput(*_appContext, fd,
					(XtPointer) XtInputReadMask,
					XCoordinator::XReadCommEndpoint, NULL);
	break;

    case MASK_REMOVE:
	XtRemoveInput(_commEndpointId[fd]);
	break;
    }

    return OK;
}


int  XCoordinator::nextWakeUpIntvlChange (long intvl)
{
    if (_lastIntervalId)
	XtRemoveTimeOut(_lastIntervalId);

    if (intvl == 0) {
	_lastIntervalId = (XtIntervalId) NULL;
	return OK;	// no more OSIMIS alarms (for the moment)
    }

    _lastIntervalId = XtAppAddTimeOut(*_appContext, intvl*1000,
					XCoordinator::XWakeUp, NULL);
    return OK;
}


void  XCoordinator::XReadCommEndpoint (XtPointer, int* fd, XtInputId*)
{
    int oldmask = sigblock(0x7fffffff);	// block signals
    ((XCoordinator*) getInstance())->_ks[*fd]->readCommEndpoint(*fd);
    sigsetmask(oldmask);		// restore them
}


void  XCoordinator::XWakeUp (XtPointer, XtIntervalId*)
{
    Coordinator::serviceAlarm(OK);
}
