// ===================================================
// Example C++ program that uses Tcl/Tk and links with
// coordinator implemented using the Tk coordinator
// 
// Savvas Constantinides [sconstan@cs.ucl.ac.uk]
//
// User types strings at stdin and these appear in the
// text widget displayed
// ===================================================

// NOTE: To build this file I had to hack a lot 
// Mr Ousterhout's main.c for the 'wish' application;
// therefore some things are there without me fully
// understanding what they are there for.
// I preferred to leave them as they are and have something
// working than removing them and have problems.
//
// The important bit is the function call in the
// main part of the program which evaluate the
// script that displays the window
//
// For any questions please contact the 'author' above


/* -------------------- */
/* INCLUDES AND DEFINES */
/* -------------------- */

extern "C" 
{
#include "tkConfig.h"
#include "tkInt.h"
}

#include "TkCoord.h"
#include "TkKboardMgr.h"

// ----------------
// GLOBAL VARIABLES
// ----------------

static Tk_Window w;
       Tcl_Interp *interp; // Interpreter for this application.
static Tcl_CmdBuf buffer;  // Used to assemble lines of terminal input into Tcl commands.

int  synchronize = 0;
char *fileName = NULL;
char *name = NULL;
char *display = NULL;
char *geometry = NULL;

char *frame = ".frm2"; // text widget where input is displayed


// instantiate the tk coordinator and the associated keyboard manager
// they cannot be instantiated in the main program because it contains
// labels (goto's) and ATT C++ cannot handle destructors that way
TkCoordinator coordinator;
TkKeyboardMgr keyboardMgr;


// -------------------
// FUNCTION PROTOTYPES
// -------------------

static void DelayedMap _ANSI_ARGS_((ClientData clientData));
static void StructureProc _ANSI_ARGS_((ClientData clientData,
					XEvent *eventPtr));
void InitTclTk ();


// --------------------
// MAIN PART OF PROGRAM
// --------------------

int main(int, char**)
{
    char *msg;
    int result;

    // Initialise Tcl & Tk
    InitTclTk();

    // tell the coordinator to catch the terminate signal
    // (the tcl/tk coordination mechanism will handle alarms)
    TkCoordinator::setTerminateSignalHandler ();

    // initialise the keyboard manager knowledge source
    keyboardMgr.tkInit(interp, frame);

    // execute sample script 
    //
    result = Tcl_Eval(interp, "source TkExample.tcl", 0, (char **) NULL);
    if (result != TCL_OK)
	goto error;

    fflush(stdout);
    buffer = Tcl_CreateCmdBuf();
    (void) Tcl_Eval(interp, "update", 0, (char **) NULL);

    // Loop infinitely, waiting for commands to execute.  When there
    // are no windows left, Tk_MainLoop returns and we clean up and
    // exit.
    Tk_MainLoop();
    Tcl_DeleteInterp(interp);
    Tcl_DeleteCmdBuf(buffer);
    exit(0);

error:
    msg = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
    if (msg == NULL)  msg = interp->result; 
    fprintf(stderr, "%s\n", msg);
    Tcl_Eval(interp, "destroy .", 0, (char **) NULL);
    exit(1);
    return 0;
}


// *************************************************
// All the functions below I did not touch except
// for the InitTclTk which I wrote to group together
// everything whose purpose I did not understand
// *************************************************

/*
 *----------------------------------------------------------------------
 *
 * StructureProc --
 *
 *	This procedure is invoked whenever a structure-related event
 *	occurs on the main window.  If the window is deleted, the
 *	procedure modifies "w" to record that fact.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Variable "w" may get set to NULL.
 *
 *----------------------------------------------------------------------
 */

static void StructureProc(ClientData, XEvent *eventPtr)
{
    if (eventPtr->type == DestroyNotify) {
	w = NULL;
    }
}


/*
 *----------------------------------------------------------------------
 *
 * DelayedMap --
 *
 *	This procedure is invoked by the event dispatcher once the
 *	startup script has been processed.  It waits for all other
 *	pending idle handlers to be processed (so that all the
 *	geometry information will be correct), then maps the
 *	application's main window.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	The main window gets mapped.
 *
 *----------------------------------------------------------------------
 */

static void DelayedMap(ClientData)
{

    while (Tk_DoOneEvent(TK_IDLE_EVENTS) != 0) {
	/* Empty loop body. */
    }
    if (w == NULL) {
	return;
    }
    Tk_MapWindow(w);
}


/* 
 * Function that initialises the Tcl and Tk environment.
 * The contents of this function are exactly as were
 * found in the wish.c file 
 */

void InitTclTk ()
{
    char *p;
    Tk_3DBorder border;

    interp = Tcl_CreateInterp();

#ifdef TCL_MEM_DEBUG
    Tcl_InitMemory(interp);
#endif

    if (name == NULL)
    { if (fileName != NULL)
        p = fileName;
      else
        p = "BROWSER";

      name = strrchr(p, '/');

      if (name != NULL)
        name++;
      else
        name = p;
    }

    /*
     * Initialize the Tk application and arrange to map the main window
     * after the startup script has been executed, if any.  This way
     * the script can withdraw the window so it isn't ever mapped
     * at all.
     */

    w = Tk_CreateMainWindow(interp, display, name);
    if (w == NULL)
    { fprintf(stderr, "%s\n", interp->result);
      exit(1);
    }

    Tk_SetClass(w, "Tk");
    Tk_CreateEventHandler(w, StructureNotifyMask, StructureProc, (ClientData) NULL);
    Tk_DoWhenIdle(DelayedMap, (ClientData) NULL);

    if (synchronize)
    {  XSynchronize(Tk_Display(w), True);
    }

    Tk_GeometryRequest(w, 200, 200);
    border = Tk_Get3DBorder(interp, w, None, "#ffe4c4");

    if (border == NULL)
    { Tcl_SetResult(interp, (char *) NULL, TCL_STATIC);
      Tk_SetWindowBackground(w, WhitePixelOfScreen(Tk_Screen(w)));
    }
    else
    { Tk_SetBackgroundFromBorder(w, border);
    }

    XSetForeground(Tk_Display(w), DefaultGCOfScreen(Tk_Screen(w)),
                   BlackPixelOfScreen(Tk_Screen(w)));
}
