/* Copyright (c) 1994 Institute of Computer Science, Foundation of      */
/* Research and Technology - Hellas (ICS-FORTH)                         */ 
/* All rights reserved.                                                 */      

/* Redistribution and use in source and binary forms are permitted      */
/* provided that the above copyright notice and this paragraph are      */
/* duplicated in all such forms and that any documentation, advertising */
/* materials, and other materials related to such distribution and use  */
/* acknowledge that the software was developed by the Institute of      */
/* Computer Science, Foundation of Research and Technology - Hellas.    */
/* The name of the Institute or the Foundation may not be used to       */
/* endorse or promote products derived from this software without       */
/* specific prior written permission.                                   */
/* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR       */
/* IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED       */
/* WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.  */

/* read an entry from the Directory (get presentation address	*/
/* and operationalStatus					*/

/* Costas Stathopoulos July 1993	*/

#include "isode/quipu/util.h"
#include "isode/quipu/common.h"
#include "isode/quipu/entry.h"
#ifndef ICR1
#include "isode/tailor.h"
#else
#include "isode/compat/tailor.h"
#endif


#ifndef _LOCATION_TRANSPARENCY_
#include "lt.h"
#endif

int     as_print();

struct  ds_read_result readresult;
char    lt_oper_status;

char *  lt_now_read();

extern char *	lt_bind();
extern int	lt_init();

extern char *	local_nsap;

/* lt_read is used by a management process in order to identify the
   presentation address where another management process listens on */

struct PSAPaddr *lt_read(agentDN)
char *agentDN;	/* a character string format of the DN of the listening management process */
{
  char  * bind_mess,
        * read_mess;
  char  * pa,
        * pa_tmp,
        * oper_str;
  int	pa_found = 0;

  char  globalName[STRINGLEN];

  PS    ps;
  register char *pstr;

#ifdef PERF_TIME

  struct timeval before_bind, before_read, after_read;
  int seconds, micros;
  struct timezone tz;

#endif

  strcpy ( globalName, agentDN );

  lt_oper_status = DOWN;

  if ( lt_init() != 0 ) return(NULLPA);

#ifdef PERF_TIME

  gettimeofday( &before_bind, &tz );

#endif

  if ( (bind_mess = lt_bind()) != NULLCP ) {
     fprintf ( stderr, "lt_bind: %s\n", bind_mess );
     return(NULLPA);
  }

#ifdef PERF_TIME

  gettimeofday( &before_read, &tz );

#endif

  if ( (read_mess = lt_now_read(str2dn(globalName))) != NULLCP ) {
    fprintf ( stderr, "lt_read: %s\n", read_mess );
    return(NULLPA);
  }

  /* Get a string representation of the AVAs you've just read */

  if ((ps = ps_alloc(str_open)) == NULLPS) return(NULLPA);
  if (str_setup(ps, NULLCP, 0, 0) == NOTOK) return(NULLPA);

  as_print(ps, (caddr_t) readresult.rdr_entry.ent_attr, EDBOUT);

  *--ps->ps_ptr = NULL, ps->ps_cnt++;

  pstr = ps->ps_base;

  ps->ps_base = NULL;
  ps->ps_cnt = 0;
  ps->ps_ptr = NULL;
  ps->ps_bufsiz = 0;

  ps_free(ps);

  pa = (char *)malloc(strlen(pstr));
  pa_tmp = (char *)malloc(strlen(pstr));
  oper_str = (char *)malloc(strlen(pstr));

  /* get the values of the two attributes from the pstr */

  fprintf(stderr, "%s\n",pstr);

  if ( sscanf(pstr, "operationalStatus= %s", oper_str) != 1 ) {
    fprintf(stderr, "No operationalStatus value  returned from the Directory!");
    return(NULLPA);
  }
  lt_oper_status = ( strcmp(oper_str, UPSTR) == 0 )? UP : DOWN;
  pstr = index (pstr, '\n');
  pstr++;

  /* get the presentation addresses; append the
     local_nsap in case we have a nsap community.  */

  if ( sscanf(pstr, "presentationAddress= %s", pa) != 1 ) {
    fprintf(stderr, "No presentation addresses returned from the Directory!");
    return(NULLPA);
  }

  /* special hack for future use when CLNS is going to be supported, too 
     Note that if present the CLNS presentation address will be first	*/
  if ( str2paddr(pa) ->pa_addr.sa_addr.ta_addrs -> na_stack == NA_NSAP )
    sprintf(pa, "%s%s", pa, local_nsap);

  /* break if you get a presentation address in a stack that we support */
  switch (str2paddr(pa) ->pa_addr.sa_addr.ta_addrs -> na_stack) {
        case NA_TCP: if (ts_stacks & TS_TCP) {
                        pa_found ++;
                        break;
                     }
        case NA_X25: if (ts_stacks & TS_X25) {
                        pa_found ++;
                        break;
                     }
            default: break;
  }
  if ( (pstr = index (pstr, '\n')) != NULL )
    pstr++;

  while ( (pstr != NULL) && (!pa_found) && 
    	  (sscanf(pstr, "presentationAddress= %s", pa_tmp) == 1) ) {
    strcpy(pa, pa_tmp);
    /* break if you get a presentation address in a stack that we support */
    switch (str2paddr(pa_tmp) ->pa_addr.sa_addr.ta_addrs -> na_stack) {
	case NA_TCP: if (ts_stacks & TS_TCP) {
			pa_found ++;
		      	break;
		     }
	case NA_X25: if (ts_stacks & TS_X25) {
			pa_found ++;
		      	break;
		     }
	    default: break;
    }
    if ( (pstr = index (pstr, '\n')) != NULL )
      pstr++;
  }

#ifdef PERF_TIME

  gettimeofday( &after_read, &tz );
  seconds = (before_bind.tv_usec > after_read.tv_usec ? after_read.tv_sec - befo
re_bind.tv_sec - 1: after_read.tv_sec - before_bind.tv_sec);
  micros = (before_bind.tv_usec > after_read.tv_usec ? after_read.tv_usec + 1000
000 - before_bind.tv_usec : after_read.tv_usec - before_bind.tv_usec);
  fprintf(stderr, " Total time spent on a read (including bind): %ds.%dms\n", se
conds, micros/1000);

#endif

  ds_unbind();

  if ( (pa == NULL) || (!pa_found) ) 
    return (NULLPA); 
  else return(str2paddr(pa));
}


char * lt_now_read(Name)
DN      Name;
{
  struct ds_read_arg    readarg;
  struct DSError        readerr;

  if (get_default_service (&readarg.rda_common) != 0)
        return("Can't get default service controls!\n");
  readarg.rda_common.ca_servicecontrol.svc_options = SVC_OPT_DONTUSECOPY;
  readarg.rda_common.ca_servicecontrol.svc_prio = SVC_PRIO_HIGH;
  readarg.rda_common.ca_servicecontrol.svc_timelimit = SVC_NOTIMELIMIT;
  readarg.rda_common.ca_servicecontrol.svc_sizelimit = SVC_NOSIZELIMIT;
  readarg.rda_common.ca_servicecontrol.svc_scopeofreferral = SVC_REFSCOPE_DMD;

  readarg.rda_object = Name;

  readarg.rda_eis.eis_allattributes = FALSE;

  readarg.rda_eis.eis_select = as_comp_new( str2AttrT(PRES_ADD), NULLAV, NULLACL_INFO);
  readarg.rda_eis.eis_select->attr_link = as_comp_new( str2AttrT(OPER_STATUS), NULLAV, NULLACL_INFO);
  readarg.rda_eis.eis_select->attr_link->attr_link = NULLATTR;

  if (ds_read (&readarg, &readerr, &readresult) != DS_OK) {
    switch (readerr.dse_type) {
    case DSE_LOCALERROR:
      return(duaerror);

    case DSE_REMOTEERROR:
      return(localdsaerror);

    case DSE_ATTRIBUTEERROR:
      return(attributerror);

    case DSE_REFERRAL:
    case DSE_DSAREFERRAL:
      return(remotedsaerror);

    case DSE_SECURITYERROR:
      return(security);

    case DSE_NAMEERROR:
      return(namerror);

    case DSE_SERVICEERROR:
      return(serviceerror);

    default:
      return(localdsaerror);
    }
  } else {
    /* use data */
    if ( readresult.rdr_entry.ent_attr == NULLATTR)
      return(nothingfound);
  }

  return(NULL);
}
