/* Copyright (c) 1994 Institute of Computer Science, Foundation of      */
/* Research and Technology - Hellas (ICS-FORTH)                         */ 
/* All rights reserved.                                                 */      

/* Redistribution and use in source and binary forms are permitted      */
/* provided that the above copyright notice and this paragraph are      */
/* duplicated in all such forms and that any documentation, advertising */
/* materials, and other materials related to such distribution and use  */
/* acknowledge that the software was developed by the Institute of      */
/* Computer Science, Foundation of Research and Technology - Hellas.    */
/* The name of the Institute or the Foundation may not be used to       */
/* endorse or promote products derived from this software without       */
/* specific prior written permission.                                   */
/* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR       */
/* IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED       */
/* WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.  */

/* implementation of the search operation in the Directory 	*/

/* Costas Stathopoulos June 1994 */

#include "isode/quipu/util.h"
#include "isode/quipu/common.h"
#include "isode/quipu/entry.h"
#ifndef ICR1
#include "isode/tailor.h"
#else
#include "isode/compat/tailor.h"
#endif


#ifndef _LOCATION_TRANSPARENCY_
#include "lt.h"
#endif

char * lt_search();

extern short    oc_sntx;
extern char	lt_local_DIT[STRINGLEN];

extern char * lt_bind();
extern int lt_init();

DNlist  mplist;

DNlist lt_get_mp(basename)
char * basename;
{
  char * bind_mess,
       * sea_mess;

  if ( lt_init() != 0 ) return(NULL);

  if ( (bind_mess = lt_bind()) != NULLCP ) {
     fprintf ( stderr, "%s\n", bind_mess );
     return(NULL);
  }

  if ( (sea_mess = lt_search(basename)) != NULLCP ) {
    fprintf ( stderr, "%s\n", sea_mess );
    return(NULL);
  }

  return(mplist);
}

char * lt_search(basename)
char * basename;
{
  struct ds_search_arg search_arg;
  struct ds_search_result result;
  struct DSError          error;
  AVA   mp;

  if (get_default_service (&search_arg.sra_common) != 0)
        return("Can't get default service controls!\n");

  search_arg.sra_common.ca_servicecontrol.svc_options = SVC_OPT_DONTUSECOPY;
  search_arg.sra_common.ca_servicecontrol.svc_prio = SVC_PRIO_HIGH;
  search_arg.sra_common.ca_servicecontrol.svc_timelimit = SVC_NOTIMELIMIT;
  search_arg.sra_common.ca_servicecontrol.svc_sizelimit = SVC_NOSIZELIMIT;
  search_arg.sra_common.ca_servicecontrol.svc_scopeofreferral = SVC_REFSCOPE_DMD
;

  if ( basename == NULL )
    search_arg.sra_baseobject = str2dn(lt_local_DIT);
  else
    search_arg.sra_baseobject = str2dn(basename);

  search_arg.sra_subset = SRA_WHOLESUBTREE;

  search_arg.sra_eis.eis_allattributes = FALSE;
  search_arg.sra_eis.eis_select = as_comp_new( str2AttrT(OPER_STATUS), NULLAV, NULLACL_INFO);
  search_arg.sra_eis.eis_select->attr_link = NULLATTR;
  search_arg.sra_eis.eis_infotypes = EIS_ATTRIBUTESANDVALUES;

  search_arg.sra_searchaliases = TRUE;

  search_arg.sra_filter = filter_alloc();
  search_arg.sra_filter->flt_type = FILTER_ITEM;
  search_arg.sra_filter->flt_un.flt_un_item.fi_type = FILTERITEM_EQUALITY;
  search_arg.sra_filter->flt_un.flt_un_item.fi_un.fi_un_ava.ava_type =
        str2AttrT(DSCLASS);
  search_arg.sra_filter->flt_un.flt_un_item.fi_un.fi_un_ava.ava_value =
        str2AttrV(ICM_AGENT_DSCLASS, oc_sntx);
  search_arg.sra_filter->flt_next = NULLFILTER;

  if(ds_search (&search_arg, &error, &result) != DS_OK) {
    mplist = NULL;
    fprintf(stderr,"error.dse_type = %d\n",error.dse_type);
    switch (error.dse_type) {
    case DSE_LOCALERROR:
      return(duaerror);

    case DSE_REMOTEERROR:
      return(localdsaerror);

    case DSE_ATTRIBUTEERROR:
      return(attributerror);

    case DSE_REFERRAL:
    case DSE_DSAREFERRAL:
      return(remotedsaerror);

    case DSE_SECURITYERROR:
      return(security);

    case DSE_NAMEERROR:
      return(namerror);

    case DSE_SERVICEERROR:
      return(serviceerror);

    default:
      return(localdsaerror);
    }
  } else {
    if (result.srr_correlated != TRUE)
      correlate_search_results (&result);
    if (result.CSR_entries != NULLENTRYINFO) {
      register EntryInfo *ptr;
      register DNlist    mpptr = (DNlist)malloc (sizeof (struct dnlist *));
      int	found = 0;

      for (ptr = result.CSR_entries,
           mpptr->next = mplist = (DNlist)malloc (sizeof (struct dnlist *));
           ptr != NULLENTRYINFO;
           ptr = ptr->ent_next) {
        PS      ps;
        char *  oper_str;
        register char *pstr;
        register char *buf;

	oper_str = strdup(DOWNSTR);
        if ((ps = ps_alloc(str_open)) == NULLPS) return(NULL);
        if (str_setup(ps, NULLCP, 0, 0) == NOTOK) return(NULL);

        dn_print(ps, (caddr_t) ptr->ent_dn, EDBOUT);

        *ps->ps_ptr = NULL, ps->ps_cnt++;

        pstr = ps->ps_base;

        ps->ps_base = NULL;
        ps->ps_cnt = 0;
        ps->ps_ptr = NULL;
        ps->ps_bufsiz = 0;

        ps_free(ps);

        buf = (char *)malloc(strlen(pstr) + 1);
	/* add the "@" in front of the DN */
        buf = sprintf(buf, "@%s", pstr);

        /* Get a string representation of the AVA you've just read */
	if ((ps = ps_alloc(str_open)) != NULLPS)
	  if (str_setup(ps, NULLCP, 0, 0) != NOTOK) {

	    as_print(ps, (caddr_t) ptr->ent_attr, EDBOUT);

	    *--ps->ps_ptr = NULL, ps->ps_cnt++;

	    pstr = ps->ps_base;

	    ps->ps_base = NULL;
	    ps->ps_cnt = 0;
    	    ps->ps_ptr = NULL;
	    ps->ps_bufsiz = 0;

	    ps_free(ps);

	    oper_str = (char *)malloc(strlen(pstr));

	    /* get the values of the operational Status */

	    if ( sscanf(pstr, "operationalStatus= %s", oper_str) != 1 )
  	      fprintf(stderr, "lt_search: No operationalStatus value  returned from the Directory!");
          }

	/* add to the list the running processes */
	
	if ( strcmp (oper_str, UPSTR) == 0 ) {
          found++;
          mpptr->next->name = buf;
          if (ptr->ent_next != NULLENTRYINFO) {
            mpptr->next->next = (DNlist)malloc (sizeof (struct dnlist *));
            mpptr = mpptr->next;
          } else
            mpptr->next->next = NULL;
	} else
	  if (ptr->ent_next == NULLENTRYINFO)
	    mpptr->next = NULL;
      }
    ds_unbind();
    if (!found) mplist = NULL;
    } else
    mplist = NULL;
  }
  return(NULL);
}

 
