//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//


/******************************************************************/
/*
	File:	cmisclient.h

	Notes:	CMIS Client Class

	Author:
		James Cowan (J.Cowan@uk.ac.ucl.cs)
*/
/******************************************************************/

class	Wait_queue : public Ptrlist
{
public:
	Wait_queue()	{}	// Stop C++ Warning!!

	~Wait_queue()
        {
		while (vdelete(0))
			;
        }

	int	vdelete(int index)
	{
        struct  MSAPIndication *mi;

		if (mi=(struct  MSAPIndication *)get(index))
		{
			mi_free(mi);
			delete mi;
			remove(index);
			return(TRUE);
		}
		return(FALSE);
	}
};

/******************************************************************/

class	Cmis_client	
{
	struct		MSAPConnect	_ms_connection;
	struct  	MSAPIndication  _ms_indication;
	Wait_queue	_wait_queue;
	int		_invoke_count;

	Boolean		convert_ok(int);
	Boolean		convert_result(Result);

	void		free_msap_indication(struct MSAPIndication *);
	int		get_invoke_count() {return (_invoke_count++);}

	Result		wait_request(int, Cmis_client_result *);
	void		process_error(char *, MSAPIndication *);

	void		encode_class_name(char *class_name, MIDentifier *oid)
	{
		oid->mid_type = MID_GLOBAL;
		if (class_name && strlen(class_name))
			oid->mid_global = name2oid(class_name);
		else
			oid->mid_global = NULLOID;
	}
	void		encode_instance(char *instance_name, MName *instance)
	{
		if (instance_name && strlen(instance_name))
		{
		GString	copy_instance = instance_name;

			instance->mn_dn = str2dn((char *) copy_instance);
		}
		else
		{
			instance->mn_dn = dn_comp_new(NULLRDN);
		}
		instance->mn_type = MN_LOCALDN;
	}
public:

	Cmis_client()
	{
		_ms_connection.mc_sd = 0;
		_invoke_count = 1;
	}

	~Cmis_client()
	{
		if (_ms_connection.mc_sd)
			mc_free(&_ms_connection);
	}

	/* Initialisation */

	Boolean	initialise(char *, char *);

	Boolean	initialise(AEI, struct PSAPaddr *, AEI, struct PSAPaddr *,
			   External *, External *);

	/* get socket connection fd */

	int	get_connection()	{return(_ms_connection.mc_sd);}
	
	/* Termination */

	Boolean	terminate(PE);
	Boolean	terminate();
	Boolean	urgent_terminate(PE);
	Boolean	urgent_terminate();
	Boolean	abort(External *);
	Boolean	abort();

	Boolean	create(MID, MN, int, MN, External *, int, CMISParam *);
	Boolean	remove(MID, MN, CMISScope *, CMISFilter *, External *,
		       CMISSync);

	Boolean	get(MID, MN, CMISScope *, CMISFilter *, External *,
		    CMISSync, int, MID, Cmis_result *);
	Boolean	get(char *, char *, int, int, Cmis_result *);

	Boolean	set(MID, MN, CMISScope *, CMISFilter *, External *,
		    CMISSync, int, CMISSetAttr *, Cmis_result *);
	Boolean	set(char *, char *, char *, char *, Cmis_result *);
	Boolean	set(char *, char *, char *, Cmis_result *);

	Boolean	event_report(int, MID, MN, MID, char *, PE);

	Boolean	action(MID, MN, CMISScope *, CMISFilter *, External *,
		       CMISSync, MID, PE);
};

/******************************************************************/
/* End of header file */
/******************************************************************/

