//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
/***********************************************************/
/* 
	File:	ivmonitor.h

	Notes:	Cmis Interviews monitor class header file

	Author:
		James Cowan (J.Cowan@uk.ac.ucl.cs)
*/
/***********************************************************/

#define XCALL_MINTERVAL 1
#define XCALL_MQUIT     2

/***********************************************************/
/* Monitor window class */
/***********************************************************/

class	Monitor_Window : public Menu_Window
{
	GString		_class_name;
	GString		_instance;
	Browser_Window	*_browser;
	Msg_line	*_msg_line;
	MenuBar		*_menu;
	StringBrowser	*_attribute_browser;
        StringDialog    *_interval_editor;
	Interactor	*_this_interactor;
	Attr_Display    _attr_display;
	long		_next_alarm_time;
	int		_alarm_interval;

	Interactor	*interior(int, int);
	Interactor	*attributes_display(int, int);
	Interactor	*commands();
	Boolean		check_interval(char *, char *);
public:
	Monitor_Window(char *class_name, char *instance, 
		       int rows, int cols, Browser_Window *browser)
	{
		_class_name = class_name;
		_instance = instance;
		_browser = browser;

		_this_interactor = interior(rows, cols);
                _interval_editor = (NEW StringDialog("Change Polling Interval",
						     round(3*inches)));
		Insert(_this_interactor);
		refresh();
		set_signal_alarm(10);
	}

	~Monitor_Window()
	{
		delete _interval_editor;
		GetWorld()->Remove(_this_interactor);
	}
	void    Xcallback(int);
	void	quit();
	void	interval();
	void	refresh();
	long	refresh(long cur_time)
	{
		if (cur_time < _next_alarm_time)
		{
			;
//	printf("\nNo refresh %s", (char *) _instance);
		}
		else
		{
			refresh();
/*
			// this makes all monitors use the same alarm

			_next_alarm_time = cur_time + _alarm_interval;
			
*/
			while (cur_time >= _next_alarm_time)
				_next_alarm_time += _alarm_interval;
/*
printf("\nRefreshed %s next %ld cur %ld interval %d",
	(char *) _instance, _next_alarm_time,  cur_time,  _alarm_interval);
*/
		}

		return(_next_alarm_time);		
	}
	void	set_signal_alarm(int);
	long	get_next_alarm()	{return(_next_alarm_time);}
};

/***********************************************************/
/* Monitor list */
/***********************************************************/

class   Monitor_list : public Ptrlist
{
public:
        Monitor_list()	{}
        ~Monitor_list()
        {
		delete_all();
        }

        int     vdelete(int index)
        {
        Monitor_Window *monitor;

                if (monitor=(Monitor_Window *) get(index))
                {
                        delete (Monitor_Window *) monitor;
                        remove(index);
                        return(TRUE);
                }
                return(FALSE);
        }
        void    delete_monitor(Monitor_Window *target_monitor)
        {
        int     index;

                if ((index=find(target_monitor)) >= 0)
                        vdelete(index);
        }

	void	delete_all()
	{
                while (vdelete(0))
                        ;
	}
};

/***********************************************************/
/* End */
/***********************************************************/


