//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
/***********************************************************/
/*
        File:   ivutils.cc

        Notes:

                Interviews utilities methods

		cobbled together from iclass.c example

	Author:
		James Cowan (J.Cowan@uk.ac.ucl.cs)
*/
/***********************************************************/
/* Menus */
/***********************************************************/

/* Menu Window */

MenuBar *Menu_Window::menu_initialise(struct Menu_Options mo[])
{
MenuBar	*menu = (NEW MenuBar());
int	count;

	for (count=0; ; count++)
	{
	struct	 Menu_Options *opt = &mo[count];

		if (!opt->text)
			break;

		if (opt->pd)
			menu->Include(makepulldown(opt));
		else
		{
		Menu_Cmd *menu_cmd; 
			
			menu_cmd = (NEW Menu_Cmd(opt->text, Left,
						this, opt->option));
			menu->Include(menu_cmd);
		}
		}
        return(menu);
	}

PulldownMenu	*Menu_Window::makepulldown(struct Menu_Options *mo)
{
PulldownMenu* menu = (NEW PulldownMenu(
	new Message(mo->text, Left, 2, hfil, 0)
//		new Message(mo->text, Left)
	));
int	count;

	for (count=0; ; count++)
	{
	struct PullDownOptions *pd = &(mo->pd[count]);

		if (!pd->text)
			break;

		menu->Include((NEW Menu_Cmd(pd->text, Left,
						   this, pd->option)));
	}
	return menu;
};

/***********************************************************/
/* Dialog methods */
/***********************************************************/

static const float fspace = .375;                // space in cm

/*****************************************************************************/

BasicDialog::BasicDialog () : Dialog(new ButtonState, nil) {
    input = new Sensor;
    input->Catch(KeyEvent);
    input->Reference();
}

boolean BasicDialog::Accept () {
    Event e;
    int v = 0;

    state->SetValue(0);

    do {
	Read(e);
        if (!KeyEquiv(e)) {
            Forward(e);
        }
        state->GetValue(v);
    } while (v == 0);

    return v == '\r';
}

void BasicDialog::Forward (Event& e) {
    Coord x = e.x, y = e.y;

    e.target->GetRelative(x, y, this);
    if (x >= 0 && y >= 0 && x <= xmax && y <= ymax) {
        e.target->Handle(e);
    }
}

boolean BasicDialog::KeyEquiv (Event& e) {
    boolean keyEquiv = false;

    if (e.eventType == KeyEvent && e.len > 0) {
        char c = e.keystring[0];

        if (c == '\r' || c == '\007' || c == '\033') {
            state->SetValue(c);
            keyEquiv = true;
        }
    }
    return keyEquiv;
}

/*****************************************************************************/
/*
AcknowledgeDialog::AcknowledgeDialog (const char* msg, const char* btnLbl) {
    int space = round(fspace*cm);

    Insert(
        new MarginFrame(
            new VBox(
                new Message(msg),
                new VGlue(space, space, 0),
                new HBox(
                    new HGlue,
                    new PushButton(btnLbl, state, '\r'),
                    new HGlue
                )
            ), space, space, 0
        )
    );
}

void AcknowledgeDialog::Acknowledge () {
    Event e;
    int v = 0;

    state->SetValue(v);

    do {
	Read(e);
        if (!KeyEquiv(e)) {
            Forward(e);
        }
	state->GetValue(v);
    } while (v == 0);
}
*/
/*****************************************************************************/
/*
ConfirmDialog::ConfirmDialog (const char* msg, const char* confirmLbl) {
    int space = round(fspace*cm);

    Insert(
        new MarginFrame(
            new VBox(
                new Message(msg),
                new VGlue(space, space, 0),
                new HBox(
                    new HGlue,
                    new PushButton(confirmLbl, state, '\r'),
                    new HGlue(space, space, 0),
                    new PushButton(" Cancel ", state, '\007'),
                    new HGlue
                )
            ), space, space, 0
        )
    );
}
*/
/*****************************************************************************/

StringDialog::StringDialog (
    const char* msg, const char* sample, const char* confirmLbl
) {
    Init(msg, confirmLbl);
    _sedit->Message(sample);
}

StringDialog::StringDialog (
    const char* msg, int width, const char* confirmLbl
) {
    Init(msg, confirmLbl, width);
}

void StringDialog::Select () {
    _sedit->Select(0, strlen(_sedit->Text()));
}

void StringDialog::Select (int pos) { _sedit->Select(pos); }
void StringDialog::Select (int left, int right) { _sedit->Select(left, right);}

void StringDialog::Init (const char* msg, const char* confirmLbl, int width) {
    int space = round(fspace*cm);
    _sedit = new StringEditor(state, "");
    HBox* framedSedit = new HBox;

    if (width == 0) {
        framedSedit->Insert(_sedit);

    } else {
        framedSedit->Insert(new HGlue);
        framedSedit->Insert(
            new VBox(
                new Frame(new MarginFrame(_sedit, 2)),
                new HGlue(width, 0, 0)
            )
        );
        framedSedit->Insert(new HGlue);
    }

    Insert(
        new MarginFrame(
            new VBox(
                new Message(msg),
                new VGlue(space, space, 0),
                framedSedit,
                new VGlue(space, space, 0),
                new HBox(
                    new HGlue,
                    new PushButton(confirmLbl, state, '\r'),
                    new HGlue(space, space, 0),
                    new PushButton(" Cancel ", state, '\007')
                )
            ), space, space, 0
        )
    );
}

const char* StringDialog::Get_String () { return _sedit->Text(); }

boolean StringDialog::Accept () {
    Event e;
    int v = 0;

    state->SetValue(0);
    Select();
    _sedit->Edit();

    for (;;) {
        state->GetValue(v);
        if (v != 0) {
            break;
        }
	Read(e);
        if (!KeyEquiv(e)) {
            Forward(e);
        }
    }

    return v == '\r';
}

/*****************************************************************************/

void StringDialog::Read_Input(Menu_Window *mw, Menu_Valid_Fn fn,
			      char *title_msg, char *value)
{
Frame*  framedDialog = new Frame(this, 2);
Coord   x, y;
GString	save_title = title_msg;	// take copy because of obscure bug

	mw->Align(Center, 0, 0, x, y);
	mw->GetRelative(x, y, mw->GetWorld());
	mw->GetWorld()->InsertTransient(framedDialog, mw, x, y, Center);
	framedDialog->SetName(title_msg);
/*
printf("\nTitle_msg %s Address %ld",
	title_msg, title_msg);
*/
	Change_Message(value);
	while (Accept())
	{
/*
printf("\nTitle_msg %s Address %ld Window %ld",
	title_msg, title_msg, mw);
*/
		if ((mw->*fn)((char *)save_title, Get_String()))
			break;
	}

	mw->GetWorld()->Remove(framedDialog);
	framedDialog->Remove(this);
	delete framedDialog;

} /* Read_Input */

/*****************************************************************************/
/* End */
/*****************************************************************************/

