//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
/***********************************************************/
/* 
	File:	ivutils.h

	Notes:	Cmis Interviews utilities classes

	Author:
		James Cowan (J.Cowan@uk.ac.ucl.cs)
*/
/***********************************************************/
/* Message line class */
/***********************************************************/

class	Msg_line : public Banner
{
public:
	Msg_line() :Banner("", "", "")
	{
	}

	void	update(char *msg)
	{
		left = msg;
		Update();
		Flush();
	}
	void	error_msg(char *msg)
	{
		bell();
		update(msg);
		sleep(1);
	}
};
/***********************************************************/
/* Menus */
/***********************************************************/

class	Menu_Window;
class	Menu_Cmd;

typedef Boolean	(Menu_Window::*Menu_Valid_Fn)(char *, const char *);

struct	PullDownOptions
{
	const char	*text;
	int	 	option;
};

struct Menu_Options
{
	const char	*text;
	int	 	option;
	struct	PullDownOptions	*pd;
};

/***********************************************************/
/* Menu Window */

class	Menu_Window : public MonoScene
{
/*
	Dummy class that enables all objects to use same 
	menu classes
*/
protected:
	MenuBar *menu_initialise(struct Menu_Options mo[]);

	PulldownMenu	*makepulldown(struct Menu_Options *mo);
public:
	virtual	void	Xcallback(int)	{}
};

/***********************************************************/
/* Menu Command class */

class	Menu_Cmd : public MenuItem
{
	Menu_Window*	_window;
	int		_option;

public:
	Menu_Cmd(const char* text, Alignment al, Menu_Window* mw,
		 int option) :MenuItem (text, al)
	{
        	_window = mw;
        	_option = option;
	}

	void Do()
	{
        	_window->Xcallback(_option);
	}
};
/***********************************************************/
/* Dialog class */
/***********************************************************/

class BasicDialog : public Dialog 
{
public:
    BasicDialog();

    virtual boolean Accept();
protected:
    void Forward(Event&);
    boolean KeyEquiv(Event&);
};
/*
class AcknowledgeDialog : public BasicDialog 
{
public:
    AcknowledgeDialog(const char* msg, const char* btnLbl = "   OK   ");

    virtual void Acknowledge();
};

class ConfirmDialog : public BasicDialog 
{
public:
    ConfirmDialog(const char* msg, const char* confirmLbl = "   OK   ");
};
*/
class StringDialog : public BasicDialog 
{
public:
    StringDialog(
        const char* msg, const char* sample = "",
        const char* confirmLbl = "   OK   "
    );
    StringDialog(
        const char* msg, int width,
        const char* confirmLbl = "   OK   "
    );

    const char* Get_String();
    void Select();
    void Select(int);
    void Select(int, int);
    void Change_Message(char *msg) {_sedit->Message(msg);} 
    void Read_Input(Menu_Window *, Menu_Valid_Fn, char *, char *);
    virtual boolean Accept();
private:
    void Init(const char*, const char*, int width = 0);
private:
    class StringEditor* _sedit;
};

/***********************************************************/
/* End */
/***********************************************************/


