//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// EvLog.cc - very primitive manager utility for EventLogs

/*
** Saleem N. Bhatti
** June 1992
**
** Security hooks added March/April 1994
*/

#if !defined(_EvLog_cc_)
#define _EvLog_cc_

#include <stdlib.h>
#include "msap.h"
#include "Error.h"
#include "SmiSntx.h"
#include "Syntaxes.h"
#include <memory.h>
#if defined(SECURITY)
#include "mauth.h"
#include "mintegrity.h"
#endif
#include "msap-pp.h"

#if defined(DEBUG2)
FILE *pduDump = (FILE *) 0;
#endif

#define EXIT_OK           0
#define EXIT_BadArgs     -1
#define EXIT_NoAssoc     -2
#define EXIT_AssocBroken -3
#define EXIT_ArgError    -4
#define EXIT_OpError     -5

int md,
    request_id = 0,
    request_invoke = 0,
    invoke = 0;

int Create(MSAPIndication *indication, Ava *filter);
int Delete(MSAPIndication *indication, int logId, int recordId = 0);
int Wait(MSAPIndication *indication);
int Set(MSAPIndication *indication, int logId, Ava *ava);
int Terminate();

void
usage()
{
    fprintf(stderr, "Usage:\n\n");
    fprintf(stderr, "evlog {<agent> <host> | -S <application>} -c [-f {<filter> | <event> ...}]\n");
    fprintf(stderr, "Create a log. <filter> gives discrminator construct\n\n");
    fprintf(stderr, "evlog {<agent> <host> | -S <application>} -d -l <n>\n");
    fprintf(stderr, "Delete log instance {logId=n}\n\n");
    fprintf(stderr, "evlog {<agent> <host> | -S <application>} -d -l <n> -r <m>\n");
    fprintf(stderr, "Delete log record instance {logId=n@logRecordId=m}\n\n");
    fprintf(stderr, "evlog {<agent> <host> | -S <application>} -s -l <n> -a <attrName=attrValue>\n");
    fprintf(stderr, "Set attribute value for log instance {logId=n}\n");
    fprintf(stderr, "<attrName>              <attrValue>\n");
    fprintf(stderr, "----------------------  -----------------------------\n");
    fprintf(stderr, "discriminatorConstruct  <filter>\n");
    fprintf(stderr, "administrativeState     locked | unlocked\n");
    fprintf(stderr, "maxLogSize              +ve integer (0 for no limit)\n");
    fprintf(stderr, "logFullAction           halt | wrap\n\n");
#if defined(SECURITY)
    fprintf(stderr, "Security options for use with all the above:\n");
    fprintf(stderr, "[-P <peer DN>] [-D <DSA name/addr>] [-I]\n\n");
#endif
}

main(int argc, char **argv)
{
    /* What is to be done? */
#define OP_CreateLog    1
#define OP_DeleteLog    2
#define OP_DeleteRecord 3
#define OP_Set          4
    static const
    char *whatStr[] = { "",
			"Create log",
			"Delete log",
			"Delete log record",
			"Set attribute value"
			};

    if (argc < 4) {
        usage();
        exit(EXIT_OK);
    }

    /* syntax info */
    if (initialiseSyntaxes((char *) 0, attrSyntaxes) == NOTOK) {
	fprintf(stderr, "cannot initialise syntaxes");
	exit(NOTOK);
    }

    int errFlag = 0, fFlag = 0, argIndex, logId = 0, recordId = 0, what = 0;
    Ava *ava = (Ava *) 0;
    char name[80], value[1024], *application = (char *) 0,
         **arg, *optFlags = "Scdsl:r:f:a:";

#if defined(SECURITY)
    int IFlag = 0;
    char *peer = (char *) 0,
         *dsa = (char *) 0;

    initialiseOsisec(argc, argv);
    initialiseMd5KeySpace();
    initialiseIdSpace();
    optFlags = "Scdsl:r:f:a:D:R:I";
#endif

    for(arg = argv, argIndex = 0; argIndex < argc && *arg != (char *) 0;
        ++arg, ++argIndex) {

        /*
        ** Can't use getopt(3) as it thinks the options all come first
        ** and any arg that does not have a '-' in front of it will
        ** cause it to stop parsing args. <agent> and <host> do not
        ** have '-' in front of them. Below is a mimic of getopt(3).
        */
        int opt = (*arg)[0] == '-' ? (*arg)[1] : 0;
        char *optArg;

        if (opt != 0) {
            /* Known option? */
            char *cp = strchr(optFlags, opt);
            if (cp == (char *) 0)
                opt = -1;

            /* Arguments? */
            if (*(++cp) == ':') {
                optArg = *(++arg);
                if (optArg == (char *) 0)
                    opt = -2;
            }
            else optArg = (char *) 0;
        }

        switch (opt) {

        case -2: /* Missing argument */
            fprintf(stderr, "Option -%c needs an argument\n", opt);
            errFlag = 1;
            break;

        case -1: /* Unknown option */
            fprintf(stderr, "Unknown option: -%c\n", opt);
            errFlag = 1;
            break;

        case 0:
            break;

        case ('S'): /* Single command line application name */
            if (application == (char *) 0)
                application = optArg;
            else {
                fprintf(stderr, "Use -S once only\n");
                errFlag = 1;
            }
            break;

        case ('c'): /* Create */
            if (what == 0)
                what = OP_CreateLog;
            else {
                fprintf(stderr, "Can't use -c, -d, -s together\n");
                errFlag = 1;
            }
            break;

        case ('d'): /* Delete -- assume log but may be logRecord */
            if (what == 0)
                what = OP_DeleteLog;
            else {
                fprintf(stderr, "Can't use -c, -d, -s together\n");
                errFlag = 1;
            }
            break;

        case ('s'): /* Set */
            if (what == 0)
                what = OP_Set;
            else {
                fprintf(stderr, "Can't use -c, -d, -s together\n");
                errFlag = 1;
            }
            break;

        case ('l'): /* logId */
            if (logId == 0) {
                logId = atoi(optArg);
                if (logId < 1) {
                    fprintf(stderr, "Bad value for logId - %d\n", logId);
                    errFlag = 1;
                }
            }
            else {
                fprintf(stderr, "Use -l once only\n");
                errFlag = 1;
            }
            break;

        case ('r'): /* logRecordId */
            if (recordId == 0 && what == OP_DeleteLog) {
                what = OP_DeleteRecord;
                recordId = atoi(optArg);
                if (recordId < 1) {
                    fprintf(stderr, "Bad value for logRecordId - %d\n",
                            recordId);
                    errFlag = 1;
                }
            }
            else {
                fprintf(stderr, "Use -r once only with -d\n");
                errFlag = 1;
            }
            break;

        case ('a'): /* Attribute value for set */
            if (what == OP_Set) {
                if (sscanf(optArg, "%[a-zA-Z0-9_-] = %[^\n]",
                           name, value) != 2) {
                    fprintf(stderr, "Bad attribute value spec - %s\n", optArg);
                    fprintf(stderr, "\t%s %s\n",
				name ? name : "", value ? value : "");
                    errFlag = 1;
                }
                else if ((ava = str2ava(name, value)) == (Ava *) 0 ||
			 ava->ava_value == (PE) 0) {
                    fprintf(stderr, "Bad attribute name/value - %s %s",
				name ? name : "", value ? value : "");
                    errFlag = 1;
                }
            }
            else {
                fprintf(stderr, "Use -a with -s only\n");
                errFlag = 1;
            }
            break;

        case ('f'): /* Filter value for Create */
            if (what == OP_CreateLog && !fFlag) {

                int manyNames = 0;
                char *bp = value;

                bp[0] = '\0';
                for( ;
                    argIndex < argc && *arg != (char *) 0 && (*arg)[0] != '\0';
                    ++arg, ++argIndex) {

                    if ((*arg)[0] == '-') { /* Next option */
                        --arg;
                        --argIndex;
                        break;
                    }

                    if ((*arg)[0] == '(') { /* filter expression */
                        memcpy(value, (*arg), strlen(*arg));
                        break;
                    }

                    if (bp == value) { /* start of list */

                        /* more than one name? */
                        if (*(arg + 1) != (char *) 0 &&
                            (*(arg + 1))[0] != '\0' &&
                            (*(arg + 1))[0] != '-') {
                            sprintf(bp, "(");
                            bp = value + strlen(value);

                            manyNames = 1;
                        }
                    }

                    /* notification name */
                    sprintf(bp, "(eventType=%s)", *arg);
                    bp = value + strlen(value);

                    if (manyNames) {

                        /* end of list? */
                        if (*(arg + 1) == (char *) 0 ||
                            (*(arg + 1))[0] == '\0' ||
                            (*(arg + 1))[0] == '-') {
                            sprintf(bp, ")");
                            break;
                        }

                        /* OR filter */
                        else {
                            sprintf(bp, "|");
                            bp = value + strlen(value);
                        }
                    }
                }

                if ((ava = str2ava("discriminatorConstruct", value)) ==
                    (Ava *) 0 || ava->ava_value == (PE) 0) {
                    fprintf(stderr, "Bad filter value - %s\n", value);
                    errFlag = 1;
                }
                else fFlag = 1;
            }
            else {
                fprintf(stderr, "Use -f once only with -c\n");
                errFlag = 1;
            }
            break;

#if defined(SECURITY)
        case ('P'): /* peer DN */
            if (peer == (char *) 0)
                peer = optArg;
            else {
                fprintf(stderr, "Use -P once only\n");
                errFlag = 1;
            }
            break;

        case ('D'): /* DSA name or address */
            if (dsa == (char *) 0)
                dsa = optArg;
            else {
                fprintf(stderr, "Use -D once only\n");
                errFlag = 1;
            }
            break;

        case ('I'): /* Use integrity checks */
            if (!IFlag)
                IFlag = 1;
            else {
                fprintf(stderr, "Use -I once only\n");
                errFlag = 1;
            }
            break;
#endif

        default: /* This should not happen */
            fprintf(stderr, "Unknown option: %c\n", opt);
            errFlag = 1;
            break;
        }

        if (errFlag) {
            fprintf(stderr, "Error parsing command line\n");
            usage();
            exit(EXIT_ArgError);
        }
    }

    if (what == 0) {
        fprintf(stderr, "Please specify -c, -d or -s\n");
        usage();
        exit(EXIT_BadArgs);
    }

    printf("\n%s\n", whatStr[what]);

    /* Set up assoc */
    MInitOption *miop[2]; /* Connect options */

#ifdef SECURITY
    MInitOption mio;
    AuthAssocIntegrityInfo *aaii;
    aaii = AuthAssocIntegrityInfo_make();

    if (peer != (char *) 0)
        aaii->peer = strdup(peer);
    if (dsa != (char *) 0)
        aaii->dsa = strdup(dsa);
    if (IFlag)
        memcpy((void *) aaii->sessionKey, (void *) makeMd5Key(), 8);

    mio.optionId = MIO_AuthAssocIntegrityInfo;
    mio.optionInfo = (void *) aaii;

    miop[0] = &mio;
    miop[1] = (MInitOption *) 0;

#else

   miop[0] = miop[1] = (MInitOption *) 0;

#endif

    char *host = strcasecmp(argv[1], "-S") == 0 ? NULLCP : argv[2];
    if (application == (char *) 0)
        application = argv[1];

    if ((md = m_initialise(application, host, miop)) < 0)
        error((char *) 0, "cannot establish association to %s at %s",
              argv[1], argv[2]);

#if defined(SECURITY)
    if (aaii != (AuthAssocIntegrityInfo *) 0)
        AuthAssocIntegrityInfo_free(aaii);
#endif

    /* Do it */
    MSAPIndication indication;

#if defined(SECURITY)
    request_id = makeInitiatorId(md, request_id);
#else
    request_id = 1;
#endif

    switch (what) {

    case OP_CreateLog :
        if (Create(&indication, ava) == NOTOK) {
            printf("Create failed\n");
            errFlag = 1;
        }
        break;

    case OP_DeleteLog :
        if (Delete(&indication, logId) == NOTOK) {
            printf("Delete logId=%d failed\n", logId);
            errFlag = 1;
        }
        break;

    case OP_DeleteRecord :
        if (Delete(&indication, logId, recordId) == NOTOK) {
            printf("Delete logId=%d@recordId=%d failed\n", logId, recordId);
            errFlag = 1;
        }
        break;

    case OP_Set :
	if (Set(&indication, logId, ava) == NOTOK) {
            printf("Set for logId=%d failed\n", logId);
            errFlag = 1;
        }
	break;
    }

    if (ava != (Ava *) 0)
	ava_free(ava);

    if (errFlag) {
	Terminate();
	exit(EXIT_ArgError);
    }
    request_invoke = indication.mi_invoke;

#if defined(DEBUG2)
    if ((pduDump = fopen("evlog.rx.log", "w")) != (FILE *) 0)
        ppSetOutputFile(pduDump);
#endif

    errFlag = Wait(&indication);

    Terminate();
    printf("Done\n");

#if defined(DEBUG2)
    if (pduDump != (FILE *) 0)
        fclose(pduDump);
#endif

    exit(errFlag == OK ? EXIT_OK : EXIT_OpError);
}


int
Create(MSAPIndication *indication, Ava *filter)
{
    /* object class */
    struct MIDentifier objectClass;

    objectClass.mid_type = MID_GLOBAL;
    objectClass.mid_global = name2oid("log"); /* should check != NULL */

    CMISParam attr;
    register
    int nAttrs = 0;

    if (filter != (Ava *) 0) {
	attr.mp_id.mid_type = MID_GLOBAL;
	attr.mp_id.mid_global = filter->ava_oid;
	attr.mp_val = filter->ava_value;
	nAttrs = 1;
    }

    /* Create */
    register
    int i = M_Create(md, request_id, &objectClass, (MN) 0, 0, (MN) 0,
                     (External *) 0,  nAttrs, (nAttrs > 0 ? &attr :
                                               (CMISParam *) 0), indication);

    mid_free(&objectClass);
    if (filter != (Ava *) 0)
	mparm_free(&attr);

    return(i);
}

int
Delete(MSAPIndication *indication, int logId, int recordId)
{
    if ((logId <= 0) || (recordId < 0)) return(NOTOK);

    /*  object class */
    struct MIDentifier objectClass;

    objectClass.mid_type = MID_GLOBAL;
    objectClass.mid_global = name2oid(recordId > 0 ? 
				(char*) "eventLogRecord" : (char*) "log");

    /* object name */
    MName objectName;
    char name[256];

    if (recordId > 0)
        sprintf(name, "logId=%d@logRecordId=%d", logId, recordId);
    else sprintf(name, "logId=%d", logId);
    objectName.mn_type = MN_LOCALDN;
    objectName.mn_dn = str2dn(name);

    /* Delete */
    register
    int i = M_Delete(md, request_id, &objectClass, &objectName,
                     (CMISScope *) 0, (CMISFilter *) 0, (External *) 0,
                     s_bestEffort, indication);

    mid_free(&objectClass);
    mn_free(&objectName);
    return(i);
}

int
Set(MSAPIndication *indication, int logId, Ava *ava)
{
    if ((logId < 0) || (ava == (Ava *) 0)) return(NOTOK);

    /* object class */
    struct MIDentifier objectClass;

    objectClass.mid_type = MID_GLOBAL;
    objectClass.mid_global = name2oid("log");

    /* object name */
    MName objectName;
    char name[256];

    sprintf(name, "logId=%d", logId);
    objectName.mn_type = MN_LOCALDN;
    objectName.mn_dn = str2dn(name);

    /* attribute value */
    CMISSetAttr attr;

    attr.sa_id.mid_type = MID_GLOBAL;
    attr.sa_id.mid_global = ava->ava_oid;
    attr.sa_val = ava->ava_value;
    attr.sa_modify = m_replace;
    attr.sa_error = m_noError; // Not required

    register
    int i = M_SetConf(md, request_id, &objectClass, &objectName,
                      (CMISScope *) 0, (CMISFilter *) 0, (External *) 0,
                      s_bestEffort, 1, &attr, indication);

    mid_free(&objectClass);
    mn_free(&objectName);
    return(i);
}

int
Wait(MSAPIndication *indication)
{
    printf("Waiting ...\n");

    PE pdu;

    M_WaitReqAux(md, NOTOK, indication, &pdu);

#if defined(DEBUG2)
    if (pduDump != (FILE *) 0)
        ppMSAPIndication(indication);
#endif

#if defined(SECURITY)
    if (indication->mi_invoke != 0) {

        MD5Value md5;

        memset((void *) &md5, 0, sizeof(MD5Value));

        if (pdu == (PE) 0) {
            if (indication->mi_type == MI_ERROR)
                pdu = int2prim(request_id);

            else
                printf("Unexpected MSAPIndication type: %d\n", indication->mi_type);
        }

        if (makeMd5Value(md, pdu, &md5) != 0)
            printf("Could not make integrity check for PDU\n");

        register
        int chk1 = ENCODE_ID(request_id, md5.md5_words[0]);

        register
        int i = indication->mi_invoke == chk1 ? request_id : 0;

        printf("check %s - invokeId=%#08x id=%#08x chk=%#08x\n",
               i == 0 ? "failed" : "ok",
               (u_int) indication->mi_invoke, (u_int) i, md5.md5_words[0]);

        if (i == 0)
            printf("chk1=%#08x request_id=%#08x\n",
                   (u_int) chk1, (u_int) request_id);
    }
#endif

    if (pdu != (PE) 0)
        pe_free(pdu);

    register
    int result;
    register
    char *name;

    switch (indication->mi_type) {

    case MI_CREATE_RES :
    {
        CMISCreateRes *cr = &(indication->mi_createres.result);

        printf("CreateResult\n");

        if (cr->cr_class.mid_global != (OID) 0)
            printf("\tclass=%s\n", oid2name(cr->cr_class.mid_global, OIDPART));
        name = dn2str(cr->cr_inst.mn_dn);
        printf("\tname={%s}\n", name);
        free(name);
        if (cr->cr_time != (char *) 0)
            printf("\ttime=%s\n", cr->cr_time);
        result = OK;
    }
        break;

    case MI_DELETE_RES :
    {
        CMISDeleteRes *dr = &(indication->mi_deleteres.result);

        printf("DeleteResult\n");

        if (dr->dr_class.mid_global != (OID) 0)
            printf("\tclass=%s\n", oid2name(dr->dr_class.mid_global, OIDPART));
        name = dn2str(dr->dr_inst.mn_dn);
        printf("\tname={%s}\n", name);
        free(name);
        if (dr->dr_time != (char *) 0)
            printf("\ttime=%s\n", dr->dr_time);
        result = OK;
    }
        break;


    case MI_SET_RES :
    {
	CMISSetRes *sr = &(indication->mi_setres.result);

	printf("SetResult\n");

	if (sr->sr_class.mid_global != (OID) 0)
	    printf("\tclass=%s\n", oid2name(sr->sr_class.mid_global, OIDPART));
        name = dn2str(sr->sr_inst.mn_dn);
        printf("\tname={%s}\n", name);
        free(name);
        if (sr->sr_time != (char *) 0)
	    printf("\ttime=%s\n", sr->sr_time);
        result = OK;
    }
        break;

    case MI_ERROR :
	printf("Error - %s : check command line\n",
               MErrString(indication->mi_errtype));
	result = NOTOK;
	break;


    case MI_PREJECT :
    case MI_ABORT :
        printf("Wait() : received %s\n", indication->mi_type == MI_ABORT ?
               "abort" : "provider reject");
        mi_free(indication);
        exit(EXIT_AssocBroken);

    default :
        printf("Wait() : Unexpected MSAP indication type : %d\n",
               indication->mi_type);
        result = NOTOK;
        break;
    }

    mi_free(indication);
    return(result);
}

int
Terminate()
{ return(m_terminate(md)); }

#endif /* !_EvLog_cc_ */
