//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// MAction.cc - enables to perform an action on managed objects in
//		a remote MIB

// By George Pavlou, January 1993

/*
** Security hooks added March/April 1994
** Saleem
*/

#include <stdlib.h>
#if	defined(__GNUG__) || (defined(__cplusplus) && defined(AIXV3))
extern "C" { int  strncasecmp(const char*, const char*, size_t); }
#endif
#include "msap.h"
#include "Error.h"
#include "Syntaxes.h"
#include <memory.h>
#if defined(SECURITY)
#include "mauth.h"
#include "mintegrity.h"
#endif


#if defined(DEBUG)
#include "msap-pp.h"
FILE *pduDump = (FILE *) 0;
#endif


int request_id = 0,
    request_invoke = 0,
    invoke = 0;
MSAPIndication mis;
Bool timing = False;


int   parseAvaArg (int, char**, char**, char**, int*);
int   parseFilterArg (int, char**, char**, int*);
 

static int   do_action (int, OID, MN, CMISScope*, CMISFilter*, CMISSync, Ava*,
			MSAPIndication*);
static int   do_wait (int, MSAPIndication*);


main (int argc, char** argv)
{
    char* application = NULLCP;
    char* host = NULLCP;
    char* strObjClass = NULLCP;
    char* strObjInst = NULLCP;
    char* strActionType = NULLCP;
    char* strActionValue = NULLCP;
    char* strScope = NULLCP;
    char* strSync = NULLCP;
    char* strFilter = NULLCP;
    int   i, j, state = 0;

    OID objClass = NULLOID;
    MN  objInst  = NULLMN;
    Ava* ava	 = NULLAVA;
    CMISScope* scope = NULLMSCOPE;	// default
    CMISSync sync = s_bestEffort;	//  ..
    CMISFilter* filter = NULLMFILTER;	//  ..
#ifdef SECURITY
    int IFlag = 0;
    char *peer = (char *) 0,
         *dsa = (char *) 0;
#endif

    if (argc < 5)
	goto usage;

    if (strcmp(argv[1], "-S") == 0)
        application = argv[2];

    for (i = 3; i < argc; i++)
	switch (state) {
	case 0:
	    if (*argv[i] == '-' && strlen(argv[i]) == 2)
		switch (*(argv[i]+1)) {
		case 'c':
		    state = 1;
		    break;
		case 'i':
		    state = 2;
		    break;
		case 's':
		    state = 3;
		    break;
		case 'f':
		    state = 4;
		    break;
		case 'a':
		    state = 5;
		    break;
		case 't':
		    timing = True;
		    break;
#ifdef SECURITY
                case 'P':
                    state = 6;
                    break;
                case 'D':
                    state = 7;
                    break;
                case 'I':
                    state = 0;
                    IFlag = 1;
                    break;
#endif
		default:
		    goto usage;
		    }
	    else
		goto usage;
	    break;

	case 1:
	    strObjClass = argv[i];
	    state = 0;
	    break;

	case 2:
	    strObjInst = argv[i];
	    state = 0;
	    break;

	case 3:
	    strScope = argv[i];
	    if (i < argc-1 && *argv[i+1] != '-')
		strSync = argv[++i];
	    state = 0;
	    break;

	case 4:
	    if (parseFilterArg(argc-i, &argv[i], &strFilter, &j) == NOTOK)
		goto usage;
	    i += j-1;
	    state = 0;
	    break;

	case 5:
	{
	    Bool more = False;
	    for (j = i; j < argc; j++)
	        if (*argv[j] == '-' && strlen(argv[j]) == 2) {
		    more = True;
		    break;
		}

	    if (parseAvaArg(argc-i, &argv[i],
			&strActionType, &strActionValue, &j) == NOTOK)
		goto usage;
	    i = more ? i+j-1 : i+j;
	}
	    state = 0;
	    break;
#ifdef SECURITY
        case 6:
            peer = argv[i];
            state = 0;
            break;
        case 7:
            dsa = argv[i];
            state = 0;
            break;
#endif
	}

    if (!strActionType)
	goto usage;

    // load syntaxes

    if (initialiseSyntaxes(NULLCP, attrSyntaxes) == NOTOK)
	error(NULLCP, "maction: cannot initialise syntaxes");

#ifdef SECURITY
    initialiseOsisec(argc, argv);
    initialiseMd5KeySpace();
    initialiseIdSpace();
#endif

    // convert the string arguments to CMIS parameters

    if (strObjClass && (objClass = name2oid(strObjClass)) == NULLOID)
	error(NULLCP, "invalid object class: %s", strObjClass);

    if (strObjInst && (objInst = ObjectInstance_parse(strObjInst)) == NULLMN)
        error(NULLCP, "invalid object instance %s", strObjInst);

    if (!(ava = str2ava_info(strActionType, strActionValue)))
        error(NULLCP, "invalid action type \"%sInfo\" or value \"%s\"",
			strActionType, strActionValue);

    if (strScope && (scope = str2scope(strScope)) == NULLMSCOPE)
	error(NULLCP, "invalid scope %s", strScope);

    if (strSync && (sync = str2sync(strSync)) == s_invalid)
	error(NULLCP, "invalid sync %s", strSync);
	
    if (strFilter && strFilter[0] != '\0'  &&
		(filter = str2mfilter(strFilter)) == NULLMFILTER)
	error(NULLCP, "invalid filter %s", strFilter);


    // now try to connect to the remote agent
    int msd;

    MInitOption *miop[2]; /* Connect options */

#ifdef SECURITY
    MInitOption mio;
    AuthAssocIntegrityInfo *aaii;
    aaii = AuthAssocIntegrityInfo_make();

    if (peer != (char *) 0)
        aaii->peer = strdup(peer);
    if (dsa != (char *) 0)
        aaii->dsa = strdup(dsa);
    if (IFlag)
        memcpy((void *) aaii->sessionKey, (void *) makeMd5Key(), 8);

    mio.optionId = MIO_AuthAssocIntegrityInfo;
    mio.optionInfo = (void *) aaii;

    miop[0] = &mio;
    miop[1] = (MInitOption *) 0;

#else

   miop[0] = miop[1] = (MInitOption *) 0;

#endif

    if (timing)
	do_timing(NULLCP);

    host = application ? NULLCP : argv[2];
    application = application ? application : argv[1];

    if ((msd = m_initialise(application, host, miop)) < 0)
	error(NULLCP, "cannot establish association to %s at %s",
			argv[1], argv[2]);

    if (timing)
	do_timing("established association");

    // perform the M-ACTION request

    if (!timing)
        printf("\n\n");

    do_action(msd, objClass, objInst, scope, filter, sync, ava, &mis);

    // wait for the replies
#if defined(DEBUG2)
    if ((pduDump = fopen("maction.rx.log", "w")) != (FILE *) 0)
        ppSetOutputFile(pduDump);
#endif

    while (do_wait(msd, &mis) != DONE);

    if (timing)
	do_timing("performed action(s)");

    // terminate

    m_terminate(msd);

    if (timing)
	do_timing("released association");

#if defined(DEBUG2)
    if (pduDump != (FILE *) 0)
        fclose(pduDump);
#endif

    exit(OK);

usage:
    fprintf(stderr, "usage: maction { <agent> <host> | -S <agent> }\n\
\t[-c <class>] [-i <instance>]\n\
\t[-s <scope> [<sync>]] [-f filter] -a <actionType[=<actionValue>]\n");
#if defined(SECURITY)
    fprintf(stderr, "\t[-P <peer DN>] [-D <dsa name/addr>] [-I]\n");
#endif

    exit(0);
}


// performs an action
static int  do_action (int msd, OID objClass, MN objInst, CMISScope* scope,
			CMISFilter* filter, CMISSync sync, Ava* action,
			MSAPIndication* mi)
{
    struct MIDentifier	obj_class;
    struct MIDentifier	actionType;
    PE		actionValue;
    int		status;

    obj_class.mid_type = MID_GLOBAL;
    obj_class.mid_global = objClass;

    if (!objInst) {
        objInst = new  MName;
        objInst -> mn_type = MN_LOCALDN;
        objInst -> mn_dn = dn_comp_new(NULLRDN);
    }

    actionType.mid_type   = MID_GLOBAL;
    actionType.mid_global = action->ava_oid;

    actionValue		  = action->ava_value;

#if defined(SECURITY)
    request_id = makeInitiatorId(msd, request_id);
#else
    request_id = 1;
#endif
    status = M_ActionConf(msd, request_id, &obj_class, objInst, scope,
			filter, NULLMACCESS, sync,
			&actionType, action->ava_value, mi);
    request_invoke = mi->mi_invoke;

    // release memory of arguments (for the sake of it...)

    mid_free(&obj_class);
    mn_free(objInst);
    delete objInst;
    if (scope)
	free((char*) scope);
    if (filter)
	mfilter_free(filter);
    ava_free(action);

    return status;
}

static int  do_wait (int msd, MSAPIndication* mi)
{
    int status = OK;
    static int nsucc = 0, nfail = 0;
    Bool linked = False;
    PE pdu;

    M_WaitReqAux(msd, NOTOK, mi, &pdu);

#if defined(SECURITY)
#if defined(DEBUG)
    if (pduDump != (FILE *) 0)
        ppMSAPIndication(mi);
#endif

    invoke = makeResponderId(msd, invoke);

    MD5Value md5;

    memset((void *) &md5, 0, sizeof(MD5Value));

    if (mi->mi_invoke != 0) {

        if (pdu == (PE) 0) {
            /*
            ** Empty Result.
            **
            ** The last of a series of linked replies.
            */
            if (mi->mi_type == MI_EMPTY_RES) {
                printf("Empty Result: ");
                pdu = int2prim(request_id);
            }

            /*
            ** Error.
            **
            ** Some error cases do not have any info and pdu will be NULL.
            */
            else if (mi->mi_type == MI_ERROR)
                pdu = int2prim(nsucc == nfail == 0 ? request_id : invoke);

            else
                printf("Unexpected MSAPIndication type: %d\n", mi->mi_type);
        }

        if (makeMd5Value(msd, pdu, &md5) != 0)
            printf("Could not make integrity check for PDU\n");

        register
        int chk1 = ENCODE_ID(invoke, md5.md5_words[0]),
            chk2 = ENCODE_ID(request_id, md5.md5_words[0]);

        register
        int i = mi->mi_invoke == chk1 ? invoke :
            (mi->mi_invoke == chk2 ? request_id : 0);

        if (i == 0)
            printf("check failed - msapId=%#08x chk1=%#08x chk2=%#08x\n",
                   (u_int) mi->mi_invoke, (u_int) chk1, (u_int) chk2);

        printf("invokeId=%#08x id=%#08x chk=%#08x\n",
               (u_int) mi->mi_invoke, (u_int) i, md5.md5_words[0]);
    }
#endif

    if (pdu != NULLPE)
        pe_free(pdu);

    if (mi->mi_type == MI_ACTION_RES || mi->mi_type == MI_LINKED_REPLY) {
	int	id;
	MID	objClass;
	MN	objInst;
	char*	time = NULLCP;
	CMISParam* result = NULLMPARM;
	CMISErrors error = m_noError;

	switch (mi->mi_type) {
	case MI_ACTION_RES:
	    id	     = mi->mi_actionres.id;
	    objClass = &mi->mi_actionres.result.ar_class;
	    objInst  = &mi->mi_actionres.result.ar_inst;
	    time     = mi->mi_actionres.result.ar_time;
	    result   = &mi->mi_actionres.result.ar_reply;
	    if (!timing)
	        printf("Action Result - id %#08x, ", (u_int) id);
	    ++nsucc;
	    status = DONE;	// action on only one object
	    break;

	case MI_LINKED_REPLY:
            linked = True;
	    id = mi->mi_linkedreplyinv.args.lr_linkid;
	    switch (mi->mi_linkedreplyinv.args.lr_type) {
	    case LR_ACTION_RES:
	        printf("Linked Reply Action - id %#08x, ", (u_int) id);
		objClass = &mi->mi_linkedreplyinv.args.lr_actionres.ar_class;
		objInst  = &mi->mi_linkedreplyinv.args.lr_actionres.ar_inst;
		time     = mi->mi_linkedreplyinv.args.lr_actionres.ar_time;
		result	 = &mi->mi_linkedreplyinv.args.lr_actionres.ar_reply;
		++nsucc;
		break;

	    case LR_ACTION_ERR:
		printf("Linked Reply Error - id %#08x\n", (u_int) id);
		ppCMISActionError(&mi -> mi_linkedreplyinv.args.lr_actionerr);
		printf("\n");
		++nfail;
		error = m_noSuchAction;	// anything other than m_noError
		break;

	    case LR_PROC_FAILURE:
		printf("Linked Reply Processing Failure - id %#08x\n",
			(u_int) id);
		ppCMISProcessingFailure(&mi ->
			mi_linkedreplyinv.args.lr_procfailure);
		printf("\n");
		++nfail;
		error = m_processingFailure;
		break;
	    }
	    break;
        }

	if (id != (linked ? request_invoke :
#if defined(SECURITY)
                   ENCODE_ID(request_id, md5.md5_words[0])
#else

                   request_id
#endif
                   )) {
	    // should not happen for a correctly behaving agent
	    printf("Action result with wrong request id received!\n");
	    mi_free(mi);
	    return DONE;
	}

	if (!error) {
	    if (!timing && objClass->mid_global)
                printf("class %s ", oid2name(objClass->mid_global, OIDPART));

	    if (!timing && objInst->mn_dn) {
	        char* inst = dn2str(objInst->mn_dn);
	        printf("instance {%s} ", inst);
	        free(inst);
	    }

	    if (!timing && time)
                printf("time %s ", time);

	    if (result->mp_val) {
	        char *actionval;
	        ava2str_reply(result->mp_id.mid_global, result->mp_val,
							NULL, &actionval);
	        if (!timing)
	            printf("\nResult: %s", actionval);
	        free(actionval);
	    }

	    if (!timing)
	        printf("\n\n");
	}
    }
    else
    if (mi->mi_type == MI_EMPTY_RES && mi->mi_emptyres.id ==
#if defined(SECURITY)
        ENCODE_ID(request_id, md5.md5_words[0])
#else
        request_id
#endif
        )
        status = DONE;
    else
    if (mi->mi_type == MI_PREJECT || mi->mi_type == MI_ABORT) {
	fprintf(stderr, "maction: received %s, exits\n",
		mi->mi_type == MI_PREJECT ?
				"provider reject" : "association abort");
#if defined(DEBUG2)
        if (pduDump != (FILE *) 0)
            fclose(pduDump);
#endif
	exit(NOTOK);
    }
    else
    if (mi->mi_type == MI_ERROR || mi->mi_type == MI_UREJECT) {
	ppCMISErrors(mi->mi_errtype);
	printf("\n");
	ppCMISErrorInfo(mi->mi_errtype, &mi->mi_error.info);
	printf("\n");
	++nfail;
	status = DONE;
    }
    else {
	printf("Unexpected mindication type: %d\n", mi->mi_type);
	status = NOTOK;
    }

    if (!timing && status == DONE) {
	if (nsucc)
	    printf("\n%d succesful actions on managed objects\n", nsucc);
	if (nfail)
	    printf("\n%d action failures\n", nfail);
    }
    mi_free(mi);
    return status;
}
