//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// MCreate.cc - enables to create a managed object in a remote MIB

// By George Pavlou, January 1993


#include <stdlib.h>
#include "msap.h"
#include "Error.h"
#include "Syntaxes.h"
#include <memory.h>
#if defined(SECURITY)
#include "mauth.h"
#include "mintegrity.h"
#endif
#include "msap-pp.h"

#if defined(DEBUG2)
FILE *pduDump = (FILE *) 0;
#endif

#define MAXATTRS	50

int request_id = 0,
    request_invoke = 0,
    invoke = 0;
MSAPIndication mis;

int   parseAvaArg (int, char**, char**, char**, int*);

static int   do_create (int, MID, MN, int, MN, int, CMISParam*,
			MSAPIndication*, External* );
static int   do_wait (int, MSAPIndication*);

main (int argc, char** argv)
{
    char* application = NULLCP;
    char* host = NULLCP;
    char* strObjClass = NULLCP;
    char* strObjInst = NULLCP;
    char* strRefInst = NULLCP;
    char* strAttrType [MAXATTRS];
    char* strAttrValue [MAXATTRS];
    int nattrs = 0;
    int   i, j, state = 0;

    struct MIDentifier objClass;
    objClass.mid_type = MID_GLOBAL;
    objClass.mid_global = NULLOID;
    MN objInst = NULLMN;
    MN refInst = NULLMN;
    int instType;
    CMISParam attrs [MAXATTRS];
#ifdef SECURITY
    int IFlag = 0;
    char *peer = (char *) 0,
         *dsa = (char *) 0;
#endif

    char *    accessExterncp = NULLCP ;    
    External* accessExtern   = NULLMACCESS ;

    if (argc < 5)
	goto usage;

    if (strcmp(argv[1], "-S") == 0)
        application = argv[2];

    for (i = 3; i < argc; i++)
	switch (state) {
	case 0:
	    if (*argv[i] == '-' && strlen(argv[i]) == 2)
		switch (*(argv[i]+1)) {
		case 'c':
		    state = 1;
		    break;
		case 'i':
		    state = 2;
		    break;
		case 's':
		    state = 3;
		    break;
		case 'r':
		    state = 4;
		    break;
		case 'a':
		    state = 5;
		    break;
#ifdef SECURITY
                case 'P':
                        state = 6;
                        break;
                case 'D':
                        state = 7;
                        break;
                case 'I':
                        state = 0;
                        IFlag = 1;
                        break;
#endif
#ifdef IQA
		case 'C':
			state = 8 ;
			break ;
#endif
		default:
		    goto usage;
		}
	    else
		goto usage;
	    break;

	case 1:
	    strObjClass = argv[i];
	    state = 0;
	    break;

	case 2:
	    strObjInst = argv[i];
	    instType = CA_OBJECT_INST;
	    state = 0;
	    break;

	case 3:
	    strObjInst = argv[i];
	    instType = CA_SUPERIOR_INST;
	    state = 0;
	    break;

	case 4:
	    strRefInst = argv[i];
	    state = 0;
	    break;

	case 5:
	    if (parseAvaArg(argc-i, &argv[i], &strAttrType[nattrs],
			&strAttrValue[nattrs], &j) == NOTOK ||
			!strAttrValue[nattrs] ||
			strAttrValue[nattrs][0] == '\0')
		goto usage;
	    ++nattrs;
#ifndef SECURITY
	    i += j;
#else
	    i += j - 1;
#endif
	    state = 0;
	    break;
#ifdef SECURITY
        case 6:
                peer = argv[i];
                state = 0;
                break;
        case 7:
                dsa = argv[i];
                state = 0;
                break;
#endif
#ifdef IQA
	case 8:

		accessExterncp = argv[i] ;
		state = 0 ;
		break ;
#endif
	}

    if (!strObjClass)
	goto usage;

    // load syntaxes

    if (initialiseSyntaxes(NULLCP, attrSyntaxes) == NOTOK)
	error(NULLCP, "mcreate: cannot initialise syntaxes");

#ifdef SECURITY
    initialiseOsisec(argc, argv);
    initialiseMd5KeySpace();
    initialiseIdSpace();

    if( (accessExterncp != NULLCP) &&
    	(accessExtern = buildAccessControl( accessExterncp ) ) == NULLMACCESS )
	error( NULLCP, "mcreate: can not encode the IQA SNMPv1 community string %s", accessExterncp ) ;
#endif

    // convert the string arguments to CMIS parameters

    if ((objClass.mid_global = name2oid(strObjClass)) == NULLOID)
	error(NULLCP, "invalid object class: %s", strObjClass);

    if (strObjInst && (objInst = ObjectInstance_parse(strObjInst)) == NULLMN)
            error(NULLCP, "invalid object instance %s", strObjInst);

    if (strRefInst && (refInst = ObjectInstance_parse(strRefInst)) == NULLMN)
            error(NULLCP, "invalid reference instance %s", strRefInst);

    for (i = 0; i < nattrs; i++) {
	Ava* ava;
	if ((ava = str2ava(strAttrType[i], strAttrValue[i])) == NULLAVA)
	    error(NULLCP, "invalid attribute type \"%s\" or value \"%s\"",
			strAttrType[i], strAttrValue[i]);
	attrs[i].mp_id.mid_type = MID_GLOBAL;
	attrs[i].mp_id.mid_global = ava->ava_oid;
	attrs[i].mp_val = ava->ava_value;
	free ((char*) ava);
    }

    // now try to connect to the remote agent

    int msd;
    MInitOption *miop[2]; /* Connect options */

#ifdef SECURITY
    MInitOption mio;
    AuthAssocIntegrityInfo *aaii;
    aaii = AuthAssocIntegrityInfo_make();

    if (peer != (char *) 0)
        aaii->peer = strdup(peer);
    if (dsa != (char *) 0)
        aaii->dsa = strdup(dsa);
    if (IFlag)
        memcpy((void *) aaii->sessionKey, (void *) makeMd5Key(), 8);

    mio.optionId = MIO_AuthAssocIntegrityInfo;
    mio.optionInfo = (void *) aaii;

    miop[0] = &mio;
    miop[1] = (MInitOption *) 0;

#else

   miop[0] = miop[1] = (MInitOption *) 0;

#endif

    host = application ? NULLCP : argv[2];
    application = application ? application : argv[1];


    if ((msd = m_initialise(application, host, miop)) < 0)
        error(NULLCP, "cannot establish association to %s at %s",
                        argv[1], argv[2]);
#if defined(SECURITY)
    if (aaii != (AuthAssocIntegrityInfo *) 0)
        AuthAssocIntegrityInfo_free(aaii);
#endif

    // perform the M-CREATE request
    do_create(msd, &objClass, objInst, instType, refInst, nattrs, attrs, &mis, accessExtern );

#if defined(DEBUG2)
    if ((pduDump = fopen("mcreate.rx.log", "w")) != (FILE *) 0)
        ppSetOutputFile(pduDump);
#endif

    while (do_wait(msd, &mis) != DONE);

    // terminate

    m_terminate(msd);

#if defined(DEBUG2)
    if (pduDump != (FILE *) 0)
        fclose(pduDump);
#endif

    exit(OK);

usage:
    fprintf(stderr, "usage: mcreate { <agent> <host> | -S <agent> }\n\
\t-c <class>\n\
\t[-i <instance> | -s <superiorInst>] [-r referenceInst]\n\
\t[-a <attrType>=<attrVal>] ...\n");
#if defined(SECURITY)
    fprintf(stderr, "\t[-P <peer DN>] [-D <dsa name/addr>] [-I]\n");
#endif
#ifdef IQA
    fprintf(stderr, "\t[-C <IQA SNMPv1 Community String>]\n");
#endif

    exit(0);
}

// perform a M-CREATE operation
static int  do_create (int msd, MID objClass, MN objInst, int instType,
		       MN refInst, int nattrs, CMISParam* attrs,
		       MSAPIndication* mi, External* accessExtern )
{
    int		status;

#if defined(SECURITY)
    request_id = makeInitiatorId(msd, request_id);
#else
    request_id = 1;
#endif

    status = M_Create(msd, request_id, objClass,
			objInst, instType, refInst,
			accessExtern, nattrs, attrs, mi);
    request_invoke = mi->mi_invoke;

    // release memory of arguments (for the sake of it...)

    mid_free(objClass);
    if (objInst) {
	mn_free(objInst);
        delete objInst;
    }
    if (refInst) {
	mn_free(refInst);
        delete refInst;
    }
    for (register int i = 0; i < nattrs; i++)
	mparm_free(&attrs[i]);

    return status;
}

static int  do_wait (int msd, MSAPIndication* mi)
{
    int status = OK;
    PE pdu;

    M_WaitReqAux(msd, NOTOK, mi, &pdu);

#if defined(DEBUG2)
    if (pduDump != (FILE *) 0)
        ppMSAPIndication(mi);
#endif

#if defined(SECURITY)
    invoke = makeResponderId(msd, invoke);

    MD5Value md5;

    memset((void *) &md5, 0, sizeof(MD5Value));

    if (mi->mi_invoke != 0) {

        if (pdu == (PE) 0) {
            /*
            ** Empty Result.
            **
            ** If caused by a Cancel Get then the invokeId value will be
            ** that of the corresponding Get else it will be a value
            ** selected by the agent.
            */
            if (mi->mi_type == MI_EMPTY_RES) {
                printf("Empty Result: ");
                pdu = int2prim(request_id);
            }

            /*
            ** Error.
            **
            ** Some error cases do not have any info and pdu will be NULL.
            */
            else if (mi->mi_type == MI_ERROR)
                pdu = int2prim(request_id);

            else
                printf("Unexpected MSAPIndication type: %d\n", mi->mi_type);
        }

        if (makeMd5Value(msd, pdu, &md5) != 0)
            printf("Could not make integrity check for PDU\n");

        register
        int chk1 = ENCODE_ID(invoke, md5.md5_words[0]),
            chk2 = ENCODE_ID(request_id, md5.md5_words[0]);

        register
        int i = mi->mi_invoke == chk1 ? invoke :
            (mi->mi_invoke == chk2 ? request_id : 0);

        printf("check %s - invokeId=%#08x id=%#08x chk=%#08x\n",
               i == 0 ? "failed" : "ok",
               (u_int) mi->mi_invoke, (u_int) i, md5.md5_words[0]);

        if (i == 0)
            printf("chk1=%#08x chk2=%#08x invoke=%#08x request_id=%#08x\n",
                   (u_int) chk1, (u_int) chk2,
                   (u_int) invoke, (u_int) request_id);
    }
#endif

    if (pdu != (PE) 0)
        pe_free(pdu);

    if (mi->mi_type == MI_CREATE_RES) {
	CMISCreateRes* cr = &mi -> mi_createres.result;

	printf("Create Result - id %#08x, ", (u_int) mi -> mi_createres.id);

	if (mi->mi_createres.id !=
#if defined(SECURITY)
            ENCODE_ID(request_id, md5.md5_words[0])
#else
            request_id
#endif
            ) {
	    // should not happen for a correctly behaving agent
	    printf("Create result with wrong request id received!\n");
	    mi_free(mi);
	    return DONE;
	}

	if (cr->cr_class.mid_global)
            printf("class %s ", oid2name(cr->cr_class.mid_global, OIDPART));

	if (cr->cr_inst.mn_dn) {
	    char* inst = dn2str(cr->cr_inst.mn_dn);
	    printf("instance {%s} ", inst);
	    free(inst);
	}

	if (cr->cr_time)
            printf("time %s", cr->cr_time);
	printf("\n");

	for (int i = 0; i < cr->cr_nattrs; i++) {
	    char *attrid, *attrval;
	    ava2str(cr->cr_attrs[i].mp_id.mid_global, cr->cr_attrs[i].mp_val,
			&attrid, &attrval);
	    fprintf(stdout, "%s:  %s\n", attrid, attrval);
	    free(attrid); free(attrval);
	}
	printf("\n\n");
	status = DONE;
    }
    else
    if (mi->mi_type == MI_PREJECT || mi->mi_type == MI_ABORT) {
	fprintf(stderr, "mcreate: received %s, exits\n",
		mi->mi_type == MI_PREJECT ?
				"provider reject" : "association abort");
#if defined(DEBUG2)
        if (pduDump != (FILE *) 0)
            fclose(pduDump);
#endif
	exit(NOTOK);
    }
    else
    if (mi->mi_type == MI_ERROR || mi->mi_type == MI_UREJECT) {
	fprintf(stderr, "Error: %s\n", MErrString(mi->mi_errtype));
	ppCMISErrorInfo(mi->mi_errtype, &mi->mi_error.info);
	status = DONE;
    }
    else {
	printf("Unexpected mindication type: %d\n", mi->mi_type);
	status = NOTOK;
    }

    mi_free(mi);
    return status;
}
