// RMIBAgent.h	- the Remote MIB Agent class methods

// By George Pavlou, December 1991 / Thurain Tin, April 1993


#ifndef _RMIB_AGENT_
#define _RMIB_AGENT_


#ifndef _LIST_ITERATOR_
#include "ListIterator.h"
#endif

#ifndef _GENERIC_KS_
#include "GenericKS.h"
#endif

#ifndef _RMIB_UTIL_
#include "RMIButil.h"
#endif


class RMIBAgent : public KS
{
    friend class SMIBAgent;

#define UNCONNECTED	-1
#define MAXATTRS	100
#define DFLT_TIMEOUT	30
#define MAXMGRS		128

    MSAPIndication	_mi;
    static int		_invoke;
    int			_msd;
    int			_interval;
    char*		_agent;
    char*		_host;
    MN			_inst_mn;
    Bool		_listening;
    Bool		_blocking;
    Bool		_hasFilter;
    int			_nFlt;
    int			_nAll;
    MID			_efd_mid;
    MN			_efd_mn;
    NotifyEventList	_evList;
    ListIterator	_evIterator;
    RMIBCallbackList	_callbackList;
    ListIterator	_callbackIterator;
    CMISObject*		_obj;
    CMISObjectList*	_objList;
    CMISEventReportArg* _report;

    static void init_invoke ()		{ _invoke = 0; }
    static int get_invoke ()		{ return ++_invoke; }
    static int get_curId ()		{ return _invoke; }

    void   clear (Bool = False);
    int	   startListening ();
    void   oid2mid (OID, MID);
    int	   checkScope (int, int, CMISScope*);
    CMISFilter* encodeFilter (char*);
    NotifyEvent* getEvent (char*, char*, char*, char*, RMIBManager*);
    NotifyEvent* getEvent (char*, RMIBManager*);
    NotifyEvent* getEvent (CMISFilter*, RMIBManager*);
    Bool   checkAllEvent (RMIBManager*);
    int    receiveAllEvent (RMIBManager*);
    int	   stopReceiveAllEvent (RMIBManager*);
    int	   removeEvent (NotifyEvent*);
    int	   createDiscrObj (PE, MID&, MN&);
    int    deleteDiscrObj (MN);
    int	   adjustReporting ();
    int	   eventNotification (CMISEventReportArg*);
    RMIBCallback* getCallback (int);
    int	   removeCallback (RMIBCallback*);
    void   brokenAssociation ();
    int	   readMsapIndication ();
    int    adjustResult (int, CMISObject*);
    int    reportResult (int, CMISObject* = NULLCMISOBJECT);

protected:

    // for Shadow MIB

    DN	   getDiscrDn ();
    MN	   getDiscrMn ();

public:

    // management association

    int	   connect ();
    int	   connect (char*, char*);
    int    disconnect ();
    int	   reset (char* = NULLCP, char* = NULLCP);
    int	   notifyBrokenAssociation (RMIBManager*);

    // event reporting interface

    int	   receiveEvent (char*, char*, char*, char*, RMIBManager*);
    int	   stopReceiveEvent (char*, char*, char*, char*, RMIBManager*);
    int	   receiveEvent (char*, RMIBManager*);
    int	   stopReceiveEvent (char*, RMIBManager*);
    int	   receiveEvent (CMISFilter*, RMIBManager*);
    int	   stopReceiveEvent (CMISFilter*, RMIBManager*);
    int    cancelTimeAssertion (RMIBManager*);

    // for the simulated time manager

    CMISEventReportArg* getCMISEventReportArg ();

    // low-level management opreations interface

    int	   CmisGet (OID, MN, CMISScope*, CMISFilter*, External*, CMISSync,
		int, OID[], CMISObjectList*&, RMIBManager* = NULLRMIBMANAGER,
		Bool = False);

    int    CmisSet (OID, MN, CMISScope*, CMISFilter*, External*, CMISSync,
		AVAArray*, CMISObjectList*&, RMIBManager* = NULLRMIBMANAGER,
		Bool = False);

    int    CmisAction (OID, MN, CMISScope*, CMISFilter*, External*, CMISSync,
		AVA*, CMISObjectList*&, RMIBManager* = NULLRMIBMANAGER,
		Bool = False);

    int    CmisCreate (OID, MN, int, MN, External*, AVAArray*, CMISObject*&,
		RMIBManager* = NULLRMIBMANAGER);

    int    CmisDelete (OID, MN, CMISScope*, CMISFilter*, External*, CMISSync,
		CMISObjectList*&, RMIBManager* = NULLRMIBMANAGER);

    // high-level management operations interface

    int	   Get (char*, char*, int, int, char*, External*, CMISSync, char**,
		CMISObjectList*&, RMIBManager* = NULLRMIBMANAGER, Bool = False);

    int	   CancelGet (int);

    int    Set (char*, char*, int, int, char*, External*, CMISSync, AVAArray*,
		CMISObjectList*&, RMIBManager* = NULLRMIBMANAGER, Bool = False);

    int	   Action (char*, char*, int, int, char*, External*, CMISSync, AVA*,
		CMISObjectList*&, RMIBManager* = NULLRMIBMANAGER, Bool = False);

    int	   Create (char*, char*, int, char*, External*, AVAArray*, CMISObject*&,
		RMIBManager* = NULLRMIBMANAGER);

    int	   Delete (char*, char*, int, int, char*, External*, CMISSync,
		CMISObjectList*&, RMIBManager* = NULLRMIBMANAGER);

    // other member functions

    int	   getMsd ();
    Bool   isListening ();
    int	   getTimeOut ();
    void   setTimeOut (int);
    char*  getAgentName ();
    void   setAgentName (char*);
    char*  getHostName ();
    void   setHostName (char*);
    int	   getnAll ();
    int	   getnFlt ();
    int    cancelCallbacks (RMIBManager*);
    void   print ();
    static int getInvoke ();

    // provide KS callbacks to the Coordinator
    // (not to be used by the top-level application)

    int	   readCommEndpoint (int);
    int	   shutdown (int);

    // constructor/destructor

    RMIBAgent (char* = NULLCP, char* = NULLCP);
    ~RMIBAgent ();
};


// inline methods

inline void RMIBAgent::oid2mid (OID oid, MID mid)
{ mid->mid_type = MID_GLOBAL; mid->mid_global = oid; }

inline CMISFilter* RMIBAgent::encodeFilter (char* s)
{ return (s && *s != '\0') ? str2mfilter(s) : NULLMFILTER; }

inline DN RMIBAgent::getDiscrDn ()
{ return _efd_mn ? _efd_mn->mn_dn : NULLDN; }

inline MN RMIBAgent::getDiscrMn ()
{ return _efd_mn ? _efd_mn : NULLMN; }

inline int RMIBAgent::connect ()
{ return (_agent && _host) ? connect(_agent, _host) : NOTOK; }

inline CMISEventReportArg* RMIBAgent::getCMISEventReportArg ()
{ return _report; }

inline int RMIBAgent::getMsd ()
{ return _msd; }

inline Bool RMIBAgent::isListening ()
{ return _listening; }

inline int RMIBAgent::getTimeOut ()
{ return _interval; }

inline void RMIBAgent::setTimeOut (int t)
{ if (t >= DFLT_TIMEOUT) _interval = t; }

inline char* RMIBAgent::getAgentName ()
{ return _agent; }

inline void RMIBAgent::setAgentName (char* a)
{ if (_agent) free(_agent); _agent = a ? strdup(a) : NULLCP; }

inline char* RMIBAgent::getHostName ()
{ return _host; }

inline void RMIBAgent::setHostName (char* h)
{ if (_host) free(_host); _host = h ? strdup(h) : NULLCP; }

inline int RMIBAgent::getnAll ()
{ return _nAll; }

inline int RMIBAgent::getnFlt ()
{ return _nFlt; }

inline int RMIBAgent::getInvoke ()
{ return _invoke; }

inline int RMIBAgent::readCommEndpoint (int)
{ return readMsapIndication(); }

inline int RMIBAgent::shutdown (int)
{ return _msd != UNCONNECTED ? disconnect() : OK; }

inline RMIBAgent::~RMIBAgent ()
{ clear(True); }


#endif /* _RMIB_AGENT_ */
