// RMIBresult.cc - the Remote MIB result realisation

// By George Pavlou / Thurain Tin, October 1993


#if defined(__GNUG__)
#include <std.h>
#elif defined(__cplusplus)
#include <stdlib.h>
#endif

#include "msap.h"
#include "Error.h"
#include "ListIterator.h"
#include "RMIBresult.h"


void CMISObject::init (OID oid, DN dn, char* time,
	AVAArray* attrs, AVA* ava, int id, int op)
{
    _oid = oid ? oid : NULLOID;
    _strClass = oid ? oid2name(oid, OIDPART) : NULLCP;
    _dn = dn ? dn : NULLDN;
    _strInst = dn ? dn2str(dn) : NULLCP;
    _strTime = time ? time : NULLCP;
    _attrs = attrs ? attrs : NULLAVAARRAY;
    _ava = ava ? ava : NULLAVA;
    _error = NULLAVA;
    _invoke = id;
    _operation = op;
}


CMISObject::CMISObject (OID oid, DN dn, char* time,
	AVAArray* attrs, AVA* ava, CMISErrors error, int id, int op)
{
    init(oid, dn, time, attrs, ava, id, op);
    _error = new AVA(error);
}


CMISObject::CMISObject (OID oid, DN dn, char* time,
	AVAArray* attrs, AVA* ava, AVA* error, int id, int op)
{
    init(oid, dn, time, attrs, ava, id, op);
    _error = error;
}


CMISObject::~CMISObject ()
{
    if (_oid) oid_free(_oid);
    if (_dn) dn_free(_dn);
    if (_strInst) free(_strInst);
    if (_strTime) free(_strTime);
    if (_attrs) {
	_attrs->clear();
	delete(_attrs);
    }
    if (_ava) delete(_ava);
    if (_error) delete _error;
}


void CMISObject::print ()
{
    advise(NULLCP, "\nclass: %s\ninstance: {%s}",
	   _strClass, _strInst);

    if (_strTime)
	advise(NULLCP, "operation time: %s", _strTime);

    if (_attrs) {
	int n = _attrs->getCount();
	for (register int i = 0; i < n; i++) {
	    AVA* ava = (AVA*) _attrs->get(i);
	    if (ava) {
		char* cp;
		Attr* value = ava->getValue();
		advise(NULLCP, "%s: %s%s", ava->getName(),
		    value ? "" : "error: ",
		    value ? (cp = value->print()) : ava->getErrorStr());
		if (cp) free(cp);
	    }
	}
    }

    if (_ava) {
	char* cp;
	Attr* value = _ava->getValue();
	advise(NULLCP, "%s: %s%s", _ava->getName(), value ? "" : "error: ",
	    value ? (cp = value->print()) : _ava->getErrorStr());
	if (cp) free(cp);
    }

    if (_error->getError()) {
	char* cp;
	advise(NULLCP, "%s", cp = _error->print());
	free(cp);
    }
}


void CMISObjectList::print ()
{
    int n = this->getCount();

    if (n) {
	advise(NULLCP, "\nreply id:  %d", _invoke);
	advise(NULLCP, "number of managed objects = %d", n);
	ListIterator iterator(this);
	CMISObject* obj;
	while (obj = (CMISObject*) iterator.getNext())
	    obj->print();
    }
}
