// RMIButil.cc - the Remote MIB util realisation

// By George Pavlou / Thurain Tin, October 1993


#ifdef __GNUG__
#include <std.h>
extern "C" { int  strncasecmp (const char*, const char*, size_t); }
#elif __cplusplus
#include <stdlib.h>
#endif

#include "msap.h"
#include "Error.h"
#include "RMIButil.h"


int strcmp1 (char* s1, char* s2)
{
    if (!s1 || !s2) {
	return (!s1 && !s2) ? OK : NOTOK;
    }
    return (strcmp(s1, s2) == 0) ? OK : NOTOK;
}


/*
DN str2dn_strip (char* s)
{
    DN dn = NULLDN;

    if (s) {
	char* cp = s;
	// OSI MIB or OSI-fied SNMP MIB
	if (cp && (strncasecmp(cp, "systemId=", 9) == 0 ||
		   strncasecmp(cp, "sysName=", 8) == 0)) {
	    cp = strchr(cp, '@');
	    if (cp) ++cp;
	}

	GString gs = "";
	if (cp && *cp != '\0') {
	    gs = cp;
	    dn = str2dn((char*) gs);
#ifdef DEBUG
	    if (!dn)
		advise(NULLCP, "\ninvalid DN: %s", s);
#endif
	}
    }
    return dn;
}
*/


CMISFilter* buildFilter (char* objClass,
	char* objInst, char* eventType, char* eventTime)
{
    CMISFilter* f;
    char buf[256], *cp = buf;

    sprintf(cp++, " ");

    if (objClass) {
	sprintf(cp, "(managedObjectClass=%s)", objClass);
	cp += strlen(cp);
    }

    if (objInst) {
	if (objClass) {
	    sprintf(cp++, "&");
	    buf[0] = '(';
	}
	sprintf(cp, "(managedObjectInstance=%s)", objInst);
	cp += strlen(cp);
    }

    if (eventType) {
	if (objClass || objInst) {
	    sprintf(cp++, "&");
	    buf[0] = '(';
	}
	sprintf(cp, "(eventType=%s)", eventType);
	cp += strlen(cp);
    }

    if (eventTime) {
	if (objClass || objInst || eventType) {
	    sprintf(cp++, "&");
	    buf[0] = '(';
	}
	sprintf(cp, "%s", eventTime);
	cp += strlen(cp);
    }

    if (buf[0] == '(')
	sprintf(cp, ")");

    return (f = str2mfilter(buf)) ? f : NULLMFILTER;
}


CMISObject* process_getres (int id, CMISGetRes* gr, CMISErrors error)
{
    if (!gr) return NULLCMISOBJECT;

    AVAArray*
    attrs = gr->gr_nattrs ? new AVAArray(gr->gr_nattrs) : NULLAVAARRAY;

    for (register int i = 0; i < gr->gr_nattrs; i++) {
	struct CMISParam* g_ava = &gr->gr_attrs[i].ga_ava;
	AVA* ava = new AVA(g_ava->mp_id.mid_global,
			g_ava->mp_val ? g_ava->mp_val : NULLPE,
			g_ava->mp_val ? NULLMERROR : gr->gr_attrs[i].ga_error);

	if (AVA::createError()) {
	    advise(NULLCP, "AVA creation failed for oid %s",
		sprintoid(g_ava->mp_id.mid_global));
	    delete ava;
	}
	else {
	    g_ava->mp_id.mid_global = NULLOID;
	    if (g_ava->mp_val) g_ava->mp_val = NULLPE;
	    attrs->add(ava);
	}
    }

    CMISObject* obj = new CMISObject(gr->gr_class.mid_global,
			gr->gr_inst.mn_dn, gr->gr_time, attrs,
			NULLAVA, error, id, M_GET);

    gr->gr_class.mid_global = NULLOID;
    gr->gr_inst.mn_dn = NULLDN;
    gr->gr_time = NULLCP;
    return obj;
}


CMISObject* process_setres (int id, CMISSetRes* sr, CMISErrors error)
{
    if (!sr) return NULLCMISOBJECT;

    AVAArray*
    attrs = sr->sr_nattrs ? new AVAArray(sr->sr_nattrs) : NULLAVAARRAY;

    for (register int i = 0; i < sr->sr_nattrs; i++) {
	struct CMISParam* s_ava = &sr->sr_attrs[i].sa_ava;
	AVA* ava = new AVA(s_ava->mp_id.mid_global,
			s_ava->mp_val ? s_ava->mp_val : NULLPE,
			s_ava->mp_val ? NULLMERROR : sr->sr_attrs[i].sa_error);

	if (AVA::createError()) {
	    advise(NULLCP, "AVA creation failed for oid %s",
		sprintoid(s_ava->mp_id.mid_global));
	    delete ava;
	}
	else {
	    s_ava->mp_id.mid_global = NULLOID;
	    if (s_ava->mp_val) s_ava->mp_val = NULLPE;
	    attrs->add(ava);
	}
    }

    CMISObject* obj = new CMISObject(sr->sr_class.mid_global,
			sr->sr_inst.mn_dn, sr->sr_time, attrs,
			NULLAVA, error, id, M_SET);

    sr->sr_class.mid_global = NULLOID;
    sr->sr_inst.mn_dn = NULLDN;
    sr->sr_time = NULLCP;
    return obj;
}


CMISObject* process_actionres (int id, CMISActionRes* ar)
{
    if (!ar) return NULLCMISOBJECT;

    struct CMISParam* ar_reply = &ar->ar_reply;
    AVA* ava = NULLAVA;

    if (ar_reply) {
	// add the "Reply" suffix to the action OID
	massage_actionReplyOID(ar_reply->mp_id.mid_global);

	ava = new AVA(ar_reply->mp_id.mid_global, ar_reply->mp_val);

	if (AVA::createError()) {
	    advise(NULLCP, "AVA creation failed for oid %s",
		sprintoid(ar_reply->mp_id.mid_global));
	    delete ava;
	}
	else {
	    ar_reply->mp_id.mid_global = NULLOID;
	    ar_reply->mp_val = NULLPE;
	}
    }

    CMISObject* obj = new CMISObject(ar->ar_class.mid_global,
			ar->ar_inst.mn_dn, ar->ar_time, NULLAVAARRAY,
			ava, NULLMERROR, id, M_ACTION);

    ar->ar_class.mid_global = NULLOID;
    ar->ar_inst.mn_dn = NULLDN;
    ar->ar_time = NULLCP;
    return obj;
}


CMISObject* process_createres (int id, CMISCreateRes* cr)
{
    if (!cr) return NULLCMISOBJECT;

    AVAArray*
    attrs = cr->cr_nattrs ? new AVAArray(cr->cr_nattrs) : NULLAVAARRAY;

    for (register int i = 0; i < cr->cr_nattrs; i++) {
	struct CMISParam* a_ava = &cr->cr_attrs[i];

	AVA* ava = a_ava->mp_val
		? new AVA(a_ava->mp_id.mid_global, a_ava->mp_val, NULLMERROR)
		: NULLAVA;

	if (AVA::createError()) {
	    advise(NULLCP, "AVA creation failed for oid %s",
		sprintoid(a_ava->mp_id.mid_global));
	    delete ava;
	}
	else {
	    a_ava->mp_id.mid_global = NULLOID;
	    a_ava->mp_val = NULLPE;
	    if (ava) attrs->add(ava);
	}
    }

    CMISObject* obj = new CMISObject(cr->cr_class.mid_global,
			cr->cr_inst.mn_dn, cr->cr_time, attrs,
			NULLAVA, NULLMERROR, id, M_CREATE);

    cr->cr_class.mid_global = NULLOID;
    cr->cr_inst.mn_dn = NULLDN;
    cr->cr_time = NULLCP;
    return obj;
}


CMISObject* process_deleteres (int id, CMISDeleteRes* dr)
{
    if (!dr) return NULLCMISOBJECT;

    CMISObject* obj = new CMISObject(dr->dr_class.mid_global,
			dr->dr_inst.mn_dn, dr->dr_time, NULLAVAARRAY,
			NULLAVA, NULLMERROR, id, M_DELETE);

    dr->dr_class.mid_global = NULLOID;
    dr->dr_inst.mn_dn = NULLDN;
    dr->dr_time = NULLCP;
    return obj;
}


CMISObject* process_procfailure (int id, CMISProcessingFailure* pf)
{
    if (!pf) return NULLCMISOBJECT;

    CMISObject* obj = new CMISObject(pf->pf_class.mid_global,
			pf->pf_inst.mn_dn, NULLCP, NULLAVAARRAY,
			NULLAVA, NULLMERROR, id);

    pf->pf_class.mid_global = NULLOID;
    pf->pf_inst.mn_dn = NULLDN;
    return obj;
}


CMISObject* process_actionerr (int id, CMISActionError* ae)
{
    if (!ae) return NULLCMISOBJECT;

    CMISObject* obj = new CMISObject(ae->ae_class.mid_global,
			ae->ae_inst.mn_dn, ae->ae_time, NULLAVAARRAY,
			NULLAVA, ae->ae_error, id, M_ACTION);

    ae->ae_class.mid_global = NULLOID;
    ae->ae_inst.mn_dn = NULLDN;
    ae->ae_time = NULLCP;
    return obj;
}


CMISObject* process_deleteerr (int id, CMISDeleteError* de)
{
    if (!de) return NULLCMISOBJECT;

    CMISObject* obj = new CMISObject(de->de_class.mid_global,
			de->de_inst.mn_dn, de->de_time, NULLAVAARRAY,
			NULLAVA, de->de_error, id, M_DELETE);

    de->de_class.mid_global = NULLOID;
    de->de_inst.mn_dn = NULLDN;
    de->de_time = NULLCP;
    return obj;
}


CMISObject* process_errinfo (int id, CMISErrorInfo* info,
		CMISErrors error)
{
    if (!info) return NULLCMISOBJECT;

    OID oid = NULLOID;
    DN dn = NULLDN;

    switch (error) {
    case m_noSuchObjectClass:
	oid = info->ei_noSuchObjectClass.mid_global;
	info->ei_noSuchObjectClass.mid_global = NULLOID;
	break;

    case m_noSuchObjectInstance:
	dn = info->ei_noSuchObjectInstance.mn_dn;
	info->ei_noSuchObjectInstance.mn_dn = NULLDN;
	break;

    case m_invalidObjectInstance:
	dn = info->ei_invalidObjectInstance.mn_dn;
	info->ei_invalidObjectInstance.mn_dn = NULLDN;
	break;

    case m_duplicateManagedObjectInstance:
	dn = info->ei_duplicateObjectInstance.mn_dn;
	info->ei_duplicateObjectInstance.mn_dn = NULLDN;
	break;

    case m_noSuchReferenceObject:
	dn = info->ei_noSuchReferenceObject.mn_dn;
	info->ei_noSuchReferenceObject.mn_dn = NULLDN;
	break;

    case m_syncNotSupported:
    case m_invalidFilter:
    case m_noSuchAttribute:
    case m_invalidAttributeValue:
    case m_noSuchAction:
    case m_processingFailure:
    case m_noSuchEventType:
    case m_noSuchArgument:
    case m_invalidArgumentValue:
    case m_invalidScope:
    case m_missingAttributeValue:
    case m_classInstanceConflict:
    case m_complexityLimitation:
    case m_noSuchInvokeId:
	break;

    default:
	break;
    }

    CMISObject* obj = new CMISObject(oid, dn, NULLCP,
			NULLAVAARRAY, NULLAVA, error, id);
    return obj;
}
