//
// Copyright (c) 1991 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// Example.cc - a simple manager application using
//	the asynchronous RMIB services. Connect to an SMA agent.

// By Thurain Tin, July 1994


#ifdef __GNUG__
#define SIGHANDLER	SignalHandler
#elif  __cplusplus
#ifndef AIXV3
#define SIGHANDLER	SIG_PF
#else
#define SIGHANDLER	void
#endif
#endif

#include <stdlib.h>
#include <signal.h>
#include "msap.h"
#include "Error.h"
#include "Coordinator.h"
#include "Syntaxes.h"
#include "RMIBAgent.h"
#include "TestMgr.h"


main (int argc, char** argv)
{
    if (argc < 3)
	error(NULLCP, "usage: example <agent> <host>");

    // load the syntaxes, see also initialiseSyntaxes(3n)
    if (initialiseSyntaxes(NULLCP, allSyntaxes) == NOTOK)
	error(NULLCP, "unable to initialise syntaxes");

    // instantiate one Coordinator for central listening
    Coordinator coordinator;

    // instantiate one RMIBAgent
    RMIBAgent agent(argv[1], argv[2]);

    // attempt connection to the remote agent
    if (agent.connect() == NOTOK)
	error(NULLCP, "unable to establish association to %s at %s",
	      agent.getAgentName(), agent.getHostName());

    // set signal handlers
    signal(SIGINT, (SIGHANDLER) Coordinator::terminate);
    signal(SIGQUIT, (SIGHANDLER) Coordinator::terminate);
    signal(SIGTERM, (SIGHANDLER) Coordinator::terminate);

    // instantiate one TestMgr knowledge source
    TestMgr testMgr(&agent);

    // start listening...
    printf("listening...\n");
    coordinator.listen();
}
