//
// Copyright (c) 1995 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// TclRMIB.h

// By Thurain Tin, October 1995


#ifndef _TCL_RMIB_
#define _TCL_RMIB_ 1


#ifndef _RMIB_AGENT_
#include "RMIBAgent.h"
#endif
 
#ifndef _TCL_RMIB_INIT_
#include "TclRMIBInit.h"
#endif


DECLARE_CALLBACK(RmibAgentCmd)
DECLARE_CALLBACK(RmibManagerCmd)
DECLARE_CALLBACK(RmibDestroyCmd)
DECLARE_CALLBACK(RmibAgentInfoCmd)
DECLARE_CALLBACK(RmibConnectCmd)
DECLARE_CALLBACK(RmibDisconnectCmd)
DECLARE_CALLBACK(RmibGetCmd)
DECLARE_CALLBACK(RmibCancelGetCmd)
DECLARE_CALLBACK(RmibSetCmd)
DECLARE_CALLBACK(RmibActionCmd)
DECLARE_CALLBACK(RmibCreateCmd)
DECLARE_CALLBACK(RmibDeleteCmd)
DECLARE_CALLBACK(RmibNotifyCmd)
DECLARE_CALLBACK(RmibScheduleWakeUpsCmd)
DECLARE_CALLBACK(RmibCancelWakeUpsCmd)


class TclRMIBAgent : public RMIBAgent
{
public:

    int	id;

    static int compare (TclRMIBAgent* a, TclRMIBAgent* b)
    { return a->id == b->id ? OK : NOTOK; }

    TclRMIBAgent (int i, char* a, char* h) : RMIBAgent (a, h)
    { id = i; }

    ~TclRMIBAgent ()
    { }
};

#define NULLTCLRMIBAGENT ((TclRMIBAgent*) 0)


class TclRMIBAgentList : public List
{
protected:

    void deleteElem (void* v)
    { if (v) delete (TclRMIBAgent*) v; }

    int compare (void* a, void* b)
    { return TclRMIBAgent::compare((TclRMIBAgent*) a, (TclRMIBAgent*) b); }
};

#define NULLTCLRMIBAGENTLIST ((TclRMIBAgentList*) 0)


class TclRMIBManager : public RMIBManager
{
public:

    int   id;
    char  Id[16];

    char* eproc;
    char* rproc;
    char* bproc;
    char* nproc;

    struct Tcl_Interp* interp;

    static int compare (TclRMIBManager* a, TclRMIBManager* b)
    { return a->id == b->id ? OK : NOTOK; }

    int eventNotification (char*, char*, char*, char*, Attr*, RMIBAgent*);
    int generalResult (CMISObjectList*, RMIBAgent*);
    int singleGetResult (CMISObject*, RMIBAgent*);
    int createResult (CMISObject*, RMIBAgent*);
    int brokenAssociation (RMIBAgent*);
    int noTimeAssertion (RMIBAgent*);

    TclRMIBManager (int i, Tcl_Interp* intrp, char* e, char* r,
		char* b, char* n)
    {
      sprintf(Id, "%d", id = i); interp = intrp;
      eproc = e ? strdup(e) : NULLCP; rproc = r ? strdup(r) : NULLCP;
      bproc = b ? strdup(b) : NULLCP; nproc = n ? strdup(n) : NULLCP;
    }

    ~TclRMIBManager ()
    {
      if (eproc) free(eproc); if (rproc) free(rproc);
      if (bproc) free(bproc); if (nproc) free(nproc);
    }
};

#define NULLTCLRMIBMANAGER ((TclRMIBManager*) 0)


class TclRMIBManagerList : public List
{
protected:

    void deleteElem (void* v)
    { if (v) delete (TclRMIBManager*) v; }

    int compare (void* a, void* b)
    { return TclRMIBManager::compare((TclRMIBManager*) a,(TclRMIBManager*) b); }
};

#define NULLTCLRMIBMANAGERLIST ((TclRMIBManagerList*) 0)


class WprocList : public List
{
protected:

    void deleteElem (void* v)
    { if (v) delete (char *) v; }

    int compare (void* a, void* b)
    { return strcmp((char *) a, (char *) b) == 0 ? OK : NOTOK; }
};


class TclKS : public KS
{
    WprocList _list;

public:

    struct Tcl_Interp* interp;

    int schedule (int, char*);
    int cancel (char*);
    void cancelAll ();
    int wakeUp (char*);

    TclKS ();
};

inline int TclKS::schedule (int i, char* p)
{ _list.append(strdup(p)); return scheduleWakeUps(i, p, False); }

inline int TclKS::cancel (char* p)
{ return cancelWakeUps(p); }

inline void TclKS::cancelAll ()
{ char* p; while (p = (char *) _list.get()) { cancelWakeUps(p); free(p); } }

inline int TclKS::wakeUp (char* p)
{ Tcl_VarEval(this->interp, p, (char *) NULL); return OK; }

inline TclKS::TclKS ()
{ interp = (Tcl_Interp *) NULL; }


#endif /* _TCL_RMIB_ */
