/*
 * Copyright (c) 1994 Technical Research Centre of Finland
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* Routines to establish a management association */

/*
 * By Matti Hallivuori, June 1994
 */
#include "files.c"
#include "xmsap.h"
#include "minitialise.h"
#include <stdlib.h>
#include <unistd.h>


OM_IMPORT(MP_CMIS_PKG)
OM_IMPORT(MP_C_COMMUNICATIONS_ERROR)
OM_IMPORT(MP_C_CMIS_SERVICE_ERROR)
OM_IMPORT (MP_C_ENTITY_NAME)
OM_IMPORT (MP_C_AVA)
OM_IMPORT (MP_C_DS_RDN)
OM_IMPORT (MP_C_DS_DN)

OM_IMPORT (MP_C_FORM1)
OM_IMPORT (MP_C_FORM2)
OM_IMPORT (MP_C_PRESENTATION_ADDRESS)
OM_IMPORT (MP_C_SESSION)
OM_IMPORT (MP_C_CONTEXT)

OM_EXPORT (MP_C_ACSE_ARGS)
OM_EXPORT (MP_C_CMIP_ASSOC_ARGS)
OM_IMPORT (MP_C_ACSE_ARGS)
OM_IMPORT (MP_C_CMIP_ASSOC_ARGS)


OM_IMPORT (MP_C_NAME_STRING)
OM_IMPORT (MP_C_NETWORK_ADDRESS)
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define _INCLUDE_HPUX_SOURCE

xom_head  xom_heads[NR_HEADS];
extern  MP_waiting_sessions session_list[NR_HEADS];
int sesCnt=0;
int gfununits;
char * gcontext=0;
struct PSAPaddr * gmyAddr;
AEI gmyTitle;
extern int IamAgent;
extern int IamSun;
int NrOfSessions=0;
int mngrNr = -1;
extern int xom_filedesc(OM_private_object);

struct ManagerAet {
  char pa[99];
  char sa[99];
  char ta[99];
  char na[99];
  int  nl;
} ManagerAet;
static struct ManagerAet managerAet; /*passes manager address from M_InitialiseReq() to managerAddress() when an event is being sent*/

char globalNameDomain[256];	/* hopefully enough */
static char *context = "management";

/* LINTLIBRARY */

OM_workspace gWorkspace;


int pub2privW(OM_workspace workspace, OM_descriptor* pub, OM_private_object* priv)
{
OM_object_identifier class;
OM_object source;
OM_private_object       res;
OM_return_code          ret;
OM_value_position       cnt;
OM_object        pubobj;
  class = pub[0].value.string;
  source = &pub[0];
  ret = om_create (class,OM_FALSE,workspace, &res);
  if (ret) 
    goto fail;
  ret = om_put (res,OM_INSERT_AT_BEGINNING,pub,NULL,0,0);
  if (ret)
    goto fail;
  ret = om_get (res,OM_NO_EXCLUSIONS,NULL,OM_FALSE,OM_ALL_VALUES,
                    OM_ALL_VALUES,&pubobj,&cnt);
  if (ret)
    goto fail;
  pub = &pubobj[0];
  *priv = res;
  return OK;

 fail:
  fprintf(stderr,"Conversion to private object failed err %d \n",ret);
  return NOTOK;
}



void dumpPublic(char* what, OM_descriptor* pub) {
#if defined(sun)
  int i = 0;
  printf("%s\n", what);
  if (!pub) {printf("  0-pointer\n"); return;}
  for (i = 0; pub[i].type; i++) {
    printf("  %s[%d]={type=%6d,syntax=%4d,v.i=%9d,v.o.o=%9d}\n", what, i, pub[i].type, pub[i].syntax, pub[i].value.integer, pub[i].value.object.object);
    if (pub[i].syntax == OM_S_OBJECT) {
      if (pub[i].value.object.object==0)
	printf("***** warning: syntax==OM_S_OBJECT but value.object.object==0\n");
      else
	dumpPublic("  sub", pub[i].value.object.object);
    }
  }
#endif
}

void printPublic(char* what, OM_descriptor* pub) {
#if defined(sun)
  OM_private_object pri;
  if (!pub) return;
  /*dumpPublic(what, pub);*/
  pub2privW(gWorkspace, pub, &pri);
  printf("%s\n", what);
  OMPexamin(pri, 2, 0);
#endif  
}

void printPrivate(char* what, OM_private_object pri) {
#if defined(sun)
  /*dumpPublic(what, pri);*/
  printf("%s\n", what);
  OMPexamin(pri, 2, 0);
#endif  
}


#define scan_char(cp,ch)  while ((*cp) && (*cp++ != (ch)))

/* persistent copy of xom  string */
static keep_xom_string(desc)
OM_descriptor *desc;
{
  char *c;
  c  = (char *) malloc(desc->value.string.length);
  memcpy(c,desc->value.string.elements,desc->value.string.length);
  desc->value.string.elements = c;
}

/* NOTE this routine is outside life time memory management 
 * - release memory by explicitly */
static OM_descriptor *  create_ava_obj(id,val)
char *id, *val;
{
void *pv;
PE pe=NULLPE;
int syntax;
OM_descriptor *desc;
const OM_descriptor ava[] = {
  OM_OID_DESC(OM_CLASS, MP_C_AVA),
{ MP_NAMING_ATTRIBUTE_ID, OM_S_OBJECT_IDENTIFIER_STRING, {0, 0} },
{ MP_NAMING_ATTRIBUTE_VALUE, OM_S_ENCODING_STRING,{0, 0} }, /* was OCTET */
  OM_NULL_DESCRIPTOR, 
 };

   desc = (OM_descriptor *) malloc(sizeof(ava));
   memcpy(desc,ava,sizeof(ava));
   assign_oid_string(&desc[1],id);
   keep_xom_string(&desc[1]);
/* value is type any defined by ID - parsing and encoding to ber */
   syntax = oid2syntax2(str2oid(id));
   if (syntax) {
     pv = get_syntax_table(syntax)->s_parse(val);
     if (pv) {
       pe = get_syntax_table(syntax)->s_encode(pv);
     }
   }
   if (pe) {
     assign_ber(&desc[2],pe);
     pe_free(pe);
     keep_xom_string(&desc[2]);
   } else {
     fprintf(stderr,"syntax not found for %s \n",id);
   }
   return desc;    
}

void update_xom_avas(aei)
AEI aei;
{
  char * cp1, *ava_id, *ava_val;
  cp1 = aei2pre_stub(aei);
  if ((cp1) && (strlen(cp1))) {
    OM_descriptor * avas[256];
    int cnt=0;
    scan_char(cp1,'/');
  loop:
/*  creating AVAs */         
    ava_id = cp1;
    scan_char(cp1,'=');
    cp1[-1] = 0;
    ava_val = cp1;
    scan_char(cp1,'/');      
    cp1[-1] = 0;
    avas[cnt++] =  create_ava_obj(ava_id,ava_val);
    if (*cp1) {
      goto loop;
    }
   xom_heads[0].ds_rdn = create_ds_rdn_obj(cnt, avas);
  } else {
    xom_heads[0].ds_rdn = 0;
  }
}

OM_descriptor *create_name_string_obj(str)
char *str;
{
OM_descriptor *desc;
const OM_descriptor                name_string[] = {
   OM_OID_DESC(OM_CLASS, MP_C_NAME_STRING),
 { MP_NAME_STRING, OM_S_IA5_STRING, {0, 0} },
   OM_NULL_DESCRIPTOR,
 };
  desc = (OM_descriptor *) om_malloc(sizeof(name_string));
  memcpy(desc,name_string,sizeof(name_string));
  assign_octet_string(&desc[1],str);
  return desc;
}


OM_descriptor * create_ip_address_obj(str)
void *str;
{
OM_descriptor *desc;
const OM_descriptor                ip_address[] = {
   OM_OID_DESC(OM_CLASS, MP_C_NETWORK_ADDRESS),
 { MP_IP_ADDRESS, OM_S_OCTET_STRING, {0, 0} },
   OM_NULL_DESCRIPTOR,
  };
  desc = (OM_descriptor *) om_malloc(sizeof(ip_address));
  memcpy(desc,ip_address,sizeof(ip_address));
  assign_byte_string(&desc[1],(char *)str,4);
  return desc;
}
static OM_descriptor *  create_entity_name_obj(str)
char *str;
{
OM_descriptor *desc=0;
const OM_descriptor                e_name[] = {
   OM_OID_DESC(OM_CLASS, MP_C_ENTITY_NAME),
 { MP_ENTITY, OM_S_PRINTABLE_STRING, {0, 0} },
   OM_NULL_DESCRIPTOR,
 };

   if (str) {
     desc = (OM_descriptor *) om_malloc(sizeof(e_name));
     memcpy(desc,e_name,sizeof(e_name));
     assign_octet_string(&desc[1],str);
   }
   return desc;    
}

OM_descriptor  *pa2xomAddress(Addr,aei)
struct PSAPaddr *Addr;
AEI aei;
{
/* Depending on the stack the addresses are:  
 * Presentation              CMIS
 * Network                   CMOT
 *  The network address may contain additional instance definig AVAs.
 */

  OM_descriptor *desc;
  char * aei_str;
  if (aei) {
    update_xom_avas(aei);
  }
  if(Addr->un_pa.pa_un_selector[0]) {
/*  OSI address - cp contains it*/
    char *na, *ta, *sa, *pa;
    struct NSAPaddr *ns;
    int nl;
    ns = Addr->pa_addr.sa_addr.ta_addrs;
    nl = ns->na_addrlen;
    na = ns->na_address;
    pa = Addr->un_pa.pa_un_selector;
    sa = Addr->pa_addr.un_sa.sa_un_selector;
    ta = Addr->pa_addr.sa_addr.un_ta.ta_un_selector;
    desc = create_pe_obj(na,pa,sa,ta,nl);
  } else {
/*  Internet */
    struct in_addr addr;
    aei_str = Addr->pa_addr.sa_addr.ta_addrs->na_un.un_na_tcp.na_tcp_domain;
    addr.s_addr = inet_network(aei_str);
    desc  = create_ip_address_obj(&addr); /* NOTE correct */
  }
  return desc;
}


#define copy_pe_row(l,r,str) \
   {if (string_defined(str)) { desc[l] = pe[r]; \
       assign_octet_string(&desc[l++],str);} \
       r++; }\

static OM_descriptor *  create_pe_obj(na,pa,sa,ta,nlen)
char *na, *pa, *sa, *ta;
int nlen;
{
int r = 2,l = 1;
OM_descriptor *desc;
const OM_descriptor                pe[] = {
   OM_OID_DESC(OM_CLASS, MP_C_PRESENTATION_ADDRESS),
 { MP_N_ADDRESSES, OM_S_OCTET_STRING, {0, 0} },
 { MP_P_SELECTOR, OM_S_OCTET_STRING, {0, 0} },
 { MP_S_SELECTOR, OM_S_OCTET_STRING, {0, 0} },
 { MP_T_SELECTOR, OM_S_OCTET_STRING, {0, 0} },
   OM_NULL_DESCRIPTOR,
  };

   desc = (OM_descriptor *) om_malloc(sizeof(pe));
   memcpy(desc,pe,2*DSCSIZE);
   assign_byte_string(&desc[l++],na,nlen);
   copy_pe_row(l,r,pa)    /* presentatation selector */
   copy_pe_row(l,r,sa)    /* session */
   copy_pe_row(l,r,ta)    /* transport */
   desc[l] = pe[5];     /* null descriptor */
   return desc;    
}
char *myHostName()
{
  static char lhost[100];
  int i=0;

  if (gethostname(lhost,100)){
    fprintf (stderr,"local host has no name !! \n");
    return 0;
  }
  while ((lhost[i]) && (lhost[i++] != '.'));
  if (lhost[i]) {
    lhost[--i] = 0;
  }
  return lhost;
}

OM_descriptor  *myAddress()
{
/* NOTE recode to take care of rfc 1006 (tcp ip addr) */
  AEI aei;
  char name[100], *host;
  struct PSAPaddr *pa;
  host = myHostName();
  printf("Using manager number:%d\n",mngrNr);
  sprintf(name,"mngr%d",mngrNr);
  if ((aei = (AEI) str2aei_stub(host,name)) == NULLAEI) {
    return 0;
  }
  if (( pa = aei2addr_stub(aei)) == NULLPA) {
    fprintf(stderr, "aei translation failed for %s at %s \n",name?name:"",host?host:"");
    return 0;
  }
  return pa2xomAddress(pa,NULLAEI);
}

OM_descriptor  *managerAddress(msd)
int msd;
{
/* NOTE recode to take care of rfc 1006 (tcp ip addr) */
  OM_descriptor *desc;
  char *na, *ta, *sa, *pa;
  int nl;

    char qua[22];
    char des[99];
    int i;
    char tmp;

  /*managerAet is set in M_InitialiseReq(); 950713*/
  pa = managerAet.pa;
  sa = managerAet.sa;
  ta = managerAet.ta;
  na = managerAet.na;
  nl = managerAet.nl;

  desc = create_pe_obj(na,pa,sa,ta,nl);
  return desc;
}

OM_descriptor  *pa2xomTitle(aei)
AEI aei;
{
  OM_descriptor *desc;
  char * aei_str;
  aei_str = aei2serv_stub(aei);
  desc  = create_entity_name_obj(aei_str);
  return desc;
}




OM_descriptor *  create_session_obj(fun,context,myTitle, myAddr,Title,Addr)
int fun;
char* context; 
AEI  myTitle, Title; 
struct PSAPaddr *myAddr, *Addr;
{

int l = 1;
int flag = 0;
char *c, cb[100];
OM_descriptor *desc,*aargs, *cargs, *obj;


const OM_descriptor  acseargs[] = {
OM_OID_DESC(OM_CLASS, MP_C_ACSE_ARGS),
 { MP_RESPONDER_TITLE, OM_S_OBJECT, {0, 0} },
 { MP_RESPONDER_ADDRESS, OM_S_OBJECT, {0, 0} },
   OM_NULL_DESCRIPTOR,
  };

const OM_descriptor cmipassocargs[] = {
OM_OID_DESC(OM_CLASS, MP_C_CMIP_ASSOC_ARGS),
 { MP_CMIS_FUNCTIONAL_UNITS, OM_S_INTEGER, {0, 0} },
   OM_NULL_DESCRIPTOR,
  };

const OM_descriptor  session[] = {
OM_OID_DESC(OM_CLASS, MP_C_SESSION),
 { MP_FILE_DESCRIPTOR, OM_S_INTEGER, {MP_NO_VALID_FILE_DESCRIPTOR,0} },
 { MP_REQUESTOR_TITLE, OM_S_OBJECT, {0, 0} },
 { MP_REQUESTOR_ADDRESS, OM_S_OBJECT, {0, 0} },
 { MP_ACSE_ARGS, OM_S_OBJECT, {0,0 } },
 { MP_CMIP_ASSOC_ARGS, OM_S_OBJECT, {0, 0} },
 { MP_ROLE, OM_S_INTEGER, {0, 0} },
   OM_NULL_DESCRIPTOR,
  };
aargs = (OM_descriptor *) om_malloc(sizeof(acseargs));
memcpy(aargs,acseargs,DSCSIZE);
if ((Title) && (!IamSun) && (obj =  pa2xomTitle(Title)) ) {
    aargs[l] = acseargs[1];
    aargs[l++].value.object.object = obj;
    flag =1;
  }
  if ((Addr) && !(IamAgent) && (obj=pa2xomAddress(Addr,Title))){
    aargs[l] = acseargs[2];
    aargs[l++].value.object.object = obj;
    flag = 1;
  }
if ((xom_heads[0].n_addr) && (IamAgent)) {
    aargs[l] = acseargs[2];
    aargs[l++].value.object.object = managerAddress(0);
    flag = 1;
  }

aargs[l] = acseargs[3];


cargs = (OM_descriptor *) om_malloc(sizeof(cmipassocargs));
memcpy(cargs,cmipassocargs,2*DSCSIZE);
cargs[1].value.integer = fun;
cargs[2] = cmipassocargs[2]; 

desc = (OM_descriptor *) om_malloc(sizeof(session));
memcpy(desc,session,2*DSCSIZE);
l = 2;
NrOfSessions++;
  if ((string_defined(context))  && !(IamSun)) {
    desc[l] = session[2];
    if (IamAgent) {
      c = context;
    } else {
      sprintf(cb,"manager%d",NrOfSessions);
      c = cb;
    }
    desc[l++].value.object.object= create_entity_name_obj(c);
  }
  if (myAddr) {
    desc[l] = session[3];
    desc[l++].value.object.object = pa2xomAddress(myAddr,myTitle);
  } else {
    if (IamSun) {
      desc[l] = session[3];
      desc[l++].value.object.object = myAddress();
    }
  }
  if (flag) {
    desc[l] = session[4];
    desc[l++].value.object.object = aargs;
  }
  desc[l] = session[5];
  desc[l++].value.object.object = cargs;

  desc[l] = session[7];     /* null descriptor */
  return desc;    
}

static void assign_oid_string(oct,str)
OM_descriptor *oct;
char *str;
{
  OID oid;
  oid = str2oid(str);
  assign_oid(oct,oid);
}  

OM_private_object create_private_context(msd,type)
int msd,type;
{
  OM_return_code  om_return;
  OM_descriptor  *context;
  OM_private_object private_context;

  om_return = om_create (MP_C_CONTEXT, OM_TRUE, xom_heads[msd].workspace, &private_context);
  if (om_return != OM_SUCCESS) {
     printf("om_create context failed \n");
     return NULL;
  }
  context = (type == 0) ?
       create_context_obj(TRUE,0,MP_T_CONFIRMED,0,NULLPA,NULLAEI,0):
       create_context_obj(TRUE,0,MP_T_NON_CONFIRMED,0,NULLPA,NULLAEI,0);
  om_return = om_put (private_context, OM_REPLACE_ALL, context, NULL, 0, 0);
  if (om_return != OM_SUCCESS) {
    (void)om_delete(private_context);
    printf("om_put context failed \n");
    return NULL;
  }
  return private_context;
}

int initial_get(msd,mi)
int msd;
MSAPIndication* mi;
{
   MIDentifier m_class;
   MName       inst;
   xom_heads[msd].context = create_private_context(msd,0);
   xom_heads[msd].context1 = create_private_context(msd,1);

/* Establish end-to-end connection by making a dummy mp_get request */
   m_class.mid_type = MID_GLOBAL;
   m_class.mid_un.mid_un_global =  NULLOID;
   inst.mn_type = MN_LOCALDN;
   inst.mn_un.mn_un_nonspec= "";  /* this prevents error message */
   if (M_Get(msd,0,&m_class, &inst, NULLMSCOPE, NULLMFILTER,
       NULLMACCESS, NULLMSYNC,GET_NOATTRS,NULLMID, mi)== NOTOK) {
       goto fail;
    }
   if ((M_WaitReq(msd,50,mi)) == NOTOK) {
     goto fail;
   }
   return OK;

 fail:
/* release the last session */
   xom_terminate(msd,mi);
   return NOTOK;
}

/*  M_InitialiseReq - Calling is equivalent to a M-ASSOCIATE.REQUEST
 * a succesful return is equivalent to a M-ASSOCIATE.CONFIRMATION
 *
 * NOTE Following parameters are currently not used/supported
 *  protvrsn  CMIP version
 *  access    access security
 *  info      user information
 * NOTE MSAPConnect structure is left practically empty because the
 *  information is not available.
 * NOTE reason for aborted communication is not usually available, error
 * status is kept same all the time.
 * NOTE event forwarding handling requires that agent used AEI2msd
 * and the manager M_InitialiseReq.
 */


int  M_InitialiseReq (callingTitle, callingAddr, calledTitle, calledAddr,
			context, protvrsn, fununits, access, info, mc, mi)
AEI callingTitle, calledTitle;
char* context; 
struct PSAPaddr *callingAddr, *calledAddr;
int protvrsn, fununits;                    /* NOTE protvrsn not used */
External *access, *info;                   /* NOTE not used currently */
MSAPConnect* mc;
MSAPIndication* mi;
{
/*  Multible connections are achieved via the usage of xom_heads table. */
    MP_feature *feature_list;
    OM_workspace            workspace;
    OM_private_object       session;
    OM_descriptor  *        new_session;
    MP_status               status;
    struct PSAPaddr *pa, pas;
    PE pe;
    int problem,msd, fd,reTries=0;
#if defined(sun)
    IamSun = 1;
#endif

    if (IamAgent) { /*950713*/
      if (!calledAddr) {
	printf("***** Error in M_InitialiseReq(): NULL calledAddr\n");
	return NOTOK;
      }
      /*save manager address to be used during event reporting*/
      strcpy(managerAet.pa, calledAddr->un_pa.pa_un_selector);
      strcpy(managerAet.sa, calledAddr->pa_addr.un_sa.sa_un_selector);
      strcpy(managerAet.ta, calledAddr->pa_addr.sa_addr.un_ta.ta_un_selector);
      managerAet.nl = calledAddr->pa_addr.sa_addr.ta_addrs[0].na_un.un_na_nsap.na_nsap_addrlen;
      memcpy(managerAet.na, calledAddr->pa_addr.sa_addr.ta_addrs[0].na_un.un_na_nsap.na_nsap_address, managerAet.nl);

      mc->mc_sd = 1;
      return OK;
    }

    if (calledTitle == NULLAEI) {
      LOCAL_ERROR("Called title missing");
    }
    if (calledAddr == NULLPA) {
      LOCAL_ERROR("Called addres missing");
    } 
    pe = pe_cpy(calledTitle->aei_ap_title);
    pas = *calledAddr;
    pa = &pas;

    if (0 == sesCnt) {
      workspace = mp_initialize();
      feature_list = create_feature_list(&MP_CMIS_PKG, NULL);
      status = mp_negotiate(feature_list, workspace);
      if (status != MP_SUCCESS) {
         problem = parse_statusCode(status,mi);
         fprintf(stderr,"mp_negotiate: error  %d\n", problem);
         return NOTOK;
       } else {
         if (feature_list[0].response != OM_TRUE) {
            fprintf(stderr,"mp_negotiate: didn't get the CMIS package\n");
            LOCAL_ERROR("CMIS package unavailable");
        }
      }
      gWorkspace = workspace;
    } else {
      workspace = gWorkspace;
    }
    NrOfSessions = sesCnt;
    mngrNr = getNewNumber(); 
    if (mngrNr == -1)
      return NOTOK;

 newSes:
    new_session =  create_session_obj(fununits,context,callingTitle, 
                                      callingAddr,calledTitle,calledAddr);
/*  open the session */
 newTry:
    status = mp_bind(new_session, workspace, &session);
    if (status != MP_SUCCESS) {
       problem = parse_statusCode(status,mi);
       reTries++;
       if (reTries < 10) {
	 sleep(1);
	 goto newTry;
       }
       fprintf(stderr,"mp_bind: error %d\n",problem);
       om_free(new_session);
       return NOTOK;
    }
    memset(mc,0,sizeof(MSAPConnect));
    fd = xom_filedesc(session);
    session_list[sesCnt].bound_session = session;
    session_list[sesCnt+1].bound_session = 0;
    sesCnt++;
    mc -> mc_sd = sesCnt; /*fd + NrOfSessions;*/
    xom_heads[mc->mc_sd].session = session;
    xom_heads[mc->mc_sd].workspace = workspace;
    xom_heads[mc->mc_sd].calledTitle =pe;
    xom_heads[mc->mc_sd].context = create_private_context(mc->mc_sd,0);
    xom_heads[mc->mc_sd].context1 = create_private_context(mc->mc_sd,1);
    xom_heads[mc->mc_sd].ds_rdn = NULL;
    xom_heads[mc->mc_sd].ds_rdn = xom_heads[0].ds_rdn;
    xom_heads[mc->mc_sd].fd = fd;
    xom_heads[mc->mc_sd].sesCnt = sesCnt-1;
    if (initial_get(mc->mc_sd,mi) == NOTOK) {
      return NOTOK;
    }
    mc -> mc_info.mai_protvrsn = mc -> mc_info.mai_funits = 0;
    mc -> mc_info.mai_access = mc -> mc_info.mai_info = NULL;
    return OK;
}

/*  M_Init - Called when program invoked to recapture context.
 *  A successful return is equivalent to a M-ASSOCIATE.INDICATION
 *
 * NOTE -- the internal semantics are now totally changed. The connection
 * is actually tried out only after first management operation - not 
 * before it. We must first receive the first (management) xmp package
 * to initialice data structures. Also we must take care to forward the
 * data to actual user. Agent side can not (for obvious reasons) use 
 * multible addresses without some not yet standardized negotiation
 * process. Therefore ONLY ONE incoming session is currently possible.
 * Multiple sessions are simulated by using addresses in context for event
 * sieve requests (one session per request). 
 *
 * M_Init is currently called in one of the following cases
 * 1) there is not yet an active connection and incoming call is found
 *    in select/mp_wait
 * 2) incoming call is captured and found to contain efd (packet is put aside
 *    and M_Init is called as soon as convenient)
 */
 /* ARGSUSED */
 int  M_Init (vecp, vec, ms, mi)
 int vecp;
 char** vec;
 MSAPStart* ms;
 MSAPIndication* mi;
 { 
  int fd;
  memset(ms,0,sizeof(MSAPStart));
  if (!xom_heads[0].session) {
     fprintf(stderr,"agent side initialization fails, session not ok \n");
     return NOTOK;
   }
   fd = xom_filedesc(xom_heads[0].session);
   sesCnt++;
   ms -> ms_sd = sesCnt;
   xom_heads[ms->ms_sd].session = xom_heads[0].session;
   xom_heads[ms->ms_sd].workspace = xom_heads[0].workspace;
   xom_heads[ms->ms_sd].calledTitle = xom_heads[0].calledTitle;
   xom_heads[ms->ms_sd].ln = xom_heads[0].ln; 
   xom_heads[ms->ms_sd].n_addr   = xom_heads[0].n_addr;
   xom_heads[ms->ms_sd].p_select = xom_heads[0].p_select;
   xom_heads[ms->ms_sd].s_select = xom_heads[0].s_select;
   xom_heads[ms->ms_sd].t_select = xom_heads[0].t_select;
   xom_heads[ms->ms_sd].context = create_private_context(ms->ms_sd,0);
   xom_heads[ms->ms_sd].context1 = create_private_context(ms->ms_sd,1);
   xom_heads[ms->ms_sd].ds_rdn = NULL;
   xom_heads[ms->ms_sd].fd = fd;
   xom_heads[ms->ms_sd].sesCnt = 0;
   xom_heads[ms->ms_sd].mi = xom_heads[0].mi;
   xom_heads[0].mi = 0;
   return OK;
}


/*  M_InitialiseResp  -  Respond to a connection request,
 *  equivalent to M-ASSOCIATE.RESPONSE
 *
 * In order to keep XOM/XMP interface compatible with both Osimis
 * and foreign XOM/XMP solutions some hacking is necessary.
 * Osimis belives it has end to end connection BEFORE any management
 * actions happen. XOM/XMP assumes no such connection at that time.
 * Due to changed protocol stack M_InitialiseResp is currently
 * without any real substance. This is however a good place to add
 * some address negotiation if need arises.
 */
/* ARGSUSED */

int  M_InitialiseResp (ms, status, respTitle, respAddr,
			context, protvrsn, fununits, access, info, mi)
MSAPStart * ms;
int status;
AEI respTitle;
struct PSAPaddr * respAddr;
char * context;
int protvrsn, fununits;
External * access, * info;
MSAPIndication * mi;
{
    return OK;
}

/*  m_initialise - higher level friendly interface to M_InitialiseReq */

int  m_initialise (agent, rhost,options)
char* agent;
char* rhost;
MInitOption **options;
{
    MSAPConnect  mcs;
    MSAPIndication  mis;
    AEI  aei;
    struct PSAPaddr *pa;

    if ((aei = (AEI) str2aei_stub(rhost, agent)) == NULLAEI) {
#if defined(SOLARIS)
       if (rhost && agent)
	fprintf(stderr, "%s-%s: unknown application entity title\n",
			rhost, agent);
       else 
	fprintf(stderr, "unknown application entity title\n");
#endif
       return NOTOK;
    }

    if ((pa = aei2addr_stub(aei)) == NULLPA) {
	fprintf(stderr, "aei translation failed\n");
	return NOTOK;
    }

    /* request all the functional units bar the extendedService */
    if (M_InitialiseReq(NULLAEI,  NULLPA, aei, pa, context,
		CMIP_VERSION2, 0x17, NULLEXTERN, NULLEXTERN, &mcs, &mis)
	    == NOTOK) {
          fprintf(stderr, "Connection to agent %s at host %s failed\n",
                             agent,rhost );
       mi_free(&mis);
	return NOTOK;
    }

    mi_free(&mis);
    return mcs.mc_sd;
}

/* m_initialise_manager   higher level friendly interface to M_InitialiseReq
 * with parametrs to name the manager.
 */

int  m_initialise_manager (agent, rhost,myName)
char* agent;
char* rhost;
char *myName;
{
    MSAPConnect  mcs;
    MSAPIndication  mis;
    AEI     aei;
    struct PSAPaddr *pa, *myAddr;
    struct PSAPaddr pas;
    char *lhost, *myLname;
    int i=0;

    if (!(lhost=myHostName())) {
      return NOTOK;
    }
    myLname = ((myName[0] == '.') && (myName[1] == '/')) ? &myName[2] : myName;

    if ((aei = (AEI) str2aei_stub(lhost, myLname)) == NULLAEI) {
	fprintf(stderr, "manager has no application entity title\n");
	return NOTOK;
    }

    if ((myAddr = aei2addr_stub(aei)) == NULLPA) {
	fprintf(stderr, "aei translation failed for %s at %s\n",myLname,lhost);
	return NOTOK;
    }
    pas = *myAddr;
    myAddr = &pas;

    if ((aei =  (AEI) str2aei_stub(rhost, agent)) == NULLAEI) {
	fprintf(stderr, "%s-%s: unknown application entity title\n",
			rhost, agent);
	return NOTOK;
    }

    if ((pa = aei2addr_stub(aei)) == NULLPA) {
	fprintf(stderr, "aei translation failed\n");
	return NOTOK;
    }


    /* request all the functional units bar the extendedService */
    if (M_InitialiseReq(NULLAEI,  myAddr, aei, pa, context,
		CMIP_VERSION2, 0x17, NULLEXTERN, NULLEXTERN, &mcs, &mis)
	    == NOTOK) {
       fprintf(stderr, "Connection to agent %s at host %s failed\n",
                             agent,rhost );
        mi_free(&mis);
	return NOTOK;
    }
    mi_free(&mis);
    return mcs.mc_sd;
}

/*  m_terminate - higher level friendly interface to M_TerminateReq */

int  m_terminate (msd)
int msd;
{
    MSAPIndication  mis;

    if (M_TerminateReq(msd, ACF_NORMAL, NULLPE, &mis) != OK) {
	fprintf(stderr, "Terminate request failed: reason = %s\n",
			mis.mi_abort.ma_data);
	return NOTOK;
    }
    return OK;
}


DN str2aet (str)
char* str;
{
    char  buf1[64],
	  buf[256];
    char* cp = NULLCP;

    strcpy(buf1, str);
    if (! (cp = strrchr(buf1, ':')))
	if (! (cp = strrchr(buf1, '@')))
	    cp = strrchr(buf1, '-');
    if (cp)
        *cp = '\0';

    sprintf(buf, "%s@cn=%s", globalNameDomain, buf1);
    if (cp)
	sprintf(buf+strlen(buf), "@cn=%s", cp+1);
    return str2dn(buf);
}
