/*
 * Copyright (c) 1988 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* header file for Common Management Information Service providers  */

/*
 * By George Pavlou, June 1991
 */

#ifndef _MSAP_
#define _MSAP_


#ifndef FunctionPrototypes
#if defined(__STDC__) || defined(__cplusplus) || defined(__GNUG__)
#define FunctionPrototypes
#endif
#endif

#if defined(__cplusplus) || defined(__GNUG__)
extern "C" {
#endif


#ifndef _RoSAP_
#ifndef	ICR1
#include <isode/rosap.h>
#else
#include <isode/ul/rosap.h>
#endif
#endif

#ifndef _TSAP_
#ifndef	ICR1
#include <isode/tsap.h>
#else
#include <isode/ll/tsap.h>
#endif
#endif

#ifndef _module_CMIP_defined_
#include "CMIP-types.h"
#endif

#ifndef _MPARM_
#include "mparm.h"
#endif

#ifdef  USE_X500
#ifndef _LOCATION_TRANSPARENCY_
#include "lt.h"
#endif
#endif


#define CMIP_VERSION1	    0x1			    /* protocol version 1 */
#define CMIP_VERSION2	    0x2			    /* protocol version 2 */
#define ACTUALCLASS_ASN_OID str2oid("2.9.3.2.3.42") /* actual class OID   */
#define DEFAULT_OID	    str2oid("0.0")	    /* default or dummy OID */


typedef struct MSAPAssocInfo
{
    int		mai_protvrsn;	/* protocol version */
    int		mai_funits;	/* functional units */
    External*	mai_access;	/* access control */
    External*	mai_info;	/* user info */
    char*	mai_bstring;	/* for pepsy use only */
    int		mai_blen;	/*  .. */

} MSAPAssocInfo;



typedef struct MSAPConnect
{
    int		mc_sd;		/* association descriptor */
    int		mc_result;
    OID		mc_context;
    AEInfo	mc_resptitle;
    MSAPAssocInfo mc_info;

} MSAPConnect;



typedef struct MSAPStart
{
    int		ms_sd;
    AEInfo	ms_calling;
    AEInfo	ms_called;
    OID		ms_context;
    MSAPAssocInfo ms_info;
    struct PSAPstart ms_start;	/* ??? */

} MSAPStart;



typedef struct MSAPTerm
{
    int		mt_reason;
    PE		mt_info;

} MSAPTerm;



typedef struct MSAPUReject
{
    int		mur_reason;
    int		mur_id;

#define MUR_SIZE	64
    char	mur_data [MUR_SIZE];

} MSAPUReject;



typedef struct MSAPPReject
{
    int		mpr_reason;
    int		mpr_id;

#define MPR_SIZE	64
    char	mpr_data [MPR_SIZE];

} MSAPPReject;



typedef struct MSAPAbort
{
    int		ma_source;
#define MA_USER		0
#define MA_PROVIDER	1
#define MA_LOCAL	2

    int		ma_reason;
#define MA_NOVRSN	30	/* no common CMIP version */
#define MA_BADPCTX	31	/* bad presentation context */
#define MA_BADARGS	32	/* bad argument */
#define MA_BADINFO	33	/* bad assoc user info */
#define MA_ROS		34	/* can't set ROS service */
#define MA_NOTREL	35	/* peer refused to release */
#define MA_ABORTED	36	/* peer aborted the association */

    union {
	External*	ma_un_info;	/* abort information */
	PE		ma_un_notrel;	/* not released information */
    } ma_un;

#define	ma_info		ma_un.ma_un_info
#define	ma_notrel	ma_un.ma_un_notrel

#define MA_SIZE	64
    char	ma_data [MA_SIZE];

} MSAPAbort;



typedef struct MSAPError
{
    int			id;
    CMISErrorInfo	info;

} MSAPError;



typedef struct MSAPEventReportInv
{
    int			id;
    CMISEventReportArg	args;

} MSAPEventReportInv;



typedef struct MSAPEventReportRes
{
    int			id;
    CMISEventReportRes	result;

} MSAPEventReportRes;



typedef struct MSAPLinkedReplyInv
{
    int			id;
    CMISLinkedReplyArg	args;

} MSAPLinkedReplyInv;



typedef struct MSAPGetInv
{
    int			id;
    CMISGetArg		args;

} MSAPGetInv;



typedef struct MSAPGetRes
{
    int			id;
    CMISGetRes		result;

} MSAPGetRes;



typedef struct MSAPSetInv
{
    int			id;
    CMISSetArg		args;

} MSAPSetInv;



typedef struct MSAPSetRes
{
    int			id;
    CMISSetRes		result;

} MSAPSetRes;



typedef struct MSAPActionInv
{
    int			id;
    CMISActionArg	args;

} MSAPActionInv;



typedef struct MSAPActionRes
{
    int			id;
    CMISActionRes	result;

} MSAPActionRes;



typedef struct MSAPCreateInv
{
    int			id;
    CMISCreateArg	args;

} MSAPCreateInv;



typedef struct MSAPCreateRes
{
    int			id;
    CMISCreateRes	result;

} MSAPCreateRes;



typedef struct MSAPDeleteInv
{
    int			id;
    CMISDeleteArg	args;

} MSAPDeleteInv;



typedef struct MSAPDeleteRes
{
    int			id;
    CMISDeleteRes	result;

} MSAPDeleteRes;



typedef struct MSAPCancelGetInv
{
    int			id;
    CMISCancelGetArg	args;

} MSAPCancelGetInv;



typedef struct MSAPEmptyRes
{
    int			id;

} MSAPEmptyRes;



typedef struct MSAPIndication
{
    int		mi_type;

#define MI_UNKNOWN	 0	/* when not initialised ("filled-in") */
#define MI_UREJECT	 1
#define MI_PREJECT	 2
#define MI_ABORT	 3
#define MI_TERM		 4
#define MI_ERROR	 5
#define MI_EVENT_REP	 6
#define MI_EVENT_REPC	 7
#define MI_EVENT_REP_RES 8
#define MI_LINKED_REPLY	 9
#define MI_GET		10
#define MI_GET_RES	11
#define MI_SET		12
#define MI_SETC		13
#define MI_SET_RES	14
#define MI_ACTION	15
#define MI_ACTIONC	16
#define MI_ACTION_RES	17
#define MI_CREATE	18
#define MI_CREATE_RES	19
#define MI_DELETE	20
#define MI_DELETE_RES	21
#define MI_CANCEL_GET	22
#define MI_EMPTY_RES	23
#define MI_CANCEL_GET_RES	MI_EMPTY_RES

    int mi_invoke; /* The ROS invokeID value */

    CMISErrors	mi_errtype;

    union {
	MSAPUReject		mi_un_ureject;
	MSAPPReject		mi_un_preject;
	MSAPAbort		mi_un_abort;
	MSAPTerm		mi_un_term;
	MSAPError		mi_un_error;
	MSAPEventReportInv 	mi_un_eventreportinv;
	MSAPEventReportRes 	mi_un_eventreportres;
	MSAPLinkedReplyInv 	mi_un_linkedreplyinv;
	MSAPGetInv		mi_un_getinv;
	MSAPGetRes		mi_un_getres;
	MSAPSetInv		mi_un_setinv;
	MSAPSetRes		mi_un_setres;
	MSAPActionInv		mi_un_actioninv;
	MSAPActionRes		mi_un_actionres;
	MSAPCreateInv		mi_un_createinv;
	MSAPCreateRes		mi_un_createres;
	MSAPDeleteInv		mi_un_deleteinv;
	MSAPDeleteRes		mi_un_deleteres;
	MSAPCancelGetInv	mi_un_cancelgetinv;
	MSAPEmptyRes		mi_un_emptyres;
    } mi_un;

#define mi_ureject		mi_un.mi_un_ureject
#define mi_preject		mi_un.mi_un_preject
#define mi_abort		mi_un.mi_un_abort
#define mi_term			mi_un.mi_un_term
#define mi_error		mi_un.mi_un_error
#define mi_eventrepinv		mi_un.mi_un_eventreportinv
#define mi_eventrepres		mi_un.mi_un_eventreportres
#define mi_linkedreplyinv	mi_un.mi_un_linkedreplyinv
#define mi_getinv		mi_un.mi_un_getinv
#define mi_getres		mi_un.mi_un_getres
#define mi_setinv		mi_un.mi_un_setinv
#define mi_setres		mi_un.mi_un_setres
#define mi_actioninv		mi_un.mi_un_actioninv
#define mi_actionres		mi_un.mi_un_actionres
#define mi_createinv		mi_un.mi_un_createinv
#define mi_createres		mi_un.mi_un_createres
#define mi_deleteinv		mi_un.mi_un_deleteinv
#define mi_deleteres		mi_un.mi_un_deleteres
#define mi_cancelgetinv		mi_un.mi_un_cancelgetinv
#define mi_emptyres		mi_un.mi_un_emptyres

} MSAPIndication;


typedef struct MInitOption_s
{
    int  optionId;
    void *optionInfo;

} MInitOption;

/* MInitOptions - all #defines are of the form MIO_<optionName> */


#define M_ACCEPT ACS_ACCEPT

#define M_EVENT_REP	0
#define M_EVENT_REPC	1
#define M_LINKED_REPLY	2
#define M_GET		3
#define M_SET		4
#define M_SETC		5
#define M_ACTION	6
#define M_ACTIONC	7
#define M_CREATE	8
#define M_DELETE	9
#define M_CANCEL_GET	10


#define M_EventRep(a, b, c, d, e, f, g, h) \
	M_EventRepAux(a, M_EVENT_REP, b, c, d, e, f, g, h)
#define M_EventRepConf(a, b, c, d, e, f, g, h) \
	M_EventRepAux(a, M_EVENT_REPC, b, c, d, e, f, g, h)
#define M_Set(a, b, c, d, e, f, g, h, i, j, k) \
	M_SetAux(a, M_SET, b, c, d, e, f, g, h, i, j, k)
#define M_SetConf(a, b, c, d, e, f, g, h, i, j, k) \
	M_SetAux(a, M_SETC, b, c, d, e, f, g, h, i, j, k)
#define M_Action(a, b, c, d, e, f, g, h, i, j, k) \
	M_ActionAux(a, M_ACTION, b, c, d, e, f, g, h, i, j, k)
#define M_ActionConf(a, b, c, d, e, f, g, h, i, j, k) \
	M_ActionAux(a, M_ACTIONC, b, c, d, e, f, g, h, i, j, k)
#define M_WaitReq(_fd, _wait, _mi) \
        M_WaitReqAux(_fd, _wait, _mi, (PE *) 0)

#define	get_rdn_type(a)		((a)->rdn_at->oa_ot.ot_oid)

#define utcts2asctime(s)	asctime(ut2tm(str2utct((s), strlen((s)))))
#define gents2asctime(s)	asctime(ut2tm(str2gent((s), strlen((s)))))
#define	utct2asctime(u)		asctime(ut2tm(u))

#define	ACTION_INFO	1
#define	ACTION_REPLY	2

#define	ava2str_info(a, b, c, d)	ava2str_aux(a, b, c, d, ACTION_INFO)
#define	ava2str_reply(a, b, c, d)	ava2str_aux(a, b, c, d, ACTION_REPLY)

#define	str2ava_info(a, b)		str2ava_aux(a, b, ACTION_INFO)
#define	str2ava_reply(a, b)		str2ava_aux(a, b, ACTION_REPLY)

#define	massage_actionInfoOID(a)	massage_actionOID_aux(a, ACTION_INFO)
#define	massage_actionReplyOID(a)	massage_actionOID_aux(a, ACTION_REPLY)

#ifndef	IsodeFunctionPrototypes
typedef IFP	ENCODE_FNX;
typedef IFP	DECODE_FNX;
typedef IFP	PARSE_FNX;
typedef IFP	PRINT_FNX;
typedef IFP	COPY_FNX;
typedef IFP	COMPARE_FNX;
typedef IFP	GETELEM_FNX;
typedef IFP	GETNEXT_FNX;
typedef IFP	FREE_FNX;
#endif

#define	rdn2str(a)		attrv2str((caddr_t) a, (PRINT_FNX) rdn_print)
#define	dn2str(a)		attrv2str((caddr_t) a, (PRINT_FNX) dn_print)


#ifdef FunctionPrototypes

/* M-EVENT-REPORT.REQUEST (conf./non-conf.) */
int	M_EventRepAux (int, int, int, MID, MN, MID, char*, PE,
			MSAPIndication*);

/* M-EVENT-REPORT.RESPONSE */
int	M_EventRepRes (int, int, MID, MN, char*, CMISParam*,
			CMISErrors, CMISErrorInfo*, MSAPIndication*);

/* M-GET.REQUEST */
int	M_Get (int, int, MID, MN, CMISScope*, CMISFilter*,
			External*, CMISSync, int, MID, MSAPIndication*);

/* M-GET.RESPONSE */
int	M_GetRes (int, int, int, MID, MN, char*, int, CMISGetAttr*,
			CMISErrors, CMISErrorInfo*, MSAPIndication*);

/* M-SET.REQUEST (confirmed/non-conf.) */
int	M_SetAux (int, int, int, MID, MN, CMISScope*, CMISFilter*, External*,
			CMISSync, int, CMISSetAttr*, MSAPIndication*);

/* M-SET.RESPONSE */
int	M_SetRes (int, int, int, MID, MN, char*, int, CMISSetAttr*,
			CMISErrors, CMISErrorInfo*, MSAPIndication*);

/* M-ACTION.REQUEST (confirmed/non-conf.) */
int	M_ActionAux (int, int, int, MID, MN, CMISScope*, CMISFilter*,
			External*, CMISSync, MID, PE, MSAPIndication*);

/* M-ACTION.RESPONSE */
int	M_ActionRes (int, int, int, MID, MN, char*, CMISParam*,
			CMISErrors, CMISErrorInfo*, MSAPIndication*);

/* M-CREATE.REQUEST */
int	M_Create (int, int, MID, MN, int, MN, External*,
			int, CMISParam*, MSAPIndication*);

/* M-CREATE.RESPONSE */
int	M_CreateRes (int, int, MID, MN, char*, int, CMISParam*,
			CMISErrors, CMISErrorInfo*, MSAPIndication*);

/* M-DELETE.REQUEST */
int	M_Delete (int, int, MID, MN, CMISScope*, CMISFilter*,
			External*, CMISSync, MSAPIndication*);

/* M-DELETE.RESPONSE */
int	M_DeleteRes (int, int, int, MID, MN, char*,
			CMISErrors, CMISErrorInfo*, MSAPIndication*);

/* M-CANCEL-GET.REQUEST */
int	M_CancelGet (int, int, int, MSAPIndication*);

/* M-CANCEL-GET.RESPONSE */
int	M_CancelGetRes (int, int,
			CMISErrors, CMISErrorInfo*, MSAPIndication*);

/* M-WAIT.REQUEST (pseudo) */
int	M_WaitReqAux (int, int, MSAPIndication*, PE *);


/* CMIS A-ASSOCIATE.REQUEST */
int	M_InitialiseReq (AEI, struct PSAPaddr*, AEI, struct PSAPaddr*,
			char*, int, int, External*, External*,
			MSAPConnect*, MSAPIndication*);

/* CMIS A-ASSOCIATE.INDICATION */
int	M_Init (int, char**, MSAPStart*, MSAPIndication*);

/* CMIS A-ASSOCIATE.RESPONSE */
int	M_InitialiseResp (MSAPStart*, int, AEI, struct PSAPaddr*, char*,
			int, int, External*, External*, MSAPIndication*);

/* CMIS A-RELEASE.REQUEST */
int	M_TerminateReq (int, int, PE, MSAPIndication*);

/* CMIS A-RELEASE.RESPONSE */
int	M_TerminateResp (int, int, int, PE, MSAPIndication*);

/* CMIS A-ABORT.REQUEST */
int	M_AbortReq (int, External*, MSAPIndication*);


/* higher level management association control */

        /* Final arg is a NULL terminated array of MInitOption */
int	m_initialise (char*, char*, MInitOption **);
int	m_terminate (int);


/* Checking Routines */

int	check_scope (CMISScope*);
int	check_filter (CMISFilter*);


/* Freeing Routines */

void  mc_free (MSAPConnect*);
void  ms_free (MSAPStart*);
void  mt_free (MSAPTerm*);
void  ma_free (MSAPAbort*);
void  mid_free (MID);
void  mn_free (MN);
void  mparm_free (CMISParam*);
void  ava_free (Ava*);
void  mfitem_free (CMISFilterItem*);	/* frees both data and structures */
void  mfitemdata_free (CMISFilterItem*);
void  mfilterdata_free (CMISFilter*);
void  mfilter_free (CMISFilter*);	/*  .. */
void  meventreparg_free (CMISEventReportArg*);
void  meventrepres_free (CMISEventReportRes*);
void  mgetarg_free (CMISGetArg*);
void  mgetres_free (CMISGetRes*);
void  msetarg_free (CMISSetArg*);
void  msetres_free (CMISSetRes*);
void  mactionarg_free (CMISActionArg*);
void  mactionres_free (CMISActionRes*);
void  mcreatearg_free (CMISCreateArg*);
void  mcreateres_free (CMISCreateRes*);
void  mdeletearg_free (CMISDeleteArg*);
void  mdeleteres_free (CMISDeleteRes*);
void  mactionerror_free (CMISActionError*);
void  mdeleteerror_free (CMISDeleteError*);
void  mprocfailure_free (CMISProcessingFailure*);
void  mlinkedreplyarg_free (CMISLinkedReplyArg*);
void  merrorinfo_free (CMISErrors, CMISErrorInfo*);
void  mi_free (MSAPIndication*);


/* Printing Routines */

char* MOperString (int);
char* MErrString (CMISErrors);

char* sprintmodifyop (CMISModifyOp);
char* sprintscope (CMISScope*);
char* sprintsync (CMISSync);

char* sprintmfilter (const CMISFilter*);
char* sprintmfilteritem (const CMISFilterItem*);
char* sprintmfiltersubstrings (const CMISFilterSubstrings*);

typedef void (*VoidFunc)();
int	initialiseSyntaxes (const char*, VoidFunc*);
char*	attrv2str (void*, PRINT_FNX);
int	ava2str (OID, PE, char**, char**);
int	ava2str_aux (OID, PE, char**, char**, int);
int	massage_actionOID_aux (OID, int);


/* Parsing Routines */

CMISScope*  str2scope (char*);
CMISSync    str2sync (char*);

CMISFilter*		str2mfilter (const char*);
CMISFilterItem*		str2mfilteritem (const char*);
CMISFilterSubstrings*	str2mfiltersubstrings (const char*);

Ava*	str2ava (const char*, /*const*/ char*);
Ava*	str2ava_aux (char*, char*, int);

/* Builds a structure of type External from its arguments */

External* external_build (int, char*, int, PE, struct qbuf*);


/* various... */

int	oid_equal (OID, OID);	/* compares two OIDs for equality */
int	pe2mem (PE, char**, int*);
int	mem2pe (PE*, char*, int);
int	pe2stream (PE, FILE*);
int	stream2pe (PE*, FILE*);
void	do_timing (char*);

struct type_SMI_AETitle;
int aet2msd(struct type_SMI_AETitle*);

#else	/* Old-style C function prototypes */


int	M_EventRepAux ();	/* M-EVENT-REPORT.REQUEST (conf./non-conf.) */
int	M_EventRepRes ();	/* M-EVENT-REPORT.RESPONSE */
int	M_Get ();		/* M-GET.REQUEST */
int	M_GetRes ();		/* M-GET.RESPONSE */
int	M_SetAux ();		/* M-SET.REQUEST (confirmed/non-conf.) */
int	M_SetRes ();		/* M-SET.RESPONSE */
int	M_ActionAux ();		/* M-ACTION.REQUEST (confirmed/non-conf.) */
int	M_ActionRes ();		/* M-ACTION.RESPONSE */
int	M_Create ();		/* M-CREATE.REQUEST */
int	M_CreateRes ();		/* M-CREATE.RESPONSE */
int	M_Delete ();		/* M-DELETE.REQUEST */
int	M_DeleteRes ();		/* M-DELETE.RESPONSE */
int	M_CancelGet ();		/* M-CANCEL-GET.REQUEST */
int	M_CancelGetRes ();	/* M-CANCEL-GET.RESPONSE */
int	M_WaitReqAux ();	/* M-WAIT.REQUEST (pseudo) */

int	M_InitialiseReq ();	/* CMIS A-ASSOCIATE.REQUEST */
int	M_Init ();		/* CMIS A-ASSOCIATE.INDICATION */
int	M_InitialiseResp ();	/* CMIS A-ASSOCIATE.RESPONSE */
int	M_TerminateReq ();	/* CMIS A-RELEASE.REQUEST */
int	M_TerminateResp ();	/* CMIS A-RELEASE.RESPONSE */
int	M_AbortReq ();		/* CMIS A-ABORT.REQUEST */


/* higher level management association control */

int	m_initialise ();
int	m_terminate ();


/* Checking Routines */

int	check_scope ();
int	check_filter ();


/* Freeing Routines */

void  mc_free ();
void  ms_free ();
void  mt_free ();
void  ma_free ();
void  mid_free ();
void  mn_free ();
void  mparm_free ();
void  ava_free ();
void  mfitem_free ();
void  mfitemdata_free ();
void  mfilterdata_free ();
void  mfilter_free ();
void  meventreparg_free ();
void  meventrepres_free ();
void  mgetarg_free ();
void  mgetres_free ();
void  msetarg_free ();
void  msetres_free ();
void  mactionarg_free ();
void  mactionres_free ();
void  mcreatearg_free ();
void  mcreateres_free ();
void  mdeletearg_free ();
void  mdeleteres_free ();
void  mactionerror_free ();
void  mdeleteerror_free ();
void  mprocfailure_free ();
void  mlinkedreplyarg_free ();
void  merrorinfo_free ();
void  mi_free ();


/* Printing Routines */

char* MOperString ();
char* MErrString ();

char* sprintmodifyop ();
char* sprintscope ();
char* sprintsync ();

char* sprintmfilter ();
char* sprintmfilteritem ();
char* sprintmfiltersubstrings ();

typedef void (*VoidFunc)();
int	initialiseSyntaxes ();
char*	attrv2str ();
int	ava2str ();
int	ava2str_aux ();
int	massage_actionOID_aux ();


/* Parsing Routines */

CMISScope*  str2scope ();
CMISSync    str2sync ();

CMISFilter*             str2mfilter ();
CMISFilterItem*         str2mfilteritem ();
CMISFilterSubstrings*   str2mfiltersubstrings ();

Ava*    str2ava ();
Ava*	str2ava_aux ();


/* Builds a structure of type External from its arguments */

External* external_build ();    /* builds a structure of type External */


/* various... */

int	oid_equal ();		/* compares two OIDs for equality */
int	pe2mem ();
int	mem2pe ();
int	pe2stream ();
int	stream2pe ();
int	m_initialise ();
int	m_terminate ();

void	do_timing ();

struct type_SMI_AETitle;
int aet2msd(struct type_SMI_AETitle*);

#endif	/* FunctionPrototypes */


#if defined(__cplusplus) || defined(__GNUG__)
}
#endif

#endif
