/*
 * Copyright (c) 1988 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* lmparm.h - lightweight CMIS/P include file with management data structures */

/*
 * By George Pavlou, July 1994
 */

#ifndef _LMPARM_
#define _LMPARM_


#ifndef _MPARM_			/* many (non-L)CMIS/P arguments are re-used */

#define	M_ATTRBLOCK	20	/* for space allocation per attribute block */

#define GET_ALLATTRS	-1
#define GET_NOATTRS	0



typedef enum
{
    m_noError,

#define	CMIS_ERROR_OFFSET	1

    /* CMIS errors */

    m_noSuchObjectClass,
    m_noSuchObjectInstance,
    m_accessDenied,
    m_syncNotSupported,
    m_invalidFilter,
    m_noSuchAttribute,
    m_invalidAttributeValue,
    m_getListError,
    m_setListError,
    m_noSuchAction,
    m_processingFailure,
    m_duplicateManagedObjectInstance,
    m_noSuchReferenceObject,
    m_noSuchEventType,
    m_noSuchArgument,
    m_invalidArgumentValue,
    m_invalidScope,
    m_invalidObjectInstance,
    m_missingAttributeValue,
    m_classInstanceConflict,
    m_complexityLimitation,
    m_mistypedOperation,
    m_noSuchInvokeId,
    m_operationCancelled,

    /* CMIS errors but no ROERs - used only in AttributeError */

    m_invalidOperation,
    m_invalidOperator,

#define	ROS_ERROR_OFFSET	27

    /* ROS Invoke-problem errors */

    m_duplicateInvocation,
    m_unrecognisedOperation,
    m_mistypedArgument,
    m_resourceLimitation,

} CMISErrors;

#define NULLMERROR m_noError



typedef enum
{
    m_noModifyOp = -1,
    m_replace,
    m_addValue,
    m_removeValue,
    m_setToDefault

} CMISModifyOp;



typedef enum
{
    s_invalid = -1,
    s_bestEffort,
    s_atomic

} CMISSync;

#define NULLMSYNC s_bestEffort



typedef struct CMISScope
{
    int	sc_type;

#define Sc_BaseObject		0
#define Sc_FirstLevel		1
#define Sc_WholeSubtree		2
#define Sc_IndividualLevel	3
#define Sc_BaseToNthLevel	4

    int	sc_level;		/* for IndividualLevel or BaseToNthLevel */

} CMISScope;

#define NULLMSCOPE	((CMISScope *) 0)



typedef struct CMISCancelGetArg
{
    int		cga_getinvoke;

} CMISCancelGetArg;

#endif	/* _MPARM_ */



#ifndef	_SMI_ASN_
typedef struct StringListVal		/* for missingAttributeValue */
{
    char*		string;
    struct LMIDListVal*	next;

} StringListVal;
#endif



typedef struct LCMISParam	/* Management Parameter: */
{				/* attribute, action, event report etc. */
    char*	mp_id;
    char*	mp_val;

} LCMISParam;

#define NULLLMPARM	((LCMISParam *) 0)



typedef struct LCMISParamListVal
{
    LCMISParam		mpl_ava;	/* Management Parameter List */
    struct LCMISParamListVal*	mpl_next;

#ifndef _MPARM_
#define mpl_id	mpl_ava.mp_id
#define mpl_val	mpl_ava.mp_val
#endif

} LCMISParamListVal;



typedef struct LCMISGetAttr
{
    LCMISParam	ga_ava;
    CMISErrors	ga_error;

#ifndef _MPARM_
#define ga_id	ga_ava.mp_id
#define ga_val	ga_ava.mp_val
#endif

} LCMISGetAttr;

#define NULLLMGETATTR	((LCMISGetAttr *) 0)



typedef struct LCMISSetAttr
{
    LCMISParam	sa_ava;
    CMISErrors	sa_error;	/* used only in SetListErrors */
    CMISModifyOp sa_modify;

#ifndef _MPARM_
#define sa_id	sa_ava.mp_id
#define sa_val	sa_ava.mp_val
#endif

} LCMISSetAttr;

#define NULLLMSETATTR	((LCMISSetAttr *) 0)



/* the following structure is a "union" of CMISParam, CMISGetAttr, CMISSetAttr
 * and it is used only by pepsy i.e. not part of the CMIS API
 */
typedef struct LCMISAttr
{
    LCMISParam		ma_ava;
    CMISErrors		ma_error;
    CMISModifyOp	ma_modify;

    struct LCMISAttr*	ma_next;

#ifndef _MPARM_
#define ma_id		ma_ava.mp_id
#define ma_val		ma_ava.mp_val

#define	MA_ATTRID	1
#define	MA_ATTR		2
#define	MA_GETATTR	3
#define	MA_SETATTR	4
#endif

} LCMISAttr;

#define NULLLMATTR	((LCMISAttr *) 0)



typedef struct LCMISEventReportArg
{
    char*	ea_class;
    char*	ea_inst;
    char*	ea_time;
    char*	ea_type;
    char*	ea_info;

} LCMISEventReportArg;



typedef struct LCMISEventReportRes
{
    char*	er_class;
    char*	er_inst;
    char*	er_time;
    char*	er_reply;

} LCMISEventReportRes;



typedef struct LCMISGetArg
{
    char*	ga_class;
    char*	ga_inst;
    char*	ga_access;
    CMISSync	ga_sync;
    CMISScope	ga_scope;
    char*	ga_filter;
    int		ga_nattrs;
    char**	ga_attrs;
    LCMISAttr*	ga_pattrs;	/* for pepsy use only */

} LCMISGetArg;



typedef struct LCMISGetRes
{
    char*	gr_class;
    char*	gr_inst;
    char*	gr_time;
    int		gr_nattrs;
    LCMISGetAttr* gr_attrs;
    LCMISAttr*	gr_pattrs;	/* for pepsy use only */

} LCMISGetRes;



typedef struct LCMISSetArg
{
    char*	sa_class;
    char*	sa_inst;
    char*	sa_access;
    CMISSync	sa_sync;
    CMISScope	sa_scope;
    char*	sa_filter;
    int		sa_nattrs;
    LCMISSetAttr* sa_attrs;
    LCMISAttr*	sa_pattrs;	/* for pepsy use only */

} LCMISSetArg;



typedef struct LCMISSetRes
{
    char*	sr_class;
    char*	sr_inst;
    char*	sr_time;
    int		sr_nattrs;
    LCMISSetAttr* sr_attrs;
    LCMISAttr*	sr_pattrs;	/* for pepsy use only */

} LCMISSetRes;



typedef struct LCMISActionArg
{
    char*	aa_class;
    char*	aa_inst;
    char*	aa_access;
    CMISSync	aa_sync;
    CMISScope	aa_scope;
    char*	aa_filter;
    LCMISParam	aa_info;

} LCMISActionArg;



typedef struct LCMISActionResErr
{
    char*	are_class;
    char*	are_inst;
    char*	are_time;
    CMISErrors	are_error;
    LCMISParam	are_reply;

} LCMISActionResErr;



typedef struct LCMISCreateArg
{
    char*	ca_class;
    char*	ca_inst;
    int		ca_inst_type;
#ifndef _MPARM_
#define CA_OBJECT_INST	 1	/* not 0 and 1 as in the past */
#define CA_SUPERIOR_INST 2
#endif

    char*	ca_reference_inst;
    char*	ca_access;
    int		ca_nattrs;
    LCMISParam*	ca_attrs;
    char*	ca_pinst;	/* for pepsy use only */
    LCMISAttr*	ca_pattrs;	/*  .. */

} LCMISCreateArg;



typedef struct LCMISCreateRes
{
    char*	cr_class;
    char*	cr_inst;
    char*	cr_time;
    int		cr_nattrs;
    LCMISParam*	cr_attrs;
    LCMISAttr*	cr_pattrs;	/* for pepsy use only */

} LCMISCreateRes;



typedef struct LCMISDeleteArg
{
    char*	da_class;
    char*	da_inst;
    char*	da_access;
    CMISSync	da_sync;
    CMISScope	da_scope;
    char*	da_filter;

} LCMISDeleteArg;



typedef struct LCMISDeleteResErr
{
    char*	dre_class;
    char*	dre_inst;
    char*	dre_time;
    CMISErrors	dre_error;

} LCMISDeleteResErr;



typedef struct LCMISBaseManagedObjectId
{
    char*	bmo_class;
    char*	bmo_inst;

} LCMISBaseManagedObjectId;



typedef struct LCMISNoSuchArgument
{
    char*	nsa_class;
    char*	nsa_type;

} LCMISNoSuchArgument;



typedef struct LCMISProcessingFailure
{
    char*	pf_class;
    char*	pf_inst;
    LCMISParam	pf_error;

} LCMISProcessingFailure;



typedef struct LCMISComplexityLimitation
{
    CMISScope	cl_scope;
    char*	cl_filter;
    CMISSync	cl_sync;

} LCMISComplexityLimitation;



typedef struct LCMISErrorInfo
{
    union {
	char*			ei_un_noSuchObjectClass,
				ei_un_noSuchAttribute,
				ei_un_linkedActionAccessDenied,
				ei_un_noSuchObjectInstance,
				ei_un_invalidObjectInstance,
				ei_un_duplicateObjectInstance,
				ei_un_noSuchReferenceObject,
				ei_un_invalidFilter;
	StringListVal*		ei_un_missingAttributeValue;
	int			ei_un_noSuchInvokeId;
	LCMISParam		ei_un_invalidAttributeValue,
				ei_un_invalidArgumentValue;
	CMISScope		ei_un_invalidScope;
	CMISSync		ei_un_syncNotSupported;
	LCMISNoSuchArgument	ei_un_noSuchAction,
				ei_un_noSuchEventType,
				ei_un_noSuchArgument;
	LCMISProcessingFailure	ei_un_processingFailure;
	LCMISComplexityLimitation ei_un_complexityLimitation;
	LCMISBaseManagedObjectId ei_un_classInstanceConflict;

    } ei_un;

#ifndef _MPARM_
#define ei_noSuchObjectClass	ei_un.ei_un_noSuchObjectClass
#define ei_noSuchAttribute	ei_un.ei_un_noSuchAttribute
#define ei_linkedActionAccessDenied ei_un.ei_un_linkedActionAccessDenied
#define ei_missingAttributeValue    ei_un.ei_un_missingAttributeValue
#define ei_noSuchObjectInstance	    ei_un.ei_un_noSuchObjectInstance
#define ei_invalidObjectInstance    ei_un.ei_un_invalidObjectInstance
#define ei_duplicateObjectInstance  ei_un.ei_un_duplicateObjectInstance
#define ei_noSuchReferenceObject    ei_un.ei_un_noSuchReferenceObject
#define ei_invalidAttributeValue    ei_un.ei_un_invalidAttributeValue
#define ei_invalidScope		ei_un.ei_un_invalidScope
#define ei_invalidFilter	ei_un.ei_un_invalidFilter
#define ei_syncNotSupported	ei_un.ei_un_syncNotSupported
#define ei_noSuchInvokeId	ei_un.ei_un_noSuchInvokeId
#define ei_noSuchAction		ei_un.ei_un_noSuchAction
#define ei_noSuchEventType	ei_un.ei_un_noSuchEventType
#define ei_noSuchArgument	ei_un.ei_un_noSuchArgument
#define ei_invalidArgumentValue	ei_un.ei_un_invalidArgumentValue
#define ei_processingFailure	ei_un.ei_un_processingFailure
#define ei_complexityLimitation	ei_un.ei_un_complexityLimitation
#define ei_classInstanceConflict     ei_un.ei_un_classInstanceConflict
#endif

} LCMISErrorInfo;

#define NULLLMERRORINFO	((LCMISErrorInfo *) 0)



typedef struct LCMISLinkedReplyArg
{
    int lr_linkid;

    int lr_type;

#ifndef _MPARM_
#define LR_GET_RES	1
#define LR_GET_LISTERR	2
#define LR_SET_RES	3
#define LR_SET_LISTERR	4
#define LR_ACTION_RES	5
#define LR_PROC_FAILURE	6
#define LR_DELETE_RES	7
#define LR_ACTION_ERR	8
#define LR_DELETE_ERR	9
#endif

    union {
	LCMISGetRes		lr_un_getres;
	LCMISSetRes		lr_un_setres;
	LCMISActionResErr	lr_un_actionres;
	LCMISDeleteResErr	lr_un_deleteres;
	LCMISProcessingFailure	lr_un_procfailure;
    } lr_un;

    caddr_t			lr_genparm;	/* for pepsy use only */

#ifndef _MPARM_
#define lr_procfailure	lr_un.lr_un_procfailure
#define lr_getres	lr_un.lr_un_getres
#define lr_setres	lr_un.lr_un_setres
#endif
#define lr_actionreserr	lr_un.lr_un_actionreserr
#define lr_deletereserr	lr_un.lr_un_deletereserr

} LCMISLinkedReplyArg;


#endif
