/*
 * Copyright (c) 1988 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* Routines for filter parsing and printing */

/*
 * By Saleem N. Bhatti, July 1992
 */

#if defined(__GNUG__)
#include <std.h>
#elif defined(__cplusplus)
#include <stdlib.h>
#else
#include <stdio.h>
#endif

#include <ctype.h>
#include <string.h>

#include "msap.h"


CMISFilter *
#if defined(__STDC__) || defined(__cplusplus)
str2mfilter(const char *str)
#else
str2mfilter(str)
char *str;
#endif
{
    static
    char *strp; /* static pointer and brackets to avoid copying strings */

    register
    CMISFilter *flt, *fPrev, *fThis, *fNext;
    register
    char *start, *itemStr, *cp;
    register
    int parenFlag, fltFlag, type, i;
    char token[256]; /* I hope this is enough! */

    if (str == NULL) return(NULL);

    strp = strchr(str, '(');
    if (strp == NULL) {
#if defined(DEBUG)
	printf("str2mfilter() : \"(\" missing at \"%s\"\n", str);
#endif
	return(NULL);
    }

    type = -1;
    fPrev = fThis = fNext = NULL;
    parenFlag = 0;

    do {
	start = strp;

	/* Read the next token/item and advance the pointer */
	while((*strp != NULL) && isspace(*strp)) ++strp;

	i = sscanf(strp, " %1[(]", token);
	if (i != 1) i = sscanf(strp, " %1[)]", token);
	if (i != 1) i = sscanf(strp, " %1[&]", token);
	if (i != 1) i = sscanf(strp, " %1[|]", token);
	if (i != 1) i = sscanf(strp, " %1[!]", token);
	if (i != 1) i = sscanf(strp, " %[^()]", token); /* FilterItem read */
	if (i != 1) { /* Bad token */
#if defined(DEBUG)
	    printf("str2mfilter() : parse error at \"%s\"\n", start);
#endif
      	    if (fThis != NULL) mfilter_free(fThis);
	    return(NULL);
	}

	/* Advance pointer */
	for(cp = token; *cp != NULL; ++cp) ++strp;
	
	/* Take out trailing spaces */
	for(--cp; isspace(*cp); --cp); ++cp; *cp = NULL;

	/*
	** See what we have ...
	*/
	/* Filter start */
	if (strcmp(token, "(") == 0) {

	    /* Another filter */
	    if (parenFlag) {
		fNext = str2mfilter(strp - 1); /* need to see the "(" */
		if (fNext == NULL) {
		    if (fThis != NULL) mfilter_free(fThis);
		    return(NULL);
		}
		if (fThis == NULL) fThis = fNext;
		if (fPrev != NULL) fPrev->flt_next = fNext;
		fPrev = fNext;
	    }
	    else parenFlag = 1;

	    fltFlag = 0;
	}

	/* Filter end */
	else if (strcmp(token, ")") == 0) {
	    parenFlag = 0;
	    fltFlag = 0;
	}

/* Save on a function call ... */
#define FILTER_TYPE(ft) {\
    if (fltFlag) {\
        fprintf(stderr, "str2mfilter() : parse error at \"%s\"\n", start);\
	if (fThis != NULL) mfilter_free(fThis);\
        return(NULL);\
    }\
    if ((type != -1) && (type != ft)) {\
        fprintf(stderr, "str2mfilter() : mixing of filter types\n");\
	if (fThis != NULL) mfilter_free(fThis);\
        return(NULL);\
    }\
    else {\
        type = ft;\
        fltFlag = 1;\
    }\
}

	/* The filter types */
	else if (strcmp(token, "!") == 0) { FILTER_TYPE(Filter_Not); }
	else if (strcmp(token, "&") == 0) { FILTER_TYPE(Filter_And); }
        else if (strcmp(token, "|") == 0) { FILTER_TYPE(Filter_Or); }
	else { FILTER_TYPE(Filter_Item); itemStr = strdup(token); }

#undef FILTER_TYPE(ft)

    } while(parenFlag && (*strp != NULL));

    if (parenFlag) { /* Cannot tell when there are too many ")"s! */
#if defined(DEBUG)
	printf("str2mfilter() : too many \"(\"s\n");
#endif
	return(NULL);
    }

    /* Make it */
    flt = (CMISFilter *) malloc(sizeof(CMISFilter));
    flt->flt_type = type;
    flt->flt_next = NULL;

    if (type == Filter_Item) {

	if (strcmp(itemStr, "NULL") == 0) { /* NULL filter */
	    flt->flt_type = Filter_And; /* See [ISO ####] */
	    flt->flt_filter = NULL;
	}

	else if ((flt->flt_item = str2mfilteritem(itemStr)) == NULL) {
	    free((char *) flt);
	    free(itemStr);
	    return(NULL);
	}

	free(itemStr);
    }

    else {

	if (fThis == NULL) {
#if defined(DEBUG)
	    printf("str2mfilter() : invalid filter\n");
#endif
	    mfilter_free(flt);
	    flt = NULL;
	}

	else flt->flt_filter = fThis;
    }

    return(flt);
}

CMISFilterItem *
#if defined(__STDC__) || defined(__cplusplus)
str2mfilteritem(const char *str)
#else
str2mfilteritem(str)
char *str;
#endif
{
    register
    int i, type;
    register
    Ava *attr;
    register
    CMISFilterSubstrings *subs;
    register
    CMISFilterItem *item;

    char name[80], value[256], /* I hope this is enough! */
         op[3];

    if (str == NULL) return(NULL);

    i = sscanf((char *) str, " %[a-zA-Z0-9_-] %[=<>:] %[^\n] ",
		name, op, value);

    if (i == EOF) {
#if defined(DEBUG)
	printf("str2mfilteritem() : EOF\n");
#endif
	return(NULL);
    }

    if (i == 0) {
#if defined(DEBUG)
	printf("str2mfilteritem() : syntax error in \"%s\"\n", str);
#endif
	return(NULL);
    }

    if ((i != 1) && (i != 3)) {
#if defined(DEBUG)
	printf("str2mfilteritem() : syntax error in \"%s\"\n", str);
#endif
	return(NULL);
    }

    if (i == 1)                     type = FI_Present;
    else if (strcmp(op, "=") == 0)  type = FI_Equality;
    else if (strcmp(op, ":=") == 0) type = FI_Substrings;
    else if (strcmp(op, "<=") == 0) type = FI_GreaterOrEqual;
    else if (strcmp(op, ">=") == 0) type = FI_LessOrEqual;
    else if (strcmp(op, ":>") == 0) type = FI_SubsetOf;
    else if (strcmp(op, ":<") == 0) type = FI_SupersetOf;
    else if (strcmp(op, "><") == 0) type = FI_NonNullSetIntersect;
    else {
#if defined(DEBUG)
	printf("str2mfilteritem() : syntax error at \"%s\"\n", str);
#endif
	return(NULL);
    }

    if (i == 3) {
	if (type == FI_Substrings) {
            if ((subs = str2mfiltersubstrings(str)) == NULL) {
#if defined(DEBUG)
                printf("str2mfilteritem() : could not encode \"%s\"\n", str);
#endif
                return(NULL);
            }
        }
	else if ((attr = str2ava(name, value)) == NULL) {
#if defined(DEBUG)
	    printf("str2mfilteritem() : could not encode \"%s\"\n", str);
#endif
	    return(NULL);
	}
    }

    item = (CMISFilterItem *) malloc(sizeof(CMISFilterItem));
    item->fi_type = type;
    item->fi_attr.mp_id.mid_type = MID_GLOBAL;
    if (type == FI_Present) {
	if ((item->fi_attr.mp_id.mid_global = name2oid(name)) == NULL) {
#if defined(DEBUG)
	    printf("str2mfilteritem() : unknown name \"%s\"\n", name);
#endif
	    free((char *) item);
	    return(NULL);
	}
	item->fi_attr.mp_val = NULL;

    }
    else if (type == FI_Substrings)
	item->fi_substrings = subs;
    else {
	item->fi_attr.mp_id.mid_global = attr->ava_oid;
	item->fi_attr.mp_val = attr->ava_value;

        free(attr);	/* the contents are now in item */
    }

    return(item);
}

CMISFilterSubstrings *
#if defined(__STDC__) || defined(__cplusplus)
str2mfiltersubstrings(const char *str)
#else
str2mfiltersubstrings(str)
char *str;
#endif
{
    register
    CMISFilterSubstrings *sPrev, *sThis, *sNext;
    register
    Ava *attr;
    register
    char *cp, *end;
    register
    int type, i;
    char name[80], value[256], /* I hope this is enough! */
         op[3], substring[256];

    i = sscanf((char *) str, " %[a-zA-Z_-] %[=<>:] %[^\n] ", name, op, value);

    if (i == EOF) {
#if defined(DEBUG)
	printf("str2mfiltersubstrings() : EOF\n");
#endif
	return(NULL);
    }

    if (i != 3) {
#if defined(DEBUG)
	printf("str2mfiltersubstrings() : syntax error in \"%s\"\n", str);
#endif
	return(NULL);
    }

    if (strcmp(op, ":=") != 0) {
#if defined(DEBUG)
	printf("str2mfiltersubstrings() : unrecoginised operator \"%s\"\n", op);
#endif
	return(NULL);
    }

    if (strchr(str, '*') == NULL) {
#if defined(DEBUG)
	printf("str2mfiltersubstrings() : syntax error in \"%s\"\n", value);
#endif
	return(NULL);
    }

    end = &value[strlen(value) - 1];
    sPrev = sThis = sNext = NULL;
    type = -1;
    for(cp = value; *cp != NULL; ++cp) {

	if (*cp == '*') {

	    if ((cp != value) && (cp == strchr(value, '*')) && (sThis == NULL)) {
		type = FI_Substring_Initial;
		sscanf(value, "%[^*]", substring);
		if (cp != end) --cp;
	    }

	    else if ((cp != end) && (cp == strrchr(value, '*'))) {
		type = FI_Substring_Final;
		sscanf(cp + 1, "%[^*]", substring);
		cp += strlen(substring);
	    }

	    else {
		if (cp == end) break;

		type = FI_Substring_Any;
		sscanf(cp + 1, "%[^*]", substring); /* ### check! */
		cp += strlen(substring);
	    }

	    attr = str2ava(name, substring); /* ### free if NULL! */
	    sNext = (CMISFilterSubstrings *) malloc(sizeof(CMISFilterSubstrings));
	    sNext->fi_sub_type = type;
	    sNext->fi_sub_attrid.mid_type = MID_GLOBAL;
	    sNext->fi_sub_attrid.mid_global = attr->ava_oid;
	    sNext->fi_sub_string = attr->ava_value;
	    sNext->fi_sub_next = NULL;

            free(attr);		/* the contents are now in sNext */

	    if (sThis == NULL) sThis = sNext;
	    if (sPrev != NULL) sPrev->fi_sub_next = sNext;
	    sPrev = sNext;
	}
    }

    return(sThis);
}

char *
#if defined(__STDC__) || defined(__cplusplus)
sprintmfilter(const CMISFilter *flt)
#else
sprintmfilter(flt)
CMISFilter *flt;
#endif
{
    static
    char buffer[1024]; /* I hope this is enough! */

    static
    char *cp = NULL;
    static
    CMISFilter *thisFilter = NULL;

#if defined(__STDC__) || defined(__cplusplus)
    const
#endif
    register
    CMISFilter *fPrev, *fThis, *fNext;
#if defined(__STDC__) || defined(__cplusplus)
    const
#endif
    register
    char *type;

    if (flt == NULL) return(NULL);

    if (thisFilter == NULL) thisFilter = (CMISFilter *) flt;
    if (cp == NULL) cp = buffer;

#define CP cp = buffer + strlen(buffer)
    
    switch (flt->flt_type) {

    case Filter_Item :
	sprintf(cp, "(%s)", sprintmfilteritem(flt->flt_item)); CP;
	break;

    default :
	fPrev = fNext = NULL;
	type = NULL;
	sprintf(cp, "("); CP;
	for(fThis = flt->flt_filter; fThis != NULL; fThis = fThis->flt_next) {

	    if (fPrev == NULL) fPrev = fThis;
	    fNext = fThis->flt_next;

	    if (type == NULL) {
		switch (flt->flt_type) {
		case Filter_Not  : type = "!";   break;
		case Filter_And  : type = " & "; break;
		case Filter_Or   : type = " | "; break;
		case Filter_Item : type = " ";   break;
		default :
		    return("sprintmfilter() : bad filter (unknown type)\n");
	        }
	    }

	    /* For NOT, put "!" at beginning */
	    if ((flt->flt_type == Filter_Not) && (fPrev == fThis)) {
       		sprintf(cp, "%s", type); CP;
	    }

	    if (sprintmfilter(fThis) == NULL) return(NULL);

	    /* Print op between items */
	    if (((fPrev == fThis) || (fNext != NULL)) && (flt->flt_type != Filter_Not)) {
		sprintf(cp, "%s", type); CP;
	    }
	}

	/* "NULL" filter */
	if ((flt->flt_type == Filter_And) && (flt->flt_filter == NULL)) {
	    sprintf(cp, "NULL"); CP;
	}

	sprintf(cp, ")"); CP;
	break;
    }
#undef CP

    if (flt == thisFilter) { /* reset */
	cp = NULL;
	thisFilter = NULL;
    }

    return(buffer);
}

char *
#if defined(__STDC__) || defined(__cplusplus)
sprintmfilteritem(const CMISFilterItem *item)
#else
sprintmfilteritem(item)
CMISFilterItem *item;
#endif
{
    static
    char buffer[256]; /* I hope this is enough! */

    char *name, *value, *op, *cp;

    if (item == NULL) return(NULL);

    switch (item->fi_type) {
    case FI_Equality            : op = "=";  break;
    case FI_Substrings          : op = ":="; break;
    case FI_GreaterOrEqual      : op = "<="; break;
    case FI_LessOrEqual         : op = ">="; break;
    case FI_Present             : op = "";   break;
    case FI_SubsetOf            : op = ":>"; break;
    case FI_SupersetOf          : op = ":<"; break;
    case FI_NonNullSetIntersect : op = "><"; break;
    default :
	sprintf(buffer, "sprintmfilteritem() : bad filter item (unknown type)\n");
	return(buffer);
    }

    if (item->fi_type == FI_Substrings) {
	if ((cp = sprintmfiltersubstrings(item->fi_substrings)) == NULL) {
	    sprintf(buffer, "sprintmfilteritem() : could not pretty print substring item\n");
	    return(buffer);
	}
    }

    else if (item->fi_type != FI_Present) {
	if (ava2str(item->fi_attr.mp_id.mid_global, item->fi_attr.mp_val, &name, &value) != OK) {
	    sprintf(buffer, "sprintmfilteritem() : could not pretty print item\n");
	    return(buffer);
	}
    }

    else name = oid2name(item->fi_attr.mp_id.mid_global, OIDPART);

    if (item->fi_type == FI_Substrings)
	sprintf(buffer,"%s", cp);
    else if (item->fi_type != FI_Present) {
	sprintf(buffer, "%s %s %s", name, op, value);
	free(name); free(value);
    }
    else sprintf(buffer, "%s", name);

    return(buffer);
}

char *
#if defined(__STDC__) || defined(__cplusplus)
sprintmfiltersubstrings(const CMISFilterSubstrings *subs)
#else
sprintmfiltersubstrings(subs)
CMISFilterSubstrings *subs;
#endif
{
    static
    char buffer[256]; /* I hope this is enough! */
    register
    CMISFilterSubstrings *fsp;

    char *name, *value, *cp;

    if (subs == NULL) return(NULL);

    name = oid2name(subs->fi_sub_attrid.mid_global, OIDPART);
    sprintf(buffer, "%s := ", name);

#define CP cp = buffer + strlen(buffer)

    CP;
    for(fsp = (CMISFilterSubstrings *) subs; fsp != NULL; fsp = fsp->fi_sub_next) {

	if (ava2str(fsp->fi_sub_attrid.mid_global, fsp->fi_sub_string, &name, &value) == NOTOK) {
	    sprintf(buffer, "CMISFilterSubstrings() : could not pretty-print substring value\n");
	    return(buffer);
	}

	switch (fsp->fi_sub_type) {

	case FI_Substring_Initial :
	    sprintf(cp, "%s*", value); CP;
	    break;

	case FI_Substring_Final :
	    if (*(cp - 1) != '*') { sprintf(cp, "*"); CP; }
	    sprintf(cp, "%s", value); CP;
	    break;

	case FI_Substring_Any :
	    if (*(cp - 1) != '*') { sprintf(cp, "*"); CP; }
	    sprintf(cp, "%s*", value); CP;
	    break;

	}

#undef CP

	free(name);
	free(value);
    }

    return(buffer);
}
