/*
 * Copyright (c) 1988 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this parasraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* Management data structures freeing routines */

/*
 * By George Pavlou, April 1990
 */

#include <stdio.h>
#include "msap.h"


/* LINTLIBRARY */

void  mc_free (mc)
MSAPConnect * mc;
{
    if (mc->mc_context)
	oid_free(mc->mc_context),
	mc->mc_context = NULLOID;

    if (mc->mc_resptitle.aei_ap_title)
	pe_free( mc->mc_resptitle.aei_ap_title),
	mc->mc_resptitle.aei_ap_title = NULLPE;

    if (mc->mc_resptitle.aei_ae_qualifier)
	pe_free( mc->mc_resptitle.aei_ae_qualifier),
	mc->mc_resptitle.aei_ae_qualifier = NULLPE;

    if (mc->mc_info.mai_access)
	external_free(mc->mc_info.mai_access),
	mc->mc_info.mai_access = NULLEXTERN;

    if (mc->mc_info.mai_info)
	external_free(mc->mc_info.mai_info),
	mc->mc_info.mai_info = NULLEXTERN;
}


void  ms_free (ms)
MSAPStart * ms;
{
    AEIFREE(&ms->ms_calling);
    AEIFREE(&ms->ms_called);
    PSFREE(&ms->ms_start);

    if (ms->ms_context)
	oid_free(ms->ms_context),
	ms->ms_context = NULLOID;

    if (ms->ms_info.mai_access)
	external_free(ms->ms_info.mai_access),
	ms->ms_info.mai_access = NULLEXTERN;

    if (ms->ms_info.mai_info)
	external_free(ms->ms_info.mai_info),
	ms->ms_info.mai_info = NULLEXTERN;
}


void  mt_free (mt)
struct MSAPTerm * mt;
{
    if (mt->mt_info)
	pe_free(mt->mt_info),
	mt->mt_info = NULLPE;
}


void  ma_free (ma)
struct MSAPAbort * ma;
{
    if (ma -> ma_reason == MA_ABORTED && ma -> ma_info) {
	external_free(ma -> ma_info);
	ma -> ma_info = NULLEXTERN;
    }
    else
    if (ma -> ma_reason == MA_NOTREL && ma -> ma_notrel) {
	pe_free(ma -> ma_notrel);
	ma -> ma_notrel = NULLPE;
    }
}


void  mid_free (mid)
MID mid;
{
    if (mid -> mid_type == MID_GLOBAL && mid -> mid_global) {
	oid_free(mid -> mid_global);
	mid -> mid_global = NULLOID;
    }
}


void  mn_free (mn)
MN mn;
{
    if ((mn -> mn_type == MN_DN || mn -> mn_type == MN_LOCALDN)
		&& mn -> mn_dn) {
	dn_free(mn -> mn_dn);
	mn -> mn_dn = NULLDN;
    }
    else
    if (mn -> mn_type == MN_NONSPEC && mn -> mn_nonspec) {
	free(mn -> mn_nonspec);
	mn -> mn_nonspec = NULLCP;
    }
}


void  mparm_free (mparm)
CMISParam* mparm;
{
    mid_free(&mparm -> mp_id);
    if (mparm -> mp_val) {
	pe_free(mparm -> mp_val);
	mparm -> mp_val = NULLPE;
    }
}


void  ava_free (pava)
Ava* pava;
/* pava and NOT ava as the latter is a struct in <isode/quipu/name.h> */
{
    if (!pava)
	return;
    if (pava->ava_oid)
	oid_free(pava->ava_oid);
    if (pava->ava_value)
	pe_free(pava->ava_value);
}


void  mfitem_free (item)
CMISFilterItem * item;
{
    switch (item -> fi_type) {
    case FI_Equality:
    case FI_GreaterOrEqual:
    case FI_LessOrEqual:
    case FI_SubsetOf:
    case FI_SupersetOf:
    case FI_NonNullSetIntersect:
	mparm_free(&item -> fi_attr);
	break;

    case FI_Present:
	mid_free(&item -> fi_attr.mp_id);
	break;

    case FI_Substrings:
    {
	register CMISFilterSubstrings *substring = item->fi_substrings, *cur;
	for (; substring;) {
	    substring = (cur = substring) -> fi_sub_next;
	    mid_free(&cur -> fi_sub_attrid);
	    if (cur -> fi_sub_string)
		pe_free(cur -> fi_sub_string);
	    free ((char *) cur);
	}
    }
	break;

    default:
	break;
    }
    free ((char *) item);
}


void  mfitemdata_free (item)
CMISFilterItem * item;
{
    switch (item -> fi_type) {
    case FI_Equality:
    case FI_GreaterOrEqual:
    case FI_LessOrEqual:
    case FI_SubsetOf:
    case FI_SupersetOf:
    case FI_NonNullSetIntersect:
	mparm_free(&item -> fi_attr);
	break;

    case FI_Present:
	mid_free(&item -> fi_attr.mp_id);
	break;

    case FI_Substrings:
    {
	register CMISFilterSubstrings *cur;
	for (cur = item->fi_substrings; cur; cur = cur -> fi_sub_next) {
	    mid_free(&cur -> fi_sub_attrid);
	    if (cur -> fi_sub_string) {
		pe_free(cur -> fi_sub_string);
		cur -> fi_sub_string = NULLPE;
	    }
	}
    }
	break;

    default:
	break;
    }
}


void  mfilter_free (filter)
CMISFilter * filter;
{
    if (filter == NULLMFILTER)
	return;
    switch (filter -> flt_type) {
    case Filter_Item:
	mfitem_free (filter -> flt_item);
	break;

    case Filter_And:
    case Filter_Or:
    {
	register CMISFilter *cur, *next;
	for (cur = filter -> flt_filter; cur; cur = next) {
	    next = cur -> flt_next;
	    mfilter_free(cur);
	}
    }
	break;

    case Filter_Not:
	mfilter_free(filter -> flt_filter);
	break;

    default:
	break;
    }
    free((char*) filter);
}


void  mfilterdata_free (filter)
CMISFilter * filter;
{
    if (filter == NULLMFILTER)
	return;
    switch (filter -> flt_type) {
    case Filter_Item:
	mfitemdata_free (filter -> flt_item);
	break;

    case Filter_And:
    case Filter_Or:
    {
	register CMISFilter *cur;
	for (cur = filter -> flt_filter; cur; cur = cur -> flt_next)
	    mfilterdata_free(cur);
    }
	break;

    case Filter_Not:
	mfilterdata_free(filter -> flt_filter);
	break;

    default:
	break;
    }
}


void  meventreparg_free (ea)
CMISEventReportArg * ea;
{
    mid_free(&ea -> ea_class);
    mn_free(&ea -> ea_inst);
    if (ea -> ea_time != NULLCP)
	free(ea -> ea_time);
    mid_free(&ea -> ea_type);
    if (ea -> ea_info != NULLPE)
	pe_free(ea -> ea_info);
}


void  meventrepres_free (er)
CMISEventReportRes * er;
{
    mid_free(&er -> er_class);
    mn_free(&er -> er_inst);
    if (er -> er_time != NULLCP)
	free(er -> er_time);
    mparm_free(&er -> er_reply);
}


void  mgetarg_free (ga)
CMISGetArg * ga;
{
    register MID attr;

    mid_free(&ga -> ga_class);
    mn_free(&ga -> ga_inst);
    if (ga -> ga_access)
	external_free(ga -> ga_access);
    if (ga -> ga_filter != NULLMFILTER)
	mfilter_free(ga -> ga_filter);
    if (ga -> ga_nattrs > 0) {
	for (attr = ga -> ga_attrs; ga -> ga_nattrs--; attr++)
	    mid_free(attr);
	free((char*) ga -> ga_attrs);
    }
}


void  mgetres_free (gr)
CMISGetRes * gr;
{
    register CMISGetAttr * attr;

    mid_free(&gr -> gr_class);
    mn_free(&gr -> gr_inst);
    if (gr -> gr_time != NULLCP)
	free(gr -> gr_time);
    if (gr -> gr_nattrs > 0) {
	for (attr = gr -> gr_attrs; gr -> gr_nattrs--; attr++)
	    mparm_free(&attr->ga_ava);
	free((char*) gr -> gr_attrs);
    }
}


void  msetarg_free (sa)
CMISSetArg * sa;
{
    register CMISSetAttr * attr;

    mid_free(&sa -> sa_class);
    mn_free(&sa -> sa_inst);
    if (sa -> sa_access)
	external_free(sa -> sa_access);
    if (sa -> sa_filter != NULLMFILTER)
	mfilter_free(sa -> sa_filter);
    for (attr = sa -> sa_attrs; sa -> sa_nattrs--; attr++)
	mparm_free(&attr->sa_ava);
    free((char*) sa -> sa_attrs);
}


void  msetres_free (sr)
CMISSetRes * sr;
{
    register CMISSetAttr * attr;

    mid_free(&sr -> sr_class);
    mn_free(&sr -> sr_inst);
    if (sr -> sr_time != NULLCP)
	free(sr -> sr_time);
    for (attr = sr -> sr_attrs; sr -> sr_nattrs--; attr++)
	mparm_free(&attr->sa_ava);
    free((char*) sr -> sr_attrs);
}


void  mactionarg_free (aa)
CMISActionArg * aa;
{
    mid_free(&aa -> aa_class);
    mn_free(&aa -> aa_inst);
    if (aa -> aa_access)
	external_free(aa -> aa_access);
    if (aa -> aa_filter != NULLMFILTER)
	mfilter_free(aa -> aa_filter);
    mparm_free(&aa -> aa_info);
}


void  mactionres_free (ar)
CMISActionRes * ar;
{
    mid_free(&ar -> ar_class);
    mn_free(&ar -> ar_inst);
    if (ar -> ar_time != NULLCP)
	free(ar -> ar_time);
    mparm_free(&ar -> ar_reply);
}


void  mcreatearg_free (ca)
CMISCreateArg * ca;
{
    register int i;

    mid_free(&ca -> ca_class);
    mn_free(&ca -> ca_inst);
    mn_free(&ca -> ca_reference_inst);
    if (ca -> ca_access)
	external_free(ca -> ca_access);
    for (i = 0; i < ca -> ca_nattrs; i++)
	mparm_free(&ca->ca_attrs[i]);
    
    if (ca -> ca_nattrs)
        free((char*) ca -> ca_attrs);
}


void  mcreateres_free (cr)
CMISCreateRes * cr;
{
    register int i;

    mid_free(&cr -> cr_class);
    mn_free(&cr -> cr_inst);
    if (cr -> cr_time != NULLCP)
	free(cr -> cr_time);
    for (i = 0; i < cr -> cr_nattrs; i++)
	mparm_free(&cr->cr_attrs[i]);
    if (cr -> cr_nattrs)
        free((char*) cr -> cr_attrs);
}


void  mdeletearg_free (da)
CMISDeleteArg * da;
{
    mid_free(&da -> da_class);
    mn_free(&da -> da_inst);
    if (da -> da_access)
	external_free(da -> da_access);
    if (da -> da_filter != NULLMFILTER)
	mfilter_free(da -> da_filter);
}


void  mdeleteres_free (dr)
CMISDeleteRes * dr;
{
    mid_free(&dr -> dr_class);
    mn_free(&dr -> dr_inst);
    if (dr -> dr_time != NULLCP)
	free(dr -> dr_time);
}


void  mactionerror_free (ae)
CMISActionError * ae;
{
    mid_free(&ae -> ae_class);
    mn_free(&ae -> ae_inst);
    if (ae -> ae_time)
	free(ae -> ae_time);
    switch (ae -> ae_error) {
    case m_accessDenied:
	mid_free(&ae -> ae_accessDenied);
	break;
    case m_noSuchAction:
	mid_free(&ae -> ae_noSuchAction);
	break;
    case m_noSuchArgument:
	mid_free(&ae -> ae_noSuchArgument.nsa_class);
	mid_free(&ae -> ae_noSuchArgument.nsa_type);
	break;
    case m_invalidArgumentValue:
	mparm_free(&ae -> ae_invalidArgumentValue);
	break;
    default:
	break;
    }
}


void  mdeleteerror_free (de)
CMISDeleteError * de;
{
    mid_free(&de -> de_class);
    mn_free(&de -> de_inst);
    if (de -> de_time)
	free(de -> de_time);
}


void  mprocfailure_free (pf)
CMISProcessingFailure * pf;
{
    mid_free(&pf -> pf_class);
    mn_free(&pf -> pf_inst);
    mparm_free(&pf -> pf_error);
}


void  mlinkedreplyarg_free (lr)
CMISLinkedReplyArg * lr;
{
    switch (lr -> lr_type) {
    case LR_GET_RES:
    case LR_GET_LISTERR:
	mgetres_free(&lr -> lr_getres);
	break;

    case LR_SET_RES:
    case LR_SET_LISTERR:
	msetres_free(&lr -> lr_setres);
	break;

    case LR_ACTION_RES:
	mactionres_free(&lr -> lr_actionres);
	break;

    case LR_DELETE_RES:
	mdeleteres_free(&lr -> lr_deleteres);
	break;

    case LR_ACTION_ERR:
	mactionerror_free(&lr -> lr_actionerr);
	break;

    case LR_DELETE_ERR:
	mdeleteerror_free(&lr -> lr_deleteerr);
	break;

    case LR_PROC_FAILURE:
	mprocfailure_free(&lr -> lr_procfailure);
	break;
    }
}


void  merrorinfo_free (error, ei)
CMISErrors  error;
CMISErrorInfo * ei;
{
    switch (error) {
    case m_noSuchObjectClass:
    case m_noSuchAttribute:
	mid_free(&ei -> ei_noSuchObjectClass);
	break;

    case m_noSuchObjectInstance:
    case m_duplicateManagedObjectInstance:
    case m_noSuchReferenceObject:
    case m_invalidObjectInstance:
	mn_free(&ei -> ei_noSuchObjectInstance);
	break;

    case m_invalidFilter:
	mfilter_free(ei -> ei_invalidFilter);
	break;

    case m_invalidAttributeValue:
	mparm_free(&ei -> ei_invalidAttributeValue);
	break;

    case m_noSuchArgument:
    case m_noSuchAction:
    case m_noSuchEventType:
	mid_free(&ei -> ei_noSuchArgument.nsa_class);
	mid_free(&ei -> ei_noSuchArgument.nsa_type);
	break;

    case m_processingFailure:
	mprocfailure_free(&ei -> ei_processingFailure);
	break;

    case m_invalidArgumentValue:
	mparm_free(&ei -> ei_invalidArgumentValue);
	break;

    case m_missingAttributeValue:
	free_CMIP_AttributeIdList(ei -> ei_missingAttributeValue);
	break;

    case m_classInstanceConflict:
	mid_free(&ei -> ei_classInstanceConflict.bmo_class);
	mn_free(&ei -> ei_classInstanceConflict.bmo_inst);
	break;

    case m_complexityLimitation:
	mfilter_free(ei -> ei_complexityLimitation.cl_filter);
	break;

    default:
	break;
    }
}


void  mi_free (mi)
MSAPIndication * mi;
{
    switch (mi -> mi_type) {
    case MI_ABORT:
	ma_free(&mi -> mi_abort);
	break;

    case MI_TERM:
	mt_free(&mi -> mi_term);
	break;

    case MI_ERROR:
	if (mi -> mi_errtype == m_getListError)
	    mgetres_free(&mi -> mi_getres.result);
	else
	if (mi -> mi_errtype == m_setListError)
	    msetres_free(&mi -> mi_setres.result);
	else
	    merrorinfo_free(mi -> mi_errtype, &mi -> mi_error.info);
	break;

    case MI_EVENT_REP:
    case MI_EVENT_REPC:
	meventreparg_free(&mi -> mi_eventrepinv.args);
	break;

    case MI_EVENT_REP_RES:
	meventrepres_free(&mi -> mi_eventrepres.result);
	break;

    case MI_LINKED_REPLY:
	mlinkedreplyarg_free(&mi -> mi_linkedreplyinv.args);
	break;

    case MI_GET:
	mgetarg_free(&mi -> mi_getinv.args);
	break;

    case MI_GET_RES:
	mgetres_free(&mi -> mi_getres.result);
	break;

    case MI_SET:
    case MI_SETC:
	msetarg_free(&mi -> mi_setinv.args);
	break;

    case MI_SET_RES:
	msetres_free(&mi -> mi_setres.result);
	break;

    case MI_ACTION:
    case MI_ACTIONC:
	mactionarg_free(&mi -> mi_actioninv.args);
	break;

    case MI_ACTION_RES:
	mactionres_free(&mi -> mi_actionres.result);
	break;

    case MI_CREATE:
	mcreatearg_free(&mi -> mi_createinv.args);
	break;

    case MI_CREATE_RES:
	mcreateres_free(&mi -> mi_createres.result);
	break;

    case MI_DELETE:
	mdeletearg_free(&mi -> mi_deleteinv.args);
	break;

    case MI_DELETE_RES:
	mdeleteres_free(&mi -> mi_deleteres.result);
	break;

    default:
	/* no info to release for MI_UREJECT, MI_PREJECT,
	 * MI_CANCEL_GET, MI_EMPTY_RES
	 */
	break;
    }
}
