/*
 * Copyright (c) 1988 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* Routines to establish a management association */

/*
 * By George Pavlou, April 1990
 */

/*
** Authenticated association and MInitOption added by Saleem
** March/April 1994
*/

#include <stdio.h>
#include "msap.h"

#if defined(SECURITY)
#include "mauth.h"
#include "mintegrity.h"
#endif

#include <string.h>

#define ACSE_ASN_OID	str2oid("2.2.1.0.1")

#ifndef	LPP
#define CMIP_ASN_OID	str2oid("2.9.1.1.4")		/* full CMIP stack */
#else
#define CMIP_ASN_OID	str2oid("1.3.6.1.2.1.9.1.2")	/* CMOT stack */
#endif


static void M_Fail (), pctxlist_copy (), pctxlist_free ();

static int cur_cmip_pctxid;		/* the current CMIP pctx id */
static struct PSAPctxlist cur_pctxlist;	/* the current pctx list */

static char *context = "management";


/* LINTLIBRARY */

/*  M_InitialiseReq - Calling is equivalent to a M-ASSOCIATE.REQUEST
 * a succesful return is equivalent to a M-ASSOCIATE.CONFIRMATION
 */

int  M_InitialiseReq (callingTitle, callingAddr, calledTitle, calledAddr,
			context, protvrsn, fununits, access, info, mc, mi)
AEI callingTitle, calledTitle;
char* context;
struct PSAPaddr *callingAddr, *calledAddr;
int protvrsn, fununits;
External *access, *info;
MSAPConnect* mc;
MSAPIndication* mi;
{
    struct AcSAPconnect accs;
    struct AcSAPindication acis;
    struct PSAPctxlist pctx;
    OID ctx;
    struct SSAPref sfs, *sf;
    struct RoSAPindication rois;
    struct MSAPAssocInfo mcis, *mci;
    PE pe;

    if ((ctx = ode2oid(context)) == NULLOID) {
	mi->mi_type = MI_ABORT;
	mi->mi_abort.ma_source = MA_LOCAL;
	mi->mi_abort.ma_reason = ACS_ADDRESS;
	mi->mi_abort.ma_info   = NULLEXTERN;
	(void) strncpy(mi->mi_abort.ma_data, "Can't find Context", MA_SIZE);
	return NOTOK;
    }
    if ((ctx = oid_cpy(ctx)) == NULLOID)
	goto no_mem;

    pctx.pc_nctx = 1;
    pctx.pc_ctx[0].pc_id  = 1;
    pctx.pc_ctx[0].pc_atn = NULLOID;
    if ((pctx.pc_ctx[0].pc_asn = oid_cpy(CMIP_ASN_OID)) == NULLOID)
	goto no_mem;

    if ((sf = addr2ref(PLocalHostName())) == NULL) {
	sf = &sfs;
	(void) bzero((char*) sf, sizeof *sf);
    }

    if ((mcis.mai_protvrsn = protvrsn) != CMIP_VERSION2)
	fprintf(stderr, "Warning: M_InitialiseReq: CMIP version not 2 (%d), \
setting it to 2\n",
			mcis.mai_protvrsn = CMIP_VERSION2);
    mcis.mai_funits   = fununits;
    mcis.mai_access   = access;
    mcis.mai_info     = info;
    if (build_CMIP_CMIPUserInfo(&pe, 1, 0, NULLCP, &mcis) == NOTOK)
	goto no_mem;
    pe -> pe_context = 1;

    if (AcAssocRequest(ctx, callingTitle, calledTitle, callingAddr, calledAddr,
		&pctx, pctx.pc_ctx[0].pc_asn, 0, ROS_MYREQUIRE, SERIAL_NONE,
		0, sf, &pe, 1, NULLQOS, &accs, &acis )
		== NOTOK) {
	mi->mi_type = MI_ABORT;
	mi->mi_abort.ma_source = acis.aci_abort.aca_source == ACA_LOCAL ?
		MA_LOCAL : MA_PROVIDER;
	mi->mi_abort.ma_reason = acis.aci_abort.aca_reason;
	mi->mi_abort.ma_info   = NULLEXTERN;
	(void) strncpy(mi->mi_abort.ma_data, acis.aci_abort.aca_data, MA_SIZE);
	pe_free(pe);
	ACAFREE(&acis.aci_abort);
	return NOTOK;
    }

    oid_free(ctx);
    oid_free(pctx.pc_ctx[0].pc_asn);
    pe_free(pe);

    if (accs.acc_result != ACS_ACCEPT) {
	mi->mi_type = MI_ABORT;
	mi->mi_abort.ma_source = acis.aci_abort.aca_source == ACA_LOCAL ?
		MA_LOCAL : MA_PROVIDER;
	mi->mi_abort.ma_reason = ACS_REJECT;
	mi->mi_abort.ma_info   = NULLEXTERN;
	(void) strncpy(mi->mi_abort.ma_data, "Association Rejected by Peer",
				MA_SIZE);
	return NOTOK;
    }

    if (accs.acc_ninfo != 1) {
	M_Fail(accs.acc_sd, MA_BADINFO, "No User Info", NULLEXTERN, mi);
	return NOTOK;
    }

    mc -> mc_info.mai_protvrsn = mc -> mc_info.mai_funits = 0;
    mc -> mc_info.mai_access = mc -> mc_info.mai_info = NULL;
    if (parse_CMIP_CMIPUserInfo(accs.acc_info[0], 0, NULLIP, NULLVP, &mci)
		== NOTOK) {
	M_Fail(accs.acc_sd, MA_BADINFO, "Malformed User Info", NULLEXTERN, mi);
	return NOTOK;
    }
    mc -> mc_info = *mci;	/* struct cpy */
    free((char*) mci);

    if (RoSetService(accs.acc_sd, RoPService, &rois) == NOTOK) {
	M_Fail(accs.acc_sd, MA_ROS, "Can't Set Service", NULLEXTERN, mi);
	return NOTOK;
    }
    mc -> mc_sd = accs.acc_sd;
    mc->mc_result = accs.acc_result;
    mc -> mc_resptitle = accs.acc_respondtitle;   /* struct copy */
    mc -> mc_context = accs.acc_context;
    accs.acc_context = NULLOID;
    ACCFREE(&accs);

    return OK;

no_mem: ;	/* comes here if it cannot form MPDU */
    mi->mi_type = MI_ABORT;
    mi->mi_abort.ma_source = MA_LOCAL;
    mi->mi_abort.ma_reason = ACS_CONGEST;
    mi->mi_abort.ma_info   = NULLEXTERN;
    (void) strncpy(mi->mi_abort.ma_data, "Out of Memory", MA_SIZE);
    return NOTOK;
}


/*  M_Init - Called when program invoked to recapture context.
 *  A successful return is equivalent to a M-ASSOCIATE.INDICATION
 */
/* ARGSUSED */

int  M_Init (vecp, vec, ms, mi)
int vecp;
char** vec;
MSAPStart* ms;
MSAPIndication* mi;
{
    struct AcSAPstart acss;
    struct AcSAPindication acis;
    struct PSAPctxlist* pctxlist = &acss.acs_start.ps_ctxlist; 
    struct MSAPAssocInfo *mci;
    OID	cmip_ctx, acse_ctx;
    int cmip_ctx_found = 0;
    register int i;

    if (AcInit(vecp, vec, &acss, &acis) == NOTOK) {
	mi->mi_type = MI_ABORT;
	mi->mi_abort.ma_source = acis.aci_abort.aca_source == ACA_LOCAL ?
		MA_LOCAL : MA_PROVIDER;
	mi->mi_abort.ma_reason = acis.aci_abort.aca_reason;
	mi->mi_abort.ma_info   = NULLEXTERN;
	(void) strncpy(mi->mi_abort.ma_data, acis.aci_abort.aca_data, MA_SIZE);
	ACAFREE(&acis.aci_abort);
	return NOTOK;
    }

    if ((cmip_ctx = oid_cpy(CMIP_ASN_OID)) == NULLOID ||
		(acse_ctx = oid_cpy(ACSE_ASN_OID)) == NULLOID) {
	mi->mi_type = MI_ABORT;
	mi->mi_abort.ma_source = MA_LOCAL;
	mi->mi_abort.ma_reason = ACS_CONGEST;
	mi->mi_abort.ma_info   = NULLEXTERN;
	(void) strncpy(mi->mi_abort.ma_data, "Out of Memory", MA_SIZE);
	if (cmip_ctx)
	    oid_free(cmip_ctx);
	return NOTOK;
    }

    for (i = 0; i < pctxlist -> pc_nctx; i++) {
	if (oid_cmp(pctxlist -> pc_ctx[i].pc_asn, cmip_ctx) == 0) {
	    cmip_ctx_found = 1;
	    pctxlist -> pc_ctx[i].pc_result = PC_ACCEPT;
	    cur_cmip_pctxid = pctxlist -> pc_ctx[i].pc_id;
	    continue;
	}
	else
	if (oid_cmp(pctxlist -> pc_ctx[i].pc_asn, acse_ctx) == 0)
	    continue;
	else				/* unknown presentation context */
	    pctxlist -> pc_ctx[i].pc_result = PC_REJECTED;
    }

    oid_free(cmip_ctx);
    oid_free(acse_ctx);

    /* copy presentation context list */
    pctxlist_free(&cur_pctxlist);
    pctxlist_copy(pctxlist, &cur_pctxlist);

    if (!cmip_ctx_found) {
	M_Fail(acss.acs_sd, MA_BADPCTX, "Bad Presentation Context",
		NULLEXTERN, mi);
	return NOTOK;
    }

    if (acss.acs_ninfo != 1) {
	M_Fail(acss.acs_sd, MA_BADINFO, "No User Info", NULLEXTERN, mi);
	return NOTOK;
    }

    ms -> ms_info.mai_protvrsn = ms -> ms_info.mai_funits = 0;
    ms -> ms_info.mai_access = ms -> ms_info.mai_info = NULL;
    if (parse_CMIP_CMIPUserInfo(acss.acs_info[0], 0, NULLIP, NULLVP, &mci)
		== NOTOK) {
	pe_free(acss.acs_info[0]);
	M_Fail(acss.acs_sd, MA_BADINFO, "Malformed User Info", NULLEXTERN, mi);
	return NOTOK;
    }
    ms -> ms_info = *mci;	/* struct cpy */
    free((char*) mci);

    if ((ms -> ms_info.mai_protvrsn & CMIP_VERSION2) == 0) {
        if (AcAssocResponse(acss.acs_sd, ACS_PERMANENT, ACS_USER_NOREASON,
		NULLOID, NULLAEI, &acss.acs_start.ps_called,
		NULLPC, acss.acs_start.ps_defctxresult,
		acss.acs_start.ps_prequirements, ROS_MYREQUIRE, SERIAL_NONE,
		acss.acs_start.ps_settings, &acss.acs_start.ps_connect,
		NULLPEP, 0, &acis) == NOTOK ) {
	    fprintf(stderr, "AcAssocResponse failed: %d\n",
					acis.aci_abort.aca_reason);
	}
	mi->mi_type = MI_ABORT;
	mi->mi_abort.ma_source = MA_PROVIDER;
	mi->mi_abort.ma_reason = MA_NOVRSN;
	mi->mi_abort.ma_info   = NULLEXTERN;
	(void) strncpy(mi->mi_abort.ma_data, "No Common CMIP Version", MA_SIZE);
	return NOTOK;
    }

    ms -> ms_sd      = acss.acs_sd;
    ms -> ms_calling = acss.acs_callingtitle;	/* struct copy */
    ms -> ms_called  = acss.acs_calledtitle;	/*  ..  */
    ms -> ms_start   = acss.acs_start;		/*  ..  */
    ms -> ms_context = acss.acs_context;
    pe_free(acss.acs_info[0]);
    
    return OK;
}


/*  M_InitialiseResp  -  Respond to a connection request,
 *  equivalent to M-ASSOCIATE.RESPONSE
 */
/* ARGSUSED */

int  M_InitialiseResp (ms, status, respTitle, respAddr,
			context, protvrsn, fununits, access, info, mi)
MSAPStart * ms;
int status;
AEI respTitle;
struct PSAPaddr * respAddr;
char * context;
int protvrsn, fununits;
External * access, * info;
MSAPIndication * mi;
{
    struct AcSAPindication acis;
    struct RoSAPindication rois;
    struct MSAPAssocInfo mcis;
    OID ctx = NULLOID, ctx_tmp;
    PE  pe;

    if (context && ((ctx_tmp = ode2oid(context)) == NULLOID ||
			(ctx = oid_cpy(ctx_tmp)) == NULLOID)) {
	mi->mi_type = MI_ABORT;
	mi->mi_abort.ma_source = MA_LOCAL;
	mi->mi_abort.ma_reason = ACS_ADDRESS;
	mi->mi_abort.ma_info   = NULLEXTERN;
	if (ctx_tmp)
	    (void) strncpy(mi->mi_abort.ma_data, "Can't find Context", MA_SIZE);
	else	/* ctx = NULLOID */
	    (void) strncpy(mi->mi_abort.ma_data, "Out of Memory", MA_SIZE);
	return NOTOK;
    }

    if ((mcis.mai_protvrsn = protvrsn) != CMIP_VERSION2)
	fprintf(stderr, "Warning: M_InitialiseResp: CMIP version not 2 (%d), \
setting it to 2\n",
			mcis.mai_protvrsn = CMIP_VERSION2);
    mcis.mai_funits   = fununits;
    mcis.mai_access   = access;
    mcis.mai_info     = info;
    if (build_CMIP_CMIPUserInfo(&pe, 1, 0, NULLCP, &mcis) == NOTOK) {
	M_Fail(ms->ms_sd, ACS_CONGEST, "Out of Memory", NULLEXTERN, mi);
	return NOTOK;
    }
    pe -> pe_context = cur_cmip_pctxid;

    if (AcAssocResponse(ms->ms_sd, status,
		status != ACS_ACCEPT ? ACS_USER_NOREASON : ACS_USER_NULL,
		ctx, respTitle, respAddr,
		&cur_pctxlist, ms->ms_start.ps_defctxresult,
		ms->ms_start.ps_prequirements, ROS_MYREQUIRE, SERIAL_NONE,
		ms->ms_start.ps_settings, &ms->ms_start.ps_connect,
		&pe, 1, &acis) == NOTOK ) {
	mi->mi_type = MI_ABORT;
	mi->mi_abort.ma_source = acis.aci_abort.aca_source == ACA_LOCAL ?
		MA_LOCAL : MA_PROVIDER;
	mi->mi_abort.ma_reason = acis.aci_abort.aca_reason;
	mi->mi_abort.ma_info   = NULLEXTERN;
	(void) strncpy(mi->mi_abort.ma_data, acis.aci_abort.aca_data, MA_SIZE);
	oid_free(ctx);
	pe_free(pe);
	ACAFREE(&acis.aci_abort);
	return NOTOK;
    }

    oid_free(ctx);
    pe_free(pe);
    if (status == M_ACCEPT &&
		RoSetService(ms->ms_sd, RoPService, &rois) == NOTOK) {
	M_Fail(ms->ms_sd, MA_ROS, "Can't Set Service", NULLEXTERN, mi);
	return NOTOK;
    }
    return OK;
}


/* M_Fail - abort an association while being established due to fatal error
 */
/* ARGSUSED */

static void  M_Fail (msd, code, expl, info, mi)
int msd, code;
char * expl;
External * info;
MSAPIndication * mi;
{
    struct AcSAPindication acis;
    struct MSAPAbort mas;
    PE pe;

    mas.ma_info   = info;
    mas.ma_source = MA_PROVIDER;
    (void) build_CMIP_CMIPAbortInfo(&pe, 1, 0, NULLCP, &mas); 
    pe -> pe_context = 1;
    (void) AcUAbortRequest(msd, &pe, 1, &acis);
    pe_free(pe);
    mi->mi_type = MI_ABORT;
    mi->mi_abort.ma_source = MA_PROVIDER;
    mi->mi_abort.ma_reason = code;
    (void) strncpy(mi->mi_abort.ma_data, expl, MA_SIZE);
}


static void  pctxlist_copy (l1, l2)
struct PSAPctxlist *l1, *l2;
{
    int i;
    l2 -> pc_nctx = l1 -> pc_nctx;
    for (i = 0; i < NPCTX && l1 -> pc_ctx[i].pc_id != 0; i++) {
	l2 -> pc_ctx[i].pc_id = l1 -> pc_ctx[i].pc_id;
	l2 -> pc_ctx[i].pc_asn = oid_cpy(l1 -> pc_ctx[i].pc_asn);
	l2 -> pc_ctx[i].pc_atn = oid_cpy(l1 -> pc_ctx[i].pc_atn);
	l2 -> pc_ctx[i].pc_result = l1 -> pc_ctx[i].pc_result;
    }
}


static void  pctxlist_free (l)
struct PSAPctxlist *l;
{
    int i;
    for (i = 0; i < NPCTX && l -> pc_ctx[i].pc_id != 0; i++) {
	oid_free(l -> pc_ctx[i].pc_asn);
	oid_free(l -> pc_ctx[i].pc_atn);
    }
}


/*  m_initialise - higher level friendly interface to M_InitialiseReq */

int  m_initialise (agent, rhost, options)
char* agent;
char* rhost;
MInitOption **options;
{
    MSAPConnect  mcs;
    MSAPIndication  mis;
    AEI  aei;
    struct PSAPaddr *pa;
    External *accessControl = (External *) 0, *userInfo = (External *) 0;
    MInitOption **opt;
#if defined(SECURITY)
    AuthAssocIntegrityInfo *aaii = (AuthAssocIntegrityInfo *) 0,
                           *aaii2 = (AuthAssocIntegrityInfo *) 0;
    register
    int authFlag = 0; /* Set up an authneticated association */
#endif
    register
    int e = 0;

#ifdef USE_X500

    char * agentDN= lt_str2dn(rhost, agent);

    if ( (pa = lt_read(agentDN)) == NULLPA ) {
        fprintf(stdout, "%s-%s:  Null PSAP address returned from the Directory.\nTrying static AE information...\n",
                rhost, agent);

        if ((aei = str2aei(rhost, agent)) == NULLAEI) {
            fprintf(stderr, "%s-%s: unknown application entity title\n",
                            rhost, agent);
            return NOTOK;
        }

        if ((pa = aei2addr(aei)) == NULLPA) {
            fprintf(stderr, "aei translation failed\n");
            return NOTOK;
        }
    } else {
        if ( lt_oper_status == DOWN ) {
            fprintf(stderr,"The specified agent is not currently running.\n");
            return NOTOK;
        }
        aei = NULLAEI;
    }

#else

    if ((aei = str2aei(rhost, agent)) == NULLAEI) {
	fprintf(stderr, "%s-%s: unknown application entity title\n",
			rhost, agent);
	return NOTOK;
    }

    if ((pa = aei2addr(aei)) == NULLPA) {
	fprintf(stderr, "aei translation failed\n");
	return NOTOK;
    }

#endif /* USE_X500 */

    for(opt = options; opt && *opt; ++opt) {

        switch ((*opt)->optionId) {

#if defined(SECURITY)
        case MIO_AuthAssocIntegrityInfo: /* See mauth.h */
            aaii = (AuthAssocIntegrityInfo *) (*opt)->optionInfo;
            if (aaii == (AuthAssocIntegrityInfo *) 0) {
                fprintf(stderr, "NULL AuthAssocIntegrityInfo\n");
                return NOTOK;
            }

            /* Make ACSE User Info */
            if ((e = makeAcseUserInfo(aaii, &userInfo)) != 0 ||
                userInfo == (External *) 0) {
                fprintf(stderr, "m_initialise(): makeAcseUserInfo() %s\n",
                        authErrorMessage(e));
                return NOTOK;
            }
            authFlag = 1;
            break;
#endif
        default:
            fprintf(stderr, "m_initialise(): option not %d available\n",
                    (*opt)->optionId);
            break;
        }
    }

    /* request all the functional units bar the extendedService */
    if (M_InitialiseReq(NULLAEI,  NULLPA, aei, pa, context,
		CMIP_VERSION2, 0x17, accessControl, userInfo, &mcs, &mis)
	    == NOTOK) {
	if (mis.mi_abort.ma_reason == ACS_TRANSIENT)
	    fprintf(stderr, "Remote agent is busy, try later...\n");
	else
	    fprintf(stderr, "M_InitialiseRequest: %s\n", mis.mi_abort.ma_data);
	mi_free(&mis);

        if (accessControl != (External *) 0)
            external_free(accessControl);
        if (userInfo != (External *) 0)
            external_free(userInfo);

	return NOTOK;
    }
    if (accessControl != (External *) 0)
        external_free(accessControl);
    if (userInfo != (External *) 0)
        external_free(userInfo);

#if defined(SECURITY)
    if (authFlag) {
        /*
        ** Authenticate peer
        */
        e = getAcseUserInfo(mcs.mc_info.mai_info, &aaii2);

        /* Sanity check */
        if (e == 0 && memcmp(aaii->sessionKey, aaii2->sessionKey, 8) != 0) {
            fprintf(stderr, "m_initialise(): different sessionKey from peer\n");
            e = AUTH_error;
        }
    }

    if (authFlag && e == 0) {

#if defined(DEBUG)
        register
        char *nameStr = (char *) 0, *cnStr;

        cnStr = "cn=";
        if ((nameStr = strstr(aaii2->peer, cnStr)) == (char *) 0) {
            cnStr = "commonName=";
            nameStr = strstr(aaii2->peer, cnStr);
        }
        if (nameStr != (char *) 0)
            nameStr += strlen(cnStr);
        fprintf(stderr, "I believe I am talking to %s\n\n",
                nameStr != (char *) 0 ? nameStr : aaii2->peer);
#endif
        /* Clear previous Id state */
        clearId(mcs.mc_sd);

        /* Set up the session key */
        setMd5Key(mcs.mc_sd, aaii->sessionKey);
    }

    else if (authFlag) {
        fprintf(stderr, "Could not athenticate %s: %s aborting assoc\n",
                sprintaei(aei), authErrorMessage(e));

        /*
        ** If authentication fails, for now, drop the assoc. Should
        ** actually give the user the choice, but that requires
        ** changing the interface to m_initialise()
        */
        if (M_AbortReq(mcs.mc_sd, (External *) 0, &mis) != OK) {
#if defined(DEBUG)
            fprintf(stderr, "m_initialise(): M_AbortReq() %s\n",
                    mis.mi_abort.ma_data);
#endif
            mi_free(&mis);
        }
    }

    AuthAssocIntegrityInfo_free(aaii2);
#endif /* SECURITY */

    mc_free(&mcs);

    return e == 0 ? mcs.mc_sd : e;
}


/*  m_terminate - higher level friendly interface to M_TerminateReq */

int  m_terminate (msd)
int msd;
{
    MSAPIndication  mis;

    if (M_TerminateReq(msd, ACF_NORMAL, NULLPE, &mis) != OK) {
	fprintf(stderr, "Terminate request failed: reason = %s\n",
			mis.mi_abort.ma_data);
	return NOTOK;
    }
#if defined(SECURITY)
    clearMd5Key(msd);
    clearId(msd);
#endif

    return OK;
}
