/*
 * Copyright (c) 1988 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* mparm.h - CMIS/P include file with management data structures */

/*
 * By George Pavlou, June 1991
 */

#ifndef _MPARM_
#define _MPARM_


#ifndef _module_UNIV_defined_
#ifndef	ICR1
#include <isode/pepsy/UNIV-types.h>
#else
#include <isode/asn1/UNIV-types.h>
#endif
#endif

#ifndef _module_IF_defined_
#ifndef	ICR1
#include <isode/pepsy/IF-types.h>
#else
#include <isode/asn1/IF-types.h>
#endif
#endif

#ifndef QUIPUNAME
#include <isode/quipu/name.h>
#endif


typedef struct type_UNIV_EXTERNAL External;
#define NULLEXTERN ((struct type_UNIV_EXTERNAL *) 0)
#define NULLMACCESS ((struct type_UNIV_EXTERNAL *) 0)

#define EXTERN_ASN1_TYPE	choice_UNIV_0_single__ASN1__type
#define EXTERN_OCTET_ALIGNED	choice_UNIV_0_octet__aligned
#define EXTERN_ARBITRARY	choice_UNIV_0_arbitrary

#define external_free free_UNIV_EXTERNAL


#define	M_ATTRBLOCK	20	/* for space allocation per attribute block */

#define GET_ALLATTRS	-1
#define GET_NOATTRS	0



typedef enum
{
    m_noError,

#define	CMIS_ERROR_OFFSET	1

    /* CMIS errors */

    m_noSuchObjectClass,
    m_noSuchObjectInstance,
    m_accessDenied,
    m_syncNotSupported,
    m_invalidFilter,
    m_noSuchAttribute,
    m_invalidAttributeValue,
    m_getListError,
    m_setListError,
    m_noSuchAction,
    m_processingFailure,
    m_duplicateManagedObjectInstance,
    m_noSuchReferenceObject,
    m_noSuchEventType,
    m_noSuchArgument,
    m_invalidArgumentValue,
    m_invalidScope,
    m_invalidObjectInstance,
    m_missingAttributeValue,
    m_classInstanceConflict,
    m_complexityLimitation,
    m_mistypedOperation,
    m_noSuchInvokeId,
    m_operationCancelled,

    /* CMIS errors but no ROERs - used only in AttributeError */

    m_invalidOperation,
    m_invalidOperator,

#define	ROS_ERROR_OFFSET	27

    /* ROS Invoke-problem errors */

    m_duplicateInvocation,
    m_unrecognisedOperation,
    m_mistypedArgument,
    m_resourceLimitation

} CMISErrors;

#define NULLMERROR m_noError



typedef enum
{
    m_noModifyOp = -1,
    m_replace,
    m_addValue,
    m_removeValue,
    m_setToDefault

} CMISModifyOp;



typedef enum
{
    s_invalid = -1,
    s_bestEffort,
    s_atomic

} CMISSync;

#define NULLMSYNC s_bestEffort



typedef struct MIDentifier	/* Management IDentifier */
{
    int mid_type;

#define MID_GLOBAL	1	/* not 0 and 1 as in the past */
#define MID_LOCAL	2

    union {
	OID mid_un_global;	/* an object identifier */
	int mid_un_local;	/* or an integer */
    } mid_un;

#define mid_global	mid_un.mid_un_global
#define mid_local	mid_un.mid_un_local

} MIDentifier, * MID;

#define NULLMID	((MID) 0)
	


typedef struct MName		/* Management Name */
{
    int mn_type;

#define MN_DN		1	/* not 0,1,2 as in the past */
#define MN_NONSPEC	2
#define MN_LOCALDN	3

    union {
	DN    mn_un_dn;		/* a dn/local dn */
	char* mn_un_nonspec;	/* or a non specific name (char string) */
    } mn_un;

#define mn_dn		mn_un.mn_un_dn
#define mn_nonspec	mn_un.mn_un_nonspec

} MName, * MN;

#define NULLMN	((MN) 0)



typedef struct			/* attribute value assertion */
{
    OID	ava_oid;
    PE	ava_value;

#define ava_val	ava_value	/* for naming consistency */

} Ava;

#define NULLAVA		((Ava*) 0)



typedef struct CMISParam	/* Management Parameter: */
{				/* attribute, action, event report etc. */
    MIDentifier	mp_id;
    PE		mp_val;

} CMISParam;

#define NULLMPARM	((CMISParam *) 0)



typedef struct MIDListVal
{					/* Management Identifier List
					   (Object Class, Event Type etc.) */
    MIDentifier		midl_mid;
    struct MIDListVal*	midl_next;

} MIDListVal;



typedef struct CMISParamListVal
{
    CMISParam		mpl_ava;	/* Management Parameter List */
    struct CMISParamListVal*	mpl_next;

#define mpl_id	mpl_ava.mp_id
#define mpl_val	mpl_ava.mp_val

} CMISParamListVal;



typedef struct CMISGetAttr
{
    CMISParam	ga_ava;
    CMISErrors	ga_error;

#define ga_id	ga_ava.mp_id
#define ga_val	ga_ava.mp_val

} CMISGetAttr;

#define NULLMGETATTR	((CMISGetAttr *) 0)



typedef struct CMISSetAttr
{
    CMISParam	sa_ava;
    CMISErrors	sa_error;	/* used only in SetListErrors */
    CMISModifyOp sa_modify;

#define sa_id	sa_ava.mp_id
#define sa_val	sa_ava.mp_val

} CMISSetAttr;

#define NULLMSETATTR	((CMISSetAttr *) 0)



/* the following structure is a "union" of CMISParam, CMISGetAttr, CMISSetAttr
 * and it is used only by pepsy i.e. not part of the CMIS API
 */
typedef struct CMISAttr
{
    CMISParam		ma_ava;
    CMISErrors		ma_error;
    CMISModifyOp	ma_modify;

    struct CMISAttr*	ma_next;

#define ma_id		ma_ava.mp_id
#define ma_val		ma_ava.mp_val

#define	MA_ATTRID	1
#define	MA_ATTR		2
#define	MA_GETATTR	3
#define	MA_SETATTR	4

} CMISAttr;

#define NULLMATTR	((CMISAttr *) 0)



typedef struct CMISScope
{
    int	sc_type;

#define Sc_BaseObject		0
#define Sc_FirstLevel		1
#define Sc_WholeSubtree		2
#define Sc_IndividualLevel	3
#define Sc_BaseToNthLevel	4

    int	sc_level;		/* for IndividualLevel or BaseToNthLevel */

} CMISScope;

#define NULLMSCOPE	((CMISScope *) 0)



typedef struct CMISFilterSubstrings
{
    int			fi_sub_type;

#define FI_Substring_Initial	1	/* not 0,1,2 as in the past */
#define FI_Substring_Any	2
#define FI_Substring_Final	3

    MIDentifier		fi_sub_attrid;
    PE			fi_sub_string;
    struct CMISFilterSubstrings* fi_sub_next;

} CMISFilterSubstrings;



typedef struct CMISFilterItem
{
    int fi_type;    
#define FI_Equality		1	/* not 0-7 as in the past */
#define FI_Substrings		2
#define FI_GreaterOrEqual	3
#define FI_LessOrEqual		4
#define FI_Present		5
#define FI_SubsetOf		6
#define FI_SupersetOf		7
#define FI_NonNullSetIntersect  8

    union {
	CMISParam		fi_un_attr;
	CMISFilterSubstrings* fi_un_substrings;
    } fi_un;
#define fi_attr		fi_un.fi_un_attr
#define fi_substrings	fi_un.fi_un_substrings

    MID fi_present;			/* for pepsy use only */

} CMISFilterItem;

#define NULLMFILTERITEM	((FilterItem *) 0)



typedef struct CMISFilter
{
    int flt_type;
#define Filter_Item	1	/* not 0,1,2,3 as in the past */
#define Filter_And	2
#define Filter_Or	3
#define Filter_Not	4

    struct CMISFilter* flt_next;

    union {
	struct CMISFilterItem*	flt_un_item;	/* an item */
	struct CMISFilter*	flt_un_filter;	/* or a ptr to a
						   chain of filters */
    } flt_un;

#define flt_item	flt_un.flt_un_item
#define flt_filter	flt_un.flt_un_filter

} CMISFilter;

#define NULLMFILTER	((CMISFilter *) 0)



typedef struct CMISEventReportArg
{
    MIDentifier	ea_class;
    MName	ea_inst;
    char*	ea_time;
    MIDentifier ea_type;
    PE		ea_info;

} CMISEventReportArg;



typedef struct CMISEventReportRes
{
    MIDentifier	er_class;
    MName	er_inst;
    char*	er_time;
    CMISParam	er_reply;

} CMISEventReportRes;



typedef struct CMISGetArg
{
    MIDentifier	ga_class;
    MName	ga_inst;
    External*	ga_access;
    CMISSync	ga_sync;
    CMISScope	ga_scope;
    CMISFilter*	ga_filter;
    int		ga_nattrs;
    MIDentifier* ga_attrs;
    CMISAttr*	ga_pattrs;	/* for pepsy use only */

} CMISGetArg;



typedef struct CMISGetRes
{
    MIDentifier	gr_class;
    MName	gr_inst;
    char*	gr_time;
    int		gr_nattrs;
    CMISGetAttr* gr_attrs;
    CMISAttr*	gr_pattrs;	/* for pepsy use only */

} CMISGetRes;



typedef struct CMISSetArg
{
    MIDentifier	sa_class;
    MName	sa_inst;
    External*	sa_access;
    CMISSync	sa_sync;
    CMISScope	sa_scope;
    CMISFilter*	sa_filter;
    int		sa_nattrs;
    CMISSetAttr* sa_attrs;
    CMISAttr*	sa_pattrs;	/* for pepsy use only */

} CMISSetArg;



typedef struct CMISSetRes
{
    MIDentifier	sr_class;
    MName	sr_inst;
    char*	sr_time;
    int		sr_nattrs;
    CMISSetAttr* sr_attrs;
    CMISAttr*	sr_pattrs;	/* for pepsy use only */

} CMISSetRes;



typedef struct CMISActionArg
{
    MIDentifier	aa_class;
    MName	aa_inst;
    External*	aa_access;
    CMISSync	aa_sync;
    CMISScope	aa_scope;
    CMISFilter*	aa_filter;
    CMISParam	aa_info;

} CMISActionArg;



typedef struct CMISActionRes
{
    MIDentifier	ar_class;
    MName	ar_inst;
    char*	ar_time;
    CMISParam	ar_reply;

} CMISActionRes;



struct type_CMIP2_CreateInstance;
typedef struct type_CMIP2_CreateInstance CreateInst;

typedef struct CMISCreateArg
{
    MIDentifier	ca_class;
    MName	ca_inst;
    int		ca_inst_type;
#define CA_OBJECT_INST	 1	/* not 0 and 1 as in the past */
#define CA_SUPERIOR_INST 2

    MName	ca_reference_inst;
    External*	ca_access;
    int		ca_nattrs;
    CMISParam*	ca_attrs;
    MN		ca_pinst;	/* for pepsy use only */
    CMISAttr*	ca_pattrs;	/*  .. */
    CreateInst* ca_cinst;	/* .. */

} CMISCreateArg;



typedef struct CMISCreateRes
{
    MIDentifier	cr_class;
    MName	cr_inst;
    char*	cr_time;
    int		cr_nattrs;
    CMISParam*	cr_attrs;
    CMISAttr*	cr_pattrs;	/* for pepsy use only */

} CMISCreateRes;



typedef struct CMISDeleteArg
{
    MIDentifier	da_class;
    MName	da_inst;
    External*	da_access;
    CMISSync	da_sync;
    CMISScope	da_scope;
    CMISFilter*	da_filter;

} CMISDeleteArg;



typedef struct CMISDeleteRes
{
    MIDentifier	dr_class;
    MName	dr_inst;
    char*	dr_time;

} CMISDeleteRes;



typedef struct CMISCancelGetArg
{
    int		cga_getinvoke;

} CMISCancelGetArg;



typedef struct CMISNoSuchArgument
{
    MIDentifier	nsa_class;
    MIDentifier	nsa_type;

} CMISNoSuchArgument;



typedef struct CMISBaseManagedObjectId
{
    MIDentifier	bmo_class;
    MName	bmo_inst;

} CMISBaseManagedObjectId;



typedef struct CMISProcessingFailure
{
    MIDentifier	pf_class;
    MName	pf_inst;
    CMISParam	pf_error;

} CMISProcessingFailure;



typedef struct CMISComplexityLimitation
{
    CMISScope	cl_scope;
    CMISFilter*	cl_filter;
    CMISSync	cl_sync;

} CMISComplexityLimitation;



/* the following provide backwards compatibility: */

#define CMISClassInstanceConflict	CMISBaseManagedObjectId
#define cic_class			bmo_class
#define cic_inst			bmo_inst

#define CMISInvalidArgumentValue	CMISParam
#define iav_id				mp_id
#define iav_val				mp_val

/* MIDListVal is used instead of CMISMissingAttrValue */


 
typedef struct CMISErrorInfo
{
    union {
	MIDentifier		    ei_un_noSuchObjectClass,
				    ei_un_noSuchAttribute,
				    ei_un_linkedActionAccessDenied;
	MIDListVal*		    ei_un_missingAttributeValue;
	MName			    ei_un_noSuchObjectInstance,
				    ei_un_invalidObjectInstance,
				    ei_un_duplicateObjectInstance,
				    ei_un_noSuchReferenceObject;
	CMISParam		    ei_un_invalidAttributeValue,
				    ei_un_invalidArgumentValue;
	CMISScope		    ei_un_invalidScope;
	CMISFilter*		    ei_un_invalidFilter;
	CMISSync		    ei_un_syncNotSupported;
	int			    ei_un_noSuchInvokeId;
	CMISNoSuchArgument	    ei_un_noSuchAction,
				    ei_un_noSuchEventType,
				    ei_un_noSuchArgument;
	CMISProcessingFailure	    ei_un_processingFailure;
	CMISComplexityLimitation    ei_un_complexityLimitation;
	CMISBaseManagedObjectId	    ei_un_classInstanceConflict;

    } ei_un;

#define ei_noSuchObjectClass	    ei_un.ei_un_noSuchObjectClass
#define ei_noSuchAttribute	    ei_un.ei_un_noSuchAttribute
#define ei_linkedActionAccessDenied ei_un.ei_un_linkedActionAccessDenied
#define ei_missingAttributeValue    ei_un.ei_un_missingAttributeValue
#define ei_noSuchObjectInstance	    ei_un.ei_un_noSuchObjectInstance
#define ei_invalidObjectInstance    ei_un.ei_un_invalidObjectInstance
#define ei_duplicateObjectInstance  ei_un.ei_un_duplicateObjectInstance
#define ei_noSuchReferenceObject    ei_un.ei_un_noSuchReferenceObject
#define ei_invalidAttributeValue    ei_un.ei_un_invalidAttributeValue
#define ei_invalidScope		    ei_un.ei_un_invalidScope
#define ei_invalidFilter	    ei_un.ei_un_invalidFilter
#define ei_syncNotSupported	    ei_un.ei_un_syncNotSupported
#define ei_noSuchInvokeId	    ei_un.ei_un_noSuchInvokeId
#define ei_noSuchAction		    ei_un.ei_un_noSuchAction
#define ei_noSuchEventType	    ei_un.ei_un_noSuchEventType
#define ei_noSuchArgument	    ei_un.ei_un_noSuchArgument
#define ei_invalidArgumentValue	    ei_un.ei_un_invalidArgumentValue
#define ei_processingFailure	    ei_un.ei_un_processingFailure
#define ei_complexityLimitation	    ei_un.ei_un_complexityLimitation
#define ei_classInstanceConflict    ei_un.ei_un_classInstanceConflict

} CMISErrorInfo;

#define NULLMERRORINFO	((CMISErrorInfo *) 0)



typedef struct CMISActionError
{
    MIDentifier	ae_class;
    MName	ae_inst;
    char*	ae_time;

    CMISErrors	ae_error;

    union {
	MIDentifier		ae_un_accessDenied;
	MIDentifier		ae_un_noSuchAction;
	CMISNoSuchArgument	ae_un_noSuchArgument;
	CMISParam		ae_un_invalidArgumentValue;
    } ae_un;

#define ae_accessDenied		ae_un.ae_un_accessDenied
#define ae_noSuchAction		ae_un.ae_un_noSuchAction
#define ae_noSuchArgument	ae_un.ae_un_noSuchArgument
#define ae_invalidArgumentValue	ae_un.ae_un_invalidArgumentValue

    caddr_t	ae_aux;		/* for pepsy use only */

} CMISActionError;



typedef struct CMISDeleteError
{
    MIDentifier	de_class;
    MName	de_inst;
    char*	de_time;
    CMISErrors	de_error;
    
} CMISDeleteError;



typedef struct CMISLinkedReplyArg
{
    int lr_linkid;

    int lr_type;

#define LR_GET_RES	1
#define LR_GET_LISTERR	2
#define LR_SET_RES	3
#define LR_SET_LISTERR	4
#define LR_ACTION_RES	5
#define LR_PROC_FAILURE	6
#define LR_DELETE_RES	7
#define LR_ACTION_ERR	8
#define LR_DELETE_ERR	9

    union {
	CMISGetRes		lr_un_getres;
	CMISSetRes		lr_un_setres;
	CMISActionRes		lr_un_actionres;
	CMISDeleteRes		lr_un_deleteres;
	CMISActionError		lr_un_actionerr;
	CMISDeleteError		lr_un_deleteerr;
	CMISProcessingFailure	lr_un_procfailure;
    } lr_un;

    caddr_t			lr_genparm;	/* for pepsy use only */

#define lr_getres	lr_un.lr_un_getres
#define lr_setres	lr_un.lr_un_setres
#define lr_actionres	lr_un.lr_un_actionres
#define lr_deleteres	lr_un.lr_un_deleteres
#define lr_actionerr	lr_un.lr_un_actionerr
#define lr_deleteerr	lr_un.lr_un_deleteerr
#define lr_procfailure	lr_un.lr_un_procfailure

} CMISLinkedReplyArg;


#endif
