/*
 * Copyright (c) 1988 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this parasraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* mprint.c - MSAP library printing routines */

/*
 * By George Pavlou, July 1990
 */

#include <stdio.h>
#include "msap.h"


/* LINTLIBRARY */


static char *msap_oper[] = {
    "M_EventRep",
    "M_EventRepC",
    "M_LinkedReply",
    "M_Get",
    "M_Set",
    "M_SetC",
    "M_Action",
    "M_ActionC",
    "M_Create",
    "M_Delete",
    "M_CancelGet"
};

static int msap_oper_count = sizeof msap_oper / sizeof msap_oper[0];


static char *msap_error[] = {
    "noError",

    "noSuchObjectClass",
    "noSuchObjectInstance",
    "accessDenied",
    "syncNotSupported",
    "invalidFilter",
    "noSuchAttribute",
    "invalidAttributeValue",
    "getListError",
    "setListError",
    "noSuchAction",
    "processingFailure",
    "duplicateManagedObjectInstance",
    "noSuchReferenceObject",
    "noSuchEventType",
    "noSuchArgument",
    "invalidArgumentValue",
    "invalidScope",
    "invalidObjectInstance",
    "missingAttributeValue",
    "classInstanceConflict",
    "complexityLimitation",
    "mistypedOperation",
    "noSuchInvokeId",
    "operationCancelled",

    "invalidOperation",
    "invalidOperator",

    "duplicateInvocation",
    "unrecognisedOperation",
    "mistypedArgument",
    "resourceLimitation"
};

static int msap_error_count = sizeof msap_error / sizeof msap_error[0];


char*  MOperString (oper)
int oper;
{
    static char buffer[64];

    if (oper < msap_oper_count && oper >= 0)
	return msap_oper[oper];

    (void) sprintf (buffer, "unknownMOperationCode: 0x%x", oper);
    return buffer;
}


char*  MErrString (error)
register CMISErrors error;
{
    static char buffer[64];

    if ((int) error < msap_error_count && (int) error >= 0)
	return msap_error[(int) error];

    (void) sprintf (buffer, "unknownMErrorCode: 0x%x", (int) error);
    return buffer;
}


char*  sprintmodifyop (modify)
CMISModifyOp modify;
{
    static char buffer[64];

    switch (modify) {
    case m_replace:
	(void) sprintf(buffer, "replace");
	break;

    case m_addValue:
	(void) sprintf(buffer, "addValue");
	break;

    case m_removeValue:
	(void) sprintf(buffer, "removeValue");
	break;

    case m_setToDefault:
	(void) sprintf(buffer, "setToDefault");
	break;

    case m_noModifyOp:
	(void) sprintf(buffer, "noModifyOp");
	break;

    default:
	(void) sprintf(buffer, "unknown");
	break;
    }
    return buffer;
}


char*  sprintscope (scope)
CMISScope* scope;
{
    static char buffer[64];
    char* cp = buffer;

    if (!scope || scope->sc_type == Sc_BaseObject) {
	(void) sprintf(buffer, "baseObject");
	return buffer;
    }

    switch (scope->sc_type) {
    case Sc_FirstLevel:
	(void) sprintf(buffer, "firstLevel");
	break;

    case Sc_WholeSubtree:
	(void) sprintf(buffer, "wholeSubtree");
	break;

    case Sc_BaseToNthLevel:
	(void) sprintf(cp, "baseTo");
	cp += 6;
    case Sc_IndividualLevel:
	(void) sprintf(cp, "%d", scope->sc_level);
	cp += strlen(cp);
    {
	char* c;
	switch (scope->sc_level) {
	case 1:
	    c = "st";
	    break;
	case 2:
	    c = "nd";
	    break;
	case 3:
	    c = "rd";
	    break;
	default:
	    c = "th";
	    break;
	}
	(void) sprintf(cp, "%sLevel", c);
    }
	break;

    default:
	if (scope->sc_type != Sc_BaseObject)
	    (void) sprintf (buffer, "invalidScope");
	break;
    }
    return buffer;
}


CMISScope*  str2scope (str)
{
    CMISScope* scope;
    char*  s;
    register char* t;
    int	   l;

    if ((scope = (CMISScope*) calloc(1, sizeof(CMISScope))) == NULLMSCOPE)
	return NULLMSCOPE;
    s = strdup(str);

    if (strcasecmp(s, "firstLevel") == 0)
	scope->sc_type = Sc_FirstLevel;
    else
    if (strcasecmp(s, "wholeSubtree") == 0)
	scope->sc_type = Sc_WholeSubtree;
    else
    if (strcasecmp(s, "baseObject") == 0)
	scope->sc_type = Sc_BaseObject;
    else {
	l = strlen(s);
	if (strncasecmp(s+l-5, "Level", 5) != 0)
	    goto err;

	s[l-7] = '\0';
	if (strncasecmp(s, "baseTo", 6) == 0) {
	    *(s+l-7) = '\0';
	    for (t = s+6; *t != '\0'; t++)
		if (!isdigit(*t))
		    goto err;
	    scope->sc_type  = Sc_BaseToNthLevel;
	    scope->sc_level = atoi(s+6);
	}
	else {
	    *(s+l-7) = '\0';
	    for (t = s; *t != '\0'; t++)
		if (!isdigit(*t))
		    goto err;
	    scope->sc_type  = Sc_IndividualLevel;
	    scope->sc_level = atoi(s);
	}
    }

    free(s);
    return scope;

err:
    free(scope); free(s);
    return NULLMSCOPE;
}


char*  sprintsync (sync)
CMISSync sync;
{
    static char buffer[12];

    switch (sync) {
    case s_bestEffort:
	(void) sprintf(buffer, "bestEffort");
	break;

    case s_atomic:
	(void) sprintf(buffer, "atomic");
	break;

    default:
	(void) sprintf (buffer, "invalidSync");
	break;
    }
    return buffer;
}


CMISSync  str2sync (str)
{
    CMISSync sync;

    if (strcasecmp(str, "bestEffort") == 0)
	sync = s_bestEffort;
    else
    if (strcasecmp(str, "atomic") == 0)
	sync = s_atomic;
    else
	return s_invalid;

    return sync;
}
