/*
 * Copyright (c) 1994 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/*
** Pretty-Print MSAP structures
*/

/*
** Saleem N. Bhatti
** March 1992, updated April 1994
*/

#if !defined(_msap_pp_c_)
#define _msap_pp_c_

#if defined(__GNUG__)
#include <std.h>
#else
#include <stdlib.h>
#endif
#include <stdio.h>
#include <string.h>

#include "msap-pp.h"

#if defined(__STDC__) || defined(__cplusplus)
#include <stdarg.h>
#else
#include <varargs.h>
#endif

#define	MSGARRAYSIZE(ma) (sizeof(ma) / sizeof(char *) - 1)

static int actionOrEventArg = 0;
unsigned long G_MSAP_PP_flags = 0;

static
char *indent = (char *) 0;
static
int indentLevel = 0;
static
int indentSize = 2;

#ifndef __linux__
static
FILE *ppFile = stdout;
#else
static
FILE *ppFile;
void ppFile_constructor() __attribute__((constructor));
void ppFile_constructor() { ppFile = stdout; }
#endif

FILE *
#if defined(__STDC__) || defined(__cplusplus)
ppSetOutputFile(FILE *fp)
#else
ppSetOutputFile(fp)
FILE *fp;
#endif
{ FILE *prev = ppFile; ppFile = fp; return prev; }

static void
#if defined(__STDC__) || defined(__cplusplus)
ppPrint(const char *format, ...)
#else
ppPrint(va_alist)
va_dcl
#endif
{
#if !(defined(__STDC__) || defined(__cplusplus))
    register
    char *format;
#endif
    va_list args;

#if defined(__STDC__) || defined(__cplusplus)
    va_start(args, format);
#else
    va_start(args);
    format = va_arg(args, char *);
#endif
    vfprintf(ppFile, format, args);
    va_end(args);
}

static void
setIndent()
{
    register
    int i;

    if (indentLevel == 0) {
        if (indent != (char *) 0) free(indent);
        indent = (char *) 0;
        return;
    }

    if (indent != (char *) 0) free(indent);
    indent = (char *) malloc((i = indentLevel * indentSize + 1));
    for(indent[--i] = '\0'; i > 0; indent[--i] = ' ');
}

/*
** Ease the pain a little ...
*/
#define PF ppPrint
#define INDENT PF("%s", indent != (char *) 0 ? indent : "")
#define IPF INDENT; PF
#define INC ++indentLevel; setIndent()
#define DEC --indentLevel; setIndent()
#define CHECK(_p) { if ((void *) _p == (void *) 0) { PF("(null)\n"); return; } }
#define OPEN PF("{\n")
#define CLOSE PF("}\n")
#define NAME(_s) PF("%s", indent == (char *) 0 ? _s : "")
#define START(_p, _s) { CHECK(_p); NAME(_s); OPEN; INC; }
#define END { DEC; INDENT; CLOSE; }
#define BADVALUE(_i, _l, _h) ((int) (_i) < (int) (_l) || (int) (_i) > (int) (_h))

static char *
#if defined(__STDC__) || defined(__cplusplus)
checkStr(const char **strList, int index, int limit)
#else
checkStr(strList, index, limit)
char **strList;
int index;
int limit;
#endif
{
    static
    char line[80]; /* I hope this is enough! */

    if (BADVALUE(index, 0, limit)) {
        sprintf(line, "%d /* bad value! */", index);
        return line;
    }

    return (char *) strList[index];
}

int
#if defined(__STDC__) || defined(__cplusplus)
ppSetIndentSize(int s)
#else
ppSetIndentSize(s)
int s;
#endif
{
    register
    int i;

    if (s < 0) s = 0; else if (s > 10) s = 10;

    i = indentSize;
    indentSize = s;
    return i;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppExternal(External *e)
#else
ppExternal(e)
External *e;
#endif
{
    CHECK(e); NAME("External");
    PF("<ppExternal() not implemented> (External *) %#08x\n", (u_long) e);
}

static
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char *CMISModifyOpStrAux[] = {
    "m_noModifyOp", /* = -1 */
    "m_replace",
    "m_addValue",
    "m_removeValue",
    "m_setToDefault",
    (char *) 0
};
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char **CMISModifyOpStr = &CMISModifyOpStrAux[1];

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISModifyOp(CMISModifyOp m)
#else
ppCMISModifyOp(m)
CMISModifyOp m;
#endif
{
    PF("%s\n", BADVALUE(m, m_noModifyOp, m_setToDefault) ? "<undefined>" :
    CMISModifyOpStrAux[(int) m + 1]);
}

static
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char *CMISSyncStrAux[] = {
    "s_invalid",
    "s_bestEffort",
    "s_atomic",
    (char *) 0
};
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char **CMISSyncStr = &CMISSyncStrAux[1];

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISSync(CMISSync s)
#else
ppCMISSync(s)
CMISSync s;
#endif
{
    PF("%s\n", BADVALUE(s, s_bestEffort, s_atomic) ? "<undefined>" :
    CMISSyncStr[(int) s]);
}

static
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char *CMISErrorsStrAux[] = {
    "m_noError",
    "m_noSuchObjectClass",
    "m_noSuchObjectInstance",
    "m_accessDenied",
    "m_syncNotSupported",
    "m_invalidFilter",
    "m_noSuchAttribute",
    "m_invalidAttributeValue",
    "m_getListError",
    "m_setListError",
    "m_noSuchAction",
    "m_processingFailure",
    "m_duplicateManagedObjectInstance",
    "m_noSuchReferenceObject",
    "m_noSuchEventType",
    "m_noSuchArgument",
    "m_invalidArgumentValue",
    "m_invalidScope",
    "m_invalidObjectInstance",
    "m_missingAttributeValue",
    "m_classInstanceConflict",
    "m_complexityLimitation",
    "m_mistypedOperation",
    "m_noSuchInvokeId",
    "m_operationCancelled",
    "m_invalidOperation",
    "m_invalidOperator",
    "m_duplicateInvocation",
    "m_unrecognisedOperation",
    "m_mistypedArgument",
    "m_resourceLimitation",
    (char *) 0
};
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char **CMISErrorsStr = &CMISErrorsStrAux[0];

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISErrors(CMISErrors e)
#else
ppCMISErrors(e)
CMISErrors e;
#endif
{
    PF("%s\n", BADVALUE(e, m_noError, m_invalidOperator) ? "<undefined>" :
    CMISErrorsStr[(int) e]);
}

static
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char *MID_TypeStrAux[] = {
    "0 /* bad value! */",
    "MID_GLOBAL",
    "MID_LOCAL",
    (char *) 0
};
#define MID_TypeLimit MSGARRAYSIZE(MID_TypeStrAux)
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char **MID_TypeStr = &MID_TypeStrAux[0];

void
#if defined(__STDC__) || defined(__cplusplus)
ppMIDentifier(MIDentifier *m)
#else
ppMIDentifier(m)
MIDentifier *m;
#endif
{
    register
    char *name;

    START(m, "MIDentifier");
    IPF("mid_type=%s\n", checkStr(MID_TypeStr, m->mid_type, MID_TypeLimit));
    INDENT;
    switch (m->mid_type) {

    case MID_GLOBAL :
        if (m->mid_global != (OID) 0) {
            if ((name = oid2name(m->mid_global, OIDPART)) == (char *) 0)
                name = sprintoid(m->mid_global);
        }
        else name = "(null)";
        PF("mid_global=%s\n", name); break;

    case MID_LOCAL :
        PF("mid_local=%d\n", m->mid_local); break;
    }
    END;
}

static
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char *MN_TypeStrAux[] = {
    "0 /* bad value! */",
    "MN_DN",
    "MN_NONSPEC",
    "MN_LOCALDN",
    (char *) 0
};
#define MN_TypeLimit MSGARRAYSIZE(MN_TypeStrAux)
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char **MN_TypeStr = &MN_TypeStrAux[0];

void
#if defined(__STDC__) || defined(__cplusplus)
ppMName(MName *m)
#else
ppMName(m)
MName *m;
#endif
{
    register char *cp;

    START(m, "MName");
    IPF("mn_type=%s\n", checkStr(MN_TypeStr, m->mn_type, MN_TypeLimit));
    INDENT;
    switch (m->mn_type) {

    case MN_DN :
    case MN_LOCALDN :
        PF("mn_dn=%s\n", cp = dn2str(m->mn_dn)); if (cp) free(cp); break;

    case MN_NONSPEC :
        PF("mn_nonspec=%s\n", m->mn_nonspec); break;
    }
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISParam(CMISParam *p)
#else
ppCMISParam(p)
CMISParam *p;
#endif
{
    char *cp = (char *) 0, *value = (char *) 0;
    int ret;

    START(p, "CMISParam");
    IPF("mp_id="); ppMIDentifier(&p->mp_id);
    switch (p->mp_id.mid_type) {

    case MID_GLOBAL :
	if (!actionOrEventArg)
            ret = ava2str(p->mp_id.mid_global, p->mp_val, &cp, &value);
	else
            ret = ava2str_info(p->mp_id.mid_global, p->mp_val, &cp, &value);
	if (ret != OK)
            cp = value = (char *) 0;
        break;

    case MID_LOCAL :
        value = strdup("<could not pretty print>");
        break;
    }
    if (value == (char *) 0) value = strdup("<could not convert mp_val>");
    IPF("mp_val=%s\n", value);
    if (cp != (char *) 0) free(cp);
    if (value != (char *) 0) free(value);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISGetAttr(CMISGetAttr *g)
#else
ppCMISGetAttr(g)
CMISGetAttr *g;
#endif
{
    START(g, "CMISGetAttr");
    IPF("ga_ava="); ppCMISParam(&g->ga_ava);
    IPF("ga_error="); ppCMISErrors(g->ga_error);
    END;
}


void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISSetAttr(CMISSetAttr *s)
#else
ppCMISSetAttr(s)
CMISSetAttr *s;
#endif
{
    START(s, "CMISSetAttr");
    IPF("sa_ava="); ppCMISParam(&s->sa_ava);
    IPF("sa_error="); ppCMISErrors(s->sa_error);
    IPF("sa_modify="); ppCMISModifyOp(s->sa_modify);
    END;
}

static
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char *SC_TypeStrAux[] = {
    "Sc_BaseObject",
    "Sc_FirstLevel",
    "Sc_WholeSubtree",
    "Sc_IndividualLevel",
    "Sc_BaseToNthLevel",
    (char *) 0
};
#define SC_TypeLimit MSGARRAYSIZE(SC_TypeStrAux)
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char **SC_TypeStr = &SC_TypeStrAux[0];

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISScope(CMISScope *s)
#else
ppCMISScope(s)
CMISScope *s;
#endif
{
    START(s, "CMISScope");
    IPF("sc_type=%s", checkStr(SC_TypeStr, s->sc_type, SC_TypeLimit));
    IPF("sc_level=%d\n", s->sc_level);
    END;
}

static
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char *FI_SubTypeStrAux[] = {
    "0 /* bad value! */",
    "FI_Substring_Initial",
    "FI_Substring_Any",
    "FI_Substring_Final",
    (char *) 0
};
#define FI_SubTypeLimit MSGARRAYSIZE(FI_SubTypeStrAux)
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char **FI_SubTypeStr = &FI_SubTypeStrAux[0];

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISFilterSubstrings(CMISFilterSubstrings *fs)
#else
ppCMISFilterSubstrings(fs)
CMISFilterSubstrings *fs;
#endif
{
    register
    CMISFilterSubstrings *f;

    START(fs, "CMISFilterSubstrings");
    for(f = fs; f != (CMISFilterSubstrings *) 0; f = f->fi_sub_next); {
        INDENT; OPEN;
        IPF("fi_sub_type=%s\n", checkStr(FI_SubTypeStr, f->fi_sub_type, FI_SubTypeLimit));
        IPF("fi_sub_attrid="); ppMIDentifier(&f->fi_sub_attrid);
        IPF("fi_sub_string=%s\n", f->fi_sub_string);
        INDENT; CLOSE;
    }
    END;
}

static
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char *FI_TypeStrAux[] = {
    "0 /* bad value! */",
    "FI_Equality",
    "FI_Substrings",
    "FI_GreaterOrEqual",
    "FI_LessOrEqual",
    "FI_Present",
    "FI_SubsetOf",
    "FI_SupersetOf",
    "FI_NonNullSetIntersect",
    (char *) 0
};
#define FI_TypeLimit MSGARRAYSIZE(FI_TypeStrAux)
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char **FI_TypeStr = &FI_TypeStrAux[0];

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISFilterItem(CMISFilterItem *f)
#else
ppCMISFilterItem(f)
CMISFilterItem *f;
#endif
{
    START(f, "CMISFilterItem");
    IPF("fi_type=%s\n", checkStr(FI_TypeStr, f->fi_type, FI_TypeLimit));
    INDENT;
    switch (f->fi_type) {

    case FI_Substrings :
        PF("fi_sunbstrings="); ppCMISFilterSubstrings(f->fi_substrings);
        break;

    default :
        PF("fi_attr="); ppCMISParam(&f->fi_attr);
        break;
    }
    END;
}

static
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char *FLT_TypeStrAux[] = {
    "0 /* bad value! */",
    "Filter_Item",
    "Filter_And",
    "Filter_Or",
    "Filter_Not",
    (char *) 0
};
#define FLT_TypeLimit MSGARRAYSIZE(FLT_TypeStrAux)
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char **FLT_TypeStr = &FLT_TypeStrAux[0];

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISFilter(CMISFilter *cf)
#else
ppCMISFilter(cf)
CMISFilter *cf;
#endif
{
    register
    CMISFilter *f;
    register
    CMISFilterItem *fi;

    START(cf, "CMISFilter");
    for(f = cf; f != (CMISFilter *) 0; f = f->flt_next) {
        INDENT; OPEN;
        IPF("flt_type=%s\n", checkStr(FLT_TypeStr, f->flt_type, FLT_TypeLimit));
        switch (f->flt_type) {

        case Filter_Item :
            IPF("flt_item=");  ppCMISFilterItem(f->flt_item);
            break;

         default :
            IPF("flt_filter="); ppCMISFilter(f->flt_filter);
            break;
        }
        INDENT; CLOSE;
    }
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISEventReportArg(CMISEventReportArg *e)
#else
ppCMISEventReportArg(e)
CMISEventReportArg *e;
#endif
{
    char *cp = (char *) 0, *value = (char *) 0;

    START(e, "CMISEventReportArg");
    IPF("ea_class="); ppMIDentifier(&e->ea_class);
    IPF("ea_inst="); ppMName(&e->ea_inst);
    IPF("ea_time=%s\n", e->ea_time);
    IPF("ea_type="); ppMIDentifier(&e->ea_type);
    switch (e->ea_type.mid_type) {

    case MID_GLOBAL :
        if (ava2str(e->ea_class.mid_global, e->ea_info, &cp, &value) != OK)
            cp = value = (char *) 0;
        break;

    case MID_LOCAL :
        value = strdup("<could not pretty print>");
        break;
    }
    if (value == (char *) 0) value = strdup("<could not convert ea_info>");
    IPF("ea_info=%s\n", value);
    if (cp != (char *) 0) free(cp);
    if (value != (char *) 0) free(value);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISEventReportRes(CMISEventReportRes *e)
#else
ppCMISEventReportRes(e)
CMISEventReportRes *e;
#endif
{
    START(e, "CMISEventReportRes");
    IPF("er_class="); ppMIDentifier(&e->er_class);
    IPF("er_inst="); ppMName(&e->er_inst);
    IPF("er_time=%s\n", e->er_time);
    IPF("er_reply="); ppCMISParam(&e->er_reply);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISGetArg(CMISGetArg *g)
#else
ppCMISGetArg(g)
CMISGetArg *g;
#endif
{
    register
    int i;

    START(g, "CMISGetArg");
    IPF("ga_class="); ppMIDentifier(&g->ga_class);
    IPF("ga_inst="); ppMName(&g->ga_inst);
    IPF("ga_access="); ppExternal(g->ga_access);
    IPF("ga_sync="); ppCMISSync(g->ga_sync);
    IPF("ga_scope="); ppCMISScope(&g->ga_scope);
    IPF("ga_filter="); ppCMISFilter(g->ga_filter);
    IPF("ga_nattrs=%d\n", g->ga_nattrs);
    for(i = 0; i < g->ga_nattrs; ++i) {
        IPF("ga_attrs[%d]=", i);
        ppMIDentifier(&g->ga_attrs[i]);
    }
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISGetRes(CMISGetRes *g)
#else
ppCMISGetRes(g)
CMISGetRes *g;
#endif
{
    register
    int i;

    START(g, "CMISGetRes");
    IPF("gr_class="); ppMIDentifier(&g->gr_class);
    IPF("gr_inst="); ppMName(&g->gr_inst);
    IPF("gr_time=%s\n", g->gr_time);
    IPF("gr_nattrs=%d\n", g->gr_nattrs);
    for(i = 0; i < g->gr_nattrs; ++i) {
        IPF("gr_attrs[%d]=", i);
        ppCMISGetAttr(&g->gr_attrs[i]);
    }
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISSetArg(CMISSetArg *s)
#else
ppCMISSetArg(s)
CMISSetArg *s;
#endif
{
    register
    int i;

    START(s, "CMISGetArg");
    IPF("sa_class="); ppMIDentifier(&s->sa_class);
    IPF("sa_inst="); ppMName(&s->sa_inst);
    IPF("sa_access="); ppExternal(s->sa_access);
    IPF("sa_sync="); ppCMISSync(s->sa_sync);
    IPF("sa_scope="); ppCMISScope(&s->sa_scope);
    IPF("sa_filter="); ppCMISFilter(s->sa_filter);
    IPF("sa_nattrs=%d\n", s->sa_nattrs);
    for(i = 0; i < s->sa_nattrs; ++i) {
        IPF("sa_attrs[%d]=", i);
        ppCMISSetAttr(&s->sa_attrs[i]);
    }
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISSetRes(CMISSetRes *s)
#else
ppCMISSetRes(s)
CMISSetRes *s;
#endif
{
    register
    int i;

    START(s, "CMISSetRes");
    IPF("sr_class="); ppMIDentifier(&s->sr_class);
    IPF("sr_inst="); ppMName(&s->sr_inst);
    IPF("sr_time=%s\n", s->sr_time);
    IPF("sr_nattrs=%d\n", s->sr_nattrs);
    for(i = 0; i < s->sr_nattrs; ++i) {
        IPF("sr_attrs[%d]=", i);
        ppCMISSetAttr(&s->sr_attrs[i]);
    }
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISActionArg(CMISActionArg *a)
#else
ppCMISActionArg(a)
CMISActionArg *a;
#endif
{
    START(a, "CMISActionArg");
    IPF("aa_class="); ppMIDentifier(&a->aa_class);
    IPF("aa_inst="); ppMName(&a->aa_inst);
    IPF("aa_access="); ppExternal(a->aa_access);
    IPF("aa_sync="); ppCMISSync(a->aa_sync);
    IPF("aa_scope=");  ppCMISScope(&a->aa_scope);
    IPF("aa_filter="); ppCMISFilter(a->aa_filter);
    IPF("aa_info="); ppCMISParam(&a->aa_info);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISActionRes(CMISActionRes *a)
#else
ppCMISActionRes(a)
CMISActionRes *a;
#endif
{
    START(a, "CMISActionRes");
    IPF("ar_class="); ppMIDentifier(&a->ar_class);
    IPF("ar_inst="); ppMName(&a->ar_inst);
    IPF("ar_time=%s\n", a->ar_time);
    IPF("ar_reply="); ppCMISParam(&a->ar_reply);
    END;
}

static
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char *CA_InstTypeStrAux[] = {
    "0 /* bad value! */",
    "CA_OBJECT_INST",
    "CA_SUPERIOR_INST",
    (char *) 0
};
#define CA_InstTypeLimit MSGARRAYSIZE(CA_InstTypeStrAux)
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char **CA_InstTypeStr = &CA_InstTypeStrAux[0];

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISCreateArg(CMISCreateArg *c)
#else
ppCMISCreateArg(c)
CMISCreateArg *c;
#endif
{
    register
    int i;

    START(c, "CMISCreateArg");
    IPF("ca_class="); ppMIDentifier(&c->ca_class);
    IPF("ca_inst="); ppMName(&c->ca_inst);
    IPF("ca_inst_type=%s\n", checkStr(CA_InstTypeStr, c->ca_inst_type, CA_InstTypeLimit));
    IPF("ca_reference_inst="); ppMName(&c->ca_reference_inst);
    IPF("ca_access="); ppExternal(c->ca_access);
    IPF("ca_nattrs=%d\n", c->ca_nattrs);
    for(i = 0; i < c->ca_nattrs; ++i) {
        IPF("ca_attrs[%d]=", i);
        ppCMISParam(&c->ca_attrs[i]);
    }
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISCreateRes(CMISCreateRes *c)
#else
ppCMISCreateRes(c)
CMISCreateRes *c;
#endif
{
    register
    int i;

    START(c, "CMISCreateRes");
    IPF("cr_class="); ppMIDentifier(&c->cr_class);
    IPF("cr_inst="); ppMName(&c->cr_inst);
    IPF("cr_time=%s\n", c->cr_time);
     IPF("cr_nattrs=%d\n", c->cr_nattrs);
    for(i = 0; i < c->cr_nattrs; ++i) {
        IPF("cr_attrs[%d]=", i);
        ppCMISParam(&c->cr_attrs[i]);
    }
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISDeleteArg(CMISDeleteArg *d)
#else
ppCMISDeleteArg(d)
CMISDeleteArg *d;
#endif
{
    START(d, "CMISDeleteArg");
    IPF("da_class="); ppMIDentifier(&d->da_class);
    IPF("da_inst="); ppMName(&d->da_inst);
    IPF("da_access="); ppExternal(d->da_access);
    IPF("da_sync="); ppCMISSync(d->da_sync);
    IPF("da_scope="); ppCMISScope(&d->da_scope);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISDeleteRes(CMISDeleteRes *d)
#else
ppCMISDeleteRes(d)
CMISDeleteRes *d;
#endif
{
    START(d, "CMISDeleteRes");
    IPF("dr_class="); ppMIDentifier(&d->dr_class);
    IPF("dr_inst="); ppMName(&d->dr_inst);
    IPF("dr_time=%s\n", d->dr_time);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISCancelGetArg(CMISCancelGetArg *c)
#else
ppCMISCancelGetArg(c)
CMISCancelGetArg *c;
#endif
{
    START(c, "CMISCancelGetArg");
    IPF("cga_getinvoke=%d\n", c->cga_getinvoke);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISNoSuchArgument(CMISNoSuchArgument *n)
#else
ppCMISNoSuchArgument(n)
CMISNoSuchArgument *n;
#endif
{
    START(n, "CMISNoSuchArgument");
    IPF("nsa_class="); ppMIDentifier(&n->nsa_class);
    IPF("nsa_type="); ppMIDentifier(&n->nsa_type);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISBaseManagedObjectId(CMISBaseManagedObjectId *c)
#else
ppCMISBaseManagedObjectId(c)
CMISBaseManagedObjectId *c;
#endif
{
    START(c, "CMISBaseManagedObjectId");
    IPF("bmo_class="); ppMIDentifier(&c->bmo_class);
    IPF("bmo_inst="); ppMName(&c->bmo_inst);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISProcessingFailure(CMISProcessingFailure *p)
#else
ppCMISProcessingFailure(p)
CMISProcessingFailure *p;
#endif
{
    START(p, "CMISProcessingFailure");
    IPF("pf_class="); ppMIDentifier(&p->pf_class);
    IPF("pf_inst="); ppMName(&p->pf_inst);
    IPF("pf_error="); ppCMISParam(&p->pf_error);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISComplexityLimitation(CMISComplexityLimitation *c)
#else
ppCMISComplexityLimitation(c)
CMISComplexityLimitation *c;
#endif
{
    START(c, "CMISComplexityLimitation");
    IPF("cl_scope="); ppCMISScope(&c->cl_scope);
    IPF("cl_filter="); ppCMISFilter(c->cl_filter);
    IPF("cl_sync="); ppCMISSync(c->cl_sync);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISActionError(CMISActionError *a)
#else
ppCMISActionError(a)
CMISActionError *a;
#endif
{
    START(a, "CMISActionError");
    IPF("ae_class="); ppMIDentifier(&a->ae_class);
    IPF("ae_inst="); ppMName(&a->ae_inst);
    IPF("ae_time=%s\n", a->ae_time);
    IPF("ae_error="); ppCMISErrors(a->ae_error);
    INDENT;
    switch ((int) a->ae_error) {

    case (int) m_accessDenied :
        PF("ae_accessDenied="); ppMIDentifier(&a->ae_accessDenied);
        break;

    case (int) m_noSuchAction :
        PF("ae_noSuchAction="); ppMIDentifier(&a->ae_noSuchAction);
        break;

    case (int) m_noSuchArgument :
        PF("ae_noSuchArgument="); ppCMISNoSuchArgument(&a->ae_noSuchArgument);
         break;

    case (int) m_invalidArgumentValue :
	actionOrEventArg = 1;
        PF("ae_invalidArgumentValue="); ppCMISParam(&a->ae_invalidArgumentValue);
	actionOrEventArg = 0;
        break;
    }
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISDeleteError(CMISDeleteError *d)
#else
ppCMISDeleteError(d)
CMISDeleteError *d;
#endif
{
    START(d, "CMISDeleteError");
    IPF("de_class="); ppMIDentifier(&d->de_class);
    IPF("de_inst=");  ppMName(&d->de_inst);
    IPF("de_time=%s\n", d->de_time);
    IPF("de_error="); ppCMISErrors(d->de_error);
    END;
}

static
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char *LR_TypeStrAux[] = {
    "0 /* bad value! */",
    "LR_GET_RES",
    "LR_GET_LISTERR",
    "LR_SET_RES",
    "LR_SET_LISTERR",
    "LR_ACTION_RES",
    "LR_PROC_FAILURE",
    "LR_DELETE_RES",
    "LR_ACTION_ERR",
    "LR_DELETE_ERR",
    (char *) 0
};
#define LR_TypeLimit MSGARRAYSIZE(LR_TypeStrAux)
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char **LR_TypeStr = &LR_TypeStrAux[0];

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISLinkedReplyArg(CMISLinkedReplyArg *l)
#else
ppCMISLinkedReplyArg(l)
CMISLinkedReplyArg *l;
#endif
{
    START(l, "CMISLinkedReplyArg");
    IPF("lr_linkid=%#08x\n", (u_int) l->lr_linkid);
    IPF("lr_type=%s\n", checkStr(LR_TypeStr, l->lr_type, LR_TypeLimit));
    INDENT;
    switch (l->lr_type) {

    case LR_GET_RES :
    case LR_GET_LISTERR :
        PF("lr_getres="); ppCMISGetRes(&l->lr_getres);
        break;

    case LR_SET_RES :
    case LR_SET_LISTERR :
        PF("lr_setres="); ppCMISSetRes(&l->lr_setres);
        break;

    case LR_ACTION_RES :
        PF("lr_actionres="); ppCMISActionRes(&l->lr_actionres);
        break;

    case LR_PROC_FAILURE :
        PF("lr_procfailure=");  ppCMISProcessingFailure(&l->lr_procfailure);
         break;

    case LR_DELETE_RES :
        PF("lr_deleteres="); ppCMISDeleteRes(&l->lr_deleteres);
        break;

    case LR_ACTION_ERR :
        PF("lr_actionerr="); ppCMISActionError(&l->lr_actionerr);
        break;

    case LR_DELETE_ERR :
        PF("lr_deleteerr="); ppCMISDeleteError(&l->lr_deleteerr);
        break;
    }
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppCMISErrorInfo(CMISErrors e, CMISErrorInfo *ei)
#else
ppCMISErrorInfo(e, ei)
CMISErrors e;
CMISErrorInfo *ei;
#endif
{
    START(ei, "CMISErrorInfo");
    INDENT;
    switch ((int) e) {

    case (int) m_noError :
    case (int) m_mistypedOperation :
    case (int) m_operationCancelled :
    case (int) m_getListError :
    case (int) m_setListError :
        PF("\n");
        break;

    case (int) m_noSuchObjectClass :
        PF("ei_noSuchObjectClass="); ppMIDentifier(&ei->ei_noSuchObjectClass);
        break;

    case (int) m_noSuchAttribute :
        PF("ei_noSuchAttribute="); ppMIDentifier(&ei->ei_noSuchAttribute);
        break;

    case (int) m_accessDenied :
        /*
        PF("ei_linkedActionAccessDenied="); ppMIDentifier(&ei->ei_linkedActionAccessDenied);
	*/
        break;

    case (int) m_missingAttributeValue :
	fprintf(stderr, "missingAttributeValue: no ppMIDListVal function (yet)\n");
	/*
        PF("ei_missingAttributeValue="); ppMIDListVal(&ei->ei_missingAttributeValue);
	*/
        break;

    case (int) m_noSuchObjectInstance :
        PF("ei_noSuchObjectInstance="); ppMName(&ei->ei_noSuchObjectInstance);
        break;

    case (int) m_invalidObjectInstance :
        PF("ei_invalidObjectInstance="); ppMName(&ei->ei_invalidObjectInstance);
        break;

    case (int) m_duplicateManagedObjectInstance :
        PF("ei_duplicateObjectInstance="); ppMName(&ei->ei_duplicateObjectInstance);
        break;

    case (int) m_noSuchReferenceObject :
        PF("ei_noSuchReferenceObject="); ppMName(&ei->ei_noSuchReferenceObject);
        break;

     case (int) m_invalidAttributeValue :
        PF("ei_invalidAttributeValue="); ppCMISParam(&ei->ei_invalidAttributeValue);
        break;

    case (int) m_invalidScope :
        PF("ei_invalidScope="); ppCMISScope(&ei->ei_invalidScope);
        break;

    case (int) m_invalidFilter :
        PF("ei_invalidFilter="); ppCMISFilter(ei->ei_invalidFilter);
        break;

    case (int) m_syncNotSupported :
        PF("ei_syncNotSupported="); ppCMISSync(ei->ei_syncNotSupported);
        break;

    case (int) m_noSuchInvokeId :
        PF("ei_noSuchInvokeId=%#08x\n", (u_int) ei->ei_noSuchInvokeId);
        break;

    case (int) m_noSuchAction :
        PF("ei_noSuchAction="); ppCMISNoSuchArgument(&ei->ei_noSuchAction);
        break;

    case (int) m_noSuchEventType :
        PF("ei_noSuchEventType="); ppCMISNoSuchArgument(&ei->ei_noSuchEventType);
        break;

    case (int) m_noSuchArgument :
        PF("ei_noSuchArgument="); ppCMISNoSuchArgument(&ei->ei_noSuchArgument);
        break;

    case (int) m_invalidArgumentValue :
	actionOrEventArg = 1;
        PF("ei_invalidArgumentValue="); ppCMISParam(&ei->ei_invalidArgumentValue);
	actionOrEventArg = 0;
        break;

    case (int) m_processingFailure :
        PF("ei_processingFailure="); ppCMISProcessingFailure(&ei->ei_processingFailure);
        break;

    case (int) m_complexityLimitation :
        PF("ei_complexityLimitation="); ppCMISComplexityLimitation(&ei->ei_complexityLimitation);
        break;

    case (int) m_classInstanceConflict :
        PF("ei_classInstanceConflict="); ppCMISBaseManagedObjectId(&ei->ei_classInstanceConflict);
        break;
    }
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppAEInfo(AEInfo *a)
#else
ppAEInfo(a)
AEInfo *a;
#endif
{
    CHECK(a); NAME("AEInfo");
    PF("AEI - %s\n", a != (AEInfo *) 0 ? sprintaei(a) : "(null)");
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPAssocInfo(MSAPAssocInfo *a)
#else
ppMSAPAssocInfo(a)
MSAPAssocInfo *a;
#endif
{
    START(a, "MSAPAssocInfo");
    IPF("mai_protvrsn=%d\n", a->mai_protvrsn);
    IPF("mai_funits=%#08x\n", (u_int) a->mai_funits);
    IPF("mai_access="); ppExternal(a->mai_access);
    IPF("mai_info="); ppExternal(a->mai_info);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPConnect(MSAPConnect *c)
#else
ppMSAPConnect(c)
MSAPConnect *c;
#endif
{
    START(c, "MSAPConnect");
    IPF("mc_sd=%d\n", c->mc_sd);
    IPF("mc_result=%d\n", c->mc_result);
    IPF("mc_context=%s\n", c->mc_context != (OID) 0 ? oid2name(c->mc_context, OIDPART) : "(null)");
    IPF("mc_resptitle="); ppAEInfo(&c->mc_resptitle);
    IPF("mc_info="); ppMSAPAssocInfo(&c->mc_info);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPStart(MSAPStart *s)
#else
ppMSAPStart(s)
MSAPStart *s;
#endif
{
    START(s, "MSAPStart");
    IPF("ms_sd=%d\n", s->ms_sd);
    IPF("ms_calling="); ppAEInfo(&s->ms_calling);
    IPF("ms_called="); ppAEInfo(&s->ms_called);
    IPF("ms_context=%s\n", s->ms_context != (OID) 0 ? oid2name(s->ms_context, OIDPART) : "(null)");
    IPF("ms_info="); ppMSAPAssocInfo(&s->ms_info);
    IPF("ms_start=<not yet implmented>\n");
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPTerm(MSAPTerm *t)
#else
ppMSAPTerm(t)
MSAPTerm *t;
#endif
{
    START(t, "MSAPTerm");
    IPF("mt_reason=%d\n", t->mt_reason);
    IPF("mt_info=<not yet implmented>\n");
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPUReject(MSAPUReject *u)
#else
ppMSAPUReject(u)
MSAPUReject *u;
#endif
{
    register
    char *cp;
    register
    int i;

    START(u, "MSAPUReject");
    IPF("mur_reason=%d\n", u->mur_reason);
    IPF("mur_id=%#08x\n", (u_int) u->mur_id);
    cp = (char *) malloc((i = sizeof(u->mur_data)) + 1);
    bcopy(u->mur_data, cp, i);
    cp[i] = '\0';
    IPF("mur_data=%s\n", cp);
    free(cp);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPPReject(MSAPPReject *p)
#else
ppMSAPPReject(p)
MSAPPReject *p;
#endif
{
    register
    char *cp;
    register
    int i;

    START(p, "MSAPPReject");
    IPF("mpr_reason=%d\n", p->mpr_reason);
    IPF("mpr_id=%#08x\n", (u_int) p->mpr_id);
    cp = (char *) malloc((i = sizeof(p->mpr_data)) + 1);
    bcopy((void *) p->mpr_data, (void *) cp, i);
    cp[i] = '\0';
    IPF("mpr_data=%s\n", cp);
    free(cp);
    END;
}

static
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char *MA_SourceStrAux[] = {
    "MA_USER /* user-initiated */",
    "MA_PROVIDER /* provider-initiated */",
    "MA_LOCAL /* localy-initiated */",
    (char *) 0
};
#define MA_SourceLimit MSGARRAYSIZE(MA_SourceStrAux)
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char **MA_SourceStr = &MA_SourceStrAux[0];

static
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char *MA_ReasonStrAux[] = {
    /* First 30 are not recognised! */
    "0",  "1",  "2",  "3",  "4",  "5",  "6",  "7",  "8",  "9",
    "10", "11", "12", "13", "14", "15", "16", "17", "18", "19",
    "20", "21", "22", "23", "24", "25", "26", "26", "28", "29",
    "MA_NOVRSN /* no common CMIP version */",
    "MA_BADPCTX /* bad presentation context */",
    "MA_BADARGS /* bad argument */",
    "MA_BADINFO /* bad assoc user info */",
    "MA_ROS /* can't set ROS service */",
    "MA_NOTREL /* peer refused to release */",
    "MA_ABORTED /* peer aborted the association */",
    (char *) 0
};
#define MA_ReasonLimit MSGARRAYSIZE(MA_ReasonStrAux)
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char **MA_ReasonStr = &MA_ReasonStrAux[0];

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPAbort(MSAPAbort *a)
#else
ppMSAPAbort(a)
MSAPAbort *a;
#endif
{
    register
    char *cp;
    register
    int i;

    START(a, "MSAPAbort");
    IPF("ma_source=%s\n", checkStr(MA_SourceStr, a->ma_source, MA_SourceLimit));
    IPF("ma_reason=%s\n", checkStr(MA_ReasonStr, a->ma_reason, MA_ReasonLimit));
    IPF("ma_info="); ppExternal(a->ma_info);
    cp = (char *) malloc((i = sizeof(a->ma_data)) + 1);
    bcopy((void *) a->ma_data, (void *) cp, i);
    cp[i] = '\0';
    IPF("ma_data=%s\n", cp);
    free(cp);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPError(CMISErrors e, MSAPError *me)
#else
ppMSAPError(e, me)
CMISErrors e;
MSAPError *me;
#endif
{
    START(me, "MSAPError");
    IPF("id=%#08x\n", (u_int) me->id);
    IPF("info="); ppCMISErrorInfo(e, &me->info);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPEventReportInv(MSAPEventReportInv *e)
#else
ppMSAPEventReportInv(e)
MSAPEventReportInv *e;
#endif
{
    START(e, "MSAPEventReportInv");
    IPF("id=%#08x\n", (u_int) e->id);
    IPF("args="); ppCMISEventReportArg(&e->args);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPEventReportRes(MSAPEventReportRes *e)
#else
ppMSAPEventReportRes(e)
MSAPEventReportRes *e;
#endif
{
    START(e, "MSAPEventReportRes");
    IPF("id=%#08x\n", (u_int) e->id);
    IPF("result="); ppCMISEventReportRes(&e->result);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPLinkedReplyInv(MSAPLinkedReplyInv *l)
#else
ppMSAPLinkedReplyInv(l)
MSAPLinkedReplyInv *l;
#endif
{
    START(l, "MSAPLinkedReplyInv");
    IPF("id=%#08x\n", (u_int) l->id);
    IPF("args="); ppCMISLinkedReplyArg(&l->args);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPGetInv(MSAPGetInv *g)
#else
ppMSAPGetInv(g)
MSAPGetInv *g;
#endif
{
    START(g, "MSAPGetInv");
    IPF("id=%#08x\n", (u_int) g->id);
    IPF("args="); ppCMISGetArg(&g->args);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPGetRes(MSAPGetRes *g)
#else
ppMSAPGetRes(g)
MSAPGetRes *g;
#endif
{
    START(g, "MSAPGetRes");
    IPF("id=%#08x\n", (u_int) g->id);
    IPF("result="); ppCMISGetRes(&g->result);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPSetInv(MSAPSetInv *s)
#else
ppMSAPSetInv(s)
MSAPSetInv *s;
#endif
{
    START(s, "MSAPSetInv");
    IPF("id=%#08x\n", (u_int) s->id);
    IPF("args="); ppCMISSetArg(&s->args);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPSetRes(MSAPSetRes *s)
#else
ppMSAPSetRes(s)
MSAPSetRes *s;
#endif
{
    START(s, "MSAPSetRes");
    IPF("id=%#08x\n", (u_int) s->id);
    IPF("result="); ppCMISSetRes(&s->result);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPActionInv(MSAPActionInv *a)
#else
ppMSAPActionInv(a)
MSAPActionInv *a;
#endif
{
    START(a, "MSAPActionInv");
    IPF("id=%#08x\n", (u_int) a->id);
    IPF("args="); ppCMISActionArg(&a->args);
     END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPActionRes(MSAPActionRes *a)
#else
ppMSAPActionRes(a)
MSAPActionRes *a;
#endif
{
    START(a, "MSAPActionRes");
    IPF("id=%#08x\n", (u_int) a->id);
    IPF("result="); ppCMISActionRes(&a->result);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPCreateInv(MSAPCreateInv *c)
#else
ppMSAPCreateInv(c)
MSAPCreateInv *c;
#endif
{
    START(c, "MSAPCreateInv");
    IPF("id=%#08x\n", (u_int) c->id);
    IPF("args="); ppCMISCreateArg(&c->args);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPCreateRes(MSAPCreateRes *c)
#else
ppMSAPCreateRes(c)
MSAPCreateRes *c;
#endif
{
    START(c, "MSAPCreateRes");
    IPF("id=%#08x\n", (u_int) c->id);
    IPF("result="); ppCMISCreateRes(&c->result);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPDeleteInv(MSAPDeleteInv *d)
#else
ppMSAPDeleteInv(d)
MSAPDeleteInv *d;
#endif
{
    START(d, "MSAPDeleteInv");
    IPF("id=%#08x\n", (u_int) d->id);
    IPF("args="); ppCMISDeleteArg(&d->args);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPDeleteRes(MSAPDeleteRes *d)
#else
ppMSAPDeleteRes(d)
MSAPDeleteRes *d;
#endif
{
    START(d, "MSAPDeleteRes");
    IPF("id=%#08x\n", (u_int) d->id);
    IPF("result="); ppCMISDeleteRes(&d->result);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPCancelGetInv(MSAPCancelGetInv *c)
#else
ppMSAPCancelGetInv(c)
MSAPCancelGetInv *c;
#endif
{
    START(c, "MSAPCancleGetInv");
    IPF("id=%#08x\n", (u_int) c->id);
    IPF("args="); ppCMISCancelGetArg(&c->args);
    END;
}

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPEmptyRes(MSAPEmptyRes *e)
#else
ppMSAPEmptyRes(e)
MSAPEmptyRes *e;
#endif
{
    START(e, "MSAPEmptyRes");
    IPF("id=%#08x\n", (u_int) e->id);
    END;
}

static
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char *MI_TypeStrAux[] = {
    "MI_UNKNOWN /* unknown type */",
    "MI_UREJECT /* user reject */",
    "MI_PREJECT /* provider reject */",
    "MI_ABORT /* abort */",
    "MI_TERM /* user-initiated terminate */",
    "MI_ERROR /* CMIP error */",
    "MI_EVENT_REP /* event report */",
    "MI_EVENT_REPC /* event report confirmed */",
    "MI_EVENT_REP_RES /* event report result */",
    "MI_LINKED_REPLY /* linked reply */",
    "MI_GET /* get */",
    "MI_GET_RES /* get result */",
    "MI_SET /* set */",
    "MI_SETC /* set confirmation */",
    "MI_SET_RES /* set result */",
    "MI_ACTION /* action */",
    "MI_ACTIONC /* action confirmed */",
    "MI_ACTION_RES /* action result */",
    "MI_CREATE /* create */",
    "MI_CREATE_RES /* create result */",
    "MI_DELETE /* delete */",
    "MI_DELETE_RES /* delete result */",
    "MI_CANCEL_GET /* cancel get */",
    "MI_EMPTY_RES /* empty result */",
    (char *) 0
};
#define MI_TypeLimit MSGARRAYSIZE(MI_TypeStrAux)
#if defined(__STDC__) || defined(__cplusplus)
const
#endif
char **MI_TypeStr = &MI_TypeStrAux[0];

void
#if defined(__STDC__) || defined(__cplusplus)
ppMSAPIndication(MSAPIndication *m)
#else
ppMSAPIndication(m)
MSAPIndication *m;
#endif
{
    START(m, "MSAPIndication");
    IPF("mi_type=%s\n", checkStr(MI_TypeStr, m->mi_type, MI_TypeLimit));
    IPF("mi_invoke=%#08x\n", (u_int) m->mi_invoke);
    IPF("mi_errtype="); ppCMISErrors(m->mi_errtype);
    INDENT;
    switch (m->mi_type) {

    case MI_UNKNOWN :
    case MI_EVENT_REPC :
    case MI_SETC :
    case MI_ACTIONC :
        PF("\n");
        break;

    case MI_UREJECT :
	PF("mi_ureject="); ppMSAPUReject(&m->mi_ureject);
	break;

    case MI_PREJECT :
        PF("mi_preject="); ppMSAPPReject(&m->mi_preject);
        break;

    case MI_ABORT :
        PF("mi_abort="); ppMSAPAbort(&m->mi_abort);
        break;

    case MI_TERM :
        PF("mi_term="); ppMSAPTerm(&m->mi_term);
        break;

    case MI_ERROR :
        PF("mi_error="); ppMSAPError(m->mi_errtype, &m->mi_error);
        break;

    case MI_EVENT_REP :
        PF("mi_eventrepinv="); ppMSAPEventReportInv(&m->mi_eventrepinv);
        break;

    case MI_EVENT_REP_RES :
        PF("mi_eventrepres="); ppMSAPEventReportRes(&m->mi_eventrepres);
        break;

    case MI_LINKED_REPLY :
        PF("mi_linkedreplyinv="); ppMSAPLinkedReplyInv(&m->mi_linkedreplyinv);
        break;

    case MI_GET :
        PF("mi_getinv="); ppMSAPGetInv(&m->mi_getinv);
        break;

    case MI_GET_RES :
        PF("mi_getres="); ppMSAPGetRes(&m->mi_getres);
        break;

    case MI_SET :
        PF("mi_setinv="); ppMSAPSetInv(&m->mi_setinv);
        break;

    case MI_SET_RES :
        PF("mi_setres="); ppMSAPSetRes(&m->mi_setres);
        break;

    case MI_ACTION :
        PF("mi_actioninv="); ppMSAPActionInv(&m->mi_actioninv);
        break;

    case MI_ACTION_RES :
        PF("mi_actionres="); ppMSAPActionRes(&m->mi_actionres);
        break;

    case MI_CREATE :
        PF("mi_createinv="); ppMSAPCreateInv(&m->mi_createinv);
        break;

    case MI_CREATE_RES :
        PF("mi_createres="); ppMSAPCreateRes(&m->mi_createres);
        break;

    case MI_DELETE :
        PF("mi_deleteinv="); ppMSAPDeleteInv(&m->mi_deleteinv);
        break;

    case MI_DELETE_RES :
        PF("mi_deleteres="); ppMSAPDeleteRes(&m->mi_deleteres);
        break;

    case MI_CANCEL_GET :
        PF("mi_cancelgetinv="); ppMSAPCancelGetInv(&m->mi_cancelgetinv);
        break;

    case MI_EMPTY_RES :
        PF("mi_emptyres="); ppMSAPEmptyRes(&m->mi_emptyres);
        break;
    }
    END;
}

#undef MID_TypeLimit
#undef MN_TypeLimit
#undef SC_TypeLimit
#undef FI_SubTypeLimit
#undef FI_TypeLimit
#undef FLT_TypeLimit
#undef CA_InstTypeLimit
#undef LR_TypeLimit
#undef MA_SourceLimit
#undef MA_ReasonLimit
#undef MI_TypeLimit

#undef PF
#undef INDENT
#undef IPF
#undef INC
#undef DEC
#undef CHECK(_p)
#undef OPEN
#undef CLOSE
#undef NAME(_s)
#undef START(_p, _s)
#undef END
#undef BADVALUE(_i, _l, _h)

#endif /* _msap_pp_c_ */
