--
-- Copyright (c) 1988 University College London
-- All rights reserved.
--
-- Redistribution and use in source and binary forms are permitted
-- provided that the above copyright notice and this paragraph are
-- duplicated in all such forms and that any documentation,
-- advertising materials, and other materials related to such
-- distribution and use acknowledge that the software was developed
-- by the Department of Computer Science, University College London.
-- The name of the University may not be used to
-- endorse or promote products derived from this software without
-- specific prior written permission.
-- THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
-- IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
-- WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
--

-- msap.py - CMIP ASN.1 module, manually augmented with
--           pepsy compiler directives

-- originally done using the pepy ASN.1 compiler by George Pavlou, June 1991
-- converted to use pepsy by George Pavlou, June 1994


CMIP	-- ISO 9596-CMIP2
{ joint-iso-ccitt ms(9) cmip(1) version2(2) abstractSyntax(4) }

DEFINITIONS ::=


%{
#include <stdio.h>

int moperation_id;	/* to encode NoSuchArgument, InvalidArgumentValue */
%}


PREFIXES build parse print

BEGIN


-- Directory Service Definitions

IMPORTS DistinguishedName, RDNSequence FROM IF
{ joint-iso-ccitt ds(5) modules(1) informationFramework(1) };



-- Association Establishment Information Carried in userInformation
-- of ACSE A-ASSOCIATE


CMIPUserInfo [[P struct MSAPAssocInfo *]] ::=
    SEQUENCE
    {
    %E{ if (parm->mai_protvrsn != CMIP_VERSION1) {
	    parm->mai_blen = 2;
	    parm->mai_bstring = int2strb(parm->mai_protvrsn, parm->mai_blen);
	}
    %}
    %D{ (*parm)->mai_protvrsn = CMIP_VERSION1; /* default */
	(*parm)->mai_blen = 0;
    %}

    protocolVersion	[0] IMPLICIT BIT STRING [[x mai_bstring $ mai_blen]]
			{ version1 (0), version2 (1) }

    %D{ if ((*parm)->mai_blen) {
	    (*parm)->mai_protvrsn = strb2int((*parm)->mai_bstring,
							(*parm)->mai_blen);
	    free((*parm)->mai_bstring);
	    (*parm)->mai_bstring = NULLCP; (*parm)->mai_blen = 0;
	}
    %}
                        -- DEFAULT { version1 },
			OPTIONAL <E< parm->mai_protvrsn != CMIP_VERSION1 >>
				 <D< (*parm)->mai_protvrsn = CMIP_VERSION1 >>,

    %E{ if (parm->mai_funits) {
	    parm->mai_blen = 5;
	    parm->mai_bstring = int2strb(parm->mai_funits, parm->mai_blen);
	}
    %}
    %D{ (*parm)->mai_funits = 0; /* default */ (*parm)->mai_blen = 0; %}

    functionalUnits	[1] IMPLICIT BIT STRING [[x mai_bstring $ mai_blen]]
			{ multipleObjectSelection (0),
			  filter (1),
			  multipleReply (2),
			  extendedService (3),
			  confirmedCancelGet (4) }

    %D{ if ((*parm)->mai_blen) {
	    (*parm)->mai_funits = strb2int((*parm)->mai_bstring,
							(*parm)->mai_blen);
	    free((*parm)->mai_bstring);
	    (*parm)->mai_bstring = NULLCP; (*parm)->mai_blen = 0;
	}
    %}
                        -- DEFAULT {},
			OPTIONAL <E< parm->mai_funits != 0 >>
				 <D< (*parm)->mai_funits = 0 >>,

    accessControl	[2] EXTERNAL [[p mai_access]]
			OPTIONAL <E< parm->mai_access != NULLEXTERN >>
				 <D< (*parm)->mai_access = NULLEXTERN >>,
    userInfo		[3] EXTERNAL [[p mai_info]]
			OPTIONAL <E< parm->mai_info != NULLEXTERN >>
				 <D< (*parm)->mai_info = NULLEXTERN >>
    }


-- Association Abort Information Carried in ACSE A-ABORT
--


CMIPAbortInfo [[P struct MSAPAbort *]] ::= SEQUENCE
    {
    abortSource		[0] IMPLICIT ENUMERATED [[i ma_source]]
			{ cmiseServiceUser (0), cmiseServiceProvider (1) },
    userinfo		[1] EXTERNAL [[p ma_info]]
			OPTIONAL <E< parm->ma_info != NULLEXTERN >>
				 <D< (*parm)->ma_info = NULLEXTERN >>
    }


--
-- CMIP definitions
--


-- note that the produced type_SMI_ObjectClass structure is identical
-- to MIDentifier (augmentation is not really needed)

ObjectClass [[P MIDentifier*]] ::= CHOICE
    [[T struct MIDentifier * $ *]] << mid_type >>
    {
    globalForm      [0] IMPLICIT OBJECT IDENTIFIER [[O mid_global]],
    localForm       [1] IMPLICIT INTEGER [[i mid_local]]
    }


ObjectInstance [[P MName*]] ::= CHOICE
    [[T struct MName * $ *]] << mn_type >>
    {
    distinguishedName	    [2] IMPLICIT DistinguishedName [[p mn_dn]]
				[[F dn_free]],
    nonSpecificForm         [3] IMPLICIT OCTET STRING [[s mn_nonspec]]
				[[F sfree]],
    localDistinguishedName  [4] IMPLICIT RDNSequence [[p mn_dn]]
				[[F dn_free]]
    }


-- note that the produced type_SMI_AttributeId structure is identical
-- to MIDentifier (augmentation is not really needed)

AttributeId [[P MIDentifier*]] ::= CHOICE
    [[T struct MIDentifier * $ *]] << mid_type >>
    {
    globalForm      [0] IMPLICIT OBJECT IDENTIFIER [[O mid_global]],
    localForm       [1] IMPLICIT INTEGER [[i mid_local]]
    }


Attribute [[P CMISParam *]] ::= SEQUENCE
    {
    attributeId		CHOICE
	[[T CMISParam * $ *]] << mp_id.mid_type >>
	{
        globalForm      [0] IMPLICIT OBJECT IDENTIFIER [[O mp_id.mid_global]],
        localForm       [1] IMPLICIT INTEGER [[i mp_id.mid_local]]
	},
    attributeValue	ANY DEFINED BY attributeId [[a mp_val]]
    } [[F Attribute_free]]


AttributeIdList [[P MIDListVal*]] ::= SET OF
    [[T struct MIDListVal * $ *]] << midl_next >>
    CHOICE
    [[T struct MIDListVal * $ *]] << midl_mid.mid_type >>
    {
    globalForm      [0] IMPLICIT OBJECT IDENTIFIER [[O midl_mid.mid_global]],
    localForm       [1] IMPLICIT INTEGER [[i midl_mid.mid_local]]
    }


AttributeList [[P CMISParamListVal*]] ::= SET OF
    [[T CMISParamListVal * $ *]] << mpl_next >>
    SEQUENCE
    [[T CMISParamListVal * $ *]]
    {
    attributeId		CHOICE
	[[T CMISParamListVal * $ *]] << mpl_ava.mp_id.mid_type >>
	{
        globalForm      [0] IMPLICIT OBJECT IDENTIFIER
			    [[O mpl_ava.mp_id.mid_global]],
        localForm       [1] IMPLICIT INTEGER
			    [[i mpl_ava.mp_id.mid_local]]
	},
    attributeValue	ANY DEFINED BY attributeId [[a mpl_ava.mp_val]]
    } [[F AttributeList_free]]


-- note that the produced type_SMI_EventTypeId structure is identical
-- to MIDentifier (augmentation is not really needed)

EventTypeId [[P MIDentifier*]] ::= CHOICE
    [[T struct MIDentifier * $ *]] << mid_type >>
    {
    globalForm      [6] IMPLICIT OBJECT IDENTIFIER [[O mid_global]],
    localForm       [7] IMPLICIT INTEGER [[i mid_local]]
    }


Scope [[P CMISScope *]] ::= CHOICE
    <E< parm -> sc_type == Sc_BaseToNthLevel ? 3 :
       (parm -> sc_type == Sc_IndividualLevel ? 2 : 1) >>
    <D<0>>
    {
    namedNumbers	INTEGER [[i sc_type]]
			{ baseObject (0),
                  	  firstLevelOnly (1),
                  	  wholeSubtree (2) }
			%D{ (*parm)->sc_level = 0; %},
    individualLevels	[1] IMPLICIT INTEGER [[i sc_level]]
			%D{ (*parm)->sc_type = Sc_IndividualLevel; %},
    baseToNthLevel	[2] IMPLICIT INTEGER [[i sc_level]]
			%D{ (*parm)->sc_type = Sc_BaseToNthLevel; %}
    }


FilterItem [[P struct CMISFilterItem *]] ::= CHOICE
    [[T struct CMISFilterItem * $ *]] << fi_type >>
    {
    equality	[0] IMPLICIT SEQUENCE
        [[T struct CMISFilterItem * $ *]]
	{
        attributeId	CHOICE
	    [[T struct CMISFilterItem * $ *]] << fi_attr.mp_id.mid_type >>
	    {
            globalForm	[0] IMPLICIT OBJECT IDENTIFIER
				[[O fi_attr.mp_id.mid_global]],
            localForm	[1] IMPLICIT INTEGER
				[[i fi_attr.mp_id.mid_local]]
	    },
	attributeValue	ANY DEFINED BY attributeId [[a fi_attr.mp_val]]
        },

    substrings	[1] IMPLICIT SEQUENCE OF
	[[T struct CMISFilterSubstrings * $ fi_substrings]] << fi_sub_next >>
	CHOICE
	[[T struct CMISFilterSubstrings * $ *]] << fi_sub_type >>
	{
	initialString	[0] IMPLICIT SEQUENCE
	    [[T struct CMISFilterSubstrings * $ *]]
	    {
	    attributeId	CHOICE
		[[T CMISFilterSubstrings * $ *]] << fi_sub_attrid.mid_type >>
		{
		globalForm  [0] IMPLICIT OBJECT IDENTIFIER
				[[O fi_sub_attrid.mid_global]],
		localForm   [1] IMPLICIT INTEGER
				[[i fi_sub_attrid.mid_local]]
		},
	    string	ANY DEFINED BY attributeId
			[[a fi_sub_string]]
	    },
	anyString	[1] IMPLICIT SEQUENCE
	    [[T struct CMISFilterSubstrings * $ *]]
	    {
	    attributeId	CHOICE
		[[T CMISFilterSubstrings * $ *]] << fi_sub_attrid.mid_type >>
		{
		globalForm  [0] IMPLICIT OBJECT IDENTIFIER
				[[O fi_sub_attrid.mid_global]],
		localForm   [1] IMPLICIT INTEGER
				[[i fi_sub_attrid.mid_local]]
		},
	    string	ANY DEFINED BY attributeId
			[[a fi_sub_string]]
	    },
	finalString	[2] IMPLICIT SEQUENCE
	    [[T struct CMISFilterSubstrings * $ *]]
	    {
	    attributeId	CHOICE
		[[T CMISFilterSubstrings * $ *]] << fi_sub_attrid.mid_type >>
		{
		globalForm  [0] IMPLICIT OBJECT IDENTIFIER
				[[O fi_sub_attrid.mid_global]],
		localForm   [1] IMPLICIT INTEGER
				[[i fi_sub_attrid.mid_local]]
		},
	    string	ANY DEFINED BY attributeId
			[[a fi_sub_string]]
	    }
	},

    greaterOrEqual	[2] IMPLICIT SEQUENCE
        [[T struct CMISFilterItem * $ *]]
	{
        attributeId	CHOICE
	    [[T struct CMISFilterItem * $ *]] << fi_attr.mp_id.mid_type >>
	    {
            globalForm	[0] IMPLICIT OBJECT IDENTIFIER
				[[O fi_attr.mp_id.mid_global]],
            localForm	[1] IMPLICIT INTEGER
				[[i fi_attr.mp_id.mid_local]]
	    },
	attributeValue	ANY DEFINED BY attributeId [[a fi_attr.mp_val]]
        },

    lessOrEqual	[3] IMPLICIT SEQUENCE
        [[T struct CMISFilterItem * $ *]]
	{
        attributeId	CHOICE
	    [[T struct CMISFilterItem * $ *]] << fi_attr.mp_id.mid_type >>
	    {
            globalForm	[0] IMPLICIT OBJECT IDENTIFIER
				[[O fi_attr.mp_id.mid_global]],
            localForm	[1] IMPLICIT INTEGER
				[[i fi_attr.mp_id.mid_local]]
	    },
	attributeValue	ANY DEFINED BY attributeId [[a fi_attr.mp_val]]
        },

    -- the following is not inline expanded because pepsy does not like
    -- the inline expansion of a CHOICE

    %E{ parm->fi_present = &parm->fi_attr.mp_id; %}

    present	[4] IMPLICIT AttributeId [[p fi_present]]

    %D{ (*parm)->fi_attr.mp_id = *(*parm)->fi_present; /* struct cpy */
	free((char*) (*parm)->fi_present); %},

    subsetOf	[5] IMPLICIT SEQUENCE
        [[T struct CMISFilterItem * $ *]]
	{
        attributeId	CHOICE
	    [[T struct CMISFilterItem * $ *]] << fi_attr.mp_id.mid_type >>
	    {
            globalForm	[0] IMPLICIT OBJECT IDENTIFIER
				[[O fi_attr.mp_id.mid_global]],
            localForm	[1] IMPLICIT INTEGER
				[[i fi_attr.mp_id.mid_local]]
	    },
	attributeValue	ANY DEFINED BY attributeId [[a fi_attr.mp_val]]
        },

    supersetOf	[6] IMPLICIT SEQUENCE
        [[T struct CMISFilterItem * $ *]]
	{
        attributeId	CHOICE
	    [[T struct CMISFilterItem * $ *]] << fi_attr.mp_id.mid_type >>
	    {
            globalForm	[0] IMPLICIT OBJECT IDENTIFIER
				[[O fi_attr.mp_id.mid_global]],
            localForm	[1] IMPLICIT INTEGER
				[[i fi_attr.mp_id.mid_local]]
	    },
	attributeValue	ANY DEFINED BY attributeId [[a fi_attr.mp_val]]
        },

    nonNullIntersection	[7] IMPLICIT SEQUENCE
        [[T struct CMISFilterItem * $ *]]
	{
        attributeId	CHOICE
	    [[T struct CMISFilterItem * $ *]] << fi_attr.mp_id.mid_type >>
	    {
            globalForm	[0] IMPLICIT OBJECT IDENTIFIER
				[[O fi_attr.mp_id.mid_global]],
            localForm	[1] IMPLICIT INTEGER
				[[i fi_attr.mp_id.mid_local]]
	    },
	attributeValue	ANY DEFINED BY attributeId [[a fi_attr.mp_val]]
        }
    }


CMISFilter [[P struct CMISFilter *]] ::= CHOICE
    [[T struct CMISFilter * $ *]] << flt_type >>
    {
    item	[8] FilterItem [[p flt_item]],
    and		[9] IMPLICIT SET OF
		[[T struct CMISFilter * $ flt_filter]] << flt_next >>
		CMISFilter [[p *]],
    or		[10] IMPLICIT SET OF
		[[T struct CMISFilter * $ flt_filter]] << flt_next >>
		CMISFilter [[p *]],
    not		[11] CMISFilter [[p flt_filter]]
    } [[F mfilter_free1]]


CMISSync ::= ENUMERATED
    {
    bestEffort      (0),
    atomic          (1)
    }


BaseManagedObjectId [[P CMISBaseManagedObjectId *]] ::= SEQUENCE
    {
    baseManagedObjectClass CHOICE
	[[T CMISBaseManagedObjectId * $ *]] << bmo_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O bmo_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i bmo_class.mid_local]]
	},
    baseManagedObjectInstance CHOICE
	[[T CMISBaseManagedObjectId * $ *]] << bmo_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p bmo_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s bmo_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p bmo_inst.mn_dn]] [[F dn_free]]
	}
    }


NoSuchAction [[P CMISNoSuchArgument *]] ::= SEQUENCE
    {
    managedObjectClass	CHOICE
	[[T CMISNoSuchArgument * $ *]] << nsa_class.mid_type >>
	{
	globalForm  [0] IMPLICIT OBJECT IDENTIFIER [[O nsa_class.mid_global]],
	localForm   [1] IMPLICIT INTEGER [[i nsa_class.mid_local]]
	},
    actionType		CHOICE
	[[T CMISNoSuchArgument * $ *]] << nsa_type.mid_type >>
        {
        globalForm  [2] IMPLICIT OBJECT IDENTIFIER [[O nsa_type.mid_global]],
        localForm   [3] IMPLICIT INTEGER [[i nsa_type.mid_local]]
        }
    }


NoSuchEventType [[P CMISNoSuchArgument *]] ::= SEQUENCE
    {
    managedObjectClass	CHOICE
	[[T CMISNoSuchArgument * $ *]] << nsa_class.mid_type >>
	{
	globalForm  [0] IMPLICIT OBJECT IDENTIFIER [[O nsa_class.mid_global]],
	localForm   [1] IMPLICIT INTEGER [[i nsa_class.mid_local]]
	},
    eventType		CHOICE
	[[T CMISNoSuchArgument * $ *]] << nsa_type.mid_type >>
        {
        globalForm  [6] IMPLICIT OBJECT IDENTIFIER [[O nsa_type.mid_global]],
        localForm   [7] IMPLICIT INTEGER [[i nsa_type.mid_local]]
        }

    }


NoSuchArgument [[P CMISNoSuchArgument *]] ::= CHOICE
    <E< moperation_id == M_ACTIONC ? 1 :
       (moperation_id == M_EVENT_REPC ? 2 : 0) >>
    <D< moperation_id >>
    {
    actionId	[0] IMPLICIT SEQUENCE
		[[T CMISNoSuchArgument * $ *]]
                {
		managedObjectClass	CHOICE
		    [[T CMISNoSuchArgument * $ *]] << nsa_class.mid_type >>
		    {
		    globalForm  [0] IMPLICIT OBJECT IDENTIFIER
				    [[O nsa_class.mid_global]],
		    localForm   [1] IMPLICIT INTEGER
				    [[i nsa_class.mid_local]]
		    }
                OPTIONAL <E< parm->nsa_class.mid_global != NULLOID >>
			 <D< (*parm)->nsa_class.mid_global = NULLOID >>,
		actionType		CHOICE
		    [[T CMISNoSuchArgument * $ *]] << nsa_type.mid_type >>
        	    {
        	    globalForm  [2] IMPLICIT OBJECT IDENTIFIER
				    [[O nsa_type.mid_global]],
        	    localForm   [3] IMPLICIT INTEGER
				    [[i nsa_type.mid_local]]
        	    }
                }
		%D{ moperation_id = M_ACTIONC; %},
    eventId     [1] IMPLICIT SEQUENCE
		[[T CMISNoSuchArgument * $ *]]
                {
		managedObjectClass	CHOICE
		    [[T CMISNoSuchArgument * $ *]] << nsa_class.mid_type >>
		    {
		    globalForm  [0] IMPLICIT OBJECT IDENTIFIER
				    [[O nsa_class.mid_global]],
		    localForm   [1] IMPLICIT INTEGER
				    [[i nsa_class.mid_local]]
		    }
                OPTIONAL <E< parm->nsa_class.mid_global != NULLOID >>
			 <D< (*parm)->nsa_class.mid_global = NULLOID >>,
		eventType		CHOICE
		    [[T CMISNoSuchArgument * $ *]] << nsa_type.mid_type >>
        	    {
        	    globalForm  [6] IMPLICIT OBJECT IDENTIFIER
				    [[O nsa_type.mid_global]],
        	    localForm   [7] IMPLICIT INTEGER
				    [[i nsa_type.mid_local]]
        	    }
                }
		%D{ moperation_id = M_EVENT_REPC; %}
    }


InvalidArgumentValue [[P CMISInvalidArgumentValue *]] ::= CHOICE
    <E< moperation_id == M_ACTIONC ? 1 :
       (moperation_id == M_EVENT_REPC ? 2 : 0) >>
    <D< moperation_id >>
    {
    actionValue	[0] IMPLICIT SEQUENCE
		[[T CMISInvalidArgumentValue * $ *]]
    		{
		actionType	CHOICE
		    [[T CMISInvalidArgumentValue * $ *]] << iav_id.mid_type >>
        	    {
        	    globalForm  [2] IMPLICIT OBJECT IDENTIFIER
				    [[O iav_id.mid_global]],
        	    localForm   [3] IMPLICIT INTEGER
				    [[i iav_id.mid_local]]
        	    },
    		actionInfoArg   [4] ANY DEFINED BY actionType [[a iav_val]]
                OPTIONAL <E< parm -> iav_val != NULLPE >>
			 <D< (*parm) -> iav_val = NULLPE >>
		}
		%D{ moperation_id = M_ACTIONC; %},
    eventValue  [1] IMPLICIT SEQUENCE
		[[T CMISInvalidArgumentValue * $ *]]
                {
		eventType	CHOICE
		    [[T CMISInvalidArgumentValue * $ *]] << iav_id.mid_type >>
        	    {
        	    globalForm  [6] IMPLICIT OBJECT IDENTIFIER
				    [[O iav_id.mid_global]],
        	    localForm   [7] IMPLICIT INTEGER
				    [[i iav_id.mid_local]]
        	    },
    		eventInfo	[8] ANY DEFINED BY eventType [[a iav_val]]
                OPTIONAL <E< parm -> iav_val != NULLPE >>
			 <D< (*parm) -> iav_val = NULLPE >>
		}
		%D{ moperation_id = M_EVENT_REPC; %}
    }


ComplexityLimitation [[P CMISComplexityLimitation *]] ::= SET
    {
    scope   [0] CHOICE
        [[T CMISComplexityLimitation * $ *]]
        <E< parm->cl_scope.sc_type == Sc_BaseToNthLevel ? 3 :
           (parm->cl_scope.sc_type == Sc_IndividualLevel ? 2 : 1) >>
        <D< (*parm)->cl_scope.sc_type >>
        {
        namedNumbers	INTEGER [[i cl_scope.sc_type]]
			{ baseObject (0), firstLevelOnly (1), wholeSubtree (2) }
			%D{ (*parm)->cl_scope.sc_level = 0; %},
        individualLevels [1] IMPLICIT INTEGER [[i cl_scope.sc_level]]
			%D{ (*parm)->cl_scope.sc_type = Sc_IndividualLevel; %},
        baseToNthLevel	[2] IMPLICIT INTEGER [[i cl_scope.sc_level]]
			%D{ (*parm)->cl_scope.sc_type = Sc_BaseToNthLevel; %}
    	}
    	OPTIONAL <E< parm->cl_scope.sc_type != Sc_BaseObject >>
		 <D< (*parm)->cl_scope.sc_type = Sc_BaseObject >>,

    filter  [1] CMISFilter [[p cl_filter]]
        OPTIONAL <E< parm->cl_filter != NULLMFILTER >>
		 <D< (*parm)->cl_filter = NULLMFILTER >>,

    sync    [2] ENUMERATED [[i cl_sync]] { bestEffort (0), atomic (1) }
        OPTIONAL <E< parm->cl_sync != s_bestEffort >>
		 <D< (*parm)->cl_sync = s_bestEffort >>
    }


ProcessingFailure [[P CMISProcessingFailure *]] ::= SEQUENCE
    {
    managedObjectClass		CHOICE
	[[T CMISProcessingFailure * $ *]] << pf_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O pf_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i pf_class.mid_local]]
	},

    managedObjectInstance	CHOICE
	[[T CMISProcessingFailure * $ *]] << pf_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p pf_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s pf_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p pf_inst.mn_dn]] [[F dn_free]]
	}
	OPTIONAL <E< parm->pf_inst.mn_dn != NULLDN >>
		 <D< (*parm)->pf_inst.mn_dn = NULLDN >>,

    %D{ (*parm) -> pf_error.mp_id.mid_type = MID_GLOBAL; %}

    specificErrorInfo		[5] SEQUENCE
	[[T CMISProcessingFailure * $ *]]
        {
        errorId OBJECT IDENTIFIER [[O pf_error.mp_id.mid_global]],
        errorInfo ANY DEFINED BY errorId [[a pf_error.mp_val]]
        }
    }


EventReportArgument [[P CMISEventReportArg *]] ::= SEQUENCE
    {
    managedObjectClass		CHOICE
	[[T CMISEventReportArg * $ *]] << ea_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O ea_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i ea_class.mid_local]]
	},

    managedObjectInstance	CHOICE
	[[T CMISEventReportArg * $ *]] << ea_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p ea_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s ea_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p ea_inst.mn_dn]] [[F dn_free]]
	},

    eventTime			[5] IMPLICIT GeneralizedTime [[s ea_time]]
	OPTIONAL <E< parm->ea_time != NULLCP >> <D< (*parm)->ea_time=NULLCP >>,

    eventType			CHOICE
	[[T CMISEventReportArg * $ *]] << ea_type.mid_type >>
       	{
        globalForm  [6] IMPLICIT OBJECT IDENTIFIER [[O ea_type.mid_global]],
        localForm   [7] IMPLICIT INTEGER [[i ea_type.mid_local]]
        },

    eventInfo			[8] ANY DEFINED BY eventType [[a ea_info]]
	OPTIONAL <E< parm->ea_info != NULLPE >> <D< (*parm)->ea_info=NULLPE >>
    }


EventReportResult [[P CMISEventReportRes *]] ::= SEQUENCE
    {
    managedObjectClass		CHOICE
	[[T CMISEventReportRes * $ *]] << er_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O er_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i er_class.mid_local]]
	}
	OPTIONAL <E< parm->er_class.mid_global != NULLOID >>
		 <D< (*parm)->er_class.mid_type = MID_GLOBAL;
		     (*parm)->er_class.mid_global = NULLOID >>,

    managedObjectInstance	CHOICE
	[[T CMISEventReportRes * $ *]] << er_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p er_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s er_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p er_inst.mn_dn]] [[F dn_free]]
	}
	OPTIONAL <E< parm->er_inst.mn_dn != NULLDN >>
		 <D< (*parm)->er_inst.mn_type = MN_LOCALDN;
		     (*parm)->er_inst.mn_dn = NULLDN; >>,

    eventTime			[5] IMPLICIT GeneralizedTime [[s er_time]]
	OPTIONAL <E< parm->er_time != NULLCP >> <D< (*parm)->er_time=NULLCP >>,

    eventReply			SEQUENCE
	[[T CMISEventReportRes * $ *]]
	{
        eventType	CHOICE
	    [[T CMISEventReportRes * $ *]] << er_reply.mp_id.mid_type >>
       	    {
            globalForm  [6] IMPLICIT OBJECT IDENTIFIER
			[[O er_reply.mp_id.mid_global]],
            localForm   [7] IMPLICIT INTEGER
			[[i er_reply.mp_id.mid_local]]
            },

        eventReplyInfo	[8] ANY DEFINED BY eventType [[a er_reply.mp_val]]
	    OPTIONAL <E< parm->er_reply.mp_val != NULLPE >>
		     <D< (*parm)->er_reply.mp_val = NULLPE >>
	}
	OPTIONAL <E< parm->er_reply.mp_id.mid_global != NULLOID >>
		 <D< (*parm)->er_reply.mp_id.mid_type = MID_GLOBAL;
		     (*parm)->er_reply.mp_id.mid_global = NULLOID >>
    }


ActionArgument [[P CMISActionArg *]] ::= SEQUENCE
    {
    baseManagedObjectClass	CHOICE
	[[T CMISActionArg * $ *]] << aa_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O aa_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i aa_class.mid_local]]
	},

    baseManagedObjectInstance	CHOICE
	[[T CMISActionArg * $ *]] << aa_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p aa_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s aa_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p aa_inst.mn_dn]] [[F dn_free]]
	},

    accessControl		[5] EXTERNAL [[p aa_access]]
        			OPTIONAL <E< parm->aa_access != NULLEXTERN >>
		 			 <D< (*parm)->aa_access = NULLEXTERN >>,

    synchronization		[6] IMPLICIT ENUMERATED [[i aa_sync]]
				{ bestEffort (0), atomic (1) }
				-- DEFAULT bestEffort
        			OPTIONAL <E< parm->aa_sync != s_bestEffort >>
		 			 <D< (*parm)->aa_sync = s_bestEffort >>,

    scope			[7] CHOICE
	[[T CMISActionArg * $ *]] << aa_inst.mn_type >>
        <E< parm->aa_scope.sc_type == Sc_BaseToNthLevel ? 3 :
           (parm->aa_scope.sc_type == Sc_IndividualLevel ? 2 : 1) >>
        <D< (*parm)->aa_scope.sc_type >>
        {
        namedNumbers	    INTEGER [[i aa_scope.sc_type]]
			    { baseObject (0),
                  	      firstLevelOnly (1),
                  	      wholeSubtree (2) }
			  %D{ (*parm)->aa_scope.sc_level = 0; %},
        individualLevels    [1] IMPLICIT INTEGER [[i aa_scope.sc_level]]
			  %D{ (*parm)->aa_scope.sc_type = Sc_IndividualLevel;%},
        baseToNthLevel	    [2] IMPLICIT INTEGER [[i aa_scope.sc_level]]
			  %D{ (*parm)->aa_scope.sc_type = Sc_BaseToNthLevel; %}
        }
	-- DEFAULT baseObject
	OPTIONAL <E< parm->aa_scope.sc_type != Sc_BaseObject >>
		 <D< (*parm)->aa_scope.sc_type = Sc_BaseObject >>,

    filter			CMISFilter [[p aa_filter]]
				-- DEFAULT and {}
        			OPTIONAL <E< parm->aa_filter != NULLMFILTER >>
		 			 <D< (*parm)->aa_filter = NULLMFILTER>>,

    actionInfo			[12] IMPLICIT SEQUENCE
	[[T CMISActionArg * $ *]]
    	{
	actionType	CHOICE
	    [[T CMISActionArg * $ *]] << aa_info.mp_id.mid_type >>
            {
            globalForm  [2] IMPLICIT OBJECT IDENTIFIER
			    [[O aa_info.mp_id.mid_global]],
            localForm   [3] IMPLICIT INTEGER
			    [[i aa_info.mp_id.mid_local]]
            },
    	actionInfoArg   [4] ANY DEFINED BY actionType [[a aa_info.mp_val]]
            OPTIONAL <E< parm -> aa_info.mp_val != NULLPE >>
		     <D< (*parm) -> aa_info.mp_val = NULLPE >>
	}
    }


ActionResult [[P CMISActionRes *]] ::= SEQUENCE
    {
    managedObjectClass		CHOICE
	[[T CMISActionRes * $ *]] << ar_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O ar_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i ar_class.mid_local]]
	}
	OPTIONAL <E< parm->ar_class.mid_global != NULLOID >>
		 <D< (*parm)->ar_class.mid_type = MID_GLOBAL;
		     (*parm)->ar_class.mid_global = NULLOID >>,

    managedObjectInstance	CHOICE
	[[T CMISActionRes * $ *]] << ar_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p ar_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s ar_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p ar_inst.mn_dn]] [[F dn_free]]
	}
	OPTIONAL <E< parm->ar_inst.mn_dn != NULLDN >>
		 <D< (*parm)->ar_inst.mn_type = MN_LOCALDN;
		     (*parm)->ar_inst.mn_dn = NULLDN; >>,

    currentTime			[5] IMPLICIT GeneralizedTime [[s ar_time]]
	OPTIONAL <E< parm->ar_time != NULLCP >> <D< (*parm)->ar_time=NULLCP >>,

    actionReply			[6] IMPLICIT SEQUENCE
	[[T CMISActionRes * $ *]]
	{
        actionType	CHOICE
	    [[T CMISActionRes * $ *]] << ar_reply.mp_id.mid_type >>
       	    {
            globalForm  [2] IMPLICIT OBJECT IDENTIFIER
			[[O ar_reply.mp_id.mid_global]],
            localForm   [3] IMPLICIT INTEGER
			[[i ar_reply.mp_id.mid_local]]
            },

        actionReplyInfo	[4] ANY DEFINED BY actionType [[a ar_reply.mp_val]]
	}
	OPTIONAL <E< parm->ar_reply.mp_id.mid_global != NULLOID &&
		     parm->ar_reply.mp_val != NULLPE >>
		 <D< (*parm)->ar_reply.mp_id.mid_type = MID_GLOBAL;
		     (*parm)->ar_reply.mp_id.mid_global = NULLOID;
		     (*parm)->ar_reply.mp_val = NULLPE; >>
    }


ActionError [[P CMISActionError *]] ::= SEQUENCE
    {
    managedObjectClass		CHOICE
	[[T CMISActionError * $ *]] << ae_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O ae_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i ae_class.mid_local]]
	}
	OPTIONAL <E< parm->ae_class.mid_global != NULLOID >>
		 <D< (*parm)->ae_class.mid_type = MID_GLOBAL;
		     (*parm)->ae_class.mid_global = NULLOID >>,

    managedObjectInstance	CHOICE
	[[T CMISActionError * $ *]] << ae_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p ae_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s ae_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p ae_inst.mn_dn]] [[F dn_free]]
	}
	OPTIONAL <E< parm->ae_inst.mn_dn != NULLDN >>
		 <D< (*parm)->ae_inst.mn_type = MN_LOCALDN;
		     (*parm)->ae_inst.mn_dn = NULLDN; >>,

    currentTime			[5] IMPLICIT GeneralizedTime [[s ae_time]]
	OPTIONAL <E< parm->ae_time != NULLCP >> <D< (*parm)->ae_time=NULLCP >>,

    actionErrorInfo		[6] IMPLICIT SEQUENCE
	[[T CMISActionError * $ *]]
	{
			%E{ ((int) parm->ae_error) -= CMIS_ERROR_OFFSET; %}
	errorStatus	ENUMERATED [[i ae_error]]
			{ accessDenied (2),
			  noSuchAction (9),
			  noSuchArgument (14),
			  invalidArgumentValue (15) }
			%E{ ((int) parm->ae_error) += CMIS_ERROR_OFFSET; %}
			%D{ ((int) (*parm)->ae_error) += CMIS_ERROR_OFFSET; %},

	%D{ (*parm)->ae_accessDenied.mid_type = MID_GLOBAL; %}

	errorInfo	CHOICE
			[[T CMISActionError * $ *]]
			<E< parm->ae_error == m_accessDenied ||
			    parm->ae_error == m_noSuchAction ? 1 :
			   (parm->ae_error == m_noSuchArgument ? 2 :
			(parm->ae_error == m_invalidArgumentValue ? 3 : 0)) >>
			<D<0>>
		{
		-- the tag [2] below was added to avoid a pepsy bug;
		-- it does not help interoperability though

		actionType	[2] CHOICE
		    [[T CMISActionError * $ *]] << ae_accessDenied.mid_type >>
		    {
		    globalForm  [2] IMPLICIT OBJECT IDENTIFIER
				    [[O ae_accessDenied.mid_global]],
		    localForm   [3] IMPLICIT INTEGER
				    [[i ae_accessDenied.mid_local]]
		    },

		-- the following two choice items follow the standard "memory
		-- allocation" pepsy model for decoding, as inline expansion
		-- results in two successive tags (pepsy has problems)

		%E{ parm->ae_aux = (char*) &parm->ae_noSuchArgument %}

		actionArgument	[0] NoSuchArgument [[p ae_aux]]

		%D{ (*parm)->ae_noSuchArgument =
			*(CMISNoSuchArgument*) (*parm)->ae_aux; /* struct cpy */
		    free((*parm)->ae_aux); %},

		%E{ parm->ae_aux = (char*) &parm->ae_invalidArgumentValue %}

		argumentValue	[1] InvalidArgumentValue [[p ae_aux]]

		%D{ (*parm)->ae_invalidArgumentValue =
			*(CMISParam*) (*parm)->ae_aux; /* struct cpy */
		    free((*parm)->ae_aux); %}
		}
	}
    }


GetArgument [[P CMISGetArg *]] ::= SEQUENCE
    {
    baseManagedObjectClass	CHOICE
	[[T CMISGetArg * $ *]] << ga_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O ga_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i ga_class.mid_local]]
	},

    baseManagedObjectInstance	CHOICE
	[[T CMISGetArg * $ *]] << ga_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p ga_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s ga_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p ga_inst.mn_dn]] [[F dn_free]]
	},

    accessControl		[5] EXTERNAL [[p ga_access]]
        			OPTIONAL <E< parm->ga_access != NULLEXTERN >>
		 			 <D< (*parm)->ga_access = NULLEXTERN >>,

    synchronization		[6] IMPLICIT ENUMERATED [[i ga_sync]]
				{ bestEffort (0), atomic (1) }
				-- DEFAULT bestEffort
        			OPTIONAL <E< parm->ga_sync != s_bestEffort >>
		 			 <D< (*parm)->ga_sync = s_bestEffort >>,

    scope			[7] CHOICE
	[[T CMISGetArg * $ *]]
        <E< parm->ga_scope.sc_type == Sc_BaseToNthLevel ? 3 :
           (parm->ga_scope.sc_type == Sc_IndividualLevel ? 2 : 1) >>
        <D<0>>
        {
        namedNumbers	    INTEGER [[i ga_scope.sc_type]]
			    { baseObject (0),
                  	      firstLevelOnly (1),
                  	      wholeSubtree (2) }
			  %D{ (*parm)->ga_scope.sc_level = 0; %},
        individualLevels    [1] IMPLICIT INTEGER [[i ga_scope.sc_level]]
			  %D{ (*parm)->ga_scope.sc_type = Sc_IndividualLevel;%},
        baseToNthLevel	    [2] IMPLICIT INTEGER [[i ga_scope.sc_level]]
			  %D{ (*parm)->ga_scope.sc_type = Sc_BaseToNthLevel; %}
        }
	-- DEFAULT baseObject
	OPTIONAL <E< parm->ga_scope.sc_type != Sc_BaseObject >>
		 <D< (*parm)->ga_scope.sc_type = Sc_BaseObject >>,

    filter			CMISFilter [[p ga_filter]]
				-- DEFAULT and {}
        			OPTIONAL <E< parm->ga_filter != NULLMFILTER >>
		 			 <D< (*parm)->ga_filter = NULLMFILTER>>,

    %E{ array_encode(MA_ATTRID, parm->ga_nattrs, (caddr_t) parm->ga_attrs,
		     &parm->ga_pattrs) %}
    %D{ (*parm)->ga_nattrs = GET_NOATTRS; %}

    attributeIdList		[12] IMPLICIT SET OF
	[[T CMISAttr * $ ga_pattrs]] << ma_next >>
	CHOICE
	[[T CMISAttr * $ *]] << ma_id.mid_type >>
	{
	globalForm      [0] IMPLICIT OBJECT IDENTIFIER [[O ma_id.mid_global]],
	localForm       [1] IMPLICIT INTEGER [[i ma_id.mid_local]]
	}
	OPTIONAL <E< parm -> ga_nattrs != GET_ALLATTRS >>
		 <D< (*parm) -> ga_nattrs = GET_ALLATTRS; >>
    }
    %E{ array_free(parm->ga_pattrs) %}
    %D{ array_decode(MA_ATTRID, &(*parm)->ga_nattrs,
             (caddr_t*) &(*parm)->ga_attrs, (*parm)->ga_pattrs); %}


GetResult [[P CMISGetRes *]] ::= SEQUENCE
    {
    managedObjectClass		CHOICE
	[[T CMISGetRes * $ *]] << gr_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O gr_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i gr_class.mid_local]]
	}
	OPTIONAL <E< parm->gr_class.mid_global != NULLOID >>
		 <D< (*parm)->gr_class.mid_type = MID_GLOBAL;
		     (*parm)->gr_class.mid_global = NULLOID >>,

    managedObjectInstance	CHOICE
	[[T CMISGetRes * $ *]] << gr_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p gr_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s gr_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p gr_inst.mn_dn]] [[F dn_free]]
	}
	OPTIONAL <E< parm->gr_inst.mn_dn != NULLDN >>
		 <D< (*parm)->gr_inst.mn_type = MN_LOCALDN;
		     (*parm)->gr_inst.mn_dn = NULLDN; >>,

    currentTime			[5] IMPLICIT GeneralizedTime [[s gr_time]]
	OPTIONAL <E< parm->gr_time != NULLCP >> <D< (*parm)->gr_time=NULLCP >>,

    %E{ array_encode(MA_GETATTR, parm->gr_nattrs, (caddr_t) parm->gr_attrs,
		     &parm->gr_pattrs) %}

    attributeList		[6] IMPLICIT SET OF
				[[T CMISAttr * $ gr_pattrs]] << ma_next >>
				SEQUENCE
				[[T CMISAttr * $ *]]
				{
				attributeId		CHOICE
				    [[T CMISAttr * $ *]] << ma_id.mid_type >>
				    {
        			    globalForm	[0] IMPLICIT OBJECT IDENTIFIER
						[[O ma_id.mid_global]],
        			    localForm	[1] IMPLICIT INTEGER
						[[i ma_id.mid_local]]
				    }
				    %D{ (*parm) -> ma_error = m_noError %},
				attributeValue	ANY DEFINED BY attributeId
						[[a ma_val]]
				}
				OPTIONAL <E< parm -> gr_nattrs != 0 >>
					 <D< (*parm) -> gr_nattrs = 0 >>
    }
    %E{ array_free(parm->gr_pattrs) %}
    %D{ array_decode(MA_GETATTR, &(*parm)->gr_nattrs,
	     (caddr_t*) &(*parm)->gr_attrs, (*parm)->gr_pattrs) %}



GetListError [[P CMISGetRes *]] ::= SEQUENCE
    {
    managedObjectClass		CHOICE
	[[T CMISGetRes * $ *]] << gr_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O gr_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i gr_class.mid_local]]
	}
	OPTIONAL <E< parm->gr_class.mid_global != NULLOID >>
		 <D< (*parm)->gr_class.mid_type = MID_GLOBAL;
		     (*parm)->gr_class.mid_global = NULLOID >>,

    managedObjectInstance	CHOICE
	[[T CMISGetRes * $ *]] << gr_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p gr_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s gr_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p gr_inst.mn_dn]] [[F dn_free]]
	}
	OPTIONAL <E< parm->gr_inst.mn_dn != NULLDN >>
		 <D< (*parm)->gr_inst.mn_type = MN_LOCALDN;
		     (*parm)->gr_inst.mn_dn = NULLDN; >>,

    currentTime			[5] IMPLICIT GeneralizedTime [[s gr_time]]
	OPTIONAL <E< parm->gr_time != NULLCP >> <D< (*parm)->gr_time=NULLCP >>,

    %E{ array_encode(MA_GETATTR, parm->gr_nattrs, (caddr_t) parm->gr_attrs,

		     &parm->gr_pattrs) %}
    getInfoList			[6] IMPLICIT SET OF
	[[T CMISAttr * $ gr_pattrs]] << ma_next >>
	CHOICE [[T CMISAttr * $ *]]
	<E< parm -> ma_error == m_noError ? 2 : 1 >> <D<0>>
	{
	attributeIdError        [0] IMPLICIT
	    SEQUENCE [[T CMISAttr * $ *]]
	    {
			    %E{ ((int) parm->ma_error) -= CMIS_ERROR_OFFSET; %}
	    errorStatus     ENUMERATED [[i ma_error]]
			    { accessDenied (2),
                              noSuchAttribute (5) }
			%E{ ((int) parm->ma_error) += CMIS_ERROR_OFFSET; %}
			%D{ ((int) (*parm)->ma_error) += CMIS_ERROR_OFFSET; %},
	    attributeId     CHOICE
		[[T CMISAttr * $ *]] << ma_id.mid_type >>
		{
        	globalForm	[0] IMPLICIT OBJECT IDENTIFIER
				[[O ma_id.mid_global]],
        	localForm	[1] IMPLICIT INTEGER
				[[i ma_id.mid_local]]
		}
		%D{ (*parm)->ma_val = NULLPE %}
	    },

	attribute               [1] IMPLICIT
	    SEQUENCE [[T CMISAttr * $ *]]
	    {
	    attributeId		CHOICE
		[[T CMISAttr * $ *]] << ma_id.mid_type >>
		{
        	globalForm	[0] IMPLICIT OBJECT IDENTIFIER
				[[O ma_id.mid_global]],
        	localForm	[1] IMPLICIT INTEGER
				[[i ma_id.mid_local]]
		}
		%D{ (*parm)->ma_error = m_noError %},
	    attributeValue	ANY DEFINED BY attributeId [[a ma_val]]
	    }
	}
    }
    %E{ array_free(parm->gr_pattrs) %}
    %D{ array_decode(MA_GETATTR, &(*parm)->gr_nattrs,
	     (caddr_t*) &(*parm)->gr_attrs, (*parm)->gr_pattrs) %}


SetArgument [[P CMISSetArg *]] ::= SEQUENCE
    {
    baseManagedObjectClass	CHOICE
	[[T CMISSetArg * $ *]] << sa_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O sa_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i sa_class.mid_local]]
	},

    baseManagedObjectInstance	CHOICE
	[[T CMISSetArg * $ *]] << sa_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p sa_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s sa_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p sa_inst.mn_dn]] [[F dn_free]]
	},

    accessControl		[5] EXTERNAL [[p sa_access]]
        			OPTIONAL <E< parm->sa_access != NULLEXTERN >>
		 			 <D< (*parm)->sa_access = NULLEXTERN >>,

    synchronization		[6] IMPLICIT ENUMERATED [[i sa_sync]]
				{ bestEffort (0), atomic (1) }
				-- DEFAULT bestEffort
        			OPTIONAL <E< parm->sa_sync != s_bestEffort >>
		 			 <D< (*parm)->sa_sync = s_bestEffort >>,

    scope			[7] CHOICE
	[[T CMISSetArg * $ *]]
        <E< parm->sa_scope.sc_type == Sc_BaseToNthLevel ? 3 :
           (parm->sa_scope.sc_type == Sc_IndividualLevel ? 2 : 1) >>
        <D<0>>
        {
        namedNumbers	    INTEGER [[i sa_scope.sc_type]]
			    { baseObject (0),
                  	      firstLevelOnly (1),
                  	      wholeSubtree (2) }
			  %D{ (*parm)->sa_scope.sc_level = 0; %},
        individualLevels    [1] IMPLICIT INTEGER [[i sa_scope.sc_level]]
			  %D{ (*parm)->sa_scope.sc_type = Sc_IndividualLevel;%},
        baseToNthLevel	    [2] IMPLICIT INTEGER [[i sa_scope.sc_level]]
			  %D{ (*parm)->sa_scope.sc_type = Sc_BaseToNthLevel; %}
        }
	-- DEFAULT baseObject
	OPTIONAL <E< parm->sa_scope.sc_type != Sc_BaseObject >>
		 <D< (*parm)->sa_scope.sc_type = Sc_BaseObject >>,

    filter			CMISFilter [[p sa_filter]]
				-- DEFAULT and {}
        			OPTIONAL <E< parm->sa_filter != NULLMFILTER >>
		 			 <D< (*parm)->sa_filter = NULLMFILTER>>,

    %E{ array_encode(MA_SETATTR, parm->sa_nattrs, (caddr_t) parm->sa_attrs,
		     &parm->sa_pattrs) %}

    modificationList		[12] IMPLICIT SET OF
	[[T CMISAttr * $ sa_pattrs]] << ma_next >>
	SEQUENCE
	[[T CMISAttr * $ *]]
	{
	modifyOperator	[2] IMPLICIT ENUMERATED [[i ma_modify]]
			{ replace (0),
			  addValues (1),
			  removeValues (2),
			  setToDefault (3) }
			-- DEFAULT replace
        		OPTIONAL <E< parm->ma_modify != m_replace >>
		 		 <D< (*parm)->ma_modify = m_replace >>,

	attributeId	CHOICE
	    [[T CMISAttr * $ *]] << ma_id.mid_type >>
	    {
            globalForm	[0] IMPLICIT OBJECT IDENTIFIER
			[[O ma_id.mid_global]],
            localForm	[1] IMPLICIT INTEGER
			[[i ma_id.mid_local]]
	    }
	    %D{ (*parm)->ma_error = m_noError %},

	attributeValue  ANY DEFINED BY attributeId [[a ma_val]]
			OPTIONAL <E< parm->ma_modify != m_setToDefault >>
				 <D< (*parm)->ma_val = NULLPE; >>
        }
    }
    %E{ array_free(parm->sa_pattrs) %}
    %D{ array_decode(MA_SETATTR, &(*parm)->sa_nattrs,
	     (caddr_t*) &(*parm)->sa_attrs, (*parm)->sa_pattrs) %}


SetResult [[P CMISSetRes *]] ::= SEQUENCE
    {
    managedObjectClass		CHOICE
	[[T CMISSetRes * $ *]] << sr_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O sr_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i sr_class.mid_local]]
	}
	OPTIONAL <E< parm->sr_class.mid_global != NULLOID >>
		 <D< (*parm)->sr_class.mid_type = MID_GLOBAL;
		     (*parm)->sr_class.mid_global = NULLOID >>,

    managedObjectInstance	CHOICE
	[[T CMISSetRes * $ *]] << sr_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p sr_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s sr_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p sr_inst.mn_dn]] [[F dn_free]]
	}
	OPTIONAL <E< parm->sr_inst.mn_dn != NULLDN >>
		 <D< (*parm)->sr_inst.mn_type = MN_LOCALDN;
		     (*parm)->sr_inst.mn_dn = NULLDN; >>,

    currentTime			[5] IMPLICIT GeneralizedTime [[s sr_time]]
	OPTIONAL <E< parm->sr_time != NULLCP >> <D< (*parm)->sr_time=NULLCP >>,

    %E{ array_encode(MA_SETATTR, parm->sr_nattrs, (caddr_t) parm->sr_attrs,
		     &parm->sr_pattrs) %}

    attributeList		[6] IMPLICIT SET OF
				[[T CMISAttr * $ sr_pattrs]] << ma_next >>
				SEQUENCE
				[[T CMISAttr * $ *]]
				{
				attributeId		CHOICE
				    [[T CMISAttr * $ *]] << ma_id.mid_type >>
				    {
        			    globalForm	[0] IMPLICIT OBJECT IDENTIFIER
						[[O ma_id.mid_global]],
        			    localForm	[1] IMPLICIT INTEGER
						[[i ma_id.mid_local]]
				    }
				    %D{ (*parm) -> ma_error = m_noError;
					(*parm) -> ma_modify = m_noModifyOp %},
				attributeValue	ANY DEFINED BY attributeId
						[[a ma_val]]
				}
				OPTIONAL <E< parm -> sr_nattrs != 0 >>
					 <D< (*parm) -> sr_nattrs = 0 >>
    }
    %E{ array_free(parm->sr_pattrs) %}
    %D{ array_decode(MA_SETATTR, &(*parm)->sr_nattrs,
	     (caddr_t*) &(*parm)->sr_attrs, (*parm)->sr_pattrs) %}


SetListError [[P CMISSetRes *]] ::= SEQUENCE
    {
    managedObjectClass		CHOICE
	[[T CMISSetRes * $ *]] << sr_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O sr_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i sr_class.mid_local]]
	}
	OPTIONAL <E< parm->sr_class.mid_global != NULLOID >>
		 <D< (*parm)->sr_class.mid_type = MID_GLOBAL;
		     (*parm)->sr_class.mid_global = NULLOID >>,

    managedObjectInstance	CHOICE
	[[T CMISSetRes * $ *]] << sr_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p sr_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s sr_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p sr_inst.mn_dn]] [[F dn_free]]
	}
	OPTIONAL <E< parm->sr_inst.mn_dn != NULLDN >>
		 <D< (*parm)->sr_inst.mn_type = MN_LOCALDN;
		     (*parm)->sr_inst.mn_dn = NULLDN; >>,

    currentTime			[5] IMPLICIT GeneralizedTime [[s sr_time]]
	OPTIONAL <E< parm->sr_time != NULLCP >> <D< (*parm)->sr_time=NULLCP >>,

    %E{ array_encode(MA_SETATTR, parm->sr_nattrs, (caddr_t) parm->sr_attrs,
		     &parm->sr_pattrs) %}

    setInfoList			[6] IMPLICIT SET OF
	[[T CMISAttr * $ sr_pattrs]] << ma_next >>
	CHOICE [[T CMISAttr * $ *]]
	<E< parm -> ma_error == m_noError ? 2 : 1 >> <D<0>>
	{
	attributeError		[0] IMPLICIT
	    SEQUENCE [[T CMISAttr * $ *]]
	    {
			    %E{ ((int) parm->ma_error) -= CMIS_ERROR_OFFSET; %}
	    errorStatus     ENUMERATED [[i ma_error]]
			    { accessDenied (2),
                              noSuchAttribute (5),
			      invalidAttributeValue (6),
			      invalidOperation (24),
			      invalidOperator (25) }
			%E{ ((int) parm->ma_error) += CMIS_ERROR_OFFSET; %}
			%D{ ((int) (*parm)->ma_error) += CMIS_ERROR_OFFSET; %},

	    modifyOperator  [2] IMPLICIT ENUMERATED [[i ma_modify]]
			    { replace (0),
			      addValues (1),
			      removeValues (2),
			      setToDefault (3) }
			    -- DEFAULT replace
        		    OPTIONAL <E< parm->ma_error != m_invalidOperation ||
					 parm->ma_error != m_invalidOperator >>
		 		     <D< (*parm)->ma_modify = m_noModifyOp >>,

	    attributeId     CHOICE
		[[T CMISAttr * $ *]] << ma_id.mid_type >>
		{
        	globalForm	[0] IMPLICIT OBJECT IDENTIFIER
				[[O ma_id.mid_global]],
        	localForm	[1] IMPLICIT INTEGER
				[[i ma_id.mid_local]]
		},

	    attributeValue  ANY DEFINED BY attributeId [[a ma_val]]
		OPTIONAL <E< parm->ma_modify != m_setToDefault >>
			 <D< (*parm)->ma_val = NULLPE;
			     (*parm)->ma_modify = m_setToDefault >>
	    },

	attribute               [1] IMPLICIT
	    SEQUENCE [[T CMISAttr * $ *]]
	    {
	    attributeId		CHOICE
		[[T CMISAttr * $ *]] << ma_id.mid_type >>
		{
        	globalForm	[0] IMPLICIT OBJECT IDENTIFIER
				[[O ma_id.mid_global]],
        	localForm	[1] IMPLICIT INTEGER
				[[i ma_id.mid_local]]
		}
		%D{ (*parm)->ma_error = m_noError;
		    (*parm)->ma_modify = m_noModifyOp %},
	    attributeValue	ANY DEFINED BY attributeId [[a ma_val]]
	    }
	}
    }
    %E{ array_free(parm->sr_pattrs) %}
    %D{ array_decode(MA_SETATTR, &(*parm)->sr_nattrs,
	     (caddr_t*) &(*parm)->sr_attrs, (*parm)->sr_pattrs) %}


CreateArgument [[P CMISCreateArg *]] ::= SEQUENCE
    {
    managedObjectClass		CHOICE
	[[T CMISCreateArg * $ *]] << ca_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O ca_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i ca_class.mid_local]]
	},

    %D{ (*parm)->ca_inst.mn_type = MN_LOCALDN; (*parm)->ca_inst.mn_dn = NULLDN;
    %}

    -- the following is an expansion of the standard CMIP syntax as
    -- pepsy cannot deal with the standard one (coredumps for the superior
    -- choice); this fixes it temporarily but it will prevent interoperability
    -- for the superior choice (better than coredumping though...)

    instance	CHOICE
	[[T CMISCreateArg * $ *]] << ca_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p ca_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm		[3] IMPLICIT OCTET STRING
				[[s ca_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p ca_inst.mn_dn]] [[F dn_free]]
	}
	OPTIONAL <E< parm->ca_inst.mn_dn != NULLDN >>
		 <D<0>>,

    instanceType		[8] IMPLICIT ENUMERATED [[i ca_inst_type]]
				{ managedObjectInstance  (1),
				  superiorObjectInstance (2) }
				-- DEFAULT managedObjectInstance
        			OPTIONAL <E< parm->ca_inst.mn_dn != NULLDN &&
					     parm->ca_inst_type !=
						CA_OBJECT_INST >>
		 			 <D< (*parm)->ca_inst_type =
					 	CA_OBJECT_INST >>,

    accessControl		[5] EXTERNAL [[p ca_access]]
        			OPTIONAL <E< parm->ca_access != NULLEXTERN >>
		 			 <D< (*parm)->ca_access = NULLEXTERN >>,

    referenceObjectInstance	[6] CHOICE
	[[T CMISCreateArg * $ *]] << ca_reference_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p ca_reference_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s ca_reference_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p ca_reference_inst.mn_dn]] [[F dn_free]]
	}
	OPTIONAL <E< parm->ca_reference_inst.mn_dn != NULLDN >>
		 <D< (*parm)->ca_reference_inst.mn_type = MN_LOCALDN;
		     (*parm)->ca_reference_inst.mn_dn = NULLDN; >>,

    %E{ array_encode(MA_ATTR, parm->ca_nattrs, (caddr_t) parm->ca_attrs,
		     &parm->ca_pattrs) %}

    attributeList		[7] IMPLICIT SET OF
				[[T CMISAttr * $ ca_pattrs]] << ma_next >>
				SEQUENCE
				[[T CMISAttr * $ *]]
				{
				attributeId		CHOICE
				    [[T CMISAttr * $ *]] << ma_id.mid_type >>
				    {
        			    globalForm	[0] IMPLICIT OBJECT IDENTIFIER
						[[O ma_id.mid_global]],
        			    localForm	[1] IMPLICIT INTEGER
						[[i ma_id.mid_local]]
				    }
				    %D{ (*parm) -> ma_error = m_noError %},
				attributeValue	ANY DEFINED BY attributeId
						[[a ma_val]]
				}
				OPTIONAL <E< parm -> ca_nattrs != 0 >>
					 <D< (*parm) -> ca_nattrs = 0 >>
    }
    %E{ array_free(parm->ca_pattrs) %}
    %D{ array_decode(MA_ATTR, &(*parm)->ca_nattrs,
	     (caddr_t*) &(*parm)->ca_attrs, (*parm)->ca_pattrs) %}


CreateResult [[P CMISCreateRes *]] ::= SEQUENCE
    {
    managedObjectClass		CHOICE
	[[T CMISCreateRes * $ *]] << cr_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O cr_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i cr_class.mid_local]]
	}
	OPTIONAL <E< parm->cr_class.mid_global != NULLOID >>
		 <D< (*parm)->cr_class.mid_type = MID_GLOBAL;
		     (*parm)->cr_class.mid_global = NULLOID >>,

    managedObjectInstance	CHOICE
	[[T CMISCreateRes * $ *]] << cr_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p cr_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s cr_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p cr_inst.mn_dn]] [[F dn_free]]
	}
	OPTIONAL <E< parm->cr_inst.mn_dn != NULLDN >>
		 <D< (*parm)->cr_inst.mn_type = MN_LOCALDN;
		     (*parm)->cr_inst.mn_dn = NULLDN; >>,

    currentTime			[5] IMPLICIT GeneralizedTime [[s cr_time]]
	OPTIONAL <E< parm->cr_time != NULLCP >> <D< (*parm)->cr_time=NULLCP >>,

    %E{ array_encode(MA_ATTR, parm->cr_nattrs, (caddr_t) parm->cr_attrs,
		     &parm->cr_pattrs) %}

    attributeList		[6] IMPLICIT SET OF
				[[T CMISAttr * $ cr_pattrs]] << ma_next >>
				SEQUENCE
				[[T CMISAttr * $ *]]
				{
				attributeId		CHOICE
				    [[T CMISAttr * $ *]] << ma_id.mid_type >>
				    {
        			    globalForm	[0] IMPLICIT OBJECT IDENTIFIER
						[[O ma_id.mid_global]],
        			    localForm	[1] IMPLICIT INTEGER
						[[i ma_id.mid_local]]
				    }
				    %D{ (*parm) -> ma_error = m_noError;
					(*parm) -> ma_modify = m_noModifyOp %},
				attributeValue	ANY DEFINED BY attributeId
						[[a ma_val]]
				}
				OPTIONAL <E< parm -> cr_nattrs != 0 >>
					 <D< (*parm) -> cr_nattrs = 0 >>
    }
    %E{ array_free(parm->cr_pattrs) %}
    %D{ array_decode(MA_ATTR, &(*parm)->cr_nattrs,
	     (caddr_t*) &(*parm)->cr_attrs, (*parm)->cr_pattrs) %}


DeleteArgument [[P CMISDeleteArg *]] ::= SEQUENCE
    {
    baseManagedObjectClass	CHOICE
	[[T CMISDeleteArg * $ *]] << da_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O da_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i da_class.mid_local]]
	},

    baseManagedObjectInstance	CHOICE
	[[T CMISDeleteArg * $ *]] << da_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p da_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s da_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p da_inst.mn_dn]] [[F dn_free]]
	},

    accessControl		[5] EXTERNAL [[p da_access]]
        			OPTIONAL <E< parm->da_access != NULLEXTERN >>
		 			 <D< (*parm)->da_access = NULLEXTERN >>,

    synchronization		[6] IMPLICIT ENUMERATED [[i da_sync]]
				{ bestEffort (0), atomic (1) }
				-- DEFAULT bestEffort
        			OPTIONAL <E< parm->da_sync != s_bestEffort >>
		 			 <D< (*parm)->da_sync = s_bestEffort >>,

    scope			[7] CHOICE
	[[T CMISDeleteArg * $ *]] << da_inst.mn_type >>
        <E< parm->da_scope.sc_type == Sc_BaseToNthLevel ? 3 :
           (parm->da_scope.sc_type == Sc_IndividualLevel ? 2 : 1) >>
        <D< (*parm)->da_scope.sc_type >>
        {
        namedNumbers	    INTEGER [[i da_scope.sc_type]]
			    { baseObject (0),
                  	      firstLevelOnly (1),
                  	      wholeSubtree (2) }
			  %D{ (*parm)->da_scope.sc_level = 0; %},
        individualLevels    [1] IMPLICIT INTEGER [[i da_scope.sc_level]]
			  %D{ (*parm)->da_scope.sc_type = Sc_IndividualLevel;%},
        baseToNthLevel	    [2] IMPLICIT INTEGER [[i da_scope.sc_level]]
			  %D{ (*parm)->da_scope.sc_type = Sc_BaseToNthLevel; %}
        }
	-- DEFAULT baseObject
	OPTIONAL <E< parm->da_scope.sc_type != Sc_BaseObject >>
		 <D< (*parm)->da_scope.sc_type = Sc_BaseObject >>,

    filter			CMISFilter [[p da_filter]]
				-- DEFAULT and {}
        			OPTIONAL <E< parm->da_filter != NULLMFILTER >>
		 			 <D< (*parm)->da_filter = NULLMFILTER>>
    }


DeleteResult [[P CMISDeleteRes *]] ::= SEQUENCE
    {
    managedObjectClass		CHOICE
	[[T CMISDeleteRes * $ *]] << dr_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O dr_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i dr_class.mid_local]]
	}
	OPTIONAL <E< parm->dr_class.mid_global != NULLOID >>
		 <D< (*parm)->dr_class.mid_type = MID_GLOBAL;
		     (*parm)->dr_class.mid_global = NULLOID >>,

    managedObjectInstance	CHOICE
	[[T CMISDeleteRes * $ *]] << dr_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p dr_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s dr_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p dr_inst.mn_dn]] [[F dn_free]]
	}
	OPTIONAL <E< parm->dr_inst.mn_dn != NULLDN >>
		 <D< (*parm)->dr_inst.mn_type = MN_LOCALDN;
		     (*parm)->dr_inst.mn_dn = NULLDN; >>,

    currentTime			[5] IMPLICIT GeneralizedTime [[s dr_time]]
	OPTIONAL <E< parm->dr_time != NULLCP >> <D< (*parm)->dr_time=NULLCP >>
    }


DeleteError [[P CMISDeleteError *]] ::= SEQUENCE
    {
    managedObjectClass		CHOICE
	[[T CMISDeleteError * $ *]] << de_class.mid_type >>
	{
	globalForm   [0] IMPLICIT OBJECT IDENTIFIER [[O de_class.mid_global]],
	localForm    [1] IMPLICIT INTEGER [[i de_class.mid_local]]
	}
	OPTIONAL <E< parm->de_class.mid_global != NULLOID >>
		 <D< (*parm)->de_class.mid_type = MID_GLOBAL;
		     (*parm)->de_class.mid_global = NULLOID >>,

    managedObjectInstance	CHOICE
	[[T CMISDeleteError * $ *]] << de_inst.mn_type >>
	{
	distinguishedName	[2] IMPLICIT DistinguishedName
				[[p de_inst.mn_dn]] [[F dn_free]],
	nonSpecificForm         [3] IMPLICIT OCTET STRING
				[[s de_inst.mn_nonspec]] [[F sfree]],
	localDistinguishedName  [4] IMPLICIT RDNSequence
				[[p de_inst.mn_dn]] [[F dn_free]]
	}
	OPTIONAL <E< parm->de_inst.mn_dn != NULLDN >>
		 <D< (*parm)->de_inst.mn_type = MN_LOCALDN;
		     (*parm)->de_inst.mn_dn = NULLDN; >>,

    currentTime			[5] IMPLICIT GeneralizedTime [[s de_time]]
	OPTIONAL <E< parm->de_time != NULLCP >> <D< (*parm)->de_time=NULLCP >>,

			%E{ ((int) parm->de_error) -= CMIS_ERROR_OFFSET; %}
    deleteErrorInfo		[6] ENUMERATED [[i de_error]]
				{ accessDenied (2) }
			%E{ ((int) parm->de_error) += CMIS_ERROR_OFFSET; %}
			%D{ ((int) (*parm)->de_error) += CMIS_ERROR_OFFSET; %}
    }


LinkedReplyArgument [[P CMISLinkedReplyArg *]] ::=
    %E{ switch (parm->lr_type) {
	case LR_GET_RES:
	case LR_GET_LISTERR:
	    parm->lr_genparm = (caddr_t) &parm->lr_getres;
	    break;
	case LR_SET_RES:
	case LR_SET_LISTERR:
	    parm->lr_genparm = (caddr_t) &parm->lr_setres;
	    break;
	case LR_ACTION_RES:
	    parm->lr_genparm = (caddr_t) &parm->lr_actionres;
	    break;
	case LR_PROC_FAILURE:
	    parm->lr_genparm = (caddr_t) &parm->lr_procfailure;
	    break;
	case LR_DELETE_RES:
	    parm->lr_genparm = (caddr_t) &parm->lr_deleteres;
	    break;
	case LR_ACTION_ERR:
	    parm->lr_genparm = (caddr_t) &parm->lr_actionerr;
	    break;
	case LR_DELETE_ERR:
	    parm->lr_genparm = (caddr_t) &parm->lr_deleteerr;
	    break;
	default:
	    return NOTOK;
	}
    %}
    CHOICE << lr_type >>
    {
    getResult		[0] IMPLICIT GetResult [[p lr_genparm]],
    getListError	[1] IMPLICIT GetListError [[p lr_genparm]],
    setResult		[2] IMPLICIT SetResult [[p lr_genparm]],
    setListError	[3] IMPLICIT SetListError [[p lr_genparm]],
    actionResult	[4] IMPLICIT ActionResult [[p lr_genparm]],
    processingFailure	[5] IMPLICIT ProcessingFailure [[p lr_genparm]],
    deleteResult	[6] IMPLICIT DeleteResult [[p lr_genparm]],
    actionError		[7] IMPLICIT ActionError [[p lr_genparm]],
    deleteError		[8] IMPLICIT DeleteError [[p lr_genparm]]
    }
    %D{ switch ((*parm)->lr_type) {
	case LR_GET_RES:
	case LR_GET_LISTERR:
	    (*parm)->lr_getres = *(CMISGetRes*) (*parm)->lr_genparm;
	    break;
	case LR_SET_RES:
	case LR_SET_LISTERR:
	    (*parm)->lr_setres = *(CMISSetRes*) (*parm)->lr_genparm;
	    break;
	case LR_ACTION_RES:
	    (*parm)->lr_actionres = *(CMISActionRes*) (*parm)->lr_genparm;
	    break;
	case LR_PROC_FAILURE:
	    (*parm)->lr_procfailure = *(CMISProcessingFailure*)
						(*parm)->lr_genparm;
	    break;
	case LR_DELETE_RES:
	    (*parm)->lr_deleteres = *(CMISDeleteRes*) (*parm)->lr_genparm;
	    break;
	case LR_ACTION_ERR:
	    (*parm)->lr_actionerr = *(CMISActionError*) (*parm)->lr_genparm;
	    break;
	case LR_DELETE_ERR:
	    (*parm)->lr_deleteerr = *(CMISDeleteError*) (*parm)->lr_genparm;
	    break;
	default:
	    return NOTOK;
	}
	free((*parm)->lr_genparm);	/* not the contents */
    %}


END


%{

int  Attribute_free (attr)
CMISParam* attr;
{
    mparm_free(attr);
    free((char*) attr);
    return OK;
}

int  AttributeList_free (attrlist)
register CMISParamListVal* attrlist;
{
    CMISParamListVal* cur;

    while (attrlist) {
	cur = attrlist;
	attrlist = attrlist->mpl_next;
	mparm_free(&cur->mpl_ava);
	free((char*) cur);
    }
    return OK;
}

int  mfilter_free1 (filter)
CMISFilter* filter;
{
    mfilter_free(filter);	/* a wrap-up because this is declared void */
    return OK;
}

int  array_encode (type, nattrs, attrs, pattrs)
int type, nattrs;
caddr_t attrs;
CMISAttr** pattrs;
{
    register int i;
    register CMISAttr** cur = pattrs;

    if (!nattrs) {
	*pattrs = NULLMATTR;
	return OK;
    }

    for (i = 0; i < nattrs; i++) {
	if ((*cur = (CMISAttr*) malloc(sizeof(CMISAttr))) == NULL)
	    return NOTOK;

	switch (type) {
	case MA_ATTRID:
	    (*cur)->ma_id = ((MID) attrs)[i];		/* struct cpy */
	    break;

	case MA_ATTR:
	    (*cur)->ma_ava = ((CMISParam*) attrs)[i];	/* struct cpy */
	    break;

	case MA_GETATTR:
	    (*cur)->ma_error = ((CMISGetAttr*) attrs)[i].ga_error;
	    (*cur)->ma_ava = ((CMISGetAttr*) attrs)[i].ga_ava;	/* struct cpy */
	    break;

	case MA_SETATTR:
	    (*cur)->ma_error = ((CMISSetAttr*) attrs)[i].sa_error;
	    (*cur)->ma_modify = ((CMISSetAttr*) attrs)[i].sa_modify;
	    (*cur)->ma_ava = ((CMISSetAttr*) attrs)[i].sa_ava;	/* struct cpy */
	    break;

	default:
	    return NOTOK;
	}
	cur = &(*cur)->ma_next;
    }
    *cur = NULL;
    return OK;
}

int  array_decode (type, nattrs, attrs, pattrs)
int type;
int* nattrs;
caddr_t* attrs;
register CMISAttr* pattrs;
{
    register int i;
    register CMISAttr* cur;

    if (!pattrs) {
	*attrs = NULLCP;
	return OK;
    }

    *nattrs = 0;
    cur = pattrs;
    while (cur) {
	++(*nattrs);
	cur = cur->ma_next;
    }

    switch (type) {
    case MA_ATTRID:
	if ((*attrs = (caddr_t)malloc(*nattrs * sizeof(MIDentifier))) == NULL)
	    return NOTOK;
	break;

    case MA_ATTR:
	if ((*attrs = (caddr_t) malloc(*nattrs * sizeof(CMISParam))) == NULL)
	    return NOTOK;
	break;

    case MA_GETATTR:
	if ((*attrs = (caddr_t) malloc(*nattrs * sizeof(CMISGetAttr))) == NULL)
	    return NOTOK;
	break;

    case MA_SETATTR:
	if ((*attrs = (caddr_t) malloc(*nattrs * sizeof(CMISSetAttr))) == NULL)
	    return NOTOK;
	break;

    default:
	return NOTOK;
    }

    for (i = 0; pattrs; i++) {
	switch (type) {
	case MA_ATTRID:
	    ((MID) *attrs)[i] = pattrs->ma_id;		/* struct cpy */
	    break;

	case MA_ATTR:
	    ((CMISParam*) *attrs)[i] = pattrs->ma_ava;	/* struct cpy */
	    break;

	case MA_GETATTR:
	    ((CMISGetAttr*) *attrs)[i].ga_error = pattrs->ma_error;
	    ((CMISGetAttr*) *attrs)[i].ga_ava = pattrs->ma_ava;	/* struct cpy */
	    break;

	case MA_SETATTR:
	    ((CMISSetAttr*) *attrs)[i].sa_error = pattrs->ma_error;
	    ((CMISSetAttr*) *attrs)[i].sa_modify = pattrs->ma_modify;
	    ((CMISSetAttr*) *attrs)[i].sa_ava = pattrs->ma_ava;
							/* struct cpy */
	    break;

	default:
	    return NOTOK;
	}

	cur = pattrs;
	pattrs = pattrs->ma_next;
	free(cur);
    }
    return OK;
}

int  array_free (pattrs)
register CMISAttr* pattrs;
{
    register CMISAttr* cur;
    while (pattrs) {
	cur = pattrs;
	pattrs = pattrs -> ma_next;
	free(cur);
    }
    return OK;
}

%}
