/*
 * Copyright (c) 1988 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* pe2mem.c - functions to convert a PE to a BER string or stream and back */

/*
 * By Per Sigmond, Agderforskning, modified by George Pavlou
 */

#include <stdio.h>
#ifndef	ICR1
#include <isode/manifest.h>
#include <isode/psap.h>
#else
#include <isode/compat/manifest.h>
#include <isode/asn1/asn1.h>
#endif


/*
Write BER-code to memory.
Arguments:
	pe: input - the encoded PE to be written
	s: output - pointer to BER-string (allocated if *s == NULL).
	len: output - length in memory (number of BER-bytes)

Returns:
	- 0 if OK
	- non-0 if not OK
*/


int pe2mem (pe, s, len)
PE pe;
char **s;
int *len;
{
    PS ps;
    int status;

    *len = ps_get_abs(pe);

    ps = ps_alloc(str_open);
    if ((status = str_setup(ps, NULLCP, *len, 0)) != OK ||
			(status = pe2ps(ps, pe)) != OK) {
	ps_free(ps);
	return status;
    }

    if (*s == NULL) {
	*s = ps->ps_base;
	ps->ps_base = NULL;
    }
    else
	memcpy(*s, ps->ps_base, *len);

    ps_free(ps);
    return status;
}


/*
Read BER-code from memory.
Arguments:
	pe: output - the encoded PE
	s: input - address in memory
	len: input - length in bytes

Returns:
	- 0 if OK
	- non-0 if not OK
*/

int mem2pe (pe, s, len)
PE *pe;
char *s;
int len;
{
    PS ps;

    ps = ps_alloc(str_open);
    if (str_setup(ps, s, len, 0) != OK || (*pe = ps2pe(ps)) == NULLPE) {
	ps_free(ps);
	return NOTOK;
    }

    ps_free(ps);
    return OK;
}


/*
Write BER-code to stream.
(File is not rewound)
Arguments:
	pe: input - the encoded PE to be written
	outfile: output - stream containing BER-code

Returns:
	- 0 if OK
	- non-0 if not OK
*/

int pe2stream (pe, outfile)
PE pe;
FILE *outfile;
{
    PS ps;

    if (outfile == NULL)
	return NOTOK;	

    ps = ps_alloc(std_open);
    if (std_setup(ps, outfile) != OK || pe2ps(ps, pe) != OK) {
	ps_free(ps);
	return NOTOK;
    }

    ps_free(ps);
    return OK;
}


/*
Read BER-code from stream.
Arguments:
	pe: output - the encoded PE
	infile: input - stream containing BER-code

Returns:
	- 0 if OK
	- non-0 if not OK
*/

int stream2pe (pe, infile)
PE *pe;
FILE *infile;
{
    PS ps;

    if (infile == NULL)
	return NOTOK;

    ps = ps_alloc(std_open);
    if (std_setup(ps, infile) != OK || (*pe = ps2pe(ps)) == NULLPE) {
	ps_free(ps);
	return NOTOK;
    }

    ps_free(ps);
    return OK;
}
