/*
 * Copyright (c) 1990 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* */


/* Saleem N. Bhatti */

/* */
/*****************************************************************************
Initialising Syntaxes



Saleem N. Bhatti
July 1992
*****************************************************************************/


#if defined(__GNUG__)
#include <std.h>
#elif defined(__cplusplus)
#include <stdlib.h>
#else
#include <stdio.h>
#endif

#include <ctype.h>
#include <string.h>
#ifndef	ICR1
#include <isode/general.h>
#include <isode/tailor.h>
#else
#include <isode/compat/general.h>
#include <isode/compat/tailor.h>
#endif

#define OSIMISETCPATH    "OSIMISETCPATH"
#define OSIMIS_name      "osimis"
#define DEFAULT_tailor   "tailor"
#define DEFAULT_oidtable "oidtable"

char *etcdir; /* The path of the etc directory */
char *tailor; /* The full pathname of the tailor */
extern char *isodetcpath; /* This is in the ISODE library */

/* In oid.c */
extern int NumEntries;
extern int attrNumEntries;
extern int ocNumEntries;

typedef void (*VoidFunc)();

int
#if defined(__STDC__) || defined(__cplusplus)
initialiseSyntaxes(const char *tailorFile, VoidFunc *syntaxes)
#else
initialiseSyntaxes(tailorFile, syntaxes)
char     *tailorFile;
VoidFunc *syntaxes;
#endif
{
    static
    int done = 0;

    register
    FILE *fp;
    register
    char *cp, *tailorName;
    VoidFunc *pfv;
    char buffer[256]; /* I hope this is enough! */

    if (done) return(OK);

    /* OSIMIS Tailor file */
    cp = getenv(OSIMISETCPATH);
    if (cp == NULL) {
        fprintf(stderr, "Error: Cannot get value for the environment variable %s\n", OSIMISETCPATH);
        return(NOTOK);
    }
    sprintf(buffer, "%s/", cp);
    etcdir = strdup(buffer);

    /* tailoring */
    tailorName = (char *) tailor;
    if ((tailorName == NULL) || (*tailorName == '\0'))
	sprintf(buffer, "%s%s%s", etcdir, OSIMIS_name, DEFAULT_tailor);
    else
	sprintf(buffer, "%s%s", etcdir, tailorName);

    /* check if the tailor file exists */
    if ((fp = fopen(buffer, "r")) != NULL) {
	close(fp);
        tailor = strdup(buffer);
	isodesetailor(tailor);
	fprintf(stderr, "tailor : %s\n", tailor);
    }
    else {
	fprintf(stderr, "Warning: Cannot open tailor file : %s\n", buffer);
        tailor = NULL;
    }

    /* The following is used to direct ISODE to the OSIMIS etc directory */
    isodetcpath = etcdir;

    isodetailor(NULLCP, 0);

    /* Syntax info */
    for(pfv = syntaxes; *pfv != NULL; ++pfv) (*pfv)();

#if 0
#if !defined(SECURITY)
    /* Directory configuration -- initialiseOsisec() does dsap_init() too */
    dsap_init((int *)0,(char ***)0);
#endif
#endif

    sprintf(buffer, "%s%s", etcdir, DEFAULT_oidtable);
    fprintf(stderr, "oidtable : %s\n", buffer);

    load_oid_table(buffer);

    fprintf(stderr, "OIDs %d : attributes %d : classes %d\n",
            NumEntries, attrNumEntries, ocNumEntries);

    done = 1;

    return(OK);
}
