//
// Copyright (c) 1994 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// K.M.T.McCarthy

#include "Create.h"
#include "eventForwardingDiscriminator.h"
#include "EventLog.h"
#ifdef   LOGGING
#include "Log.h"
#endif

#ifdef  MONMET
#include "Monitor-Sntx.h"
#include "monitorMetric.h"
#include "movingAverageMeanMonitor.h"
#endif

#ifdef MONSUP
#include "monitoringSupportMode2.h"
#include "monitoringSupportMode1.h"
#include "monitoringSupportMode3.h"
#endif

#include "System.h"

#include "proxySystem.h"
#include "remoteSystem.h"
#include "cmipsnmpProxy.h"
#include "cmipsnmpProxyAgent.h"
#include "snmpSecurityParameter.h"
#include "SnmpImageMO.h"

#define MIB_INIT_FILE		"mib.init.iqa"	     	   // MIB initialisation info
#define IQA_INPUT_FILE 		"iqadir/iqamibconfig.icm"  // IQA start up files configuration


MOClass moclasses[]  =
{
// class name			  CMIS M_Create			  MIB initialise


//  SMF-related classes first and the MIT-top system

{ "top",
   Top::getClassInfo,
   Top::initialiseClass },

{ "system",
   System::getClassInfo,
   System::initialiseClass },

{ "discriminator",
   discriminator::getClassInfo,
   discriminator::initialiseClass },

{ "eventForwardingDiscriminator",
   eventForwardingDiscriminator::getClassInfo,
   eventForwardingDiscriminator::initialiseClass },

#ifdef	LOGGING
{ "log",
   Log::getClassInfo,
   Log::initialiseClass },
#endif

{ "logRecord",
   LogRecord::getClassInfo,
   LogRecord::initialiseClass },

{ "eventLogRecord",
   EventLogRecord::getClassInfo,
   EventLogRecord::initialiseClass },


#ifdef	MONMET
{ "scanner",
   scanner::getClassInfo,
   scanner::initialiseClass },

{ "monitorMetric",
   monitorMetric::getClassInfo,
   monitorMetric::initialiseClass },

{ "movingAverageMeanMonitor",
   movingAverageMeanMonitor::getClassInfo,
   movingAverageMeanMonitor::initialiseClass },
#endif

#ifdef  MONSUP
{ "MSscanner",
   MSscanner::getClassInfo,
   MSscanner::initialiseClass },

{ "monitoringSupport",
   monitoringSupport::getClassInfo,
   monitoringSupport::initialiseClass },

{ "monitoringSupportMode1",
   monitoringSupportMode1::getClassInfo,
   monitoringSupportMode1::initialiseClass },

{ "monitoringSupportMode2",
   monitoringSupportMode2::getClassInfo,
   monitoringSupportMode2::initialiseClass },

{ "monitoringSupportMode3",
   monitoringSupportMode3::getClassInfo,
   monitoringSupportMode3::initialiseClass },
#endif

// classes specific to this application i.e. IQA

{ "proxySystem",
   proxySystem::getClassInfo,
   proxySystem::initialiseClass },

{ "remoteSystem",
   remoteSystem::getClassInfo,
   remoteSystem::initialiseClass },

{ "cmipsnmpProxy",
   cmipsnmpProxy::getClassInfo,
   cmipsnmpProxy::initialiseClass },

{ "cmipsnmpProxyAgent",
   cmipsnmpProxyAgent::getClassInfo,
   cmipsnmpProxyAgent::initialiseClass },

{ "snmpSecurityParameter",
   snmpSecurityParameter::getClassInfo,
   snmpSecurityParameter::initialiseClass }

};


int NCLASSES = sizeof moclasses / sizeof moclasses[0];


NotificationMapping notificationMappingTable[] =
{
#if defined(MONMET)
{"qualityofServiceAlarm",          	"eventLogRecord"},
#endif

{"objectCreation",             		"eventLogRecord"},
{"objectDeletion",              	"eventLogRecord"},
{"attributeValueChange",        	"eventLogRecord"},
{"stateChange",                 	"eventLogRecord"},

{"internetAlarmColdStart",            	"eventLogRecord"},
{"internetAlarmWarmStart",            	"eventLogRecord"},
{"internetAlarmLinkDown",             	"eventLogRecord"},
{"internetAlarmLinkUp",               	"eventLogRecord"},
{"internetAlarmAuthenticationFailure",	"eventLogRecord"},
{"internetAlarmEgpNeighbourLoss",       "eventLogRecord"},


// At least for now will need to add enterprise specific ones individually :-(

#if defined (ICMPON)

// ICM PON specific...

{"ponCompActionConf",			"eventLogRecord"},
{"ponSystTexVerConf",			"eventLogRecord"},
{"tempOutofLimit",			"eventLogRecord"},
{"currOutofLimit",			"eventLogRecord"},
{"powerOutofLimit",			"eventLogRecord"},
{"frequencyAllocDeAlloc",		"eventLogRecord"},
{"connectRefused",			"eventLogRecord"},
{"opticalPeak",				"eventLogRecord"},
{"tableError",				"eventLogRecord"},
{"ctxReqRecal",				"eventLogRecord"},
{"ctxTimeout",				"eventLogRecord"},
{"osaTimeout",				"eventLogRecord"},
#endif

{"internetAlarmTestEnterpriseSpecific", "eventLogRecord"}  // test case

};

int NNOTIFICATIONS = sizeof notificationMappingTable/sizeof notificationMappingTable[0];

