/*
 * Copyright (c) 1993 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* */


/* Saleem N. Bhatti */

/* */

/* @(#)Library.h 	 01/04/1993  */

/*M*******************************************************************


  Module name:       Library.h
     
  Description:       This module implements the 
                     library that comes with the Attribute Compiler
		     and that implements some basic comparing functions.

  Creation date:     April 1st, 1993

  Coded by:          Luca Deri


  Modification history: 

**********************************************************************/


#ifndef _LIBRARY_C_
#define _LIBRARY_C_

#include <stdio.h>
#include "Library.h"


#define START_WITH_STRING   0
#define CONTAIN_STRING      1
#define END_WITH_STRING     2

#define NULL_AREA          (char*)0x0

/**********************************************************************
  
  Name:	                MemoryContainment
  
  Description:	        This function checks if the memory area pointed by 
                        the 'theArea' contains a string that 
			starts/contains/ends with the string pointed by
			the `theContainedArea' pointer.
                        
  Input parameters:	char*
			The pointer to the memory area to be checked if 
			starts/contains/ends with the string pointed by
			the 'theContainedArea' pointer.

			int
			The lenght (number of bytes) of the memory 
			area to be checked.

			char*
			The pointer to the contained memory area 
			to be checked.

			int
			The lenght (number of bytes) of the contained memory 
			area to checked.

			int
			The checking operation to be performed:
 		        START_WITH_STRING: 'theArea' starts with 
			                   `theContainedArea'
                        CONTAIN_STRING: 'theArea' contains the 
			                `theContainedArea'
                        END_WITH_STRING: 'theArea' ends with 
			                 `theContainedArea'


  Output parameters:    Bool
                        True if the result is positive according to the
			requested operation, False otherwise.

  Modification history:                         

  Coded by:             Luca Deri
  
  Date:                 July 8th, 1993
  
***********************************************************************/
#if defined(__STDC__) || defined(__cplusplus)
Bool MemoryContainment(char* theArea, int theAreaLen,
		       char* theContainedArea, int theContainedAreaLen,
		       int theAreaOperation)
#else
Bool MemoryContainment(theArea, theAreaLen, theContainedArea, 
		       theContainedAreaLen, theAreaOperation)
char* theArea;
int theAreaLen;
char* theContainedArea;
int theContainedAreaLen;
int theAreaOperation;
#endif
{
  register int i;

  if(theContainedArea == NULL_AREA) /* Null string */
    return(True);
  else if ((theArea == NULL_AREA) && (theContainedArea != NULL_AREA))
    return(False);

  /* If the flow arrives here, certainly both strings are not NULL */

  switch(theAreaOperation)
    {
    case START_WITH_STRING:
      if(memcmp(theArea, theContainedArea, 
		 theContainedAreaLen) == 0)
	return(True);
      else
	return(False);
    case CONTAIN_STRING:
      for(i=0; i<=(theAreaLen-theContainedAreaLen); i++)
	{
	  if(memcmp(&theArea[i], theContainedArea,
		    theContainedAreaLen) == 0)
	    return(True);	  
	}
      return(False);
    case END_WITH_STRING:
      if(theAreaLen < theContainedAreaLen)
	return(False); /* Certainly the operation fails */
      if(memcmp(&theArea[theAreaLen-theContainedAreaLen],
		 theContainedArea, theContainedAreaLen) == 0)
	return(True);
      else
	return(False);
    default:
      return(False); /* Invalid operation */
    }

  /* NOTREACHED */
}

/**********************************************************************
  
  Name:	                StringsContainment
  
  Description:	        This function is just like the previous function
                        where the memory areas are C string pointers
			null terminated
                        
  Input parameters:	char*
			The pointer to the string to be checked if 
			starts/contains/ends with the string pointed by
			the 'theContainedString' pointer.

			char*
			The pointer to the contained string to be checked.

			int
			The checking operation to be performed:
 		        START_WITH_STRING: 'theString' starts with 
			                   `theContainedString'
                        CONTAIN_STRING: 'theString' contains the 
			                `theContainedString'
                        END_WITH_STRING: 'theString' ends with 
			                 `theContainedString'


  Output parameters:    Bool
                        True if the result is positive according to the
			requested operation, False otherwise.

  Modification history:                         

  Coded by:             Luca Deri
  
  Date:                 July 8th, 1993
  
***********************************************************************/
#define SubstringContainment(a, b, c) \
        MemoryContainment(a, strlen(a), b, strlen(b), c)


/**********************************************************************
  
  Name:	                SeqOfPrint
  
  Description:	        This function prints a SEQUENCE OF structure. 
                        
  Input parameters:	PS
                        The presentation stream where to print.

			void*
			The pointer to the element to print.

			int
			Set to True if the output have to be quoted,
			False otherwise.

			Bool
			True if the element is a pointer to an element,
			False otherwise.

			PrintType
			Pointer to the procedure that prints the 
			elements.

			NextElement
			Pointer to the procedure that given an
			element, return the next one in the sequence.
			
			
  Output parameters:    int
                        1 if any error is occurred, 0 otherwise

  Modification history:                         

  Coded by:             Luca Deri
  
  Date:                 April, 1993
  
***********************************************************************/
#if defined(__STDC__) || defined(__cplusplus)
int SeqOfPrint(PS ps, void *elem, int quoted, Bool isAPointer,
	       PrintType printProc, NextElement nextProc)
#else
int SeqOfPrint(ps, elem, quoted, isAPointer, printProc, nextProc)
PS ps;
void *elem;
int quoted;
Bool isAPointer;
PrintType printProc;
NextElement nextProc;
#endif
{
   if(elem == NULL)
     return(OK); /* Nothing to print */

   while(elem != NULL)
   {
     if(isAPointer)
       {
	 if(printProc(ps, *(void**)elem, quoted) != OK)
	   return(NOTOK);
	 else
	   elem = *nextProc(elem);
       }
     else
       {
	 if(printProc(ps, elem, quoted) != OK)
	   return(NOTOK);
	 else
	   elem = *nextProc(elem);
       }
   }

   return(OK);
}


/**********************************************************************
  
  Name:	                SeqOfCompare
  
  Description:	        This function compare a SEQUENCE OF structure. 
                        
  Input parameters:	void*
			The pointer to the first element element to compare.

			void*
			The pointer to the second element element to compare.

			Bool
			True if the elements are a pointer to an element,
			False otherwise.

			CompareType
			Pointer to the procedure that compares the 
			elements.

			NextElement
			Pointer to the procedure that given an
			element, return the next one in the sequence.
			
			
  Output parameters:    int
                        A_EQUAL_B: 1st element == 2nd element
                        A_LESS_EQUAL_B: 1st element <= 2nd element
                        A_GREATER_EQUAL_B: 1st element == 2nd element
                        A_NOT_EQUAL_B: 1st element != 2nd element

  Modification history:                         

  Coded by:             Luca Deri
  
  Date:                 April, 1993
  
***********************************************************************/
#if defined(__STDC__) || defined(__cplusplus)
int SeqOfCompare(void *elem_1, void *elem_2, Bool isAPointer,
		 CompareType compareProc, NextElement nextProc)
#else
int SeqOfCompare(elem_1, elem_2, isAPointer, compareProc, nextProc)
void *elem_1, *elem_2;
Bool isAPointer;
CompareType compareProc;
NextElement nextProc;
#endif
{
  int theResult = A_EQUAL_B, cmpRes;
  
  if((elem_1 == NULL) && (elem_2 == NULL))
    return(A_EQUAL_B);
  else if((elem_1 == NULL) || (elem_2 == NULL))
    return(A_NOT_EQUAL_B);
  
  /* SEQUENCE OF - ordered list */
  while((elem_1 != NULL) && (elem_2 != NULL))
    {
     if(isAPointer)
       cmpRes = compareProc(*(void**)elem_1, *(void**)elem_2);
     else
       cmpRes = compareProc(elem_1, elem_2);
      
      if(theResult != A_EQUAL_B)
	{
	  if((theResult != cmpRes) && (cmpRes != A_EQUAL_B))
	    return(A_NOT_EQUAL_B);
	}
      else
	theResult = cmpRes;
      
      elem_1 = *nextProc(elem_1);
      elem_2 = *nextProc(elem_2);
    }
  
  switch(theResult) /* Check sequence lenght */
    {
    case A_EQUAL_B:
      if((elem_1 != NULL) && (elem_2 == NULL))
	theResult = A_GREATER_EQUAL_B;
      else if((elem_1 == NULL) && (elem_2 != NULL))
	theResult = A_LESS_EQUAL_B;
      break;
    case A_LESS_EQUAL_B:
      if((elem_1 != NULL) && (elem_2 == NULL))
	return(A_NOT_EQUAL_B); /* #elements(elem_1) > #elements(elem_2) */
      break;
    case A_GREATER_EQUAL_B:
      if((elem_1 == NULL) && (elem_2 != NULL))
	return(A_NOT_EQUAL_B); /* #elements(elem_1) < #elements(elem_2) */
    }
  
  return(theResult);
}

#define SeqOfAdd SetOfAdd
#define SeqOfRemove SetOfRemove
#define SeqOfSetCompare SeqOfSetCompare


/**********************************************************************
  
  Name:	                SetOfCompare
  
  Description:	        This function compare a SET OF structure. 
                        
  Input parameters:	void*
			The pointer to the first element element to compare.

			void*
			The pointer to the second element element to compare.

			Bool
			True if the elements are a pointer to an element,
			False otherwise.

			CompareType
			Pointer to the procedure that compares the 
			elements.

			NextElement
			Pointer to the procedure that given an
			element, return the next one in the sequence.
			
			
  Output parameters:    int
                        A_EQUAL_B: 1st element == 2nd element
                        A_LESS_EQUAL_B: 1st element <= 2nd element
                        A_GREATER_EQUAL_B: 1st element == 2nd element
                        A_NOT_EQUAL_B: 1st element != 2nd element

  Modification history:                         

  Coded by:             Luca Deri
  
  Date:                 April, 1993
  
***********************************************************************/
#if defined(__STDC__) || defined(__cplusplus)
int SetOfCompare(void *elem_1, void *elem_2, Bool isAPointer,
		 CompareType compareProc, NextElement nextProc)
#else
int SetOfCompare(elem_1, elem_2, isAPointer, compareProc, nextProc)
void *elem_1, *elem_2;
Bool isAPointer;
CompareType compareProc;
NextElement nextProc;
#endif
{
  int theResult = A_EQUAL_B, cmpRes;
  Bool foundElem = False;
  void *tempScan1 = elem_1, *tempScan2 = elem_2;
  
  if((elem_1 == NULL) && (elem_2 == NULL))
    return(A_EQUAL_B);
  else if((elem_1 == NULL) || (elem_2 == NULL))
    return(A_NOT_EQUAL_B);
  
  /* SET OF - unordered list */
  while((elem_1 != NULL) && (elem_2 != NULL))
    {
      while((elem_1 != NULL)  && (!foundElem))
	{
	  if(isAPointer)
	    cmpRes = compareProc(*(void**)elem_1, *(void**)elem_2);
	  else
	    cmpRes = compareProc(elem_1, elem_2);
	  
	  if((theResult == cmpRes) || (cmpRes == A_EQUAL_B))
	    foundElem = True;
	  else
	    elem_1 = *nextProc(elem_1);
	}
      
      if(!foundElem)
	{
	  if(theResult == A_EQUAL_B) /* I try with <= or >= */
	    theResult = cmpRes;
	  else
            return(A_NOT_EQUAL_B); /* The previous element was not found */
	}
      
      elem_1 = tempScan1; /* Rescan the list */
      elem_2 = *nextProc(elem_2);
    }
  
  /* A_EQUAL_B: this is True if the sets contain the same numer of elements */
  elem_1 = tempScan1, elem_2 = tempScan2; /* Reset scan */
  while((elem_1 != NULL) && (elem_2 != NULL))
    elem_1 = *nextProc(elem_1), elem_2 = *nextProc(elem_2);
  
  if((elem_1 != NULL) || (elem_2 != NULL))
    return(A_NOT_EQUAL_B); /* Different number of elements */
  
  return(theResult);
};


/**********************************************************************
  
  Name:	                SetOfAdd
  
  Description:	        This function adds two SET OF structures,
                        deleting duplicated elements. The added set
			is integrated in the main set, so it should not
			be freed.
                        
  Input parameters:	void*
			The pointer to the main set.

			void*
			The pointer to the set to be added.

			Bool
			True if the elements are a pointer to an element,
			False otherwise.

			CompareType
			Pointer to the procedure that compares the 
			elements.

			NextElement
			Pointer to the procedure that given an
			element, return the next one in the sequence.
			
			
  Output parameters:    void*
                        The new chain that cointains the previou two chains.

  Modification history:                         

  Coded by:             Luca Deri
  
  Date:                 April, 1993
  
***********************************************************************/
#if defined(__STDC__) || defined(__cplusplus)
void* SetOfAdd(void *mainElem, void *elemToAdd, FreeType freeProc,
	       Bool isAPointer, CompareType compareProc, 
	       NextElement nextProc)
#else
void* SetOfAdd(mainElem, elemToAdd, freeProc, isAPointer, 
	       compareProc, nextProc)
void *mainElem, *elemToAdd;
FreeType freeProc;
Bool isAPointer;
CompareType compareProc;
NextElement nextProc;
#endif
{
  void *theScanMain = mainElem, *theScanElem = elemToAdd,
    *thePrevMain=NULL, *tempScan;
  
  int cmpRes;
  Bool isFound;
  
  /* The type is a SET OF */
  
  if(mainElem == NULL)
    return(elemToAdd); /* The first element is empty */
  
  while(theScanElem != NULL)
    {
      isFound = False;
      
      while((theScanMain != NULL) && (!isFound))
	{
	  if(isAPointer)
	    cmpRes = compareProc(*(void**)theScanMain, *(void**)theScanElem);
	  else
	    cmpRes = compareProc(theScanMain, theScanElem);
	  switch(cmpRes)
	    {
	    case A_EQUAL_B: /* Equal: duplicate element */
	      isFound = True;
	      break;
	    case A_LESS_EQUAL_B: /* mainElem < elemToAdd */
	    case A_GREATER_EQUAL_B: /* mainElem > elemToAdd */
	    case A_NOT_EQUAL_B:
	      thePrevMain = theScanMain;
	      theScanMain = *nextProc(theScanMain); /* Next element to add */
	      break;
	    }
	}
      
      if(!isFound)
	{
	  /* No duplicate element: insert the element at the
	     end of the list */
	  *nextProc(thePrevMain) = theScanElem;
	  theScanElem = *nextProc(theScanElem); /* Next Element to add */
	  thePrevMain = *nextProc(thePrevMain);
	  *nextProc(thePrevMain) = NULL; /* End of the list */
	}
      else
	{
	  /* Duplicate element: I delete it and keep on adding */
	  tempScan = theScanElem;
	  if(freeProc != NULL)
	    freeProc(theScanElem);
	  theScanElem = *nextProc(theScanElem);  /* Next element to add */
          free((void*)tempScan);
	}
      
      theScanMain = mainElem; /* Reset scan */
    }
  
  return(mainElem);
};


/**********************************************************************
  
  Name:	                SetOfRemove
  
  Description:	        This function remove one SET OF structure, from
                        an other SET OF. The set to be removed is
			deleted and should NOT be deallocated again.
                        An element not present in the main set but present
			int the set to be removed, doesn't generate an error
			but it is simply freed like defined in the
			standard.

  Input parameters:	void*
			The pointer to the main set.

			void*
			The pointer to the set to be removed.

			Bool
			True if the elements are a pointer to an element,
			False otherwise.

			CompareType
			Pointer to the procedure that compares the 
			elements.

			NextElement
			Pointer to the procedure that given an
			element, return the next one in the sequence.
			
  Output parameters:    void*
                        The new chain that cointains the previou two chains.

  Modification history:                         

  Coded by:             Luca Deri
  
  Date:                 April, 1993
  
***********************************************************************/
#if defined(__STDC__) || defined(__cplusplus)
void* SetOfRemove(void *mainElem, void *elemToRemove, FreeType freeProc,
		  Bool isAPointer, CompareType compareProc, 
		  NextElement nextProc)
#else
void* SetOfRemove(mainElem, elemToRemove, freeProc, isAPointer, compareProc, 
		  nextProc)
void *mainElem, *elemToRemove;
FreeType freeProc;
Bool isAPointer;
CompareType compareProc;
NextElement nextProc;
#endif
{
  void *theScanMain = mainElem, *theScanElem = elemToRemove,
    *thePrevMain=NULL;
  int cmpRes;
  Bool isFound;
  
  /* The type is a SET OF */
  
  while(theScanElem != NULL)
    {
      isFound = False;
      while((theScanMain != NULL) && (!isFound))
	{
	  if(isAPointer)
	    cmpRes = compareProc(*(void**)theScanMain, *(void**)theScanElem);
	  else
	    cmpRes = compareProc(theScanMain, theScanElem);
	  switch(cmpRes)
	    {
	    case A_EQUAL_B: /* Equal : found the element to be removed */
	      if(thePrevMain != NULL)
		{
		  *nextProc(thePrevMain) = *nextProc(theScanMain);
		  if(freeProc != NULL)
		    freeProc(theScanMain);
		  /* free((void*)theScanMain); */
		  theScanMain = *nextProc(thePrevMain);
		}
	      else
		{
		  theScanMain = *nextProc(theScanMain);
		  if(freeProc != NULL)
		    freeProc(mainElem);
		  /* free((void*)mainElem); */
		  mainElem = theScanMain;
		  isFound = True;
		}
	      break;
	    case A_LESS_EQUAL_B: /* mainElem < elemToAdd */
	    case A_GREATER_EQUAL_B: /* mainElem > elemToAdd */
	    case A_NOT_EQUAL_B: /* mainElem != elemToAdd */
	      thePrevMain = theScanMain;
	      theScanMain = *nextProc(theScanMain); /* Next element to scan */
	      break;
	    } /* End switch */
	}
      
      theScanElem = *nextProc(theScanElem);  /* Next element to remove */
      if(freeProc != NULL)
	freeProc(elemToRemove);
      /* free((void*)elemToRemove); */
      elemToRemove = theScanElem;
      theScanMain = mainElem; /* Back to the top of the list */
    }
  
  while(theScanElem != NULL) /* Free the sequence to remove, if any */
    { 
      if(freeProc != NULL)
	freeProc(theScanMain);
      elemToRemove = theScanElem;
      theScanElem = *nextProc(theScanElem);
      free((void*)elemToRemove);
    }
  
  return(mainElem);
};

/**********************************************************************
  
  Name:	                SetOfSetCompare
  
  Description:	        This function implements the comparison of
                        two SET OF structures.
                        
  Input parameters:	int
			This is the comparison mode (see. CMISFilter).

			void*
			The pointer to the first element element to compare.

			void*
			The pointer to the second element element to compare.

			Bool
			True if the elements are a pointer to an element,
			False otherwise.

			CompareType
			Pointer to the procedure that compares the 
			elements.

			NextElement
			Pointer to the procedure that given an
			element, return the next one in the sequence.
			
			
  Output parameters:    Bool
                        True if the comparison has been satisfied,
			False otherwise.

  Modification history:                         

  Coded by:             Luca Deri
  
  Date:                 April, 1993
  
***********************************************************************/
#if defined(__STDC__) || defined(__cplusplus)
Bool SetOfSetCompare(int mode, void *elem_1, void *elem_2, 
		     Bool isAPointer, CompareType compareProc, 
		     NextElement nextProc)
#else
Bool SetOfSetCompare(mode, elem_1, elem_2, isAPointer, compareProc, 
		     nextProc)
int mode;
void *elem_1, *elem_2;
Bool isAPointer;
CompareType compareProc;
NextElement nextProc;
#endif
{
  void *scanVar = elem_1;
  int cmpRes;
  
  if((elem_1 == NULL) && (elem_2 == NULL))
    return(True);
  else if((elem_1 == NULL) || (elem_2 == NULL))
    return(False);
  
  switch(mode)
    {
    case FI_GreaterOrEqual:
      if((elem_2 == NULL) || (*nextProc(elem_2) != NULL))
	return(False); /* elem_2 must contain only one element */
      
      while(elem_1 != NULL) /* Try to find, if exists, at least an element
                               of elem_1 <= elem_2 */
 	{
	  if(isAPointer)
	    cmpRes = compareProc(*(void**)elem_2, *(void**)elem_1);
	  else
	    cmpRes = compareProc(elem_2, elem_1);
	  switch(cmpRes)
	    {
            case A_GREATER_EQUAL_B:
            case A_EQUAL_B:
              return(True);
            default:
              elem_1 = *nextProc(elem_1);
              break;
	    }
        };
      return(False);
    case FI_LessOrEqual:
      if((elem_2 == NULL) || (*nextProc(elem_2) != NULL))
	return(False); /* elem_2 must contain only one element */
      while(elem_1 != NULL) /* Try to find, if exists, at least an element
                               of elem_1 <= elem_2 */
 	{
	  if(isAPointer)
	    cmpRes = compareProc(*(void**)elem_1, *(void**)elem_2);
	  else
	    cmpRes = compareProc(elem_1, elem_2);
	  switch(cmpRes)
	    {
            case A_GREATER_EQUAL_B:
            case A_EQUAL_B:
              return(True);
            default:
              elem_1 = *nextProc(elem_1);
              break;
	    }
        };
      return(False);
    case FI_SubsetOf:
      while(elem_2 != NULL)
	{
	  while(scanVar != NULL)
	    {
	      if(isAPointer)
		cmpRes = compareProc(*(void**)elem_2, *(void**)scanVar);
	      else
		cmpRes = compareProc(elem_2, scanVar);
	      switch(cmpRes)
		{
		case A_EQUAL_B:
		  elem_2 = *nextProc(elem_2);
		  if(elem_2 == NULL)
		    return(True);
		  scanVar = elem_1; /* Reset scanning */
		  break;
		default:
		  scanVar = *nextProc(scanVar);
		  break;
		}
	    }
	  /* There are some elements of elem_1
	     that are non present in elem_2 */
      	  return(False);
	}
      return(True);
    case FI_SupersetOf:
      return(SetOfSetCompare(FI_SubsetOf, elem_2, elem_1, isAPointer,
			     compareProc, nextProc));
    case FI_NonNullSetIntersect:
      while(elem_2 != NULL)
        {
          while(scanVar != NULL)
	    {
	      if(isAPointer)
		cmpRes = compareProc(*(void**)elem_2, *(void**)scanVar);
	      else
		cmpRes = compareProc(elem_2, scanVar);
	      switch(cmpRes)
		{
		case A_EQUAL_B:
		  return(True);
		  break;
		default:
		  scanVar = *nextProc(scanVar);
		  break;
		}
	    }
	  elem_2 = *nextProc(elem_2);
	  scanVar = elem_1; /* Reset scanning */
	}
      return(False);
    default:
      /* Error: wrong `mode` parameter value */
      return(False);
    }
}


/* *** Other library functions *** */

/**********************************************************************
  
  Name:	                compareInt
  
  Description:	        This function compares two integers and it is used
                        when two complex structures, containing integer
			fields, have to be compared.
                        
  Input parameters:	void*
			The pointer to the first element element to compare.

			void*
			The pointer to the second element element to compare.
			
  Output parameters:    Bool
                        True if the comparison has been satisfied,
			False otherwise.

  Modification history:                         

  Coded by:             Luca Deri
  
  Date:                 April, 1993
  
***********************************************************************/
#if defined(__STDC__) || defined(__cplusplus)
int compareInt(void* theElem1, void* theElem2)
#else
int compareInt(theElem1, theElem2)
void* theElem1, *theElem2;
#endif
{
  int elem_1 = *(int*)theElem1, elem_2 = *(int*)theElem2;

   if(elem_1 == elem_2) /* Equal */
     return(A_EQUAL_B);
   else if(elem_1 <= elem_2) /* Not equal */
     return(A_LESS_EQUAL_B);
   else
     return(A_GREATER_EQUAL_B);
}

/**********************************************************************
  
  Name:	                compareChar
  
  Description:	        This function compares two chars (NOT strings) and it
                        is used when two complex structures, containing 
			char or boolean	fields, have to be compared.
                        
  Input parameters:	void*
			The pointer to the first element element to compare.

			void*
			The pointer to the second element element to compare.
			
  Output parameters:    Bool
                        True if the comparison has been satisfied,
			False otherwise.

  Modification history:                         

  Coded by:             Luca Deri
  
  Date:                 April, 1993
  
***********************************************************************/
#if defined(__STDC__) || defined(__cplusplus)
int compareChar(void* theElem1, void* theElem2)
#else
int compareChar(theElem1, theElem2)
void* theElem1, *theElem2;
#endif
{
  char elem_1 = *(char*)theElem1, elem_2 = *(char*)theElem2;

   if(elem_1 == elem_2) /* Equal */
     return(A_EQUAL_B);
   else if(elem_1 <= elem_2) /* Not equal */
     return(A_LESS_EQUAL_B);
   else
     return(A_GREATER_EQUAL_B);
}

/**********************************************************************
  
  Name:	                compareReal
  
  Description:	        This function compares two reals and it is used
                        when two complex structures, containing real
			fields, have to be compared.
                        
  Input parameters:	void*
			The pointer to the first element element to compare.

			void*
			The pointer to the second element element to compare.
			
  Output parameters:    Bool
                        True if the comparison has been satisfied,
			False otherwise.

  Modification history:                         

  Coded by:             Luca Deri
  
  Date:                 April, 1993
  
***********************************************************************/
#if defined(__STDC__) || defined(__cplusplus)
int compareReal(void* theElem1, void* theElem2)
#else
int compareReal(theElem1, theElem2)
void* theElem1, *theElem2;
#endif
{
  double elem_1 = *(double*)theElem1, elem_2 = *(double*)theElem2;

   if(elem_1 == elem_2) /* Equal */
     return(A_EQUAL_B);
   else if(elem_1 <= elem_2) /* Not equal */
     return(A_LESS_EQUAL_B);
   else
     return(A_GREATER_EQUAL_B);
}


/**********************************************************************
  
  Name:	                printInt
  
  Description:	        This function prints two integers and it is used
                        when two complex structures, containing integer
			fields, have to be printed.
                        
  Input params.:        PS
                        The presentation stream where the function prints.

			int* 
			The value to be printed.
		    
			int (not used)
			Flag: 1 = quoted print, 0 = not quoted print.


  Output params.:       int
                        NOTOK if some error is occurred, OK otherwise.


  Modification history:                         

  Coded by:             Luca Deri
  
  Date:                 July 1993
  
***********************************************************************/
#if defined(__STDC__) || defined(__cplusplus)
int printInt(PS ps, int *elem, int quoted)
#else
int printInt(ps, elem, quoted)
PS ps;
int *elem;
int quoted;
#endif
{
  int theElem = *(int*)elem;

  ps_printf(ps, "%d", theElem); 
  return(OK);
}


/**********************************************************************
  
  Name:	                printChar
  
  Description:	        This function prints two char and it is used
                        when two complex structures, containing integer
			fields, have to be printed.
                        
  Input params.:        PS
                        The presentation stream where the function prints.

			char*
			The value to be printed.
		    
			int (not used)
			Flag: 1 = quoted print, 0 = not quoted print.


  Output params.:       int
                        NOTOK if some error is occurred, OK otherwise.


  Modification history:                         

  Coded by:             Luca Deri
  
  Date:                 July 1993
  
***********************************************************************/
#if defined(__STDC__) || defined(__cplusplus)
int printChar(PS ps, char *elem, int quoted)
#else
int printChar(ps, elem, quoted)
PS ps;
char *elem;
int quoted;
#endif
{
  char theElem = *(char*)elem;

  ps_printf(ps, "%c", theElem); 
  return(OK);
}



/**********************************************************************
  
  Name:	                printReal
  
  Description:	        This function prints two real and it is used
                        when two complex structures, containing integer
			fields, have to be printed.
                        
  Input params.:        PS
                        The presentation stream where the function prints.

			real*
			The value to be printed.
		    
			int (not used)
			Flag: 1 = quoted print, 0 = not quoted print.


  Output params.:       int
                        NOTOK if some error is occurred, OK otherwise.


  Modification history:                         

  Coded by:             Luca Deri
  
  Date:                 July 1993
  
***********************************************************************/
#if defined(__STDC__) || defined(__cplusplus)
int printReal(PS ps, double *elem, int quoted)
#else
int printReal(ps, elem, quoted)
PS ps;
double *elem;
int quoted;
#endif
{
  double theElem = *(double*)elem;

  ps_printf(ps, "%c", theElem); 
  return(OK);
}

/* This function converts a qbuf to a printable string 	*/
/* ie characters less than ASCII 32 or greater than	*/ 
/* 126 are hex dumped...				*/

#if defined(__STDC__) || defined(_cplusplus)
char* printFilter(struct qbuf* qb)
#else
char* printFilter(qb)
struct qbuf* qb;
#endif
{
	static char op[500];			/* NB returns pointer to this static space. 'Orrid but true */
	register char* pos = op;
	register struct qbuf* lnk = qb->qb_forw;

	register char* s;
	int i;
	*op = 0;
	do
		{
			s = lnk->qb_data;
			for(i=0; i < lnk->qb_len; i++)
				{
					if((unsigned int)(*s) > 31 && (unsigned int)(*s) < 127)
						*pos++ = *s;
					else
						pos += strlen(sprintf(pos," 0x%.2x ",(unsigned char)(*s)));
					s++;
				}
		}
	while((lnk = lnk->qb_forw) != qb );
	*(pos) = '\0';

	return op;
}

/* This function is called by on some enclosed qbufs */

#if defined(__STDC__) || defined(_cplusplus)
int String_print(PS ps, struct qbuf* elem, int quoted)
#else
int String_print(ps, elem, quoted)
PS ps;
struct qbuf* elem;
int quoted;
#endif
{
	char* s;
	if(elem == NULL)
		{
			ps_printf(ps, " NULL");
			return(OK); /* Nothing to print */
		}

	s = printFilter(elem);
	if(s)
		{
			ps_printf(ps, " {%s} ", s);
			return OK;
		}
	else
		{
			return (NOTOK);
		}

}

/* ditto for the compare */

#if defined(__STDC__) || defined(__cplusplus)
int String_compare(void *elem_a, void *elem_b)
#else
int String_compare(elem_a, elem_b)
void *elem_a;
void *elem_b;
#endif
{
	struct qbuf* elem_1 = (struct qbuf*)(elem_a);
	struct qbuf* elem_2 = (struct qbuf*)(elem_b);

	char* s_1;
	char* s_2;

	int result;

	if((elem_a && !elem_b) || (!elem_a && elem_b))
		return A_NOT_EQUAL_B;

	if(!elem_a && !elem_b)
		return A_EQUAL_B;

	s_1 = strdup(printFilter(elem_1));	/* copy this one 	*/
	s_2 = printFilter(elem_2);		/* use the static mem 	*/

	result = strcmp(s_1, s_2);

	free(s_1);

	if(result == 0) return A_EQUAL_B;
	if(result > 0) 	return A_GREATER_EQUAL_B;
	if(result < 0)	return A_LESS_EQUAL_B;
}

#endif

