#include "Error.h"
#include "SetSupportedMIBs.h"
#include "SnmpImageMOClassInfo.h"
/*
 * We need to check that simple input files are present for the
 * requested MIB OIDs. Also an "internetSystem" object specification
 * must be available.
 */
int	
SetSupportedMIBs::set( void* newval )
{
int	status = OK ;
Bool	internetSystemDefnFound = False ;
OID	mibOID ;
struct	type_PrxIIMCSntx_SupportedMIBs * newSMsval = (struct type_PrxIIMCSntx_SupportedMIBs *)newval ;
static	OID internetSystemOID = NULLOID ;

	if( internetSystemOID == NULLOID ) {

	    internetSystemOID = name2oid( "internetSystem" ) ;

	    if( internetSystemOID == NULLOID ) {
	    	advise( NULLCP, "proxyAgent.cc: SupportedMIBs set rejected as \"internetSystem\" is not in \"oidtable.gen\"" ) ;
	    	status = NOTOK ;
	    }
	}

	// Run through the supportedMIBs finding
	struct type_PrxIIMCSntx_SupportedMIBs * supportedMIBp, **supportedMIBpp ;

	for( supportedMIBpp = &newSMsval ;
	     status == OK && supportedMIBpp != NULL && (supportedMIBp = *supportedMIBpp) != NULL ;
	     supportedMIBpp = (struct type_PrxIIMCSntx_SupportedMIBs**)
	     				SupportedMIBs::_getnext( *supportedMIBpp ) ) {

	    if( (mibOID = supportedMIBp->ObjIdent) == NULLOID ) {
	    	advise( NULLCP, "proxyAgent.cc: SupportedMIBs set rejected due to presence of a NULL OID" ) ;
	    	status = NOTOK ;
		
	    } else {

		// Do we have a simple input file for this MIB ?
		if( MIBInfo::findMIBInfo( mibOID ) == NULLMIBINFO ) {
		    advise( NULLCP, "proxyAgent.cc: SupportedMIBs set rejected due to absence of MIB input file for %s", oid2name(mibOID, OIDPART)) ;
		    status = NOTOK ;
		    
		} else {
		    // Does the simple input file for this MIB contain 
		    // the definition for the "internetSystem" object ?
		    if( MIBInfo::getSnmpImageMOClassInfo( mibOID, internetSystemOID, False ) != NULLIMAGEINFO ) {
			internetSystemDefnFound = True ;
		    }
		}
	    }
	}

	if( status == OK && internetSystemDefnFound == False ) {
	    advise( NULLCP, "proxyAgent.cc: SupportedMIBs set rejected due to absence of an \"internetSystem\" input file definition" ) ;
	    status = NOTOK ;
	}
	     
	if( status == OK )
	    status = Attr::set( newval ) ; // complete the attribute set operation
	else
	    (void)SupportedMIBs_free( newSMsval ) ;

	return status ;
}

