#include "cmipsnmpProxyAgent.h"

/*
 * Remove leading and trailing spaces...
 */
char*
stripSpaces( char * tokencp )
{
	if( tokencp == NULLCP || *tokencp == '\0' )
	    return NULLCP ;

	// leading spaces..
	while( *tokencp && isspace( *tokencp ) )
	    tokencp++ ;

	// trailing spaces..
	char * lastcp = tokencp + strlen( tokencp ) - 1 ;

	while( lastcp >= tokencp && isspace( *lastcp ) ) {
	    lastcp-- ;
	}

	// Remove trailing spaces..
	*(lastcp+1) = '\0' ;

	return tokencp ;
}


int
str2attr( CMISParam& attr, char* attrTypecp, char* attrValuecp )
{
int	status = OK ;
Ava* 	ava ;

	if ((ava = str2ava(attrTypecp, attrValuecp)) == (Ava*)0 ) {
	    advise(NULLCP, "Error: invalid attribute type \"%s\" or value \"%s\"",
						    		attrTypecp, attrValuecp ) ;
	    status = NOTOK ;
	} else {
	    attr.mp_id.mid_type   = MID_GLOBAL;
	    attr.mp_id.mid_global = ava->ava_oid;
            attr.mp_val 	  = ava->ava_value;
            (void)free ((char*) ava);
	}
	
	return status ;
}

static int
bufferFullLine( char* linecp, int maxBufferLen, FILE* fp )
{
int	status = OK ;
int	maxBuffLenRemaining = maxBufferLen ;
char *	nextlinecp = linecp, *lcp ;
Bool	lineIsAComment = False, lineToBeContinued = False, validDataRead = False, lineIgnored = False ;

	*nextlinecp = '\0' ;

	do {

	    lineIsAComment = lineIgnored = False ;

	    if( (status = (fgets( nextlinecp, maxBuffLenRemaining, fp ) == NULL ) ? NOTOK : OK) == OK &&
		(lineIsAComment = ( nextlinecp[0] == '#' ) ? True : False ) == False ) {

		int len = strlen( nextlinecp );

		if( len >= maxBuffLenRemaining - 2 ) {
		    advise(NULLCP, "cmipsnmpProxy.cc:bufferFullLine() insufficient buffer size for line %s", linecp ) ;
		    status = NOTOK ; break ;
		}

	    	lcp = nextlinecp + len - 1 ;

	    	// work back down through the trailing whitespaces..
	    	while( lcp >= nextlinecp && isspace( *lcp ) )
		    --lcp ;

		if( lcp < nextlinecp ) { 

		    // ...all whitespace
		    lineIgnored = True ;

		} else {

		    if( (*lcp) == '\\' ) { // line continuation

		        (*lcp) =  ' ' ; // ...to guarantee the presence of a seperator 
		    	lineToBeContinued = True ;

		    } else lineToBeContinued = False ;

		    validDataRead = True ;

		    nextlinecp		+= len ; // prepare for the next iteration
		    maxBuffLenRemaining -= len ;
		}
	    }

	} while( status == OK && (lineIsAComment == True || lineToBeContinued == True || lineIgnored == True)) ;

	if( lineToBeContinued == True ) {
	    advise( NULLCP, "iqastartupagents: Continued line missing %s", linecp ) ;
	    status = NOTOK ;
	}

	return( status ) ;
}

#define CMIPSNMPPROXYAGENTID 	0
#define TRANSPORTADDRESSES	1
#define MANAGEMENTPROTOCOL	2
#define SUPPORTEDMIBS		3
#define ACCESSCONTROLMECHANISM	4
#define ADMINISTRATIVESTATE	5
#define SNMPMSGRETRYLIMIT	6
#define SNMPMSGTIMEOUT		7
int 
cmipsnmpProxy::createRR( AVA*& errorInfo, void* crTypep, int )
{
int	createRRstatus = OK ;

    	ProxyAlarmKS* proxyAlarmKS;
    	if (!(proxyAlarmKS = ProxyAlarmKS::getInstance()))
            proxyAlarmKS = new ProxyAlarmKS;

    	// register with it
    	proxyAlarmKS->registerMO( this, "cmipsnmpProxy" );
			

	// Now check to see if there are any predefined
	// remote agents that are to be proxied for..
#define CMIPSNMPSTARTUPBUFFLEN 		4096
	char 	filename[512], line[CMIPSNMPSTARTUPBUFFLEN];
	FILE * 	fp;

	if( (fp = fopen(strcat(strcat(strcpy(filename, etcdir), "iqadir/" ), IQA_START_UP_AGENTS_FILE ), "r")) == NULL ) {
	    advise( NULLCP, "iqastartupagents:: WARNING could not find the IQA SNMP agent initialisaion\nfile %s%s", "iqadir/", IQA_START_UP_AGENTS_FILE ) ;

	} else {

	    // Parse the agents file..

	    while( bufferFullLine( line, CMIPSNMPSTARTUPBUFFLEN-1, fp ) == OK ) {

		// Read in the parameters...
#ifdef DEBUG
		advise( NULLCP, "Processing: %s", line ) ;
#endif
		if( strchr( line, ',' ) == NULLCP ) {
		    advise( NULLCP, "iqastartupagents: no tokens in this line %s", line ) ;
		    continue ;
		}

		int	linelen ;
		char *	linecp, * delimcp, * tokencp ;
		line[ (linelen = strlen(line)) ] = ',' ; // change trailing '\0' to ','
		line[ linelen+1 ] = '\0' ;		 // and add a new string terminator

		linecp = line ;
		
		cmipsnmpProxyAgent * currProxyAgentMO = NULL ;
		RDN	proxyAgentRdn = NULLRDN ;
		char	rdnBuffer[1024] ;
		char *	trapCommcp = NULLCP ;
		struct	qbuf * communityString = NULL ;
		int	magicNumber = 0, tokenCount = 0, status = OK ;
		int	nattrs = I_cmipsnmpProxyAgentNATTRS, trapPort = -1 ;
		CMISParam*    attrs = (CMISParam*)malloc( sizeof(CMISParam) * nattrs ) ;

		// set up those attribute that are not loaded from a file..
		(void)str2attr( attrs[ACCESSCONTROLMECHANISM], "accessControlMechanism", "agent"  ) ;
		(void)str2attr( attrs[ADMINISTRATIVESTATE],    "administrativeState",    "unlocked"  ) ;

		//
		// Remember that the input line now ends in a trailing ','
		//
		while( status == OK && linecp != NULLCP &&
				(delimcp = strchr( linecp, ',' )) != NULLCP ) {

		    *delimcp = '\0' ; // extract the token string
		    tokencp  = stripSpaces( linecp ) ;
		    linecp   = delimcp + 1 ;
		    
		    switch( tokenCount ) {
#define INIT_PROXYSYSTEMID	0
#define INIT_TRANSPORTADDRESSES	1
#define INIT_MANAGEMENTPROTOCOL	2
#define INIT_SUPPORTEDMIBS	3
#define INIT_SNMPMSGRETRYLIMIT	4
#define INIT_SNMPMSGTIMEOUT	5
#define INIT_COMMUNITYSTRING	6
#define INIT_MAGICNUMBER	7
#define INIT_MinToken		7

#define INIT_TRAPPORT		8	/* Paired optional fields, which may appear */
#define INIT_TRAPCOMMSTRING	9	/* more than once 			    */
#define INIT_MaxToken		9

		    	case	INIT_PROXYSYSTEMID   :				
			case	INIT_COMMUNITYSTRING :
			case	INIT_TRAPCOMMSTRING  :
			
				// Quote delimited field..
				char * quote1cp, * quote2cp ;

				if( (quote1cp = strchr(  tokencp, '"' )) == NULLCP ||
				    (quote2cp = strrchr( tokencp, '"' )) == NULLCP ||
				    quote1cp == quote2cp ) {

				    advise( NULLCP, "iqastartupagents: quote problem with token %s", tokencp ) ;
				    status = NOTOK ;

				    break ;
				}

				// remote the quotes..
				++quote1cp ; *quote2cp = '\0' ;

				if( tokenCount == INIT_PROXYSYSTEMID ) {
				    sprintf( rdnBuffer, "cmipsnmpProxyAgentId=%s", quote1cp ) ;
				    proxyAgentRdn = str2rdn( rdnBuffer ) ;
				    attrs[CMIPSNMPPROXYAGENTID].mp_id.mid_type   = MID_GLOBAL ;
				    attrs[CMIPSNMPPROXYAGENTID].mp_id.mid_global = name2oid( "cmipsnmpProxyAgentId" ) ;
				    attrs[CMIPSNMPPROXYAGENTID].mp_val	 	 = NULLPE  ;

				} else if( tokenCount == INIT_COMMUNITYSTRING ) {
				    communityString = str2qb( quote1cp, strlen( quote1cp ), 1 ) ; 

				} else if( tokenCount == INIT_TRAPCOMMSTRING ) {

				    trapCommcp = strdup( quote1cp ) ;

				    if( currProxyAgentMO != NULL &&
						trapCommcp != NULLCP && trapPort > -1 ) {

					currProxyAgentMO->appendTrap( trapPort, trapCommcp ) ;

					if( proxyAlarmKS->initialisePort( trapPort ) == NOTOK ) {
					    advise(NULLCP, "iqastartupagents: Initialise UDP Port failure for port %d", trapPort ) ;
					    status = NOTOK ;
			    		}
				    }

				    if( trapCommcp != NULLCP ) {
					(void)free( trapCommcp ) ; trapCommcp = NULLCP ;
				    }
				}

				break ;

			case	INIT_TRANSPORTADDRESSES :

				status = str2attr( attrs[TRANSPORTADDRESSES],
							"transportAddresses", tokencp ) ;
				break ;

			case	INIT_MANAGEMENTPROTOCOL :
				status = str2attr( attrs[MANAGEMENTPROTOCOL],
							"managementProtocol", tokencp ) ;

				break ;

			case	INIT_SUPPORTEDMIBS :
				status = str2attr( attrs[SUPPORTEDMIBS],
							"supportedMIBs", tokencp ) ;
				break ;

			case	INIT_SNMPMSGRETRYLIMIT :
				status = str2attr( attrs[SNMPMSGRETRYLIMIT],
							"snmpMsgRetryLimit", tokencp ) ;
				break ;

			case	INIT_SNMPMSGTIMEOUT :
				status = str2attr( attrs[SNMPMSGTIMEOUT],
							"snmpMsgTimeout", tokencp ) ;
				break ;

			case	INIT_MAGICNUMBER :
				magicNumber = atoi( tokencp ) ;

				// ...last mandatory token...

				CMISErrors err = m_noError ;
			        AVA * 	   errorInfop = NULLAVA ;

				// attempt to construct the agent objects..
				currProxyAgentMO = 
				    (cmipsnmpProxyAgent*)cmipsnmpProxyAgent::
				    	cmipsnmpStartUpProxyAgent(
					    errorInfop,
					    proxyAgentRdn, this, nattrs, attrs, 
					    communityString, magicNumber, &err ) ;

				if( errorInfop )
				    delete errorInfop ;
				    
				break ;

			case	INIT_TRAPPORT :
				trapPort = atoi( tokencp ) ;

				if( trapPort != PROXYALARMDEFTRAPPORT &&
				    trapPort <  PROXYALARMNONDEFMINTRAPPORT ) {

				    advise( NULLCP, "iqastartupagents: invalid trap port number %s", tokencp ) ;
				    status = NOTOK ;
				}
				break ;
			
			default:advise( NULLCP, "iqastartupagents: Too many tokens !" ) ;
				status = NOTOK ; // should never occur due to multiple trap fields !
				break ;
		    }

		    if( tokenCount == INIT_TRAPCOMMSTRING )
		    	tokenCount =  INIT_TRAPPORT ; /* Multiple trap fields possible */
		    else
			++tokenCount ;

		} // while loop


		if( status == OK ) {

		    if( tokenCount > INIT_MaxToken + 1 )
		    	advise( NULLCP, "iqastartupagents: too many tokens" ) ;
		    else if( tokenCount <= INIT_MinToken )
		    	advise( NULLCP, "iqastartupagents: not enough tokens" ) ;
		}

		(void)free( (char*) attrs ) ;
	    }
	    
	    (void)fclose( fp ) ;
	}
	
	return createRRstatus ;
}
