#include "remoteSystem.h"
#include "SetSupportedMIBs.h"

int
cmipsnmpProxyAgent::initialiseAttrs( RDN rdn, int nAttrs, CMISParam *attrs )
{
int		status = OK ;
CMISErrors 	localError = m_noError ;
AVA *		errorInfop ;

	_snmpTrapDetailsArrayp = new ProxyAgentTrapDetailsArray( True ) ;

	accessControlMechanism()->set( int_PrxIIMCSntx_AccessControlEnforcement_agent ) ;


	for(register int i = 0; status == OK && localError == m_noError && i < nAttrs ; i++) {

	    localError = m_noError ;

	    if (attrs[i].mp_id.mid_type != MID_GLOBAL)
		localError = m_noSuchAttribute;

	    else switch( getClassInfo()->getAttrId(attrs[i].mp_id.mid_global ) ) {

        	case I_cmipsnmpProxyAgentId :
			// RDN attribute Id has already been processed
			localError = m_noError ;
			break ;
            
        	case I_transportAddresses :
                	localError = _attrs[I_transportAddresses]->set(attrs[i].mp_val);
            		break;
		case I_managementProtocol :
			localError = _attrs[I_managementProtocol]->set(attrs[i].mp_val);
			break;
		case I_supportedMIBs :
			localError = _attrs[I_supportedMIBs]->set( attrs[i].mp_val );
			break ;
		case I_accessControlMechanism :
			localError = _attrs[I_accessControlMechanism]->set( attrs[i].mp_val );
			break ;
		case I_administrativeState :
			localError = _attrs[I_administrativeState]->set( attrs[i].mp_val );
			break ;
		case I_snmpMsgRetryLimit :
			localError = _attrs[I_snmpMsgRetryLimit]->set( attrs[i].mp_val );
			break ;
		case I_snmpMsgTimeout :
			localError = _attrs[I_snmpMsgTimeout]->set( attrs[i].mp_val );
			break ;

        	default:
            		localError = m_noSuchAttribute;
            		break;
	    }

	

	    if ( localError != m_noError )
	    	status = NOTOK ;
	    
	    switch ((int) localError ) {

        	case (int) m_noError :
			mid_free(&attrs[i].mp_id);
			break;

		case (int) m_processingFailure :
		case (int) m_noSuchAttribute :
			mid_free(&attrs[i].mp_id);

			for( ++i ; i < nAttrs; i++) {
			    mid_free(&attrs[i].mp_id);
			}
			return status ;  

        	case (int) m_invalidAttributeValue:
			mid_free(&attrs[i].mp_id);

			for( ++i ; i < nAttrs; i++) {
			    mid_free(&attrs[i].mp_id);
			}
            		return status ;
	    }
    	}

	return status ;
}

int
cmipsnmpProxyAgent::createRR( AVA*& errorInfo, void*, int )
{
int	status = OK ;

	_snmpTrapDetailsArrayp = new ProxyAgentTrapDetailsArray( True ) ;

	accessControlMechanism()->set( int_PrxIIMCSntx_AccessControlEnforcement_agent ) ;

	status = cmipsnmpStartUpRemoteAgent( errorInfo, this->getSuperior() ) ;

	return status ;
}

int
cmipsnmpProxyAgent::deleteRR( AVA*& err, void* info, Bool checkOnly, int )
{
int	status = OK ;

	if( _snmpRMIBagentp )
	    delete( _snmpRMIBagentp ) ;

	if( _snmpRemoteSystemMO )
	    delete _snmpRemoteSystemMO ;

	if( _snmpTrapDetailsArrayp ) {
	    _snmpTrapDetailsArrayp->clear() ;
	    delete _snmpTrapDetailsArrayp ;
	}

	return status ;
}

void
cmipsnmpProxyAgent::appendTrap( int trapPort, char* trapCommcp )
{
	if( trapPort != -1 && trapCommcp != NULLCP ) {

	    /* Buffer the trap details to this object's storage.. */
	    struct  ProxyAgentTrapDetails* paTrapDetailsp = new ProxyAgentTrapDetails ;
	    paTrapDetailsp->udpTrapPort = trapPort ;
	    paTrapDetailsp->trapCommunityStringqb = str2qb( trapCommcp, strlen( trapCommcp ), 1 ) ;
	    _snmpTrapDetailsArrayp->add( paTrapDetailsp ) ;
	}

	return ;
}



//
// Note: if a NULL community string is provided then the "snmpSecurityParameter" object is examined.
//
int
cmipsnmpProxyAgent::cmipsnmpStartUpRemoteAgent( AVA*& errorInfo, MO* superior, 
				struct qbuf* communityString, int magicNumber)
{
int	status = OK ;
Bool	remoteAgentIsReachible = False ;
MO * 	remoteSystemMO         = NULLMO ;
SnmpRMIBerrorEnum snmpRMIBerrorEnum = SnmpRMIBnoError ;

	SnmpRMIBAgent * snmpRMIBagentp = _snmpRMIBagentp ;

	// Now proceed to create the related objects..
	if( snmpRMIBagentp = new SnmpRMIBAgent( magicNumber ) ) {

	    int * reqTimeOutmsp = (int*)getAttrVal("snmpMsgTimeout"),
 		* retryLimitp   = (int*)getAttrVal("snmpMsgRetryLimit") ;

	    // If either SNMP parameter has not been set then use the
	    // values held by the superior cmipsnmpProxy object..
	    if( *reqTimeOutmsp == -1 )
		reqTimeOutmsp = (int*)superior->getAttrVal( "snmpMsgTimeout" ) ;

	    if( *retryLimitp == -1 )
		retryLimitp = (int*)superior->getAttrVal( "snmpMsgRetryLimit" ) ;

	    setSnmpRMIBAgent( snmpRMIBagentp ) ;

	    // determine the community string..
	    if( communityString == NULL ) {

		// ...it was not supplied as a method parameter...

		// Check if the PAC has supplied the community string...
		if( (communityString = retrieveCommunityFromPAC( getAccessControl() )) == NULL ) {
		    
		    // ...PAC did not provide it so attempt to retrieve the default value 
		    // from the "snmpSecurityParameter" object that is contained immediately 
		    // below the "cmipsnmpProxy" object..

		    MO *subordMO = superior->getSubordinate(), *secureMO=NULLMO ;
		    char* classNamecp ;

		    while( subordMO != NULLMO && 
		    		(classNamecp = subordMO->getClassName()) != NULLCP ) {

 			if( !strcmp( classNamecp, "snmpSecurityParameter" ) ) {
			    secureMO = subordMO ; break ;
			 
			} else subordMO = subordMO->getPeer() ;
		    }

		    struct type_PrxIIMCSntx_SnmpSecurity * sp ;

		    if( secureMO == NULLMO ||
			 (sp = (struct type_PrxIIMCSntx_SnmpSecurity*)secureMO->getAttrVal( "snmpSecurity" )) == NULL ||
			 (communityString = retrieveCommunityStringSecurityParameter( sp )) == NULL ) {

			// ...no luck there either so use "public"...

			// If called during "cmipsnmpProxy" creation then the "snmpSecurityParameter"
			// MO will be unavailable as it is created just afterwards..
		    	communityString = str2qb( "public", strlen( "public" ), 1 ) ;
		    }
		}
	    }


	    if( reqTimeOutmsp != (int*)0 && retryLimitp != (int*)0 && communityString != NULL ) {
		
		struct type_PrxIIMCSntx_TransportAddresses * transportAddressesp =
				(struct type_PrxIIMCSntx_TransportAddresses *)getAttrVal("transportAddresses") ;

		for( ; 
		     remoteAgentIsReachible == False && transportAddressesp != NULL ; 
		     transportAddressesp = transportAddressesp->next ) {

		    if( transportAddressesp->member_PrxIIMCSntx_1->address &&
		        (snmpRMIBerrorEnum   = snmpRMIBagentp->snmpV1openSession(
						transportAddressesp->member_PrxIIMCSntx_1->address,
						communityString,
						*retryLimitp,
						(*reqTimeOutmsp)/1000) ) == SnmpRMIBnoError ) {

			// Finally attempt to create the corresponding remote system object..
			char remoteSystemRdnBuff[ 256 ] ;
			RDN  remoteSystemRdn ;
			MO * remoteSystemSuperior = superior->getSuperior()->getSuperior() ;
			char * rdncp = getRDNString() ;

			if( rdncp != NULLCP ) {

			    char* rdnValuecp ;
			    if( (rdnValuecp = strchr( rdncp, '=' )) != NULLCP ) {

				++rdnValuecp ;
				sprintf( remoteSystemRdnBuff, "systemId=%s", rdnValuecp ) ;
				if( (remoteSystemRdn = str2rdn( remoteSystemRdnBuff )) != NULLRDN ) {

				    remoteSystemMO = new remoteSystem( errorInfo, remoteSystemRdn, remoteSystemSuperior,
					    			this, remoteAgentIsReachible ) ;

				} else {
				    advise( NULLCP, "Failed to parse remoteSystem RDN %s", remoteSystemRdnBuff ) ;
				}
			    }
			
			    (void)free( rdncp ) ;
			}
		    }

		} // for loop
	    }

	    if( communityString != NULL )
		(void)qb_free( communityString ) ;
	}

#ifdef	DEBUG
	if( remoteAgentIsReachible == True )
	    fprintf(stderr, "cmipsnmpProxyAgent Object created\n");
#endif

	_snmpRemoteSystemMO = remoteSystemMO ;

	if( remoteAgentIsReachible == False ) {

	    status = NOTOK ;
	}
	
        return status ;
}


MO * cmipsnmpProxyAgent::cmipsnmpStartUpProxyAgent( AVA*& errorInfo, RDN rdn, MO *superior,
				int nAttrs, CMISParam *attrs, 
				struct qbuf* communityString, int magicNumber, CMISErrors *errp )
{
	MO *mo = new cmipsnmpProxyAgent( rdn, superior, True, NULLMO, nAttrs, attrs, errp, (CMISErrorInfo*)0, (OID*)0 );

	if( ((cmipsnmpProxyAgent*)mo)->initialiseAttrs( rdn, nAttrs, attrs ) == NOTOK ||
	    ((cmipsnmpProxyAgent*)mo)->cmipsnmpStartUpRemoteAgent( errorInfo, superior, 
					communityString, magicNumber )  == NOTOK   ) {
	    mo->deleteRR( DISCARDAVA ) ;
	    delete mo, mo = NULLMO ;
	}
	
	return mo ;
}


//
// Note: no new storage is created so the returned list must
//       NOT be freed.
//
ObjIdListVal * 	cmipsnmpProxyAgent::getMIBOIDs( void )
{
ObjIdListVal * 	supportedMIBsListp = (ObjIdListVal*)0 ;

	supportedMIBsListp = (ObjIdListVal*)(supportedMIBs()->getval()) ;

	return supportedMIBsListp ;
}
