#if !defined( _SNMP_IF_ )
#define _SNMP_IF_

#include 	"SnmpRMIBerrors.h"
#include        "SnmpRMIBparty.h"

#define NULLAUTHMSG	((struct type_SNMP_SnmpAuthMsg *) 0)
#define NULLPARTY	((struct snmp_party *) 0)
#define NULLCONTEXT	((struct snmp_context *) 0)


typedef enum {

	SnmpVunknown = -1,
	SnmpV1 	     =  1,
	SnmpV2       =  2

} SnmpVersionEnum ;

typedef enum 	{

	/*
	 * These must have the same values as the "type_SNMP_PDUs_..."
	 * within SNMP-types.h
	 */

	UnknownOperator	    = -1,
	GetOperator 	    = 1,
	GetNextOperator     = 2,
	GetBulkOperator     = 3,
	GetResponseOperator = 4,
	SetOperator 	    = 5,
	InformOperator      = 6,
	SnmpV2TrapOperator  = 7,
	TrapOperator 	    = 8

} SnmpOperationsEnum ;


/*
 * Contains all the results received (so far)
 * for a given operationId.
 */
struct	SnmpRMIBrxedResults {

	SnmpOperationsEnum snmpOperation ;

	int	numVarBindings ;

	/* NOTE: THE RESULTS WILL NOT NECESSARILY BE ORDERED */
	OID *	resOIDvec ; /* Note: resOIDvec, resPEvec and resAttrIds are indexed 0..(numVarBindings-1) */
	PE  *	resPEvec  ; /* which may be less than the number of OIDs requested in  */	                      
	int *	resAttrIds; /* the originating operation. */

	int	numOperOids ;
	int *	OIDspecificSNMPerrorp ; /* 0..(numOperOids-1) */
		/* ..Indexed as the originating operation OIDvec causing the error */
		/* Set to the SNMP-types.h int_SNMP_error__status_XXX              */
		/* values plus int_SNMP_error__status_timeOut                      */
		/* Note: buffered across from SnmpRMIBoperationNode when operation is complete */

	int *	operAttrIds ; /* 0..(numOperOids-1) */
		/* ..the MO attribute Id for each index in OIDspecificSNMPerrorp */
		/* Note: buffered across from SnmpRMIBoperationNode when operation is complete */
	
	/* The following pointer is used solely for grouping
	   the results of multiple operations... */
	struct  SnmpRMIBrxedResults * nextSnmpRMIBrxedResultsNodep ;
} ;

#define NULLSNMPRMIBRXEDRESULTS ((struct SnmpRMIBrxedResults *) 0)



struct	SnmpRMIBtxPduNode {

	SnmpVersionEnum version ;
	
	struct	SnmpRMIBoperationNode * superiorSnmpRMIBoperNodep ; /* cross reference */

	int	requestId ;

	int	baseIndexWithinOperationOIDvec ;

	OID *	OIDvec  ;
	int	numOids ;
	PE  *	PEvec   ;
	
	int	reTxCount ; /* Incremented upon each timeout re-transmission */

	struct	type_SNMP_SnmpAuthMsg * authMsg ;

	int	snmpErrorReceived ; /* filled in when PDU response received */
	int	snmpErrorIndex	  ; /* filled in when PDU response received */

	int	attemptToRecoverFromPossibleMissingTooBigResponse ; /* TRUE/FALSE */
	
	int	pduTxComplete ; /* TRUE/FALSE */
	
	struct	SnmpRMIBtxPduNode * nextSnmpRMIBpeerTxPduNodep ;
	struct	SnmpRMIBtxPduNode * nextSnmpRMIBsubordTxPduNodep ;
} ;

#define NULLSNMPRMIBTXPDUNODE  ((struct SnmpRMIBtxPduNode *) 0)


struct	SnmpRMIBoperationNode {

	SnmpVersionEnum version ;

	SnmpOperationsEnum snmpOperation ;

	int	operationId ;
	int	numOperOids ;

	int	tooBigMaxOidsPerPdu ; /* 1 unless a "tooBig" error has occurred */

	struct	SnmpRMIBtxPduNode     	* snmpRMIBtxPduNodep ;
	struct	SnmpRMIBrxedResults * snmpRMIBreceivedResultsNodep ;
		/* ..Note: this pointer will be NULLed when a full set of results have been retrieved
		   and the SnmpRMIBrxedResults node is to be made known to the SnmpRMIBAgent */
					
#define  int_SNMP_error__status_timeOut -1 
	int *	OIDspecificSNMPerrorp ; /* indexed as the operation OIDvec causing the error */
		/* ..set to the SNMP-types.h int_SNMP_error__status_*
		   values plus int_SNMP_error__status_timeOut
		   Note: this pointer will be NULLed when a full set of results have been retrieved
		   and the SnmpRMIBrxedResults node is to be made known to the SnmpRMIBAgent */

	int *	operAttrIds ; /* ..the MO attribute Id for each index in OIDspecificSNMPerrorp */
		/* Note: this pointer will be NULLed when a full set of results have been retrieved
		   and the SnmpRMIBrxedResults node is to be made known to the SnmpRMIBAgent */

	int	numberOfResponsesReceivedThisOper ;
	long	numberOfResponsesReceivedUpToThisOper ;
	
	struct  SnmpRMIBoperationNode * nextSnmpRMIBoperationNodep ;
} ;

#define NULLSNMPRMIBOPERATIONNODE  ((struct SnmpRMIBoperationNode *) 0)



/* Data required internally to the SNMPv1/v2 library
   and initialised within the SnmpRMIBAgent class... */

struct	SnmpRMIBsnmpifData {

	SnmpVersionEnum	version ;

	int	sd ;
	PS	ps ;
	
	struct	snmp_party   * agentParty   ;
	struct	snmp_party   * managerParty ;
	struct	snmp_context * mgmtContext  ;

	int	maxRetries ;
	int	timeoutS ;

	int	agentOperationId ; /* holds the last used operation ID        */
	int	agentRequestId   ; /* holds the last used Snmp PDU request ID */

#define SNMPRMIBOUTSANDINGIDSBLOCKSIZE 	50
	int  *	outstandingRequestIDs    ;
	long *	pduTransmittedAtS        ; /* Note: reTxs mean these will not necessarily be ordered by time */
	int	numOutstandingRequestIDs ;

	long	numOfResponsesReceived ;

#if defined( __cplusplus )
	int	(*tooBigReduceRetryFN)(int) ; /* Function pointer to the procedure that determines how to
					      reduce the number of OIDs in the retry after a "tooBig" error */
#else
	int	(*tooBigReduceRetryFN)()    ; /* Function pointer to the procedure that determines how to
					      reduce the number of OIDs in the retry after a "tooBig" error */
#endif

	int	remoteAgentMayIgnoreTooBigMessages ; /* TRUE/FALSE */
	
	struct	SnmpRMIBoperationNode   * snmpRMIBoperationNodep ;
} ;

#define NULLSNMPRMIBIFDATA	((struct SnmpRMIBsnmpifData *) 0)

#ifdef __cplusplus

extern "C" {

SnmpRMIBerrorEnum SnmpRMIBpartyConnect( struct SnmpRMIBsnmpifData * ) ;
void SnmpRMIBpartyDisconnect( struct SnmpRMIBsnmpifData * ) ;
SnmpRMIBerrorEnum SnmpRMIBsendMessageToAgent( struct SnmpRMIBsnmpifData *, SnmpOperationsEnum, 
							OID *, int, PE *, int *, int *, int ) ;
SnmpRMIBerrorEnum SnmpRMIBreceiveMessageFromAgent( struct SnmpRMIBsnmpifData *, int, 
						struct  SnmpRMIBrxedResults **, int * ) ;
void SnmpRMIBremoveReceivedResultsNode( struct SnmpRMIBrxedResults ** ) ;
void SnmpRMIBremoveReceivedResultsList( struct SnmpRMIBrxedResults ** ) ;
int  SnmpRMIBnumResultNodeBindings( struct SnmpRMIBrxedResults * ) ;
struct type_SNMP_Snmp1157Msg * SnmpRMIBdecodeTrap( PE ) ;

}

#else

SnmpRMIBerrorEnum SnmpRMIBpartyConnect() ;
void SnmpRMIBpartyDisconnect() ;
SnmpRMIBerrorEnum SnmpRMIBsendMessageToAgent() ;
SnmpRMIBerrorEnum SnmpRMIBreceiveAndProcess() ;
void SnmpRMIBremoveReceivedResultsNode() ;
void SnmpRMIBremoveReceivedResultsList() ;
int  SnmpRMIBnumResultNodeBindings() ;
struct type_SNMP_Snmp1157Msg * SnmpRMIBdecodeTrap() ;

#endif

#endif
