/*
// Copyright (c) 1994 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/

#include	<string.h>
#include 	<stdio.h>

#include	"SnmpRMIBerrors.h"
#include 	"objects.h"
#include	"party.h"


#ifndef	NULL
#define	NULL	0
#endif


static unsigned qbcharlen( struct qbuf* );


SnmpRMIBerrorEnum 
buildSNMPv1agentParty( transportAddress, communityString, agentPartypp )
struct 	qbuf *transportAddress , *communityString ;
struct	snmp_party ** agentPartypp ;
{
SnmpRMIBerrorEnum       SnmpRMIBerror = SnmpRMIBnoError ;
struct  snmp_party *	partyp ;

int i ;
	if( transportAddress == NULL || communityString == NULL ) {
	    SnmpRMIBerror = SnmpRMIBnullParameter ;

	} else if( (partyp = *agentPartypp = (struct snmp_party *)calloc( 1, sizeof *partyp)) == NULL ) {
	    SnmpRMIBerror = SnmpRMIBmemAllocFailure ;		

	} else if( (partyp -> party_TAddress = (u_char*)qb2str( transportAddress )) == NULL ) {

	    SnmpRMIBerror = SnmpRMIBmemAllocFailure ; /* Probably ! */
	    (void)free( (char *) partyp ) ;

	} else if( (partyp -> party_AuthPrivate	= (u_char*)qb2str( communityString )) == NULL ) {

	    SnmpRMIBerror = SnmpRMIBmemAllocFailure ; /* Probably ! */

	    (void)free( (char *) partyp -> party_TAddress ) ;
	    (void)free( (char *) partyp ) ;

	} else {
		   
	    partyp -> party_TAddress_len = qbcharlen( transportAddress ) ;
	    
	    partyp -> party_tDomain  = RFC1157DOMAIN ;

	    partyp -> party_AuthPrivate_len = qbcharlen( communityString ) ;

	    partyp -> party_authProt = RFC1157NOAUTH ;
	    partyp -> party_privProt = NOPRIV ;

	    partyp -> party_MaxMessageSize = MAXSNMP ;

	    partyp -> party_location = AGENT ;

	    partyp -> party_status   = ROW_ACTIVE ;
	    partyp -> party_storage  = STORAGE_NONVOLATILE ;

	    partyp -> party_flags = PARTY_NULL ;
	}

	return( SnmpRMIBerror ) ;
}

            
void
removeSNMPv1agentParty( agentPartypp )
struct	snmp_party ** agentPartypp ;
{
struct	snmp_party * agentPartyp ;

	if( agentPartypp && (agentPartyp = *agentPartypp) ) {

	    if( agentPartyp->party_TAddress )
	    	(void)free((char*) agentPartyp->party_TAddress ) ;

	    if( agentPartyp->party_AuthPrivate )
	        (void)free((char*) agentPartyp->party_AuthPrivate ) ;

	    (void)free((char*) agentPartyp ) ;
	    
	    *agentPartypp = NULL ;
	}
	
	return ;
}


SnmpRMIBerrorEnum 
buildSNMPv1managerParty( managerPartypp )
struct  snmp_party ** managerPartypp ;
{
SnmpRMIBerrorEnum       SnmpRMIBerror = SnmpRMIBnoError ;
struct  snmp_party *	partyp ;

	if( (partyp = *managerPartypp = (struct snmp_party *)calloc( 1, sizeof *partyp)) == NULL ) {
	    SnmpRMIBerror = SnmpRMIBmemAllocFailure ;		
	}
	else {

	    partyp -> party_tDomain  = RFC1157DOMAIN ;

	    partyp -> party_authProt = RFC1157NOAUTH ;
	    partyp -> party_privProt = NOPRIV ;

	    partyp -> party_MaxMessageSize = MAXSNMP ;

	    partyp -> party_location = MANAGER ;

	    partyp -> party_status   = ROW_ACTIVE ;
	    partyp -> party_storage  = STORAGE_NONVOLATILE ;

	    partyp -> party_flags = PARTY_NULL ;
	}

	return( SnmpRMIBerror ) ;
}

void
removeSNMPv1managerParty( managerPartypp )
struct	snmp_party ** managerPartypp ;
{
struct	snmp_party * partyp ;

	if( managerPartypp && (partyp = *managerPartypp) ) {

	    (void)free((char*) partyp ) ;
	    
	    *managerPartypp = NULL ;
	}
	

	return ;
}


SnmpRMIBerrorEnum
buildSNMPv1Context( mgmtContextpp )
struct snmp_context ** mgmtContextpp ;
{
SnmpRMIBerrorEnum       SnmpRMIBerror = SnmpRMIBnoError ;
struct snmp_context *	ctxtp ;

	if( (ctxtp = *mgmtContextpp = (struct snmp_context *)calloc( 1, sizeof *ctxtp)) == NULL ) {
	    SnmpRMIBerror = SnmpRMIBmemAllocFailure ;
	}
	
	return( SnmpRMIBerror ) ;
}

void
removeSNMPv1Context( mgmtContextpp )
struct snmp_context **  mgmtContextpp ;
{
struct snmp_context *	ctxtp ;

	if( mgmtContextpp && (ctxtp = *mgmtContextpp) ) {

	    (void)free((char*) ctxtp ) ;
	    
	    *mgmtContextpp = NULL ;
	}
	

	return ;
}


/* To return string lengths when NULLs may be present.. */
static unsigned 
qbcharlen( q )
struct qbuf * q ;
{
register unsigned len ;
register char  *b, *d ;
register struct qbuf   *p;

    	p = q -> qb_forw, len = 0;

    	do {

	    len += p -> qb_len;

	    p = p -> qb_forw;

    	} while (p != q);

	return len ;
}
