//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// SimOS.cc

// By George Pavlou, February 1994


#include <stdlib.h>
#include "msap.h"
#include "Error.h"
#include "IsodeCoord.h"
#include "CmisAgent.h"
#include "SimOS.h"
#include "TimeManagerRelay.h"


main (int argc, char** argv)
{
    if (argc < 2)
        error(NULLCP, "usage: simos <host>");

    // initialise the ASN.1 syntaxes
    if (initialiseSyntaxes(NULLCP, syntaxes) == NOTOK)
	error(NULLCP, "cannot initialise syntaxes");

    // instantiate the coordinator object
    ISODECoordinator coordinator;
    TimeManagerRelay timeManager;	// to relay time "upwards"

    // instantiate and initialise the CMIS agent object
    CMISAgent agent;
    TSAPdisconnect tds;
    if (agent.initialise(1, argv, service, &tds) == NOTOK)
	error(NULLCP, "isode error: [%s]", TErrString(tds.td_reason));

    // create the initial managed objects through the ETC/MIB_INIT_FILE
    if (MO::initialiseMIB(MIB_INIT_FILE) == NOTOK)
	error(NULLCP, "MIB initialisation failed");

    // instantiate the RMIB agent and try to connect to the agent
    // that can tell if the time is real or simulated -
    // then initialise the time manager who will find this out

    RMIBAgent rmibAgent;
    if (rmibAgent.connect("SMA", argv[1]) == NOTOK)
        error(NULLCP, "unable to establish association to simsma at %s",
                        argv[1]);

    if (timeManager.initialise(&rmibAgent) == NOTOK)
        error(NULLCP, "cannot initialise time manager!");

    // start listening...
#ifdef DEBUG
    advise(NULLCP, "listening...");
#endif
    coordinator.listen();
}

