//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// GenericKS.cc - generic knowledge source realisation

// By George Pavlou, February 1994


#include <stdlib.h>
#include "msap.h"
#include "Coordinator.h"
#include "TimeManager.h"
#include "GenericKS.h"


int  KS::scheduleWakeUp (long intvl, char* token, Bool ifReporting)
{
    if (intvl <= 0) {
	fprintf(stderr, "KS::scheduleWakeUp: invalid wake-up interval!\n");
	return NOTOK;
    }

    Coordinator* coord = Coordinator::getInstance();
    if (!coord)
	return NOTOK;

    TimeManager* timeMgr = TimeManager::getInstance();

    if (!timeMgr || timeMgr -> getTimeMode() == TM_REAL)
	return coord ->
	    scheduleWakeUps(this, token ? token : "", intvl, ifReporting, True);
    else	// TM_SIMULATED
	return timeMgr ->
	    scheduleWakeUps(this, token ? token : "", intvl, ifReporting, True);
}


int  KS::scheduleWakeUps (long intvl, char* token, Bool ifReporting)
{
    if (intvl <= 0) {
	fprintf(stderr, "KS::scheduleWakeUp: invalid wake-up interval!\n");
	return NOTOK;
    }
    Coordinator* coord = Coordinator::getInstance();
    if (!coord)
	return NOTOK;

    TimeManager* timeMgr = TimeManager::getInstance();

    if (!timeMgr || timeMgr -> getTimeMode() == TM_REAL)
	return coord ->
	    scheduleWakeUps(this, token ? token:"", intvl, ifReporting, False);
    else	// timeMode == TM_SIMULATED
	return timeMgr ->
	    scheduleWakeUps(this, token ? token:"", intvl, ifReporting, False);
}


int  KS::cancelWakeUps (char* token)
{
    Coordinator* coord = Coordinator::getInstance();
    if (!coord)
	return NOTOK;

    TimeManager* timeMgr = TimeManager::getInstance();

    if (!timeMgr || timeMgr -> getTimeMode() == TM_REAL)
	return coord -> cancelWakeUps(this, token ? token : "");
    else	// timeMode == TM_SIMULATED
	return timeMgr -> cancelWakeUps(this, token ? token : "");
}


int  KS::startListen (int fd)
{
    if (fd < 0) {
	fprintf(stderr, "KS::start_listen: invalid fd %d!\n", fd);
	return NOTOK;
    }
    Coordinator* coord = Coordinator::getInstance();
    if (!coord)
	return NOTOK;
    return coord -> registerCommEndpoint(this, fd);
}


int  KS::stopListen (int fd)
{
    Coordinator* coord = Coordinator::getInstance();
    if (!coord)
	return NOTOK;
    return coord -> deregisterCommEndpoint(this, fd);
}


int  KS::notifyShutdown ()
{
    Coordinator* coord = Coordinator::getInstance();
    if (!coord)
	return NOTOK;
    coord -> notifyShutdown(this);
    return OK;
}

