/*
 * Copyright (c) 1994 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/*
** MSAP Credential syntaxes for authentication dialogue
*/

/*
** Saleem N. Bhatti
** March 1994
*/

#include <stdio.h>

#if defined(__GNUG__)
#include <std.h>
#else
#include <stdlib.h>
#endif

#include "CredSntx.h"
#include "isode/quipu/authen.h"

PE
#if defined(__STDC__) || defined(__cplusplus)
MsapCredentials_encode(struct type_Cred_MsapCredentials *v)
#else
MsapCredentials_encode(v)
struct type_Cred_MsapCredentials *v;
#endif
{
    PE pe;

    if (build_Cred_MsapCredentials(&pe, 1, 0, (char *) 0, v) != OK)
        return (PE) 0;

    return pe;
}

struct type_Cred_MsapCredentials *
#if defined(__STDC__) || defined(__cplusplus)
MsapCredentials_decode(PE pe)
#else
MsapCredentials_decode(pe)
PE pe;
#endif
{
    struct type_Cred_MsapCredentials *v;

    if (parse_Cred_MsapCredentials(pe, 1, (int *) 0, (char **) 0, &v) != OK)
        return (struct type_Cred_MsapCredentials *) 0;

    return v;
}

void
#if defined(__STDC__) || defined(__cplusplus)
MsapCredentials_free(struct type_Cred_MsapCredentials *v)
#else
MsapCredentials_free(v)
struct type_Cred_MsapCredentials *v;
#endif
{
    if (v != (struct type_Cred_MsapCredentials *) 0)
        free_Cred_MsapCredentials(v);
}

PE
#if defined(__STDC__) || defined(__cplusplus)
SessionKey_encode(struct type_Cred_SessionKey *v)
#else
SessionKey_encode(v)
struct type_Cred_SessionKey *v;
#endif
{
    PE pe;

    if (build_Cred_SessionKey(&pe, 1, 0, (char *) 0, v) != OK)
        return (PE) 0;

    return pe;
}

struct type_Cred_SessionKey *
#if defined(__STDC__) || defined(__cplusplus)
SessionKey_decode(PE pe)
#else
SessionKey_decode(pe)
PE pe;
#endif
{
    struct type_Cred_SessionKey *v;

    if (parse_Cred_SessionKey(pe, 1, (int *) 0, (char **) 0, &v) != OK)
        return (struct type_Cred_SessionKey *) 0;

    return v;
}

void
#if defined(__STDC__) || defined(__cplusplus)
SessionKey_free(struct type_Cred_SessionKey *v)
#else
SessionKey_free(v)
struct type_Cred_SessionKey *v;
#endif
{
    if (v != (struct type_Cred_SessionKey *) 0)
        free_Cred_SessionKey(v);
}


PE
#if defined(__STDC__) || defined(__cplusplus)
SessionToken_encode(struct type_Cred_SessionToken *v)
#else
SessionToken_encode(v)
struct type_Cred_SessionToken *v;
#endif
{
    PE pe;

    if (build_Cred_SessionToken(&pe, 1, 0, (char *) 0, v) != OK)
        return (PE) 0;

    return pe;
}

struct type_Cred_SessionToken *
#if defined(__STDC__) || defined(__cplusplus)
SessionToken_decode(PE pe)
#else
SessionToken_decode(pe)
PE pe;
#endif
{
    struct type_Cred_SessionToken *v;

    if (parse_Cred_SessionToken(pe, 1, (int *) 0, (char **) 0, &v) != OK)
        return (struct type_Cred_SessionToken *) 0;

    return v;
}

void
#if defined(__STDC__) || defined(__cplusplus)
SessionToken_free(struct type_Cred_SessionToken *v)
#else
SessionToken_free(v)
struct type_Cred_SessionToken *v;
#endif
{
    if (v != (struct type_Cred_SessionToken *) 0) {
        /*
        ** As we use a dncomp and not type_IF_DistinguishedName,
        ** we have to free this separately.
        */
        if (v->recipient != (struct type_IF_DistinguishedName *) 0) {
            dn_free((DN) v->recipient);
            v->recipient = (struct type_IF_DistinguishedName *) 0;
        }

        /*
        ** As we use a alg_id and not struct type_AF_AlgorithmIdentifier 
        ** we have to free this separately.
        */
        if (v->algorithm != (struct type_AF_AlgorithmIdentifier *) 0) {
            register
            struct alg_id *a = (struct alg_id *) v->algorithm;

            if (a->algorithm != (OID) 0)
                oid_free(a->algorithm);
            if (a->asn != (PE) 0)
                pe_free(a->asn);
            free(a);

            v->algorithm = (struct type_AF_AlgorithmIdentifier *) 0;
        }

        free_Cred_SessionToken(v);
    }
}


PE
#if defined(__STDC__) || defined(__cplusplus)
SessionCredentials_encode(struct type_Cred_SessionCredentials *v)
#else
SessionCredentials_encode(v)
struct type_Cred_SessionCredentials *v;
#endif
{
    PE pe;

    if (build_Cred_SessionCredentials(&pe, 1, 0, (char *) 0, v) != OK)
        return (PE) 0;

    return pe;
}

struct type_Cred_SessionCredentials *
#if defined(__STDC__) || defined(__cplusplus)
SessionCredentials_decode(PE pe)
#else
SessionCredentials_decode(pe)
PE pe;
#endif
{
    struct type_Cred_SessionCredentials *v;

    if (parse_Cred_SessionCredentials(pe, 1, (int *) 0, (char **) 0, &v) != OK)
        return (struct type_Cred_SessionCredentials *) 0;

    return v;
}

void
#if defined(__STDC__) || defined(__cplusplus)
SessionCredentials_free(struct type_Cred_SessionCredentials *v)
#else
SessionCredentials_free(v)
struct type_Cred_SessionCredentials *v;
#endif
{
    if (v != (struct type_Cred_SessionCredentials *) 0) {
        /* type_Cred_SessionToken needs uses DN so ... */
        SessionToken_free(v->token);
        v->token = (struct type_Cred_SessionToken *) 0;
        free_Cred_SessionCredentials(v);
    }
}
