/*
 * Copyright (c) 1994 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/*
** A simple plug-in interface to MSAP for MD5 based integrity checks
*/

/*
** Saleem N. Bhatti
** February 1994
*/

#if !defined(_mintegrity_h_)
#define _mintegrity_h_

#include "msap.h"

#if !defined(_a_)
#if defined(__STDC__) || defined(__cplusplus)
#define _a_(_args_) _args_
#else
#define _a_(_args_) ()
#endif
#endif

#if defined(__cplusplus)
extern "C" {
#endif

/* MD5 Key management */
void initialiseMd5KeySpace _a_(());
char *getMd5Key _a_((const int fd)); /* returned pointer should not be freed */
int setMd5Key _a_((const int fd, const char *key)); /* makes a copy of key */
void clearMd5Key _a_((const int fd));
char *makeMd5Key _a_(()); /* return should not be free()d */

/* Invoke ID management */
void initialiseIdSpace _a_(());
int getResponderIdSeed _a_((const int fd));
int getInitiatorIdSeed _a_((const int fd));
int makeResponderId _a_((const int fd, const int prev));
int makeInitiatorId _a_((const int fd, const int prev));
void clearId _a_((const int fd));

void saveResponderIdState _a_((const int fd, char **state));
void restoreResponderIdState _a_((const int fd, char *state));
void saveInitiatorIdState _a_((const int fd, char **state));
void restoreInitiatorIdState _a_((const int fd, char *state));

/* MD5 interface */
typedef struct MD5Value_s { /* 128 bits */
    union {
        unsigned char bytes[16];
        unsigned long words[4];  /* 32 bit long */
    }  union_MD5Value;

#define md5_bytes union_MD5Value.bytes
#define md5_words union_MD5Value.words
} MD5Value;

int makeMd5Value _a_((const int fd, PE pdu, MD5Value *check));

/* Integrity checks */
#define ENCODE_ID(_i, _c) \
    ((int) (((unsigned long) (_i)) ^ ((unsigned long) (_c))))
#define DECODE_ID(_iXORc, _c) \
    ((int) ~(((unsigned long) (_iXORc)) ^ ~((unsigned long) (_c))))

#if defined(__cplusplus)
}
#endif

#endif /* _m_integrity_h_ */
