/* Copyright (c) 1994 Institute of Computer Science, Foundation of      */
/* Research and Technology - Hellas (ICS-FORTH)                         */
/* All rights reserved.                                                 */

/* Redistribution and use in source and binary forms are permitted      */
/* provided that the above copyright notice and this paragraph are      */
/* duplicated in all such forms and that any documentation, advertising */
/* materials, and other materials related to such distribution and use  */
/* acknowledge that the software was developed by the Institute of      */
/* Computer Science, Foundation of Research and Technology - Hellas.    */
/* The name of the Institute or the Foundation may not be used to       */
/* endorse or promote products derived from this software without       */
/* specific prior written permission.                                   */
/* THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR       */
/* IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED       */
/* WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.  */

/* smk.c - Routines used for the SMK attribute types */

/* Costas Stathopoulos Feb 1995 */

#ifndef ICR1
#include "isode/tailor.h"
#else
#include "isode/compat/tailor.h"
#endif

#include "util.h"
#include "attr.h"
#include "SMK-types.h"

extern LLog * log_dsap;


/*********************************************************************/
/***************** CMIS Functional Units Syntax **********************/
/*********************************************************************/

static PE cmis_fu_enc (p)
CMIS_FU_type     * p;
{
PE	ret_pe;

	if (build_SMK_CMISFunctionalUnits (&ret_pe,1,0,NULLCP,p) == NOTOK ) {
		ret_pe = NULLPE;
		LLOG (log_dsap,LLOG_EXCEPTIONS, ("Failed to encode CMIS_FUs"));
	}
        return (ret_pe);
}


static CMIS_FU_type * cmis_fu_dec (pe)
PE	pe;
{
CMIS_FU_type	* p;

        if (parse_SMK_CMISFunctionalUnits (pe,1,NULLIP,NULLVP,&p) == NOTOK) {
		free(p);
		return((CMIS_FU_type *)NULL);
	}
        return (p);
}


static CMIS_FU_type * cmis_fu_parse (s)
char	* s;
{
        return (strb2bitstr(int2strb(atoi(s), 5), 5, PE_CLASS_UNIV, PE_PRIM_BITS));
}


static cmis_fu_print (ps, p, format)
PS	ps;
CMIS_FU_type     * p;
int	format;
{
char	* pbuf = (char *) malloc (128);
int	len;

	if (format == READOUT) {
#define BITS    "\020\01multipleObjectSelection\02filter\03multipleReply\04extendedService\05confirmedCancelGet"
		ps_printf(ps, "%s", bit2str (p, BITS));
#undef BITS
	} else {
		pbuf = bitstr2strb(p, &len);
		ps_printf(ps, "%i", strb2int(pbuf, len));
	}
	free (pbuf);
}


CMIS_FU_type * cmis_fu_cpy (p)
CMIS_FU_type    * p;
{
	return((CMIS_FU_type *)pe_cpy((PE)p));
}


int cmis_fu_cmp (p, q)
CMIS_FU_type     * p, * q;
{
	return (pe_cmp ((PE)p, (PE)q));
}


CMIS_FU_syntax ()
{
        (void) add_attribute_syntax ("CMISFunctionalUnits",
                (IFP) cmis_fu_enc,      (IFP) cmis_fu_dec,
                (IFP) cmis_fu_parse,    (IFP) cmis_fu_print,
                (IFP) cmis_fu_cpy,      cmis_fu_cmp,
                pe_free,                NULLCP,
                NULLIFP,                FALSE);
}


/*********************************************************************/
/********************** CMIP Version Syntax **************************/
/*********************************************************************/

static PE cmip_version_enc (p)
CMIP_Version_type     * p;
{
PE      ret_pe;

        if (build_SMK_CMIPVersion (&ret_pe,1,0,NULLCP,p) == NOTOK ) {
                ret_pe = NULLPE;
                LLOG (log_dsap,LLOG_EXCEPTIONS, ("Failed to encode CMIP_vers"));
        }
        return (ret_pe);
}


static CMIP_Version_type * cmip_version_dec (pe)
PE      pe;
{
CMIP_Version_type    * p;

        if (parse_SMK_CMIPVersion (pe,1,NULLIP,NULLVP,&p) == NOTOK) {
                free(p);
                return((CMIP_Version_type *)NULL);
        }
        return (p);
}


static CMIP_Version_type * cmip_version_parse (s)
char    * s;
{
        return (strb2bitstr(int2strb(atoi(s), 2), 2, PE_CLASS_UNIV, PE_PRIM_BITS));
}


static cmip_version_print (ps, p, format)
PS      ps;
CMIP_Version_type     * p;
int     format;
{
char    * pbuf = (char *) malloc (128);
int     len;

        if (format == READOUT) {
#define BITS    "\020\01version1\02version2"
                ps_printf(ps, "%s", bit2str (p, BITS));
#undef BITS
        } else {
                pbuf = bitstr2strb(p, &len);
                ps_printf(ps, "%i", strb2int(pbuf, len));
        }
        free (pbuf);
}


CMIP_Version_type * cmip_version_cpy (p)
CMIP_Version_type    * p;
{
	return ((CMIP_Version_type *) pe_cpy((PE)p));
}


int cmip_version_cmp (p, q)
CMIP_Version_type     * p, * q;
{
	return (pe_cmp((PE)p, (PE)q));
}


CMIP_version_syntax ()
{
        (void) add_attribute_syntax ("CMIPVersion",
                (IFP) cmip_version_enc,      (IFP) cmip_version_dec,
                (IFP) cmip_version_parse,    (IFP) cmip_version_print,
                (IFP) cmip_version_cpy,      cmip_version_cmp,
                pe_free,                     NULLCP,
                NULLIFP,                     FALSE);
}


/*********************************************************************/
/****************** Operational Status Syntax ************************/
/*********************************************************************/

PE oper_status_enc (status)
OperStatusVal	* status;
{
	PE pe;

	if (build_SMK_OperationalStatus(&pe,1,0,NULLCP,status) == NOTOK)
	    return NULLPE;
	return pe;
}


OperStatusVal	* oper_status_dec (pe)
PE	pe;
{
	OperStatusVal	* status;

	if (parse_SMK_OperationalStatus(pe,1,NULLIP,NULLVP,&status) == NOTOK)
	    return NULL;
	return status;
}


extern int * enumint_parse(); /* defined in SmiSntx.c */
extern enumint_print(); /* defined in SmiSntx.c */
char * _OperationalStatus [] = { "enabled", "disabled" };
int _OperationalStatus_size = sizeof _OperationalStatus / 
			      sizeof _OperationalStatus[0];

OperStatusVal	* oper_status_parse (s)
char	* s;
{
	return (OperStatusVal *)
		enumint_parse(s,_OperationalStatus,_OperationalStatus_size);
}


int oper_status_print (ps, status, format)
PS 		ps;
OperStatusVal	* status;
int		format;
{
	enumint_print(ps, status,_OperationalStatus,_OperationalStatus_size);
	return OK;
}


int oper_status_cmp (os1, os2)
OperStatusVal	* os1, * os2;
{
    return ((os1->parm == os2->parm)? 0 : 2);
}


OperStatusVal	* oper_status_cpy (status)
OperStatusVal	* status;
{
	OperStatusVal * temp;

	if ( (temp = (OperStatusVal *) malloc (sizeof (OperStatusVal))) == NULL)
	    return NULL;

	*temp = *status;
	return temp;
}


int oper_status_free (status)
OperStatusVal   * status;
{
	free_SMK_OperationalStatus(status);
	return OK;
}


Operational_status_syntax ()
{	(void) add_attribute_syntax ("operationalStatus",
		(IFP) oper_status_enc,		(IFP) oper_status_dec,
		(IFP) oper_status_parse,	(IFP) oper_status_print,
		(IFP) oper_status_cpy,		oper_status_cmp,
		oper_status_free,		NULLCP,
		NULLIFP,			FALSE);
}


/*********************************************************************/
/********************** Management Role Syntax ***********************/
/*********************************************************************/

PE MRole_enc (role)
MRoleVal   * role;
{
        PE pe;

        if (build_SMK_ManagementRole(&pe,1,0,NULLCP,role) == NOTOK)
            return NULLPE;
        return pe;
}


MRoleVal   * MRole_dec (pe)
PE      pe;
{
        MRoleVal   * role;

        if (parse_SMK_ManagementRole(pe,1,NULLIP,NULLVP,&role) == NOTOK)
            return NULL;
        return role;
}


char* _ManagementRole[] = { "agent", "manager" };
int _ManagementRole_size = sizeof _ManagementRole /
                              sizeof _ManagementRole[0];

MRoleVal   * MRole_parse (s)
char    * s;
{
        return (MRoleVal *)
                enumint_parse(s,_ManagementRole,_ManagementRole_size);
}


int MRole_print (ps, role, format)
PS		ps;
MRoleVal	* role;
int		format;
{
	enumint_print(ps,role,_ManagementRole,_ManagementRole_size);
	return OK;
}


int MRole_cmp (r1, r2)
MRoleVal   * r1, * r2;
{
    return ((r1->parm == r2->parm)? 0 : 2);
}


MRoleVal   * MRole_cpy (role)
MRoleVal   * role;
{
        MRoleVal * temp;

        if ( (temp = (MRoleVal *) malloc (sizeof (MRoleVal))) == NULL)
            return NULL;

        *temp = *role;
        return temp;
}


int MRole_free (role)
MRoleVal   * role;
{
        free_SMK_ManagementRole(role);
        return OK;
}


Management_role_syntax ()
{	(void) add_attribute_syntax ("managementRole",
		(IFP) MRole_enc,	(IFP) MRole_dec,
		(IFP) MRole_parse,	(IFP) MRole_print,
		(IFP) MRole_cpy,	(IFP) MRole_cmp,
		MRole_free,		NULLCP,
		NULLIFP,		FALSE);
}


/*********************************************************************/
/********************** TMN Block Syntax *****************************/
/*********************************************************************/

PE TMNBlock_enc (block)
TMNBlockVal   * block;
{
        PE pe;

        if (build_SMK_TMNBlock(&pe,1,0,NULLCP,block) == NOTOK)
            return NULLPE;
        return pe;
}


TMNBlockVal   * TMNBlock_dec (pe)
PE      pe;
{
        TMNBlockVal   * block;

        if (parse_SMK_TMNBlock(pe,1,NULLIP,NULLVP,&block) == NOTOK)
            return NULL;
        return block;
}

char* _TMNBlock[] = {"NE", "QA", "MD", "SL-OS", "NL-OS", "NE-OS", "WS"};
int _TMNBlock_size = sizeof _TMNBlock /
                              sizeof _TMNBlock[0];

TMNBlockVal   * TMNBlock_parse (s)
char    * s;
{
        return (TMNBlockVal *)
                enumint_parse(s,_TMNBlock,_TMNBlock_size);
}


int TMNBlock_print (ps, block, format)
PS      ps;
TMNBlockVal    * block;
int     format;
{
    enumint_print(ps,block,_TMNBlock,_TMNBlock_size);
    return OK;
}


int TMNBlock_cmp (b1, b2)
TMNBlockVal   * b1, * b2;
{
    return ((b1->parm == b2->parm)? 0 : 2);
}


TMNBlockVal   * TMNBlock_cpy (block)
TMNBlockVal   * block;
{
        TMNBlockVal * temp;

        if ( (temp = (TMNBlockVal *) malloc (sizeof (TMNBlockVal))) == NULL)
            return NULL;

        *temp = *block;
        return temp;
}


int TMNBlock_free (block)
TMNBlockVal   * block;
{
        free_SMK_TMNBlock(block);
        return OK;
}


TMN_block_syntax()
{   (void) add_attribute_syntax ("TMNBlock",
        (IFP) TMNBlock_enc,    (IFP) TMNBlock_dec,
        (IFP) TMNBlock_parse,  (IFP) TMNBlock_print,
        (IFP) TMNBlock_cpy,    (IFP) TMNBlock_cmp,
        TMNBlock_free,  NULLCP,
        NULLIFP,        FALSE);
}


/*********************************************************************/
/**************** TMN Management Service Syntax **********************/
/*********************************************************************/

PE tmn_ms_enc (ms)
TMNMSVal   * ms;
{
        PE pe;

        if (build_SMK_TMNMS(&pe,1,0,NULLCP,ms) == NOTOK)
            return NULLPE;
        return pe;
}


TMNMSVal   * tmn_ms_dec (pe)
PE      pe;
{
        TMNMSVal   * ms;

        if (parse_SMK_TMNMS(pe,1,NULLIP,NULLVP,&ms) == NOTOK)
            return NULL;
        return ms;
}


char* _tmn_ms[] = {"Customer Administration", "Management of the TMN",
                   "Traffic Management", "Accounting Management",
                   "Switching Management", "Restoration and Recovery"};
int _tmn_ms_size = sizeof _tmn_ms /
                              sizeof _tmn_ms[0];

TMNMSVal   * tmn_ms_parse (s)
char    * s;
{
        return (TMNMSVal *)
                enumint_parse(s,_tmn_ms,_tmn_ms_size);
}


int tmn_ms_print (ps, ms, format)
PS      ps;
TMNMSVal    * ms;
int     format;
{
    enumint_print(ps,ms,_tmn_ms,_tmn_ms_size);
    return OK;
}


int tmn_ms_cmp (ms1, ms2)
TMNMSVal   * ms1, * ms2;
{
    return ((ms1->parm == ms2->parm)? 0 : 2);
}


TMNMSVal   * tmn_ms_cpy (ms)
TMNMSVal   * ms;
{
        TMNMSVal * temp;

        if ( (temp = (TMNMSVal *) malloc (sizeof (TMNMSVal))) == NULL)
            return NULL;

        *temp = *ms;
        return temp;
}


int tmn_ms_free (ms)
TMNMSVal   * ms;
{
        free_SMK_TMNMS(ms);
        return OK;
}


TMN_ms_syntax()
{   (void) add_attribute_syntax ("TMNMS",
        (IFP) tmn_ms_enc,    (IFP) tmn_ms_dec,
        (IFP) tmn_ms_parse,  (IFP) tmn_ms_print,
        (IFP) tmn_ms_cpy,    (IFP) tmn_ms_cmp,
        tmn_ms_free,    NULLCP,
        NULLIFP,        FALSE);
}


/*********************************************************************/
/****************** Responsible SMAP Syntax **************************/
/*********************************************************************/

PE rmp_enc (rmp)
resp_SMAP   * rmp;
{
        PE pe;

        if (build_SMK_ResponsibleSMAP(&pe,1,0,NULLCP,rmp) == NOTOK)
            return NULLPE;
        return pe;
}


resp_SMAP   * rmp_dec (pe)
PE      pe;
{
        resp_SMAP   * rmp;

        if (parse_SMK_ResponsibleSMAP(pe,1,NULLIP,NULLVP,&rmp) == NOTOK)
            return NULL;
        return rmp;
}


int rmp_print(ps, rmp, format)
PS ps;
resp_SMAP *rmp;
int format;
{
    if (rmp == NULLRMP)
        return;

    if (rmp->mp_name != NULLDN)
       dn_print (ps, rmp->mp_name, format);
    ps_print (ps,"#");
    if (format == EDBOUT)
       ps_printf(ps, "%d", rmp->tmn_ms);
    else
	   enumint_print(ps, rmp->tmn_ms, _tmn_ms, _tmn_ms_size);
}


resp_SMAP *rmp_parse(str)
char *str;
{
register char 	*ptr;
resp_SMAP 		*rmp;
int				*ip;

    /* the format of the string should be "dn # num" or just a dn */
    if ((ptr = index (str,'#')) == 0) {
	  parse_error ("# missing in responsibleSMAP syntax '%s'",str);
	  return(NULLRMP);
	} else {

      rmp = (resp_SMAP *)malloc(sizeof (resp_SMAP));

      /* get the SMAP's name */
      if (*str == '#') {
        parse_error ("dn missing in responsibleSMAP syntax '%s'",str);
        return(NULLRMP);
      } else {
		*ptr = 0;
        if ((rmp->mp_name = str2dn(str)) == NULLDN) {
            free ((char *) rmp);
            return (NULLRMP);
        }
      }

      str = SkipSpace(++ptr);
	  if ((ip = enumint_parse(str, _tmn_ms, _tmn_ms_size)) == NULL) {
		parse_error ("invalid TMNMS '%s' in responsibleSMAP syntax", str);
		return(NULLRMP);
      } else
		rmp->tmn_ms = *ip;
	}

	return rmp;
}


int rmp_cmp (a, b)
resp_SMAP   * a;
resp_SMAP   * b;
{

    if (a == NULLRMP)
        return ( b == NULLRMP ? 0 : -1 );

    if (b == NULLRMP)
        return (1);

    return ((dn_cmp(a->mp_name, b->mp_name) && 
            (a->tmn_ms == b->tmn_ms))? 0 : 2);
}


resp_SMAP   * rmp_cpy (rmp)
resp_SMAP   * rmp;
{
        resp_SMAP * temp;

        if ( (temp = (resp_SMAP *) malloc (sizeof (resp_SMAP))) == NULL)
            return NULLRMP;

        temp->mp_name = dn_cpy(rmp->mp_name);
        temp->tmn_ms  = rmp->tmn_ms;
        return temp;
}


int rmp_free(rmp)
resp_SMAP   * rmp;
{
        if (rmp == NULLRMP)
          return;

        dn_free(rmp->mp_name);
        free ((char *) rmp);
        return OK;
}


Responsible_smap_syntax()
{   (void) add_attribute_syntax ("ResponsibleSMAP",
        (IFP) rmp_enc,    (IFP) rmp_dec,
        (IFP) rmp_parse,  (IFP) rmp_print,
        (IFP) rmp_cpy,    (IFP) rmp_cmp,
        rmp_free,       NULLCP,
        NULLIFP,        TRUE);
}


void smk_syntaxes ()
{
	CMIS_FU_syntax();
	CMIP_version_syntax();
	Operational_status_syntax();
	Management_role_syntax();
	TMN_block_syntax();
	TMN_ms_syntax();
	Responsible_smap_syntax();
}
