//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// GenericList.h - generic linked list

// By George Pavlou, February 1991


#ifndef _GEN_LIST_
#define _GEN_LIST_


#ifndef _BOOLEAN_
#include "Boolean.h"
#endif

#if !defined(OK) || !defined(NOTOK)
#define OK	0
#define NOTOK	-1
#endif

#if !defined(NULLVD)
#define	NULLVD	((void*) 0)
#endif


class Link
{
friend class List;
friend class ListIterator;

    Link* next;
    void* item;

    Link (void*, Link*);
};
#define NULLINK	((Link*) 0)


class List
{
friend class ListIterator;

    Link* l;				// l -> next is head of list
    int   count;

protected:
    Bool  del;

    virtual int  compare (void*, void*);

public:
    virtual void deleteElem (void*);

    void   append (void*);		// add at tail
    void   prepend (void*);		// add at head
    void*  get ();			// remove and return head
    void*  getLast ();			// remove and return tail
    int    insert (void*);		// insert keeping sorted
    void*  remove (void*);		// remove and return element
    void*  first ();			// return head without removing it
    void*  last ();			// return tail without removing it
    void*  find (void*);		// return element without removing it
    void   clear ();			// delete all links
    void   setDeleteMode (Bool);	// change the delete mode
    int    getCount();                  // get count

    List ();
    List (Bool);
    virtual ~List ();
};
#define NULLIST	((List*) 0)


inline Link::Link (void* i, Link* n)	{ item = i; next = n; }

inline void  List::deleteElem (void* e)	{ if (del) delete e; }
inline int   List::compare (void*, void*) { return NOTOK; }
inline void* List::first ()		{ return l == NULLINK ?
					  NULLVD : l->next->item; }
inline void* List::last ()		{ return l == NULLINK ?
					  NULLVD : l->item; }
inline void  List::setDeleteMode (Bool d) { del = d; }
inline int   List::getCount ()          { return count; }
inline	     List::List ()		{ l = NULLINK; count = 0; del = True; }
inline	     List::List (Bool d)	{ l = NULLINK; count = 0; del = d; }
inline	     List::~List ()		{ clear(); }


#endif
