//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
/******************************************************************/
/*

        File:           GenStr.cc

                String Class methods


        Author:
                James Cowan (J.Cowan@uk.ac.ucl.cs)
*/
/*****************************************************************/
/* String Characters Class */
/*****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "DebugMem.h"
#include "GenString.h"

/*****************************************************************/
/* String Class */
/*****************************************************************/

// Had to do this for GDMO compiler to compile, see also GenString.h

#if defined(__GNUG__)
GString operator +(GString str1, GString str2)
#elif defined(__cplusplus)
GString operator +(GString &str1, GString &str2)
#endif
{
char    *temp = (NEW char[strlen(str1) + strlen(str2) + 1]);
GString result;

        strcpy(temp, str1);
        strcat(temp, str2);

        result = temp;

        DELETE temp;
        return(result);
}

/*****************************************************************/

GString operator +=(GString &str1, GString &str2)
{
        str1 = str1 + str2;
        return(str1);
}

/*****************************************************************/

GString operator +=(GString &str1, char *str_char)
{
GString temp(str_char);

        str1 = str1 + temp;
        return(str1);
}

/*****************************************************************/

GString operator +=(GString &str1, char achar)
{
GString temp(achar);

        str1 = str1 + temp;
        return(str1);
}
/*****************************************************************/

GString &GString::operator =(GString &str)
{

        if (&str == this)
                return (*this);

        if (!--_str_chars->_ref_count)
                DELETE _str_chars;

        _str_chars = str._str_chars;
        _str_chars->_ref_count++;

#ifdef  DEBUG_STRING
        printf("\nGString Assign:--%s-- Ref count --%d--",
                _str_chars->_str,  _str_chars->_ref_count);
#endif

        return (*this);
}

/*****************************************************************/

GString &GString::operator =(char *str_chars)
{

        if (_str_chars->_str == str_chars)
                return (*this);

        if (!--_str_chars->_ref_count)
                DELETE _str_chars;

        initialise(str_chars, strlen(str_chars));

#ifdef  DEBUG_STRING
        printf("\nGString Char Assign:--%s-- Ref count --%d--",
                _str_chars->_str,  _str_chars->_ref_count);
#endif
        return (*this);
}

/*****************************************************************/

GString::operator char *()
{
#ifdef  DEBUG_STRING
        printf("\nGString Char Cast:--%s--", _str_chars->_str);
        fflush(stdout);
#endif
        return(_str_chars->_str);
}

/*****************************************************************/

GString GString::before(char *target)
{
char    *new_str;
char    *str = _str_chars->_str;

        if ((new_str=search(str, target)) != (char *) 0)
        {
        int     lgth = new_str - str;
        GString result(str, lgth);

                return(result);
        }
        else
                return(*this);

} /* before */

/*****************************************************************/

GString GString::before(int type)
{
        if (type == RXwhite)
        {
        char    *str = _str_chars->_str;
        char    *new_str = str;

                while (*new_str && (!isspace(*new_str)))
                        new_str++;

                if (*new_str)
                {
                GString result(str, new_str - str);

                        return(result);
                }
                return(*this);
        }
        else
                fprintf(stderr, "\nUnknown operator to GString::before");

        return (*this);

} /* before */

/*****************************************************************/

GString GString::after(char *target)
{
char    *str;

        if ((str=search(_str_chars->_str, target)) != (char *) 0)
        {
        char    *new_str = str + strlen(target);
        GString result(new_str);

                return(result);
        }
        else
                return(*this);

} /* after */

/*****************************************************************/

GString GString::after(int type)
{
        if (type == RXwhite)
        {
        char    *new_str = _str_chars->_str;

                while (isspace(*new_str))
                        new_str++;

                GString result(new_str);	// GP - 060318
                return(result);
        }
        else
                fprintf(stderr, "\nUnknown operator to GString::after");

        return (*this);

} /* after */

/*****************************************************************/

char    *GString::search(char *str, char *target)
{
char    *find;
int     lgth = strlen(target);

        for (find=str; *find; find++)
        {
                if (*find == *target)
                        if (strncmp(find, target, lgth) == 0)
                        {
#ifdef  DEBUG_STRING
        printf("\nGString::search: Found --%s-- in --%s--", target, str);
#endif
                                return(find);
                        }
        }

#ifdef  DEBUG_STRING
        printf("\nGString::search: NOT Found --%s-- in --%s--", target, str);
#endif
        return(0);

} /* search */

/*****************************************************************/
/* End of module */
/*****************************************************************/


