/*
 * Copyright (c) 1993 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/

/* Jonathan Ladipo, January 1994 */

#include "xmp.h"
#include "mp_utilities.h"
static int do_wait(int, int, int, MIDAS_RESULT*);
extern void dn_print();
extern int debug_test(char *);
extern int get_association_descriptor(MIDAS_SESSION *, MIDAS_CONTEXT *);

/*
 * mp_get_rsp_M - reply to a requested Get or Get-Next operation.
 *
 */

MIDAS_STATUS_P mp_get_rsp_M(session, context, response, invoke_id)
        MIDAS_SESSION *         session;
        MIDAS_CONTEXT *         context;
        MIDAS_RESULT *          response;       /*response*/
        OM_sint32 *             invoke_id;      /*invoke_id*/

{
	MID  obj_class;
	MN   obj_inst;
	char* cur_time;
	int  nattrs;
	CMISErrors error;
	CMISGetAttr* attrs;
        CMISGetRes reply;
        MSAPIndication get_reply, mi;
        int msapres, secs, linked, assoc_descriptor, response_invoke_id;
        static MIDAS_STATUS status; /*Will be returned if an error occurs*/
	static MSAPIndication indication;

	status.MIDAS_parameter = (MSAPIndication *)NULL;

        debug_test("In mp_get_rsp_M @1");	/*Debugging aid*/

        /*Do some sanity checks on the parameters*/
        if (check_session(session, &status) != OK) {
            fprintf(stderr, "mp_get_rsp_M: bad session\n");
            return(&status);
        }

        if (check_context(context, &status) != OK) {
            fprintf(stderr, "mp_get_rsp_M: bad context");
            return(&status);
        }

        /*Get the association descriptor*/
        assoc_descriptor = get_association_descriptor(session, context);

        if(assoc_descriptor == NOTOK){
            status.MIDAS_problem.midas_problem_type = midas_library_error;
            status.MIDAS_problem.midas_problem_value.error_val
                                                 = MP_E_INVALID_CONNECTION_ID;

            fprintf(stderr, "mp_get_rsp_M: invalid association descriptor\n");
            return (&status);
        }

	/*Process the response*/

        debug_test("In mp_get_rsp_M @3");	/*Debugging aid*/
	get_reply =  response->MIDAS_result_info;
        error = get_reply.mi_errtype;

	switch (get_reply.mi_type) {
   	case MI_LINKED_REPLY:
	    debug_test("In mp_get_rsp_M @3C");	/*Debugging aid*/

	    /*Extract attributes from CMISLinkedRepArg*/
            reply = get_reply.mi_linkedreplyinv.args.lr_getres;

  	    response_invoke_id = get_reply.mi_linkedreplyinv.id;
	    obj_class = &reply.gr_class;
	    obj_inst  = &reply.gr_inst;
	    cur_time  = reply.gr_time;
	    nattrs    = reply.gr_nattrs;
	    attrs     = reply.gr_attrs;
	    linked    = 1;
	    break;

	case MI_GET_RES:
	    debug_test("In mp_get_rsp_M @3B");	/*Debugging aid*/

	    /*Extract attributes from CMISGetRes*/
	    reply = get_reply.mi_getres.result;
	    response_invoke_id = get_reply.mi_getres.id;

            obj_class = &reply.gr_class;
            obj_inst  = &reply.gr_inst;
            cur_time  = reply.gr_time;
            nattrs    = reply.gr_nattrs;
            attrs     = reply.gr_attrs;
	    linked    = 0;
	    break;

	default:
            /*This should not happen but if we do receive a wrong*/
            /*mi_type we bring the response processing to and end*/

            fprintf(stderr, "mp_get_rsp_M: Invalid mi_type\n");
	    return (MIDAS_SUCCESS);
	}


        /*Now make the get response*/
        msapres = M_GetRes(
                assoc_descriptor,
                response_invoke_id,
		linked,
                obj_class,
                obj_inst,
                cur_time,
                nattrs,
		attrs,
                error,
                NULLMERRORINFO,
                &indication 
                );	/*must point to valid memory space*/

        if(msapres == NOTOK){   /*Get reponse call has failed*/
            fprintf(stderr, "Error: Unable to send get response\n");
	    status.MIDAS_parameter = &indication;
	    return (&status);
        }

        debug_test("In mp_get_rsp_M @4");	/*Debugging aid*/
        return (MIDAS_SUCCESS);
}

