#if defined(SECURITY)

#if defined(__GNUG__)
#include <std.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>

#include "mp_integrity.h"
#include "mintegrity.h"

static
IRStatus S_status[FD_SETSIZE];

void
#if defined(__STDC__) || defined(__cplusplus)
initialiseIRStatus()
#else
initialiseIRStatus()
#endif
{
    register
    int i;

    for(i = 0; i < FD_SETSIZE; ++i)
        S_status[i] = IRStatus_none;
}

IRStatus
#if defined(__STDC__) || defined(__cplusplus)
getIRStatus(const int fd)
#else
getIRStatus(fd)
int fd;
#endif
{
    if (fd < 0 || fd >= FD_SETSIZE) {
#if defined(DEBUG)
        fprintf(stderr, "getIRStatus(): bad fd value, fd=%d\n", fd);
#endif
        return IRStatus_none;
    }

    return S_status[fd];
}

void
#if defined(__STDC__) || defined(__cplusplus)
setIRStatus(const int fd, const IRStatus s)
#else
setIRStatus(fd, i)
int fd;
IRStatus i;
#endif
{
    if (fd < 0 || fd >= FD_SETSIZE) {
#if defined(DEBUG)
        fprintf(stderr, "setIRStatus(): bad fd value, fd=%d\n", fd);
#endif
        return;
    }

    S_status[fd] = i;
}

void
#if defined(__STDC__) || defined(__cplusplus)
clearIRStatus(const int fd)
#else
clearIRStatus(fd)
int fd;
#endif
{
    if (fd < 0 || fd >= FD_SETSIZE) {
#if defined(DEBUG)
        fprintf(stderr, "clearIRStatus(): bad fd value, fd=%d\n", fd);
#endif
        return;
    }

    S_status[fd] = IRStatus_none;
}

int
#if defined(__STDC__) || defined(__cplusplus)
makeId(const int fd)
#else
makeId(fd)
int fd;
#endif
{
    register
    int i;

    if (fd < 0 || fd >= FD_SETSIZE) {
#if defined(DEBUG)
        fprintf(stderr, "makeId(): bad fd value, fd=%d\n", fd);
#endif
        return;
    }

    switch ((int) S_status[fd]) {
    case (int) IRStatus_initiator: i = makeInitiatorId(fd, 1); break;
    case (int) IRStatus_responder: i = makeResponderId(fd, 1); break;
    default: i = 0; break;
    }

    return i;
}


int
#if defined(__STDC__) || defined(__cplusplus)
makePeerId(const int fd)
#else
makePeerId(fd)
int fd;
#endif
{
    register
    int i;

    if (fd < 0 || fd >= FD_SETSIZE) {
#if defined(DEBUG)
        fprintf(stderr, "makePeerId(): bad fd value, fd=%d\n", fd);
#endif
        return;
    }

    switch ((int) S_status[fd]) {
    case (int) IRStatus_initiator: i = makeResponderId(fd, 1); break;
    case (int) IRStatus_responder: i = makeInitiatorId(fd, 1); break;
    default: i = 0; break;
    }

    return i;
}

void
#if defined(__STDC__) || defined(__cplusplus)
saveState(const int fd, char **state)
#else
saveState(fd, state)
int fd;
char **state;
#endif
{
    *state = (char *) 0;

    if (fd < 0 || fd >= FD_SETSIZE) {
#if defined(DEBUG)
        fprintf(stderr, "saveState(): bad fd value, fd=%d\n", fd);
#endif
        return;
    }

    switch ((int) S_status[fd]) {
    case (int) IRStatus_initiator: saveInitiatorIdState(fd, state); break;
    case (int) IRStatus_responder: saveResponderIdState(fd, state); break;
    }
}

void
#if defined(__STDC__) || defined(__cplusplus)
savePeerState(const int fd, char **state)
#else
savePeerState(fd, state)
int fd;
char **state;
#endif
{
    *state = (char *) 0;

    if (fd < 0 || fd >= FD_SETSIZE) {
#if defined(DEBUG)
        fprintf(stderr, "savePeerState(): bad fd value, fd=%d\n", fd);
#endif
        return;
    }

    switch ((int) S_status[fd]) {
    case (int) IRStatus_initiator: saveResponderIdState(fd, state); break;
    case (int) IRStatus_responder: saveInitiatorIdState(fd, state); break;
    }
}

void
#if defined(__STDC__) || defined(__cplusplus)
restoreState(const int fd, char *state)
#else
restoreState(fd, state)
int fd;
char *state;
#endif
{

    if (fd < 0 || fd >= FD_SETSIZE) {
#if defined(DEBUG)
        fprintf(stderr, "retoreState(): bad fd value, fd=%d\n", fd);
#endif
        return;
    }

    switch ((int) S_status[fd]) {
    case (int) IRStatus_initiator: restoreInitiatorIdState(fd, state); break;
    case (int) IRStatus_responder: restoreResponderIdState(fd, state); break;
    }
}

void
#if defined(__STDC__) || defined(__cplusplus)
restorePeerState(const int fd, char *state)
#else
restorePeerState(fd, state)
int fd;
char *state;
#endif
{

    if (fd < 0 || fd >= FD_SETSIZE) {
#if defined(DEBUG)
        fprintf(stderr, "retorePeerState(): bad fd value, fd=%d\n", fd);
#endif
        return;
    }

    switch ((int) S_status[fd]) {
    case (int) IRStatus_initiator: restoreResponderIdState(fd, state); break;
    case (int) IRStatus_responder: restoreInitiatorIdState(fd, state); break;
    }
}

#else

int NoSecurityOptionsForXMP_provide = 1;

#endif /* SECURITY */
