/*
 * Copyright (c) 1993 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/

#include "xmp.h"
#include "midas_xmp.h"
#include "msap.h"
#include "mp_utilities.h"

#if defined(SECURITY)
#include "mintegrity.h"
#include "minvoke.h"
#include "mp_integrity.h"
#endif

#if !defined(_a_)
#if defined(__STDC__) || defined(__cplusplus)
#define _a_(_args_) _args_
#else
#define _a_(_args_) ()
#endif
#endif

extern int session_initialised;
extern int debug_test _a_((char *));

/***********************************************************************
 *
 * UTILITY ROUTINES
 *
 ************************************************************************/

/*
 * check_session - sanity check on some session values
 * Not very comprehensive at present
 * Do we really need to do this each time?
 */

int
check_session(session, statusp)
    MIDAS_SESSION *	session;
    MIDAS_STATUS_P	statusp;{

    debug_test("In check_session @ 1");

    /*If we are in UCL mode of operation*/
    if(session->file_descriptor == MP_NO_VALID_FILE_DESCRIPTOR)
        if (session->responder_address != NULLPA ||
	    session->responder_title != NULLAEI ||
	    !session_initialised){

            debug_test("In check_session @ 2"); /*Debugging aid*/
	    statusp->MIDAS_problem.midas_problem_type = midas_library_error;
	    statusp->MIDAS_problem.midas_problem_value.error_val = MP_E_BAD_SESSION;
	    statusp->MIDAS_parameter = (MSAPIndication *)NULL;
	    return(NOTOK);
        }
     /*else we are in GMD mode of operation*/
     else if(session->file_descriptor != MP_NO_VALID_FILE_DESCRIPTOR)
        if (session->responder_address == NULLPA ||
                        session->responder_title == NULLAEI){

            debug_test("In check_session @ 3"); /*Debugging aid*/
            statusp->MIDAS_problem.midas_problem_type = midas_library_error;
            statusp->MIDAS_problem.midas_problem_value.error_val = MP_E_BAD_SESSION;
            statusp->MIDAS_parameter = (MSAPIndication *)NULL;
            return(NOTOK);
        }

     debug_test("Leaving check_session @ 4"); /*Debugging aid*/
     return(OK);
}



/*
 * check_context - sanity check on some context values
 * Not very comprehensive at present
 *
 */

int
check_context(context, statusp)
    MIDAS_CONTEXT *	context;
    MIDAS_STATUS_P	statusp;{

    debug_test("In check_context @ 1");
    if(context->extensions != (caddr_t*)NULL 		||
       context->responder_address != NULLPA  		||
       context->responder_title != NULLAEI   		||
       (context->mode != MP_T_CONFIRMED 	&& 
       context->mode != MP_T_NON_CONFIRMED)){

	statusp->MIDAS_problem.midas_problem_type = midas_library_error;
	statusp->MIDAS_problem.midas_problem_value.error_val = MP_E_BAD_CONTEXT;
	statusp->MIDAS_parameter = (MSAPIndication *)NULL;
	return(NOTOK);
    }
    
    debug_test("Leaving check_context @2"); /*Debugging aid*/
    return(OK);
}


/*
 * error_status - deal with returned MSAPIndication in the
 * error case
 *
 */

MIDAS_STATUS *
error_status(statp, mip, problem_type, error_val)
    MIDAS_STATUS* statp;
    MSAPIndication* mip;
    enum problem_type problem_type;
    int error_val;{           /*should be union problem_value*/

    static MSAPIndication ind;

    statp->MIDAS_problem.midas_problem_type = problem_type;
    statp->MIDAS_problem.midas_problem_value.error_val = error_val;
    if (mip) {

	ind = *mip;	/*struct copy*/
        statp->MIDAS_parameter = &ind;       

    }
    else
        statp->MIDAS_parameter = (MSAPIndication *)NULL;

    return(statp);
}

/*
 * mrp_Q_free - free a list of MIDAS_RESULT structures
 *
 */

void
mrp_Q_free(mrp)
    MIDAS_RESULT* mrp;
{
    register MIDAS_RESULT* pp;

    if (mrp == (MIDAS_RESULT *)NULL) 
	return;

    do{
        pp=mrp->MIDAS_result_next;
        free(mrp);
    }while (mrp = pp);
}

/*
 * get_invoke_id - generate an invoke id
 *
 */

get_invoke_id(){
    static id = 0;

    return(++id);
}

/*Determines if an MIDentifier is empty*/

static int is_empty(obj_class)
MIDentifier obj_class;
{
    int result_1, result_2;
    MIDentifier empty_class;

    debug_test("In is_empty() @1");     /*Debugging aid*/
    bzero((char*) &empty_class, sizeof(empty_class));

    result_1 = (obj_class.mid_type == empty_class.mid_type);
    result_2 = (obj_class.mid_local == empty_class.mid_local);

    /*TEMPORARY FIX*/
    fprintf(stderr, "is_empty(): result_1 = %d, result_2 = %d\n",
                        result_1, result_2);

    if(result_1 && result_2)
        return (1);
    else
        return (0);

}

/*
 * Wait for a incoming PDU
 *
 * mip must point to an instance of the MSAPIndication structure
 * statp must point to an instance of the MIDAS_STATUS structure
 *
 * Return values are:
 *
 * DONE                 Operation completed successfully
 * OK                   Operation successful but more replies expected
 * NOTOK                Error prevents further processing
 * INTEGRITY_FAILURE    Integrity check failed
 *
 * The statusp structure will be filled in iff NOTOK is returned.
 */

int
mp_wait_pdu(assoc_descriptor, mip, statusp, secs)
    int		    assoc_descriptor, secs;
    MSAPIndication  *mip;
    MIDAS_STATUS    *statusp;{

    PE pdu;

    if(M_WaitReqAux(assoc_descriptor, secs, mip, &pdu) != OK){
	fprintf(stderr, "mp_wait_pdu: M_Wait failed\n");
	fprintf(stderr, "mp_wait_pdu: %s\n", mip->mi_preject.mpr_data);
	statusp->MIDAS_problem.midas_problem_type = midas_service_error;
	statusp->MIDAS_problem.midas_problem_value.error_val
				  = MP_E_PROCESSING_FAILURE;
	statusp->MIDAS_parameter = mip;
	return (NOTOK);	
    }


#if defined(SECURITY)
    /*
    ** Perform the integrity check for the PDU
    */
    {
    MD5Value md5;
    register
    int check, id = 0, flag = 0;
    char *state = (char *) 0;

    if (getIRStatus(assoc_descriptor) == IRStatus_none) {
        /* shouldn not happen */
        fprintf(stderr, "mp_wait_pdu: IRStatus_none, fd=%d", assoc_descriptor);
        error_status(statusp, (MSAPIndication *) 0,
                            midas_library_error, 0);
	return(NOTOK);
    }

    /*
    ** If PDU is NULL then construct it from the integer value of either the
    ** generated PDU id or the id of the request.
    **
    ** An empty PDU could occur in the case of an EmptyResult or in the
    ** case of an Error. For the EmptyResult the id value will always be
    ** the same as the request id. For Error cases, if the Error is part of
    ** a set of linked replies, then the id will be part of the expected
    ** generated sequence, but if it is the one and only PDU returned as
    ** part of a request, it will take the id value of the original
    ** request, in the same manner as for EmptyResult.
    */

    /* check all outstanding ids */
    if (pdu == (PE) 0 &&
        (mip->mi_type == MI_EMPTY_RES || mip->mi_type == MI_ERROR)) {

        for(id = checkInvoke(assoc_descriptor, 0, InvokeSearch_id,
                         InvokeSearchType_first); id != 0 && !flag;
            id = checkInvoke(assoc_descriptor, id, InvokeSearch_id,
                             InvokeSearchType_next)) {

            /* id will have the invokeId value but we want the PDU id so ... */
            id = checkInvoke(assoc_descriptor, id, InvokeSearch_invokeId,
                             InvokeSearchType_search); /* aaarrghh! */

            pdu = int2prim(id);

            /* Make the MD5 checksum for the PDU */
            if (makeMd5Value(assoc_descriptor, pdu, &md5) != 0) {

                /* Integrity check routines failure -- should never happen */
                fprintf(stderr, "mp_wait_pdu: MD5 generation failed\n");

                pe_free(pdu);

                /*
                ** Although the integrity check generation failed, pass
                ** back up the indication so that the user can still
                ** process it if he/she really wants to.
                */
                error_status(statusp, mip, midas_library_error,
                                 MP_E_PROCESSING_FAILURE);
		return(NOTOK);
            }

            pe_free(pdu); pdu = (PE) 0;
            if (mip->mi_invoke == ENCODE_ID(id, md5.md5_words[0]))
                flag = 1;
        }

        /* check the sequence */
        if (!flag) {

            /* save state just in case integrity check fails */
            savePeerState(assoc_descriptor, &state);

            id = makePeerId(assoc_descriptor);
            pdu = int2prim(id);

            /* Make the MD5 checksum for the PDU */
            if (makeMd5Value(assoc_descriptor, pdu, &md5) != 0) {

                /* Integrity check routines failure -- should never happen */
                fprintf(stderr, "mp_wait_pdu: MD5 generation failed\n");

                pe_free(pdu);

                /* roll back the id sequnece */
                restorePeerState(assoc_descriptor, &state);

                /*
                ** Although the integrity check generation failed, pass
                ** back up the indication so that the user can still
                ** process it if he/she really wants to.
                */
                error_status(statusp, mip, midas_library_error,
                                 MP_E_PROCESSING_FAILURE);
		return(NOTOK);
            }

            pe_free(pdu); pdu = (PE) 0;
            if (mip->mi_invoke == ENCODE_ID(id, md5.md5_words[0]))
                flag = 1;
            else{
                /* roll back the id sequence */
		fprintf(stderr, "mp_wait_pdu: NULL PDU, check failed 2\n");
                restorePeerState(assoc_descriptor, state);
	    }
        }

        /* If flag is still zero, the integrity check has failed! */
        if (!flag) {
#if defined(DEBUG)
            fprintf(stderr, "mp_wait_pdu: integrity check failed!\n");
#endif
            /*
            ** Although the integrity check failed, pass back up the
            ** indication ** so that the user can still process it if
            ** he/she really wants to.
            */
            error_status(statusp, mip, midas_library_error,
                             -7); /* check this ### */
	    return(NOTOK);
        }
        else /* OK - now remove this id */
            removeInvoke(assoc_descriptor, id, InvokeSearch_id);
    }

    else { /* non NULL PDU */

        if (mip->mi_type == MI_LINKED_REPLY) {
            /* save state just in case integrity check fails */
            savePeerState(assoc_descriptor, &state);

            id = makePeerId(assoc_descriptor);
        }

        /* else must be a single Result */

        /* Make the MD5 checksum for the PDU */
        if (makeMd5Value(assoc_descriptor, pdu, &md5) != 0) {

            /* Integrity check routines failure -- should never happen */
            fprintf(stderr, "mp_wait_pdu: MD5 generation failed\n");

            pe_free(pdu);
            /* roll back the id sequnce */
            restorePeerState(assoc_descriptor, state);

            /*
            ** Although the integrity check generation failed, pass back up
            ** the indication so that the user can still process it if
            ** he/she really wants to.
            */
            error_status(statusp, mip, midas_library_error,
                                 MP_E_PROCESSING_FAILURE);
	    return(NOTOK);
        }

        if (mip->mi_type == MI_LINKED_REPLY) { /* linked reply */
            if (mip->mi_invoke == ENCODE_ID(id, md5.md5_words[0]))
                flag = 1; /* check ok */

            else{
                /* check failed -- roll back the id sequence */
		fprintf(stderr, "mp_wait_pdu: non-NULL PDU, check failed 1\n");
                restorePeerState(assoc_descriptor, state);
	    }
        }

        else { /* single result */
            id = DECODE_ID(mip->mi_invoke, md5.md5_words[0]);
            if (checkInvoke(assoc_descriptor, id, InvokeSearch_id,
                            InvokeSearchType_search) != 0) {
                flag = 1; /* check ok */
                removeInvoke(assoc_descriptor, id, InvokeSearch_id);
            }
	    else
		fprintf(stderr, "mp_wait_pdu: non-NULL PDU, check failed 2\n");
        }
    }

    /* If flag is still zero, the integrity check has failed! */
    if (!flag) {
#if defined(DEBUG)
        fprintf(stderr, "mp_wait_pdu: integrity check failed!\n");
#endif
        /*
        ** Although the integrity check failed, pass back up the indication
        ** so that the user can still process it if he/she really wants to.
        */
        error_status(statusp, mip, midas_library_error,
                         -7); /* check this ### */
	return(NOTOK);
    }
    }

#endif /* SECURITY */

    if (pdu != (PE) 0)
        pe_free(pdu);

    if (mip->mi_type == MI_LINKED_REPLY)
    /*If a Linked Reply is received then we can expect at least one
      more reply.
     */
        return (OK);
    else
        return (DONE);
}

