/*
 * Copyright (c) 1988 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/

#include <sys/types.h>
#include "xmp.h"
#include "midas_xmp.h"
#include "msap.h"
#include <errno.h>
#include <stdio.h>

/*
 * mp_wait_M - wait for the availability of management message(s) from one or 
 * more bound Sessions.
 *
 * N.B. Since we do the association control outside XMP the session does not
 * really "know" about associations. This call relies on the fact that
 * M_InitialiseReq_u() maintains a list of open fds in MIDAS_read_fdset.
 *
 */

extern fd_set MIDAS_read_fdset;

MIDAS_STATUS *
mp_wait_M(bound_session_list, workspace, timeout)
    MIDAS_WAITING_SESSIONS *bound_session_list;
    MIDAS_WORKSPACE workspace;
    OM_uint32 timeout;{
    fd_set read_fdset, junk;
    register int i, ret, fd;
    int gmdmode = FALSE;
    struct timeval to;
    struct timeval *top = (struct timeval *)NULL; /*Tells select() to return 
						   *only when a file descriptor 						   *is ready for I/O
						   */
    register MIDAS_WAITING_SESSIONS *mwsp = bound_session_list;

    /*Strictly speaking, since GMD mode is based on HP XMP mp_wait
      will never be called. However it seems reasonable that we should
      be able to deal with multiple sessions in the
      bound_session_list.
     */

    if (mwsp->bound_session->file_descriptor
					== MP_NO_VALID_FILE_DESCRIPTOR){
        /*There should be just one session in the list*/

        if (mwsp->next != (MIDAS_WAITING_SESSIONS *)NULL)
            return (MIDAS_INVALID_SESSION);

	/*UCL mode. Assume the relevant FDs are the ones set in
          MIDAS_read_fdset.
         */
        read_fdset = MIDAS_read_fdset;
    }
    else {
	/*GMD mode. Pick out the file descriptors*/
	gmdmode = TRUE;
	FD_ZERO(&read_fdset);
	do {
    	    fd = mwsp->bound_session->file_descriptor;
	    if (fd < 0 || fd >= FD_SETSIZE) 
    		return (MIDAS_INVALID_SESSION);
	    FD_SET(fd, &read_fdset);
	} while (mwsp = mwsp->next);
    }

    FD_ZERO(&junk);

    if (timeout > 0){
	to.tv_sec = timeout/1000;
	to.tv_usec = (timeout%1000)*1000;
	top = &to;
    }

    ret = select(FD_SETSIZE, &read_fdset, &junk, &junk, top);

    if (ret < 0){
	perror("mp_wait_M");
        fprintf(stderr, "mp_wait_M: select error\n");
        return(MIDAS_INSUFFICIENT_RESOURCES);
    }

    if (ret == 0){
        fprintf(stderr, "mp_wait_M: reply timed out\n");

        /*Indicates to the calling function that there are no fd's ready for
          reading. This is needed in the case when timeout is greater than 
	  zero. In this case we clear the "activated" bits, return "success" 
	  and leave mp_receive() to discover the truth (this may be daft but 
	  its what the spec. says!)
	 */
	FD_ZERO(&read_fdset);
    }

    /*Set the "activated" bits as needed*/
    mwsp = bound_session_list; 
    do {
	mwsp->activated = 
		gmdmode ? FD_ISSET(mwsp->bound_session->file_descriptor,
							    &read_fdset):
			  TRUE;
    } while (mwsp->next);

    return(MIDAS_SUCCESS);
}
