//
// Copyright (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// MDelete.cc - enables to delete managed objects in a remote MIB

// By George Pavlou, January 1993


#include <stdlib.h>
#if	defined(__GNUG__) || (defined(__cplusplus) && defined(AIXV3))
extern "C" { int  strncasecmp(const char*, const char*, size_t); }
#endif

#include "msap.h"
#include "msap_u.h" /*XMP version calls*/

#include "Error.h"
#include "Syntaxes.h"
#include <memory.h>

#if defined(SECURITY)
#include "mauth.h"
#include "mintegrity.h"
#endif

#if defined(DEBUG)
#include "msap-pp.h"
FILE *pduDump = (FILE *) 0;
#endif

int     request_id = 0,
        request_invoke = 0,
        invoke = 0;
MSAPIndication mis;

int   parseFilterArg (int, char**, char**, int*);

static int   do_delete (int, OID, MN, CMISScope*, CMISFilter*, CMISSync,
			MSAPIndication*);
static int   do_wait (int, MSAPIndication*);

main (int argc, char** argv)
{
    char* strObjClass = NULLCP;
    char* strObjInst = NULLCP;
    char* strScope = NULLCP;
    char* strSync = NULLCP;
    char* strFilter = NULLCP;
    int   i, j, state = 0;

    OID objClass = NULLOID;
    MN  objInst  = NULLMN;
    CMISScope* scope = NULLMSCOPE;	// default
    CMISSync sync = s_bestEffort;	//  ..
    CMISFilter* filter = NULLMFILTER;

#ifdef SECURITY
    int IFlag = 0;
    char *peer = (char *) 0,
         *dsa = (char *) 0;
#endif

    // parse command line arguments

    if (argc < 3)
	goto usage;

    for (i = 3; i < argc; i++)
	switch (state) {
	case 0:
	    if (*argv[i] == '-' && strlen(argv[i]) == 2)
		switch (*(argv[i]+1)) {
		case 'c':
		    state = 1;
		    break;
		case 'i':
		    state = 2;
		    break;
		case 's':
		    state = 3;
		    break;
		case 'f':
		    state = 4;
		    break;
#ifdef SECURITY
                case 'P':
                        state = 5;
                        break;
                case 'D':
                        state = 6;
                        break;
                case 'I':
                        state = 0;
                        IFlag = 1;
                        break;
#endif
		default:
		    goto usage;
		    }
	    else
		goto usage;
	    break;

	case 1:
	    strObjClass = argv[i];
	    state = 0;
	    break;

	case 2:
	    strObjInst = argv[i];
	    state = 0;
	    break;

	case 3:
	    strScope = argv[i];
	    if (i < argc-1 && *argv[i+1] != '-')
		strSync = argv[++i];
	    state = 0;
	    break;

	case 4:
	    if (parseFilterArg(argc-i, &argv[i], &strFilter, &j) == NOTOK)
		goto usage;
	    i += j-1;
	    state = 0;
	    break;
#ifdef SECURITY
        case 5:
            peer = argv[i];
            state = 0;
            break;
        case 6:
            dsa = argv[i];
            state = 0;
            break;
#endif
	}


    // load syntaxes

    if (initialiseSyntaxes(NULLCP, attrSyntaxes) == NOTOK)
	error(NULLCP, "mdelete: cannot initialise syntaxes");

#ifdef SECURITY
    initialiseOsisec(argc, argv);
    initialiseMd5KeySpace();
    initialiseIdSpace();
#endif

    // convert the string arguments to CMIS parameters

    if (strObjClass && (objClass = name2oid(strObjClass)) == NULLOID)
	error(NULLCP, "invalid object class: %s", strObjClass);

    if (strObjInst && (objInst = ObjectInstance_parse(strObjInst)) == NULLMN)
        error(NULLCP, "invalid object instance %s", strObjInst);

    if (strScope && (scope = str2scope(strScope)) == NULLMSCOPE)
	error(NULLCP, "invalid scope %s", strScope);

    if (strSync && (sync = str2sync(strSync)) == s_invalid)
	error(NULLCP, "invalid sync %s", strSync);
	
    if (strFilter && strFilter[0] != '\0'  &&
		(filter = str2mfilter(strFilter)) == NULLMFILTER)
	error(NULLCP, "invalid filter %s", strFilter);


    // now try to connect to the remote agent
    int msd;

    MInitOption *miop[2]; /* Connect options */

#ifdef SECURITY
    MInitOption mio;
    AuthAssocIntegrityInfo *aaii;
    aaii = AuthAssocIntegrityInfo_make();

    if (peer != (char *) 0)
        aaii->peer = strdup(peer);
    if (dsa != (char *) 0)
        aaii->dsa = strdup(dsa);
    if (IFlag)
        memcpy((void *) aaii->sessionKey, (void *) makeMd5Key(), 8);

    mio.optionId = MIO_AuthAssocIntegrityInfo;
    mio.optionInfo = (void *) aaii;

    miop[0] = &mio;
    miop[1] = (MInitOption *) 0;

#else

   miop[0] = miop[1] = (MInitOption *) 0;

#endif

    if ((msd = GMD_initialise_u(argv[1], argv[2], miop)) < 0)
        error(NULLCP, "cannot establish association to %s at %s",
                        argv[1], argv[2]);
#if defined(SECURITY)
    if (aaii != (AuthAssocIntegrityInfo *) 0)
        AuthAssocIntegrityInfo_free(aaii);
#endif

    // perform the M-DELETE request

    printf("\n\n");

    do_delete(msd, objClass, objInst, scope, filter, sync, &mis);

    // wait for the replies
#if defined(DEBUG)
    if ((pduDump = fopen("mdelete.rx.log", "w")) != (FILE *) 0)
        ppSetOutputFile(pduDump);
#endif

    while (do_wait(msd, &mis) != DONE);

    // terminate

    m_terminate_u(msd);

#if defined(DEBUG)
    if (pduDump != (FILE *) 0)
        fclose(pduDump);
#endif

    exit(OK);

usage:
    fprintf(stderr, "usage: mdelete <agent> <host>\n\
\t[-c <class>] [-i <instance>]\n\
\t\[-s <scope> [<sync>]] [-f<filter>]\n");
#if defined(SECURITY)
    fprintf(stderr, "\t[-P <peer DN>] [-D <dsa name/addr>] [-I]\n");
#endif
    exit(0);
}

// deletes a managed object
static int  do_delete (int msd, OID objClass, MN objInst,
                        CMISScope* scope, CMISFilter* filter, CMISSync sync,
                        MSAPIndication* mi)
{
    struct MIDentifier obj_class;
    int	status;

    obj_class.mid_type = MID_GLOBAL;
    obj_class.mid_global = objClass;

    if (!objInst) {
        objInst = new  MName;
        objInst -> mn_type = MN_LOCALDN;
        objInst -> mn_dn = dn_comp_new(NULLRDN);
    }

    request_id = 1;

    status = M_Delete_u(msd, request_id, &obj_class, objInst, scope,
				filter, NULLMACCESS, sync, mi);
    request_invoke = mi->mi_invoke;

    // release memory of arguments (for the sake of it...)

    mid_free(&obj_class);
    mn_free(objInst);
    delete objInst;
    if (scope)
	free((char*) scope);
    if (filter)
	mfilter_free(filter);

    return status;
}

static int  do_wait (int msd, MSAPIndication* mi)
{
    int status = OK;
    static int nsucc = 0, nfail = 0;
    Bool linked = False;

    M_WaitReq_u(msd, NOTOK, mi);

#if defined(DEBUG)
    if (pduDump != (FILE *) 0)
        ppMSAPIndication(mi);
#endif

    if (mi->mi_type == MI_DELETE_RES || mi->mi_type == MI_LINKED_REPLY) {
	int	id;
	MID	objClass;
	MN	objInst;
	char*	time = NULLCP;
	CMISErrors error = m_noError;

	switch (mi->mi_type) {
	case MI_DELETE_RES:
	    id	     = mi->mi_deleteres.id;
	    objClass = &mi->mi_deleteres.result.dr_class;
	    objInst  = &mi->mi_deleteres.result.dr_inst;
	    time     = mi->mi_deleteres.result.dr_time;
	    printf("Delete Result - id %#08x, ", (u_int) id);
	    ++nsucc;
	    status = DONE;	// only one deleted
	    break;

	case MI_LINKED_REPLY:
            linked = True;
	    id = mi->mi_linkedreplyinv.args.lr_linkid;
	    switch (mi->mi_linkedreplyinv.args.lr_type) {
	    case LR_DELETE_RES:
		objClass = &mi->mi_linkedreplyinv.args.lr_deleteres.dr_class;
		objInst  = &mi->mi_linkedreplyinv.args.lr_deleteres.dr_inst;
		time     = mi->mi_linkedreplyinv.args.lr_deleteres.dr_time;
		++nsucc;
		break;

	    case LR_DELETE_ERR:
		objClass = &mi->mi_linkedreplyinv.args.lr_deleteerr.de_class;
		objInst  = &mi->mi_linkedreplyinv.args.lr_deleteerr.de_inst;
		time     = mi->mi_linkedreplyinv.args.lr_deleteerr.de_time;
		error	 = mi->mi_linkedreplyinv.args.lr_deleteerr.de_error;
		++nfail;
		break;

	    case LR_PROC_FAILURE:
		// this should not really happen for a well debugged agent
		objClass = &mi->mi_linkedreplyinv.args.lr_procfailure.pf_class;
		objInst  = &mi->mi_linkedreplyinv.args.lr_procfailure.pf_inst;
		error    = m_processingFailure;
		++nfail;
		break;
	    }
	    printf("Linked Reply Delete - id %#08x, ", (u_int) id);
	    break;
        }

	if (id != (linked ? request_invoke :
                   request_id
                   )) {
	    printf("Delete result with wrong request id received!\n");
	    mi_free(mi);
	    return DONE;
	}

	if (objClass->mid_global)
            printf("class %s ", oid2name(objClass->mid_global, OIDPART));

	if (objInst->mn_dn) {
	    char* inst = dn2str(objInst->mn_dn);
	    printf("instance {%s} ", inst);
	    free(inst);
	}

	if (time)
            printf("time %s ", time);

	if (error)
	    printf("\nError: %s", MErrString(error));

	printf("\n\n");
    }
    else
    if (mi->mi_type == MI_EMPTY_RES && mi->mi_emptyres.id ==
        request_id
        )
        status = DONE;
    else
    if (mi->mi_type == MI_PREJECT || mi->mi_type == MI_ABORT) {
	fprintf(stderr, "mdelete: received %s, exits\n",
		mi->mi_type == MI_PREJECT ?
				"provider reject" : "association abort");
#if defined(DEBUG)
        if (pduDump != (FILE *) 0)
            fclose(pduDump);
#endif
	exit(NOTOK);
    }
    else
    if (mi->mi_type == MI_ERROR || mi->mi_type == MI_UREJECT) {
	fprintf(stderr, "Error: %s\n", MErrString(mi->mi_errtype));
	status = DONE;
    }
    else {
	printf("Unexpected mindication type: %d\n", mi->mi_type);
	status = NOTOK;
    }

    if (status == DONE) {
	if (nsucc)
	    printf("\n%d managed objects deleted\n", nsucc);
	if (nfail)
	    printf("\n%d managed objects failed to delete\n", nfail);
    }
    mi_free(mi);
    return status;
}
