//
// COPYRIGHT (c) 1990 University College London
// All rights reserved.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the Department of Computer Science, University College London.
// The name of the University may not be used to
// endorse or promote products derived from this software without
// specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//

// MibDump.cc - enables to read managed objects/attributes from a remote MIB

// By George Pavlou, February 1991


#include <stdlib.h>
#include <strings.h>
#include <signal.h>

#ifdef	__GNUG__
#ifndef __HPUX__
#define SIGHANDLER	SignalHandler
#else
#define SIGHANDLER      void
#endif
#include <std.h>
extern "C" { int  strncasecmp (const char*, const char*, unsigned int); }
#elif	__cplusplus
#ifndef AIXV3
#define SIGHANDLER	SIG_PF
#else	/* AIXV3 */
#define SIGHANDLER	void
extern "C" { int  strncasecmp (const char*, const char*, int);
             int  strcasecmp (const char*, const char*); }
#endif
#endif

#define _module_CMIP_defined_ /* CMIS-type.h causes msap_u.h to screw up */
#include "msap.h"
#undef _module_CMIP_defined_
#include "msap_u.h" /*XMP version calls*/

#include "Error.h"
#include "Syntaxes.h"
#include <memory.h>

#if defined(SECURITY)
#include "mauth.h"
#include "mintegrity.h"
#endif

#if defined(DEBUG)
#include "msap-pp.h"
FILE *pduDump = (FILE *) 0;
#endif

#define MAXATTRS	50

int	msd,
        request_id = 0,     /* Sent by mibdump */
        request_invoke = 0, /* For use in Cancel Get */
        invoke = 0;         /* Received from agent */
Bool cancel_get = False;
MSAPIndication mis;
Bool	timing = False;

int   parseFilterArg (int, char**, char**, int*);
static int   do_get (int, OID, MN, CMISScope*, CMISFilter*, CMISSync,
			int, MID, MSAPIndication*);
static int   do_wait (int, MSAPIndication*);

static void  do_cancelget (int);
static void  do_time (timeval*, char*);

main (int argc, char** argv)
{
    timeval prev;

    char* strObjClass = NULLCP;
    char* strObjInst = NULLCP;
    char* strScope = NULLCP;
    char* strSync = NULLCP;
    char* strFilter = NULLCP;
    char* strAttrs[MAXATTRS];
    int   nattrs = GET_ALLATTRS;	// default
    int   i, j, k, l, state = 0;

    OID objClass = NULLOID;
    MN  objInst  = NULLMN;
    CMISScope* scope = NULLMSCOPE;	// default
    CMISSync sync = s_bestEffort;	//  ..
    CMISFilter* filter = NULLMFILTER;	//  ..
    struct MIDentifier attrs[MAXATTRS];

#ifdef SECURITY
    int IFlag = 0;
    char *peer = (char *) 0,
         *dsa = (char *) 0;
#endif

    // parse command line arguments

    if (argc < 3)
	goto usage;

    if (argc > 3)
	for (i = 3; i < argc; i++)
	    switch (state) {
	    case 0:
		if (*argv[i] == '-' && strlen(argv[i]) == 2)
		    switch (*(argv[i]+1)) {
		    case 'c':
			state = 1;
			break;
		    case 'i':
			state = 2;
			break;
		    case 's':
			state = 3;
			break;
		    case 'f':
			state = 4;
			break;
		    case 'a':
			state = 5;
			break;
		    case 't':
			timing = True;
			break;
#ifdef SECURITY
                    case 'P':
                        state = 6;
                        break;
                    case 'D':
                        state = 7;
                        break;
                    case 'I':
                        state = 0;
                        IFlag = 1;
                        break;
#endif
		    default:
			goto usage;
		    }
		else
		    goto usage;
		break;

	    case 1:
		strObjClass = argv[i];
		state = 0;
		break;

	    case 2:
		strObjInst = argv[i];
		state = 0;
		break;

	    case 3:
		strScope = argv[i];
		if (i < argc-1 && *argv[i+1] != '-')
		    strSync = argv[++i];
		state = 0;
		break;

	    case 4:
		if (parseFilterArg(argc-i, &argv[i], &strFilter, &j) == NOTOK)
		    goto usage;
		i += j-1;
		state = 0;
		break;

	    case 5:
	    {
		Bool more = False;
		for (j = i; ; j++)
		    if (*argv[j] == '-' && strlen(argv[j]) == 2) {
			more = True;
			break;
		    }
		    else
		    if (j == argc-1)
			break;

		nattrs = more ? j-i : j-i+1;
		if (nattrs > MAXATTRS)
		    error(NULLCP, "too many attrs - max number %s", MAXATTRS);

		for (k = i, l = 0; k < j+1; k++, l++)
		    strAttrs[l] = argv[k];
		state = 0;
		i = more ? j-1 : j;
	    }
		break;
#ifdef SECURITY
            case 6:
                peer = argv[i];
                state = 0;
                break;
            case 7:
                dsa = argv[i];
                state = 0;
                break;
#endif
	    }


    // load syntaxes

    if (initialiseSyntaxes(NULLCP, attrSyntaxes) == NOTOK)
	error(NULLCP, "mibdump: cannot initialise syntaxes");

#ifdef SECURITY
    initialiseOsisec(argc, argv);
    initialiseMd5KeySpace();
    initialiseIdSpace();
#endif

    // convert the string arguments to CMIS parameters

    if (strObjClass && (objClass = name2oid(strObjClass)) == NULLOID)
	error(NULLCP, "invalid object class: %s", strObjClass);

    if (strObjInst && (objInst = ObjectInstance_parse(strObjInst)) == NULLMN)
        error(NULLCP, "invalid object instance %s", strObjInst);

    if (strScope && (scope = str2scope(strScope)) == NULLMSCOPE)
	error(NULLCP, "invalid scope %s", strScope);

    if (strSync && (sync = str2sync(strSync)) == s_invalid)
	error(NULLCP, "invalid sync %s", strSync);
	
    if (strFilter && strFilter[0] != '\0'  &&
		(filter = str2mfilter(strFilter)) == NULLMFILTER)
	error(NULLCP, "invalid filter %s", strFilter);

    if (nattrs && strcasecmp(strAttrs[0], "none") == 0)
	nattrs = GET_NOATTRS;
    else
	for (i = 0; i < nattrs; i++) {
	    if ((attrs[i].mid_global = name2oid(strAttrs[i])) == NULLOID)
		error(NULLCP, "invalid attribute name %s", strAttrs[i]);
	    attrs[i].mid_type = MID_GLOBAL;
	}


    // now try to connect to the remote agent

    MInitOption *miop[2]; /* Connect options */

#ifdef SECURITY
    MInitOption mio;
    AuthAssocIntegrityInfo *aaii;
    aaii = AuthAssocIntegrityInfo_make();

    if (peer != (char *) 0)
        aaii->peer = strdup(peer);
    if (dsa != (char *) 0)
        aaii->dsa = strdup(dsa);
    if (IFlag)
        memcpy((void *) aaii->sessionKey, (void *) makeMd5Key(), 8);

    mio.optionId = MIO_AuthAssocIntegrityInfo;
    mio.optionInfo = (void *) aaii;

    miop[0] = &mio;
    miop[1] = (MInitOption *) 0;

#else

   miop[0] = miop[1] = (MInitOption *) 0;

#endif

    if (timing)
	gettimeofday(&prev, NULL);

    if ((msd = GMD_initialise_u(argv[1], argv[2], miop)) < 0)
	error(NULLCP, "cannot establish association to %s at %s",
			argv[1], argv[2]);
    if (timing)
	do_time(&prev, "established association");

#if defined(SECURITY)
    if (aaii != (AuthAssocIntegrityInfo *) 0)
        AuthAssocIntegrityInfo_free(aaii);
#endif

    // perform the M-GET request

    printf("\n\n");

    do_get(msd, objClass, objInst, scope, filter, sync, nattrs, attrs, &mis);

    // set signal handlers

#ifdef	__GNUG__
    signal(SIGINT,  do_cancelget);
    signal(SIGQUIT, do_cancelget);
    signal(SIGTERM, do_cancelget);
#else
    signal(SIGINT,  (SIGHANDLER) do_cancelget);
    signal(SIGQUIT, (SIGHANDLER) do_cancelget);
    signal(SIGTERM, (SIGHANDLER) do_cancelget);
#endif

#if defined(DEBUG)
    if (!timing && (pduDump = fopen("mibdump.rx.log", "w")) != (FILE *) 0)
        ppSetOutputFile(pduDump);
#endif

    m_terminate_u(msd);	// release association
    if (timing)
	do_time(&prev, "released association");

#if defined(DEBUG)
    if (pduDump != (FILE *) 0)
        fclose(pduDump);
#endif

    exit (OK);

usage:
    fprintf(stderr, "usage: mibdump <agent> <host>\n\
\t[-c <class>] [-i <instance>]\n\
\t[-s <scope> [<sync>]] [-f filter] [-a attr ...] ]\n");
#if defined(SECURITY)
    fprintf(stderr, "\t[-P <peer DN>] [-D <dsa name/addr>] [-I]\n");
#endif
    exit(0);
}

static int  do_get (int msd, OID objClass, MN objInst,
			CMISScope* scope, CMISFilter* filter, CMISSync sync,
			int nattrs, MID attrs, MSAPIndication* mi)
{
    struct MIDentifier obj_class;
    int		status;

    obj_class.mid_type = MID_GLOBAL;
    obj_class.mid_global = objClass;

    if (!objInst) {
        objInst = new  MName;
        objInst -> mn_type = MN_LOCALDN;
        objInst -> mn_dn = dn_comp_new(NULLRDN);
    }

    request_id = 1;
    status = M_Get_u_sync(msd, request_id, &obj_class, objInst, scope, filter,
			NULLMACCESS, sync, nattrs, attrs, mi);
    request_invoke = mi->mi_invoke;

    // release memory of arguments (for the sake of it...)

    mid_free(&obj_class);
    mn_free(objInst);
    delete objInst;
    if (scope)
	free((char*) scope);
    if (filter)
	mfilter_free(filter);
    for (int i = 0; i < nattrs; i++)
	mid_free(&attrs[i]);

    return status;
}

static void  do_cancelget (int)
{
    if (!cancel_get) {
        ++request_id;
	M_CancelGet(msd, request_id, request_invoke, &mis);
        cancel_get = True;
    }
}

static void  do_time (timeval* prev, char* msg)
{
    timeval now;

    gettimeofday(&now, NULL);

    long diffsec = now.tv_sec - prev->tv_sec, diffusec;
    if ((diffusec = now.tv_usec - prev->tv_usec) < 0)
        --diffsec, diffusec += 1000000;

    *prev = now;	// struct copy

    printf("%s in %d.%06d seconds\n\n", msg, diffsec, diffusec);
}
