/* Coiyright (c) 1993 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/


#include <stdio.h>
#include <sys/types.h>
#include "msap.h"
#include "xmp.h"
#if defined(SECURITY)
#include "mauth.h"
#include "mintegrity.h"
#endif
#if defined(DEBUG)
#include <string.h>
#endif

#if !defined(_a_)
#if defined(__STDC__) || defined(__cplusplus)
#define _a_(_args_) _args_
#else
#define _a_(_args_) ()
#endif
#endif

extern int xmp_initialised;
extern MIDAS_WORKSPACE midas_workspace;
extern MIDAS_WORKSPACE mp_initialise_M();
static char *context = "management";
extern int debug_test _a_((char *));
extern int XMP_assoc_fail_u();

/*  GMD_InitialiseReq_u - Calling is equivalent to a M-ASSOCIATE.REQUEST
 *  a succesful return is equivalent to a M-ASSOCIATE.CONFIRMATION
 *
 *  N.B. IF SECURITY IS IN USE THEN THE "info" PARAMETER MUST BE A POINTER
 *  TO AN AuthAssocIntegrityInfo  OR NULL
 */

int
GMD_InitialiseReq_u (callingTitle, callingAddr, calledTitle, calledAddr,
            context, protvrsn, fununits, access, info, mc, mi)
    AEI callingTitle, calledTitle;
    char* context;
    struct PSAPaddr *callingAddr, *calledAddr;
    int protvrsn, fununits;
    External *access;
#if defined(SECURITY)
    AuthAssocIntegrityInfo *info;
#else
    External *info;
#endif
    MSAPConnect* mc;
    MSAPIndication* mi;
{
    MIDAS_FEATURE feature;
    int ret;
    MIDAS_SESSION supplied_session, *sessp;
    MIDAS_STATUS *retstat;
#if defined(SECURITY)
    static OID auth_oidp;
#endif
    register int e = 0;

    /*check xmp has been initialised*/

    /*If a workspace has not been created, then create one*/
    if (!xmp_initialised){
        if ((midas_workspace = mp_initialise_M()) == (MIDAS_WORKSPACE)NULL){
            fprintf(stderr, "GMD_InitialiseReq: mp_initialise_M failed\n");
            return(XMP_assoc_fail_u(MA_LOCAL, 0, 
			"XMP: Bad workspace", mi));
    	}

#if defined(SECURITY)
	if ((auth_oidp = oid_cpy(str2oid(CREDENTIAL_OID))) == (OID)NULL){
	    fprintf(stderr, "GMD_InitialiseReq_u: Can't get OID\n");
	    return(XMP_assoc_fail_u(MA_LOCAL, 0, 
			"XMP: Resource problem", mi));
	}
	feature.feature = *auth_oidp;		/*struct copy*/
	feature.request = MP_ACTIVATE;
	feature.MIDAS_feature_next = (MIDAS_FEATURE *)NULL;

	if (mp_negotiate_M(&feature, midas_workspace) != MIDAS_SUCCESS){
	    fprintf(stderr, "GMD_InitialiseReq_u: mp_negotiate_M failed\n");
	    return(XMP_assoc_fail_u(MA_LOCAL, 0, 
			"XMP: Bad negotiation", mi));
	}

	if (feature.response != TRUE){
	    fprintf(stderr, "GMD_InitialiseReq_u: security not allowed\n");
	    return(XMP_assoc_fail_u(MA_LOCAL, 0, 
			"XMP: Bad feature", mi));
	}
#endif
	xmp_initialised = 1;
    }

    supplied_session.requestor_title = callingTitle;
    supplied_session.requestor_address = callingAddr;
    supplied_session.responder_title = calledTitle;
    supplied_session.responder_address = calledAddr;
    supplied_session.CMIS_functional_units = fununits;
    supplied_session.access_control = access;

#if defined(SECURITY)
    if (info == (AuthAssocIntegrityInfo *)0){ 
	fprintf(stderr, "GMD_InitialiseReq_u: No authentication info\n");
	return(XMP_assoc_fail_u(MA_LOCAL, MA_BADINFO, 
				"XMP: No auth. info.", mi));
    }

    supplied_session.user_info = (External *)NULL;
    supplied_session.authentication_mechanism = auth_oidp;
    supplied_session.authentication_value = (char *)info;
#else
    supplied_session.user_info = info;
#endif

    if ((retstat = mp_bind_M(&supplied_session, midas_workspace, &sessp)) !=
			MIDAS_SUCCESS) {
	    fprintf(stderr, "GMD_InitialiseReq_u: mp_bind_M failed\n");
	    free(sessp);
	    if (retstat->MIDAS_parameter){
		*mi = *(retstat->MIDAS_parameter);	/*struct copy*/
		return(NOTOK);
	    }
	    else
	    return(XMP_assoc_fail_u(MA_LOCAL, 0, 
				"XMP: mp_bind failed", mi));
    }
#if defined(SECURITY)
    /* oid_free(auth_oidp);*/
#endif

    /*Keep track of open sessions*/
    if (session_add(sessp) == NOTOK){
	fprintf(stderr, "GMD_InitialiseReq_u: session_add fails\n");
	free(sessp);
	return(NOTOK);
    }

    mc->mc_sd = sessp->file_descriptor;
    mc->mc_result = ACS_ACCEPT; 
    mc->mc_context = sessp->application_context;
    mc->mc_resptitle = *(sessp->responder_title); /*struct copy*/
    mc->mc_info.mai_protvrsn = CMIP_VERSION2;
    mc->mc_info.mai_funits = sessp->CMIS_functional_units;
    mc->mc_info.mai_access = (External *)NULL;
    mc->mc_info.mai_info = sessp->user_info;
    return(OK);
}

/*  GMD_initialise_u - higher level friendly interface to M_InitialiseReq 
 *  (Same as m_initialise except it calls GMD_InitialiseReq_u() and
 *  M_AbortReq_u()
 */

int  GMD_initialise_u (agent, rhost, options)
char* agent;
char* rhost;
MInitOption **options;
{
    MSAPConnect  mcs;
    MSAPIndication  mis;
    AEI  aei;
    struct PSAPaddr *pa;
    External *accessControl = (External *) 0;
    MInitOption **opt;
#if defined(SECURITY)
    AuthAssocIntegrityInfo *userInfo = (AuthAssocIntegrityInfo *) 0;
#else
    External *userInfo = (External *) 0;
#endif
    register
    int e = 0;

#ifdef USE_X500

    char * agentDN= lt_str2dn(rhost, agent);

    if ( (pa = lt_read(agentDN)) == NULLPA ) {
        advise(NULLCP, "%s-%s:  Null PSAP address returned from the Directory.\nTrying static AE information...\n",
                rhost, agent);

        if ((aei = str2aei(rhost, agent)) == NULLAEI) {
            fprintf(stderr, "%s-%s: unknown application entity title\n",
                            rhost, agent);
            return NOTOK;
        }

        if ((pa = aei2addr(aei)) == NULLPA) {
            fprintf(stderr, "aei translation failed\n");
            return NOTOK;
        }
    } else {
        if ( lt_oper_status == DOWN ) {
            fprintf(stderr,"The specified agent is not currently running.\n");
            return NOTOK;
        }
        aei = NULLAEI;
    }

#else

    if ((aei = str2aei(rhost, agent)) == NULLAEI) {
	fprintf(stderr, "%s-%s: unknown application entity title\n",
			rhost, agent);
	return NOTOK;
    }

    if ((pa = aei2addr(aei)) == NULLPA) {
	fprintf(stderr, "aei translation failed\n");
	return NOTOK;
    }

#endif /* USE_X500 */

    for(opt = options; opt && *opt; ++opt) {

        switch ((*opt)->optionId) {

#if defined(SECURITY)
        case MIO_AuthAssocIntegrityInfo: /* See mauth.h */
            userInfo = (AuthAssocIntegrityInfo *) (*opt)->optionInfo;
            if (userInfo == (AuthAssocIntegrityInfo *) 0) {
                fprintf(stderr, "NULL AuthAssocIntegrityInfo\n");
                return NOTOK;
            }
	    break;
#endif
        default:
            fprintf(stderr, "m_initialise(): option %d not available\n",
                    (*opt)->optionId);
            break;
        }
    }

    /* request all the functional units bar the extendedService */
    if (GMD_InitialiseReq_u(NULLAEI,  NULLPA, aei, pa, context,
		CMIP_VERSION2, 0x17, accessControl, userInfo, &mcs, &mis)
	    == NOTOK) {
	if (mis.mi_abort.ma_reason == ACS_TRANSIENT)
	    fprintf(stderr, "Remote agent is busy, try later...\n");
	else
	    fprintf(stderr, "M_InitialiseRequest: %s\n", mis.mi_abort.ma_data);
	mi_free(&mis);

        if (accessControl != (External *) 0)
            external_free(accessControl);
#ifndef SECURITY
        if (userInfo != (External *) 0)
            external_free(userInfo);
#endif

	return NOTOK;
    }
    if (accessControl != (External *) 0)
        external_free(accessControl);
#ifndef SECURITY
    if (userInfo != (External *) 0)
        external_free(userInfo);
#endif

/*
  All the security verification gets done inside mp_bind in GMD mode
  hence there is no need for such code here.
*/

    mc_free(&mcs);

    return e == 0 ? mcs.mc_sd : e;
}
