/* Copyright (c) 1993 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* Jonathan Ladipo, November 1993 */

 /*************************************************************************
 *
 * INTERFACE ROUTINE: M_Delete_u - delete managed object(s)
 *
 *************************************************************************/

#include "xmp.h"
#include "msap.h"

#if !defined(_a_)
#if defined(__STDC__) || defined(__cplusplus)
#define _a_(_args_) _args_
#else
#define _a_(_args_) ()
#endif
#endif

extern MIDAS_CONTEXT context;
extern MIDAS_SESSION default_session;
extern MIDAS_WORKSPACE midas_workspace;
extern int xmp_initialised;
extern MIDAS_SESSION *session_get();
extern int arg_fail _a_((int, int, int, char*, MSAPIndication*));
extern char*  op2str _a_((int));
extern int debug_test _a_((char *));

#define missing_midP(p)         (!(p) || !(p)->mid_global)
#define missing_mnP(p)          (!(p) || !(p)->mn_dn)
#define null_midP(p)            (!(p)->mid_global)
#define null_mnP(p)             (!(p)->mn_dn)
#define null_mparmP(p)          (!(p)->mp_id.mid_global || !(p)->mp_val)
/*************************************************************************/

int
M_Delete_u(msd, invoke, obj_class, obj_inst, scope, filter, access, sync, mi)

    int  msd, invoke;
    MID  obj_class;
    MN   obj_inst;
    CMISScope* scope;
    CMISFilter* filter;
    External* access;
    CMISSync sync;
    MSAPIndication* mi; {

    CMISDeleteArg  argument;
    OM_sint32           invoke_id_return = (OM_sint32)invoke; /*NON
							    STANDARD XMP USAGE*/
    MIDAS_STATUS_P      statp;
    MIDAS_RESULT	*resultp;
    MIDAS_SESSION	*sessp;
    int 		i;

    /*check xmp has been initialised*/

    debug_test("In M_Delete_u");

    if (!xmp_initialised) {
        fprintf(stderr, "M-InitialiseReq: XMP not initialised\n");
        return(NOTOK);
    }

    if ((sessp = session_get(msd))
                == (MIDAS_SESSION *)NULL){
        /*assume UCL mode*/
        sessp = &default_session;
    }

    /*set up the context*/

    context.connection_id = msd;
    context.mode = MP_T_CONFIRMED;
    context.asynchronous = 1;

    /*set up the argument*/

    if (missing_midP(obj_class))
        return arg_fail(invoke, M_DELETE, 0, "Object Class", mi);
    if (missing_mnP(obj_inst))
        return arg_fail(invoke, M_DELETE, 0, "Object Instance", mi);

    if(scope)
        argument.da_scope = *scope;         /*struct copy*/
    else
        argument.da_scope.sc_type = Sc_BaseObject;         /*struct copy*/


    argument.da_class = *obj_class;     /* struct copy */
    argument.da_inst  = *obj_inst;      /* struct copy */
    argument.da_access = access;
    argument.da_sync = sync;
    argument.da_filter = filter;

    /*Now do the delete*/

    statp = mp_delete_req_M(sessp, &context, &argument,
                               &resultp, &invoke_id_return);

    if((statp == MIDAS_NO_WORKSPACE)||(statp == MIDAS_INVALID_SESSION)||
		(statp == MIDAS_INSUFFICIENT_RESOURCES)){

	fprintf(stderr, "M_Delete_u: local error\n");
	return(NOTOK);
     }
    
     else if(statp == MIDAS_SUCCESS){
	/*Since this was an async call there will be no result*/
     	return(OK);
     }

    else{
    	/*pointer to a valid MIDAS_STATUS structure*/
	/*copy status to caller's structure*/

         if(statp->MIDAS_parameter)
             *mi = *(statp->MIDAS_parameter);            /*struct copy*/
         else
             mi = (MSAPIndication*)NULL;

	debug_test("Leaving M_Delete_u");
     	return(NOTOK);
    }
}


