/* Copyright (c) 1993 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/


/*  M_InitialiseReq_u - Calling is equivalent to a M-ASSOCIATE.REQUEST
 *  a succesful return is equivalent to a M-ASSOCIATE.CONFIRMATION
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include "msap.h"
#include "xmp.h"
#if defined(SECURITY)
#include "mauth.h"
#include "mintegrity.h"
#include "../provide/mp_integrity.h"
#endif
#if defined(DEBUG)
#include <string.h>
#endif


int xmp_initialised = 0;
MIDAS_SESSION *sessionp;
MIDAS_WORKSPACE midas_workspace;
extern MIDAS_WORKSPACE mp_initialise_M();
extern MIDAS_SESSION default_session;
fd_set MIDAS_read_fdset;
extern int XMP_assoc_fail_u();

int
M_InitialiseReq_u (callingTitle, callingAddr, calledTitle, calledAddr,
            context, protvrsn, fununits, access, info, mc, mi)
    AEI callingTitle, calledTitle;
    char* context;
    struct PSAPaddr *callingAddr, *calledAddr;
    int protvrsn, fununits;
    External *access, *info;
    MSAPConnect* mc;
    MSAPIndication* mi;{
    MIDAS_FEATURE feature;
    int ret;

#if defined(SECURITY)
    OID op;
#endif

    /*check xmp has been initialised*/

    /*If a workspace has not been created, then create one*/
    if (!xmp_initialised) {
        if ((midas_workspace = mp_initialise_M()) == (MIDAS_WORKSPACE)NULL){
            fprintf(stderr, "M-InitialiseReq: mp_initialise_M failed\n");
            return(NOTOK);
        }
        if (mp_bind_M(&default_session, midas_workspace, &sessionp) !=
			MIDAS_SUCCESS) {
	    fprintf(stderr, "M-InitialiseReq: mp_bind_M failed\n");
	    return(NOTOK);
	}
        /*Initialise the set of association descriptors to zero.*/
	FD_ZERO(&MIDAS_read_fdset);
        xmp_initialised = 1;
    }

#if defined(SECURITY)

    /*Negotiate security*/

    if ((op = oid_cpy(str2oid(CREDENTIAL_OID))) == (OID)NULL){
        fprintf(stderr, "GMD_InitialiseReq_u: Can't get OID\n");
        return(XMP_assoc_fail_u(MA_LOCAL, 0,
                        "XMP: Resource problem", mi));
    }
    feature.feature = *op;              /*struct copy*/
    feature.request = MP_ACTIVATE;
    feature.MIDAS_feature_next = (MIDAS_FEATURE *)NULL;

    if (mp_negotiate_M(&feature, midas_workspace) != MIDAS_SUCCESS){
        fprintf(stderr, "GMD_InitialiseReq_u: mp_negotiate_M failed\n");
        return(XMP_assoc_fail_u(MA_LOCAL, 0,
                        "XMP: Bad negotiation", mi));
    }

    if (feature.response != TRUE){
        fprintf(stderr, "GMD_InitialiseReq_u: security not allowed\n");
        return(XMP_assoc_fail_u(MA_LOCAL, 0,
                        "XMP: Bad feature", mi));
    }
#endif


    /*now do the normal stuff*/

    /*Establish a management association via MSAP.*/
    ret = M_InitialiseReq (callingTitle, callingAddr, calledTitle,
         calledAddr, context, protvrsn, fununits, access, info, mc, mi);

    if (ret == OK)
	/*Add the association descriptor to the read_set for this session*/
	FD_SET(mc->mc_sd, &MIDAS_read_fdset);

    return(ret);
}



/*  m_initialise_u - higher level friendly interface to M_InitialiseReq 
 *  (Same as m_initialise except it calls M_InitialiseReq_u() and
 *  M_AbortReq_u()
 */

static char *context = "management";

int  m_initialise_u (agent, rhost, options)
char* agent;
char* rhost;
MInitOption **options;
{
    MSAPConnect  mcs;
    MSAPIndication  mis;
    AEI  aei;
    struct PSAPaddr *pa;
    External *accessControl = (External *) 0, *userInfo = (External *) 0;
    MInitOption **opt;
#if defined(SECURITY)
    AuthAssocIntegrityInfo *aaii = (AuthAssocIntegrityInfo *) 0,
                           *aaii2 = (AuthAssocIntegrityInfo *) 0;
    register
    int authFlag = 0; /* Set up an authneticated association */
#endif
    register
    int e = 0;

#ifdef USE_X500

    char * agentDN= lt_str2dn(rhost, agent);

    if ( (pa = lt_read(agentDN)) == NULLPA ) {
        advise(NULLCP, "%s-%s:  Null PSAP address returned from the Directory.\nTrying static AE information...\n",
                rhost, agent);

        if ((aei = str2aei(rhost, agent)) == NULLAEI) {
            fprintf(stderr, "%s-%s: unknown application entity title\n",
                            rhost, agent);
            return NOTOK;
        }

        if ((pa = aei2addr(aei)) == NULLPA) {
            fprintf(stderr, "aei translation failed\n");
            return NOTOK;
        }
    } else {
        if ( lt_oper_status == DOWN ) {
            fprintf(stderr,"The specified agent is not currently running.\n");
            return NOTOK;
        }
        aei = NULLAEI;
    }

#else

    if ((aei = str2aei(rhost, agent)) == NULLAEI) {
	fprintf(stderr, "%s-%s: unknown application entity title\n",
			rhost, agent);
	return NOTOK;
    }

    if ((pa = aei2addr(aei)) == NULLPA) {
	fprintf(stderr, "aei translation failed\n");
	return NOTOK;
    }

#endif /* USE_X500 */

    for(opt = options; opt && *opt; ++opt) {

        switch ((*opt)->optionId) {

#if defined(SECURITY)
        case MIO_AuthAssocIntegrityInfo: /* See mauth.h */
            aaii = (AuthAssocIntegrityInfo *) (*opt)->optionInfo;
            if (aaii == (AuthAssocIntegrityInfo *) 0) {
                fprintf(stderr, "NULL AuthAssocIntegrityInfo\n");
                return NOTOK;
            }

            /* Make ACSE User Info */
            if ((e = makeAcseUserInfo(aaii, &userInfo)) != 0 ||
                userInfo == (External *) 0) {
                fprintf(stderr, "m_initialise(): makeAcseUserInfo() %s\n",
                        authErrorMessage(e));
                return NOTOK;
            }
            authFlag = 1;
            break;
#endif
        default:
            fprintf(stderr, "m_initialise(): option not %d available\n",
                    (*opt)->optionId);
            break;
        }
    }

    /* request all the functional units bar the extendedService */
    if (M_InitialiseReq_u(NULLAEI,  NULLPA, aei, pa, context,
		CMIP_VERSION2, 0x17, accessControl, userInfo, &mcs, &mis)
	    == NOTOK) {
	if (mis.mi_abort.ma_reason == ACS_TRANSIENT)
	    fprintf(stderr, "Remote agent is busy, try later...\n");
	else
	    fprintf(stderr, "M_InitialiseRequest: %s\n", mis.mi_abort.ma_data);
	mi_free(&mis);

        if (accessControl != (External *) 0)
            external_free(accessControl);
        if (userInfo != (External *) 0)
            external_free(userInfo);

	return NOTOK;
    }
    if (accessControl != (External *) 0)
        external_free(accessControl);
    if (userInfo != (External *) 0)
        external_free(userInfo);

#if defined(SECURITY)
    if (authFlag) {
        /*
        ** Authenticate peer
        */
        e = getAcseUserInfo(mcs.mc_info.mai_info, &aaii2);

        /* Sanity check */
        if (e == 0 && memcmp(aaii->sessionKey, aaii2->sessionKey, 8) != 0) {
            fprintf(stderr, "m_initialise(): different sessionKey from peer\n");
            e = AUTH_error;
        }
    }

    if (authFlag && e == 0) {

#if defined(DEBUG)
        register
        char *nameStr = (char *) 0, *cnStr;

        cnStr = "cn=";
        if ((nameStr = strstr(aaii2->peer, cnStr)) == (char *) 0) {
            cnStr = "commonName=";
            nameStr = strstr(aaii2->peer, cnStr);
        }
        if (nameStr != (char *) 0)
            nameStr += strlen(cnStr);
        fprintf(stderr, "I believe I am talking to %s\n\n",
                nameStr != (char *) 0 ? nameStr : aaii2->peer);
#endif
        /* Clear previous Id state */
        clearId(mcs.mc_sd);

        /* Set up the session key */
        setMd5Key(mcs.mc_sd, aaii->sessionKey);

        /* Clear the invokeId manager */
        clearInvoke(mcs.mc_sd);

        /* Set the IRStatus */
        setIRStatus(mcs.mc_sd, IRStatus_initiator);

    }

    else if (authFlag) {
        fprintf(stderr, "Could not athenticate %s: %s aborting assoc\n",
                sprintaei(aei), authErrorMessage(e));

        /*
        ** If authentication fails, for now, drop the assoc. Should
        ** actually give the user the choice, but that requires
        ** changing the interface to m_initialise()
        */
        if (M_AbortReq_u(mcs.mc_sd, (External *) 0, &mis) != OK) {
#if defined(DEBUG)
            fprintf(stderr, "m_initialise(): M_AbortReq() %s\n",
                    mis.mi_abort.ma_data);
#endif
            mi_free(&mis);
        }
    }

    AuthAssocIntegrityInfo_free(aaii2);
#endif /* SECURITY */

    mc_free(&mcs);

    return e == 0 ? mcs.mc_sd : e;
}
