/* Copyright (c) 1993 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/* Jonathan Ladipo, November 1993 */

/*************************************************************************
 *
 * INTERFACE ROUTINE: M_SetAux_u_sync - set management information
 * (SYNCHRONOUS VERSION)
 *************************************************************************/

#include "xmp.h"
#include "msap.h"

#if !defined(_a_)
#if defined(__STDC__) || defined(__cplusplus)
#define _a_(_args_) _args_
#else
#define _a_(_args_) ()
#endif
#endif

extern MIDAS_CONTEXT context;
extern MIDAS_SESSION default_session;
extern MIDAS_WORKSPACE midas_workspace;
extern int xmp_initialised;
extern MIDAS_SESSION *session_get();
extern int debug_test _a_((char *));
extern int arg_fail _a_((int, int, int, char*, MSAPIndication*));
extern char*  op2str _a_((int));

#define missing_midP(p)         (!(p) || !(p)->mid_global)
#define missing_mnP(p)          (!(p) || !(p)->mn_dn)
#define null_midP(p)            (!(p)->mid_global)
#define null_mnP(p)             (!(p)->mn_dn)
#define null_mparmP(p)          (!(p)->mp_id.mid_global || !(p)->mp_val)
/*************************************************************************/

int
M_SetAux_u_sync(msd, mode, invoke, obj_class, obj_inst, scope, filter,
                        access, sync, nattrs, attrs, mi)
    int  msd, mode, invoke;
    MID  obj_class;
    MN   obj_inst;
    CMISScope* scope;
    CMISFilter* filter;
    External* access;
    CMISSync sync;
    int nattrs;
    CMISSetAttr* attrs;
    MSAPIndication* mi; {

    CMISSetArg  argument;
    OM_sint32           invoke_id_return; 
    MIDAS_STATUS_P      statp;
    MIDAS_RESULT	*resultp, *mrp_first; 
    MIDAS_SESSION	*sessp;
    int 		i;

    /*check xmp has been initialised*/

    debug_test("In M_SetAux_u_sync");

    if (!xmp_initialised) {
        fprintf(stderr, "M-InitialiseReq: XMP not initialised\n");
        return(NOTOK);
    }

    if ((sessp = session_get(msd))
                == (MIDAS_SESSION *)NULL){
        /*assume UCL mode*/
        sessp = &default_session;
    }

    /*set up the context*/

    context.connection_id = msd;
    context.asynchronous = 0;
    context.time_limit = 0;

    if(mode ==  MP_T_NON_CONFIRMED)
        context.mode = MP_T_NON_CONFIRMED;
    else
        context.mode = MP_T_CONFIRMED;

    /*set up the argument*/

    if (missing_mnP(obj_inst))
        return arg_fail(invoke, M_SETC, 0, "Object Instance", mi);
    if (nattrs > 0 && !attrs)
        return arg_fail(invoke, M_SETC, 0, "AttributeIdList", mi);

        argument.sa_class = *obj_class;     /* struct copy */

    if (nattrs > 0)
        for (i = 0; i < nattrs; i++)
            if (null_midP(&attrs[i].sa_id))
                return arg_fail(invoke, M_SETC, 0, "AttributeId", mi);
    argument.sa_nattrs = nattrs;
    argument.sa_attrs = attrs;          /*struct copy*/


    if(scope)
	argument.sa_scope = *scope;         /*struct copy*/
    else
	argument.sa_scope.sc_type = Sc_BaseObject;         /*struct copy*/

    argument.sa_inst  = *obj_inst;      /*struct copy*/
    argument.sa_access = access;
    argument.sa_sync = sync;
    argument.sa_filter = filter;

    /*Now do the get*/

    statp = mp_set_req_M(sessp, &context, &argument,
                               &resultp, &invoke_id_return);

    if((statp == MIDAS_NO_WORKSPACE)||(statp == MIDAS_INVALID_SESSION)||
		(statp == MIDAS_INSUFFICIENT_RESOURCES)){

	fprintf(stderr, "M_SetAux_u_sync: local error\n");
	return(NOTOK);
     }
    
     else if(statp == MIDAS_SUCCESS){
        /*Since this was a sync call so print the results*/

         mrp_first = resultp;

         while (resultp) {
             print_result(resultp);
             resultp = resultp->MIDAS_result_next;
         }

         mrp_Q_free(mrp_first);
         debug_test("Leaving M_SetAux_u_sync @ 1");
         return(OK);
     }
     else{
         /*pointer to a valid MIDAS_STATUS structure*/
         /*copy status to caller's structure*/

         if(statp->MIDAS_parameter)
             *mi = *(statp->MIDAS_parameter);            /*struct copy*/
         else
             mi = (MSAPIndication*)NULL;

         debug_test("Leaving M_SetAux_u_sync @ 2");
         print_status(statp);
         return(NOTOK);
     }
}


