/* Copyright (c) 1993 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/


/*  M_WaitReq_u - Wait for an incoming indication or confirm
 *
 */

#include <stdio.h>
#include "msap.h"
#include "xmp.h"

extern int xmp_initialised;
extern MIDAS_SESSION *sessionp;
extern MIDAS_WORKSPACE midas_workspace;
extern MIDAS_WORKSPACE mp_initialise_M();
extern MIDAS_SESSION default_session;
extern MIDAS_CONTEXT context;

extern MIDAS_SESSION *session_get();

/*
 * M_WaitReq_u - wait for the the result of an asynchronous operation
 * N.B. This only makes sense in UCL mode where there is only one
 * possible session
 */

int M_WaitReq_u (msd, secs, mi)
int msd, secs;
MSAPIndication *mi;
{

    MIDAS_WAITING_SESSIONS waiting_session;
    OM_sint primitive, completion_flag, invoke_id;
    MIDAS_STATUS *statp, *opstatp; /*filled in when an error occurs*/
    MIDAS_RESULT *resultp;
    int gmdmode = TRUE;

    /*check xmp has been initialised*/
    if (!xmp_initialised) {
        fprintf(stderr, "M-WaitReq_u: XMP not initialised\n");
		return(NOTOK);
    }

    if ((waiting_session.bound_session = session_get(msd)) 
		== (MIDAS_SESSION *)NULL){
	/*assume UCL mode*/
	gmdmode = FALSE;
	waiting_session.bound_session = &default_session;
    } 

    waiting_session.activated = 1;
    waiting_session.next = (MIDAS_WAITING_SESSIONS *)NULL;

    statp = mp_wait_M(&waiting_session, 
		      midas_workspace, 0); /*indefinite timeout*/

    if (statp != MIDAS_SUCCESS || waiting_session.activated == FALSE) 
	    return(NOTOK);

    /*something has arrived - call mp_receive to process it*/
	 
    if (!gmdmode) context.connection_id = msd;
    context.time_limit = secs;

    statp = mp_receive_M(waiting_session.bound_session, &context, &primitive, 
	           &completion_flag, &opstatp, &resultp, &invoke_id);

    switch ((int)statp){
        case MIDAS_SUCCESS:
            switch ((int)opstatp) {
	        case MIDAS_SUCCESS:
		    /*resultp should point at a valid structure*/
		    *mi = resultp->MIDAS_result_info;	/*struct copy*/
		    return(OK);

		default:
		    /*should point to a valid status structure*/
		    if(opstatp->MIDAS_parameter)
			*mi = *(opstatp->MIDAS_parameter);  /*struct copy*/
  		    else
			mi = (MSAPIndication*)NULL;
		    return(NOTOK);
	    }

	case MIDAS_INSUFFICIENT_RESOURCES:
			return(NOTOK);

	default:
	    /*some sort of error opstatp should point at the right thing*/

            if(opstatp){
                if(opstatp->MIDAS_parameter){
		    debug_test("In M_Wait_u.c @ 1A");
	            *mi = *(opstatp->MIDAS_parameter);  /*struct copy*/
		}
	    }
	    else{
		debug_test("In M_Wait_u.c @ 2");
		 mi = (MSAPIndication*)NULL;
	    }
	    debug_test("In M_Wait_u.c @ 3");
	    return(NOTOK);
    }
}

