/* Copyright (c) 1993 University College London
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the Department of Computer Science, University College London.
 * The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/


/*  Maintain a table of active sessions
 */

#include <stdio.h>
#include <sys/types.h>
#include "msap.h"
#include "xmp.h"

static MIDAS_SESSION *session_tab[FD_SETSIZE];	/*Assume initialised
						  to zero (should do better 
						  than this)*/

/*
 * Place a session in the table
 */

int
session_add(sessp)
    MIDAS_SESSION *sessp;{
    register int fd = sessp->file_descriptor;

    if (fd == MP_NO_VALID_FILE_DESCRIPTOR 	||
	fd < 0 					||
	fd >= FD_SETSIZE){
	fprintf(stderr, "session_add: bad fd %d\n", fd);
	return(NOTOK);
    }

    if (session_tab[fd]){
	fprintf(stderr, "session_add: fd %d already in use\n", fd);
	return(NOTOK);
    }

    session_tab[fd] = sessp;
    return(OK);
}

/*
 * Remove a session from the table
 */

session_del(fd)
    int fd; {

    if (fd == MP_NO_VALID_FILE_DESCRIPTOR 	||
	fd < 0 					||
	fd >= FD_SETSIZE){
	fprintf(stderr, "session_del: bad fd %d\n", fd);
	return(NOTOK);
    }

    if (session_tab[fd] == (MIDAS_SESSION *)NULL){
	fprintf(stderr, "session_del: fd %d not in use\n", fd);
	return(NOTOK);
    }

    session_tab[fd] = (MIDAS_SESSION *)NULL;
    return(OK);
}


/*
 * Find a session from the table
 *
 * N.B. This will return NULL for UCL mode
 */

MIDAS_SESSION *
session_get(fd)
    int fd; {

    if (fd == MP_NO_VALID_FILE_DESCRIPTOR 	||
	fd < 0 					||
	fd >= FD_SETSIZE){
	fprintf(stderr, "session_del: bad fd %d\n", fd);
	return((MIDAS_SESSION *)NULL);
    }

    return(session_tab[fd]);
}
